/***************************************************************************
* ======================================================================
* Computer Vision/Image Processing Tool Project - Dr. Scott Umbaugh SIUE
* ======================================================================
*
*             File Name: bandcast.c
*           Description: cast image bands to a specified type
*         Related Files: Imakefile, libband.a, libmatrix.a
*   Initial Coding Date: 11/01/92
*           Portability: Standard (ANSI) C
*             Credit(s): Gregory Hance
*                        Southern Illinois University @ Edwardsville
*
** Copyright (C) 1993 SIUE - by Gregory Hance.
**
** Permission to use, copy, modify, and distribute this software and its
** documentation for any purpose and without fee is hereby granted, provided
** that the above copyright notice appear in all copies and that both that
** copyright notice and this permission notice appear in supporting
** documentation.  This software is provided "as is" without express or
** implied warranty.
**
* RCS (Revision Control System) Information
* ...Added automatically by RCS 
*
* $Log: bandcast.c,v $
* Revision 1.3  1997/03/08 00:58:27  yawei
* Name Changes:
* 	BOOLEAN ==> CVIP_BOOLEAN
* 	FALSE ==> CVIP_NO
* 	TRUE ==> CVIP_YES
* 	BYTE ==> CVIP_BYTE
* 	SHORT ==> CVIP_SHORT
* 	INTEGER ==> CVIP_INTEGER
* 	FLOAT ==> CVIP_FLOAT
* 	DOUBLE ==> CVIP_DOUBLE
* 	TYPE ==> CVIP_TYPE
*
* Revision 1.2  1996/08/09 15:11:05  akjoele
* Changed function prototype for ANSI C conformity.
*
 * Revision 1.1  1996/08/09  15:09:51  akjoele
 * Initial revision
 *
 * Revision 1.1  1996/08/09  15:08:51  akjoele
 * Initial revision
 *
 * Revision 1.1  1993/05/03  05:53:15  hanceg
 * Initial revision
 *
*
*
****************************************************************************/

#include "stdlib.h"
#include "stdio.h"
#include "CVIPmatrix.h"


/****************************************************************
                   Name: bandcast
          Expanded Name: Band Cast
             Parameters: <srcBANDS> - pointer to an array of MATRIX pointers
			 <no_of_bands> - number of spectral data bands
			 <data_type> - data type to be promoted to
                Returns: pointer to new array of MATRIX pointers cast into
			 the specified data type
            Description: cast the data of an image pointed to by <srcBANDS?
			 into a more precise type.
	    Diagnostics: returns with NULL pointer and error message when
			 cast cannot be performed.
              Credit(s): Greg Hance, Arve Kjoelen
****************************************************************/

MATRIX **bandcast(MATRIX **srcBANDS, int no_of_bands, CVIP_TYPE data_type)
{
   MATRIX **desBANDS;
   int i = 0, rows, cols;
   FORMAT format;
   const char fn[] = {"bandcast"};
   
   /* simplify variables */
   rows = srcBANDS[0]->rows;
   cols = srcBANDS[0]->cols;
   format = srcBANDS[0]->data_format;
   
   if ( !(desBANDS = (MATRIX **) malloc(no_of_bands*sizeof(MATRIX *))) ) {
      fprintf(stderr,"%s: could not allocate new data bands for image.\n",fn);
      return NULL;
   }
   
   for(; i < no_of_bands; i++) {
      if ( !(desBANDS[i] = matrix_allocate(rows, cols, data_type, format)) ) {
         fprintf(stderr,"%s: could not allocate new matrix for data band.\n",fn);
         return NULL;
      }
      (void) copy_Matrix(srcBANDS[i], desBANDS[i]);
   }
   for(i=0; i<no_of_bands; i++) {
   }
   return desBANDS;
}
      
