/***************************************************************************
* ======================================================================
* Computer Vision/Image Processing Tool Project - Dr. Scott Umbaugh SIUE
* ======================================================================
*
*             File Name: img2imgmap.c
*           Description: perform image to image mapping procedure
*         Related Files: img2imgmap.h, Imakefile
*   Initial Coding Date: Sun May 30 14:48:06 CDT 1993
*           Portability: Standard (ANSI) C
*             Credit(s): Gregory Hance
*                        Southern Illinois University @ Edwardsville
*
** Copyright (C) 1993 SIUE - by Gregory Hance.
**
** Permission to use, copy, modify, and distribute this software and its
** documentation for any purpose and without fee is hereby granted, provided
** that the above copyright notice appear in all copies and that both that
** copyright notice and this permission notice appear in supporting
** documentation.  This software is provided "as is" without express or
** implied warranty.
**
* RCS (Revision Control System) Information
* ...Added automatically by RCS 
*
** $Log: img2imgmap.c,v $
** Revision 1.3  1997/01/15 17:17:38  yawei
** Global Change:  IMAGE ==> Image
**
 * Revision 1.2  1997/01/15  16:33:08  yawei
 * updated changes
 *
 * Revision 1.1  1993/05/31  03:23:40  hanceg
 * Initial revision
 *
*
****************************************************************************/
#include "CVIPtoolkit.h"
#include "CVIPdef.h"
#include "ColorMap.h"


static meanmap(byte **srcP, byte **mapP, unsigned int vdim, 
              int colors, colorhist_vector chv, pixel *pixelP)
{
   long **sum;
   register int i = 1, j = 0;

   sum = (long **) malloc(sizeof(long *)*colors);
   *sum = (long *) calloc(colors*3, sizeof(long));
   for(; i < colors; i++) sum[i] = *sum + i*3;

   fprintf(stderr, "Checking before!\n");
   for(i=0; i < vdim; j=0, i++) {
      while(!(PPM_GETR(chv[j].color) == mapP[0][i] &&
              PPM_GETG(chv[j].color) == mapP[1][i] &&
              PPM_GETB(chv[j].color) == mapP[2][i]) ) { j++;
      /*fprintf(stderr, "PPM_GETR(chv[%d].color)=%d\n",j,PPM_GETR(chv[j].color));
      fprintf(stderr, "PPM_GETG(chv[%d].color)=%d\n",j,PPM_GETG(chv[j].color));
      fprintf(stderr, "PPM_GETB(chv[%d].color)=%d\n",j,PPM_GETB(chv[j].color));
      fprintf(stderr, "mapP[i][0]=%d\n",mapP[0][i]);
      fprintf(stderr, "mapP[i][1]=%d\n",mapP[1][i]);
      fprintf(stderr, "mapP[i][2]=%d\n",mapP[2][i]);

      fprintf(stderr, "j=%d\n", j); */ }
      sum[j][0] += srcP[0][i]; 
      sum[j][1] += srcP[1][i];
      sum[j][2] += srcP[2][i];
   }
   fprintf(stderr, "Checking after!\n");

   for(i=0; i < colors; i++, pixelP++) {
      pixelP->r = (byte) ROUND(((float) sum[i][0])/chv[i].value);
      pixelP->g = (byte) ROUND(((float) sum[i][1])/chv[i].value);
      pixelP->b = (byte) ROUND(((float) sum[i][2])/chv[i].value);
   }

   free(*sum);
   free(sum);
}
    

void img2img_map(const Image *srcImage, Image *mapImage, colorhist_vector chv, int colors)
{
   byte **mapP, **srcP;
   pixel *pixelP;
   register unsigned int i = 0, j = 0;
   unsigned int rows, cols, vdim;

   /* Simplify some variables */
   rows = mapImage->bandP[0]->rows;   
   cols = mapImage->bandP[0]->cols;   
   vdim = rows*cols;


   mapP = (byte **) malloc(sizeof(byte *)*3);
   srcP = (byte **) malloc(sizeof(byte *)*3);
   pixelP = (pixel *) malloc(sizeof(pixel)*colors);

   for(; i < 3; i++) {
      mapP[i] = (byte *) mapImage->bandP[i]->rptr[0];
      srcP[i] = (byte *) srcImage->bandP[i]->rptr[0];
   }
   fprintf(stderr, "checking!\n");
   fprintf(stderr, "colors = %d\n", colors);
   fprintf(stderr, "vdim = %d\n", vdim);


   meanmap(srcP, mapP, vdim, colors, chv, pixelP);
   fprintf(stderr, "checking after!\n");


   for(i=0; i < vdim; j=0, i++) {
      while(!(PPM_GETR(chv[j].color) == mapP[0][i] &&
              PPM_GETG(chv[j].color) == mapP[1][i] &&
              PPM_GETB(chv[j].color) == mapP[2][i]) ) j++;
      mapP[0][i] = pixelP[j].r;
      mapP[1][i] = pixelP[j].g;
      mapP[2][i] = pixelP[j].b;

   }
}
   

