
/*****************************************************************************
 * ======================================================================    *
 * Image Processing/Computer Vision Tool Project - Dr. Scott Umbaugh SIUE    *
 * ======================================================================    *
 *                                                                           *
 * ROUTINE/FUNCTION: huffcompress                                            *                    
 *                                                                           *
 *                                                                           *
 *              DESCRIPTION: This module will compress the input image using *
 *                           huffman(runlength) encoding tech.               *
 *                                                                           *                                                                          
 *                   AUTHOR: Ramesh Reddy                                    *
 *                                                                           *
 *            RELATED FILES: dct.c, quant.c,com_setup.c,huffdecompress.c     * 
 *                           dcpred.c, ima_setup.c jpeg.c                    *
 *                                                                           *                                                             
 *              CODING DATE: 12/10/92                                        *
 *  LAST MODIFICATIONS MADE: 1/18/93                                         *
 *                                                                           *
 *  VARIABLES PASSED TO YOUR_FUNCTION:					     *
 *        input_Image - An image structure with necessary info. about the    *
 *        image as well as a **ptr to the image data. (see image.h)          *
 *                                                                           *
 *  RETURNED VARIABLES:                                                      *
 *        huffcompress() -  returns a file of compressed image.              *
 *                                                                           *
 *****************************************************************************/
#include <stdio.h>
#include <math.h>
#include "CVIPimage.h"

/*  
     huff_compress: This routine takes the input image pointer and filename to store the 
     compressed image contents as input arguments and returns compressed file. for detail
     operation of the routine see the documentation of the huffcompress.
*/

void huff_compress(input_Image, filename)
Image *input_Image;
char *filename;
{
   unsigned int     bitts[35],nn= 0, *b;
   int              out[64],i,j,k,q,rows,cols,color,x,y,m=0,l=0,total = 0;
   FILE             *output_file;
   const char fn[] = {"Compression"};
 
   struct z_array {       /*  structure to design a zig zag sequence  */
       int zig_order;
       long pix_val;
   }zzag_array[8][8];
   
  int init_array[8][8]={                                 /*  how zig zag array travels  */
			  0,  1,  5,  6, 14, 15, 27, 28,
		          2,  4,  7, 13, 16, 26, 29, 42,
			  3,  8, 12, 17, 25, 30, 41, 43,
			  9, 11, 18, 24, 31, 40, 44, 53,
		         10, 19, 23, 32, 39, 45, 52, 54,
		         20, 22, 33, 38, 46, 51, 55, 60,
		         21, 34, 37, 47, 50, 56, 59, 61,
		         35, 36, 48, 49, 57, 58, 62, 63 };
			 
  rows  = (int) input_Image->image_ptr[0]->rows;        /*  # of rows in image  */
  cols  = (int) input_Image->image_ptr[0]->cols;        /* # of cols in the image  */
  if (input_Image->bands == 3) color = 3; else color = 1;  /* color ?  */
  b = bitts; 
  
  #ifndef WIN32
  output_file = fopen(filename, "a");    /*  open file to write  */
  #else
  output_file = fopen(filename, "ab");    /*  open file to write  */
  #endif
  if ( output_file == NULL )
     printf("Could'nt open %s for Writing", filename);
  
  for ( k = 0; k < color; k++){		/*  main routine to compress block by block  */	 
    x = 0; m = 0;
    while( x < rows ){
       y =0;
       while( y < cols ){
          for(i=0; i<=7; i++){
             for(j=0; j<=7; j++) {
                zzag_array[i][j].zig_order= init_array[i][j];  /*  may be this can be placeed outside loop */
                zzag_array[i][j].pix_val=(((short**)input_Image->image_ptr[k]->rptr)[(x+i)][(y+j)]);
                q = zzag_array[i][j].zig_order;
                out[q] = zzag_array[i][j].pix_val;  /* zig zaged  values  */
             }
	  }
	  nn = 0;
	  
	  nn = HUF_COMPRESS(out, bitts);    /*  call the 8x8 compress  */
          nn =  fwrite(bitts,4,(nn+1),output_file);    /* nn returns # of bytes not # of unsigneds,write */
	  total = total +nn;                           /* to keep track  */
          /* m++;*/
	  y+=8;
	}   /* end of cols  */
	x+=8;
     }  /* end of rows */
   }  /* end of color  */
   fseek(output_file,0,2);
   nn = ftell(output_file);  /* # of bytes in the compressed file  */
   fclose (output_file);
   printf("# of bytes of %s (compressed file) =  %d bytes\n",filename, nn);
   
}      /* end of compress algorithm file */

/* 
   HUF_COMPRESS: This is actual compression routine  for 8x8 data block. it takes the 
   8x8 block as input and returns the # of bytes the code was compressed to.
*/

int HUF_COMPRESS(out,bitts)  
int *out;
unsigned int *bitts;
{ 
   float        totalsfac=0;    /* variable assignments   */
   int          total,yes=0,cnt=0,marks=0,i,j,k,q,siz,runs=0,ii=0;  
   int          Found;
   int          re,x,yy,xx,mm,m,jj,val,nn = 0; 
   unsigned int vv=0,orig=0,last=0,carryover=0,sfac=0,sfacold,n,p,pp,shfac,co_len;
   unsigned int intval,codew,outbits,hufcode;
    
  struct hold_it {                /* utility structure in compression  */
      unsigned long transmit;
      unsigned shift_fac;
    }outarray[65];
    
   struct huff_code{           /*  structure for AC coefficiants  */
      unsigned int run;        /* runlength  */
      unsigned int size;       /* size of run  */
      unsigned int code_len;   /* code word len  */
      int code_word;           /* amplitude  */
  }; 

  struct dc_code{              /*  structure for dc values   */
     unsigned int category;
     unsigned int Code_len;    /* size code length  */
     unsigned int Code_word;   /* amplitude    */
  };

  struct huff_code huff_table[162]={  /*  table for ac coefficiants   */
        {0,0,4,10},     {0,1,2,0},      {0,2,2,1},      {0,3,3,4},      {0,4,4,11},
        {0,5,5,26},     {0,6,7,120},    {0,7,8,248},    {0,8,10,1014},  {0,9,16,0xFF82}, {0,10,16,0xFF83},
        {1,1,4,12},     {1,2,5,27},     {1,3,7,121},    {1,4,9,502},    {1,5,11,2038},
	{1,6,16,0xff84},{1,7,16,0xff85},{1,8,16,0xff86},{1,9,16,0xff87},{1,10,16,0xff88},
	{2,1,5,28},     {2,2,8,249},    {2,3,10,1015},  {2,4,12,4084},  {2,5,16,0xff89},          
	{2,6,16,0xff8a},{2,7,16,0xff8b}, {2,8,16,0xff8c},{2,9,16,0xff8d},{2,10,16,0xff8e},
	{3,1,6,58},     {3,2,9,503},    {3,3,12,4085},  {3,4,16,0xff8f},{3,5,16,0xff90},
        {3,6,16,0xff91},{3,7,16,0xff92},{3,8,16,0xff93},{3,9,16,0xff94},{3,10,16,0xff95},
	{4,1,6,59},     {4,2,10,1016},  {4,3,16,0xff96},{4,4,16,0xff97},{4,5,16,0xff98},
	{4,6,16,0xff99},{4,7,16,0xff9a},{4,8,16,0xff9b},{4,9,16,0xff9c},{4,10,16,0xff9d},
	{5,1,7,122},    {5,2,11,2039},  {5,3,16,0xff9e},{5,4,16,0xff9f},{5,5,16,0xffa0},
	{5,6,16,0xffa1},{5,7,16,0xffa2},{5,8,16,0xffa3},{5,9,16,0xffa4},{5,10,16,0xffa5},
	{6,1,7,123},    {6,2,12,4086},  {6,3,16,0xffa6},{6,4,16,0xffa7},{6,5,16,0xffa8},
        {6,6,16,0xffa9},{6,7,16,0xffaa},{6,8,16,0xffab},{6,9,16,0xffac},{6,10,16,0xffad},
	{7,1,8,250},    {7,2,12,4087},  {7,3,16,0xffae},{7,4,16,0xffaf},{7,5,16,0xffb0},
	{7,6,16,0xffb1},{7,7,16,0xffb2},{7,8,16,0xffb3},{7,9,16,0xffb4},{7,10,16,0xffb5},
	{8,1,9,504},    {8,2,15,32704}, {8,3,16,0xffb6},{8,4,16,0xffb7},{8,5,16,0xffb8},
	{8,6,16,0xffb9},{8,7,16,0xffba},{8,8,16,0xffbb},{8,9,16,0xffbc},{8,10,16,0xffbd},
	{9,1,9,505},    {9,2,16,0xffbe},{9,3,16,0xffbf},{9,4,16,0xffc0},{9,5,16,0xffc1},
        {9,6,16,0xffc2},{9,7,16,0xffc3},{9,8,16,0xffc4},{9,9,16,0xffc5},{9,10,16,0xffc6},
        {10,1,9,506},   {10,2,16,0xffc7},{10,3,16,0xffc8},{10,4,16,0xffc9},{10,5,16,0xffca},
	{10,6,16,0xffcb},{10,7,16,0xffcc},{10,8,16,0xffcd},{10,9,16,0xffce},{10,10,16,0xffcf},
	{11,1,10,1017},  {11,2,16,0xffd0},{11,3,16,0xffd1},{11,4,16,0xffd2},{11,5,16,0xffd3},
	{11,6,16,0xffd4},{11,7,16,0xffd5},{11,8,16,0xffd6},{11,9,16,0xffd7},{11,10,16,0xffd8},
	{12,1,10,1018},  {12,2,16,0xffd9},{12,3,16,0xffda},{12,4,16,0xffdb},{12,5,16,0xffdc},
        {12,6,16,0xffdd},{12,7,16,0xffde},{12,8,16,0xffdf},{12,9,16,0xffe0},{12,10,16,0xffe1},
	{13,1,11,2040},{13,2,16,0xffe2},{13,3,16,0xffe3},{13,4,16,0xffe4},{13,5,16,0xffe5},
	{13,6,16,0xffe6},{13,7,16,0xffe7},{13,8,16,0xffe8},{13,9,16,0xffe9},{13,10,16,0xffea},
	{14,1,16,0xffeb},{14,2,16,0xffec},{14,3,16,0xffed},{14,4,16,0xffee},{14,5,16,0xffef},
	{14,6,16,0xfff0},{14,7,16,0xfff1},{14,8,16,0xfff2},{14,9,16,0xfff3},{14,10,16,0xfff4},
	{15,0,11,2041},{15,1,16,0xfff5},{15,2,16,0xfff6},{15,3,16,0xfff7},{15,4,16,0xfff8},{15,5,16,0xfff9},
	{15,6,16,0xfffa},{15,7,16,0xfffb},{15,8,16,0xfffc},{15,9,16,0xfffd},{15,10,16,0xfffe}};


    struct dc_code dc_code_table[12] = {                       /*  table for dc coeffcients  */
				     { 0, 2,   0}, {1,  3,  2},
				     { 2, 3,   3}, {3,  3,  4},
				     { 4, 3,   5}, {5,  3,  6},
				     { 6, 4,  14}, {7,  5, 30},
				     { 8, 6,  62}, {9,  7, 126},
				     {10, 8, 254}, {11, 9, 510},
				     };


 for(i=0; i < 64; i++) {      /* for all data bits  */

  val = out[i];

  if(i==0){               /* dc bit  */
     if(val==0) {
	outbits=0; shfac=2; 
     }  /* dc value = 0, only the code word is sent */
     else if( val==-1 || val==1) {
	if(val==1)
	   codew=5;   /* one's complement: cw=010 +0001=101=5 */
	else
	   codew=4;  /*one's complement of -1 is 1110, take last bit 0 || cw=0101 =5 */
	codew<<=28; outbits=codew; shfac=4;
     }
     else if(-3 <= val && val <= 3) {  /* ssss = 2 */
       if(val<0)
       val = val-1;
       codew=3;  intval=val; intval<<=30;  intval >>= 3;/* codew = 3 + 27 = 32 bits */
       codew<<=29;  shfac=5;
       outbits = codew | intval; /*shfac=total #bits outbits needed*/
     }
     else if(-7 <= val && val<= 7) { /* ssss = 3 */
	if(val<0)
	val = val -1;
	codew=4;  intval=val; intval<<=29; intval >>= 3;
       codew<<=29; shfac=6;
       outbits = codew | intval;
     }
     else if(-15 <= val && val<= 15) { /* ssss = 4 */
	if(val<0)
	val = val - 1;
	codew=5;  intval=val; intval<<=28 ; intval >>= 3;
	codew<<=29; shfac=7;
	outbits = codew | intval;
     }
     else if(-31 <= val && val<= 31) { /* ssss = 5 */
	if(val<0)
	val = val -1;
	codew=6;  intval=val;
	intval<<=27;
	intval >>=3;
	codew<<=29; shfac=8;
	outbits = codew | intval;
     }
     else if(-63 <= val && val<= 63) { /* ssss = 6 */
	if(val<0)
	val = val -1;
	codew=14;  intval=val; intval <<= 26; intval >>= 4;
	codew<<=28; shfac=10;
	outbits =codew | intval;
     }
     else if(-127<= val && val<= 127) {  /* ssss = 7 */
	if(val<0)
	val = val -1;
	codew=30; intval=val; intval <<= 25; intval >>=5;
	codew <<=27; shfac=12;
	outbits=codew | intval;
     }
    else if(-255 <=  val && val <= 255) { /* ssss = 8 */
	if(val<0)
	val = val -1;
	codew=62; intval=val; intval <<=24; intval >>= 6;
	codew <<=26; shfac=14; /*code bits + int bits=shfac*/
	outbits = codew | intval;
	/*printf("\n This works !! %lu", outbits);*/
    }
    else if(-511 <= val && val<= 511) { /* ssss = 9 */
	if(val<0)
	val = val -1;
	codew=126; intval=val; intval <<=23; intval>>=7;
	codew<<=25;
	outbits=codew | intval;   shfac =16;
    }
    else if(-1023 <= val && val<= 1023) {  /* ssss = 10 */
	if(val<0)
	val = val -1;
	codew=254; intval=val; intval <<= 22; intval >>=8;
	outbits=codew | intval;   shfac=18;
   }
   else if(-4095 <= val && val<= 4095) {  /* ssss = 11 */
	if(val<0)
	val = val -1;
	codew=510; intval=val; intval <<=21; intval >>= 9;
	outbits=codew | intval;   shfac=20;
   }
   else { 
         printf("Error ! Out of Range ");
	 exit(0);
   }
   /* printf("\n DC code=  %lu shfac= %u\n", outbits,shfac);*/

   (*(outarray)).transmit=outbits;
   (*(outarray)).shift_fac=shfac;
   vv++;
   totalsfac=shfac;
    /* printf("DC code in outarray = %lu",(*(outarray)).transmit);*/

 } /* end if(i==0) */

 else {     /*   ac bits  1 - 63   */

       if( val == 0)                 {  runs++;  siz=0;}  /*  zero length  */
       else if  (val==-1 || val==1)        siz=1; 
       else if  (-3<=val && val<=3)        siz=2;
       else if  (-7<=val && val<=7)        siz=3;
       else if  (-15<=val && val<=15)      siz=4;
       else if  (-31<=val && val<=31)      siz=5;
       else if  (-63<=val && val<=63)      siz=6;
       else if  (-127<=val && val<=127)    siz=7;
       else if  (-255<=val && val<=255)    siz=8;
       else if  (-511<=val && val<=511)    siz=9;
       else if  (-1023<=val && val<=1023)  siz=10;
       else if  (-2047<=val && val<=2047)  siz=11;
       else     printf("AC coefficient input out of range. Add more please !");
	   
       if(val != 0 || i==63 || runs==15) {       /*  consider EOB & ZRL case */

           if(val==-1) val=0;
           if(val==1) val=1;                      
           if (siz == 0 && i==63){                 /* End of the block  */
	        outbits=(*(huff_table)).code_word;
	        outbits<<=28;	
                (*(outarray+vv)).transmit=outbits;     /* put it in the structure  */
                (*(outarray+vv)).shift_fac=4; vv++;    /* put the # of bits to shift  */
           }
           else{
	        if ( runs == 15 )
	              n=runs*10 + siz +1 ;  /*  +1 because of 11 table items for zero and siz = 0  */
		else 
	              n=runs*10 + siz;
	        co_len = (*(huff_table+n)).code_len;
                hufcode=(*(huff_table+n)).code_word;   /* hufcode is unsigned long*/
		                               /* code_word has changed fr int to unsigned long */

                if( co_len < 16){
	          shfac=16+(16-(*(huff_table+n)).code_len);
	          hufcode <<= shfac;       
                }
                else {
	           hufcode<<=16; shfac=16;            /* This shfac=16 may not be useful */
                } 

                if((val !=0 || val !=1 || val !=-1) && val < 0)
                     val = val -1;
		      
                intval=val;                              /* val is long */
                intval<<= (32-siz);
                intval >>= co_len;
                outbits= hufcode | intval;                  /*( 32- co_len - siz ));*/
                shfac=co_len + siz;
                (*(outarray+vv)).transmit=outbits; /* dont use nn here */
                (*(outarray+vv)).shift_fac=shfac;
                vv++;
                totalsfac=totalsfac+shfac;
                if(i!=63)
                   runs=0;

               if(i==63){   /* EOB */
	          outbits=(*(huff_table)).code_word;
	          outbits<<=28;	
                  (*(outarray+vv)).transmit=outbits;
                  (*(outarray+vv)).shift_fac=4; vv++;
               }
          }                /*   end of else     */
     }                     /* end of if(val!=0 || i=63 || runs==15) */
  }                        /* end of big else   for ac coefficients*/
}                          /*  end FOR loop   */

  nn=0;
  bitts[0] = 0;       /*  array for the packing  */
  bitts[0] = (*(outarray)).transmit;  /*  put the dc bit  */
  for(i=1; i<=vv; i++) {       /* packing the bits into a array to compress */
	
	sfac += ((*(outarray+(i-1))).shift_fac);    
		
	if (sfac >=32) {       /* if cant fit in last unsigned  */
		nn++;
		bitts[nn] = 0;  /* initialize first  */
		bitts[nn] = bitts[nn] | ((*(outarray+(i-1))).transmit << (32-sfacold));
		/* printf("bitts = %x  nn = %i shift factor = %i\n",bitts[nn],nn,(32-sfacold));*/
		sfac = sfac-32;	
	}
	bitts[nn] = bitts[nn] | ((*(outarray+i)).transmit >> sfac);
	sfacold = sfac;
	/*printf("bitts = %x  nn = %i shift factor = %i\n",bitts[nn],nn,sfac);*/
  }
 return (nn);   
}
