/*****************************************************************************
 * ======================================================================    *
 * Image Processing/Computer Vision Tool Project - Dr. Scott Umbaugh SIUE    *
 * ======================================================================    *
 *                                                                           *
 * ROUTINE/FUNCTION: huffdecompress.c                                        *                    
 *                                                                           *
 *                                                                           *
 *              DESCRIPTION: This module will decompress the image which     *
 *                           is compressed by huffcompress routine.          *
 *                                                                           *                                                                          
 *                   AUTHOR: Ramesh Reddy                                    *
 *                                                                           *
 *            RELATED FILES: dct.c, quant.c,huffcompress.c,dcpred.c          *   
 *                           ima_setup.c, com_setup.c, jpeg.c                *
 *                                                                           *                                                             
 *              CODING DATE: 12/15/92                                        *
 *  LAST MODIFICATIONS MADE: 1/18/93                                         *
 *                                                                           *
 *  VARIABLES PASSED TO YOUR_FUNCTION:					     *
 *        input_Image - An image structure with necessary info. about the    *
 *        image as well as a **ptr to the image data. (see image.h)          *
 *                                                                           *
 *  RETURNED VARIABLES:                                                      *
 *        hufdecompress() -  returns the image structure that was            *
 *        uncompressed from the filename supplied      .                     *                                                                       
 *                                                                           *
 *****************************************************************************/
#include <stdio.h>
#include <math.h>
#include "CVIPimage.h"

/*
    HUFDECOMPRESS: This routine file pointer to the compressed file and decompresses
    that 8x8 block and writes the decompressed values in the supplied image structure.
    
*/

int HUF_DECOMPRESS(input_Image, rows, cols,color, input_file, total_bytes,current_pos)
Image  *input_Image;
int    rows,cols,color;
FILE   *input_file;
int    total_bytes,current_pos;
 { 
  int            sbase,temp,temp1,i,iii = 0,j,u,v,siz,runs=0,ii=0,prev = 0;  
  int            deout[65], fdct[8][8],need_byte = 0,curent_pos; 
  unsigned int   n,nn=1;
  unsigned int   tempbit,tempword1, tempword2;
  int            readbit[100];
  
  /* The following are utility look up tables for decompression   */
  
  int lut[12] = {0xFFFFFFFF,0xFFFFFFFE,0xFFFFFFFC,0xFFFFFFF8,0xFFFFFFF0,0xFFFFFFE0,0xFFFFFFC0,0xFFFFFF80,
		 0xFFFFFF00,0xFFFFFE00,0xFFFFFC00,0xFFFFF800};

  int lut0[19] = {0x00000000,0x00000001,0x00000003,0x00000007,0x0000000F,0x0000001F,0x0000003F,0x0000007F,
		  0x000000FF,0x000001FF,0x000003FF,0x000007FF,0x00000FFF,0x00001FFF,0x00003FFF,0x00007FFF,
		  0x0000FFFF};			
  int lut2[19] = {0x00000000,0x00000001,0x00000002,0x00000004,0x00000008,0x00000010,0x00000020,0x00000040,
		  0x00000080,0x00000100,0x00000200,0x00000400,0x00000800,0x00001000,0x00002000,0x00004000,
		  0x00008000};
		  
  struct huff_code{          /* structure model for ac coeffcients  */
     unsigned int run;       /*  runlaength  */
     unsigned int size;      /*  size  */
     unsigned int code_len;  /* code length  size */
     int code_word;          /* amplitude  */
  };
  
  /*  table for the ac coefficients  */

  struct huff_code huff_table[162]={{0,0,4,10},{0,1,2,0},{0,2,2,1},{0,3,3,4},{0,4,4,11},
      {0,5,5,26},{0,6,7,120},{0,7,8,248},{0,8,10,1014},{0,9,16,0xFF82},{0,10,16,0xFF83},
      {1,1,4,12},{1,2,5,27},{1,3,7,121},{1,4,9,502},{1,5,11,2038},{1,6,16,0xff84},{1,7,16,0xff85},
      {1,8,16,0xff86},{1,9,16,0xff87},{1,10,16,0xff88},{2,1,5,28},{2,2,8,249},{2,3,10,1015},
      {2,4,12,4084},{2,5,16,0xff89},{2,6,16,0xff8a},{2,7,16,0xff8b},{2,8,16,0xff8c},{2,9,16,0xff8d},
      {2,10,16,0xff8e},{3,1,6,58},{3,2,9,503},{3,3,12,4085},{3,4,16,0xff8f},{3,5,16,0xff90},
      {3,6,16,0xff91},{3,7,16,0xff92},{3,8,16,0xff93},{3,9,16,0xff94},{3,10,16,0xff95},{4,1,6,59},
      {4,2,10,1016},{4,3,16,0xff96},{4,4,16,0xff97},{4,5,16,0xff98},{4,6,16,0xff99},{4,7,16,0xff9a},
      {4,8,16,0xff9b},{4,9,16,0xff9c},{4,10,16,0xff9d},{5,1,7,122},{5,2,11,2039},{5,3,16,0xff9e},
      {5,4,16,0xff9f},{5,5,16,0xffa0},{5,6,16,0xffa1},{5,7,16,0xffa2},{5,8,16,0xffa3},{5,9,16,0xffa4},
      {5,10,16,0xffa5},{6,1,7,123},{6,2,12,4086},{6,3,16,0xffa6},{6,4,16,0xffa7},{6,5,16,0xffa8},
      {6,6,16,0xffa9},{6,7,16,0xffaa},{6,8,16,0xffab},{6,9,16,0xffac},{6,10,16,0xffad},{7,1,8,250},
      {7,2,12,4087},{7,3,16,0xffae},{7,4,16,0xffaf},{7,5,16,0xffb0},{7,6,16,0xffb1},{7,7,16,0xffb2},
      {7,8,16,0xffb3},{7,9,16,0xffb4},{7,10,16,0xffb5},{8,1,9,504},{8,2,15,32704},{8,3,16,0xffb6},
      {8,4,16,0xffb7},{8,5,16,0xffb8},{8,6,16,0xffb9},{8,7,16,0xffba},{8,8,16,0xffbb},{8,9,16,0xffbc},
      {8,10,16,0xffbd},{9,1,9,505},{9,2,16,0xffbe},{9,3,16,0xffbf},{9,4,16,0xffc0},{9,5,16,0xffc1},
      {9,6,16,0xffc2},{9,7,16,0xffc3},{9,8,16,0xffc4},{9,9,16,0xffc5},{9,10,16,0xffc6},{10,1,9,506},
      {10,2,16,0xffc7},{10,3,16,0xffc8},{10,4,16,0xffc9},{10,5,16,0xffca},{10,6,16,0xffcb},{10,7,16,0xffcc},
      {10,8,16,0xffcd},{10,9,16,0xffce},{10,10,16,0xffcf},{11,1,10,1017},{11,2,16,0xffd0},{11,3,16,0xffd1},
      {11,4,16,0xffd2},{11,5,16,0xffd3},{11,6,16,0xffd4},{11,7,16,0xffd5},{11,8,16,0xffd6},{11,9,16,0xffd7},
      {11,10,16,0xffd8},{12,1,10,1018},{12,2,16,0xffd9},{12,3,16,0xffda},{12,4,16,0xffdb},{12,5,16,0xffdc},
      {12,6,16,0xffdd},{12,7,16,0xffde},{12,8,16,0xffdf},{12,9,16,0xffe0},{12,10,16,0xffe1},{13,1,11,2040},
      {13,2,16,0xffe2},{13,3,16,0xffe3},{13,4,16,0xffe4},{13,5,16,0xffe5},{13,6,16,0xffe6},{13,7,16,0xffe7},
      {13,8,16,0xffe8},{13,9,16,0xffe9},{13,10,16,0xffea},{14,1,16,0xffeb},{14,2,16,0xffec},{14,3,16,0xffed},
      {14,4,16,0xffee},{14,5,16,0xffef},{14,6,16,0xfff0},{14,7,16,0xfff1},{14,8,16,0xfff2},{14,9,16,0xfff3},
      {14,10,16,0xfff4},{15,0,11,2041},{15,1,16,0xfff5},{15,2,16,0xfff6},{15,3,16,0xfff7},{15,4,16,0xfff8},
      {15,5,16,0xfff9},{15,6,16,0xfffa},{15,7,16,0xfffb},{15,8,16,0xfffc},{15,9,16,0xfffd},{15,10,16,0xfffe}};

  
  struct dc_code {             /* structure model for the dc values  */ 
     unsigned int cate;        /* catagory  */
     unsigned int code_len;    /* size of the code  */
     unsigned int code_word;   /*  amplitude   */
  };


  struct hold_it {              /* utility structure   */
     unsigned long transmit;     /* value  */
     unsigned shift_fac;         /* # of bits  */
  };

  struct dc_code dc_table[12] = {         /* table for dc values  */
				     { 0, 2,   0}, {1,  3,  2},
				     { 2, 3,   3}, {3,  3,  4},
				     { 4, 3,   5}, {5,  3,  6},
				     { 6, 4,  14}, {7,  5, 30},
				     { 8, 6,  62}, {9,  7, 126},
				     {10, 8, 254}, {11, 9, 510},
				     };
  int init_array[8][8]={      					/*  zig zag array  */
			  0,  1,  5,  6, 14, 15, 27, 28,
			  2,  4,  7, 13, 16, 26, 29, 42,
			  3,  8, 12, 17, 25, 30, 41, 43,
			  9, 11, 18, 24, 31, 40, 44, 53,
		         10, 19, 23, 32, 39, 45, 52, 54,
		         20, 22, 33, 38, 46, 51, 55, 60,
		         21, 34, 37, 47, 50, 56, 59, 61,
		         35, 36, 48, 49, 57, 58, 62, 63 };


   sbase = 0;   /* initialize */
   n=0;
   nn=0;
   /* decoding for the dc coefficient       */
  i=0;
  fseek(input_file,current_pos,0);   /* position the pointer  */
  fread(readbit,4,2,input_file);
  
  for(ii=2;ii<=16; ii++){         /* for min code length to max code length  */
  
      	if( ii+sbase >= 32) {
		temp = ii+sbase-32;
		need_byte = 1;
		tempbit = ((readbit[n] << temp)&lut0[ii]) | ((readbit[n+1] >> (32-temp))&lut0[temp]);
	 }
		
	else	tempbit = readbit[n] >> (32 - ii - sbase);
	
	tempbit = tempbit & lut0[ii];
	/* printf("readbit = %x, sbase = %d, ii = %d, tempbit = %8x\n",readbit[n],sbase,ii,tempbit);*/
	for (nn=0; nn<=11; nn++) {   /* for # of entries intable */
	    if( (*(dc_table + nn)).code_len == ii && (*(dc_table+nn)).code_word == tempbit){
		siz = (*(dc_table+nn)).cate;
		temp = ii+sbase;
		if(32-temp < siz) {
		     if (temp > 32) {
			  temp = temp - 32 + siz;
			  need_byte = 1;
			  tempword1 = ((readbit[n+1] >> (32-temp))&lut0[siz]);	
		      }	
		     else {
			   temp += siz - 32;
			   need_byte = 1;
			   tempword1 = ((readbit[n] << temp)&lut0[siz]) | ((readbit[n+1] >> (32-temp))&lut0[temp]);	
		      }
		}
		else {
						
		      tempword1 = (readbit[n] >> (32-ii-sbase-siz));
		}
		
		tempword2 = tempword1 & lut0[siz];
	
		temp1 = ii + siz;
		if( (tempword1 & lut2[siz]) != lut2[siz]) tempword2 = (tempword2|lut[siz])+1;
		/*printf("found code = %d, size = %d\n",tempword2,siz);*/
		deout[i] = tempword2;				
		nn = 11;
		ii=16;
		break;
	   }	 /*   end of if   */
       }       /*  end of for nn */
   }           /* end of for ii   */

   sbase += temp1;
   if (sbase >=32) {
	sbase -= 32;
	n++; printf(" n = %d\n" , n);
	fread(&readbit[n+1],4,1,input_file);
        if ( need_byte ==1){
	   need_byte = 0;
	}
   }   

/*   now decoding for the ac coefficient  */
  for (i=1; i <= 64; i++) {   
       need_byte = 0;
      for(ii=2;ii<=16; ii++){     /*  for min<>max code length  */
           prev = 0;
      	  if( ii+sbase >= 32) {
	       temp = ii+sbase-32; prev = 1;
	       tempbit = ((readbit[n] << temp)&lut0[ii]) | ((readbit[n+1] >> (32-temp))&lut0[temp]);
	  }
		
	  else	{   tempbit = readbit[n] >> (32 - ii - sbase); prev = 0; }
		
	   tempbit = tempbit & lut0[ii];
	   for (nn=0; nn<=161; nn++) {
	      	if( (*(huff_table+nn)).code_len == ii && (*(huff_table+nn)).code_word == tempbit){
			siz = (*(huff_table+nn)).size;
			runs = (*(huff_table+nn)).run;
			temp = ii+sbase;
			if ( prev == 1 )  need_byte = 1;
		     /*   printf("siz = %d, runs = %d, i = %d\n",siz,runs,i);*/
			if( (siz != 0) || (runs != 0)) {
				if  (32-temp < siz) {
					if (temp > 32) {
					     temp = temp - 32 + siz;
					     need_byte = 1; /* printf("second\t");*/
					     tempword1 = ((readbit[n+1] >> (32-temp))&lut0[siz]);	
					 }	
					else {
					      temp += siz - 32;
					      need_byte = 1; /* printf("third\t");*/
					      tempword1 = ((readbit[n] << temp)&lut0[siz]) | ((readbit[n+1] >>
					                                                      (32-temp))&lut0[temp]);	
					}
				}
				else  tempword1 = (readbit[n] >> (32-ii-sbase-siz));
		  		tempword2 = tempword1 & lut0[siz];
				temp1 = ii + siz;
		 	        if( (tempword1 & lut2[siz]) != lut2[siz]) tempword2 = (tempword2|lut[siz])+1;
				iii = runs + i; if ( iii > 64)  iii = 64;
			      /* printf("found code = %d, size = %d, iii = %d, i = %d\n",tempword2,siz, iii, i);*/
			}
			else {
			      iii=64;
			      temp1 = ii;
			}
			for (; i<iii; i++) deout[i] = 0;
		       if ( siz ==0 && iii != 64 )  i = i-1;
			else
			   deout[i] = tempword2;				
		  	ii=16;
			break;
	        }  /*   end of if code  */
	   }      /*  end of for nn     break applies here */
      }         /*  end of for ii,  ii = 16 applies here  */

      sbase += temp1;
     
      if (sbase >=32) {
         sbase -= 32;
         n++;                  /* printf(" n = %d , %d\n" , n,i);*/
	 curent_pos = ftell(input_file);
	 if ( curent_pos < total_bytes && i < 64)
	    fread(&readbit[n+1],4,1,input_file);
	 if ( need_byte ==1 && i == 63)
	      need_byte = 0;

      }
      
   /*   printf("sbase = %d, temp1 = %d, i = %d\n",sbase,temp1, i);*/
  }	/*   end of i   */

  for (i=0; i<=7; i++) { 
     for (j=0; j<=7; j++)  {
	 (((short**)input_Image->image_ptr[color]->rptr)[(rows+i)][(cols+j)]) = (short) deout[init_array[i][j]];
     }
  }
  curent_pos = ftell(input_file);
  
  if ( curent_pos == total_bytes){ 
     return ( curent_pos );
  }
  else
     return (need_byte == 0 ? (curent_pos - 4) : curent_pos);
}

/*  when i = 64 or EOB or the EOF  file write the resulte to image block and go back  */

/*  
    huff_decompress: This is utility routine which calls the HUFFDECOMPRESS routine to 
    de compress the image from the given file. it takes file name as argument and returns a 
    decoded or uncompressed image.
*/

Image *huff_decompress(input_Image, filename)
Image *input_Image; 
char  *filename;
 {
   int     total_bytes,rows,cols,d,color,x,y,k,current_pos =0,n=80;
   FILE    *input_file;
   
			 
	#ifndef WIN32
   if ((input_file = fopen(filename,"r")) == NULL) {      /* open compreed file */
	#else
   if ((input_file = fopen(filename,"rb")) == NULL) {      /* open compreed file */
	#endif
	printf("cannot open %s for output\n", filename);		/* input compressed file */
	exit(0);
   }
   fseek(input_file,0,2);
   total_bytes = ftell(input_file);
   fseek(input_file,0,0);
   printf("# of bytes in %s (compressed file) =  %d bytes\n",filename, total_bytes);
	
  rows  = (int) input_Image->image_ptr[0]->rows;        /* # of rows */
  cols  = (int) input_Image->image_ptr[0]->cols;        /* # of cols */
  if (input_Image->bands == 3) color = 3; else color = 1;  /* color ? */
    
  
  for ( k = 0; k < color; k++){   /* main routine for block by block  */
    x = 0; 
    while( x < rows  ){
       y =0;
       while( y < cols ){
          fseek(input_file,0,0); 
          current_pos =  HUF_DECOMPRESS(input_Image, x, y,k, input_file, total_bytes,n);/* decompress */
	  n = current_pos;
          y+=8; 
       }
     x+=8;
    }
  } 
  return (input_Image);   /* return the decompressed image  */
  }
/* end of compression routine  */
