/***************************************************************************
* ======================================================================
* Computer Vision/Image Processing Tool Project - Dr. Scott Umbaugh SIUE
* ======================================================================
*
*             File Name: tcompress.c
*           Description: Test program for functions in libcompress.a.
*         Related Files: Imakefile, libcompress.a
*   Initial Coding Date: 1/27/94
*           Portability: Standard (ANSI) C
*             Credit(s): Gregory Hance
*                        Southern Illinois University @ Edwardsville
*
* RCS (Revision Control System) Information
* ...Added automatically by RCS 
*
** $Log: tc.c,v $
** Revision 1.5  1997/03/08 17:04:50  yawei
** Swaped CVIP_YES and CVIP_NO
**
** Revision 1.4  1997/03/08 00:56:36  yawei
** Name Changes:
** 	BOOLEAN ==> CVIP_BOOLEAN
** 	FALSE ==> CVIP_YES
** 	TRUE ==> CVIP_NO
** 	BYTE ==> CVIP_BYTE
** 	SHORT ==> CVIP_SHORT
** 	INTEGER ==> CVIP_INTEGER
** 	FLOAT ==> CVIP_FLOAT
** 	DOUBLE ==> CVIP_DOUBLE
** 	TYPE ==> CVIP_TYPE
**
** Revision 1.3  1997/01/16 21:18:55  yawei
** Global Change:  Image_FORMAT ==> IMAGE_FORMAT
**
 * Revision 1.2  1997/01/15  17:15:34  yawei
 * Global Change:  IMAGE ==> Image
 *
 * Revision 1.1  1994/02/16  21:05:50  kjoelena
 * Initial revision
 *
*
****************************************************************************/
/* Define Preprocessor statements */
#include "CVIPtoolkit.h"
#include "CVIPconvert.h"
#include "CVIPdef.h"
#include "CVIPcompress.h"

main(int argc, char *argv[])
{
/* Declare variables local to main() */
Image *cvipImage;
IMAGE_FORMAT format;
char inputfile[50], outputfile[50];
const char *usage = "[-display] [infile] [outfile]", *fn = argv[0];
int argn=1;
CVIP_BOOLEAN display = CVIP_NO;

/* initialize global command options */
cvip_init( &argc, argv );

/* process command line options */
while ( argn < argc && argv[argn][0] == '-' && argv[argn][1] != '\0' ) {
   if ( cvip_keymatch( argv[argn], "-display", 2 ) )
      display = CVIP_YES;
   else
      cvip_usage( fn, usage );
   argn++;
}

/* decide where to read input from and where to send output */
if(argc > argn) {
   if ( (argv[argn][0] == '-' && argv[argn][1] != '\0') || sscanf( argv[argn], "%s", inputfile ) != 1 )
      cvip_usage( fn, usage );
   argn++;
   if ( argc > argn ) {
      if ( (argv[argn][0] == '-' && argv[argn][1] != '\0') || sscanf( argv[argn], "%s", outputfile ) != 1 )
         cvip_usage( fn, usage );
      argn++;
   }
   else
      (void) strcpy(outputfile, "-");
   if( argc != argn )
      cvip_usage( fn, usage );
}
else {
   (void) strcpy(inputfile, "-");
   (void) strcpy(outputfile, "-");
}

format = CVIPgetformat(inputfile);
cvipImage = CVIPreadimage(inputfile, format, 1);


CVIPwriteimage(cvipImage, outputfile, CVIP_NO, CVIP_NO, format, 1);

if( display && strcmp(outputfile,"-") )
   cvip_display( outputfile, format );
else if( display )
   cvip_error(fn, "can not display an image directed to \"stdout\".");

image_free(cvipImage);
exit(0);
} /* End Main */

     
         





   
      
