/***************************************************************************
* ======================================================================
* Computer Vision/ImageRec Processing Tool Project - Dr. Scott Umbaugh SIUE
* ======================================================================
*
*             File Name: cvipInit.c
*           Description: 
*         Related Files: 
*   Initial Coding Date: Tue Nov 28 21:51:37 CST 1995
*           Portability: Standard (ANSI) C
*            References:
*             Author(s): Kun Luo, Greg Hance,
*                        Southern Illinois University @ Edwardsville
*
** Copyright (C) 1995 SIUE - by Kun Luo, Greg Hance.
**
** Permission to use, copy, modify, and distribute this software and its
** documentation for any purpose and without fee is hereby granted, provided
** that the above copyright notice appear in all copies and that both that
** copyright notice and this permission notice appear in supporting
** documentation.  This software is provided "as is" without express or
** implied warranty.
**
* RCS (Revision Control System) Information
* ...Added automatically by RCS 
*
** $Log: cvipInit.c,v $
** Revision 1.23  1997/08/08 14:56:22  yawei
** Win32 temporary fix
**
** Revision 1.22  1997/08/08 14:39:45  yawei
** final fix the Win32 temp directory
**
** Revision 1.21  1997/08/08 14:07:36  yawei
** fixed temporary directory for Win32
**
** Revision 1.20  1997/08/08 13:30:42  yawei
** added del for WinNT
**
** Revision 1.19  1997/08/06 21:51:23  smakam
** made the rcs file consistent
**
** Revision 1.18  1997/06/17 20:54:47  yawei
** added NULL to the end of execlp
**
** Revision 1.17  1997/06/16 20:57:51  yawei
** added directives for SGI
**
** Revision 1.16  1997/06/05 22:00:47  yawei
** cleaned up function prototype problems
**
** Revision 1.15  1997/06/02 19:34:27  yawei
** modified setDisplay_Image
**
** Revision 1.14  1997/05/29 22:28:43  yawei
** modified for WIN32 temporary directory handling
**
** Revision 1.13  1997/05/27 22:49:30  yawei
** modified for WIN32
**
** Revision 1.12  1997/05/19 03:57:18  yawei
** added Win32 support
**
** Revision 1.11  1997/01/15 17:18:21  yawei
** Global Change:  IMAGE ==> Image
**
 * Revision 1.10  1996/09/20  00:32:20  yawei
 * Changed default viewer to RamViewer.
 *
 * Revision 1.9  1996/07/23  02:52:04  wli
 * when 'quit' an displaying image, that question shouldn't be asked
 *
 * Revision 1.8  1996/07/18  23:41:42  wli
 * change cviptcl to be: not ask that question at the begining, assume
 * it is yes; before exit, if more than one dirs exist, ask that question,
 * if yes, only remove the dir associated with current process; if no,
 * remove all dirs under .cvip_temp/, then exit
 *
 * Revision 1.7  1996/07/02  19:15:01  akjoele
 *  Defined M_FILE_DEFINED to the compiler, so compilation
 * does not bomb on other systems.  I really dislike all these
 * special define directives to get around compiling problems.
 *
 * Revision 1.6  1996/07/02  18:56:14  akjoele
 * Changed ordering of include files <sys/types.h> and
 * <sys/stat.h> so that <sys/types.h> is first.  This eliminates
 * incompatibility on systems such as IRIX whose <sys/stat.h>
 * does not include <sys/types.h>
 *
 * Revision 1.5  1996/04/02  17:52:25  kluo
 * check in
 *
 * Revision 1.4  1996/02/24  00:31:09  kluo
 * add new remove directory handler
 *
 * Revision 1.3  1996/02/16  04:35:03  kluo
 * add delete path handler
 *
 * Revision 1.2  1996/01/19  22:32:57  kluo
 * add ImagePATH enviroment into
 *
 * Revision 1.1  1996/01/16  16:30:49  kluo
 * Initial revision
 *
* ****************************************************************************/

/*This define is to remove the error message from sun SPARC compiler when using
  -Xc option. */
#define __EXTENSIONS__
#define M_FILE_DEFINED

/* to get sigaction compiled on SGI */
#ifdef __sgi
#define _SVR4_SOURCE
#endif

#include <sys/stat.h>
#include <sys/types.h>
#include <signal.h>
#include <tcl.h>
#include "cvipInit.h"
#include "CVIPmanager.h"
#include "cvipImg.h"
#include <errno.h>
#ifndef WIN32
#include <unistd.h>
#include <pwd.h>
#include <dirent.h>
#else
#include <io.h>
#include <direct.h>
#endif

#if defined(i386BSD)
int ftw(char *path,int (*fn)(), int depth);
#elif !defined(WIN32)
#include <ftw.h>
#endif


typedef dlinked_list *ImagePathList;
ImagePathList	CVIPTCL_iplP;
extern char temp_dir[100];
#ifndef WIN32
static int tcl_prfile(char *fname, struct stat *statptr, int flag);
#else
static int tcl_prfile(char *fname, struct _stat *statptr, int flag);
#endif
static void setup_tcl_cvipdir(void);
static void remove_dir(ClientData clientdata);
#ifndef WIN32
extern pid_t cpid;
#else
extern int cpid;
#endif
static int process_id;


static int
initial_ImagePathList(
	ImagePathList	iplP,
	const char 	*name
)
{
	register int	i;
	char 		*valP, **pnP, *pP;
	int		n;

	valP = getEnv_CVIP(name);
	if(valP == NULL) return 0;

	pnP = seperateStrings_CVIP(valP, ':', &n);

	head_DLL(iplP);

	for(i=0; i < n; i++) {
		pP = malloc(strlen(pnP[i])+1);
		(void) strcpy(pP, pnP[i]);
   		addtail_DLL(iplP, pP);
	}

	free(*pnP);
	free(pnP);
	free(valP);

	return 1;
}

#ifndef WIN32
static int tcl_prfile(char *fname, struct stat *statptr, int flag){
#else
static int tcl_prfile(char *fname, struct _stat *statptr, int flag){
#endif

	char answer;
	char subdir[20];
	int count=0,len;

	if(strcmp(fname,temp_dir)==0)
		return 0;
        fflush(stdin);
#ifndef WIN32
	strcpy(subdir,"image");
#else
	strcpy(subdir,"");
#endif
	strcat(temp_dir,subdir);
	len=strlen(temp_dir);
	while(1){/*loop until find a directory name which doesn't exist*/
		temp_dir[len+1]=0;
		temp_dir[len]='0'+count;
		count++;
#ifndef WIN32
		if(access(temp_dir,0)==-1)
#else
		if(_access(temp_dir,0)==-1)
#endif
			break;
	}
	return 1;
}
	
#ifndef WIN32
static int ntcl_prfile(char *fname, struct stat *statptr, int flag)
#else
static int ntcl_prfile(char *fname, struct _stat *statptr, int flag)
#endif
{       char answer;

        if (strcmp(fname, temp_dir) == 0) return 0;

        fflush(stdin);
#ifdef WIN32
		  if(strstr(temp_dir,".cvip_temp")) {
		      system(strcat("deltree /Y ", temp_dir));
		      system(strcat("del /F /S /Q ", temp_dir));
			}
        return 1;
#endif
        printf("\nDo you have other CVIP program running?(y/n)");
        answer=getchar();
        if(answer=='n'||answer=='N')
           switch(fork()){
              case 0:
                 if(strstr(temp_dir,".cvip_temp"))
                      execlp("rm","rm","-rf",temp_dir, (char *)NULL);
		      /*use exec is more efficient than system*/
                 fprintf(stderr,"\n should not happen");
                 exit(1);
              case -1:
                 fprintf(stderr,"\n Error in fork");
                 exit(1);
              default:
                 wait(NULL);
                 mkdir(temp_dir,0755);
           }
        return 1;
}


static void setup_tcl_cvipdir(void){
	char *home;
	char *user;
	char old_direc[100];
#ifndef WIN32
	struct passwd *pwentry;
#endif

#ifndef WIN32
	if((pwentry=getpwuid(getuid()))==NULL){/*get home directory of the user*/
		fprintf(stderr,"\nCannot find the user'entry from passwd file.");
		exit(1);
	}
	strcpy(temp_dir,pwentry->pw_dir);
	strcat(temp_dir,"/.cvip_temp/"); /*the directory name is "cvip_temp" */
#else
	if (getenv("temp")) {
		strcpy(temp_dir, getenv("temp"));
		strcat(temp_dir, "\\.cvip_temp");
		if (_access(temp_dir, 0) < 0) {
			int i, len = strlen(temp_dir); 
			char *copyDir = (char *) malloc(sizeof(char) * (len + 1));
			for (i = 0; i < len; i++) if (temp_dir[i] == '/') temp_dir[i] = '\\';

			memset(copyDir, 0, len+1);
			for (i = 0; i < len; i++) {
				if (temp_dir[i] == '\\') {
					strncpy(copyDir, temp_dir, i+1);
					if (_access(copyDir, 0) < 0) _mkdir(copyDir);
				}
			}
			if (_access(temp_dir, 0) < 0) _mkdir(temp_dir);
			free(copyDir);
		}
	} else  {
		strcpy(temp_dir, "\\temp\\.cvip_temp");
		if (_access("\\temp", 0) < 0) _mkdir("\\temp");
		if (_access("\\temp\\.cvip_temp", 0) < 0) _mkdir("\\temp\\.cvip_temp");
	}
#endif
#ifndef WIN32
	if(access(temp_dir,0)==-1)
	mkdir(temp_dir,0755);/*if this directory doesn't exist, make one*/
#else
	if(_access(temp_dir,0)==-1)
	_mkdir(temp_dir);
#endif
	strcpy(old_direc,temp_dir);/*save the temp_dir*/
#ifndef WIN32
	ftw(temp_dir,tcl_prfile,2);/*check if there is files in this dirctory*/
	if(strcmp(temp_dir,old_direc)==0)/*if no change, append "image" behind it*/
		strcat(temp_dir,"image");
	mkdir(temp_dir,0755);
#endif
}

static void remove_dir(ClientData clientdata){
#ifndef WIN32
    DIR * dptr;
#endif
    struct dirent *entry;
    char filename[150];
    char *pc;
	
#ifndef WIN32
    if(process_id == getpid()){
        if(strstr(temp_dir,".cvip_temp")){
	    dptr = opendir(temp_dir);
	    if(dptr == NULL){
		fprintf(stderr,"Error: directory %s doesn't exist",temp_dir);
		return;
	    }
	    while((entry=readdir(dptr)) != NULL){
		if(strcmp(entry->d_name,".") != 0 &&
		   		strcmp(entry->d_name,"..") != 0){
		     strcpy(filename,temp_dir);
		     strcat(filename,"/");
		     strcat(filename,entry->d_name);
		     if(unlink(filename)!=0)
			 perror("file");
		}
	    }
	    closedir(dptr);
	    if(rmdir(temp_dir)!=0){
	        dptr = opendir(temp_dir);
	        while((entry=readdir(dptr)) != NULL){
		  if(strcmp(entry->d_name,".") != 0 &&
		   		strcmp(entry->d_name,"..") != 0){
			fprintf(stderr,"%s",entry->d_name);
		  }
	        }
	    }
	}
    }
    if (getpid() != cpid)
    { if (pc=strstr(temp_dir,".cvip_temp")) {pc+=10; *pc = 0;}
      ftw(temp_dir, ntcl_prfile, 2);
    }
#else
	 system(strcat("deltree /Y ", temp_dir));
    system(strcat("del /F /S /Q ", temp_dir));
#endif
}


/*
** ---------------------------
** Initialization procedures.
** ---------------------------
*/

int CvipTcl_Init( Tcl_Interp *interp ){
#ifndef WIN32
	struct sigaction act;
#endif
	void justwait(int signum); 

	CVIPTCL_iplP = new_DLL();
	if(CVIPTCL_iplP == NULL){
		return TCL_ERROR;
	}

   	if(!initial_ImagePathList(CVIPTCL_iplP, "CVIP_IMGPATH")) {
		printf("warning: CVIP_IMGPATH environment variable not set\n");
		printf("warning: image path search list being set to NULL\n");
	}
	setDisplay_Image("RamViewer", "Default");
	setup_tcl_cvipdir();
#ifndef WIN32
	   process_id = getpid();
#else
	   process_id = _getpid();
#endif
#ifndef WIN32
 	act.sa_handler = exit;
	sigemptyset(&act.sa_mask);
	act.sa_flags = 0;
	if(sigaction(SIGINT, &act,NULL) <0){
	 	fprintf(stderr,"Fatal: Cannot setup signal handler for interrupt\n");
		exit(1);
	}
#endif
	Tcl_CreateExitHandler(remove_dir,NULL);

	if (CvipImage_Init(interp) == TCL_ERROR) {
		return TCL_ERROR;
	}

	return TCL_OK;
}

