/***************************************************************************
* **********************************************************************
* Computer Vision/Image Processing Tool Project - Dr. Scott Umbaugh SIUE
* **********************************************************************
*
*             File Name: cviptcl.c
*           Description: one of two files for tcl shell for CVIPtools 
*         Related Files: cvipImg.c 
*   Initial Coding Date: January 1996 
*           Portability: Standard (ANSI) C
*            References:
*             Author(s): Kun Luo, Zhen Li, Arve Kjoelen, 
*                        Wenxing Li, Yansheng Wei 
*                        Southern Illinois University @ Edwardsville
*
** Copyright (C) 1996 SIUE - by Scott Umbaugh.
**
** Permission to use, copy, modify, and distribute this software and its
** documentation for any purpose and without fee is hereby granted, provided
** that the above copyright notice appear in all copies and that both that
** copyright notice and this permission notice appear in supporting
** documentation.  This software is provided "as is" without express or
** implied warranty.
**
****************************************************************************/
/*$Log: cviptcl.c,v $
 *Revision 1.73  1997/07/09 20:51:51  smakam
 *removed the delete_Image after the convolve_filter function becauseimage is deleted inside the function
 *
 *Revision 1.72  1997/06/16 20:53:51  yawei
 *added cvip_free for SGI
 *
 *Revision 1.71  1997/04/16 14:38:15  yawei
 *duplicate image before gray_linear
 *
 *Revision 1.70  1997/04/16 14:30:30  yawei
 *added another parameter for gray_linear
 *
 *Revision 1.69  1997/03/28 22:07:56  yawei
 *changed divide usage message
 *
 *Revision 1.68  1997/03/21 01:46:49  yawei
 *Added copy_paste
 *
 *Revision 1.67  1997/03/08 17:06:44  yawei
 *Swaped CVIP_YES and CVIP_NO
 *
 *Revision 1.66  1997/03/08 01:00:29  yawei
 *Name Changes:
 *	BOOLEAN ==> CVIP_BOOLEAN
 *	FALSE ==> CVIP_NO
 *	TRUE ==> CVIP_YES
 *	BYTE ==> CVIP_BYTE
 *	SHORT ==> CVIP_SHORT
 *	INTEGER ==> CVIP_INTEGER
 *	FLOAT ==> CVIP_FLOAT
 *	DOUBLE ==> CVIP_DOUBLE
 *	TYPE ==> CVIP_TYPE
 *
 *Revision 1.65  1997/01/15 17:18:49  yawei
 *Global Change:  IMAGE ==> Image
 *
 * Revision 1.64  1997/01/11  05:31:14  yawei
 * minor change
 *
 * Revision 1.63  1996/12/31  20:17:36  yawei
 * function name change
 *
 * Revision 1.62  1996/12/27  05:53:36  yawei
 * changed function names, get rid of unnecessary function prototype
 * declaration.
 *
 * Revision 1.61  1996/12/24  20:17:20  yawei
 * changed:
 *   convolve_Image ==> convolve_filter
 *
 * Revision 1.60  1996/12/24  00:46:50  yawei
 * *** empty log message ***
 *
 * Revision 1.59  1996/12/23  18:56:52  yawei
 * removed "MedianCut.h"
 *
 * Revision 1.58  1996/12/23  17:23:17  yawei
 * Changed function name:
 *  threshold_Image ==> threshold_segment
 *
 * Revision 1.57  1996/12/23  15:33:25  yawei
 * changed multiresolution .
 *
 * Revision 1.56  1996/12/23  15:05:35  yawei
 * *** empty log message ***
 *
 * Revision 1.55  1996/12/22  16:58:39  yawei
 * Changed function names
 *
 * Revision 1.54  1996/12/18  04:23:05  yawei
 * Changed ypmean_filter usage from Ypmean_filter to ypmean_filter
 *
 * Revision 1.53  1996/12/18  03:50:15  yawei
 * Changed function prototypes
 *
 * Revision 1.52  1996/12/17  22:53:06  yawei
 * Finished the clean up of the geometry library
 *
 * Revision 1.51  1996/12/16  23:39:42  yawei
 * Changed function names
 *
 * Revision 1.50  1996/12/16  21:49:33  yawei
 * Changed function names
 *
 * Revision 1.49  1996/11/16  00:54:07  sumbaug
 * corrected spelling on gray_multiply usage note (clipping)
 *
 * Revision 1.48  1996/11/14  04:59:44  yawei
 * Added threshval_inbyte to binary thresholding
 *
 * Revision 1.47  1996/11/14  03:56:38  yawei
 * Changed unsharp to used percentage parameters for hist_stretch.
 *
 * Revision 1.46  1996/11/08  16:39:35  mzuke
 * changed error prompt for S_ImageGRAY_QUANT
 *
 * Revision 1.45  1996/11/08  05:09:40  wli
 * modify hist_stretch to call new function hist_stretch() instead of call
 * remap_Image() only.
 *
 * Revision 1.44  1996/11/07  22:58:55  yawei
 * 1) Changed gray_multiply so it will take another parameter about whether it will CVIP_BYTE clipping
 *    or not.
 *
 * Revision 1.43  1996/10/15  03:14:29  yawei
 * changed dataremap in Median_cut to fix SEG_FLT bug
 *
 * Revision 1.42  1996/09/22  20:24:17  yawei
 * added data_remap in "median_cut"
 *
 * Revision 1.41  1996/09/13  16:38:30  wli
 * modify gray_linear to take one more parameter
 *
 * Revision 1.40  1996/09/09  14:21:38  mzuke
 * changed usage message for square from 'ending column' and 'ending row'
 * to 'rectangle width' and 'rectangle height'
 *
 * Revision 1.39  1996/09/04  01:05:32  sumbaug
 * added CVIPtools header to file
 *
 * Revision 1.38  1996/08/29  21:04:46  sumbaug
 * modified ussage message for morph_open, morph_close, morph_erode, morph_dilate . Specifically for the kernal type cross.
 *
 * Revision 1.37  1996/08/27  03:07:59  wli
 * modify usage note for morph_close, morph_open,
 * morph_erode and morph_dilate
 *
 * Revision 1.36  1996/08/21  03:47:59  wli
 * put new kernels into cviptcl
 * for morphological operations
 *
 * Revision 1.35  1996/08/10  10:22:50  wli
 * lots of usage changes
 *
 * Revision 1.34  1996/08/10  03:07:03  wli
 * add error checking for 'projection'
 *
 * Revision 1.33  1996/08/08  03:55:36  wli
 * put gray_multiply
 *
 * Revision 1.32  1996/08/03  01:14:21  wli
 * change GUI_Texture_Setup() to be split_merge()
 *
 * Revision 1.31  1996/08/03  00:56:35  wli
 * put 'irregular' into cviptcl
 * modify 'multi_resolution'
 *
 * Revision 1.30  1996/07/31  05:41:39  wli
 * modify 'projection' to call size_norm_projection
 * modify 'gray_linear' to call new GrayLin_Image()
 *
 * Revision 1.29  1996/07/26  19:27:29  akjoele
 * Added include file CVIPxformfilter.h
 *
 * Revision 1.28  1996/07/25  02:52:34  wli
 * modify 'multi_resolution' to call Steve Costello's S&M segmentation functions
 *
 * Revision 1.27  1996/07/17  22:31:19  wli
 * for texture()
 *
 * Revision 1.26  1996/07/13  08:53:32  wli
 * put GUI_Texture_Setup() and rst_features() into CVIPTCL
 *
 * Revision 1.25  1996/07/09  23:04:39  mzuke
 * changed usage notes for spectral_feature
 * from <no_of_bands> to <no_of_rings>
 *
 * Revision 1.24  1996/07/08  07:28:24  wli
 * add hist_feature() and spectral_feature()
 * change alsm to orientation
 * modify projection()
 *
 * Revision 1.23  1996/07/03  00:20:27  wli
 * modify projection() so that 0's filtered out before it returns
 *
 * Revision 1.22  1996/07/02  19:13:49  akjoele
 * Defined M_FILE_DEFINED to the compiler, so compilation
 * does not bomb on other systems.  I really dislike all these
 * special define directives to get around compiling problems.
 *
 * Revision 1.21  1996/07/01  07:46:10  wli
 * add area(), aspect(), alsm(), centroid() and projection(),
 * modify irregular()
 *
 * Revision 1.20  1996/06/26  00:21:38  wli
 * modify euler, perimeter and thinness to accept row and column
 * coordinates in place of gray level values for selecting an
 * object of interest
 *
 * Revision 1.19  1996/05/06  00:01:26  zli
 * change the name of blur_spatial to gauss_blur
 *
 * Revision 1.18  1996/03/27  02:06:26  zli
 * *** empty log message ***
 *
 * Revision 1.17  1996/03/27  01:41:41  zli
 * add commands of cosine_wave, square_wave, display_mesh, area_points
 *
 * Revision 1.16  1996/03/20  04:15:40  zli
 * add commands for creating simple image
 *
 * Revision 1.15  1996/03/13  04:01:54  zli
 * data_remap		remap_Image()  & cast_Image()
 * gamma_noise		gamma_noise()
 * gaussian_noise		gaussian_noise()
 * negative_exp_noise	neg_exp_noise()
 * rayleigh_noise		rayleigh_noise()
 * speckle_noise		speckle_noise()
 * uniform_noise		uniform_noise()
 * edge_link		link_Image()
 * create_checkboard
 *
 * Revision 1.14  1996/02/27  21:24:48  zli
 * modify function for median_cut command
 *
 * Revision 1.14  1996/02/27  21:24:48  zli
 * modify function for median_cut command
 *
 * Revision 1.13  1996/02/21  03:26:39  zli
 * remove prototype declaration
 *
 * Revision 1.12  1996/02/21  03:07:36  zli
 * add comands of gray_linear, median_filter, median_cut, hist_slide, hist_shrink, hist_stretch, add, subtract, multiply, divide, and, or, xor, not
 *
 * Revision 1.11  1996/02/20  17:25:48  kluo
 * fix sharp
 *
 * Revision 1.10  1996/02/14  03:52:10  zli
 * *** empty log message ***
 *
 * Revision 1.9  1996/02/14  02:42:42  zli
 * add comands of thinness, irregularity, unsharp, spatial_quant, minimum_filter, mean_filter, alpha_filter, contra_filter, geometric_filter, harmonic_filter, ypmean_filter, midpoint_filter, maximum_filter
 *
 * Revision 1.8  1996/02/07  04:01:03  zli
 * add comands of "specify_spatial","euler","perimeter","gray_quant"
 *
 * Revision 1.7  1996/01/31  04:06:20  zli
 * add commands of cut_paste, multi_resolution, blur_spatial, diff_spatial, lowpass_spatial, laplacian_spatial, highpass_spatial
 *
 * Revision 1.6  1996/01/24  03:50:24  zli
 * add commands of morph_dilate, morph_erode, morph_open, morph_close, enlarge, translate
 *
 * Revision 1.5  1996/01/19  21:15:09  kluo
 * move CMD declaration into cvipImg.c
 *
 * Revision 1.4  1996/01/17  02:50:21  zli
 * add function of CMD ImageSmooth
 * add function of CMD ImageSharp
 * add function of CMD ImageRasterFilter
 * add function of CMD ImageBinthresh
 * add the mainframe of CMD ImageMorpho and the function chk_bin called in it
 *
 * Revision 1.3  1996/01/16  19:31:18  kluo
 * add include header file
 *
 * Revision 1.2  1996/01/16  19:28:  kluo
 * add log title
 **/
#define M_FILE_DEFINED
#include <tcl.h>

#include "cvipImg.h"
#include "CVIPdef.h"
#include "CVIPimage.h"
#include "CVIPtools.h"
#include "CVIPfs.h"
#include <limits.h>
#include "CVIPtoolkit.h"
#include "CVIPmesh.h"
#include "CVIPhisto.h"
#include "newfeature.h"
#include "CVIPtexture.h"
#include "CVIPxformfilter.h"
#include "CVIPdrawimg.h"
#include "CVIPsegment.h"
#include "CVIPmorph.h"
#include "waves.h"
#include "CVIPgeometry.h"
#include "CVIPspfltr.h"
#include "CVIPnoise.h"
#include "CVIParithlogic.h"

extern void quite_CVIP(void);
extern Matrix* get_default_filter(PROGRAMS, int, int);
extern long area(Image * labeledImage, int r, int c);
extern int * centroid(Image * labeledImage, int r, int c);
extern double orientation(Image * labeledImage, int r, int c);
extern double aspect(Image * labeledImage, int r, int c);
extern int * projection(Image * labeledImage, int r, int c, int height, int width);
extern int euler(Image *, int, int);
extern int perimeter(Image *, int, int);
extern double thinness(Image *, int, int);
extern double irregular(Image *, int, int);
extern double * hist_feature(Image * originalImage, Image * labeledImage, int r, int c);
extern TEXTURE * texture(const Image *, const Image *, int, int, int, long int, int);
extern double * rst_invariant(Image *label_image, int r, int c);

extern Image *remap_Image(const Image *imageP, CVIP_TYPE dtype, long	dmin, long dmax);

/*
** ------------------------
** 	Prototypes.
** ------------------------
*/


int User_Init( Tcl_Interp *interp );

static Image * convertImage_Queue(Image *inputImage, 
			CVIP_TYPE *desir_type, int num_type){
    Image * copyImage;
    int counter;
    char *name[] ={"CVIP_BYTE","CVIP_SHORT","CVIP_INTEGER","CVIP_FLOAT","CVIP_DOUBLE"};

    for(counter = 0; counter < num_type; counter++){
	if(inputImage->bandP[0]->data_type == desir_type[counter]){
	     return (Image*)duplicate_Image(inputImage);
	}
    }
    if(inputImage->bandP[0]->data_type > desir_type[0]){
	fprintf(stderr,"Image is remapped into %s\n",name[desir_type[0]]);
	switch(desir_type[0]){
	    case CVIP_BYTE:
     		copyImage = (Image*)remap_Image(inputImage,CVIP_BYTE,0,255);
		break;
	    case CVIP_SHORT:
		copyImage =(Image*)remap_Image(inputImage,CVIP_SHORT,
						SHRT_MIN,SHRT_MAX);
		break;
	    case CVIP_INTEGER:
		copyImage =(Image*)remap_Image(inputImage,CVIP_INTEGER,
						LONG_MIN,LONG_MAX);
		break;
	    default:
		fprintf(stderr,"Is is a double precision image?\n");
	}
    }
    else{
	copyImage = duplicate_Image(inputImage);
	if(cast_Image(copyImage,desir_type[0]))
 	    fprintf(stderr,"Image is cast into %s\n",name[desir_type[0]]);
	else
	    return NULL;
    }

    return copyImage;
}


/*
** ----------------------
** 	  Functions.
** ----------------------
*/
/*
** -----------------------------
** 		Sub Commands.
** -----------------------------
*/

int
S_ImageSmooth(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP;
	CVIP_TYPE	required_type[5];
	int	num_type;

	required_type[0] = CVIP_BYTE;
	num_type = 1;
	

	if (argc != 4) {
		sprintf(interp->result, "Usage: cvip smooth <image> <kernel>\n\twhere <kernel> is an integer among 3, 5 and 7");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	imageP = convertImage_Queue(imageP,required_type,num_type);
	if ( (imageP = smooth_filter(imageP, atoi(argv[3]))) == (Image *)NULL)
		return TCL_ERROR;

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}


int
S_ImageSharp(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP;
	CVIP_TYPE	required_type[5];
	int	num_type;

	required_type[0] = CVIP_BYTE;
	num_type = 1;

	if (argc != 3) {
		sprintf(interp->result, "Usage: cvip sharp <image>");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	imageP = convertImage_Queue(imageP,required_type,num_type);
	image_sharp(imageP);

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}

int
S_ImageRasterFilter(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP;

	if (argc != 3) {
		sprintf(interp->result, "Usage: cvip raster_deblur <image>");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	imageP = image_duplicate(imageP);
	imageP = (Image*)raster_deblur_filter(imageP);
	if(imageP == NULL)
		return TCL_ERROR;

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}

int
S_ImageBinthresh(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP;
	CVIP_BOOLEAN thresh_inbyte = CVIP_NO;

	if (argc != 4 && argc != 5) {
		sprintf(interp->result, "Usage: cvip threshold <image> <threshval> <threshval in CVIP_BYTE (1/0) >");
		return TCL_ERROR;
	}

	if (argc == 5 && atoi(argv[4]) == 1) thresh_inbyte = CVIP_YES;

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	imageP = image_duplicate(imageP);
	imageP = (Image*)threshold_segment(imageP, atoi(argv[3]), thresh_inbyte);
	if(imageP == NULL)
		return TCL_ERROR;

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}




int
S_ImageMORPH_DILATE(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP;

	if (argc != 7) {
		sprintf(interp->result, "Usage: cvip morph_dilate <image> <kernel type (1-4)> <kernel dimension> <height> <width>\nThose arguments which are needed should be greater than 0; For argument(s) you don't need, pass -1\nwhere\tfor kernel type, 1 means disk; 2 square; 3 rectangle; 4 cross\n\tdisk doesn't need height and width\n\tsquare doesn't need width\n\tcross: height is used for thickness \n\tof lines and width is used for cross size");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	imageP = image_duplicate(imageP);
	imageP = (Image*)MorphDilate(imageP, atoi(argv[3]), atoi(argv[4]), atoi(argv[5]), atoi(argv[6]));
	if(imageP == NULL)
		return TCL_ERROR;

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}


int
S_ImageMORPH_ERODE(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP;

	if (argc != 7) {
		sprintf(interp->result, "Usage: cvip morph_erode <image> <kernel type (1-4)> <kernel dimension> <height> <width>\nThose arguments which are needed should be greater than 0; For argument(s) you don't need, pass -1\nwhere\tfor kernel type, 1 means disk; 2 square; 3 rectangle; 4 cross\n\tdisk doesn't need height and width\n\tsquare doesn't need width\n\tcross: height is used for thickness \n\tof lines and width is used for cross size");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	imageP = image_duplicate(imageP);
	imageP = (Image*)MorphErode(imageP, atoi(argv[3]), atoi(argv[4]), atoi(argv[5]), atoi(argv[6]));
	if(imageP == NULL)
		return TCL_ERROR;

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}

int
S_ImageMORPH_OPEN(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP;

	if (argc != 7) {
		sprintf(interp->result, "Usage: cvip morph_open <image> <kernel type (1-4)> <kernel dimension> <height> <width>\nThose arguments which are needed should be greater than 0; For argument(s) you don't need, pass -1\nwhere\tfor kernel type, 1 means disk; 2 square; 3 rectangle; 4 cross\n\tdisk doesn't need height and width\n\tsquare doesn't need width\n\tcross: height is used for thickness \n\tof lines and width is used for cross size");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	imageP = image_duplicate(imageP);
	imageP = (Image*)MorphOpen(imageP, atoi(argv[3]), atoi(argv[4]), atoi(argv[5]), atoi(argv[6]));
	if(imageP == NULL)
		return TCL_ERROR;

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}

int
S_ImageMORPH_CLOSE(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP;

	if (argc != 7) {
		sprintf(interp->result, "Usage: cvip morph_close <image> <kernel type (1-4)> <kernel dimension> <height> <width>\nThose arguments which are needed should be greater than 0; For argument(s) you don't need, pass -1\nwhere\tfor kernel type, 1 means disk; 2 square; 3 rectangle; 4 cross\n\tdisk doesn't need height and width\n\tsquare doesn't need width\n\tcross: height is used for thickness \n\tof lines and width is used for cross size");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	imageP = image_duplicate(imageP);
	imageP = (Image*)MorphClose(imageP, atoi(argv[3]), atoi(argv[4]), atoi(argv[5]), atoi(argv[6]));
	if(imageP == NULL)
		return TCL_ERROR;

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}


int
S_ImageENLARGE(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP;

	if (argc != 5) {
		sprintf(interp->result, "Usage: cvip enlarge <image> <rows> <cols>");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	imageP = image_duplicate(imageP);
	imageP = (Image*)enlarge(imageP, atoi(argv[3]), atoi(argv[4]));
	if(imageP == NULL)
		return TCL_ERROR;

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}

int
S_ImageTRANSLATE(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP;
	CVIP_BOOLEAN wrap = CVIP_YES;
	float fill_out;

	if (argc != 7) {
		sprintf(interp->result, "Usage: cvip translate <image> <wrap(y/n)>  <y_slide> <x_slide> <fill_out_value(0-255)>\n\tIf you do not know the value, pass -1 to it.");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
	if(argv[3][0]!='y')
		wrap = CVIP_NO;

	imageP = image_duplicate(imageP);
	fill_out = (float)atof(argv[6]);
	imageP = (Image*)translate(imageP, wrap, 0, 0, imageP->bandP[0]->rows,
		imageP->bandP[0]->cols, atoi(argv[4]), atoi(argv[5]), fill_out);
	if(imageP == NULL)
		return TCL_ERROR;

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}

int
S_ImageCOPY_PASTE(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image *imageP1, *imageP2;
	unsigned r_start, c_start, height, width, r_dest, c_dest, trans;
	CVIP_BOOLEAN transparent;
	float fill_out;

	if (argc != 11) {
		sprintf(interp->result, "Usage: cvip copy_paste <src_image> <dest_image> <r_start> <c_start> <height> <width> <r_dest> <c_dest> <transparent(1/0)>.");
		return TCL_ERROR;
	}

	r_start = atoi(argv[4]);
	c_start = atoi(argv[5]);
	if (r_start < 0 || c_start < 0) {
		sprintf(interp->result, "Negative starting point on the source image");
		return TCL_ERROR;
	}
	height  = atoi(argv[6]);
	width   = atoi(argv[7]);
	if (width <= 0 || height <= 0) {
		sprintf(interp->result, "positvie subimage size needed");
		return TCL_ERROR;
	}
	r_dest = atoi(argv[8]);
	c_dest = atoi(argv[9]);
	if (r_dest < 0 || c_dest < 0) {
		sprintf(interp->result, "Negative starting point on the destination image");
		return TCL_ERROR;
	}
	trans  = atoi(argv[10]);
	switch (trans) {
		case 0: transparent = CVIP_NO; break;
		case 1: transparent = CVIP_YES; break;
		default: 
		sprintf(interp->result, "transparent can only be 0 or 1");
		return TCL_ERROR;
	}
	if(!(imageP1 = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
	if(!(imageP2 = S_ImageGetFromHash(interp, argv[3]))) return TCL_ERROR;
	imageP1 = image_duplicate(imageP1);
	imageP2 = image_duplicate(imageP2);

	imageP2 = (Image*)copy_paste(imageP1, imageP2, r_start,  c_start, height, width,
		r_dest, c_dest, transparent);
	if(imageP2 == NULL)
		return TCL_ERROR;

	(void) S_ImageAddToHash(interp, imageP2, (char *)NULL);

	return TCL_OK;
}

int
S_ImageMULTIRES(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image       	* imageP;
	CVIP_TYPE		required_type = CVIP_BYTE;
	int		num_type = 1;

	int		choice;
	void 		*  parameter = (void *)malloc(2*sizeof(float));
	CVIP_BOOLEAN 	Run_PCT;

	if (argc != 7) {
		sprintf(interp->result, "Usage: cvip multi_resolution <image, color or gray> <choice(1-6)> <PCT (1/0=y/n)> <threshold/similarity> <percentage/pix distance>\n\tFor choice 1,2 & 3: <threshold> & <percentage> are -1\n\tFor choice 5:       <percentage> is -1\n\tFor choice 4 & 6:   no -1");
		return TCL_ERROR;
	}

	choice = atoi(argv[3]);
	Run_PCT = atoi(argv[4]);
        *((float *)parameter) = atof(argv[5]);
  	*((float *)parameter + 1) = atof(argv[6]);

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	imageP = convertImage_Queue(imageP,&required_type,num_type);
	imageP = (Image*)multi_resolution_segment(imageP, choice,
		parameter, Run_PCT);

	free((char *)parameter);

	if(imageP == NULL)
		return TCL_ERROR;
	else
		(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}

int
S_ImageBLUR_SPATIAL(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Matrix 	*mask;
	Image * imageP, *destImage;
	CVIP_TYPE	required_type[5];
	int	num_type;

	required_type[0] = CVIP_BYTE;
	required_type[1] = CVIP_SHORT;
	num_type = 2;
	

	if (argc != 4) {
		sprintf(interp->result, "Usage: cvip gauss_blur <image> <kernel(3, 5, 7 only)>");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	destImage = convertImage_Queue(imageP,required_type,num_type);
	mask = (Matrix*)get_default_filter(BLUR_SPATIAL,atoi(argv[3]),0);

	if (mask == NULL)
		return TCL_ERROR; 

	imageP = (Image*)convolve_filter(destImage, mask);
	/*delete_Image(destImage);*/

	if(imageP == NULL)
		return TCL_ERROR;
	else
		(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}

int
S_ImageDIFFERENCE_SPATIAL(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Matrix 	*mask;
	Image * imageP, *destImage;
	CVIP_TYPE	required_type[5];
	int	num_type;

	required_type[0] = CVIP_BYTE;
	required_type[1] = CVIP_SHORT;
	num_type = 2;
	

	if (argc != 4) {
		sprintf(interp->result, "Usage: cvip diff_spatial <image> <filter('1' for horizontal filter, '2' for vertical filter)>");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	destImage = convertImage_Queue(imageP,required_type,num_type);
	mask = (Matrix*)get_default_filter(DIFFERENCE_SPATIAL,3,(atoi(argv[3])-1));

	if (mask == NULL)
		return TCL_ERROR; 
	
	imageP = (Image*)convolve_filter(destImage, mask);
	/*delete_Image(destImage);*/

	if(imageP == NULL)
		return TCL_ERROR;
	else
		(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}


int
S_ImageLOWPASS_SPATIAL(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Matrix 	*mask;
	Image * imageP, *destImage;
	CVIP_TYPE	required_type[5];
	int	num_type;

	required_type[0] = CVIP_BYTE;
	required_type[1] = CVIP_SHORT;
	num_type = 2;
	

	if (argc != 3) {
		sprintf(interp->result, "Usage: cvip lowpass_spatial <image>");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	destImage = convertImage_Queue(imageP,required_type,num_type);
	mask = (Matrix*)get_default_filter(LOWPASS_SPATIAL,3,0);

	if (mask == NULL)
		return TCL_ERROR; 
	
	imageP = (Image*)convolve_filter(destImage, mask);
	/*delete_Image(destImage);*/

	if(imageP == NULL)
		return TCL_ERROR;
	else
		(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}


int
S_ImageLAPLACIAN_SPATIAL(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Matrix 	*mask;
	Image * imageP, *destImage;
	CVIP_TYPE	required_type[5];
	int	num_type;

	required_type[0] = CVIP_BYTE;
	required_type[1] = CVIP_SHORT;
	num_type = 2;
	

	if (argc != 3) {
		sprintf(interp->result, "Usage: cvip laplacian_spatial <image>");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	destImage = convertImage_Queue(imageP,required_type,num_type);
	mask = (Matrix*)get_default_filter(LAPLACIAN_SPATIAL,3,0);

	if (mask == NULL)
		return TCL_ERROR; 
	
	imageP = (Image*)convolve_filter(destImage, mask);
	/*delete_Image(destImage);*/

	if(imageP == NULL)
		return TCL_ERROR;
	else
		(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}


int
S_ImageHIGHPASS_SPATIAL(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Matrix 	*mask;
	Image * imageP, *destImage;
	CVIP_TYPE	required_type[5];
	int	num_type;

	required_type[0] = CVIP_BYTE;
	required_type[1] = CVIP_SHORT;
	num_type = 2;
	

	if (argc != 3) {
		sprintf(interp->result, "Usage: cvip highpass_spatial <image>");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	destImage = convertImage_Queue(imageP,required_type,num_type);

	mask = (Matrix*)get_default_filter(HIGHPASS_SPATIAL,3,0);

	if (mask == NULL)
		return TCL_ERROR; 
	
	imageP = (Image*)convolve_filter(destImage, mask);
	/*delete_Image(destImage);*/

	if(imageP == NULL)
		return TCL_ERROR;
	else
		(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}


int
S_ImageSPECIFY_SPATIAL(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Matrix 	*mask;
	Image * imageP, *destImage;
	CVIP_TYPE	required_type[5];
	int	num_type;
	char **split;
	int number, x, y,i;
	float *data;
	float **temp;

	required_type[0] = CVIP_BYTE;
	required_type[1] = CVIP_SHORT;
	num_type = 2;
	

	if (argc != 6) {
		sprintf(interp->result, "Usage: cvip specify_spatial <image> <rows of mask> <cols of mask> <mask array (enclosed by curly brackets)");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
	destImage = convertImage_Queue(imageP,required_type,num_type);

	data = (float*)calloc((atoi(argv[3]))*(atoi(argv[4]))+1,sizeof(float));
	temp = (float**)calloc((atoi(argv[3])), sizeof(float*));
    	temp[0] = (float*)calloc((atoi(argv[3]))*(atoi(argv[4])), sizeof(float));
	for (i=1; i<atoi(argv[3]); i++)
		temp[i] = temp[0] +i*atoi(argv[4]);
	if(data == NULL){
	    sprintf(interp->result,"Fatal Error: malloc()");
	    return TCL_ERROR;
	}

	if(Tcl_SplitList(interp, argv[5], &number, &split) == TCL_ERROR){
		sprintf(interp->result,"Error in interpreting mask array");
		return TCL_ERROR;
	}
	if(number != (atoi(argv[3]))*(atoi(argv[4]))){
 		sprintf(interp->result,
				"Insufficient arguments in mask array");
		return TCL_ERROR;
	}	
	number--; 
	for(;number>=0;number--)
		data[number+1] = atof(split[number]);

   	for(y = 0; y < (atoi(argv[3])); y++){
		for(x = 0; x < (atoi(argv[4])); x++){
			temp[y][x] = data[x+y*(atoi(argv[4]))];
		}
    	}

	mask = (Matrix*)specify_filter((atoi(argv[3])),(atoi(argv[4])),temp);

	if (mask == NULL)
		return TCL_ERROR; 
	
	imageP = (Image*)convolve_filter(destImage, mask);
	/*delete_Image(destImage);*/

	if(imageP == NULL)
		return TCL_ERROR;
	else
		(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

    	free(temp[0]);
    	free(temp);
	free(split);
  	free(data);
	return TCL_OK;
}

int
S_ImageGRAY_QUANT(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP;

	if (argc != 4) {
		sprintf(interp->result, "Usage: cvip gray_quant <image> <gray_level (choose one from 2,4,8,16,32,64,128)>");
		return TCL_ERROR;
	}
	if(!((atoi(argv[3]) == 2) ||(atoi(argv[3]) == 4) || (atoi(argv[3]) == 8)|| (atoi(argv[3]) == 16) || (atoi(argv[3]) == 32)|| (atoi(argv[3]) == 64)|| (atoi(argv[3]) == 128))){
	    	sprintf(interp->result,"The number of gray levels must be a power of 2 (2,4,8,...,128)\n");
	    	return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	imageP = image_duplicate(imageP);
	imageP = (Image*)gray_quant_segment(imageP, atoi(argv[3]));
	if(imageP == NULL)
		return TCL_ERROR;

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}

int
S_ImageAREA(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
        Image * imageP;
        long    aarea;

        if (argc != 5) {
                sprintf(interp->result, "Usage: cvip area <labeled image> <row coordi> <col coordi>");
                return TCL_ERROR;
        }

        if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
        if (imageP->image_ptr[0]->data_type != CVIP_INTEGER)
          { sprintf(interp->result, "not a labeled image"); return TCL_ERROR;}

        aarea = area(imageP, atoi(argv[3]), atoi(argv[4]));

        sprintf(interp->result, "%ld",aarea);
        return TCL_OK;
}

int
S_ImageCENTROID(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
        Image * imageP;
        int   * ptr;

        if (argc != 5) {
                sprintf(interp->result, "Usage: cvip centroid <labeled image> <row coordi> <col coordi>");
                return TCL_ERROR;
        }

        if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
        if (imageP->image_ptr[0]->data_type != CVIP_INTEGER)
          { sprintf(interp->result, "not a labeled image"); return TCL_ERROR;}

        ptr = centroid(imageP, atoi(argv[3]), atoi(argv[4]));

        sprintf(interp->result, "%d %d",ptr[0], ptr[1]);

	free(ptr);
        return TCL_OK;
}

int
S_ImageORIENTATION(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
        Image * imageP;
        double  theta;

        if (argc != 5) {
                 sprintf(interp->result, "Usage: cvip orientation <labeled image> <row coordi> <col coordi>");
                return TCL_ERROR;
        }

        if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
        if (imageP->image_ptr[0]->data_type != CVIP_INTEGER)
          { sprintf(interp->result, "not a labeled image"); return TCL_ERROR;}

        theta = orientation(imageP, atoi(argv[3]), atoi(argv[4]));

        sprintf(interp->result,"%f", theta);
        return TCL_OK;
}

int
S_ImageASPECT(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
        Image * imageP;
        double  aaspect;

        if (argc != 5) {
                 sprintf(interp->result, "Usage: cvip aspect <labeled image> <row coordi> <col coordi>");
                return TCL_ERROR;
        }

        if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
        if (imageP->image_ptr[0]->data_type != CVIP_INTEGER)
          { sprintf(interp->result, "not a labeled image"); return TCL_ERROR;}

        aaspect = aspect(imageP, atoi(argv[3]), atoi(argv[4]));

        sprintf(interp->result,"%f", aaspect);
        return TCL_OK;
}

static void cvip_free(char *block) {
	free((void *)block);
}

int
S_ImagePROJECTION(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
        Image * imageP;
        int   * ptr, size, i;
	char  * pc;
 
        if (argc != 7) {
                sprintf(interp->result, "Usage: cvip projection <labeled image> <row coordi> <col coordi> <height> <width>");
                return TCL_ERROR;
        }
 
        if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
        if (imageP->image_ptr[0]->data_type != CVIP_INTEGER)
          { sprintf(interp->result, "not a labeled image"); return TCL_ERROR;}

        ptr = projection(imageP, atoi(argv[3]), atoi(argv[4]), atoi(argv[5]), atoi(argv[6]));
 	size = atoi(argv[5]) + atoi(argv[6]);

	if (ptr == NULL) {sprintf(interp->result,"error happens");
			  return TCL_ERROR;
			 }

	interp->result = (char *)malloc((size+2) * 5);
	interp->freeProc = cvip_free;
	pc = interp->result;
	for (i=0; i<size; i++) 
	  { sprintf(pc, "%d ",ptr[i]);
	    while (*pc != 0) pc++;
	  }	

	free(ptr);
        return TCL_OK;
}

int
S_ImageEULER(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP;
	int     eeuler;

	if (argc != 5) {
		sprintf(interp->result, "Usage: cvip euler <labeled image> <row coordi> <col coordi>");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
        if (imageP->image_ptr[0]->data_type != CVIP_INTEGER) 
	  { sprintf(interp->result, "not a labeled image"); return TCL_ERROR;}
 
	eeuler = euler(imageP, atoi(argv[3]), atoi(argv[4]));

	sprintf(interp->result, "%d",eeuler);
	return TCL_OK;
}

int
S_ImagePERIMETER(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP;
        int     pperimeter;

        if (argc != 5) {
		 sprintf(interp->result, "Usage: cvip perimeter <labeled image> <row coordi> <col coordi>");
                return TCL_ERROR;
        }

        if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
        if (imageP->image_ptr[0]->data_type != CVIP_INTEGER)
          { sprintf(interp->result, "not a labeled image"); return TCL_ERROR;}

	pperimeter = perimeter(imageP, atoi(argv[3]), atoi(argv[4]));

	sprintf(interp->result,"%d",pperimeter);
	return TCL_OK;
}

int
S_ImageTHINNESS(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP;
	double  tthinness;

        if (argc != 5) {
                 sprintf(interp->result, "Usage: cvip thinness <labeled image> <row coordi> <col coordi>");
                return TCL_ERROR;
        }

        if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
        if (imageP->image_ptr[0]->data_type != CVIP_INTEGER)
          { sprintf(interp->result, "not a labeled image"); return TCL_ERROR;}

        tthinness = thinness(imageP, atoi(argv[3]), atoi(argv[4]));

	sprintf(interp->result,"%f", tthinness);
	return TCL_OK;
}

int
S_ImageIRREGULARITY(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
        Image * imageP;
        double  iirregular;

        if (argc != 5) {
                 sprintf(interp->result, "Usage: cvip irregular <labeled image> <row coordi> <col coordi>");
                return TCL_ERROR;
        }
 
        if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
        if (imageP->image_ptr[0]->data_type != CVIP_INTEGER)
          { sprintf(interp->result, "not a labeled image"); return TCL_ERROR;}
 
	iirregular = irregular(imageP, atoi(argv[3]), atoi(argv[4]));

	sprintf(interp->result,"%f", iirregular);
	return TCL_OK;
}

int
S_ImageHIST_FEATURE(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
        Image 	* originalImage, * labeledImage;
        int    	  bands, i, k;
        char  	* pc;
	double  * ptr;

        if (argc != 6) {
                sprintf(interp->result, "Usage: cvip hist_feature <original image,color or gray> <labeled image> <row coordi> <col coordi>");                return TCL_ERROR;
        }

        if(!(originalImage = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
        if(!(labeledImage = S_ImageGetFromHash(interp, argv[3]))) return TCL_ERROR; 
        if (labeledImage->image_ptr[0]->data_type != CVIP_INTEGER)
          { sprintf(interp->result, "the second image should be a labeled image"); return TCL_ERROR;}

        ptr = hist_feature(originalImage, labeledImage, atoi(argv[4]), atoi(argv[5]));
	bands = originalImage->bands;

        interp->result = (char *)malloc(bands * 5 * 50);
        interp->freeProc = cvip_free;
        pc = interp->result;
	for (k=0; k<bands; k++)
          for (i=0; i<5; i++)
            { sprintf(pc, "%f ", ptr[5*k + i]);
              while (*pc != 0) pc++;
            }

        free(ptr);
        return TCL_OK;
}

int
S_ImageSPECTRAL_FEATURE(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
        Image   * originalImage, * labeledImage;
        int       bands, total, i, k;
        char    * pc;
        POWER   * PP;

        if (argc != 8) {
                sprintf(interp->result, "Usage: cvip spectral_feature <original image,color or gray> <labeled image> <no_of_rings> <no_of_sectors> <row coordi> <col coordi>");                return TCL_ERROR;
        }

        if(!(originalImage = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
        if(!(labeledImage = S_ImageGetFromHash(interp, argv[3]))) return TCL_ERROR;
        if (labeledImage->image_ptr[0]->data_type != CVIP_INTEGER)
          { sprintf(interp->result, "the second image should be a labeled image"); return TCL_ERROR;}

        PP = spectral_feature(originalImage, labeledImage, atoi(argv[4]), atoi(argv[5]), 
		       atoi(argv[6]), atoi(argv[7]));
        bands = originalImage->bands;
	total = 1 + PP->no_of_bands + PP->no_of_sectors;

        interp->result = (char *)malloc(bands * total * 50);
        interp->freeProc = cvip_free;
        pc = interp->result;
   
        for (k=0; k<bands; k++)
         { sprintf(pc, "%f ", (PP->dc)[k]);  
           while (*pc != 0) pc++;
	   for (i=0; i<PP->no_of_bands; i++) 
            { sprintf(pc, "%f ", (PP->band)[k*PP->no_of_bands + i]);  
              while (*pc != 0) pc++;
            }
           for (i=0; i<PP->no_of_sectors; i++)
            { sprintf(pc, "%f ", (PP->sector)[k*PP->no_of_sectors + i]);  
              while (*pc != 0) pc++;
            }
	}

  	free((char *)PP->dc);
  	free((char *)PP->sector);
  	free((char *)PP->band);
  	free((char *)PP); 
        return TCL_OK;
}

void texture_convert(TEXTURE * P, float * ptr)
{ int i;
  
 for (i=0; i<6; i++) { *ptr = P->ASM[i]; ptr++;} 
 for (i=0; i<6; i++) { *ptr = P->contrast[i]; ptr++;} 
 for (i=0; i<6; i++) { *ptr = P->correlation[i]; ptr++;}
 for (i=0; i<6; i++) { *ptr = P->variance[i]; ptr++;}
 for (i=0; i<6; i++) { *ptr = P->IDM[i]; ptr++;} 
 for (i=0; i<6; i++) { *ptr = P->sum_avg[i]; ptr++;} 
 for (i=0; i<6; i++) { *ptr = P->sum_var[i]; ptr++;} 
 for (i=0; i<6; i++) { *ptr = P->sum_entropy[i]; ptr++;} 
 for (i=0; i<6; i++) { *ptr = P->entropy[i]; ptr++;}
 for (i=0; i<6; i++) { *ptr = P->diff_var[i]; ptr++;}
 for (i=0; i<6; i++) { *ptr = P->diff_entropy[i]; ptr++;} 
 for (i=0; i<6; i++) { *ptr = P->meas_corr1[i]; ptr++;}
 for (i=0; i<6; i++) { *ptr = P->meas_corr2[i]; ptr++;} 
 for (i=0; i<6; i++) { *ptr = P->max_corr_coef[i]; ptr++;} 
}
 

int
S_ImageTEXTURE(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
        Image   * originalImage, * labeledImage, * convertImage;
        int       i, k, pos[14];
	long 	  h, n;
        char    * pc;
        double  * ptr;
	TEXTURE * P;
	float     r[84];
	CVIP_TYPE      dtype = CVIP_BYTE;

        if (argc != 9) {
                sprintf(interp->result, "Usage: cvip texture <original image,color or gray> <labeled image> <ith band of original image> <row coordi> <col coordi> <hex number> <distance>");
		return TCL_ERROR;
        }

        if(!(originalImage = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
        if(!(labeledImage = S_ImageGetFromHash(interp, argv[3]))) return TCL_ERROR;
        if (labeledImage->image_ptr[0]->data_type != CVIP_INTEGER)
          { sprintf(interp->result, "the second image should be a labeled image"); return TCL_ERROR;}

	if (originalImage->image_ptr[0]->data_type > CVIP_BYTE)
	   convertImage = convertImage_Queue(originalImage, &dtype, 1);
	else convertImage = originalImage; 

	h = strtol(argv[7], (char **)NULL, 16); 
        P = texture(convertImage, labeledImage, atoi(argv[4]), atoi(argv[5]), atoi(argv[6]), h, atoi(argv[8]));

	texture_convert(P, r);
	for (i=0; i<14; i++) pos[i] = 0;

	for (i=0, n=1, k=0; i<14; i++, n*=2) 
	  if ((h & n) != 0) 
	    { k++; pos[i] = 1;}
 
        interp->result = (char *)malloc(k * 6 * 50);
        interp->freeProc = cvip_free;
        pc = interp->result;
        for (k=0; k<14; k++)
	  if (pos[k] != 0)
            for (i=0; i<6; i++)
              { sprintf(pc, "%f ", r[6*k + i]);
                while (*pc != 0) pc++;
              }
 
        free((char *)P);
        return TCL_OK;
}

int
S_Imagerst_invariant(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
        Image  * imageP;
        int      i;
        char   * pc;
	double * ptr;

        if (argc != 5) {
                sprintf(interp->result, "Usage: cvip rst_invariant <labeled image> <row coordi> <col coordi>");
                return TCL_ERROR;
        }

        if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
        if (imageP->image_ptr[0]->data_type != CVIP_INTEGER)
          { sprintf(interp->result, "not a labeled image"); return TCL_ERROR;}

        ptr = rst_invariant(imageP, atoi(argv[3]), atoi(argv[4]));

        interp->result = (char *)malloc(7 * 50);
        interp->freeProc = cvip_free;
        pc = interp->result;
        for (i=0; i<7; i++)
          { sprintf(pc, "%f ",ptr[i]);
            while (*pc != 0) pc++;
          }

        free(ptr);
        return TCL_OK;
}


int
S_ImageUNSHARP_SPATIAL(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP;
	
	if (argc != 7) {
		sprintf(interp->result, "Usage: cvip unsharp <image> <shrink lower limit> <upper limit>, <stretch low clip(percent)> <high clip(percent)>");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	imageP = image_duplicate(imageP);
	imageP = (Image*) unsharp_filter(imageP, atoi(argv[3]), atoi(argv[4]),
				atof(argv[5]), atof(argv[6]));
	if(imageP == NULL)
		return TCL_ERROR;

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}

int
S_ImageSPATIAL_QUANT(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP;

	if (argc != 6) {
		sprintf(interp->result, "Usage: cvip spatial_quant <image> <method(1-3) (1. Average, 2.Median, 3. Decimate) <result_height> <result_width>");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	imageP = image_duplicate(imageP);
	imageP = (Image*) spatial_quant(imageP, atoi(argv[4]), atoi(argv[5]), atoi(argv[3]));
	if(imageP == NULL)
		return TCL_ERROR;

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}

int
S_ImageMINIMUM_FILTER(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP;

	if (argc != 4) {
		sprintf(interp->result, "Usage: cvip minimum_filter <image> <mask_size(3,5,7,9 or 11)>");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

 	
	imageP = image_duplicate(imageP);
	imageP = (Image*) minimum_filter(imageP, atoi(argv[3]));
	if(imageP == NULL)
		return TCL_ERROR;

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}


int
S_ImageMEAN_FILTER(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP;

	if (argc != 4) {
		sprintf(interp->result, "Usage: cvip mean_filter <image> <mask_size(3,5,7,9 or 11)>");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

 	
	imageP = image_duplicate(imageP);
	imageP = (Image*) mean_filter(imageP, atoi(argv[3]));
	if(imageP == NULL)
		return TCL_ERROR;

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}


int
S_ImageALPHA_FILTER(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP;

	if (argc != 5) {
		sprintf(interp->result, "Usage: cvip alpha_filter <image> <mask_size(3,5,7,9 or 11)> <trim_size ( an number between 0 and mask_size)>");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

 	
	imageP = image_duplicate(imageP);
	imageP = (Image*)alpha_filter(imageP, atoi(argv[3]), atoi(argv[4]));
	if(imageP == NULL)
		return TCL_ERROR;

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}

int
S_ImageCONTRA_FILTER(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP;

	if (argc != 5) {
		sprintf(interp->result, "Usage: cvip contra_filter <image> <mask_size(3,5,7,9 or 11)> <filter order [ -5 to +5 ]>");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

 	
	imageP = image_duplicate(imageP);
	imageP = (Image*)contra_filter(imageP, atoi(argv[3]), atoi(argv[4]));
	if(imageP == NULL)
		return TCL_ERROR;

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}

int 
S_ImageGEOMETRIC_FILTER(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP;

	if (argc != 4) {
		sprintf(interp->result, "Usage: cvip geometric_filter <image> <mask_size(3,5,7,9 or 11)> ");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

 	
	imageP = image_duplicate(imageP);
	imageP = (Image*)geometric_filter(imageP, atoi(argv[3]));
	if(imageP == NULL)
		return TCL_ERROR;

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}

int 
S_ImageHARMONIC_FILTER(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP;

	if (argc != 4) {
		sprintf(interp->result, "Usage: cvip harmonic_filter <image> <mask_size(3,5,7,9 or 11)> ");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

 	
	imageP = image_duplicate(imageP);
	imageP = (Image*)harmonic_filter(imageP, atoi(argv[3]));
	if(imageP == NULL)
		return TCL_ERROR;

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}

int
S_ImageYP_MEAN_FILTER(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP;

	if (argc != 5) {
		sprintf(interp->result, "Usage: cvip ypmean_filter <image> <mask_size(3,5,7,9 or 11)> <filter order [-5, 0) or (0, +5 )>");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

 	
	imageP = image_duplicate(imageP);
	imageP = (Image*)Ypmean_filter(imageP, atoi(argv[3]), atoi(argv[4]));
	if(imageP == NULL)
		return TCL_ERROR;

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}

S_ImageMIDPOINT_FILTER(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP;

	if (argc != 4) {
		sprintf(interp->result, "Usage: cvip midpoint_filter <image> <mask_size(3,5,7,9 or 11)> ");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

 	
	imageP = image_duplicate(imageP);
	imageP = (Image*)midpoint_filter(imageP, atoi(argv[3]));
	if(imageP == NULL)
		return TCL_ERROR;

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}

S_ImageMAXIMUM_FILTER(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP;

	if (argc != 4) {
		sprintf(interp->result, "Usage: cvip maximum_filter <image> <mask_size(3,5,7,9 or 11)> ");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

 	
	imageP = image_duplicate(imageP);
	imageP = (Image*)maximum_filter(imageP, atoi(argv[3]));
	if(imageP == NULL)
		return TCL_ERROR;

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}

int
S_GRAY_LIN_MOD(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP;
	int band = -1;

	if (argc != 8 && argc != 9) {
		sprintf(interp->result, "Usage: cvip gray_linear <image, any data type> <begin range> <end range> <start grayvalue> <slope> <set out-of-range to 0/do not change out-of-range(0/1)> <band to modify (-1 for all bands)>");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	imageP = duplicate_Image(imageP);
	if (argc == 9) band = atoi(argv[8]);
	imageP = gray_linear(imageP, atof(argv[3]), atof(argv[4]),
		atof(argv[5]), atof(argv[6]), atoi(argv[7]), band);

	if(imageP == NULL)
		return TCL_ERROR;

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}

int
S_GRAY_MULTIPLY(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP;
	CVIP_TYPE    required_type[5];
	int     num_type;

	required_type[0] = CVIP_BYTE;
	num_type = 1;


	if (argc != 4 && argc != 5) {
		sprintf(interp->result,
		"Usage: cvip gray_multiply <image> <multiplier> <CVIP_BYTE clipping/Not (1/0)> ");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	if (argc == 5 && atoi(argv[4]) == 0) {
		imageP = (Image*)duplicate_Image(imageP);
		imageP = gray_multiply2(imageP, atof(argv[3]));
	} else if (argc == 5 && atoi(argv[4]) == 1 || argc == 4) {
		imageP = convertImage_Queue(imageP,required_type,num_type);
		imageP = gray_multiply(imageP, atof(argv[3]));
	} else {
		sprintf(interp->result,
		"Usage: cvip gray_multiply <image> <multiplier> <CVIP_BYTE clinping/Not (1/0)> ");
		return TCL_ERROR;
	}

	if(imageP == NULL)
		return TCL_ERROR;

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}


int
S_MEDIAN_FILTER(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP;
	CVIP_TYPE	required_type[5];
	int	num_type;

	required_type[0] = CVIP_BYTE;
	num_type = 1;
	

	if (argc != 4) {
		sprintf(interp->result, "Usage: cvip median_filter <image> <an odd number for mask size ( >=3 )>");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	imageP = convertImage_Queue(imageP,required_type,num_type);
	imageP = median_filter(imageP, atoi(argv[3]));

	if(imageP == NULL)
		return TCL_ERROR;

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}


S_ImageMEDIANCUT(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image 		*imageP;
	Color		bg = {0, 0, 0};
	CVIP_TYPE	required_type[5];
	int	num_type = 1;

	required_type[0] = CVIP_BYTE;

	if (argc < 4 || argc > 7) {
		sprintf(interp->result,
	"Usage: cvip median_cut <image> <# of the color( 2-1000 )>\
[background: r g b]");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	if (imageP->bands != 3){
		sprintf(interp->result,"The input image should be color image");
		return TCL_ERROR;
	}
 	
/*	imageP = image_duplicate(imageP); */
	imageP = convertImage_Queue(imageP,required_type,num_type);
	if (argc == 7) {
		bg.r=(byte)atoi(argv[4]);
		bg.g=(byte)atoi(argv[5]);
		bg.b=(byte)atoi(argv[6]);
		imageP = median_cut_segment(imageP, atoi(argv[3]), CVIP_YES, bg);
	} else
	imageP = median_cut_segment(imageP, atoi(argv[3]), CVIP_NO, bg);

	if(imageP == NULL)
		return TCL_ERROR;

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}


int
S_HIST_SLIDE(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP;
	CVIP_TYPE	required_type[5];
	int	num_type;

	required_type[0] = CVIP_BYTE;
	num_type = 1;
	

	if (argc != 5) {
		sprintf(interp->result, "Usage: cvip hist_slide <image> < 1 for Slide up/ -1 for Slide down> <slide value>");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) 
		return TCL_ERROR;

	imageP = convertImage_Queue(imageP,required_type,num_type);

	imageP = hist_slide(imageP, atoi(argv[3])*atoi(argv[4]));

	if(imageP == NULL)
		return TCL_ERROR;

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}


int
S_HIST_SHRINK(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP, *temp;
	CVIP_TYPE	required_type[5];
	int	num_type;

	required_type[0] = CVIP_BYTE;
	num_type = 1;
	

	if (argc != 5) {
		sprintf(interp->result, "Usage: cvip hist_shrink <image> <lower limit(0-244)> <upper limit(1-255)> ");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) 
		return TCL_ERROR;

	imageP = convertImage_Queue(imageP,required_type,num_type);

	temp = remap_Image(imageP, CVIP_BYTE, atoi(argv[3]),atoi(argv[4]));
	delete_Image(imageP);

	if(temp == NULL)
		return TCL_ERROR;

	(void) S_ImageAddToHash(interp, temp, (char *)NULL);

	return TCL_OK;
}



int
S_HIST_STRETCH(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP, *temp;
	CVIP_TYPE	required_type[5];
	int	num_type;

	required_type[0] = CVIP_BYTE;
	num_type = 1;
	

	if (argc != 7) {
		sprintf(interp->result, "Usage: cvip hist_stretch <image> <lower limit(0-244)> <upper limit(1-255)> <low clip(percent)> <high clip(percent)>");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) 
		return TCL_ERROR;

	imageP = hist_stretch(imageP, atoi(argv[3]), atoi(argv[4]), atof(argv[5]),atof(argv[6]));

	if(imageP == NULL)
		return TCL_ERROR;

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}

int
S_ImageADD(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image 		*imageP, *imageT;

	if (argc != 4) {
		sprintf(interp->result, "Usage: cvip add <image1> <image2> ");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
	if(!(imageT = S_ImageGetFromHash(interp, argv[3]))) return TCL_ERROR;
 	
	imageP = image_duplicate(imageP);
	imageT = image_duplicate(imageT);

	imageP = add_Image(imageP, imageT);

	if(imageP == NULL)
		return TCL_ERROR;

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}

int
S_ImageSUBTRACT(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image 		*imageP, *imageT;

	if (argc != 4) {
		sprintf(interp->result, "Usage: cvip subtract <image1> <image2> ");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
	if(!(imageT = S_ImageGetFromHash(interp, argv[3]))) return TCL_ERROR;
 	
	imageP = image_duplicate(imageP);
	imageT = image_duplicate(imageT);

	imageP = subtract_Image(imageP, imageT);

	if(imageP == NULL)
		return TCL_ERROR;

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}

int
S_ImageMULTIPLY(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image 		*imageP, *imageT;

	if (argc != 4) {
		sprintf(interp->result, "Usage: cvip multiply <image1> <image2> ");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
	if(!(imageT = S_ImageGetFromHash(interp, argv[3]))) return TCL_ERROR;
 	
	imageP = image_duplicate(imageP);
	imageT = image_duplicate(imageT);

	imageP = multiply_Image(imageP, imageT);

	if(imageP == NULL)
		return TCL_ERROR;

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}


int
S_ImageDIVIDE(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image 		*imageP, *imageT;
	CVIP_BOOLEAN zero2num = CVIP_YES;

	if (argc == 4);
	else if (argc == 5) 
		 zero2num = (atoi(argv[4]) == 0)? zero2num = CVIP_NO: CVIP_YES;
   else  {
		sprintf(interp->result,
			"Usage: cvip divide <image1> <image2> \n [use numerator for 0 denomitor (0 for no/1 for yes, default to yes)]");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
	if(!(imageT = S_ImageGetFromHash(interp, argv[3]))) return TCL_ERROR;
 	
	imageP = image_duplicate(imageP);
	imageT = image_duplicate(imageT);

	imageP = divide_Image(imageP, imageT, zero2num);

	if(imageP == NULL)
		return TCL_ERROR;

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}


int
S_ImageAND(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP, *imageT;
	CVIP_TYPE	required_type[5];
	int	num_type;

	required_type[0] = CVIP_BYTE;
	num_type = 1;
	if (argc != 4) {
		sprintf(interp->result, "Usage: cvip and <image1> <image2> ");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
	if(!(imageT = S_ImageGetFromHash(interp, argv[3]))) return TCL_ERROR;
 	
	imageP = convertImage_Queue(imageP,required_type,num_type);
	imageT = convertImage_Queue(imageT,required_type,num_type);

	imageP = and_Image(imageP, imageT);

	if(imageP == NULL)
		return TCL_ERROR;

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}

int
S_ImageOR(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP, *imageT;
	CVIP_TYPE	required_type[5];
	int	num_type;

	required_type[0] = CVIP_BYTE;
	num_type = 1;
	if (argc != 4) {
		sprintf(interp->result, "Usage: cvip or <image1> <image2> ");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
	if(!(imageT = S_ImageGetFromHash(interp, argv[3]))) return TCL_ERROR;
 	
	imageP = convertImage_Queue(imageP,required_type,num_type);
	imageT = convertImage_Queue(imageT,required_type,num_type);

	imageP = or_Image(imageP, imageT);

	if(imageP == NULL)
		return TCL_ERROR;

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}


int
S_ImageXOR(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP, *imageT;
	CVIP_TYPE	required_type[5];
	int	num_type;

	required_type[0] = CVIP_BYTE;
	num_type = 1;
	if (argc != 4) {
		sprintf(interp->result, "Usage: cvip xor <image1> <image2> ");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
	if(!(imageT = S_ImageGetFromHash(interp, argv[3]))) return TCL_ERROR;
 	
	imageP = convertImage_Queue(imageP,required_type,num_type);
	imageT = convertImage_Queue(imageT,required_type,num_type);

	imageP = xor_Image(imageP, imageT);

	if(imageP == NULL)
		return TCL_ERROR;

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}


int
S_ImageNOT(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP;
	CVIP_TYPE	required_type[5];
	int	num_type;

	required_type[0] = CVIP_BYTE;
	num_type = 1;
	if (argc != 3) {
		sprintf(interp->result, "Usage: cvip not <image> ");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
 	
	imageP = convertImage_Queue(imageP,required_type,num_type);

	imageP = not_Image(imageP);

	if(imageP == NULL)
		return TCL_ERROR;

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}


int
S_DATA_REMAP(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image 		*imageP;
	char		*type[] = {"CVIP_BYTE","CVIP_SHORT","CVIP_INTEGER","CVIP_FLOAT","CVIP_DOUBLE"};

	if (argc != 4) {
		sprintf(interp->result, "Usage: cvip data_remap <image> <choice for data_type(0,1,2 or 3)>\n\twhere 0 for byte, 1 for short, 2 for integer, 3 for float");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;


	if(imageP->bandP[0]->data_type > atoi(argv[3])) {
		sprintf(interp->result, "image is remapped from %s to %s\n", type[imageP->bandP[0]->data_type], type[atoi(argv[3])]);
		switch(atoi(argv[3])){  
			case CVIP_BYTE:
			     imageP = (Image*)remap_Image(imageP,CVIP_BYTE,
						0,255);
			     break;
			case CVIP_SHORT:
			     imageP =(Image*)remap_Image(imageP,CVIP_SHORT,
						SHRT_MIN,SHRT_MAX);
			     break;
			case CVIP_INTEGER:
			     imageP =(Image*)remap_Image(imageP,CVIP_INTEGER,
						LONG_MIN,LONG_MAX);
			     break;
			default:
			     printf("Wrong data type");
			     delete_Image(imageP);
			     return TCL_ERROR;
		 }
	} 
 	else if(atoi(argv[3]) > imageP->bandP[0]->data_type && atoi(argv[3]) <= CVIP_FLOAT){
		sprintf(interp->result,"Image is cast into %s\n",type[atoi(argv[3])]);
		imageP = image_duplicate(imageP);
		cast_Image(imageP,atoi(argv[3]));
	}
	else {
		printf("Wrong data type");
		delete_Image(imageP);
		return TCL_ERROR;
	}


	if(imageP == NULL)
		return TCL_ERROR;

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}


int
S_ImageGAMMA_NOISE(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image 		*imageP;
	float    	variance;
	int 		alpha;

	if (argc != 5) {
		sprintf(interp->result, "Usage: cvip gamma_noise <image> <variance(100-400)> <alpha(1-6)> ");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
	variance = atof(argv[3]);
	alpha = atoi(argv[4]); 
	
	imageP = image_duplicate(imageP);
	imageP = gamma_noise(imageP, &variance, &alpha);

	if(imageP == NULL)
		return TCL_ERROR;

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}


int
S_ImageGAUSSIAN_NOISE(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image 		*imageP;
	float    	variance, mean;

	if (argc != 5) {
		sprintf(interp->result, "Usage: cvip gaussian_noise <image> <variance(100-400)> <mean(negative will darken image, positive will brighten image)> ");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
	variance = atof(argv[3]);
	mean = atoi(argv[4]); 
	
	imageP = image_duplicate(imageP);
	imageP = gaussian_noise(imageP, &variance, &mean);

	if(imageP == NULL)
		return TCL_ERROR;

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}

int
S_ImageNEGATIVE_NOISE(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image 		*imageP;
	float    	variance;

	if (argc != 4) {
		sprintf(interp->result, "Usage: cvip negative_exp_noise <image> <variance(100-400)> ");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
	variance = atof(argv[3]);
	
	imageP = image_duplicate(imageP);
	imageP = neg_exp_noise(imageP, &variance);

	if(imageP == NULL)
		return TCL_ERROR;

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}


int
S_ImageRAYLEIGH_NOISE(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image 		*imageP;
	float    	variance;

	if (argc != 4) {
		sprintf(interp->result, "Usage: cvip rayleigh_noise <image> <variance(100-400)> ");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
	variance = atof(argv[3]);
	
	imageP = image_duplicate(imageP);
	imageP = rayleigh_noise(imageP, &variance);

	if(imageP == NULL)
		return TCL_ERROR;

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}


int
S_ImageSALT_NOISE(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image 		*imageP;
	float    	phigh, plow;

	if (argc != 5) {
		sprintf(interp->result, "Usage: cvip speckle_noise <image> <probability of high grey levels(0.01-0.05)> <probablity of low grey levels(0.01-0.05)> ");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
	phigh = atof(argv[3]);
	plow = atof(argv[4]); 
	
	imageP = image_duplicate(imageP);
	imageP = speckle_noise(imageP, &phigh, &plow);

	if(imageP == NULL)
		return TCL_ERROR;

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}


int
S_ImageUNIFORM_NOISE(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image 		*imageP;
	float    	variance, mean;

	if (argc != 5) {
		sprintf(interp->result, "Usage: cvip uniform_noise <image> <variance(100-400)> <mean(negative will darken image, positive will brighten image)> ");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
	variance = atof(argv[3]);
	mean = atoi(argv[4]); 
	
	imageP = image_duplicate(imageP);
	imageP = uniform_noise(imageP, &variance, &mean);

	if(imageP == NULL)
		return TCL_ERROR;

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}


int
S_ImageEDGE_LINK(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image 		*imageP;

	if (argc != 4) {
		sprintf(interp->result, "Usage: cvip edge_link <image> <maximum link distance> ");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
	
	imageP = image_duplicate(imageP);
	imageP = edge_link_filter(imageP, atoi(argv[3]));

	if(imageP == NULL)
		return TCL_ERROR;

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}

int
S_CHECKBOARD(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image 		*imageP;

	if (argc != 8) {
		sprintf(interp->result, "Usage: cvip create_checkboard <width> <height> <starting column> <starting row> <cell width> <cell height> ");
		return TCL_ERROR;
	}

	
	imageP = create_checkboard(atoi(argv[2]), atoi(argv[3]), atoi(argv[4]), atoi(argv[5]), atoi(argv[6]), atoi(argv[7]));

	if(imageP == NULL)
		return TCL_ERROR;

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}

int
S_CIRCLE(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image 		*imageP;

	int 		rsquare,max,min,distance;


	if (argc != 7) {
		sprintf(interp->result, "Usage: cvip circle <width> <height> <centerx> <centery> <radius> ");
		return TCL_ERROR;
	}

	rsquare = pow(atoi(argv[6]),2);
	distance = pow(atoi(argv[4])-0,2) + pow(atoi(argv[5])-0,2);
	min = distance; 
	max = distance;
	distance = pow(atoi(argv[4])-0,2) + pow(atoi(argv[5])-atoi(argv[3]),2);
	if(distance < min)
		min = distance; 
	if(distance > max)
		max = distance;
	distance = pow(atoi(argv[4])-atoi(argv[2]),2) + pow(atoi(argv[5])-0,2);
	if(distance < min)
		min = distance; 
	if(distance > max)
		max = distance;
	distance = pow(atoi(argv[4])-0,2) + pow(atoi(argv[5])-atoi(argv[3]),2);
	if(distance < min)
		min = distance; 
	if(distance > max)
		max = distance;

	/*if the center is out of the image frame*/
	if(atoi(argv[4])<0 || atoi(argv[4]) > atoi(argv[2]) || atoi(argv[5])<0 || atoi(argv[5]) > atoi(argv[3])) {  
		if(rsquare > max) { 
		sprintf(interp->result,"\n\tError:(create_circle_setup)radius too large.\n");
		return TCL_ERROR;
		}
		else if(rsquare < min) { 
		sprintf(interp->result,"\n\tError:(create_circle_setup)radius too small.\n");
		return TCL_ERROR;
		}
		else {
		imageP = create_circle(atoi(argv[2]), atoi(argv[3]), atoi(argv[4]), atoi(argv[5]), atoi(argv[6]));

		if(imageP == NULL)
			return TCL_ERROR;

		(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

		return TCL_OK;
		}
	}
	/*if the center is inside the image frame*/
	else if(rsquare > max){   
		sprintf(interp->result,"\n\tError:(create_circle_setup)radius too large.\n");
		return TCL_ERROR;
	}
	else {	
		imageP = create_circle(atoi(argv[2]), atoi(argv[3]), atoi(argv[4]), atoi(argv[5]), atoi(argv[6]));

		if(imageP == NULL)
			return TCL_ERROR;

		(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

		return TCL_OK;
	}
}

int
S_LINE(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image 		*imageP;

	if (argc != 8) {
		sprintf(interp->result, "Usage: cvip line <width> <height> <starting column> <starting row> <ending column> <ending row> ");
		return TCL_ERROR;
	}

	
	imageP = create_line(atoi(argv[2]), atoi(argv[3]), atoi(argv[4]), atoi(argv[5]), atoi(argv[6]), atoi(argv[7]));

	if(imageP == NULL)
		return TCL_ERROR;

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}


int
S_RECTANGLE(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image 		*imageP;

	if (argc != 8) {
		sprintf(interp->result, "Usage: cvip rectangle <width> <height> <starting column> <starting row> <rectangle width> <rectangle height> ");
		return TCL_ERROR;
	}

	
	imageP = create_rectangle(atoi(argv[2]), atoi(argv[3]), atoi(argv[4]), atoi(argv[5]), atoi(argv[6]), atoi(argv[7]));

	if(imageP == NULL)
		return TCL_ERROR;

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}


int
S_SINE_WAVE(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image 		*imageP;

	if (argc != 5) {
		sprintf(interp->result, "Usage: cvip sine_wave <image size> <1 for horizontal, 2 for vertical> <desired frequency>");
		return TCL_ERROR;
	}

	
	imageP = create_sine(atoi(argv[2]), atoi(argv[4]), atoi(argv[3]));

	if(imageP == NULL)
		return TCL_ERROR;

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}

int
S_COSINE_WAVE(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image 		*imageP;

	if (argc != 5) {
		sprintf(interp->result, "Usage: cvip cosine_wave <image size> <1 for horizontal, 2 for vertical> <desired frequency>");
		return TCL_ERROR;
	}

	
	imageP = create_cosine(atoi(argv[2]), atoi(argv[4]), atoi(argv[3]));

	if(imageP == NULL)
		return TCL_ERROR;

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}

int
S_SQUARE_WAVE(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image 		*imageP;

	if (argc != 5) {
		sprintf(interp->result, "Usage: cvip square_wave <image size> <1 for horizontal, 2 for vertical> <desired frequency>");
		return TCL_ERROR;
	}

	
	imageP = create_squarewave(atoi(argv[2]), atoi(argv[4]), atoi(argv[3]));

	if(imageP == NULL)
		return TCL_ERROR;

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}

int 
S_Display_Regular(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	char    image_name[100];
	struct mesh *inmesh;
	Image	*cvipImage;
	if (argc != 3) {
		sprintf(interp->result, "Usage: cvip display_mesh <mesh_file>");
		return TCL_ERROR;
	}

	inmesh = file_to_mesh(argv[2]);
	if(inmesh == NULL){
		sprintf(interp->result,"\nError in reading the mesh file\n");
		return TCL_ERROR;
	}

	cvipImage = create_black(512,512);
	print_CVIP("\n\tDisplaying original mesh...");
 	strcpy(image_name,argv[2]);
	strcat(image_name,"-ORIGINAL");
	display_mesh(cvipImage,inmesh,image_name);	
	inmesh = regular_mesh(inmesh,512,512);
 	strcpy(image_name,argv[2]);
	strcat(image_name,"-REGULAR");
	cvipImage = create_black(512,512);
	print_CVIP("\n\tDisplaying regular mesh...");
	display_mesh(cvipImage,inmesh,image_name);

	return TCL_OK;

}


int
S_Area_Points(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP;
	CVIP_TYPE	required_type[5];
	int	num_type;
	struct mesh *inmesh;
	byte **out;
	struct mesh_node intie[4];


	required_type[0] = CVIP_BYTE;
	num_type = 1;
	if (argc != 6) {
		sprintf(interp->result, "Usage: cvip paint_cell <image> <mesh_file> <i> <j>");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
 	
	imageP = convertImage_Queue(imageP,required_type,num_type);
	out = getData_Image(imageP, 0);
	inmesh = file_to_mesh(argv[3]);
	intie[0]=inmesh->nodes[atoi(argv[4])][atoi(argv[5])];
	intie[1]=inmesh->nodes[atoi(argv[4])+1][atoi(argv[5])];
	intie[2]=inmesh->nodes[atoi(argv[4])][atoi(argv[5])+1];
	intie[3]=inmesh->nodes[atoi(argv[4])+1][atoi(argv[5])+1];

	cell_paint(out,intie);

	if(imageP == NULL)
		return TCL_ERROR;

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}
