#***************************************************************************
# **********************************************************************
# Computer Vision/Image Processing Tool Project - Dr. Scott Umbaugh SIUE
# **********************************************************************
#
#             File Name: analysis.tcl
#           Description: Analysis module for graphical user interface
#         Related Files: 
#   Initial Coding Date: Mon Jul  8 16:28:06 CDT 1996
#           Portability: TCL script file
#            References:
#             Author(s): Yansheng Wei, Mark Zuke
#                        Southern Illinois University @ Edwardsville
#
## Copyright (C) 1996 SIUE - by Scott Umbaugh, Yansheng Wei, and Mark Zuke..
##
## Permission to use, copy, modify, and distribute this software and its
## documentation for any purpose and without fee is hereby granted, provided
## that the above copyright notice appear in all copies and that both that
## copyright notice and this permission notice appear in supporting
## documentation.  This software is provided "as is" without express or
## implied warranty.
##
# RCS (Revision Control System) Information
# ...Added automatically by RCS 
#
## $Log: analysis.tcl,v $
## Revision 1.28  1997/06/09 20:10:42  yawei
## move the geometry to a separate module
##
## Revision 1.27  1997/06/03 22:37:49  yawei
## added configuration and removed geometry to separate module
##
## Revision 1.26  1997/06/02 16:55:29  wzheng
## using Average on spatial_quant in resize part
##
## Revision 1.25  1997/05/28 02:41:54  wzheng
## *** empty log message ***
##
## Revision 1.24  1997/05/27 18:16:02  wzheng
## add bilinear_interp choice for resize
##
## Revision 1.23  1997/04/23 14:00:54  yawei
## added bilinear_interp to the zoom function
##
## Revision 1.22  1997/04/11 19:43:20  yawei
## counterclock => clock
##
## Revision 1.21  1997/04/08 13:44:50  mzuke
## corrected grammar
##
## Revision 1.20  1997/04/01 21:39:52  yawei
## changed copy_paste
##
## Revision 1.19  1997/03/22 00:08:34  yawei
## added copy-paste
##
## Revision 1.18  1997/03/11 20:55:36  yawei
## Tcl/Tk script upgrading from tcl7.3/tk3.6 to tcl7.6/tk4.2
## compatible. This process is automated with tkboost perl
## script. @ Tue Mar 11 14:51:36 CST 1997
##
## Revision 1.17  1996/12/31 00:22:15  yawei
## crop_Image ==> crop
##
# Revision 1.16  1996/12/13  18:27:18  akjoele
# Chanegd crop_inner() to crop_Image()
#
# Revision 1.15  1996/10/13  19:41:12  mzuke
# installed watch cursor; shortened some result image names;
# moved view image to after post_image
#
# Revision 1.14  1996/09/03  18:02:28  yawei
# changed anal_ to analysis_
#
# Revision 1.13  1996/08/16  19:07:53  mzuke
# periodic checkin
#
# Revision 1.12  1996/08/15  13:10:25  yawei
# changed "geometry" procedure, so the window won't go to the default position when geometry popups up.
#
# Revision 1.11  1996/08/13  22:44:26  yawei
# check in with initial transforms.
#
# Revision 1.10  1996/08/12  19:46:56  yawei
# added edge/detection/link
#
# Revision 1.9  1996/08/07  16:11:12  yawei
# modified endEntry so it can pass the input focus to the next widget
#
# Revision 1.8  1996/07/17  23:20:18  yawei
# remored the prompt message in the opening window of Analysis.
#
# Revision 1.6  1996/07/17  17:06:04  yawei
# modifed to faciliate button bar setup.
# 1. Changed preprocessing with geometry, there will be no Arith/Logic function
#    for analysis window.
# 2. changed the opening window to a small one with a prompt message, instead
#    of the default geometry window.
# 3. added HELP button
#
# Revision 1.5  1996/07/10  21:09:06  yawei
# regular check in
#
# Revision 1.4  1996/07/10  20:33:40  yawei
# switch the order of row<->col , height<->width to follow the convention of (row, col, height, width)
#
# Revision 1.3  1996/07/10  18:38:52  yawei
# modifed, the current_image won't change when doing geometry operation on one image
#
# Revision 1.2  1996/07/10  15:02:28  yawei
# added geometry features to GUI. done, but need testing.
#
# Revision 1.1  1996/07/08  22:11:56  mzuke
# Initial revision
#
#
#****************************************************************************
# Module: analysis.tcl
# Tcl version: 7.3 (Tcl/Tk/XF)
# Tk version: 3.6
# XF version: 2.3
#

# module contents
global moduleList
global autoLoadList
set moduleList(analysis.tcl) { analysis}
set autoLoadList(analysis.tcl) {1}

# procedures to show toplevel windows


# User defined procedures


# Procedure: analysis
proc analysis {} {
# xf ignore me 7
##########
# Procedure: analysis
# Author: Mark Zuke (mzuke@ee.siue.edu) May 13, 1996
# Description: calls toplevel screen to allow use of analysis functions
#             
# Arguments: none
# Returns: none
# Sideeffects: none
##########


  # build widget $topwin(analysis_name)
#  if {"[info procs XFEdit]" != ""} {
#    catch "XFDestroy $topwin(analysis_name)"
#  } {
#    catch "destroy $topwin(analysis_name)"
#  }

global toplevel_position
global message
global topwin
global {GuiColor}


  # Window manager configurations
  global tk_version
  wm positionfrom $topwin(analysis_name) user
  wm sizefrom $topwin(analysis_name) user
  wm geometry $topwin(analysis_name) [top_pos $topwin(analysis_top)]
  wm maxsize $topwin(analysis_name) [lindex [split $topwin(analysis_top_max)] 0] \
  	[lindex [split $topwin(analysis_top_max)] 1]
  wm minsize $topwin(analysis_name) [lindex [split $topwin(analysis_top_min)] 0] \
  	[lindex [split $topwin(analysis_top_min)] 1]
  wm title $topwin(analysis_name) {ANALYSIS}

  # build widget $topwin(analysis_name).menubar
  frame $topwin(analysis_name).menubar  -background "$GuiColor(frame_bg)"  -borderwidth {2}  -relief {raised}

  # build widget $topwin(analysis_name).menubar.preprocessing
	menubutton $topwin(analysis_name).menubar.preprocessing -padx {1} -pady {1} \
	-highlightthickness {0}  \
	-activebackground "$GuiColor(btn_act_bg)" \
	-background "$GuiColor(btn_bg)" \
	-menu "$topwin(analysis_name).menubar.preprocessing.menu"  -text {Preprocessing}

  # build widget $topwin(analysis_name).menubar.geometry
	button $topwin(analysis_name).menubar.geometry -padx {1} -pady {1} -highlightthickness {0}\
	-activebackground "$GuiColor(btn_act_bg)" \
	-background "$GuiColor(btn_bg)" \
	-command {geometry}  -relief {flat} \
	-text {Geometry}

  # build widget $topwin(analysis_name).menubar.edge_detection
	button $topwin(analysis_name).menubar.edge_detection -padx {1} -pady {1} \
	-highlightthickness {0}   \
	-activebackground "$GuiColor(btn_act_bg)" \
	-background "$GuiColor(btn_bg)" \
  -relief {flat} \
  -command {analysis_edge} \
  -text {Edge/Line detection}

  # build widget $topwin(analysis_name).menubar.segmentation
	button $topwin(analysis_name).menubar.segmentation -padx {1} -pady {1} \
	-highlightthickness {0}   \
	-activebackground "$GuiColor(btn_act_bg)" \
	-background "$GuiColor(btn_bg)" \
	-command {segmentation}  \
	-relief {flat} \
	-text {Segmentation}

  # build widget $topwin(analysis_name).menubar.transforms
	button $topwin(analysis_name).menubar.transforms -padx {1} -pady {1} -highlightthickness {0}   \
	-activebackground "$GuiColor(btn_act_bg)" \
	-background "$GuiColor(btn_bg)" \
  -relief {flat} \
  -command {analysis_xform} \
  -text {Transforms}


  # build widget $topwin(analysis_name).menubar.button0
	button $topwin(analysis_name).menubar.button0 -padx {1} -pady {1} -highlightthickness {0}\
	-activebackground "$GuiColor(btn_act_bg)" \
	-background "$GuiColor(btn_bg)" \
	-command {get_features} \
	-relief {flat} \
	-text {Features}

  # build $topwin(analysis_name).menubar.help
	button $topwin(analysis_name).menubar.help -padx {1} -pady {1} -highlightthickness {0}  \
	 -activebackground "$GuiColor(alert_act_bg)" \
	 -background "$GuiColor(alert_bg)" \
	 -fg "$GuiColor(alert_fg)" \
    -command {help_screen} \
    -relief {flat} \
    -command {help_screen} \
    -text {Help}

  # pack widget $topwin(analysis_name).menubar
  pack append $topwin(analysis_name).menubar  $topwin(analysis_name).menubar.geometry {left frame center expand}  $topwin(analysis_name).menubar.edge_detection {left frame center expand}  $topwin(analysis_name).menubar.segmentation {left frame center expand}  $topwin(analysis_name).menubar.transforms {left frame center expand}  $topwin(analysis_name).menubar.button0 {left frame center} $topwin(analysis_name).menubar.help {right frame center expand}

  # build widget $topwin(analysis_name).perform
  frame $topwin(analysis_name).perform  -background "$GuiColor(frame_bg)"  \
  -borderwidth {2}  -relief {flat}

  # build widget $topwin(analysis_name).perform.dismiss
	button $topwin(analysis_name).perform.dismiss -padx {1} -pady {1} -highlightthickness {0}\
	 -activebackground "$GuiColor(dis_act_bg)" \
	 -background "$GuiColor(dis_bg)" \
	-command {destroy $topwin(analysis_name); set topwin(analysis_name) NONE}  \
	-text {DISMISS}  -width {10}

  pack append $topwin(analysis_name).perform  \
  $topwin(analysis_name).perform.dismiss {left frame w} \

  pack append $topwin(analysis_name)  \
  $topwin(analysis_name).menubar {top frame center fillx} \
  $topwin(analysis_name).perform {bottom frame center fillx}

  if {"[info procs XFEdit]" != ""} {
    catch "XFMiscBindWidgetTree $topwin(analysis_name)"
    after 2 "catch {XFEditSetShowWindows}"
  }
}


# Internal procedures

# eof
#
