# **********************************************************************
# Computer Vision/Image Processing Tool Project - Dr. Scott Umbaugh SIUE
# **********************************************************************
#
#             File Name: ana_edge.tcl
#           Description: 
#         Related Files: 
#   Initial Coding Date: Mon Aug 12 14:48:53 CDT 1996
#           Portability: TCL script file
#            References:
#             Author(s): Yansheng Wei
#                        Southern Illinois University @ Edwardsville
#
## Copyright (C) 1996 SIUE - by Scott Umbaugh and Yansheng Wei.
##
## Permission to use, copy, modify, and distribute this software and its
## documentation for any purpose and without fee is hereby granted, provided
## that the above copyright notice appear in all copies and that both that
## copyright notice and this permission notice appear in supporting
## documentation.  This software is provided "as is" without express or
## implied warranty.
##
# RCS (Revision Control System) Information
# ...Added automatically by RCS 
#
## $Log: ana_edge.tcl,v $
## Revision 1.23  1997/08/07 16:49:29  smakam
## made the rcs file to be consistent
##
## Revision 1.22  1997/06/19 22:06:01  yawei
## setup initial position
##
## Revision 1.21  1997/06/09 21:44:16  yawei
## gui configuration performed
##
## Revision 1.20  1997/06/06 20:46:08  yawei
## anal ==> analysis
##
## Revision 1.19  1997/06/02 14:43:26  wzheng
## fix the entry box in Analysis/Edge deteciton/Frei-chen
## change the values on the dropdown to correspond to 45 to 90 degrees (not 0 to 60)
##
## Revision 1.18  1997/03/11 22:46:12  yawei
## updated arrow buttons
##
## Revision 1.17  1997/03/11 20:55:12  yawei
## Tcl/Tk script upgrading from tcl7.3/tk3.6 to tcl7.6/tk4.2
## compatible. This process is automated with tkboost perl
## script. @ Tue Mar 11 14:51:36 CST 1997
##
## Revision 1.16  1996/11/15 23:04:10  yawei
## Added the threshold image in edge_detect to the queue.
##
# Revision 1.15  1996/11/15  02:28:07  yawei
# hooked up the view threshold button for edge detectors.
#
# Revision 1.14  1996/10/13  19:11:08  mzuke
# moved 'view image' after 'image_post' to correctly label
# displayed images; installed watch cursor during perform;
# checked for too-long new image names, none changed
#
# Revision 1.13  1996/09/18  18:52:36  yawei
# changed the post_threshold boundary to -5000 - 5000. How can we
# tell whether it doesn't need post_thresh or the value is -1?
#
# Revision 1.12  1996/09/17  22:02:38  mzuke
# removed threshold from edge_link, placed button and
# entry box on PERFORM bar
#
# Revision 1.11  1996/09/14  03:32:38  yawei
# added threshold to Edge link
#
# Revision 1.10  1996/09/13  13:52:43  mzuke
# added code to show laplacian masks/ Roberts types
#
# Revision 1.9  1996/09/03  18:05:10  yawei
# changed anal_ to analysis_
#
# Revision 1.8  1996/08/29  18:45:26  mzuke
# one more 'symbol' correction
#
# Revision 1.7  1996/08/29  18:33:55  mzuke
# changed font designation from {symbol} to full listing
# to restore arrow labels after systemwide font changes
#
# Revision 1.6  1996/08/28  19:52:44  mzuke
# added edge_link to toplevel
#
# Revision 1.5  1996/08/20  18:17:35  mzuke
# fixed bug in Sobel perform
#
# Revision 1.4  1996/08/16  00:56:01  yawei
# 1) changed the foreground colors of Display_entries, so it is different from
#    ordinary entries
# 2) switched the order of edge subspace and line subspace
# 3) for Frei-chen. With different selection of Projection, the Threshold
#    angle and value will be packed or forgot. This is clearer than just
# entryconfigure the entry (the user can still play with the arrow) -state disabled 

# 4) added 1, 2 as the default values to Connect distance in hough
#
# Revision 1.3  1996/08/15  10:47:27  mzuke
# removed toplevel_position from wm geometry to allow window
# positioning by user
#
# Revision 1.2  1996/08/13  01:31:57  yawei
# make changes so Hough works.
#
# Revision 1.1  1996/08/12  19:49:24  yawei
# Initial revision
#
#
#****************************************************************************
# procedure to show window $topwin(analysis_name).
proc analysis_edge { } {# xf ignore me 7

  # Window manager configurations
  global tk_version
  global topwin
  global toplevel_position
  global analysis
  global GuiColor
  global GUI_DEBUG
  global GuiMisc
  set analysis(ed_threshangle_yn) "yes"

  if [ winfo exists  $topwin(analysis_name).info] {
     set tempname $topwin(analysis_name);\
destroy $topwin(analysis_name).info;\
set topwin(analysis_name) $tempname;}
  if [ winfo exists  $topwin(analysis_name).current_image] {
     set tempname $topwin(analysis_name);\
destroy $topwin(analysis_name).current_image;\
set topwin(analysis_name) $tempname;}
  if [ winfo exists  $topwin(analysis_name).perform] {
     set tempname $topwin(analysis_name);\
destroy $topwin(analysis_name).perform;\
set topwin(analysis_name) $tempname;}
  wm positionfrom  $topwin(analysis_name) user
  wm sizefrom  $topwin(analysis_name) user

  wm geometry  $topwin(analysis_name) $topwin(analysis_edge)[get_pos $topwin(analysis_name)]
  wm maxsize  $topwin(analysis_name) [lindex $topwin(analysis_edge_max) 0] \
     [lindex $topwin(analysis_edge_max) 1]
  wm minsize  $topwin(analysis_name) [lindex $topwin(analysis_edge_min) 0] \
     [lindex $topwin(analysis_edge_min) 1]
  wm title  $topwin(analysis_name) {ANALYSIS -> Edge/Line detection}


  # build widget $topwin(analysis_name).info
  frame $topwin(analysis_name).info \
    -background "$GuiColor(frame_bg)"  \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame11
  frame $topwin(analysis_name).info.frame11 \
    -background "$GuiColor(frame_bg)"  \
    -borderwidth {2} \
    -relief {ridge}

  # build widget $topwin(analysis_name).info.frame11.frame18
  frame $topwin(analysis_name).info.frame11.frame18 \
    -background "$GuiColor(frame_bg)"  \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame11.frame18.radiobutton20
	radiobutton $topwin(analysis_name).info.frame11.frame18.radiobutton20 -padx {1} -pady {1} -highlightthickness {0}  \
   -activebackground "$GuiColor(btn_act_bg)" \
   -background "$GuiColor(btn_bg)" \
    -relief {flat} \
    -text {Kirsch     } \
    -value {kirsch} \
    -variable {analysis(edge)}

  # build widget $topwin(analysis_name).info.frame11.frame18.radiobutton21
	radiobutton $topwin(analysis_name).info.frame11.frame18.radiobutton21 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -relief {flat} \
    -text {Robinson } \
    -value {robinson} \
    -variable {analysis(edge)}

  # build widget $topwin(analysis_name).info.frame11.frame18.frame45
  frame $topwin(analysis_name).info.frame11.frame18.frame45 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame11.frame18.frame45.frame0
  frame $topwin(analysis_name).info.frame11.frame18.frame45.frame0 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame11.frame18.frame45.frame0.label1
  label $topwin(analysis_name).info.frame11.frame18.frame45.frame0.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -text {  Pre-filter:}

  # build widget $topwin(analysis_name).info.frame11.frame18.frame45.frame0.value
  entry $topwin(analysis_name).info.frame11.frame18.frame45.frame0.value \
    -background "$GuiColor(rentry_bg)" \
    -foreground "$GuiColor(rentry_fg)" \
    -relief {sunken} \
    -state {disabled} \
    -textvariable {analysis(ed_smooth)} \
    -width {10}

  # build widget $topwin(analysis_name).info.frame11.frame18.frame45.frame0.menubutton2
	menubutton $topwin(analysis_name).info.frame11.frame18.frame45.frame0.menubutton2 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)" \
    -text "$GuiMisc(pop_up_text)" \
	 -width {2}

#    -menu ".frame1.frame11.frame18.frame45.frame0.menubutton2.menu" \

  # bindings
  bind $topwin(analysis_name).info.frame11.frame18.frame45.frame0.menubutton2 <Button-1> {MenuPopupPost $topwin(analysis_name).info.frame11.frame18.frame45.frame0.menubutton2.menu %X %Y}
  bind $topwin(analysis_name).info.frame11.frame18.frame45.frame0.menubutton2 <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame11.frame18.frame45.frame0.menubutton2.menu %W}

  # build widget $topwin(analysis_name).info.frame11.frame18.frame45.frame0.menubutton2.menu
  menu $topwin(analysis_name).info.frame11.frame18.frame45.frame0.menubutton2.menu \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" 
  $topwin(analysis_name).info.frame11.frame18.frame45.frame0.menubutton2.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(ed_smooth) none} \
    -label {none}
  $topwin(analysis_name).info.frame11.frame18.frame45.frame0.menubutton2.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(ed_smooth) average} \
    -label {average}
  $topwin(analysis_name).info.frame11.frame18.frame45.frame0.menubutton2.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(ed_smooth) ctr_wt_avg} \
    -label {center-weighted average}
  $topwin(analysis_name).info.frame11.frame18.frame45.frame0.menubutton2.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(ed_smooth) gaussian1} \
    -label {gaussian1}
  $topwin(analysis_name).info.frame11.frame18.frame45.frame0.menubutton2.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(ed_smooth) gaussian2} \
    -label {gaussian2}
  bindtags $topwin(analysis_name).info.frame11.frame18.frame45.frame0.menubutton2.menu {Menu all}
  # bindings
  bind $topwin(analysis_name).info.frame11.frame18.frame45.frame0.menubutton2.menu <B1-Motion> {MenuPopupMotion $topwin(analysis_name).info.frame11.frame18.frame45.frame0.menubutton2.menu %W %X %Y}
  bind $topwin(analysis_name).info.frame11.frame18.frame45.frame0.menubutton2.menu <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame11.frame18.frame45.frame0.menubutton2.menu %W}

  # pack widget $topwin(analysis_name).info.frame11.frame18.frame45.frame0
  pack append $topwin(analysis_name).info.frame11.frame18.frame45.frame0 \
    $topwin(analysis_name).info.frame11.frame18.frame45.frame0.label1 {left frame center fill} \
    $topwin(analysis_name).info.frame11.frame18.frame45.frame0.value {left frame center expand fill} \
    $topwin(analysis_name).info.frame11.frame18.frame45.frame0.menubutton2 {right frame center fill}

  # build widget $topwin(analysis_name).info.frame11.frame18.frame45.frame4
  frame $topwin(analysis_name).info.frame11.frame18.frame45.frame4 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame11.frame18.frame45.frame4.label1
  label $topwin(analysis_name).info.frame11.frame18.frame45.frame4.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -text {Post-threshold:}

  # build widget $topwin(analysis_name).info.frame11.frame18.frame45.frame4.value
  entry $topwin(analysis_name).info.frame11.frame18.frame45.frame4.value \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
    -textvariable {analysis(ed_thresh)} \
    -width {5}
  endEntry $topwin(analysis_name).info.frame11.frame18.frame45.frame4.value

  # build widget $topwin(analysis_name).info.frame11.frame18.frame45.frame4.menubutton2
	menubutton $topwin(analysis_name).info.frame11.frame18.frame45.frame4.menubutton2 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)" \
    -text "$GuiMisc(pop_up_text)" \
	 -width {2}

#    -menu ".frame1.frame11.frame18.frame45.frame4.menubutton2.menu" \

  # bindings
  bind $topwin(analysis_name).info.frame11.frame18.frame45.frame4.menubutton2 <Button-1> {MenuPopupPost $topwin(analysis_name).info.frame11.frame18.frame45.frame4.menubutton2.menu %X %Y}
  bind $topwin(analysis_name).info.frame11.frame18.frame45.frame4.menubutton2 <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame11.frame18.frame45.frame4.menubutton2.menu %W}

  # build widget $topwin(analysis_name).info.frame11.frame18.frame45.frame4.menubutton2.menu
  menu $topwin(analysis_name).info.frame11.frame18.frame45.frame4.menubutton2.menu \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" 
  $topwin(analysis_name).info.frame11.frame18.frame45.frame4.menubutton2.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(ed_thresh) none} \
    -label {none}
  $topwin(analysis_name).info.frame11.frame18.frame45.frame4.menubutton2.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(ed_thresh) 32} \
    -label {32}
  $topwin(analysis_name).info.frame11.frame18.frame45.frame4.menubutton2.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(ed_thresh) 64} \
    -label {64}
  $topwin(analysis_name).info.frame11.frame18.frame45.frame4.menubutton2.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(ed_thresh) 128} \
    -label {128}
  $topwin(analysis_name).info.frame11.frame18.frame45.frame4.menubutton2.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(ed_thresh) 192} \
    -label {192}

  bindtags $topwin(analysis_name).info.frame11.frame18.frame45.frame4.menubutton2.menu {Menu all}

  # bindings
  bind $topwin(analysis_name).info.frame11.frame18.frame45.frame4.menubutton2.menu <B1-Motion> {MenuPopupMotion $topwin(analysis_name).info.frame11.frame18.frame45.frame4.menubutton2.menu %W %X %Y}
  bind $topwin(analysis_name).info.frame11.frame18.frame45.frame4.menubutton2.menu <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame11.frame18.frame45.frame4.menubutton2.menu %W}

  # pack widget $topwin(analysis_name).info.frame11.frame18.frame45.frame4
  pack append $topwin(analysis_name).info.frame11.frame18.frame45.frame4 \
    $topwin(analysis_name).info.frame11.frame18.frame45.frame4.label1 {left frame center fill} \
    $topwin(analysis_name).info.frame11.frame18.frame45.frame4.value {left frame center expand fill} \
    $topwin(analysis_name).info.frame11.frame18.frame45.frame4.menubutton2 {right frame center fill}

  # pack widget $topwin(analysis_name).info.frame11.frame18.frame45
  pack append $topwin(analysis_name).info.frame11.frame18.frame45 \
    $topwin(analysis_name).info.frame11.frame18.frame45.frame0 {left frame center} \
    $topwin(analysis_name).info.frame11.frame18.frame45.frame4 {left frame center}

  # build widget $topwin(analysis_name).info.frame11.frame18.radiobutton0
	radiobutton $topwin(analysis_name).info.frame11.frame18.radiobutton0 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -relief {flat} \
    -text {Pyramid} \
    -value {pyramid} \
    -variable {analysis(edge)}

  # pack widget $topwin(analysis_name).info.frame11.frame18
  pack append $topwin(analysis_name).info.frame11.frame18 \
    $topwin(analysis_name).info.frame11.frame18.radiobutton20 {left frame center} \
    $topwin(analysis_name).info.frame11.frame18.radiobutton0 {left frame center} \
    $topwin(analysis_name).info.frame11.frame18.radiobutton21 {left frame center} \
    $topwin(analysis_name).info.frame11.frame18.frame45 {top frame center expand fill}

  # pack widget $topwin(analysis_name).info.frame11
  pack append $topwin(analysis_name).info.frame11 \
    $topwin(analysis_name).info.frame11.frame18 {top frame center expand fill}

  # build widget $topwin(analysis_name).info.frame12
  frame $topwin(analysis_name).info.frame12 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2} \
    -relief {ridge}

  # build widget $topwin(analysis_name).info.frame12.frame26
  frame $topwin(analysis_name).info.frame12.frame26 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame12.frame26.radiobutton28
	radiobutton $topwin(analysis_name).info.frame12.frame26.radiobutton28 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -command {set analysis(ed_kernel) 3} \
    -relief {flat} \
    -text {Sobel      } \
    -value {sobel} \
    -variable {analysis(edge)}

  # build widget $topwin(analysis_name).info.frame12.frame26.radiobutton29
	radiobutton $topwin(analysis_name).info.frame12.frame26.radiobutton29 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -command {set analysis(ed_kernel) 3} \
    -relief {flat} \
    -text {Prewitt   } \
    -value {prewitt} \
    -variable {analysis(edge)}

  # build widget $topwin(analysis_name).info.frame12.frame26.frame0
  frame $topwin(analysis_name).info.frame12.frame26.frame0 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame12.frame26.frame0.value
  entry $topwin(analysis_name).info.frame12.frame26.frame0.value \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
    -textvariable {analysis(ed_kernel)} \
    -width {3}
  endEntry $topwin(analysis_name).info.frame12.frame26.frame0.value 0\
  	$topwin(analysis_name).info.frame12.frame45.frame4.value \
  	$topwin(analysis_name).info.frame12.frame45.frame4.value 

  # build widget $topwin(analysis_name).info.frame12.frame26.frame0.menubutton2
	menubutton $topwin(analysis_name).info.frame12.frame26.frame0.menubutton2 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)" \
    -text "$GuiMisc(pop_up_text)" -width {2}
  # bindings
  bind $topwin(analysis_name).info.frame12.frame26.frame0.menubutton2 <Button-1> {MenuPopupPost $topwin(analysis_name).info.frame12.frame26.frame0.menubutton2.menu %X %Y}
  bind $topwin(analysis_name).info.frame12.frame26.frame0.menubutton2 <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame12.frame26.frame0.menubutton2.menu %W}

  # build widget $topwin(analysis_name).info.frame12.frame26.frame0.menubutton2.menu
  menu $topwin(analysis_name).info.frame12.frame26.frame0.menubutton2.menu \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" 
  $topwin(analysis_name).info.frame12.frame26.frame0.menubutton2.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(ed_kernel) 3} \
    -label {3}
  $topwin(analysis_name).info.frame12.frame26.frame0.menubutton2.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(ed_kernel) 5} \
    -label {5}
  $topwin(analysis_name).info.frame12.frame26.frame0.menubutton2.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(ed_kernel) 7} \
    -label {7}
  bindtags $topwin(analysis_name).info.frame12.frame26.frame0.menubutton2.menu {Menu all}
  # bindings
  bind $topwin(analysis_name).info.frame12.frame26.frame0.menubutton2.menu <B1-Motion> {MenuPopupMotion $topwin(analysis_name).info.frame12.frame26.frame0.menubutton2.menu %W %X %Y}
  bind $topwin(analysis_name).info.frame12.frame26.frame0.menubutton2.menu <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame12.frame26.frame0.menubutton2.menu %W}

  # build widget $topwin(analysis_name).info.frame12.frame26.frame0.label1
  label $topwin(analysis_name).info.frame12.frame26.frame0.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -text {   Use a kernel size of:}

  # pack widget $topwin(analysis_name).info.frame12.frame26.frame0
  pack append $topwin(analysis_name).info.frame12.frame26.frame0 \
    $topwin(analysis_name).info.frame12.frame26.frame0.label1 {left frame center fill} \
    $topwin(analysis_name).info.frame12.frame26.frame0.value {left frame center expand fill} \
    $topwin(analysis_name).info.frame12.frame26.frame0.menubutton2 {right frame center fill}

  # pack widget $topwin(analysis_name).info.frame12.frame26
  pack append $topwin(analysis_name).info.frame12.frame26 \
    $topwin(analysis_name).info.frame12.frame26.radiobutton28 {left frame center} \
    $topwin(analysis_name).info.frame12.frame26.radiobutton29 {left frame center} \
    $topwin(analysis_name).info.frame12.frame26.frame0 {left frame center fillx}

  # build widget $topwin(analysis_name).info.frame12.frame45
  frame $topwin(analysis_name).info.frame12.frame45 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame12.frame45.frame0
  frame $topwin(analysis_name).info.frame12.frame45.frame0 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame12.frame45.frame0.label1
  label $topwin(analysis_name).info.frame12.frame45.frame0.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -text {Pre-filter:}

  # build widget $topwin(analysis_name).info.frame12.frame45.frame0.value
  entry $topwin(analysis_name).info.frame12.frame45.frame0.value \
    -background "$GuiColor(rentry_bg)" \
    -foreground "$GuiColor(rentry_fg)" \
    -relief {sunken} \
    -state {disabled} \
    -textvariable {analysis(ed_smooth)} \
    -width {10}

  # build widget $topwin(analysis_name).info.frame12.frame45.frame0.menubutton2
	menubutton $topwin(analysis_name).info.frame12.frame45.frame0.menubutton2 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)" \
    -text "$GuiMisc(pop_up_text)" -width {2}
  # bindings
  bind $topwin(analysis_name).info.frame12.frame45.frame0.menubutton2 <Button-1> {MenuPopupPost $topwin(analysis_name).info.frame12.frame45.frame0.menubutton2.menu %X %Y}
  bind $topwin(analysis_name).info.frame12.frame45.frame0.menubutton2 <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame12.frame45.frame0.menubutton2.menu %W}

  # build widget $topwin(analysis_name).info.frame12.frame45.frame0.menubutton2.menu
  menu $topwin(analysis_name).info.frame12.frame45.frame0.menubutton2.menu \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" 
  $topwin(analysis_name).info.frame12.frame45.frame0.menubutton2.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(ed_smooth) none} \
    -label {none}
  $topwin(analysis_name).info.frame12.frame45.frame0.menubutton2.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(ed_smooth) average} \
    -label {average}
  $topwin(analysis_name).info.frame12.frame45.frame0.menubutton2.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(ed_smooth) ctr_wt_avg} \
    -label {center-weighted average}
  $topwin(analysis_name).info.frame12.frame45.frame0.menubutton2.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(ed_smooth) gaussian1} \
    -label {gaussian1}
  $topwin(analysis_name).info.frame12.frame45.frame0.menubutton2.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(ed_smooth) gaussian2} \
    -label {gaussian2}
  bindtags $topwin(analysis_name).info.frame12.frame45.frame0.menubutton2.menu {Menu all}
  # bindings
  bind $topwin(analysis_name).info.frame12.frame45.frame0.menubutton2.menu <B1-Motion> {MenuPopupMotion $topwin(analysis_name).info.frame12.frame45.frame0.menubutton2.menu %W %X %Y}
  bind $topwin(analysis_name).info.frame12.frame45.frame0.menubutton2.menu <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame12.frame45.frame0.menubutton2.menu %W}

  # pack widget $topwin(analysis_name).info.frame12.frame45.frame0
  pack append $topwin(analysis_name).info.frame12.frame45.frame0 \
    $topwin(analysis_name).info.frame12.frame45.frame0.label1 {left frame center fill} \
    $topwin(analysis_name).info.frame12.frame45.frame0.value {left frame center expand fill} \
    $topwin(analysis_name).info.frame12.frame45.frame0.menubutton2 {right frame center fill}

  # build widget $topwin(analysis_name).info.frame12.frame45.frame3
  frame $topwin(analysis_name).info.frame12.frame45.frame3 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame12.frame45.frame3.frame5
  frame $topwin(analysis_name).info.frame12.frame45.frame3.frame5 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame12.frame45.frame3.frame5.radiobutton7
	radiobutton $topwin(analysis_name).info.frame12.frame45.frame3.frame5.radiobutton7 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -relief {flat} \
    -text {and keep         } \
    -value {1} \
    -variable {analysis(ed_dc)}

  # build widget $topwin(analysis_name).info.frame12.frame45.frame3.frame5.radiobutton8
	radiobutton $topwin(analysis_name).info.frame12.frame45.frame3.frame5.radiobutton8 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -relief {flat} \
    -text {and don't keep} \
    -value {0} \
    -variable {analysis(ed_dc)}

  # pack widget $topwin(analysis_name).info.frame12.frame45.frame3.frame5
  pack append $topwin(analysis_name).info.frame12.frame45.frame3.frame5 \
    $topwin(analysis_name).info.frame12.frame45.frame3.frame5.radiobutton7 {top frame center} \
    $topwin(analysis_name).info.frame12.frame45.frame3.frame5.radiobutton8 {top frame center}

  # build widget $topwin(analysis_name).info.frame12.frame45.frame3.frame6
  frame $topwin(analysis_name).info.frame12.frame45.frame3.frame6 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame12.frame45.frame3.frame6.label9
  label $topwin(analysis_name).info.frame12.frame45.frame3.frame6.label9 \
    -background "$GuiColor(label_bg)" \
    -text {the dc.}

  # pack widget $topwin(analysis_name).info.frame12.frame45.frame3.frame6
  pack append $topwin(analysis_name).info.frame12.frame45.frame3.frame6 \
    $topwin(analysis_name).info.frame12.frame45.frame3.frame6.label9 {top frame center expand fillx}

  # pack widget $topwin(analysis_name).info.frame12.frame45.frame3
  pack append $topwin(analysis_name).info.frame12.frame45.frame3 \
    $topwin(analysis_name).info.frame12.frame45.frame3.frame5 {left frame center} \
    $topwin(analysis_name).info.frame12.frame45.frame3.frame6 {left frame center expand fill}

  # build widget $topwin(analysis_name).info.frame12.frame45.frame4
  frame $topwin(analysis_name).info.frame12.frame45.frame4 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame12.frame45.frame4.label1
  label $topwin(analysis_name).info.frame12.frame45.frame4.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -text {Post-threshold:}

  # build widget $topwin(analysis_name).info.frame12.frame45.frame4.value
  entry $topwin(analysis_name).info.frame12.frame45.frame4.value \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
    -textvariable {analysis(ed_thresh)} \
    -width {5}
  endEntry $topwin(analysis_name).info.frame12.frame45.frame4.value 0 \
  	$topwin(analysis_name).info.frame12.frame26.frame0.value \
  	$topwin(analysis_name).info.frame12.frame26.frame0.value 

  # build widget $topwin(analysis_name).info.frame12.frame45.frame4.menubutton0
	menubutton $topwin(analysis_name).info.frame12.frame45.frame4.menubutton0 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)" \
    -text "$GuiMisc(pop_up_text)" -width {2}
  # bindings
  bind $topwin(analysis_name).info.frame12.frame45.frame4.menubutton0 <Button-1> {MenuPopupPost $topwin(analysis_name).info.frame12.frame45.frame4.menubutton0.menu %X %Y}
  bind $topwin(analysis_name).info.frame12.frame45.frame4.menubutton0 <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame12.frame45.frame4.menubutton0.menu %W}

  # build widget $topwin(analysis_name).info.frame12.frame45.frame4.menubutton0.menu
  menu $topwin(analysis_name).info.frame12.frame45.frame4.menubutton0.menu \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" 
  $topwin(analysis_name).info.frame12.frame45.frame4.menubutton0.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(ed_thresh) none} \
    -label {none}
  $topwin(analysis_name).info.frame12.frame45.frame4.menubutton0.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(ed_thresh) 32} \
    -label {32}
  $topwin(analysis_name).info.frame12.frame45.frame4.menubutton0.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(ed_thresh) 64} \
    -label {64}
  $topwin(analysis_name).info.frame12.frame45.frame4.menubutton0.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(ed_thresh) 128} \
    -label {128}
  $topwin(analysis_name).info.frame12.frame45.frame4.menubutton0.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(ed_thresh) 192} \
    -label {192}
  bindtags $topwin(analysis_name).info.frame12.frame45.frame4.menubutton0.menu  {Menu all}
  # bindings
  bind $topwin(analysis_name).info.frame12.frame45.frame4.menubutton0.menu <B1-Motion> {MenuPopupMotion $topwin(analysis_name).info.frame12.frame45.frame4.menubutton0.menu %W %X %Y}
  bind $topwin(analysis_name).info.frame12.frame45.frame4.menubutton0.menu <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame12.frame45.frame4.menubutton0.menu %W}

  # pack widget $topwin(analysis_name).info.frame12.frame45.frame4
  pack append $topwin(analysis_name).info.frame12.frame45.frame4 \
    $topwin(analysis_name).info.frame12.frame45.frame4.label1 {left frame center fill} \
    $topwin(analysis_name).info.frame12.frame45.frame4.value {left frame center expand fill} \
    $topwin(analysis_name).info.frame12.frame45.frame4.menubutton0 {right frame center fill}

  # build widget $topwin(analysis_name).info.frame12.frame45.label10
  label $topwin(analysis_name).info.frame12.frame45.label10 \
    -background "$GuiColor(label_bg)" \
    -text {} \
    -width {24}

  # pack widget $topwin(analysis_name).info.frame12.frame45
  pack append $topwin(analysis_name).info.frame12.frame45 \
    $topwin(analysis_name).info.frame12.frame45.label10 {left frame center} \
    $topwin(analysis_name).info.frame12.frame45.frame0 {left frame center} \
    $topwin(analysis_name).info.frame12.frame45.frame4 {left frame center} \
    $topwin(analysis_name).info.frame12.frame45.frame3 {left frame center}

  # pack widget $topwin(analysis_name).info.frame12
  pack append $topwin(analysis_name).info.frame12 \
    $topwin(analysis_name).info.frame12.frame26 {top frame center expand fill} \
    $topwin(analysis_name).info.frame12.frame45 {top frame center expand fill}

  # build widget $topwin(analysis_name).info.frame13
  frame $topwin(analysis_name).info.frame13 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2} \
    -relief {ridge}

  # build widget $topwin(analysis_name).info.frame13.frame26
  frame $topwin(analysis_name).info.frame13.frame26 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame13.frame26.radiobutton28
	radiobutton $topwin(analysis_name).info.frame13.frame26.radiobutton28 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -command {set roblap1 Type1
set roblap2 Type2; set roblap_show "Show Types"} \
    -relief {flat} \
    -text {Roberts   } \
    -value {roberts} \
    -variable {analysis(edge)}

  # build widget $topwin(analysis_name).info.frame13.frame26.radiobutton29
	radiobutton $topwin(analysis_name).info.frame13.frame26.radiobutton29 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -command {set roblap1 Mask1
set roblap2 Mask2
set roblap_show "Show Masks"} \
    -relief {flat} \
    -text {Laplacian} \
    -value {laplacian} \
    -variable {analysis(edge)}

  # build widget $topwin(analysis_name).info.frame13.frame26.frame0
  frame $topwin(analysis_name).info.frame13.frame26.frame0 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame13.frame26.frame0.radiobutton38
	radiobutton $topwin(analysis_name).info.frame13.frame26.frame0.radiobutton38 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -relief {flat} \
    -text {Mask1} \
    -textvariable {roblap1} \
    -value {1} \
    -variable {analysis(rob_lap)}

  # build widget $topwin(analysis_name).info.frame13.frame26.frame0.radiobutton39
	radiobutton $topwin(analysis_name).info.frame13.frame26.frame0.radiobutton39 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -relief {flat} \
    -text {Mask2} \
    -textvariable {roblap2} \
    -value {2} \
    -variable {analysis(rob_lap)}

  # build widget $topwin(analysis_name).info.frame13.frame26.frame0.label40
  label $topwin(analysis_name).info.frame13.frame26.frame0.label40 \
    -background "$GuiColor(label_bg)" \
    -text {  Use:}

  # build widget $topwin(analysis_name).info.frame13.frame26.frame0.button41
	button $topwin(analysis_name).info.frame13.frame26.frame0.button41 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -command {if {$analysis(edge)=="roberts" || $analysis(edge)=="laplacian"} {
   if {$roblap_show=="Show Masks"} {help_screen; catch {source $env(CVIPHOME)/man/tcl_help/cvip_laplacian_masks.hlp} cresult}
   if {$roblap_show=="Show Types"} {help_screen; catch {source $env(CVIPHOME)/man/tcl_help/cvip_roberts_types.hlp} cresult}
} } \
    -text {Show Masks} \
    -textvariable {roblap_show} \
    -width {12}

  # build widget $topwin(analysis_name).info.frame13.frame26.frame0.label16
  label $topwin(analysis_name).info.frame13.frame26.frame0.label16 \
    -background "$GuiColor(label_bg)" \
    -text {     } \
    -width {5}

  # pack widget $topwin(analysis_name).info.frame13.frame26.frame0
  pack append $topwin(analysis_name).info.frame13.frame26.frame0 \
    $topwin(analysis_name).info.frame13.frame26.frame0.label40 {left frame center fillx} \
    $topwin(analysis_name).info.frame13.frame26.frame0.radiobutton38 {left frame center fill} \
    $topwin(analysis_name).info.frame13.frame26.frame0.radiobutton39 {left frame center fill} \
    $topwin(analysis_name).info.frame13.frame26.frame0.label16 {left frame center fillx} \
    $topwin(analysis_name).info.frame13.frame26.frame0.button41 {left frame center fillx}

  # pack widget $topwin(analysis_name).info.frame13.frame26
  pack append $topwin(analysis_name).info.frame13.frame26 \
    $topwin(analysis_name).info.frame13.frame26.radiobutton28 {left frame center} \
    $topwin(analysis_name).info.frame13.frame26.radiobutton29 {left frame center} \
    $topwin(analysis_name).info.frame13.frame26.frame0 {left frame center fillx}

  # build widget $topwin(analysis_name).info.frame13.frame45
  frame $topwin(analysis_name).info.frame13.frame45 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame13.frame45.frame0
  frame $topwin(analysis_name).info.frame13.frame45.frame0 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame13.frame45.frame0.label1
  label $topwin(analysis_name).info.frame13.frame45.frame0.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -text {Pre-filter:}

  # build widget $topwin(analysis_name).info.frame13.frame45.frame0.value
  entry $topwin(analysis_name).info.frame13.frame45.frame0.value \
    -background "$GuiColor(rentry_bg)" \
    -foreground "$GuiColor(rentry_fg)" \
    -relief {sunken} \
    -state {disabled} \
    -textvariable {analysis(ed_smooth)} \
    -width {10}

  # build widget $topwin(analysis_name).info.frame13.frame45.frame0.menubutton2
	menubutton $topwin(analysis_name).info.frame13.frame45.frame0.menubutton2 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)" \
    -text "$GuiMisc(pop_up_text)" -width {2}
  # bindings
  bind $topwin(analysis_name).info.frame13.frame45.frame0.menubutton2 <Button-1> {MenuPopupPost $topwin(analysis_name).info.frame13.frame45.frame0.menubutton2.menu %X %Y}
  bind $topwin(analysis_name).info.frame13.frame45.frame0.menubutton2 <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame13.frame45.frame0.menubutton2.menu %W}

  # build widget $topwin(analysis_name).info.frame13.frame45.frame0.menubutton2.menu
  menu $topwin(analysis_name).info.frame13.frame45.frame0.menubutton2.menu \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" 
  $topwin(analysis_name).info.frame13.frame45.frame0.menubutton2.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(ed_smooth) none} \
    -label {none}
  $topwin(analysis_name).info.frame13.frame45.frame0.menubutton2.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(ed_smooth) average} \
    -label {average}
  $topwin(analysis_name).info.frame13.frame45.frame0.menubutton2.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(ed_smooth) ctr_wt_avg} \
    -label {center-weighted average}
  $topwin(analysis_name).info.frame13.frame45.frame0.menubutton2.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(ed_smooth) gaussian1} \
    -label {gaussian1}
  $topwin(analysis_name).info.frame13.frame45.frame0.menubutton2.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(ed_smooth) gaussian2} \
    -label {gaussian2}
  bindtags $topwin(analysis_name).info.frame13.frame45.frame0.menubutton2.menu {Menu all}
  # bindings
  bind $topwin(analysis_name).info.frame13.frame45.frame0.menubutton2.menu <B1-Motion> {MenuPopupMotion $topwin(analysis_name).info.frame13.frame45.frame0.menubutton2.menu %W %X %Y}
  bind $topwin(analysis_name).info.frame13.frame45.frame0.menubutton2.menu <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame13.frame45.frame0.menubutton2.menu %W}

  # pack widget $topwin(analysis_name).info.frame13.frame45.frame0
  pack append $topwin(analysis_name).info.frame13.frame45.frame0 \
    $topwin(analysis_name).info.frame13.frame45.frame0.label1 {left frame center fill} \
    $topwin(analysis_name).info.frame13.frame45.frame0.value {left frame center expand fill} \
    $topwin(analysis_name).info.frame13.frame45.frame0.menubutton2 {right frame center fill}

  # build widget $topwin(analysis_name).info.frame13.frame45.frame3
  frame $topwin(analysis_name).info.frame13.frame45.frame3 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame13.frame45.frame3.frame5
  frame $topwin(analysis_name).info.frame13.frame45.frame3.frame5 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame13.frame45.frame3.frame5.radiobutton7
	radiobutton $topwin(analysis_name).info.frame13.frame45.frame3.frame5.radiobutton7 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -relief {flat} \
    -text {and keep         } \
    -value {1} \
    -variable {analysis(ed_dc)}

  # build widget $topwin(analysis_name).info.frame13.frame45.frame3.frame5.radiobutton8
	radiobutton $topwin(analysis_name).info.frame13.frame45.frame3.frame5.radiobutton8 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -relief {flat} \
    -text {and don't keep} \
    -value {0} \
    -variable {analysis(ed_dc)}

  # pack widget $topwin(analysis_name).info.frame13.frame45.frame3.frame5
  pack append $topwin(analysis_name).info.frame13.frame45.frame3.frame5 \
    $topwin(analysis_name).info.frame13.frame45.frame3.frame5.radiobutton7 {top frame center} \
    $topwin(analysis_name).info.frame13.frame45.frame3.frame5.radiobutton8 {top frame center}

  # build widget $topwin(analysis_name).info.frame13.frame45.frame3.frame6
  frame $topwin(analysis_name).info.frame13.frame45.frame3.frame6 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame13.frame45.frame3.frame6.label9
  label $topwin(analysis_name).info.frame13.frame45.frame3.frame6.label9 \
    -background "$GuiColor(label_bg)" \
    -text {the dc.}

  # pack widget $topwin(analysis_name).info.frame13.frame45.frame3.frame6
  pack append $topwin(analysis_name).info.frame13.frame45.frame3.frame6 \
    $topwin(analysis_name).info.frame13.frame45.frame3.frame6.label9 {top frame center expand fillx}

  # pack widget $topwin(analysis_name).info.frame13.frame45.frame3
  pack append $topwin(analysis_name).info.frame13.frame45.frame3 \
    $topwin(analysis_name).info.frame13.frame45.frame3.frame5 {left frame center} \
    $topwin(analysis_name).info.frame13.frame45.frame3.frame6 {left frame center expand fill}

  # build widget $topwin(analysis_name).info.frame13.frame45.frame4
  frame $topwin(analysis_name).info.frame13.frame45.frame4 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame13.frame45.frame4.label1
  label $topwin(analysis_name).info.frame13.frame45.frame4.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -text {Post-threshold:}

  # build widget $topwin(analysis_name).info.frame13.frame45.frame4.value
  entry $topwin(analysis_name).info.frame13.frame45.frame4.value \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
    -textvariable {analysis(ed_thresh)} \
    -width {5}

  # build widget $topwin(analysis_name).info.frame13.frame45.frame4.menubutton3
	menubutton $topwin(analysis_name).info.frame13.frame45.frame4.menubutton3 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)" \
    -text "$GuiMisc(pop_up_text)" -width {2}
  # bindings
  bind $topwin(analysis_name).info.frame13.frame45.frame4.menubutton3 <Button-1> {MenuPopupPost $topwin(analysis_name).info.frame13.frame45.frame4.menubutton3.menu %X %Y}
  bind $topwin(analysis_name).info.frame13.frame45.frame4.menubutton3 <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame13.frame45.frame4.menubutton3.menu %W}

  # build widget $topwin(analysis_name).info.frame13.frame45.frame4.menubutton3.menu
  menu $topwin(analysis_name).info.frame13.frame45.frame4.menubutton3.menu \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" 
  $topwin(analysis_name).info.frame13.frame45.frame4.menubutton3.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(ed_thresh) none} \
    -label {none}
  $topwin(analysis_name).info.frame13.frame45.frame4.menubutton3.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(ed_thresh) 32} \
    -label {32}
  $topwin(analysis_name).info.frame13.frame45.frame4.menubutton3.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(ed_thresh) 64} \
    -label {64}
  $topwin(analysis_name).info.frame13.frame45.frame4.menubutton3.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(ed_thresh) 128} \
    -label {128}
  $topwin(analysis_name).info.frame13.frame45.frame4.menubutton3.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(ed_thresh) 192} \
    -label {192}
  bindtags $topwin(analysis_name).info.frame13.frame45.frame4.menubutton3.menu {Menu all}
  # bindings
  bind $topwin(analysis_name).info.frame13.frame45.frame4.menubutton3.menu <B1-Motion> {MenuPopupMotion $topwin(analysis_name).info.frame13.frame45.frame4.menubutton3.menu %W %X %Y}
  bind $topwin(analysis_name).info.frame13.frame45.frame4.menubutton3.menu <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame13.frame45.frame4.menubutton3.menu %W}

  # pack widget $topwin(analysis_name).info.frame13.frame45.frame4
  pack append $topwin(analysis_name).info.frame13.frame45.frame4 \
    $topwin(analysis_name).info.frame13.frame45.frame4.label1 {left frame center fill} \
    $topwin(analysis_name).info.frame13.frame45.frame4.value {left frame center expand fill} \
    $topwin(analysis_name).info.frame13.frame45.frame4.menubutton3 {right frame center fill}

  # build widget $topwin(analysis_name).info.frame13.frame45.label10
  label $topwin(analysis_name).info.frame13.frame45.label10 \
    -background "$GuiColor(label_bg)" \
    -text {} \
    -width {24}

  # pack widget $topwin(analysis_name).info.frame13.frame45
  pack append $topwin(analysis_name).info.frame13.frame45 \
    $topwin(analysis_name).info.frame13.frame45.label10 {left frame center} \
    $topwin(analysis_name).info.frame13.frame45.frame0 {left frame center} \
    $topwin(analysis_name).info.frame13.frame45.frame4 {left frame center} \
    $topwin(analysis_name).info.frame13.frame45.frame3 {left frame center}

  # pack widget $topwin(analysis_name).info.frame13
  pack append $topwin(analysis_name).info.frame13 \
    $topwin(analysis_name).info.frame13.frame26 {top frame center expand fill} \
    $topwin(analysis_name).info.frame13.frame45 {top frame center expand fill}

  # build widget $topwin(analysis_name).info.frame14
  frame $topwin(analysis_name).info.frame14 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2} \
    -relief {ridge}

  # build widget $topwin(analysis_name).info.frame14.frame44
  frame $topwin(analysis_name).info.frame14.frame44 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame14.frame44.radiobutton43
	radiobutton $topwin(analysis_name).info.frame14.frame44.radiobutton43 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -relief {flat} \
    -text {Frei-Chen} \
    -value {frei_chen} \
    -variable {analysis(edge)}

  # build widget $topwin(analysis_name).info.frame14.frame44.frame0
  frame $topwin(analysis_name).info.frame14.frame44.frame0 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame14.frame44.frame0.label1
  label $topwin(analysis_name).info.frame14.frame44.frame0.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -text {                      Projection: }

  # build widget $topwin(analysis_name).info.frame14.frame44.frame0.value
  entry $topwin(analysis_name).info.frame14.frame44.frame0.value \
    -background "$GuiColor(rentry_bg)" \
    -foreground "$GuiColor(rentry_fg)" \
    -relief {sunken} \
    -state {disabled} \
    -textvariable {analysis(ed_projection)} \
    -width {17}

  # build widget $topwin(analysis_name).info.frame14.frame44.frame0.menubutton2
	menubutton $topwin(analysis_name).info.frame14.frame44.frame0.menubutton2 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)" \
    -text "$GuiMisc(pop_up_text)" -width {2}
  # bindings
  bind $topwin(analysis_name).info.frame14.frame44.frame0.menubutton2 <Button-1> {MenuPopupPost $topwin(analysis_name).info.frame14.frame44.frame0.menubutton2.menu %X %Y}
  bind $topwin(analysis_name).info.frame14.frame44.frame0.menubutton2 <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame14.frame44.frame0.menubutton2.menu %W}

  # build widget $topwin(analysis_name).info.frame14.frame44.frame0.menubutton2.menu
  menu $topwin(analysis_name).info.frame14.frame44.frame0.menubutton2.menu \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" 
  $topwin(analysis_name).info.frame14.frame44.frame0.menubutton2.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command "#OptionButtonSet $topwin(analysis_name).info.frame14.frame44.frame0
set analysis(ed_projection) {edge subspace}; set analysis(ed_threshangle_yn) yes
pack $topwin(analysis_name).info.frame14.frame44.frame1 -side left
pack $topwin(analysis_name).info.frame14.frame44.frame2 -side left " \
    -label {edge subspace}
  $topwin(analysis_name).info.frame14.frame44.frame0.menubutton2.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command "#OptionButtonSet $topwin(analysis_name).info.frame14.frame44.frame0
set analysis(ed_projection) {line subspace}; set analysis(ed_threshangle_yn) yes
pack $topwin(analysis_name).info.frame14.frame44.frame1 -side left
pack $topwin(analysis_name).info.frame14.frame44.frame2 -side left" \
    -label {line subspace}
  $topwin(analysis_name).info.frame14.frame44.frame0.menubutton2.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command "#OptionButtonSet $topwin(analysis_name).info.frame14.frame44.frame0
set analysis(ed_projection) {max\[edge, line\]}
pack forget $topwin(analysis_name).info.frame14.frame44.frame1
pack forget $topwin(analysis_name).info.frame14.frame44.frame2 " \
    -label {max[edge,line]}
  $topwin(analysis_name).info.frame14.frame44.frame0.menubutton2.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command "#OptionButtonSet $topwin(analysis_name).info.frame14.frame44.frame0
set analysis(ed_projection) {min angle\[edge,line\]}
pack forget $topwin(analysis_name).info.frame14.frame44.frame1
pack forget $topwin(analysis_name).info.frame14.frame44.frame2
" \
    -label {min angle[edge,line]}
  bindtags $topwin(analysis_name).info.frame14.frame44.frame0.menubutton2.menu {Menu all}
  # bindings
  bind $topwin(analysis_name).info.frame14.frame44.frame0.menubutton2.menu <B1-Motion> {MenuPopupMotion $topwin(analysis_name).info.frame14.frame44.frame0.menubutton2.menu %W %X %Y}
  bind $topwin(analysis_name).info.frame14.frame44.frame0.menubutton2.menu <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame14.frame44.frame0.menubutton2.menu %W}

  # pack widget $topwin(analysis_name).info.frame14.frame44.frame0
  pack append $topwin(analysis_name).info.frame14.frame44.frame0 \
    $topwin(analysis_name).info.frame14.frame44.frame0.label1 {left frame center fill} \
    $topwin(analysis_name).info.frame14.frame44.frame0.value {left frame center expand fill} \
    $topwin(analysis_name).info.frame14.frame44.frame0.menubutton2 {right frame center fill}

  # build widget $topwin(analysis_name).info.frame14.frame44.frame1
  frame $topwin(analysis_name).info.frame14.frame44.frame1 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame14.frame44.frame1.label1
  label $topwin(analysis_name).info.frame14.frame44.frame1.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -relief {sunken} \
    -text {Threshold angle:}

  # build widget $topwin(analysis_name).info.frame14.frame44.frame1.value
  entry $topwin(analysis_name).info.frame14.frame44.frame1.value \
    -background "$GuiColor(rentry_bg)" \
    -foreground "$GuiColor(rentry_fg)" \
    -relief {sunken} \
	 -state {disabled} \
    -textvariable {analysis(ed_threshangle_yn)} \
    -width {3}

  # build widget $topwin(analysis_name).info.frame14.frame44.frame1.menubutton2
	menubutton $topwin(analysis_name).info.frame14.frame44.frame1.menubutton2 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)" \
    -text "$GuiMisc(pop_up_text)" -width {2}
  # bindings
  bind $topwin(analysis_name).info.frame14.frame44.frame1.menubutton2 <Button-1> {MenuPopupPost $topwin(analysis_name).info.frame14.frame44.frame1.menubutton2.menu %X %Y}
  bind $topwin(analysis_name).info.frame14.frame44.frame1.menubutton2 <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame14.frame44.frame1.menubutton2.menu %W}

  # build widget $topwin(analysis_name).info.frame14.frame44.frame1.menubutton2.menu
  menu $topwin(analysis_name).info.frame14.frame44.frame1.menubutton2.menu \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" 
  $topwin(analysis_name).info.frame14.frame44.frame1.menubutton2.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command "set analysis(ed_threshangle_yn) yes
pack $topwin(analysis_name).info.frame14.frame44.frame2 -side left " \
    -label {yes}
  $topwin(analysis_name).info.frame14.frame44.frame1.menubutton2.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command "set analysis(ed_threshangle_yn) no
    pack forget $topwin(analysis_name).info.frame14.frame44.frame2 -side left" \
    -label {no}
  bindtags  $topwin(analysis_name).info.frame14.frame44.frame1.menubutton2.menu {Menu all}
  # bindings
  bind $topwin(analysis_name).info.frame14.frame44.frame1.menubutton2.menu <B1-Motion> {MenuPopupMotion $topwin(analysis_name).info.frame14.frame44.frame1.menubutton2.menu %W %X %Y}
  bind $topwin(analysis_name).info.frame14.frame44.frame1.menubutton2.menu <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame14.frame44.frame1.menubutton2.menu %W}

  # pack widget $topwin(analysis_name).info.frame14.frame44.frame1
  pack append $topwin(analysis_name).info.frame14.frame44.frame1 \
    $topwin(analysis_name).info.frame14.frame44.frame1.label1 {left frame center fill} \
    $topwin(analysis_name).info.frame14.frame44.frame1.value {left frame center expand fill} \
    $topwin(analysis_name).info.frame14.frame44.frame1.menubutton2 {right frame center fill}

  # build widget $topwin(analysis_name).info.frame14.frame44.frame2
  frame $topwin(analysis_name).info.frame14.frame44.frame2 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame14.frame44.frame2.label1
  label $topwin(analysis_name).info.frame14.frame44.frame2.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -text {, value(radians)}

  # build widget $topwin(analysis_name).info.frame14.frame44.frame2.value
  entry $topwin(analysis_name).info.frame14.frame44.frame2.value \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
    -textvariable {analysis(ed_threshangle_value)} \
    -width {5}
  endEntry $topwin(analysis_name).info.frame14.frame44.frame2.value 0 \
  	$topwin(analysis_name).info.frame14.frame45.frame4.frame4.value \
  	$topwin(analysis_name).info.frame14.frame45.frame4.frame4.value


  # build widget $topwin(analysis_name).info.frame14.frame44.frame2.menubutton2
	menubutton $topwin(analysis_name).info.frame14.frame44.frame2.menubutton2 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)" \
    -text "$GuiMisc(pop_up_text)" -width {2}
  # bindings
  bind $topwin(analysis_name).info.frame14.frame44.frame2.menubutton2 <Button-1> {MenuPopupPost $topwin(analysis_name).info.frame14.frame44.frame2.menubutton2.menu %X %Y}
  bind $topwin(analysis_name).info.frame14.frame44.frame2.menubutton2 <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame14.frame44.frame2.menubutton2.menu %W}

  # build widget $topwin(analysis_name).info.frame14.frame44.frame2.menubutton2.menu
  menu $topwin(analysis_name).info.frame14.frame44.frame2.menubutton2.menu \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" 
  $topwin(analysis_name).info.frame14.frame44.frame2.menubutton2.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(ed_threshangle_value) 0.79} \
    -label {0.79 (45 degrees)}
  $topwin(analysis_name).info.frame14.frame44.frame2.menubutton2.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(ed_threshangle_value) 0.87} \
    -label {0.87 (50 degrees)}
  $topwin(analysis_name).info.frame14.frame44.frame2.menubutton2.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(ed_threshangle_value) 1.05} \
    -label {1.05 (60 degrees)}
  $topwin(analysis_name).info.frame14.frame44.frame2.menubutton2.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(ed_threshangle_value) 1.22} \
    -label {1.22 (70 degrees)}
  $topwin(analysis_name).info.frame14.frame44.frame2.menubutton2.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(ed_threshangle_value) 1.40} \
    -label {1.40 (80 degrees)}
  $topwin(analysis_name).info.frame14.frame44.frame2.menubutton2.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(ed_threshangle_value) 1.57} \
    -label {1.57 (90 degrees)}

  bindtags $topwin(analysis_name).info.frame14.frame44.frame2.menubutton2.menu {Menu all}
  # bindings
  bind $topwin(analysis_name).info.frame14.frame44.frame2.menubutton2.menu <B1-Motion> {MenuPopupMotion $topwin(analysis_name).info.frame14.frame44.frame2.menubutton2.menu %W %X %Y}
  bind $topwin(analysis_name).info.frame14.frame44.frame2.menubutton2.menu <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame14.frame44.frame2.menubutton2.menu %W}

  # pack widget $topwin(analysis_name).info.frame14.frame44.frame2
  pack append $topwin(analysis_name).info.frame14.frame44.frame2 \
    $topwin(analysis_name).info.frame14.frame44.frame2.label1 {left frame center fill} \
    $topwin(analysis_name).info.frame14.frame44.frame2.value {left frame center expand fill} \
    $topwin(analysis_name).info.frame14.frame44.frame2.menubutton2 {right frame center fill}

  # pack widget $topwin(analysis_name).info.frame14.frame44
  pack append $topwin(analysis_name).info.frame14.frame44 \
    $topwin(analysis_name).info.frame14.frame44.radiobutton43 {left frame center} \
    $topwin(analysis_name).info.frame14.frame44.frame0 {left frame center} \
    $topwin(analysis_name).info.frame14.frame44.frame1 {left frame center fillx} \
    $topwin(analysis_name).info.frame14.frame44.frame2 {left frame center}

  # build widget $topwin(analysis_name).info.frame14.frame45
  frame $topwin(analysis_name).info.frame14.frame45 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame14.frame45.frame0
  frame $topwin(analysis_name).info.frame14.frame45.frame0 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame14.frame45.frame0.label1
  label $topwin(analysis_name).info.frame14.frame45.frame0.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -text {Pre-filter:}

  # build widget $topwin(analysis_name).info.frame14.frame45.frame0.value
  entry $topwin(analysis_name).info.frame14.frame45.frame0.value \
    -background "$GuiColor(rentry_bg)" \
    -foreground "$GuiColor(rentry_fg)" \
    -relief {sunken} \
    -state {disabled} \
    -textvariable {analysis(ed_smooth)} \
    -width {10}

  # build widget $topwin(analysis_name).info.frame14.frame45.frame0.menubutton2
	menubutton $topwin(analysis_name).info.frame14.frame45.frame0.menubutton2 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)" \
    -text "$GuiMisc(pop_up_text)" -width {2}
  # bindings
  bind $topwin(analysis_name).info.frame14.frame45.frame0.menubutton2 <Button-1> {MenuPopupPost $topwin(analysis_name).info.frame14.frame45.frame0.menubutton2.menu %X %Y}
  bind $topwin(analysis_name).info.frame14.frame45.frame0.menubutton2 <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame14.frame45.frame0.menubutton2.menu %W}

  # build widget $topwin(analysis_name).info.frame14.frame45.frame0.menubutton2.menu
  menu $topwin(analysis_name).info.frame14.frame45.frame0.menubutton2.menu \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" 
  $topwin(analysis_name).info.frame14.frame45.frame0.menubutton2.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(ed_smooth) none} \
    -label {none}
  $topwin(analysis_name).info.frame14.frame45.frame0.menubutton2.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(ed_smooth) average} \
    -label {average}
  $topwin(analysis_name).info.frame14.frame45.frame0.menubutton2.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(ed_smooth) ctr_wt_avg} \
    -label {center-weighted average}
  $topwin(analysis_name).info.frame14.frame45.frame0.menubutton2.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(ed_smooth) gaussian1} \
    -label {gaussian1}
  $topwin(analysis_name).info.frame14.frame45.frame0.menubutton2.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(ed_smooth) gaussian2} \
    -label {gaussian2}
  bindtags $topwin(analysis_name).info.frame14.frame45.frame0.menubutton2.menu {Menu all}
  # bindings
  bind $topwin(analysis_name).info.frame14.frame45.frame0.menubutton2.menu <B1-Motion> {MenuPopupMotion $topwin(analysis_name).info.frame14.frame45.frame0.menubutton2.menu %W %X %Y}
  bind $topwin(analysis_name).info.frame14.frame45.frame0.menubutton2.menu <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame14.frame45.frame0.menubutton2.menu %W}

  # pack widget $topwin(analysis_name).info.frame14.frame45.frame0
  pack append $topwin(analysis_name).info.frame14.frame45.frame0 \
    $topwin(analysis_name).info.frame14.frame45.frame0.label1 {left frame center fill} \
    $topwin(analysis_name).info.frame14.frame45.frame0.value {left frame center expand fill} \
    $topwin(analysis_name).info.frame14.frame45.frame0.menubutton2 {right frame center fill}

  # build widget $topwin(analysis_name).info.frame14.frame45.frame3
  frame $topwin(analysis_name).info.frame14.frame45.frame3 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame14.frame45.frame3.frame5
  frame $topwin(analysis_name).info.frame14.frame45.frame3.frame5 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame14.frame45.frame3.frame5.radiobutton7
	radiobutton $topwin(analysis_name).info.frame14.frame45.frame3.frame5.radiobutton7 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -relief {flat} \
    -text {and keep         } \
    -value {1} \
    -variable {analysis(ed_dc)}

  # build widget $topwin(analysis_name).info.frame14.frame45.frame3.frame5.radiobutton8
	radiobutton $topwin(analysis_name).info.frame14.frame45.frame3.frame5.radiobutton8 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -relief {flat} \
    -text {and don't keep} \
    -value {0} \
    -variable {analysis(ed_dc)}

  # pack widget $topwin(analysis_name).info.frame14.frame45.frame3.frame5
  pack append $topwin(analysis_name).info.frame14.frame45.frame3.frame5 \
    $topwin(analysis_name).info.frame14.frame45.frame3.frame5.radiobutton7 {top frame center} \
    $topwin(analysis_name).info.frame14.frame45.frame3.frame5.radiobutton8 {top frame center}

  # build widget $topwin(analysis_name).info.frame14.frame45.frame3.frame6
  frame $topwin(analysis_name).info.frame14.frame45.frame3.frame6 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame14.frame45.frame3.frame6.label9
  label $topwin(analysis_name).info.frame14.frame45.frame3.frame6.label9 \
    -background "$GuiColor(label_bg)" \
    -text {the dc.}

  # pack widget $topwin(analysis_name).info.frame14.frame45.frame3.frame6
  pack append $topwin(analysis_name).info.frame14.frame45.frame3.frame6 \
    $topwin(analysis_name).info.frame14.frame45.frame3.frame6.label9 {top frame center expand fillx}

  # pack widget $topwin(analysis_name).info.frame14.frame45.frame3
  pack append $topwin(analysis_name).info.frame14.frame45.frame3 \
    $topwin(analysis_name).info.frame14.frame45.frame3.frame5 {left frame center} \
    $topwin(analysis_name).info.frame14.frame45.frame3.frame6 {left frame center expand fill}

  # build widget $topwin(analysis_name).info.frame14.frame45.frame4
  frame $topwin(analysis_name).info.frame14.frame45.frame4 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame14.frame45.frame4.frame4
  frame $topwin(analysis_name).info.frame14.frame45.frame4.frame4 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame14.frame45.frame4.frame4.label1
  label $topwin(analysis_name).info.frame14.frame45.frame4.frame4.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -text {Post-threshold:}

  # build widget $topwin(analysis_name).info.frame14.frame45.frame4.frame4.value
  entry $topwin(analysis_name).info.frame14.frame45.frame4.frame4.value \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
    -textvariable {analysis(ed_thresh)} \
    -width {5}
  endEntry $topwin(analysis_name).info.frame14.frame45.frame4.frame4.value 0 \
  	$topwin(analysis_name).info.frame14.frame44.frame2.value \
  	$topwin(analysis_name).info.frame14.frame44.frame2.value 


  # build widget $topwin(analysis_name).info.frame14.frame45.frame4.frame4.menubutton3
	menubutton $topwin(analysis_name).info.frame14.frame45.frame4.frame4.menubutton3 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)" \
    -text "$GuiMisc(pop_up_text)" -width {2}
  # bindings
  bind $topwin(analysis_name).info.frame14.frame45.frame4.frame4.menubutton3 <Button-1> {MenuPopupPost $topwin(analysis_name).info.frame14.frame45.frame4.frame4.menubutton3.menu %X %Y}
  bind $topwin(analysis_name).info.frame14.frame45.frame4.frame4.menubutton3 <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame14.frame45.frame4.frame4.menubutton3.menu %W}

  # build widget $topwin(analysis_name).info.frame14.frame45.frame4.frame4.menubutton3.menu
  menu $topwin(analysis_name).info.frame14.frame45.frame4.frame4.menubutton3.menu \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" 
  $topwin(analysis_name).info.frame14.frame45.frame4.frame4.menubutton3.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(ed_thresh) none} \
    -label {none}
  $topwin(analysis_name).info.frame14.frame45.frame4.frame4.menubutton3.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(ed_thresh) 32} \
    -label {32}
  $topwin(analysis_name).info.frame14.frame45.frame4.frame4.menubutton3.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(ed_thresh) 64} \
    -label {64}
  $topwin(analysis_name).info.frame14.frame45.frame4.frame4.menubutton3.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(ed_thresh) 128} \
    -label {128}
  $topwin(analysis_name).info.frame14.frame45.frame4.frame4.menubutton3.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(ed_thresh) 192} \
    -label {192}
  bindtags $topwin(analysis_name).info.frame14.frame45.frame4.frame4.menubutton3.menu {Menu all}
  # bindings
  bind $topwin(analysis_name).info.frame14.frame45.frame4.frame4.menubutton3.menu <B1-Motion> {MenuPopupMotion $topwin(analysis_name).info.frame14.frame45.frame4.frame4.menubutton3.menu %W %X %Y}
  bind $topwin(analysis_name).info.frame14.frame45.frame4.frame4.menubutton3.menu <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame14.frame45.frame4.frame4.menubutton3.menu %W}

  # pack widget $topwin(analysis_name).info.frame14.frame45.frame4.frame4
  pack append $topwin(analysis_name).info.frame14.frame45.frame4.frame4 \
    $topwin(analysis_name).info.frame14.frame45.frame4.frame4.label1 {left frame center fill} \
    $topwin(analysis_name).info.frame14.frame45.frame4.frame4.value {left frame center expand fill} \
    $topwin(analysis_name).info.frame14.frame45.frame4.frame4.menubutton3 {right frame center fill}

  # pack widget $topwin(analysis_name).info.frame14.frame45.frame4
  pack append $topwin(analysis_name).info.frame14.frame45.frame4 \
    $topwin(analysis_name).info.frame14.frame45.frame4.frame4 {left frame center}

  # build widget $topwin(analysis_name).info.frame14.frame45.label10
  label $topwin(analysis_name).info.frame14.frame45.label10 \
    -background "$GuiColor(label_bg)" \
    -text {} \
    -width {24}

  # pack widget $topwin(analysis_name).info.frame14.frame45
  pack append $topwin(analysis_name).info.frame14.frame45 \
    $topwin(analysis_name).info.frame14.frame45.label10 {left frame center} \
    $topwin(analysis_name).info.frame14.frame45.frame0 {left frame center} \
    $topwin(analysis_name).info.frame14.frame45.frame4 {left frame center} \
    $topwin(analysis_name).info.frame14.frame45.frame3 {left frame center}

  # pack widget $topwin(analysis_name).info.frame14
  pack append $topwin(analysis_name).info.frame14 \
    $topwin(analysis_name).info.frame14.frame44 {top frame center expand fill} \
    $topwin(analysis_name).info.frame14.frame45 {top frame center expand fill}

  # build widget $topwin(analysis_name).info.frame25
  frame $topwin(analysis_name).info.frame25 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2} \
    -relief {ridge}

  # build widget $topwin(analysis_name).info.frame25.radiobutton18
	radiobutton $topwin(analysis_name).info.frame25.radiobutton18 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -command {if {$analysis(ed_hough_angles)!=""} {
   set ab1 [AlertBox "Do you want to clear the line angles listed?"\
	"" "300x150" "Hough Transform Alert" "Clear" "DO NOT Clear"]
   if {!$ab1} {
      $topwin(analysis_name).info.frame25.frame20.frame3.entry5 delete 0 end
   }
}} \
    -relief {flat} \
    -text {Hough} \
    -value {hough} \
    -variable {analysis(edge)}

  # build widget $topwin(analysis_name).info.frame25.frame0
  frame $topwin(analysis_name).info.frame25.frame0 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame25.frame0.label1
  label $topwin(analysis_name).info.frame25.frame0.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -text {Line pixels(min):}

  # build widget $topwin(analysis_name).info.frame25.frame0.value
  entry $topwin(analysis_name).info.frame25.frame0.value \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
    -textvariable {analysis(hough_line)} \
    -width {5}
  endEntry $topwin(analysis_name).info.frame25.frame0.value 0 \
  	$topwin(analysis_name).info.frame25.frame19.value \
  	$topwin(analysis_name).info.frame25.frame20.frame3.entry5 

  # build widget $topwin(analysis_name).info.frame25.frame0.menubutton2
	menubutton $topwin(analysis_name).info.frame25.frame0.menubutton2 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)" \
    -text "$GuiMisc(pop_up_text)" -width {2}
  # bindings
  bind $topwin(analysis_name).info.frame25.frame0.menubutton2 <Button-1> {MenuPopupPost $topwin(analysis_name).info.frame25.frame0.menubutton2.menu %X %Y}
  bind $topwin(analysis_name).info.frame25.frame0.menubutton2 <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame25.frame0.menubutton2.menu %W}

  # build widget $topwin(analysis_name).info.frame25.frame0.menubutton2.menu
  menu $topwin(analysis_name).info.frame25.frame0.menubutton2.menu \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" 
  $topwin(analysis_name).info.frame25.frame0.menubutton2.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(hough_line) 10} \
    -label {10}
  $topwin(analysis_name).info.frame25.frame0.menubutton2.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(hough_line) 20} \
    -label {20}
  $topwin(analysis_name).info.frame25.frame0.menubutton2.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(hough_line) 50} \
    -label {50}
  $topwin(analysis_name).info.frame25.frame0.menubutton2.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(hough_line) 100} \
    -label {100}
  bindtags $topwin(analysis_name).info.frame25.frame0.menubutton2.menu  {Menu all}
  # bindings
  bind $topwin(analysis_name).info.frame25.frame0.menubutton2.menu <B1-Motion> {MenuPopupMotion $topwin(analysis_name).info.frame25.frame0.menubutton2.menu %W %X %Y}
  bind $topwin(analysis_name).info.frame25.frame0.menubutton2.menu <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame25.frame0.menubutton2.menu %W}

  # pack widget $topwin(analysis_name).info.frame25.frame0
  pack append $topwin(analysis_name).info.frame25.frame0 \
    $topwin(analysis_name).info.frame25.frame0.label1 {left frame center fill} \
    $topwin(analysis_name).info.frame25.frame0.value {left frame center expand fill} \
    $topwin(analysis_name).info.frame25.frame0.menubutton2 {right frame center fill}

  # build widget $topwin(analysis_name).info.frame25.frame19
  frame $topwin(analysis_name).info.frame25.frame19 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame25.frame19.label1
  label $topwin(analysis_name).info.frame25.frame19.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -text {Connect distance(max):}

  # build widget $topwin(analysis_name).info.frame25.frame19.value
  entry $topwin(analysis_name).info.frame25.frame19.value \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
    -textvariable {analysis(hough_distance)} \
    -width {5}
  endEntry $topwin(analysis_name).info.frame25.frame19.value 0 \
  	$topwin(analysis_name).info.frame25.frame20.frame3.entry5 \
  	$topwin(analysis_name).info.frame25.frame0.value

  # build widget $topwin(analysis_name).info.frame25.frame19.menubutton2
	menubutton $topwin(analysis_name).info.frame25.frame19.menubutton2 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)" \
    -text "$GuiMisc(pop_up_text)" -width {2}
  # bindings
  bind $topwin(analysis_name).info.frame25.frame19.menubutton2 <Button-1> {MenuPopupPost $topwin(analysis_name).info.frame25.frame19.menubutton2.menu %X %Y}
  bind $topwin(analysis_name).info.frame25.frame19.menubutton2 <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame25.frame19.menubutton2.menu %W}

  # build widget $topwin(analysis_name).info.frame25.frame19.menubutton2.menu
  menu $topwin(analysis_name).info.frame25.frame19.menubutton2.menu \
   -activebackground "$GuiColor(btn_act_bg)" \
   -background "$GuiColor(btn_bg)" 
  $topwin(analysis_name).info.frame25.frame19.menubutton2.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(hough_distance) 1} \
    -label {  1 }
  $topwin(analysis_name).info.frame25.frame19.menubutton2.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(hough_distance) 2} \
    -label {  2 }
  $topwin(analysis_name).info.frame25.frame19.menubutton2.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(hough_distance) 5} \
    -label {  5 }
  $topwin(analysis_name).info.frame25.frame19.menubutton2.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(hough_distance) 10} \
    -label { 10 }
  $topwin(analysis_name).info.frame25.frame19.menubutton2.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(hough_distance) 15} \
    -label { 15 }
  $topwin(analysis_name).info.frame25.frame19.menubutton2.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(hough_distance) 20} \
    -label { 20 }
  bindtags $topwin(analysis_name).info.frame25.frame19.menubutton2.menu {Menu all}
  # bindings
  bind $topwin(analysis_name).info.frame25.frame19.menubutton2.menu <B1-Motion> {MenuPopupMotion $topwin(analysis_name).info.frame25.frame19.menubutton2.menu %W %X %Y}
  bind $topwin(analysis_name).info.frame25.frame19.menubutton2.menu <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame25.frame19.menubutton2.menu %W}

  # pack widget $topwin(analysis_name).info.frame25.frame19
  pack append $topwin(analysis_name).info.frame25.frame19 \
    $topwin(analysis_name).info.frame25.frame19.label1 {left frame center fill} \
    $topwin(analysis_name).info.frame25.frame19.value {left frame center expand fill} \
    $topwin(analysis_name).info.frame25.frame19.menubutton2 {right frame center fill}

  # build widget $topwin(analysis_name).info.frame25.frame20
  frame $topwin(analysis_name).info.frame25.frame20 \
    -background "$GuiColor(frame_bg)" \

  # build widget $topwin(analysis_name).info.frame25.frame20.scrollbar7
  scrollbar $topwin(analysis_name).info.frame25.frame20.scrollbar7 \
    -activebackground {gray75} \
    -troughcolor {gray90} \
    -command "$topwin(analysis_name).info.frame25.frame20.frame3.entry5 xview" \
    -background {gray60} \
    -orient {horizontal} \
    -width {9}

  # build widget $topwin(analysis_name).info.frame25.frame20.frame3
  frame $topwin(analysis_name).info.frame25.frame20.frame3 \
    -background "$GuiColor(frame_bg)" 

  # build widget $topwin(analysis_name).info.frame25.frame20.frame3.entry5
  entry $topwin(analysis_name).info.frame25.frame20.frame3.entry5 \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
 -xscrollcommand "$topwin(analysis_name).info.frame25.frame20.scrollbar7 set" \
    -textvariable {analysis(ed_hough_angles)} \
    -width {23}
  endEntry $topwin(analysis_name).info.frame25.frame20.frame3.entry5 0 \
  	$topwin(analysis_name).info.frame25.frame0.value \
  	$topwin(analysis_name).info.frame25.frame19.value  

  # build widget $topwin(analysis_name).info.frame25.frame20.frame3.label4
  label $topwin(analysis_name).info.frame25.frame20.frame3.label4 \
    -background "$GuiColor(label_bg)" \
    -padx {2} \
    -text {Line angles:}

  # pack widget $topwin(analysis_name).info.frame25.frame20.frame3
  pack append $topwin(analysis_name).info.frame25.frame20.frame3 \
    $topwin(analysis_name).info.frame25.frame20.frame3.label4 {left frame center} \
    $topwin(analysis_name).info.frame25.frame20.frame3.entry5 {top frame center expand fill}

  # pack widget $topwin(analysis_name).info.frame25.frame20
  pack append $topwin(analysis_name).info.frame25.frame20 \
    $topwin(analysis_name).info.frame25.frame20.frame3 {top frame center fillx} \
    $topwin(analysis_name).info.frame25.frame20.scrollbar7 {top frame center fillx}

  # build widget $topwin(analysis_name).info.frame25.label21
  label $topwin(analysis_name).info.frame25.label21 \
    -background "$GuiColor(label_bg)" \
    -text {} \
    -width {2}

  # pack widget $topwin(analysis_name).info.frame25
  pack append $topwin(analysis_name).info.frame25 \
    $topwin(analysis_name).info.frame25.radiobutton18 {left frame center} \
    $topwin(analysis_name).info.frame25.label21 {left frame center} \
    $topwin(analysis_name).info.frame25.frame20 {left frame s} \
    $topwin(analysis_name).info.frame25.frame0 {left frame center} \
    $topwin(analysis_name).info.frame25.frame19 {left frame center}

########################### EDGE LINK START ############################
 # build widget $topwin(analysis_name).info.frame26
  frame $topwin(analysis_name).info.frame26 \
    -background "$GuiColor(frame_bg)"  \
    -borderwidth {2} \
    -relief {ridge}

  # build widget $topwin(analysis_name).info.frame26.radiobutton18
	radiobutton $topwin(analysis_name).info.frame26.radiobutton18 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -relief {flat} \
    -text {Edge link} \
    -value {edge_link} \
    -variable {analysis(edge)}



  # build widget $topwin(analysis_name).info.frame26.frame19
  frame $topwin(analysis_name).info.frame26.frame19 \
    -background "$GuiColor(frame_bg)"  \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame26.frame19.label1
  label $topwin(analysis_name).info.frame26.frame19.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -text {Connect distance(max):}

  # build widget $topwin(analysis_name).info.frame26.frame19.value
  entry $topwin(analysis_name).info.frame26.frame19.value \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
    -textvariable {analysis(link_distance)} \
    -width {5}
  endEntry $topwin(analysis_name).info.frame26.frame19.value

  # build widget $topwin(analysis_name).info.frame26.frame19.menubutton2
	menubutton $topwin(analysis_name).info.frame26.frame19.menubutton2 -padx {1} -pady {1} -highlightthickness {0}  \
   -activebackground "$GuiColor(btn_act_bg)" \
   -background "$GuiColor(btn_bg)" \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)" \
    -text "$GuiMisc(pop_up_text)" -width {2}
  # bindings
  bind $topwin(analysis_name).info.frame26.frame19.menubutton2 <Button-1> {MenuPopupPost $topwin(analysis_name).info.frame26.frame19.menubutton2.menu %X %Y}
  bind $topwin(analysis_name).info.frame26.frame19.menubutton2 <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame26.frame19.menubutton2.menu %W}

  # build widget $topwin(analysis_name).info.frame26.frame19.menubutton2.menu
  menu $topwin(analysis_name).info.frame26.frame19.menubutton2.menu \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" 
  $topwin(analysis_name).info.frame26.frame19.menubutton2.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(link_distance) 1} \
    -label {  1 }
  $topwin(analysis_name).info.frame26.frame19.menubutton2.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(link_distance) 2} \
    -label {  2 }
  $topwin(analysis_name).info.frame26.frame19.menubutton2.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(link_distance) 5} \
    -label {  5 }
  $topwin(analysis_name).info.frame26.frame19.menubutton2.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(link_distance) 10} \
    -label { 10 }
  $topwin(analysis_name).info.frame26.frame19.menubutton2.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(link_distance) 15} \
    -label { 15 }
  $topwin(analysis_name).info.frame26.frame19.menubutton2.menu add command \
    -background "$GuiColor(btn_bg)" -activebackground "$GuiColor(btn_act_bg)" \
    -command {set analysis(link_distance) 20} \
    -label { 20 }
  bindtags $topwin(analysis_name).info.frame26.frame19.menubutton2.menu {Menu all}
  # bindings
  bind $topwin(analysis_name).info.frame26.frame19.menubutton2.menu <B1-Motion> {MenuPopupMotion $topwin(analysis_name).info.frame26.frame19.menubutton2.menu %W %X %Y}
  bind $topwin(analysis_name).info.frame26.frame19.menubutton2.menu <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame26.frame19.menubutton2.menu %W}

  # pack widget $topwin(analysis_name).info.frame26.frame19
  pack append $topwin(analysis_name).info.frame26.frame19 \
    $topwin(analysis_name).info.frame26.frame19.label1 {left frame center fill} \
    $topwin(analysis_name).info.frame26.frame19.value {left frame center expand fill} \
    $topwin(analysis_name).info.frame26.frame19.menubutton2 {right frame center fill}

  # build widget $topwin(analysis_name).info.frame26.frame20
  frame $topwin(analysis_name).info.frame26.frame20 \
    -background "$GuiColor(frame_bg)"  \
    -borderwidth {2}


  # pack widget $topwin(analysis_name).info.frame26
  pack append $topwin(analysis_name).info.frame26 \
    $topwin(analysis_name).info.frame26.radiobutton18 {left frame center} \
    $topwin(analysis_name).info.frame26.frame19 {left frame center} \
    $topwin(analysis_name).info.frame26.frame20 {left frame e padx 10}

########################### EDGE LINK  END  ############################

  # pack widget $topwin(analysis_name).info
  pack append $topwin(analysis_name).info \
    $topwin(analysis_name).info.frame11 {top frame center fill} \
    $topwin(analysis_name).info.frame12 {top frame center expand fill} \
    $topwin(analysis_name).info.frame13 {top frame center expand fill} \
    $topwin(analysis_name).info.frame14 {top frame center expand fill} \
    $topwin(analysis_name).info.frame25 {top frame center expand fill} \
    $topwin(analysis_name).info.frame26 {top frame center expand fill} 


  # build widget $topwin(analysis_name).current_image
  frame $topwin(analysis_name).current_image \
    -background "$GuiColor(frame_bg)"  \
	 -relief {ridge} \
    -borderwidth {2}

  # build widget $topwin(analysis_name).current_image.label15
  label $topwin(analysis_name).current_image.label15 \
    -background "$GuiColor(label_bg)" \
    -text {Current image: }

  # build widget $topwin(analysis_name).current_image.label16
  label $topwin(analysis_name).current_image.label16 \
    -background "$GuiColor(label_bg)" \
    -foreground {Blue} \
    -text {} \
    -textvariable {current_image}

  # pack widget $topwin(analysis_name).current_image
  pack append $topwin(analysis_name).current_image \
    $topwin(analysis_name).current_image.label15 {left frame center} \
    $topwin(analysis_name).current_image.label16 {left frame center}

  # build widget $topwin(analysis_name).perform
  frame $topwin(analysis_name).perform \
    -background "$GuiColor(frame_bg)"  \
    -borderwidth {2} \
	 -relief {raised}

  # build widget $topwin(analysis_name).perform.button17
	button $topwin(analysis_name).perform.button17 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(dis_act_bg)" \
    -background "$GuiColor(dis_bg)" \
    -foreground "$GuiColor(dis_fg)" \
	 -command {destroy $topwin(analysis_name)} \
	 -width {10} \
    -text {DISMISS}

  # build widget $topwin(analysis_name).perform.perform1
	button $topwin(analysis_name).perform.perform1 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(perf_act_bg)" \
    -background "$GuiColor(perf_bg)" \
    -foreground "$GuiColor(perf_fg)" \
    -command {
 	$topwin(analysis_name) configure -cursor watch; update
      if {$current_image == ""} {
         CvipAlert "Please select an image first."
	 $topwin(analysis_name) configure -cursor top_left_arrow; update
         return
      }

		if {$analysis(ed_thresh) == "none" } {set temp1 -1 } else {set temp1 $analysis(ed_thresh) }
		switch -exact -- $analysis(ed_smooth) {
			average {set temp 1}
			"center-weighted average" {set temp 2}
			gaussian1 {set temp 3}
			gaussian2 {set temp 4}
			default {set temp 0}
		}
		if [catch {expr $temp1}] {
			CvipAlert "Numerical values required for the threshold value: \n Post-threshold= $temp1"
			$topwin(analysis_name) configure -cursor top_left_arrow; update
			return
			} elseif {$temp1 < -5000 || $temp1 > 5000 } {
				CvipAlert "The threshold value (=$temp1) should be \n in the range (-5000 - 5000, or -1 for none)"
				$topwin(analysis_name) configure -cursor top_left_arrow; update
				return
			}

		switch -exact -- $analysis(edge) {
			kirsch -
			pyramid -
			robinson {
				set temp  $current_image
				if [catch {set result [cvip $analysis(edge) $current_image $temp $temp1]} cresult] {
					CvipAlert "$cresult"
					$topwin(analysis_name) configure -cursor top_left_arrow; update
					return
				} else {
					set new_name [cvip rename $result ${current_image}_$analysis(edge)]
					img_q insert end $new_name
					image_post $new_name
					cvip view $new_name
					CvipMessage "$analysis(edge) edge detection on image \"$temp\""
				}
				if { $GUI_DEBUG == "1" } { puts stderr "cvip $analysis(edge) $current_image $temp $temp1"}
			}
			sobel -
			prewitt {
				set temp  $current_image
				if [catch {expr $analysis(ed_kernel)}] {
					CvipAlert "Numerical values required for the kernel size: \n \
					Kernel size= $analysis(ed_kernel)"
					$topwin(analysis_name) configure -cursor top_left_arrow; update
					return
				} elseif {$analysis(ed_kernel) < "3.0" || $analysis(ed_kernel) > "15" } {
					CvipAlert "The kernel size(=$analysis(ed_kernel)) should be \n\
					in the range (3 - 15)"
					$topwin(analysis_name) configure -cursor top_left_arrow; update
					return
				}

				if [catch {set result [cvip $analysis(edge) $current_image $temp $analysis(ed_dc) $analysis(ed_kernel) $temp1]} cresult] {
					CvipAlert "$cresult"
					$topwin(analysis_name) configure -cursor top_left_arrow; update
					return
				} else {
                                        if { $analysis(ed_dc) == "1" && $analysis(edge) == "sobel"} {
                                                set result [cvip add $temp $result];
                                                puts "cvip add $current_image $result"
                                        }
					set new_name [cvip rename $result ${current_image}_$analysis(edge)]
					img_q insert end $new_name
					image_post $new_name
					cvip view $new_name
					CvipMessage "$analysis(edge) edge detection on image \"$temp\""
				}
				if { $GUI_DEBUG == "1" } { 
					puts stderr "cvip $analysis(edge) $current_image $temp $analysis(ed_dc) $analysis(ed_kernel) $temp1"
				}
			}

			roberts -
			laplacian {
				set temp  $current_image
				if { $GUI_DEBUG == "1" } { 
					puts "cvip $analysis(edge) $current_image $temp $analysis(ed_dc) $analysis(rob_lap) $temp1"
				}
				if [catch {set result [cvip $analysis(edge) $current_image $temp $analysis(ed_dc) $analysis(rob_lap) $temp1]} cresult] {
					CvipAlert "$cresult"
					$topwin(analysis_name) configure -cursor top_left_arrow; update
					return
				} else {
					set new_name [cvip rename $result ${current_image}_$analysis(edge)]
					img_q insert end $new_name
					image_post $new_name
					cvip view $new_name
					CvipMessage "$analysis(edge) edge detection on image \"$temp\""
				}

			}
			frei_chen {
				set temp2  $current_image
				set temp "cvip $analysis(edge) $current_image $temp $analysis(ed_dc) "
				switch -exact -- $analysis(ed_projection) {
					{edge subspace} {append temp "1 "
										  if {$analysis(ed_threshangle_yn) == "yes"} {
												append temp "1 $temp1 $analysis(ed_threshangle_value)"
										  } else {
												append temp "-1 $temp1 -1"
										  }
										 }
					{line subspace} {append temp "2 "
											if {$analysis(ed_threshangle_yn) == "yes"} {
												append temp "2 $temp1 $analysis(ed_threshangle_value)" 
											} else { 
												append temp "-1 $temp1 -1" 
											} 
										 }
					{max[edge, line]} {append temp "3 -1 $temp1 -1"}
					{min angle[edge,line]} {append temp "3 3 $temp1 -1"}
				}
				if [catch {set result [eval $temp]} cresult] {
					CvipAlert "$cresult"
					$topwin(analysis_name) configure -cursor top_left_arrow; update
					return
				} else {
					set new_name [cvip rename $result ${current_image}_$analysis(edge)]
					img_q insert end $new_name
					image_post $new_name
					cvip view $new_name
					CvipMessage "$analysis(edge) edge detection on image \"$temp2\""
				}
				if { $GUI_DEBUG == "1" } { 
					puts $temp
				}
			}


			edge_link {
				set temp2  $current_image
				if [catch {set result [cvip $analysis(edge) $current_image $analysis(link_distance)]} cresult] {
					CvipAlert "$cresult"
					$topwin(analysis_name) configure -cursor top_left_arrow; update
					return
				} else {
					set new_name [cvip rename $result ${current_image}_$analysis(edge)]
					img_q insert end $new_name
					image_post $new_name
					cvip view $new_name
					CvipMessage "edge linking on image \"$temp2\""
				}
				if { $GUI_DEBUG == "1" } { 
					puts stderr "cvip $analysis(edge) $current_image $temp $temp1"
				}
			}

			hough {
				set temp2  $current_image
				if {$bands != "1" } {
         		CvipAlert "Hough transform applies to binary image only."
 			$topwin(analysis_name) configure -cursor top_left_arrow; update
        		return
				}
				if {$analysis(ed_hough_angles) == "" } {
         		CvipAlert "Please specify the angles."
			$topwin(analysis_name) configure -cursor top_left_arrow; update
         		return
				}
				if [catch {expr $analysis(hough_line) + $analysis(hough_distance)}] {
         		CvipAlert "Numerical values required for line pixels (min) and connect distance (max)\n\
					line pixels (min) = $analysis(hough_line), connect distance (max) = $analysis(hough_distance)."
			$topwin(analysis_name) configure -cursor top_left_arrow; update
         		return
				} elseif {$analysis(hough_line) < "0" } {
         		CvipAlert "Line pixels (=$analysis(hough_line)) must be a positive value."
			$topwin(analysis_name) configure -cursor top_left_arrow; update
         		return
				} elseif {$analysis(hough_distance) < "0"  } {
         		CvipAlert "Connect distance  (=$analysis(hough_distance)) must be a positive value."
			$topwin(analysis_name) configure -cursor top_left_arrow; update
         		return
				}
				if [catch {set result [eval "cvip $analysis(edge) $current_image $analysis(ed_hough_angles) $analysis(hough_line) $analysis(hough_distance)"]} cresult] {
					CvipAlert "$cresult"
					$topwin(analysis_name) configure -cursor top_left_arrow; update
					return
				} else {
					set new_name [cvip rename $result ${current_image}_$analysis(edge)]
					img_q insert end $new_name
					image_post $new_name
					cvip view $new_name
					CvipMessage "hough transform on image \"$temp2\""
				}
				if { $GUI_DEBUG == "1" } { 
					puts stderr "cvip $analysis(edge) $current_image $analysis(ed_hough_angles) \
						$analysis(hough_line) $analysis(hough_distance)"
				}
			}
		}
			$topwin(analysis_name) configure -cursor top_left_arrow; update
	 } \
    -text {PERFORM}


  # build widget $topwin(analysis_name).perform.thresh
  frame $topwin(analysis_name).perform.thresh \
    -background "$GuiColor(frame_bg)"  \
   -relief {raised} \
   -borderwidth {2}


  # build widget $topwin(analysis_name).perform.thresh.thresh1
	button $topwin(analysis_name).perform.thresh.thresh1 -padx {1} -pady {1} -highlightthickness {0}  \
   -activebackground "$GuiColor(btn_act_bg)" \
   -background "$GuiColor(btn_bg)" \
   -relief {raised} \
   -width {28} \
   -command {
		if [catch {set result [ cvip threshold $current_image $analysis(bottom_thresh)] } cresult] {
         CvipAlert "$cresult ."
			return;
		} else {
			if { $GUI_DEBUG == "1" } { 
				puts "cvip threshold $current_image $analysis(bottom_thresh)"
			}
			set new_name [cvip rename $result ${current_image}_thresh]
			img_q insert end $new_name
			image_post $new_name
			cvip view $new_name
		}
	}\
   -text {Threshold Current image at: } 

  # build widget $topwin(analysis_name).perform.thresh.thresh2
  entry $topwin(analysis_name).perform.thresh.thresh2 \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
    -width {7} \
    -textvariable {analysis(bottom_thresh)} 
  endEntry $topwin(analysis_name).perform.thresh.thresh2

  # pack widget $topwin(analysis_name).perform.thresh
  pack append $topwin(analysis_name).perform.thresh \
    $topwin(analysis_name).perform.thresh.thresh1 {left frame center} \
    $topwin(analysis_name).perform.thresh.thresh2 {left frame center}
 
  # pack widget $topwin(analysis_name).perform
  pack append $topwin(analysis_name).perform \
    $topwin(analysis_name).perform.button17 {left frame center} \
    $topwin(analysis_name).perform.thresh {left frame center expand} \
    $topwin(analysis_name).perform.perform1 {right frame se}

  # pack widget $topwin(analysis_name).
  pack append $topwin(analysis_name) \
    $topwin(analysis_name).info {top frame center expand fill} \
    $topwin(analysis_name).current_image {top frame center fillx} \
    $topwin(analysis_name).perform {bottom frame center fillx}

  if {"[info procs XFEdit]" != ""} {
    catch "XFMiscBindWidgetTree $topwin(analysis_name)."
    after 2 "catch {XFEditSetShowWindows}"
  }
}

