#***************************************************************************
# **********************************************************************
# Computer Vision/Image Processing Tool Project - Dr. Scott Umbaugh SIUE
# **********************************************************************
#
#             File Name: ana_feature.tcl
#           Description: Feature module for graphical user interface
#         Related Files:
#   Initial Coding Date: Mon Jul  8 16:27:28 CDT 1996
#           Portability: TCL script file
#            References:
#             Author(s): Yansheng Wei, Mark Zuke
#                        Southern Illinois University @ Edwardsville
#
## Copyright (C) 1996 SIUE - by Scott Umbaugh, Yansheng Wei, and Mark Zuke.
##
## Permission to use, copy, modify, and distribute this software and its
## documentation for any purpose and without fee is hereby granted, provided
## that the above copyright notice appear in all copies and that both that
## copyright notice and this permission notice appear in supporting
## documentation.  This software is provided "as is" without express or
## implied warranty.
##
# RCS (Revision Control System) Information
# ...Added automatically by RCS
#
## $Log: ana_feature.tcl,v $
## Revision 1.33  1997/07/28 22:21:31  wzheng
## avoid memory leak
##
## Revision 1.32  1997/07/28 19:05:23  wzheng
## error message return on labal overflow
##
## Revision 1.31  1997/06/24 20:01:38  yawei
## finished configuration
##
## Revision 1.30  1997/06/21 15:35:22  yawei
## check in for freezing
##
## Revision 1.29  1997/06/16 21:50:37  yawei
## m_anal ==> m_analysis
##
## Revision 1.28  1997/03/26 20:46:18  mzuke
## corrected spectral header; added luminance
## transform for multiband texture features
##
## Revision 1.27  1997/03/25 16:11:36  mzuke
## reformatted screen; moved features around; added gray line (frame)
## between frames 2 and 3; changes relief of checkbuttons to raised;
## changed order of arrays to agree with features on screen
##
## Revision 1.26  1997/03/20 18:31:23  mzuke
## changed texture_distance to 6
##
## Revision 1.25  1997/03/20 17:59:19  mzuke
## fixed spectral_feature bug
##
## Revision 1.24  1997/03/12 00:09:00  yawei
## changed size
##
## Revision 1.23  1997/03/11 20:55:18  yawei
## Tcl/Tk script upgrading from tcl7.3/tk3.6 to tcl7.6/tk4.2
## compatible. This process is automated with tkboost perl
## script. @ Tue Mar 11 14:51:36 CST 1997
##
## Revision 1.22  1996/10/13 19:28:28  mzuke
## added watch cursor
##
# Revision 1.21  1996/09/27  18:28:34  mzuke
# ana_feature.tcl
#  made window resizable
#
# Revision 1.20  1996/09/17  13:10:38  mzuke
# fixed bug that prevented a new segmented image from
# being available to the feature program
#
# Revision 1.19  1996/09/03  18:05:23  yawei
# changed anal_ to analysis_
#
# Revision 1.18  1996/08/15  11:48:53  mzuke
# added case statement for Projection to header checking in PERFORM
# to correct bug
#
# Revision 1.17  1996/08/15  10:44:58  mzuke
# removed toplevel_position from wm geometry to allow window
# positioning by user
#
# Revision 1.16  1996/08/13  12:53:43  mzuke
# added alert to inform user if coordinate selection is made
# from other than original or segmented image in Features
#
# Revision 1.15  1996/08/07  15:21:29  mzuke
# changed projection to accept normalizing height and width
#
# Revision 1.14  1996/07/23  18:44:49  mzuke
# added auto handling of RGB images for hist_feature selections
#
# Revision 1.13  1996/07/22  21:12:25  mzuke
# added rst_invariant and texture feature code to menu
#
# Revision 1.12  1996/07/17  22:25:50  yawei
# modified: so you can still view the labeled image, although it will deleted
# in the end.
#
# Revision 1.11  1996/07/17  21:08:16  yawei
# changed so it will delete the labeled image in the feature window
#
# Revision 1.10  1996/07/17  17:31:21  yawei
# no big change, just added the lost header in revisions 1.6-1.8
#
# Revision 1.5  1996/07/10  00:07:38  mzuke
# changed 'alsm' to 'orientation' in feature_tcl_name list
#
# Revision 1.4  1996/07/09  18:19:06  mzuke
# histogram and spectral features working
# for single band images; header files are correct
#
# Revision 1.3  1996/07/09  00:50:35  mzuke
# added switch statement for histogram features; they work
# well and print properly in header and in data fields
#
# Revision 1.2  1996/07/08  23:57:53  mzuke
# added check of feature file's first four lines
#
# Revision 1.1  1996/07/08  22:08:19  mzuke
# Initial revision
#
#
#****************************************************************************

# Module: ana_feature.tcl
# Tcl version: 7.3 (Tcl/Tk/XF)
# Tk version: 3.6
# XF version: 2.3
#

# procedure to show window $topwin(analysis_name)
proc get_features {} {# xf ignore me 7

  global topwin
  global toplevel_position
  global labeled_image
  global {GuiColor}
  global {GuiMisc}
  global {GUI_DEBUG}

  global temp_label
  set temp_label ""
  # Window manager configurations
  if [ winfo exists $topwin(analysis_name).info] {
	  set tempname $topwin(analysis_name);\
destroy $topwin(analysis_name).info;\
set topwin(analysis_name) $tempname;}
  if [ winfo exists $topwin(analysis_name).current_image] {
	  set tempname $topwin(analysis_name);\
destroy $topwin(analysis_name).current_image;\
set topwin(analysis_name) $tempname;}
  if [ winfo exists $topwin(analysis_name).perform] {
	  set tempname $topwin(analysis_name);\
destroy $topwin(analysis_name).perform;\
set topwin(analysis_name) $tempname;}
  global tk_version
  wm positionfrom $topwin(analysis_name) user
  wm sizefrom $topwin(analysis_name) user

  wm geometry $topwin(analysis_name) $topwin(analysis_feature)[get_pos $topwin(analysis_name)]
  wm maxsize $topwin(analysis_name) [lindex $topwin(analysis_feature_max) 0] \
  	[lindex $topwin(analysis_feature_max) 1]
  wm minsize $topwin(analysis_name) [lindex $topwin(analysis_feature_min) 0] \
  	[lindex $topwin(analysis_feature_min) 1]
  wm title $topwin(analysis_name) {ANALYSIS -> Features}


  # build widget $topwin(analysis_name).current_image
  frame $topwin(analysis_name).current_image \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {1}

  # build widget $topwin(analysis_name).current_image.frame17
  frame $topwin(analysis_name).current_image.frame17 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {1}

  # build widget $topwin(analysis_name).current_image.frame17.frame
  frame $topwin(analysis_name).current_image.frame17.frame \
    -background "$GuiColor(frame_bg)"  \
    -borderwidth {1}

  # build widget $topwin(analysis_name).current_image.frame17.frame.scrollbar7
  scrollbar $topwin(analysis_name).current_image.frame17.frame.scrollbar7 \
    -activebackground {gray50} \
    -troughcolor {gray90} \
    -command "$topwin(analysis_name).current_image.frame17.frame.frame3.entry5 xview" \
    -background {gray70} \
    -orient {horizontal} \
    -width {11}

  # build widget $topwin(analysis_name).current_image.frame17.frame.frame3
  frame $topwin(analysis_name).current_image.frame17.frame.frame3 \
    -background "$GuiColor(frame_bg)"  \
    -borderwidth {1}

  # build widget $topwin(analysis_name).current_image.frame17.frame.frame3.entry5
  entry $topwin(analysis_name).current_image.frame17.frame.frame3.entry5 \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
 -xscrollcommand "$topwin(analysis_name).current_image.frame17.frame.scrollbar7 set" \
    -textvariable {current_image}

  endEntry $topwin(analysis_name).current_image.frame17.frame.frame3.entry5 0 \
  	$topwin(analysis_name).current_image.frame18.frame.frame3.entry5 \
  	$topwin(analysis_name).current_image.frame19.frame.frame3.entry5 

  # build widget $topwin(analysis_name).current_image.frame17.frame.frame3.label4
  label $topwin(analysis_name).current_image.frame17.frame.frame3.label4 \
    -background "$GuiColor(label_bg)" \
    -padx {2} \
	 -anchor {w} \
    -text {Original image:} \
    -width {18}

  # pack widget $topwin(analysis_name).current_image.frame17.frame.frame3
  pack append $topwin(analysis_name).current_image.frame17.frame.frame3 \
    $topwin(analysis_name).current_image.frame17.frame.frame3.label4 {left frame center} \
    $topwin(analysis_name).current_image.frame17.frame.frame3.entry5 {top frame center expand fill}

  # pack widget $topwin(analysis_name).current_image.frame17.frame
  pack append $topwin(analysis_name).current_image.frame17.frame \
    $topwin(analysis_name).current_image.frame17.frame.frame3 {top frame center fillx} \
    $topwin(analysis_name).current_image.frame17.frame.scrollbar7 {top frame center fillx}

  # pack widget $topwin(analysis_name).current_image.frame17
  pack append $topwin(analysis_name).current_image.frame17 \
    $topwin(analysis_name).current_image.frame17.frame {top frame center fill}

  # build widget $topwin(analysis_name).current_image.frame18
  frame $topwin(analysis_name).current_image.frame18 \
    -background "$GuiColor(frame_bg)"  \
    -borderwidth {1}

  # build widget $topwin(analysis_name).current_image.frame18.frame
  frame $topwin(analysis_name).current_image.frame18.frame \
    -background "$GuiColor(frame_bg)"   \
    -borderwidth {1}

  # build widget $topwin(analysis_name).current_image.frame18.frame.scrollbar7
  scrollbar $topwin(analysis_name).current_image.frame18.frame.scrollbar7 \
    -activebackground {gray50} \
    -troughcolor {gray90} \
    -command "$topwin(analysis_name).current_image.frame18.frame.frame3.entry5 xview" \
    -background {gray70} \
    -orient {horizontal} \
    -width {11}

  # build widget $topwin(analysis_name).current_image.frame18.frame.frame3
  frame $topwin(analysis_name).current_image.frame18.frame.frame3 \
    -background "$GuiColor(frame_bg)"  \
    -borderwidth {1}

  # build widget $topwin(analysis_name).current_image.frame18.frame.frame3.entry5
  entry $topwin(analysis_name).current_image.frame18.frame.frame3.entry5 \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
 -xscrollcommand "$topwin(analysis_name).current_image.frame18.frame.scrollbar7 set" \
    -textvariable {second_image}
  endEntry $topwin(analysis_name).current_image.frame18.frame.frame3.entry5 0 \
  	$topwin(analysis_name).current_image.frame19.frame.frame3.entry5 \
  	$topwin(analysis_name).current_image.frame17.frame.frame3.entry5 

  # build widget $topwin(analysis_name).current_image.frame18.frame.frame3.label4
  label $topwin(analysis_name).current_image.frame18.frame.frame3.label4 \
    -background "$GuiColor(label_bg)" \
    -padx {2} \
	 -anchor {w} \
    -text {Segmented image:} \
    -width {18}

  # pack widget $topwin(analysis_name).current_image.frame18.frame.frame3
  pack append $topwin(analysis_name).current_image.frame18.frame.frame3 \
    $topwin(analysis_name).current_image.frame18.frame.frame3.label4 {left frame center} \
    $topwin(analysis_name).current_image.frame18.frame.frame3.entry5 {top frame center expand fill}

  # pack widget $topwin(analysis_name).current_image.frame18.frame
  pack append $topwin(analysis_name).current_image.frame18.frame \
    $topwin(analysis_name).current_image.frame18.frame.frame3 {top frame center fillx} \
    $topwin(analysis_name).current_image.frame18.frame.scrollbar7 {top frame center fillx}

  # pack widget $topwin(analysis_name).current_image.frame18
  pack append $topwin(analysis_name).current_image.frame18 \
    $topwin(analysis_name).current_image.frame18.frame {top frame center fill}

  # build widget $topwin(analysis_name).current_image.frame19
  frame $topwin(analysis_name).current_image.frame19 \
    -background "$GuiColor(frame_bg)"   \
    -borderwidth {1}

  # build widget $topwin(analysis_name).current_image.frame19.frame
  frame $topwin(analysis_name).current_image.frame19.frame \
    -background "$GuiColor(frame_bg)"    \
    -borderwidth {1}

  # build widget $topwin(analysis_name).current_image.frame19.frame.scrollbar7
  scrollbar $topwin(analysis_name).current_image.frame19.frame.scrollbar7 \
    -activebackground {gray50} \
    -troughcolor {gray90} \
    -command "$topwin(analysis_name).current_image.frame19.frame.frame3.entry5 xview" \
    -background {gray70} \
    -orient {horizontal} \
    -width {11}

  # build widget $topwin(analysis_name).current_image.frame19.frame.frame3
  frame $topwin(analysis_name).current_image.frame19.frame.frame3 \
    -background "$GuiColor(frame_bg)"  \
    -borderwidth {1}

  # build widget $topwin(analysis_name).current_image.frame19.frame.frame3.entry5
  entry $topwin(analysis_name).current_image.frame19.frame.frame3.entry5 \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
 -xscrollcommand "$topwin(analysis_name).current_image.frame19.frame.scrollbar7 set" \
    -textvariable {analysis(feature_fn)} \
    -width {25}
  endEntry $topwin(analysis_name).current_image.frame19.frame.frame3.entry5 0 \
 	$topwin(analysis_name).current_image.frame17.frame.frame3.entry5 \
  	$topwin(analysis_name).current_image.frame18.frame.frame3.entry5 

  # build widget $topwin(analysis_name).current_image.frame19.frame.frame3.label4
  label $topwin(analysis_name).current_image.frame19.frame.frame3.label4 \
    -background "$GuiColor(label_bg)" \
    -foreground {black} \
    -padx {2} \
	 -anchor {w} \
    -text {Feature file name:} \
    -width {18}

  # pack widget $topwin(analysis_name).current_image.frame19.frame.frame3
  pack append $topwin(analysis_name).current_image.frame19.frame.frame3 \
    $topwin(analysis_name).current_image.frame19.frame.frame3.label4 {left frame center} \
    $topwin(analysis_name).current_image.frame19.frame.frame3.entry5 {top frame center expand fill}

  # pack widget $topwin(analysis_name).current_image.frame19.frame
  pack append $topwin(analysis_name).current_image.frame19.frame \
    $topwin(analysis_name).current_image.frame19.frame.frame3 {top frame center fillx} \
    $topwin(analysis_name).current_image.frame19.frame.scrollbar7 {top frame center fillx}

  # build widget $topwin(analysis_name).current_image.frame19.label0
  label $topwin(analysis_name).current_image.frame19.label0 \
    -background "$GuiColor(label_bg)" \
    -foreground {blue} \
    -text {The coordinates of the current object are :} \
    -width {50}

  # build widget $topwin(analysis_name).current_image.frame19.label1
  label $topwin(analysis_name).current_image.frame19.label1 \
    -background "$GuiColor(label_bg)" \
    -foreground {blue} \
    -text {     row}

  # build widget $topwin(analysis_name).current_image.frame19.label3
  label $topwin(analysis_name).current_image.frame19.label3 \
    -background "$GuiColor(label_bg)" \
    -foreground {blue} \
    -text {, column}

  # build widget $topwin(analysis_name).current_image.frame19.entry1
  entry $topwin(analysis_name).current_image.frame19.entry1 \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
    -textvariable {analysis(obj_row)} \
    -width {3}
  endEntry $topwin(analysis_name).current_image.frame19.entry1 0\
  	 $topwin(analysis_name).current_image.frame19.entry2 \
  	 $topwin(analysis_name).current_image.frame19.entry2 

  # build widget $topwin(analysis_name).current_image.frame19.entry2
  entry $topwin(analysis_name).current_image.frame19.entry2 \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
    -textvariable {analysis(obj_col)} \
    -width {3}
  endEntry $topwin(analysis_name).current_image.frame19.entry2 0 \
  	 $topwin(analysis_name).current_image.frame19.entry1 \
  	 $topwin(analysis_name).current_image.frame19.entry1 

  # build widget $topwin(analysis_name).current_image.frame19.frame4
  frame $topwin(analysis_name).current_image.frame19.frame4 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {1}

  # build widget $topwin(analysis_name).current_image.frame19.frame4.button5
	button $topwin(analysis_name).current_image.frame19.frame4.button5 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -command {set textBox(background) white
set textBox(scrollForeground) gray80
catch {TextBoxFile "$analysis(feature_fn)" "" "500x600" "Feature File: $analysis(feature_fn)"} cresult} \
    -text {View Feature File}

  # build widget $topwin(analysis_name).current_image.frame19.frame4.button6
	button $topwin(analysis_name).current_image.frame19.frame4.button6 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -command {
	 	if [ catch {cvip view $labeled_image} cresult] {
			CvipMessage "$cresult" 
		}
		} \
    -text {View Labeled Image}

  # pack widget $topwin(analysis_name).current_image.frame19.frame4
  pack append $topwin(analysis_name).current_image.frame19.frame4 \
    $topwin(analysis_name).current_image.frame19.frame4.button5 {left frame center expand} \
    $topwin(analysis_name).current_image.frame19.frame4.button6 {left frame center expand}

  # pack widget $topwin(analysis_name).current_image.frame19
  pack append $topwin(analysis_name).current_image.frame19 \
    $topwin(analysis_name).current_image.frame19.frame {left frame center expand fill} \
    $topwin(analysis_name).current_image.frame19.frame4 {top frame center expand fillx} \
    $topwin(analysis_name).current_image.frame19.label0 {top frame center filly} \
    $topwin(analysis_name).current_image.frame19.label1 {left frame center} \
    $topwin(analysis_name).current_image.frame19.entry1 {left frame center fill} \
    $topwin(analysis_name).current_image.frame19.label3 {left frame center} \
    $topwin(analysis_name).current_image.frame19.entry2 {left frame center fill}

  # build widget $topwin(analysis_name).current_image.frame
  frame $topwin(analysis_name).current_image.frame \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {1}

  # build widget $topwin(analysis_name).current_image.frame.scrollbar7
  scrollbar $topwin(analysis_name).current_image.frame.scrollbar7 \
    -activebackground {gray50} \
    -troughcolor {gray90} \
    -command "$topwin(analysis_name).current_image.frame.frame3.entry5 xview" \
    -background {gray70} \
    -orient {horizontal} \
    -width {11}

  # build widget $topwin(analysis_name).current_image.frame.frame3
  frame $topwin(analysis_name).current_image.frame.frame3 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {1}

  # build widget $topwin(analysis_name).current_image.frame.frame3.entry5
  entry $topwin(analysis_name).current_image.frame.frame3.entry5 \
    -background {#ddddff} \
    -relief {sunken} \
 -xscrollcommand "$topwin(analysis_name).current_image.frame.scrollbar7 set" \
    -textvariable {analysis(feature_class)} \
    -width {25}
  endEntry $topwin(analysis_name).current_image.frame.frame3.entry5 

  # build widget $topwin(analysis_name).current_image.frame.frame3.label4
  label $topwin(analysis_name).current_image.frame.frame3.label4 \
    -background "$GuiColor(label_bg)" \
    -padx {2} \
	 -anchor {w} \
    -text {Class (optional):} \
    -width {18}

  # pack widget $topwin(analysis_name).current_image.frame.frame3
  pack append $topwin(analysis_name).current_image.frame.frame3 \
    $topwin(analysis_name).current_image.frame.frame3.label4 {left frame center} \
    $topwin(analysis_name).current_image.frame.frame3.entry5 {top frame center expand fill}

  # pack widget $topwin(analysis_name).current_image.frame
  pack append $topwin(analysis_name).current_image.frame \
    $topwin(analysis_name).current_image.frame.frame3 {top frame center fillx} \
    $topwin(analysis_name).current_image.frame.scrollbar7 {top frame center fillx}

  # pack widget $topwin(analysis_name).current_image
  pack append $topwin(analysis_name).current_image \
    $topwin(analysis_name).current_image.frame17 {top frame center fillx} \
    $topwin(analysis_name).current_image.frame18 {top frame center fillx} \
    $topwin(analysis_name).current_image.frame19 {top frame center fillx} \
    $topwin(analysis_name).current_image.frame {top frame w}

  # build widget $topwin(analysis_name).info
  frame $topwin(analysis_name).info \
    -background "$GuiColor(frame_bg)"  \
    -borderwidth {1}
  bind $topwin(analysis_name).info <Destroy> {
	 if {$labeled_image != ""} {catch {cvip dq $labeled_image}; set labeled_image ""} }

  # build widget $topwin(analysis_name).info.frame15
  frame $topwin(analysis_name).info.frame15 \
    -background "$GuiColor(frame_bg)"  \
    -borderwidth {2} \
    -relief {ridge}

 # build widget $topwin(analysis_name).info.frame15.frame0
  frame $topwin(analysis_name).info.frame15.frame0 \
    -background "$GuiColor(frame_bg)"  \
    -borderwidth {1} \
    -relief {flat}

  # build widget $topwin(analysis_name).info.frame15.frame0.label20
  label $topwin(analysis_name).info.frame15.frame0.label20 \
    -background "$GuiColor(label_bg)" \
    -text {BINARY OBJECT FEATURES}

  # build widget $topwin(analysis_name).info.frame15.upper
  frame $topwin(analysis_name).info.frame15.upper \
    -background "$GuiColor(frame_bg)"  \
    -borderwidth {1}

  # build widget $topwin(analysis_name).info.frame15.upper.checkbutton22
	checkbutton $topwin(analysis_name).info.frame15.upper.checkbutton22 -padx {1} -pady {2} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -text {Area} \
	 -anchor {w} \
    -variable {area} \
    -relief {raised} \
    -width {15}


  # build widget $topwin(analysis_name).info.frame15.upper.checkbutton23
	checkbutton $topwin(analysis_name).info.frame15.upper.checkbutton23 -padx {1} -pady {2} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -text {Centroid} \
	 -anchor {w} \
    -variable {centroid} \
    -relief {raised} \
    -width {15}

  # build widget $topwin(analysis_name).info.frame15.upper.checkbutton24
	checkbutton $topwin(analysis_name).info.frame15.upper.checkbutton24 -padx {1} -pady {2} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -text {Orientation} \
	 -anchor {w} \
    -variable {orientation} \
    -relief {raised} \
    -width {15}

  # build widget $topwin(analysis_name).info.frame15.upper.checkbutton25
	checkbutton $topwin(analysis_name).info.frame15.upper.checkbutton25 -padx {1} -pady {2} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -text {Perimeter} \
	 -anchor {w} \
    -variable {perimeter} \
    -relief {raised} \
    -width {15}

  # pack widget $topwin(analysis_name).info.frame15.upper
  pack append $topwin(analysis_name).info.frame15.upper \
    $topwin(analysis_name).info.frame15.upper.checkbutton22 {left frame center expand} \
    $topwin(analysis_name).info.frame15.upper.checkbutton23 {left frame center expand} \
    $topwin(analysis_name).info.frame15.upper.checkbutton24 {left frame center expand} \
    $topwin(analysis_name).info.frame15.upper.checkbutton25 {left frame center expand}

  # build widget $topwin(analysis_name).info.frame15.lower
  frame $topwin(analysis_name).info.frame15.lower \
    -background "$GuiColor(frame_bg)"  \
    -borderwidth {1}

  # build widget $topwin(analysis_name).info.frame15.lower.checkbutton26
	checkbutton $topwin(analysis_name).info.frame15.lower.checkbutton26 -padx {1} -pady {2} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -text {Euler number} \
	 -anchor {w} \
    -variable {euler} \
    -relief {raised} \
    -width {15}

  # build widget $topwin(analysis_name).info.frame15.lower.checkbutton27
	checkbutton $topwin(analysis_name).info.frame15.lower.checkbutton27 -padx {1} -pady {2} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -text {Projections} \
	 -anchor {w} \
    -variable {projection} \
    -command {if {$projection} {pack $topwin(analysis_name).info.frame15.frame2.frame0}
              if {!$projection} {pack forget $topwin(analysis_name).info.frame15.frame2.frame0}} \
    -relief {raised} \
    -width {15}

  # build widget $topwin(analysis_name).info.frame15.lower.checkbutton28
	checkbutton $topwin(analysis_name).info.frame15.lower.checkbutton28 -padx {1} -pady {2} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -text {Thinness} \
	 -anchor {w} \
    -variable {thinness} \
    -relief {raised} \
    -width {15}

  # build widget $topwin(analysis_name).info.frame15.lower.checkbutton29
	checkbutton $topwin(analysis_name).info.frame15.lower.checkbutton29 -padx {1} -pady {2} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -text {Aspect  ratio} \
	 -anchor {w} \
    -variable {aspect} \
    -relief {raised} \
    -width {15}

  # pack widget $topwin(analysis_name).info.frame15.lower
  pack append $topwin(analysis_name).info.frame15.lower \
    $topwin(analysis_name).info.frame15.lower.checkbutton26 {left frame center expand} \
    $topwin(analysis_name).info.frame15.lower.checkbutton27 {left frame center expand} \
    $topwin(analysis_name).info.frame15.lower.checkbutton28 {left frame center expand} \
    $topwin(analysis_name).info.frame15.lower.checkbutton29 {left frame center expand}

  # build widget $topwin(analysis_name).info.frame15.frame0.button0
	button $topwin(analysis_name).info.frame15.frame0.button0 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -command {set temp $analysis(binary_all_button)
if {$temp == "Select All"} {foreach i [ list area cent orie peri eule proj thin aspe ] {binary_$i select;set analysis(binary_all_button) "Deselect All"; pack $topwin(analysis_name).info.frame15.frame2.frame0 }}
if {$temp == "Deselect All"} {foreach i [ list area cent orie peri eule proj thin aspe ] {binary_$i deselect; set analysis(binary_all_button) "Select All"; pack forget $topwin(analysis_name).info.frame15.frame2.frame0}}
} \
    -text {Select All} \
    -textvariable {analysis(binary_all_button)} \
    -width {15}

  # build widget $topwin(analysis_name).info.frame15.frame2
  frame $topwin(analysis_name).info.frame15.frame2 \
    -background "$GuiColor(frame_bg)"  \
    -borderwidth {1}

  # build widget $topwin(analysis_name).info.frame15.frame2.frame0
  frame $topwin(analysis_name).info.frame15.frame2.frame0 \
    -background "$GuiColor(frame_bg)"  \
    -borderwidth {1}

  # build widget $topwin(analysis_name).info.frame15.frame2.frame0.frame
  frame $topwin(analysis_name).info.frame15.frame2.frame0.frame \
    -background "$GuiColor(frame_bg)"   \
    -borderwidth {1} 

  # build widget $topwin(analysis_name).info.frame15.frame2.frame0.frame.label4
  label $topwin(analysis_name).info.frame15.frame2.frame0.frame.label4 \
    -background "$GuiColor(label_bg)" \
    -text {For projections, enter the normalizing height:}

  # build widget $topwin(analysis_name).info.frame15.frame2.frame0.frame.entry5
  entry $topwin(analysis_name).info.frame15.frame2.frame0.frame.entry5 \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
    -textvariable {analysis(proj_height)} \
    -width {7}
  endEntry $topwin(analysis_name).info.frame15.frame2.frame0.frame.entry5 0 \
  	 $topwin(analysis_name).info.frame15.frame2.frame0.frame1.entry5 \
  	 $topwin(analysis_name).info.frame15.frame2.frame0.frame1.entry5 

  # pack widget $topwin(analysis_name).info.frame15.frame2.frame0.frame
  pack append $topwin(analysis_name).info.frame15.frame2.frame0.frame \
    $topwin(analysis_name).info.frame15.frame2.frame0.frame.label4 {left frame center} \
    $topwin(analysis_name).info.frame15.frame2.frame0.frame.entry5 {top frame center expand fill}

  # build widget $topwin(analysis_name).info.frame15.frame2.frame0.frame1
  frame $topwin(analysis_name).info.frame15.frame2.frame0.frame1 \
    -background "$GuiColor(frame_bg)"  \
    -borderwidth {1} 

  # build widget $topwin(analysis_name).info.frame15.frame2.frame0.frame1.label4
  label $topwin(analysis_name).info.frame15.frame2.frame0.frame1.label4 \
    -background "$GuiColor(label_bg)" \
    -text {and width:}

  # build widget $topwin(analysis_name).info.frame15.frame2.frame0.frame1.entry5
  entry $topwin(analysis_name).info.frame15.frame2.frame0.frame1.entry5 \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
    -textvariable {analysis(proj_width)} \
    -width {7}
  endEntry $topwin(analysis_name).info.frame15.frame2.frame0.frame1.entry5 0\
  	 $topwin(analysis_name).info.frame15.frame2.frame0.frame.entry5 \
  	 $topwin(analysis_name).info.frame15.frame2.frame0.frame.entry5 

  # pack widget $topwin(analysis_name).info.frame15.frame2.frame0.frame1
  pack append $topwin(analysis_name).info.frame15.frame2.frame0.frame1 \
    $topwin(analysis_name).info.frame15.frame2.frame0.frame1.label4 {left frame center} \
    $topwin(analysis_name).info.frame15.frame2.frame0.frame1.entry5 {top frame center expand fill}

  # pack widget $topwin(analysis_name).info.frame15.frame2.frame0
  pack append $topwin(analysis_name).info.frame15.frame2.frame0 \
    $topwin(analysis_name).info.frame15.frame2.frame0.frame {left frame center} \
    $topwin(analysis_name).info.frame15.frame2.frame0.frame1 {left frame center}

  # pack widget $topwin(analysis_name).info.frame15.frame2
  pack append $topwin(analysis_name).info.frame15.frame2 \
    $topwin(analysis_name).info.frame15.frame2.frame0 {top frame center}


   # pack widget $topwin(analysis_name).info.frame15.frame0
  pack append $topwin(analysis_name).info.frame15.frame0 \
    $topwin(analysis_name).info.frame15.frame0.label20 {left frame center} \
    $topwin(analysis_name).info.frame15.frame0.button0 {left frame center pady 10} \


   # pack widget $topwin(analysis_name).info.frame15
  pack append $topwin(analysis_name).info.frame15 \
    $topwin(analysis_name).info.frame15.frame0 {top frame center} \
    $topwin(analysis_name).info.frame15.upper {top frame center fillx} \
    $topwin(analysis_name).info.frame15.lower {top frame center fillx} \
    $topwin(analysis_name).info.frame15.frame2 {top frame center fillx}

  # build widget $topwin(analysis_name).info.frame16
  frame $topwin(analysis_name).info.frame16 \
    -background "$GuiColor(frame_bg)"  \
    -borderwidth {2} \
    -relief {ridge}

  # build widget $topwin(analysis_name).info.frame16.frame0
  frame $topwin(analysis_name).info.frame16.frame0 \
    -background "$GuiColor(frame_bg)"  \
    -borderwidth {1} \
    -relief {flat}

  # build widget $topwin(analysis_name).info.frame16.frame0.label21
  label $topwin(analysis_name).info.frame16.frame0.label21 \
    -background "$GuiColor(label_bg)" \
    -text {HISTOGRAM FEATURES}

  # build widget $topwin(analysis_name).info.frame16.upper
  frame $topwin(analysis_name).info.frame16.upper \
    -background "$GuiColor(frame_bg)"  \
    -borderwidth {1}

  # build widget $topwin(analysis_name).info.frame16.upper.checkbutton22
	checkbutton $topwin(analysis_name).info.frame16.upper.checkbutton22 -padx {1} -pady {2} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -text {Mean} \
	 -anchor {w} \
    -variable {mean} \
    -relief {raised} \
    -width {12}

  # build widget $topwin(analysis_name).info.frame16.upper.checkbutton23
	checkbutton $topwin(analysis_name).info.frame16.upper.checkbutton23 -padx {1} -pady {2} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -text {Std Dev} \
	 -anchor {w} \
    -variable {stddev} \
    -relief {raised} \
    -width {12}

  # build widget $topwin(analysis_name).info.frame16.upper.checkbutton24
	checkbutton $topwin(analysis_name).info.frame16.upper.checkbutton24 -padx {1} -pady {2} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -text {Skew} \
	 -anchor {w} \
    -variable {skew} \
    -relief {raised} \
    -width {12}

  # build widget $topwin(analysis_name).info.frame16.upper.checkbutton25
	checkbutton $topwin(analysis_name).info.frame16.upper.checkbutton25 -padx {1} -pady {2} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -text {Energy} \
	 -anchor {w} \
    -variable {histo_energy} \
    -relief {raised} \
    -width {12}

  # build widget $topwin(analysis_name).info.frame16.upper.checkbutton5
	checkbutton $topwin(analysis_name).info.frame16.upper.checkbutton5 -padx {1} -pady {2} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -text {Entropy} \
	 -anchor {w} \
    -variable {histo_entropy} \
    -relief {raised} \
    -width {12}

  # pack widget $topwin(analysis_name).info.frame16.upper
  pack append $topwin(analysis_name).info.frame16.upper \
    $topwin(analysis_name).info.frame16.upper.checkbutton22 {left frame center expand} \
    $topwin(analysis_name).info.frame16.upper.checkbutton23 {left frame center expand} \
    $topwin(analysis_name).info.frame16.upper.checkbutton24 {left frame center expand} \
    $topwin(analysis_name).info.frame16.upper.checkbutton25 {left frame center expand} \
    $topwin(analysis_name).info.frame16.upper.checkbutton5 {left frame center expand}

  # build widget $topwin(analysis_name).info.frame16.frame0.button0
	button $topwin(analysis_name).info.frame16.frame0.button0 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -command {set temp $analysis(his_all_button)
if {$temp == "Select All"} {foreach i [ list mean dev skew energy entropy ] {his_$i select;set analysis(his_all_button) "Deselect All"}}
if {$temp == "Deselect All"} {foreach i [ list mean dev skew energy entropy ] {his_$i deselect; set analysis(his_all_button) "Select All"}}
} \
    -text {Select All} \
    -textvariable {analysis(his_all_button)} \
    -width {15}

  # pack widget $topwin(analysis_name).info.frame16.frame0
  pack append $topwin(analysis_name).info.frame16.frame0 \
    $topwin(analysis_name).info.frame16.frame0.label21 {left frame center} \
    $topwin(analysis_name).info.frame16.frame0.button0 {left frame center pady 10} \


  # pack widget $topwin(analysis_name).info.frame16
  pack append $topwin(analysis_name).info.frame16 \
    $topwin(analysis_name).info.frame16.frame0 {top frame center} \
    $topwin(analysis_name).info.frame16.upper {top frame center fillx}

  # build widget $topwin(analysis_name).info.frame17
  frame $topwin(analysis_name).info.frame17 \
    -background "$GuiColor(frame_bg)"  \
    -borderwidth {2} \
    -relief {ridge}

  # build widget $topwin(analysis_name).info.frame17.label0
  label $topwin(analysis_name).info.frame17.label0 \
    -background "$GuiColor(label_bg)" \
    -text {SPECTRAL FEATURES}

  # build widget $topwin(analysis_name).info.frame17.frame17
  frame $topwin(analysis_name).info.frame17.frame17 \
    -background "$GuiColor(frame_bg)"  \
    -borderwidth {1}

  # build widget $topwin(analysis_name).info.frame17.frame17.checkbutton1
	checkbutton $topwin(analysis_name).info.frame17.frame17.checkbutton1 -padx {1} -pady {2} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
	 -anchor {w} \
    -command {set temp 0
if {$spectral_feature} {$topwin(analysis_name).info.frame17.frame17.frame.frame1.entry config -fg black
$topwin(analysis_name).info.frame17.frame17.frame2.frame1.entry config -fg black}
if {!$spectral_feature} {$topwin(analysis_name).info.frame17.frame17.frame.frame1.entry config -fg white
$topwin(analysis_name).info.frame17.frame17.frame2.frame1.entry config -fg white}
} \
    -text {Get Spectral Features} \
    -variable {spectral_feature} \
    -relief {raised} \
    -width {25}

  # build widget $topwin(analysis_name).info.frame17.frame17.frame
  frame $topwin(analysis_name).info.frame17.frame17.frame \
    -background "$GuiColor(frame_bg)"  \
    -borderwidth {1}

  # build widget $topwin(analysis_name).info.frame17.frame17.frame.label4
  label $topwin(analysis_name).info.frame17.frame17.frame.label4 \
    -background "$GuiColor(label_bg)" \
    -padx {2} \
    -text {Number of Rings: }

  # build widget $topwin(analysis_name).info.frame17.frame17.frame.frame1.entry

  # build widget $topwin(analysis_name).info.frame17.frame17.frame.frame1
  frame $topwin(analysis_name).info.frame17.frame17.frame.frame1 \
    -borderwidth {1} \
    -highlightthickness {0} \
	 -background "$GuiColor(frame_bg)" \
    -height {30} \
    -width {30}

  # build widget $topwin(analysis_name).info.frame17.frame17.frame.frame1.entry
  entry $topwin(analysis_name).info.frame17.frame17.frame.frame1.entry \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
    -textvariable {analysis(spectral_ring)} -width {3}
  endEntry $topwin(analysis_name).info.frame17.frame17.frame.frame1.entry 0 \
  	 $topwin(analysis_name).info.frame17.frame17.frame2.frame1.entry \
  	 $topwin(analysis_name).info.frame17.frame17.frame2.frame1.entry

  # build widget $topwin(analysis_name).info.frame17.frame17.frame.frame1.menubutton7
  menubutton $topwin(analysis_name).info.frame17.frame17.frame.frame1.menubutton7 \
	 -activebackground "$GuiColor(btn_act_bg)" -background "$GuiColor(btn_bg)"  \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)" \
    -padx {1} -pady {1} -highlightthickness {0} \
    -text "$GuiMisc(pop_up_text)" -width {2}

  # bindings
  bind $topwin(analysis_name).info.frame17.frame17.frame.frame1.menubutton7 <Button-1> { MenuPopupPost $topwin(analysis_name).info.frame17.frame17.frame.frame1.menubutton7.menu0 %X %Y}
  bind $topwin(analysis_name).info.frame17.frame17.frame.frame1.menubutton7 <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame17.frame17.frame.frame1.menubutton7.menu0 %W }

  # build widget $topwin(analysis_name).info.frame17.frame17.frame.frame1.menubutton7.menu0
  menu $topwin(analysis_name).info.frame17.frame17.frame.frame1.menubutton7.menu0  \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" 

  $topwin(analysis_name).info.frame17.frame17.frame.frame1.menubutton7.menu0 add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"  \
    -label { 2} \
	 -command {set analysis(spectral_ring) 2}
  $topwin(analysis_name).info.frame17.frame17.frame.frame1.menubutton7.menu0 add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"  \
    -label { 3} \
	 -command {set analysis(spectral_ring) 3}
  $topwin(analysis_name).info.frame17.frame17.frame.frame1.menubutton7.menu0 add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"  \
    -label { 4} \
	 -command {set analysis(spectral_ring) 4}
  $topwin(analysis_name).info.frame17.frame17.frame.frame1.menubutton7.menu0 add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"  \
    -label { 5} \
	 -command {set analysis(spectral_ring) 5}
  $topwin(analysis_name).info.frame17.frame17.frame.frame1.menubutton7.menu0 add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"  \
    -label { 7} \
	 -command {set analysis(spectral_ring) 7}
  $topwin(analysis_name).info.frame17.frame17.frame.frame1.menubutton7.menu0 add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"  \
    -label {10} \
	 -command {set analysis(spectral_ring) 10}

  bind $topwin(analysis_name).info.frame17.frame17.frame.frame1.menubutton7.menu0 <B1-Motion> { MenuPopupMotion $topwin(analysis_name).info.frame17.frame17.frame.frame1.menubutton7.menu0 %W %X %Y}
  bind $topwin(analysis_name).info.frame17.frame17.frame.frame1.menubutton7.menu0 <ButtonRelease-1>  { MenuPopupRelease $topwin(analysis_name).info.frame17.frame17.frame.frame1.menubutton7.menu0 %W }

  # pack master $topwin(analysis_name).info.frame17.frame17.frame.frame1
  pack configure $topwin(analysis_name).info.frame17.frame17.frame.frame1.menubutton7 \
    -fill both \
    -side right
  pack configure $topwin(analysis_name).info.frame17.frame17.frame.frame1.entry \
    -fill both \
    -side left

  # pack widget $topwin(analysis_name).info.frame17.frame17.frame
  pack append $topwin(analysis_name).info.frame17.frame17.frame \
    $topwin(analysis_name).info.frame17.frame17.frame.label4 {left frame center} \
    $topwin(analysis_name).info.frame17.frame17.frame.frame1 {top frame center expand fill}

  # build widget $topwin(analysis_name).info.frame17.frame17.frame2
  frame $topwin(analysis_name).info.frame17.frame17.frame2 \
    -background "$GuiColor(frame_bg)"  \
    -borderwidth {1} 

  # build widget $topwin(analysis_name).info.frame17.frame17.frame2.label4
  label $topwin(analysis_name).info.frame17.frame17.frame2.label4 \
    -background "$GuiColor(label_bg)" \
    -padx {2} \
    -text {Number of Sectors: }

  # build widget $topwin(analysis_name).info.frame17.frame17.frame2.frame1
  frame $topwin(analysis_name).info.frame17.frame17.frame2.frame1 \
    -borderwidth {1} \
    -highlightthickness {0} \
	 -background "$GuiColor(frame_bg)" \
    -height {30} \
    -width {30}

  # build widget $topwin(analysis_name).info.frame17.frame17.frame2.frame1.entry
  entry $topwin(analysis_name).info.frame17.frame17.frame2.frame1.entry \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
    -textvariable {analysis(spectral_sector)} -width {3}
  endEntry $topwin(analysis_name).info.frame17.frame17.frame2.frame1.entry 0\
  	 $topwin(analysis_name).info.frame17.frame17.frame.frame1.entry \
  	 $topwin(analysis_name).info.frame17.frame17.frame.frame1.entry 

  # build widget $topwin(analysis_name).info.frame17.frame17.frame2.frame1.menubutton7
  menubutton $topwin(analysis_name).info.frame17.frame17.frame2.frame1.menubutton7 \
	 -activebackground "$GuiColor(btn_act_bg)" -background "$GuiColor(btn_bg)"  \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)" \
    -padx {1} -pady {1} -highlightthickness {0} \
    -text "$GuiMisc(pop_up_text)" -width {2}

  # bindings
  bind $topwin(analysis_name).info.frame17.frame17.frame2.frame1.menubutton7 <Button-1> { MenuPopupPost $topwin(analysis_name).info.frame17.frame17.frame2.frame1.menubutton7.menu0 %X %Y}
  bind $topwin(analysis_name).info.frame17.frame17.frame2.frame1.menubutton7 <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame17.frame17.frame2.frame1.menubutton7.menu0 %W }

  # build widget $topwin(analysis_name).info.frame17.frame17.frame2.frame1.menubutton7.menu0
  menu $topwin(analysis_name).info.frame17.frame17.frame2.frame1.menubutton7.menu0  \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" 

  $topwin(analysis_name).info.frame17.frame17.frame2.frame1.menubutton7.menu0 add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"  \
    -label { 2} \
	 -command {set analysis(spectral_sector) 2}
  $topwin(analysis_name).info.frame17.frame17.frame2.frame1.menubutton7.menu0 add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"  \
    -label { 3} \
	 -command {set analysis(spectral_sector) 3}
  $topwin(analysis_name).info.frame17.frame17.frame2.frame1.menubutton7.menu0 add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"  \
    -label { 4} \
	 -command {set analysis(spectral_sector) 4}
  $topwin(analysis_name).info.frame17.frame17.frame2.frame1.menubutton7.menu0 add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"  \
    -label { 5} \
	 -command {set analysis(spectral_sector) 5}
  $topwin(analysis_name).info.frame17.frame17.frame2.frame1.menubutton7.menu0 add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"  \
    -label { 7} \
	 -command {set analysis(spectral_sector) 7}
  $topwin(analysis_name).info.frame17.frame17.frame2.frame1.menubutton7.menu0 add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"  \
    -label {10} \
	 -command {set analysis(spectral_sector) 10}

  bind $topwin(analysis_name).info.frame17.frame17.frame2.frame1.menubutton7.menu0 <B1-Motion> { MenuPopupMotion $topwin(analysis_name).info.frame17.frame17.frame2.frame1.menubutton7.menu0 %W %X %Y}
  bind $topwin(analysis_name).info.frame17.frame17.frame2.frame1.menubutton7.menu0 <ButtonRelease-1>  { MenuPopupRelease $topwin(analysis_name).info.frame17.frame17.frame2.frame1.menubutton7.menu0 %W }

  # pack master $topwin(analysis_name).info.frame17.frame17.frame2.frame1
  pack configure $topwin(analysis_name).info.frame17.frame17.frame2.frame1.menubutton7 \
    -fill both \
    -side right
  pack configure $topwin(analysis_name).info.frame17.frame17.frame2.frame1.entry \
    -fill both \
    -side left

  # pack widget $topwin(analysis_name).info.frame17.frame17.frame2
  pack append $topwin(analysis_name).info.frame17.frame17.frame2 \
    $topwin(analysis_name).info.frame17.frame17.frame2.label4 {left frame center} \
    $topwin(analysis_name).info.frame17.frame17.frame2.frame1 {top frame center expand fill}

  # pack widget $topwin(analysis_name).info.frame17.frame17
  pack append $topwin(analysis_name).info.frame17.frame17 \
    $topwin(analysis_name).info.frame17.frame17.checkbutton1 {left frame center pady 5} \
    $topwin(analysis_name).info.frame17.frame17.frame {left frame center} \
    $topwin(analysis_name).info.frame17.frame17.frame2 {left frame w}

  # pack widget $topwin(analysis_name).info.frame17
  pack append $topwin(analysis_name).info.frame17 \
    $topwin(analysis_name).info.frame17.label0 {top frame center} \
    $topwin(analysis_name).info.frame17.frame17 {top frame center}

  # build widget $topwin(analysis_name).info.frame18
  frame $topwin(analysis_name).info.frame18 \
    -background "$GuiColor(frame_bg)"  \
    -borderwidth {2} \
    -relief {ridge}

 # build widget $topwin(analysis_name).info.frame18.frame0
  frame $topwin(analysis_name).info.frame18.frame0 \
    -background "$GuiColor(frame_bg)"  \
    -borderwidth {1} \
    -relief {flat}

  # build widget $topwin(analysis_name).info.frame18.frame0.label0
  label $topwin(analysis_name).info.frame18.frame0.label0 \
    -background "$GuiColor(label_bg)" \
    -text {RST-INVARIANT MOMENT-BASED FEATURES}

  # build widget $topwin(analysis_name).info.frame18.frame0.button1
	button $topwin(analysis_name).info.frame18.frame0.button1 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -command {set temp $analysis(rst_all_button)
if {$temp == "Select All"} {foreach i [list 1 2 3 4 5 6 7] {phi$i select; set analysis(rst_all_button) "Deselect All"}}
if {$temp == "Deselect All"} {foreach i [list 1 2 3 4 5 6 7] {phi$i deselect; set analysis(rst_all_button) "Select All"}}
} \
    -text {Select All} \
    -textvariable {analysis(rst_all_button)} \
    -width {15}

   # pack widget $topwin(analysis_name).info.frame18.frame0
  pack append $topwin(analysis_name).info.frame18.frame0 \
    $topwin(analysis_name).info.frame18.frame0.label0 {left frame center} \
    $topwin(analysis_name).info.frame18.frame0.button1 {left frame center pady 10} \


   # build widget $topwin(analysis_name).info.frame18.checkbutton2
	checkbutton $topwin(analysis_name).info.frame18.checkbutton2 -padx {1} -pady {2} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -text {rst1} \
	 -anchor {w} \
    -variable {rst1} \
    -relief {raised} \
    -width {7}

  # build widget $topwin(analysis_name).info.frame18.checkbutton3
	checkbutton $topwin(analysis_name).info.frame18.checkbutton3 -padx {1} -pady {2} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -text {rst2} \
    -variable {rst2} \
	 -anchor {w} \
    -relief {raised} \
    -width {7}

  # build widget $topwin(analysis_name).info.frame18.checkbutton4
	checkbutton $topwin(analysis_name).info.frame18.checkbutton4 -padx {1} -pady {2} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -text {rst3} \
	 -anchor {w} \
    -variable {rst3} \
    -relief {raised} \
    -width {7}

  # build widget $topwin(analysis_name).info.frame18.checkbutton5
	checkbutton $topwin(analysis_name).info.frame18.checkbutton5 -padx {1} -pady {2} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -text {rst4} \
	 -anchor {w} \
    -variable {rst4} \
    -relief {raised} \
    -width {7}

  # build widget $topwin(analysis_name).info.frame18.checkbutton6
	checkbutton $topwin(analysis_name).info.frame18.checkbutton6 -padx {1} -pady {2} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -text {rst5} \
	 -anchor {w} \
    -variable {rst5} \
    -relief {raised} \
    -width {7}

  # build widget $topwin(analysis_name).info.frame18.checkbutton7
	checkbutton $topwin(analysis_name).info.frame18.checkbutton7 -padx {1} -pady {2} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -text {rst6} \
	 -anchor {w} \
    -variable {rst6} \
    -relief {raised} \
    -width {7}

  # build widget $topwin(analysis_name).info.frame18.checkbutton8
	checkbutton $topwin(analysis_name).info.frame18.checkbutton8 -padx {1} -pady {2} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -text {rst7} \
	 -anchor {w} \
    -variable {rst7} \
    -relief {raised} \
    -width {7}

  # pack widget $topwin(analysis_name).info.frame18
  pack append $topwin(analysis_name).info.frame18 \
    $topwin(analysis_name).info.frame18.frame0 {top frame center} \
    $topwin(analysis_name).info.frame18.checkbutton2 {left frame center pady 5 expand} \
    $topwin(analysis_name).info.frame18.checkbutton3 {left frame center expand} \
    $topwin(analysis_name).info.frame18.checkbutton4 {left frame center expand} \
    $topwin(analysis_name).info.frame18.checkbutton5 {left frame center expand} \
    $topwin(analysis_name).info.frame18.checkbutton6 {left frame center expand} \
    $topwin(analysis_name).info.frame18.checkbutton7 {left frame center expand} \
    $topwin(analysis_name).info.frame18.checkbutton8 {left frame center expand}

  # build widget $topwin(analysis_name).info.frame19
  frame $topwin(analysis_name).info.frame19 \
    -background "$GuiColor(frame_bg)"  \
    -borderwidth {2} \
    -relief {ridge}

# build widget $topwin(analysis_name).info.frame19.frame0
  frame $topwin(analysis_name).info.frame19.frame0 \
    -background "$GuiColor(frame_bg)"  \
    -borderwidth {1} \
    -relief {flat}

  # build widget $topwin(analysis_name).info.frame19.frame0.label9
  label $topwin(analysis_name).info.frame19.frame0.label9 \
    -background "$GuiColor(label_bg)" \
    -text {TEXTURE FEATURES}

  # build widget $topwin(analysis_name).info.frame19.frame0.button10
	button $topwin(analysis_name).info.frame19.frame0.button10 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -command {set temp $analysis(text_all_button)
if {$temp == "Select All"} {foreach i [list 0 45 90 135 avg] {texture_$i select; set analysis(text_all_button) "Deselect All" }}
if {$temp == "Deselect All"} {foreach i [list 0 45 90 135 avg] {texture_$i deselect; set analysis(text_all_button) "Select All" }}} \
    -text {Select All} \
    -textvariable {analysis(text_all_button)} \
    -width {15}

 # pack widget $topwin(analysis_name).info.frame19.frame0
  pack append $topwin(analysis_name).info.frame19.frame0 \
    $topwin(analysis_name).info.frame19.frame0.label9 {left frame center} \
    $topwin(analysis_name).info.frame19.frame0.button10 {left frame center pady 10} \
  
# build widget $topwin(analysis_name).info.frame19.checkbutton11
	checkbutton $topwin(analysis_name).info.frame19.checkbutton11 -padx {1} -pady {2} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -text {Energy} \
	 -anchor {w} \
    -variable {texture_energy} \
    -relief {raised} \
    -width {12}

  # build widget $topwin(analysis_name).info.frame19.checkbutton12
	checkbutton $topwin(analysis_name).info.frame19.checkbutton12 -padx {1} -pady {2} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -text {Inertia} \
	 -anchor {w} \
    -variable {inertia} \
    -relief {raised} \
    -width {12}

  # build widget $topwin(analysis_name).info.frame19.checkbutton13
	checkbutton $topwin(analysis_name).info.frame19.checkbutton13 -padx {1} -pady {2} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -text {Correlation} \
	 -anchor {w} \
    -variable {correlation} \
    -relief {raised} \
    -width {12}

  # build widget $topwin(analysis_name).info.frame19.checkbutton14
	checkbutton $topwin(analysis_name).info.frame19.checkbutton14 -padx {1} -pady {2} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -text {Inverse Diff  } \
	 -anchor {w} \
    -variable {inverse_diff} \
    -relief {raised} \
    -width {12}

  # build widget $topwin(analysis_name).info.frame19.checkbutton15
	checkbutton $topwin(analysis_name).info.frame19.checkbutton15 -padx {1} -pady {2} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -text {Entropy} \
	 -anchor {w} \
    -variable {texture_entropy} \
    -relief {raised} \
    -width {12}

  # build widget $topwin(analysis_name).info.frame19.frame1
  frame $topwin(analysis_name).info.frame19.frame1 \
    -background "$GuiColor(frame_bg)"  \
    -borderwidth {1} \
    -relief {flat}

  # build widget $topwin(analysis_name).info.frame19.frame1.label1
  label $topwin(analysis_name).info.frame19.frame1.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -text {Texture distance (pixels):}

  # build widget $topwin(analysis_name).info.frame19.frame1.value
  entry $topwin(analysis_name).info.frame19.frame1.value \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
    -textvariable {analysis(texture_dist)} \
    -width {3}
  endEntry $topwin(analysis_name).info.frame19.frame1.value 

  # build widget $topwin(analysis_name).info.frame19.frame1.menubutton2
	menubutton $topwin(analysis_name).info.frame19.frame1.menubutton2 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)" \
    -text "$GuiMisc(pop_up_text)" \
	 -width {2}
  bind $topwin(analysis_name).info.frame19.frame1.menubutton2 <Button-1> {
  MenuPopupPost $topwin(analysis_name).info.frame19.frame1.menubutton2.menu %X %Y
  }
  bind $topwin(analysis_name).info.frame19.frame1.menubutton2 <ButtonRelease-1> {
  MenuPopupRelease $topwin(analysis_name).info.frame19.frame1.menubutton2.menu %W
  }
  # build widget $topwin(analysis_name).info.frame19.frame1.menubutton2.menu
  menu $topwin(analysis_name).info.frame19.frame1.menubutton2.menu \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"
  $topwin(analysis_name).info.frame19.frame1.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(texture_dist) 2} \
    -label { 2 } \
    -state {active}
  $topwin(analysis_name).info.frame19.frame1.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(texture_dist) 3} \
    -label { 3 }
  $topwin(analysis_name).info.frame19.frame1.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(texture_dist) 4} \
    -label { 4 }
  $topwin(analysis_name).info.frame19.frame1.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(texture_dist) 5} \
    -label { 5 }
  $topwin(analysis_name).info.frame19.frame1.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(texture_dist) 6} \
    -label { 6 }
  bindtags  $topwin(analysis_name).info.frame19.frame1.menubutton2.menu {Menu all}
  bind $topwin(analysis_name).info.frame19.frame1.menubutton2.menu <B1-Motion> {
  MenuPopupMotion $topwin(analysis_name).info.frame19.frame1.menubutton2.menu %W %X %Y
  }
  bind $topwin(analysis_name).info.frame19.frame1.menubutton2.menu <ButtonRelease-1> {
  MenuPopupRelease $topwin(analysis_name).info.frame19.frame1.menubutton2.menu %W
  }

  # pack widget $topwin(analysis_name).info.frame19.frame1
  pack append $topwin(analysis_name).info.frame19.frame1 \
    $topwin(analysis_name).info.frame19.frame1.label1 {left frame center fill} \
    $topwin(analysis_name).info.frame19.frame1.value {left frame center expand fill} \
    $topwin(analysis_name).info.frame19.frame1.menubutton2 {right frame center fill}

  # pack widget $topwin(analysis_name).info.frame19
  pack append $topwin(analysis_name).info.frame19 \
    $topwin(analysis_name).info.frame19.frame0 {top frame center} \
    $topwin(analysis_name).info.frame19.frame1 {top frame center} \
    $topwin(analysis_name).info.frame19.checkbutton11 {left frame center pady 5 expand} \
    $topwin(analysis_name).info.frame19.checkbutton12 {left frame center expand} \
    $topwin(analysis_name).info.frame19.checkbutton13 {left frame center expand} \
    $topwin(analysis_name).info.frame19.checkbutton14 {left frame center expand} \
    $topwin(analysis_name).info.frame19.checkbutton15 {left frame center expand}

  # build widget $topwin(analysis_name).info.frame20
  frame $topwin(analysis_name).info.frame20 \
    -background "$GuiColor(frame_bg)"  \
    -highlightthickness {0} \
    -borderwidth {2} \
    -relief {raised}

  # pack widget $topwin(analysis_name).info
  pack append $topwin(analysis_name).info \
    $topwin(analysis_name).info.frame15 {top frame center pady 5 fill} \
    $topwin(analysis_name).info.frame18 {top frame center pady 5 fill} \
    $topwin(analysis_name).info.frame20 {top frame center pady 1 fill} \
    $topwin(analysis_name).info.frame16 {top frame center pady 5 fill} \
    $topwin(analysis_name).info.frame19 {top frame center pady 5 fill} \
    $topwin(analysis_name).info.frame17 {top frame center pady 5 fill}

  # build widget $topwin(analysis_name).perform
  frame $topwin(analysis_name).perform \
    -background "$GuiColor(frame_bg)"  \
	 -relief {raised} \
    -borderwidth {1}

  # build widget $topwin(analysis_name).perform.dismiss
	button $topwin(analysis_name).perform.dismiss -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(dis_act_bg)" \
    -background "$GuiColor(dis_bg)" \
    -foreground "$GuiColor(dis_fg)" \
    -command {destroy $topwin(analysis_name); set topwin(analysis_name) NONE
	 if {$labeled_image != ""} {catch {cvip dq $labeled_image}; set labeled_image ""}} \
    -text {DISMISS}

  # build widget $topwin(analysis_name).perform.button6
	button $topwin(analysis_name).perform.button6 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(perf_act_bg)" \
    -background "$GuiColor(perf_bg)" \
    -foreground "$GuiColor(perf_fg)" \
    -command {
set temp3 $current_image
CvipMessage "be patient if you selected any texture and spectral features take some time, \n\
which takes computing time"
update
$topwin(analysis_name) configure -cursor watch; update

###### CREATE ARRAY "feature_tcl_name" , USED IN CVIPTCL FUNCTION CALLS ######

set countr1 0
foreach i [list area centroid orientation perimeter euler projection thinness aspect rst1 rst2 rst3 rst4 rst5 rst6 rst7 mean stddev skew histo_energy histo_entropy texture_energy inertia correlation inverse_diff texture_entropy spectral_feature ] {set feature_tcl_name($countr1) $i; incr countr1} 

set number_features [expr $countr1]

###### CREATE ARRAY "feature_file_name", USED IN FEATURE FILE OUTPUT ######

set countr1 0

foreach i [list Area {Centroid (row, column)} {Orientation (Axis of Least Second Moment)} Perimeter {Euler number} Projections Thinness {Aspect ratio} RST1 RST2 RST3 RST4 RST5 RST6 RST7 Histo_Mean {Standard Deviation} Histo_Skew Histo_Energy Histo_Entropy Texture_energy Inertia Correlation Inverse_diff Texture_entropy {Spectral features} ] {set feature_file_name($countr1) $i; incr countr1}

###### GET INFO FROM FEATURE GUI ENTRY BOXES ######

catch {set coordinate_info [selection get]
scan $coordinate_info {%s %s %d %d %d %d} cname null2 row_coord col_coord d_row d_col} cresult

if {$analysis(obj_row) == "" || $analysis(obj_col) == ""} {
	if  {$row_coord == ""} {
		set alertBox(background) white
		CvipAlert "\nBoth row and column must be filled in. Enter values from the keyboard or select an object with the mouse."
		$topwin(analysis_name) configure -cursor top_left_arrow; update
		return
	}
}

if {$row_coord != "" } { 
	if {[string compare $cname $current_image] == 0 || [string compare $cname $second_image] == 0 || [string compare $cname $labeled_image] == 0  } {
		if {[string compare $row_coord $old_mouse_row] != 0} {
			set analysis(obj_row) $row_coord
			set old_mouse_row $row_coord
		}
		if {[string compare $col_coord $old_mouse_col] != 0} {
			set analysis(obj_col) $col_coord
			set old_mouse_col $col_coord
		}
	}	else {
			set alertBox(background) white
			CvipAlert "\nYou have selected coordinates from the image: \n$cname\n\n Please select your coordinates from either:\n$current_image\n or \n$second_image"
			$topwin(analysis_name) configure -cursor top_left_arrow; update
			return
		}
	
}

set missing_info 0
foreach i [list $current_image $second_image $analysis(feature_fn) $analysis(obj_row) $analysis(obj_col)] {if {$i == ""} {set missing_info 1} }
if {$missing_info} {set alertBox(background) white;AlertBox "\nEach of the five white entry boxes at the top of 
the FEATURE menu must contain data. Please provide the needed information";
$topwin(analysis_name) configure -cursor top_left_arrow; update
return}


###### NOW, PERFORM SELECTED FUNCTIONS ######

set fcounter -1
set total_selected 0
set add_for_spectral 0
if { $temp_label == "" } {
  if [ catch {set labeled_image [cvip label $second_image]} cresult ] {
	CvipAlert "$cresult."
        $topwin(analysis_name) configure -cursor top_left_arrow; update
        return

  }
} else {
  
  if [ catch {cvip dq $labeled_image; set labeled_image [cvip label $second_image]} cresult ] {
        CvipAlert "$cresult."
        $topwin(analysis_name) configure -cursor top_left_arrow; update
        return
  }
  
}

#set temp_label $current_image
get_image_info


###### If we are going to find texture features on a color image, perform a luminance transform ######
set lum_image ""
if {$bands == 3} {
	if {[set texture_energy] || [set inertia] || [set correlation] || [set inverse_diff] || [set texture_entropy] } {
		if { $GUI_DEBUG == "1" } {
			puts "Performing transform for use with texture features"
		}
		set lum_image [cvip luminance $current_image]
	}
}

###### Most functions use the default procedure; only functions that need to pass other parameters have a 'case' statement ######

for {incr fcounter} {$fcounter < $number_features} {incr fcounter} {
	if [set $feature_tcl_name($fcounter)] {
		switch -exact -- $feature_tcl_name($fcounter) {


		histo_entropy -
		histo_energy -
		skew -
		stddev -
		mean 	{
		if {$bands == 1} {
		if [ catch {set histo_data [cvip hist_feature $current_image $labeled_image $analysis(obj_row) $analysis(obj_col)]} cresult ] {
			CvipMessage "$cresult"
		}
		scan $histo_data {%s %s %s %s %s} mean_data stddev_data  skew_data h_energy_data h_entropy_data
		   switch -exact -- $feature_tcl_name($fcounter) {
			mean	{set feature_data($total_selected) $mean_data}
			stddev	{set feature_data($total_selected) $stddev_data}
			skew	{set feature_data($total_selected) $skew_data}
			histo_energy	{set feature_data($total_selected) $h_energy_data}
			histo_entropy	{set feature_data($total_selected) $h_entropy_data}
			default	{set feature_data($total_selected) $mean_data}
	

		   }
		set which_name($total_selected) $feature_file_name($fcounter)
		incr total_selected
		}

		if {$bands == 3} {
		if [ catch { set histo_data [cvip hist_feature $current_image $labeled_image $analysis(obj_row) $analysis(obj_col)] } cresult ] {
			CvipMessage "$cresult"
		}
		scan $histo_data {%s %s %s %s %s %s %s %s %s %s %s %s %s %s %s} meanarr(0) stddevarr(0)  skewarr(0) h_energyarr(0) h_entropyarr(0) meanarr(1) stddevarr(1) skewarr(1) h_energyarr(1) h_entropyarr(1) meanarr(2) stddevarr(2)  skewarr(2) h_energyarr(2) h_entropyarr(2)
		set i 0; while {$i < 3} {switch -exact -- $feature_tcl_name($fcounter) {
			mean	{set feature_data($total_selected) $meanarr($i) ; set which_name($total_selected) $feature_file_name($fcounter)_band$i; incr total_selected; incr i}
			stddev	{set feature_data($total_selected) $stddevarr($i) ; set which_name($total_selected) $feature_file_name($fcounter)_band$i; incr total_selected; incr i }
			skew	{set feature_data($total_selected) $skewarr($i) ; set which_name($total_selected) $feature_file_name($fcounter)_band$i; incr total_selected; incr i }
			histo_energy	{set feature_data($total_selected) $h_energyarr($i) ; set which_name($total_selected) $feature_file_name($fcounter)_band$i; incr total_selected; incr i }
			histo_entropy	{set feature_data($total_selected) $h_entropyarr($i) ; set which_name($total_selected) $feature_file_name($fcounter)_band$i; incr total_selected; incr i }
						}
		 }



		}

		        }

		rst1 -
		rst2 -
		rst3 -
		rst4 -
		rst5 -
		rst6 -
		rst7 	{
		if [catch {set rst_data [cvip rst_invariant $labeled_image $analysis(obj_row) $analysis(obj_col)]} cresult ] {
			CvipMessage "$cresult"
		}
		scan $rst_data {%s %s %s %s %s %s %s} rst1_data rst2_data  rst3_data rst4_data rst5_data rst6_data rst7_data
		switch -exact -- $feature_tcl_name($fcounter) {
			rst1	{set feature_data($total_selected) $rst1_data}
			rst2	{set feature_data($total_selected) $rst2_data}
			rst3	{set feature_data($total_selected) $rst3_data}
			rst4	{set feature_data($total_selected) $rst4_data}
			rst5	{set feature_data($total_selected) $rst5_data}
			rst6	{set feature_data($total_selected) $rst6_data}
			rst7	{set feature_data($total_selected) $rst7_data}

		}
		set which_name($total_selected) $feature_file_name($fcounter)
		incr total_selected}

		texture_energy -
		inertia -
		correlation -
		inverse_diff - 
		texture_entropy 	{
			set texture_band 0
			switch -exact -- $feature_tcl_name($fcounter) {
				texture_energy	{set hex_selected 0x0001}
				inertia		{set hex_selected 0x0002}
				inverse_diff	{set hex_selected 0x0010}
				correlation	{set hex_selected 0x0004}
				texture_entropy	{set hex_selected 0x0100}
			}
			if {$bands == 1} {
			   if [ catch { set texture_data [cvip texture $current_image $labeled_image $texture_band $analysis(obj_row) \
						$analysis(obj_col) $hex_selected $analysis(texture_dist)] } cresult ] {
					CvipMessage "$cresult"
				}
			}
			if {$bands == 3} {
			   if [ catch { set texture_data [cvip texture $lum_image $labeled_image $texture_band $analysis(obj_row) \
						$analysis(obj_col) $hex_selected $analysis(texture_dist)] } cresult ] {
					CvipMessage "$cresult"
				}
			}
			scan $texture_data {%s %s %s %s %s %s} n1 n2  n3 n4 texture_avg texture_range
			switch -exact -- $feature_tcl_name($fcounter) {
				texture_energy -
				inertia -
				correlation -
				inverse_diff -
				texture_entropy		{set feature_data($total_selected) $texture_avg
						 set which_name($total_selected) $feature_file_name($fcounter)_average
						 incr total_selected
						 set feature_data($total_selected) $texture_range
						 set which_name($total_selected) $feature_file_name($fcounter)_range
						}					
			}
	
		incr total_selected
				}


		spectral_feature	{
			if [ catch { set feature_data($total_selected) [cvip $feature_tcl_name($fcounter) $current_image $labeled_image \
					$analysis(spectral_ring)  $analysis(spectral_sector) $analysis(obj_row) $analysis(obj_col)] } cresult ] {
				CvipMessage "$cresult"
			}
		set which_name($total_selected) $feature_file_name($fcounter)
		incr total_selected
		set add_for_spectral [expr 1 + $analysis(spectral_ring) + $analysis(spectral_sector)] }

		
		projection		{
			if [catch {set feature_data($total_selected) [cvip $feature_tcl_name($fcounter) $labeled_image $analysis(obj_row) \
					$analysis(obj_col) $analysis(proj_height) $analysis(proj_width)] } cresult ] {
				CvipMessage "$cresult"
			}
		set which_name($total_selected) $feature_file_name($fcounter)
		incr total_selected}


		default	{
		if [catch {set feature_data($total_selected) [cvip $feature_tcl_name($fcounter) $labeled_image $analysis(obj_row) \
				$analysis(obj_col)]} cresult ] {
			CvipMessage "$cresult"
		}
		set which_name($total_selected) $feature_file_name($fcounter)
		incr total_selected}

		}

	}

}

set header_total_selected $total_selected

###### IF FEATURE FILE EXIST, MAKE SURE CURRENT FEATURE SELECTIONS MATCH THOSE OF DATA IN THE FILE ######

set open_file_type w
if [file exists $analysis(feature_fn)] {set open_file_type a}


if [file exists $analysis(feature_fn)] {
	set fileCk [open $analysis(feature_fn) r]

###### WE NOW READ THE FIRST FOUR LINES ######

set temp_img1 $image_to_load
set image_to_load $current_image
get_image_info 
set bandword "band"
if {$bands == 3} {set bandword "bands"}

	set hdr_new "CVIPtools feature file for images with $bands $bandword."
	set hdr_old [gets $fileCk]
	if {[string compare $hdr_old $hdr_new] != 0} {set alertBox(background) white; set temp1 [AlertBox "The number of bands in the current image ($bands) is not the same as in the selected feature file. All original images in this file must have the same number of bands." "" "350x150" "Alert box" OK {View Feature File}]; if { $temp1 } {TextBoxFile $analysis(feature_fn) "" "500x600"}; unset hdr_old; unset hdr_new;  close $fileCk; $topwin(analysis_name) configure -cursor top_left_arrow; update; return}

	set hdr_new "1: image name"
	set hdr_old [gets $fileCk] 
	if {[string compare $hdr_old $hdr_new] != 0} {set alertBox(background) white; AlertBox "There appears to be a problem with the program. The second line of the feature file should read \"1: image name\". Please notify the system administrator."; unset hdr_old; unset hdr_new; close $fileCk; $topwin(analysis_name) configure -cursor top_left_arrow; update; return}


	set hdr_new "2: object's row coordinate"
	set hdr_old [gets $fileCk] 
	if {[string compare $hdr_old $hdr_new] != 0} {set alertBox(background) white; AlertBox "There appears to be a problem with the program. The second line of the feature file should read \"2: object's row coordinate\". Please notify the system administrator."; unset hdr_old; unset hdr_new; close $fileCk; $topwin(analysis_name) configure -cursor top_left_arrow; update; return}

	set hdr_new "3: object's col coordinate"
	set hdr_old [gets $fileCk]
	if {[string compare $hdr_old $hdr_new] != 0} {set alertBox(background) white; AlertBox "There appears to be a problem with the program. The second line of the feature file should read \"3: object's col coordinate\". Please notify the system administrator."; unset hdr_old; unset hdr_new; close $fileCk; $topwin(analysis_name) configure -cursor top_left_arrow; update; return}

	set next_num 4
	set temp2 0
	while {[set hdr_old [gets $fileCk]] != "*** END OF HEADER; CLASS (optional) SHOWN AT END OF DATA ***"} {
		switch -exact $which_name([expr $next_num - 4]) {


{Spectral features} 	{set temp $next_num; set spectral_suffix "";
			 for {set bh 0} { $bh < $bands} {incr bh} {
			 if {$bands > 1} {set spectral_suffix "_band${bh}"}
			 if {$bh > 0} {set hdr_old [gets $fileCk]}
			 set hdr_new "$temp: Spectral DC$spectral_suffix"; incr temp; if {[string compare $hdr_old $hdr_new] != 0} {set alertBox(background) white; AlertBox "The features you have selected are not the same as those in feature file \"$analysis(feature_fn)\". Either select the same features as shown in the file header or select a new name for your feature file. Spectral DC_band${bh} error."; unset hdr_old; unset hdr_new; close $fileCk; $topwin(analysis_name) configure -cursor top_left_arrow; update; return}
			 for {set i 0} {$i < $analysis(spectral_ring)} {incr i; incr temp} {set hdr_old [gets $fileCk]; set hdr_new "$temp: Ring[expr $i + 1]$spectral_suffix"; if {[string compare $hdr_old $hdr_new] != 0} {set alertBox(background) white; AlertBox "The features you have selected are not the same as those in feature file \"$analysis(feature_fn)\". Either select the same features as shown in the file header or select a new name for your feature file. Ring[expr $i + 1]_band${bh} error."; unset hdr_old; unset hdr_new; close $fileCk; $topwin(analysis_name) configure -cursor top_left_arrow; update; return}}
			 for {set i 0} {$i < $analysis(spectral_sector)} {incr i; incr temp} {set hdr_old [gets $fileCk]; set hdr_new "$temp: Sector[expr $i + 1]$spectral_suffix"; if {[string compare $hdr_old $hdr_new] != 0} {set alertBox(background) white; AlertBox "The features you have selected are not the same as those in feature file \"$analysis(feature_fn)\". Either select the same features as shown in the file header or select a new name for your feature file.  Sector[expr $i + 1]_band${bh} error."; unset hdr_old; unset hdr_new; close $fileCk; $topwin(analysis_name) configure -cursor top_left_arrow; update; return}}
			
			 }

			 set temp2 1
			}


Texture_entropy_average -
Texture_energy_average -
Inertia_average -
Correlation_average -
Inverse_diff_average 	{set temp $next_num; if {$temp2} {set temp [expr $next_num + $analysis(spectral_ring) + $analysis(spectral_sector) ]}
set hdr_new "$temp: $which_name([expr $next_num - 4])  (Texture distance: $analysis(texture_dist))"}

{Projections}		{set temp $next_num; if {$temp2} {set temp [expr $next_num + $analysis(spectral_ring) + $analysis(spectral_sector) ]}
set hdr_new "$temp: Projections  (Height: $analysis(proj_height)  Width: $analysis(proj_width))"}


default 	{ set temp $next_num; if {$temp2} {set temp [expr $next_num + $analysis(spectral_ring) + $analysis(spectral_sector) ]}
set hdr_new "$temp: $which_name([expr $next_num - 4])" }

}

if {[string compare $hdr_old $hdr_new] != 0} {set alertBox(background) white; if [AlertBox "The features you have selected are not the same as those in feature file \"$analysis(feature_fn)\". Either select the same features as shown in the file header or select a new name for your feature file." "" "350x150" "Alert box" OK {View Feature File}] {TextBoxFile $analysis(feature_fn) "" "500x600"}; unset hdr_old; unset hdr_new;  close $fileCk; $topwin(analysis_name) configure -cursor top_left_arrow; update; return}

incr next_num

### end of while ###
}

set hdr_new "*** END OF HEADER; CLASS (optional) SHOWN AT END OF DATA ***"

if {[string compare $hdr_old $hdr_new] != 0} {set alertBox(background) white; if [AlertBox "The features you have selected are not the same as those in feature file \"$analysis(feature_fn)\". Either select the same features as shown in the file header or select a new name for your feature file." "" "350x150" "Alert box" OK {View Feature File}] {TextBoxFile $analysis(feature_fn) "" "500x600"}; unset hdr_old; unset hdr_new;  close $fileCk; $topwin(analysis_name) configure -cursor top_left_arrow; update; return}
### IF SPECTRAL FEATURES ARE SELECTED, NEED TO ACCOUNT FOR EXTRA HEADER LISTINGS ###

if {[expr $next_num - 4] != $total_selected} {set alertBox(background) white; AlertBox "The features you have selected are not the same as those in feature file \"$analysis(feature_fn)\". Either select the same features as shown in the file header or select a new name for your feature file."; unset hdr_old; unset hdr_new; close $fileCk; $topwin(analysis_name) configure -cursor top_left_arrow; update; return}

unset hdr_old
unset hdr_new
close $fileCk
}

###### WRITE SELECTIONS TO A FEATURE FILE; FIRST, THE HEADER. The switch statement uses the function names as in "feature_file_name" for selection. ######
###### Only features that require extra parameters be printed in the header need to be listed as switch items. ######
set temp_img1 $image_to_load
set image_to_load $current_image
get_image_info 
set bandword "band"
if {$bands == 3} {set bandword "bands"}
 
set fileID [open $analysis(feature_fn) $open_file_type+ 0666]
set temp2 0
if {$open_file_type == "w"} {
	puts $fileID "CVIPtools feature file for images with $bands $bandword."
	puts $fileID "1: image name"
	puts $fileID "2: object's row coordinate"
	puts $fileID "3: object's col coordinate"
	for {set next_num 4} {$next_num < [expr 4 + $total_selected]} {incr next_num} {

switch -exact $which_name([expr $next_num - 4]) {

{Spectral features} 	{set temp $next_num; set spectral_suffix "";
                            for {set b 0} {$b < $bands} {incr b} {
				if {$bands > 1} {set spectral_suffix "_band${b}"}
			 	puts $fileID "$temp: Spectral DC$spectral_suffix"; incr temp
			 	for {set i 0} {$i < $analysis(spectral_ring)} {incr i; incr temp} {puts $fileID "$temp: Ring[expr $i + 1]$spectral_suffix"}
			 	for {set i 0} {$i < $analysis(spectral_sector)} {incr i; incr temp} {puts $fileID "$temp: Sector[expr $i + 1]$spectral_suffix"}
			 	set temp2 1
			    }
			}

Texture_entropy_average -
Texture_energy_average -
Inertia_average -
Correlation_average -
Inverse_diff_average 	{set temp $next_num; if {$temp2} {set temp [expr $next_num + $analysis(spectral_ring) + $analysis(spectral_sector)]} 
		puts $fileID "$temp: $which_name([expr $next_num - 4])  (Texture distance: $analysis(texture_dist))"}


Projections	{set temp $next_num; if {$temp2} {set temp [expr $next_num + $analysis(spectral_ring) + $analysis(spectral_sector) ]}
		puts $fileID "$temp: $which_name([expr $next_num - 4])  (Height: $analysis(proj_height)  Width: $analysis(proj_width))"}


default 	{set temp $next_num; if {$temp2} {set temp [expr $next_num + $analysis(spectral_ring) + $analysis(spectral_sector) ]}
		puts $fileID "$temp: $which_name([expr $next_num - 4])"}


}

}
puts $fileID "*** END OF HEADER; CLASS (optional) SHOWN AT END OF DATA ***"
puts $fileID ""
}

###### END OF HEADER; NOW WRITE DATA ###### 
set fcounter -1
puts -nonewline $fileID "$current_image $analysis(obj_row) $analysis(obj_col)  "
for {incr fcounter} {$fcounter < $header_total_selected} {incr fcounter} {puts -nonewline $fileID "$feature_data($fcounter) " }
if {$analysis(feature_class) != ""} {puts -nonewline $fileID $analysis(feature_class)}
puts $fileID ""
puts $fileID ""
close $fileID
set image_to_load $temp_img1
unset temp_img1
$topwin(analysis_name) configure -cursor top_left_arrow; update
CvipMessage "Feature extraction performed on image \"$temp3\""
cvip dq $labeled_image;
} \
    -text {PERFORM}

  # pack widget $topwin(analysis_name).perform
  pack append $topwin(analysis_name).perform \
    $topwin(analysis_name).perform.dismiss {left frame w} \
    $topwin(analysis_name).perform.button6 {right frame e}

  # pack widget $topwin(analysis_name)
  pack append $topwin(analysis_name) \
    $topwin(analysis_name).current_image {top frame center expand fill} \
    $topwin(analysis_name).perform {bottom frame center expand fill} \
    $topwin(analysis_name).info {top frame center expand fill}


  if {"[info procs XFEdit]" != ""} {
    catch "XFMiscBindWidgetTree $topwin(analysis_name)"
    after 2 "catch {XFEditSetShowWindows}"
  }
}


# Procedure: binary_area
proc binary_area { args} {
#xf ignore me 4
global topwin
return [eval "$topwin(analysis_name).info.frame15.upper.checkbutton22 $args"]
}


# Procedure: binary_aspe
proc binary_aspe { args} {
#xf ignore me 4
global topwin
return [eval "$topwin(analysis_name).info.frame15.lower.checkbutton29 $args"]
}


# Procedure: binary_cent
proc binary_cent { args} {
#xf ignore me 4
global topwin
return [eval "$topwin(analysis_name).info.frame15.upper.checkbutton23 $args"]
}


# Procedure: binary_eule
proc binary_eule { args} {
#xf ignore me 4
global topwin
return [eval "$topwin(analysis_name).info.frame15.lower.checkbutton26 $args"]
}


# Procedure: binary_orie
proc binary_orie { args} {
#xf ignore me 4
global topwin
return [eval "$topwin(analysis_name).info.frame15.upper.checkbutton24 $args"]
}


# Procedure: binary_peri
proc binary_peri { args} {
#xf ignore me 4
global topwin
return [eval "$topwin(analysis_name).info.frame15.upper.checkbutton25 $args"]
}


# Procedure: binary_proj
proc binary_proj { args} {
#xf ignore me 4
global topwin
return [eval "$topwin(analysis_name).info.frame15.lower.checkbutton27 $args"]
}


# Procedure: binary_thin
proc binary_thin { args} {
#xf ignore me 4
global topwin
return [eval "$topwin(analysis_name).info.frame15.lower.checkbutton28 $args"]
}


# Procedure: his_dev
proc his_dev { args} {
#xf ignore me 4
global topwin
return [eval "$topwin(analysis_name).info.frame16.upper.checkbutton23 $args"]
}


# Procedure: his_energy
proc his_energy { args} {
#xf ignore me 4
global topwin
return [eval "$topwin(analysis_name).info.frame16.upper.checkbutton25 $args"]
}


# Procedure: his_entropy
proc his_entropy { args} {
#xf ignore me 4
global topwin
return [eval "$topwin(analysis_name).info.frame16.upper.checkbutton5 $args"]
}


# Procedure: his_mean
proc his_mean { args} {
#xf ignore me 4
global topwin
return [eval "$topwin(analysis_name).info.frame16.upper.checkbutton22 $args"]
}


# Procedure: his_skew
proc his_skew { args} {
#xf ignore me 4
global topwin
return [eval "$topwin(analysis_name).info.frame16.upper.checkbutton24 $args"]
}


# Procedure: num_rings
proc num_rings { args} {
#xf ignore me 4
global topwin
return [eval "$topwin(analysis_name).info.frame17.frame17.frame.frame1.entry $args"]
}


# Procedure: num_sectors
proc num_sectors { args} {
#xf ignore me 4
global topwin
return [eval "$topwin(analysis_name).info.frame17.frame17.frame2.frame1.entry $args"]
}


# Procedure: phi1
proc phi1 { args} {
#xf ignore me 4
global topwin
return [eval "$topwin(analysis_name).info.frame18.checkbutton2 $args"]
}


# Procedure: phi2
proc phi2 { args} {
#xf ignore me 4
global topwin
return [eval "$topwin(analysis_name).info.frame18.checkbutton3 $args"]
}


# Procedure: phi3
proc phi3 { args} {
#xf ignore me 4
global topwin
return [eval "$topwin(analysis_name).info.frame18.checkbutton4 $args"]
}


# Procedure: phi4
proc phi4 { args} {
#xf ignore me 4
global topwin
return [eval "$topwin(analysis_name).info.frame18.checkbutton5 $args"]
}


# Procedure: phi5
proc phi5 { args} {
#xf ignore me 4
global topwin
return [eval "$topwin(analysis_name).info.frame18.checkbutton6 $args"]
}


# Procedure: phi6
proc phi6 { args} {
#xf ignore me 4
global topwin
return [eval "$topwin(analysis_name).info.frame18.checkbutton7 $args"]
}


# Procedure: phi7
proc phi7 { args} {
#xf ignore me 4
global topwin
return [eval "$topwin(analysis_name).info.frame18.checkbutton8 $args"]
}


# Procedure: texture_0
proc texture_0 { args} {
#xf ignore me 4
global topwin
return [eval "$topwin(analysis_name).info.frame19.checkbutton11 $args"]
}


# Procedure: texture_135
proc texture_135 { args} {
#xf ignore me 4
global topwin
return [eval "$topwin(analysis_name).info.frame19.checkbutton14 $args"]
}


# Procedure: texture_45
proc texture_45 { args} {
#xf ignore me 4
global topwin
return [eval "$topwin(analysis_name).info.frame19.checkbutton12 $args"]
}


# Procedure: texture_90
proc texture_90 { args} {
#xf ignore me 4
global topwin
return [eval "$topwin(analysis_name).info.frame19.checkbutton13 $args"]
}


# Procedure: texture_avg
proc texture_avg { args} {
#xf ignore me 4
global topwin
return [eval "$topwin(analysis_name).info.frame19.checkbutton15 $args"]
}


