# **********************************************************************
# Computer Vision/Image Processing Tool Project - Dr. Scott Umbaugh SIUE
# **********************************************************************
#
#             File Name: ana_segmentation.tcl
#           Description: 
#         Related Files: 
#   Initial Coding Date: Wed Jul 17 17:57:57 CDT 1996
#           Portability: TCL script file
#            References:
#             Author(s): Yansheng Wei
#                        Southern Illinois University @ Edwardsville
#
## Copyright (C) 1996 SIUE - by Scott Umbaugh and Yansheng Wei.
##
## Permission to use, copy, modify, and distribute this software and its
## documentation for any purpose and without fee is hereby granted, provided
## that the above copyright notice appear in all copies and that both that
## copyright notice and this permission notice appear in supporting
## documentation.  This software is provided "as is" without express or
## implied warranty.
##
# RCS (Revision Control System) Information
# ...Added automatically by RCS 
#
## $Log: ana_segmentation.tcl,v $
## Revision 1.45  1997/08/07 20:11:36  yawei
## set upd efault for morph
##
## Revision 1.44  1997/07/25 17:57:35  wzheng
## raised checkbutton
##
## Revision 1.42  1997/07/09 21:16:39  yawei
## fixed flash problem
##
## Revision 1.41  1997/06/25 03:55:38  yawei
## removed trailing code
##
## Revision 1.40  1997/06/23 21:54:39  yawei
## *** empty log message ***
##
## Revision 1.39  1997/06/23 20:52:52  yawei
## fixed PCM_MEDIAN braces problem
##
## Revision 1.38  1997/06/19 22:03:42  yawei
## added initial position setup
## fixed update problem
##
## Revision 1.37  1997/06/19 19:52:19  yawei
## added debug control
##
## Revision 1.36  1997/06/18 21:03:01  yawei
## win size configure
##
## Revision 1.35  1997/06/18 20:49:40  yawei
## everything fixed up, except for the warning message of texture doesn't
## show up
##
## Revision 1.34  1997/06/06 20:47:09  yawei
## anal ==> analysis
##
## Revision 1.33  1997/04/07 23:09:02  yawei
## checked the texture distance to be positive integer
##
## Revision 1.32  1997/03/11 23:05:14  yawei
## 1) fixed size problem
## 2) fixed arrow problem
##
## Revision 1.31  1997/03/11 20:55:24  yawei
## Tcl/Tk script upgrading from tcl7.3/tk3.6 to tcl7.6/tk4.2
## compatible. This process is automated with tkboost perl
## script. @ Tue Mar 11 14:51:36 CST 1997
##
## Revision 1.30  1997/01/06 16:06:22  yawei
## changed pct/median
##
# Revision 1.29  1996/12/19  22:29:46  mzuke
# modified PCT/Median to NOT convert to grayscale
#
# Revision 1.28  1996/11/18  13:31:23  yawei
# Added igs to gray_level quantization
#
# Revision 1.27  1996/11/12  21:14:22  yawei
# Removed the boundary check for percentage/pix dist for S&M and MuilRes temporarily.
# Wait for further testing
#
# Revision 1.26  1996/11/12  15:46:14  yawei
# Changed default for hist_stretch to 0.025
#
# Revision 1.25  1996/11/08  17:49:23  yawei
# added default "50 50" to hist_stretch
#
# Revision 1.24  1996/11/06  03:31:16  yawei
# restore boundary checking for the pixel distance and pertance in split_merge
#
# Revision 1.23  1996/10/15  12:57:22  mzuke
# moved 'view image' after 'image_post' to correctly label
# displayed images
#
# Revision 1.22  1996/09/18  18:38:06  yawei
# changed cursor to clock in execution
#
# Revision 1.21  1996/09/17  17:52:15  yawei
# 1) changed "Gray level image only" functions to work on color and gray images.
# 2) changed "color image only" functions to work on color and gray images.
# 3) removed "Color or gray level image" message.
# 4) Move some text to the radiobutton "text" so the button can be selected by
#    pushing the adjacent text.
#
# Revision 1.20  1996/09/03  18:04:32  yawei
# changed anal_ to analysis_
#
# Revision 1.19  1996/08/29  18:51:59  mzuke
#  changed font designation from {symbol} to full listing
# to restore arrow labels after systemwide font changes
#
# Revision 1.18  1996/08/28  22:19:16  yawei
# check in with the changes in 3.0
#
# Revision 1.18  1996/08/28  20:35:31  yawei
# added width (cross size) to the cross option in morphological filters
#
# Revision 1.17  1996/08/23  23:39:25  yawei
# Changed the fourth parameter of "cross" so the width and height will take the same number
#
# Revision 1.16  1996/08/22  23:42:18  yawei
# added type, height, width to morphological filters
#
# Revision 1.15  1996/08/16  16:02:05  mzuke
# corrected spelling of 'homogeneity'
#
# Revision 1.14  1996/08/15  23:00:14  yawei
# changed the entry labels to S&M M&R, as
# If we select 1-3, the threshold/similarity and percentage/pix-distance
# entries will be disabled;
#
# If we select 4, the labels will be (Threshold, percentage)
# If we select 5, the lavel will be (Threshold,  disabled)
# If we select 6, the level will be (Similarity, pix distance)
#
# Revision 1.13  1996/08/15  11:01:23  mzuke
# removed toplevel_position from wm geometry to allow window
# positioning by user
#
# Revision 1.12  1996/08/12  13:59:50  yawei
# changed the S&M.
# 1) for option_4, it is threshold and percent
# 2) for option_6, it is similarity and pix distance.
# 3) the upper boundary for percent/pix distance is 1.0/100
#
# Revision 1.11  1996/08/09  19:02:29  yawei
# changed boundary limits of S&M and multi_res.
# percetn: 0-2.0
# thresh: 0 - 1000
#
# Revision 1.10  1996/08/05  17:06:55  yawei
# changed multi_resolution and split_merge to final shape.
#
# Revision 1.9  1996/08/05  15:54:03  yawei
# corrected the problem with {} in morpho of segmentation.
#
# Revision 1.8  1996/08/01  14:06:48  yawei
# set default value (3) for the kernel dimension of morphological filters.
#
# Revision 1.7  1996/07/31  16:52:33  yawei
# 1) Add additional parameters to multi_resolution and split_merge
# 2) cases 1,2,3,5 don't work in multi_resolution, waiting for changes in cviptcl
# 3) cases 4,5,6 don't work in split_merge, waiting changes in cviptcl
#
# Revision 1.6  1996/07/25  13:46:33  yawei
# 1) Interactive --> Iterative
# 2) iteration : 1-99 --> 0-99
#
# Revision 1.5  1996/07/23  17:29:31  yawei
# modify PCT/Median so it works.
#
# Revision 1.4  1996/07/23  17:16:00  yawei
# set the default value of gray level quantization to 4, instead of 3
#
# Revision 1.3  1996/07/23  17:09:46  yawei
# segmentation works now. Boundary checks are present for all parameters except:
#  1. number of colors A, B in SCT/Center
#  2. Kernel dimension for morphological filters
#
# Revision 1.2  1996/07/22  16:20:07  yawei
# 1) make the main header say" Analysis-> Segmentation"
# 2) swap perform and dismiss button location
# 3) "A axix" = "A axis"
# 4) "Locat mean..." = "Local mean..."
# 5) Under clustering include "Median Cut"
# 6) remove "Clustering" level
#
# Revision 1.1  1996/07/17  23:00:55  yawei
# Initial revision
#
#
#****************************************************************************

# procedure to show window $topwin(analysis_name)
proc segmentation { } {# xf ignore me 7
  # Window manager configurations
  global tk_version
  global topwin
  global toplevel_position
  global {GuiColor}
  global {GuiMisc}
  global {GUI_DEBUG}
  global {morph}

  if [ winfo exists $topwin(analysis_name).info] {
     set tempname $topwin(analysis_name);\
destroy $topwin(analysis_name).info;\
set topwin(analysis_name) $tempname;}
  if [ winfo exists $topwin(analysis_name).current_image] {
     set tempname $topwin(analysis_name);\
destroy $topwin(analysis_name).current_image;\
set topwin(analysis_name) $tempname;}
  if [ winfo exists $topwin(analysis_name).perform] {
     set tempname $topwin(analysis_name);\
destroy $topwin(analysis_name).perform;\
set topwin(analysis_name) $tempname;}
  wm positionfrom $topwin(analysis_name) user
  wm sizefrom $topwin(analysis_name) user

  wm geometry $topwin(analysis_name) $topwin(analysis_segment)[get_pos $topwin(analysis_name)]
  wm maxsize $topwin(analysis_name) [lindex $topwin(analysis_segment_max) 0] \
  	[lindex $topwin(analysis_segment_max) 1]
  wm minsize $topwin(analysis_name) [lindex $topwin(analysis_segment_min) 0] \
  	[lindex $topwin(analysis_segment_min) 1]
  wm title $topwin(analysis_name) {ANALYSIS -> Segmentation}

  # bindings
  bind $topwin(analysis_name) <Destroy> {set topwin(analysis_name) NONE}

  # build widget $topwin(analysis_name).info
  frame $topwin(analysis_name).info \
    -background "$GuiColor(frame_bg)"

  # build widget $topwin(analysis_name).info.frame6
  frame $topwin(analysis_name).info.frame6 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2} \
    -relief {ridge}

  # build widget $topwin(analysis_name).info.frame6.frame14
  frame $topwin(analysis_name).info.frame6.frame14 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  set analysis(multi_option) 1
  # build widget $topwin(analysis_name).info.frame6.frame14.radiobutton1
	radiobutton $topwin(analysis_name).info.frame6.frame14.radiobutton1 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -command {
			if {$bands != 3} {
				set analysis(multi_pct) 0
			   $topwin(analysis_name).info.frame6.frame0.checkbutton12 configure -state disabled
			} else {
			   $topwin(analysis_name).info.frame6.frame0.checkbutton12 configure -state normal
			}
			if { $analysis(multi_option) == "" } {
			   set analysis(multi_option) 1
            set analysis(multi_percent) "-1"; set analysis(multi_thresh) "-1"
            pack forget $topwin(analysis_name).info.frame6.frame0.frame11
            pack forget $topwin(analysis_name).info.frame6.frame0.frame10
         } elseif {$analysis(multi_option) == "5" } {
            set analysis(multi_thresh) "-1"
            if { $analysis(multi_thresh) == "" || $analysis(multi_thresh) == "-1" } {set analysis(multi_thresh) "50"}
				pack forget $topwin(analysis_name).info.frame6.frame0.frame11 
            pack forget $topwin(analysis_name).info.frame6.frame0.frame10 
            pack configure $topwin(analysis_name).info.frame6.frame0.frame10 \
					-side right \
					-padx 20
         } elseif { $analysis(multi_option) == "4" || $analysis(multi_option) == "6" } {
            if { $analysis(multi_percent) == "" || $analysis(multi_percent) == "-1" } {set analysis(multi_percent) "0.70"}
            if { $analysis(multi_thresh) == "" || $analysis(multi_thresh) == "-1" } {set analysis(multi_thresh) "50"}
				pack forget $topwin(analysis_name).info.frame6.frame0.frame11 
				pack configure $topwin(analysis_name).info.frame6.frame0.frame11 \
					-side right \
					-padx 20
            pack forget $topwin(analysis_name).info.frame6.frame0.frame10 
            pack configure $topwin(analysis_name).info.frame6.frame0.frame10 \
					-side right \
					-padx 20
         } else {
            set analysis(multi_percent) "-1"; set analysis(multi_thresh) "-1"
            pack forget $topwin(analysis_name).info.frame6.frame0.frame11
            pack forget $topwin(analysis_name).info.frame6.frame0.frame10
         }		
			set analysis(sam_option) ""; set analysis(morph_choice) "" } \
    -relief {flat} \
    -text {Multi_Resolution} \
    -value {multi_resolution} \
    -variable {analysis(seg_choice)}

  # pack widget $topwin(analysis_name).info.frame6.frame14
  pack append $topwin(analysis_name).info.frame6.frame14 \
    $topwin(analysis_name).info.frame6.frame14.radiobutton1 {left frame center} 

  # build widget $topwin(analysis_name).info.frame6.frame3
  frame $topwin(analysis_name).info.frame6.frame3 \
    -background "$GuiColor(frame_bg)" 

  # build widget $topwin(analysis_name).info.frame6.frame3.frame12
  frame $topwin(analysis_name).info.frame6.frame3.frame12 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2} \
    -width {30} -height {0}

  # build widget $topwin(analysis_name).info.frame6.frame3.frame14
  frame $topwin(analysis_name).info.frame6.frame3.frame14 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame6.frame3.frame14.frame17
  frame $topwin(analysis_name).info.frame6.frame3.frame14.frame17 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame6.frame3.frame14.frame17.radiobutton7
	radiobutton $topwin(analysis_name).info.frame6.frame3.frame14.frame17.radiobutton7 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -relief {flat} \
    -text {Local mean vs. global mean } \
    -value {2} \
	 -command {
            set analysis(multi_percent) "-1"; set analysis(multi_thresh) "-1"
            pack forget $topwin(analysis_name).info.frame6.frame0.frame11
            pack forget $topwin(analysis_name).info.frame6.frame0.frame10
	 } \
    -variable {analysis(multi_option)}

  # build widget $topwin(analysis_name).info.frame6.frame3.frame14.frame17.radiobutton12
	radiobutton $topwin(analysis_name).info.frame6.frame3.frame14.frame17.radiobutton12 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -relief {flat} \
    -text {Weighted gray level distance} \
    -value {5} \
	 -command {
            set analysis(multi_percent) "-1"
				if { $analysis(multi_thresh) == "" || $analysis(multi_thresh) == "-1" } {set analysis(multi_thresh) "50"}
				pack forget $topwin(analysis_name).info.frame6.frame0.frame11 
            pack configure $topwin(analysis_name).info.frame6.frame0.frame10 \
					-side right \
					-padx 20
				$topwin(analysis_name).info.frame6.frame0.frame10.label1 configure -text "Threshold: "
	 } \
    -variable {analysis(multi_option)}

  # pack widget $topwin(analysis_name).info.frame6.frame3.frame14.frame17
  pack append $topwin(analysis_name).info.frame6.frame3.frame14.frame17 \
    $topwin(analysis_name).info.frame6.frame3.frame14.frame17.radiobutton7 {top frame w filly} \
    $topwin(analysis_name).info.frame6.frame3.frame14.frame17.radiobutton12 {top frame w filly}

  # build widget $topwin(analysis_name).info.frame6.frame3.frame14.frame18
  frame $topwin(analysis_name).info.frame6.frame3.frame14.frame18 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame6.frame3.frame14.frame18.radiobutton7
	radiobutton $topwin(analysis_name).info.frame6.frame3.frame14.frame18.radiobutton7 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -relief {flat} \
    -text {Local std. vs. global mean} \
    -value {3} \
	 -command {
            set analysis(multi_percent) "-1"; set analysis(multi_thresh) "-1"
            pack forget $topwin(analysis_name).info.frame6.frame0.frame11
            pack forget $topwin(analysis_name).info.frame6.frame0.frame10
	 } \
    -variable {analysis(multi_option)}

  # build widget $topwin(analysis_name).info.frame6.frame3.frame14.frame18.radiobutton13
	radiobutton $topwin(analysis_name).info.frame6.frame3.frame14.frame18.radiobutton13 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -relief {flat} \
    -text {Texture homogeneity} \
    -value {6} \
    -command {
            if { $analysis(multi_percent) == "" || $analysis(multi_percent) == "-1" } {
					set analysis(multi_percent) "2"
				} else {
					set analysis(multi_percent) [expr int($analysis(multi_percent))]
					if { $analysis(multi_percent) <= "0" } {set analysis(multi_percent) 2}
				}

            if { $analysis(multi_thresh) == "" || $analysis(multi_thresh) == "-1" } {set analysis(multi_thresh) "50"}
				pack forget $topwin(analysis_name).info.frame6.frame0.frame11 
				pack configure $topwin(analysis_name).info.frame6.frame0.frame11 \
					-side right \
					-padx 20
            pack forget $topwin(analysis_name).info.frame6.frame0.frame10 
            pack configure $topwin(analysis_name).info.frame6.frame0.frame10 \
					-side right \
					-padx 20
				$topwin(analysis_name).info.frame6.frame0.frame10.label1 configure -text "Similarity: "
				$topwin(analysis_name).info.frame6.frame0.frame11.label1 configure -text "Pix distance: "
    } \
    -variable {analysis(multi_option)}

  # pack widget $topwin(analysis_name).info.frame6.frame3.frame14.frame18
  pack append $topwin(analysis_name).info.frame6.frame3.frame14.frame18 \
    $topwin(analysis_name).info.frame6.frame3.frame14.frame18.radiobutton7 {top frame nw filly} \
    $topwin(analysis_name).info.frame6.frame3.frame14.frame18.radiobutton13 {top frame nw filly}

  # build widget $topwin(analysis_name).info.frame6.frame3.frame14.frame16
  frame $topwin(analysis_name).info.frame6.frame3.frame14.frame16 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame6.frame3.frame14.frame16.radiobutton7
	radiobutton $topwin(analysis_name).info.frame6.frame3.frame14.frame16.radiobutton7 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -relief {flat} \
    -text {Pure uniformity} \
    -value {1} \
	 -command {
            set analysis(multi_percent) "-1"; set analysis(multi_thresh) "-1"
            pack forget $topwin(analysis_name).info.frame6.frame0.frame11
            pack forget $topwin(analysis_name).info.frame6.frame0.frame10
	 } \
    -variable {analysis(multi_option)}

  # build widget $topwin(analysis_name).info.frame6.frame3.frame14.frame16.radiobutton14
	radiobutton $topwin(analysis_name).info.frame6.frame3.frame14.frame16.radiobutton14 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -relief {flat} \
    -text {Variance} \
    -value {4} \
    -command {
            if { $analysis(multi_percent) == "" || $analysis(multi_percent) == "-1" } {set analysis(multi_percent) "0.70"}
            if { $analysis(multi_thresh) == "" || $analysis(multi_thresh) == "-1" } {set analysis(multi_thresh) "50"}
				pack forget $topwin(analysis_name).info.frame6.frame0.frame11 
				pack configure $topwin(analysis_name).info.frame6.frame0.frame11 \
					-side right \
					-padx 20
            pack forget $topwin(analysis_name).info.frame6.frame0.frame10 
            pack configure $topwin(analysis_name).info.frame6.frame0.frame10 \
					-side right \
					-padx 20
				$topwin(analysis_name).info.frame6.frame0.frame10.label1 configure -text "Threshold: "
				$topwin(analysis_name).info.frame6.frame0.frame11.label1 configure -text "Percentage: "
    } \
    -variable {analysis(multi_option)}

  # pack widget $topwin(analysis_name).info.frame6.frame3.frame14.frame16
  pack append $topwin(analysis_name).info.frame6.frame3.frame14.frame16 \
    $topwin(analysis_name).info.frame6.frame3.frame14.frame16.radiobutton7 {top frame nw filly} \
    $topwin(analysis_name).info.frame6.frame3.frame14.frame16.radiobutton14 {top frame nw filly}

  # pack widget $topwin(analysis_name).info.frame6.frame3.frame14
  pack append $topwin(analysis_name).info.frame6.frame3.frame14 \
    $topwin(analysis_name).info.frame6.frame3.frame14.frame16 {left frame center expand fillx} \
    $topwin(analysis_name).info.frame6.frame3.frame14.frame17 {left frame center expand fillx} \
    $topwin(analysis_name).info.frame6.frame3.frame14.frame18 {left frame center expand fillx}

  # pack widget $topwin(analysis_name).info.frame6.frame3
  pack append $topwin(analysis_name).info.frame6.frame3 \
    $topwin(analysis_name).info.frame6.frame3.frame12 {left frame center fill} \
    $topwin(analysis_name).info.frame6.frame3.frame14 {left frame center expand fill}

  # build widget $topwin(analysis_name).info.frame6.frame0
  frame $topwin(analysis_name).info.frame6.frame0 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame6.frame0.checkbutton12
	checkbutton $topwin(analysis_name).info.frame6.frame0.checkbutton12 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -relief {raised} \
    -text {PCT} \
    -variable {analysis(multi_pct)} \
    -width {5}

  # build widget $topwin(analysis_name).info.frame6.frame0.frame10
  frame $topwin(analysis_name).info.frame6.frame0.frame10 \
    -background "$GuiColor(frame_bg)" \
	 -highlightthickness {0} \
    -borderwidth {0}

  # build widget $topwin(analysis_name).info.frame6.frame0.frame10.label1
  label $topwin(analysis_name).info.frame6.frame0.frame10.label1 \
    -background "$GuiColor(frame_bg)" \
	 -highlightthickness {0} \
    -borderwidth {0} \
    -text {Threshold: }

  # build widget $topwin(analysis_name).info.frame6.frame0.frame10.value
  entry $topwin(analysis_name).info.frame6.frame0.frame10.value \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
	 -highlightthickness {0} \
    -relief {sunken} \
    -textvariable {analysis(multi_thresh)} \
    -width {4}
  endEntry $topwin(analysis_name).info.frame6.frame0.frame10.value 0 \
  	$topwin(analysis_name).info.frame6.frame0.frame11.value \
  	$topwin(analysis_name).info.frame6.frame0.frame11.value

  # pack widget $topwin(analysis_name).info.frame6.frame0.frame10
  pack append $topwin(analysis_name).info.frame6.frame0.frame10 \
    $topwin(analysis_name).info.frame6.frame0.frame10.label1 {left frame center fill} \
    $topwin(analysis_name).info.frame6.frame0.frame10.value {left frame center fill}

  # build widget $topwin(analysis_name).info.frame6.frame0.frame11
  frame $topwin(analysis_name).info.frame6.frame0.frame11 \
	 -highlightthickness {0} \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {0}

  # build widget $topwin(analysis_name).info.frame6.frame0.frame11.label1
  label $topwin(analysis_name).info.frame6.frame0.frame11.label1 \
	 -highlightthickness {0} \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -text {Percentage: }

  # build widget $topwin(analysis_name).info.frame6.frame0.frame11.value
  entry $topwin(analysis_name).info.frame6.frame0.frame11.value \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
	 -highlightthickness {0} \
    -relief {sunken} \
    -textvariable {analysis(multi_percent)} \
    -width {4}
  endEntry $topwin(analysis_name).info.frame6.frame0.frame11.value 0 \
  	$topwin(analysis_name).info.frame6.frame0.frame10.value \
  	$topwin(analysis_name).info.frame6.frame0.frame10.value 

  # pack widget $topwin(analysis_name).info.frame6.frame0.frame11
  pack append $topwin(analysis_name).info.frame6.frame0.frame11 \
    $topwin(analysis_name).info.frame6.frame0.frame11.label1 {left frame center fill} \
    $topwin(analysis_name).info.frame6.frame0.frame11.value {left frame center fill}

  # pack widget $topwin(analysis_name).info.frame6.frame0
  pack append $topwin(analysis_name).info.frame6.frame0 \
    $topwin(analysis_name).info.frame6.frame0.checkbutton12 {left frame center padx 100}

  # pack widget $topwin(analysis_name).info.frame6
  pack append $topwin(analysis_name).info.frame6 \
    $topwin(analysis_name).info.frame6.frame14 {top frame center expand fill} \
    $topwin(analysis_name).info.frame6.frame3 {top frame center expand fill} \
    $topwin(analysis_name).info.frame6.frame0 {top frame center expand fill}

  # build widget $topwin(analysis_name).info.frame14
  frame $topwin(analysis_name).info.frame14 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2} \
    -relief {ridge}

  # build widget $topwin(analysis_name).info.frame14.radiobutton1
	radiobutton $topwin(analysis_name).info.frame14.radiobutton1 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -command {set analysis(multi_option) ""; set analysis(sam_option) ""; set analysis(morph_choice) "" } \
    -relief {flat} \
    -text {Histogram Thresholding} \
    -value {hist_thresh} \
    -variable {analysis(seg_choice)}

  # pack widget $topwin(analysis_name).info.frame14
  pack append $topwin(analysis_name).info.frame14 \
    $topwin(analysis_name).info.frame14.radiobutton1 {left frame center} 

  # build widget $topwin(analysis_name).info.frame16
  frame $topwin(analysis_name).info.frame16 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2} \
    -relief {ridge}


  # build widget $topwin(analysis_name).info.frame16.frame1
  frame $topwin(analysis_name).info.frame16.frame1 \
    -borderwidth {0} \
    -highlightthickness {0} \
	 -background "$GuiColor(frame_bg)" \
    -height {30} \
    -width {30}

  # build widget $topwin(analysis_name).info.frame16.frame1.entry
  entry $topwin(analysis_name).info.frame16.frame1.entry \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
    -textvariable {analysis(fuzzyc_kernel)} -width {5}
  endEntry $topwin(analysis_name).info.frame16.frame1.entry

  # build widget $topwin(analysis_name).info.frame16.frame1.menubutton7
  menubutton $topwin(analysis_name).info.frame16.frame1.menubutton7 \
	 -activebackground "$GuiColor(btn_act_bg)" -background "$GuiColor(btn_bg)"  \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)" \
    -padx {1} -pady {1} -highlightthickness {0} \
    -text "$GuiMisc(pop_up_text)" -width {2}

  # bindings
  bind $topwin(analysis_name).info.frame16.frame1.menubutton7 <Button-1> { MenuPopupPost $topwin(analysis_name).info.frame16.frame1.menubutton7.menu0 %X %Y}
  bind $topwin(analysis_name).info.frame16.frame1.menubutton7 <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame16.frame1.menubutton7.menu0 %W }

  # build widget $topwin(analysis_name).info.frame16.frame1.menubutton7.menu0
  menu $topwin(analysis_name).info.frame16.frame1.menubutton7.menu0 
  $topwin(analysis_name).info.frame16.frame1.menubutton7.menu0 add command \
    -label { 0} \
	 -command {set analysis(fuzzyc_kernel) 0}
  $topwin(analysis_name).info.frame16.frame1.menubutton7.menu0 add command \
    -label { 2} \
	 -command {set analysis(fuzzyc_kernel) 2}
  $topwin(analysis_name).info.frame16.frame1.menubutton7.menu0 add command \
    -label { 4} \
	 -command {set analysis(fuzzyc_kernel) 4}
  $topwin(analysis_name).info.frame16.frame1.menubutton7.menu0 add command \
    -label { 6} \
	 -command {set analysis(fuzzyc_kernel) 6}
  $topwin(analysis_name).info.frame16.frame1.menubutton7.menu0 add command \
    -label {10} \
	 -command {set analysis(fuzzyc_kernel) 10}
  $topwin(analysis_name).info.frame16.frame1.menubutton7.menu0 add command \
    -label {12} \
	 -command {set analysis(fuzzyc_kernel) 12}
  $topwin(analysis_name).info.frame16.frame1.menubutton7.menu0 add command \
    -label {15} \
	 -command {set analysis(fuzzyc_kernel) 15}
  $topwin(analysis_name).info.frame16.frame1.menubutton7.menu0 add command \
    -label {20} \
	 -command {set analysis(fuzzyc_kernel) 20}

  bind $topwin(analysis_name).info.frame16.frame1.menubutton7.menu0 <B1-Motion> { MenuPopupMotion $topwin(analysis_name).info.frame16.frame1.menubutton7.menu0 %W %X %Y}
  bind $topwin(analysis_name).info.frame16.frame1.menubutton7.menu0 <ButtonRelease-1>  { MenuPopupRelease $topwin(analysis_name).info.frame16.frame1.menubutton7.menu0 %W }

  # pack master $topwin(analysis_name).info.frame16.frame1
  pack configure $topwin(analysis_name).info.frame16.frame1.menubutton7 \
    -fill both \
    -side right
  pack configure $topwin(analysis_name).info.frame16.frame1.entry \
    -fill both \
    -side left

  # build widget $topwin(analysis_name).info.frame16.radiobutton7
	radiobutton $topwin(analysis_name).info.frame16.radiobutton7 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -command {set analysis(multi_option) ""; set analysis(sam_option) ""
				  set analysis(morph_choice) ""; if {$analysis(fuzzyc_kernel) == ""} {set analysis(fuzzyc_kernel) 3}} \
    -relief {flat} \
    -text {Fuzzy C Mean, } \
    -value {fuzzyc} \
    -variable {analysis(seg_choice)}

  # build widget $topwin(analysis_name).info.frame16.label19
  label $topwin(analysis_name).info.frame16.label19 \
    -background "$GuiColor(label_bg)" \
    -text {with the Gaussian kernel variance (0 - 20.0): }

  # pack widget $topwin(analysis_name).info.frame16
  pack append $topwin(analysis_name).info.frame16 \
    $topwin(analysis_name).info.frame16.radiobutton7 {left frame center} \
    $topwin(analysis_name).info.frame16.label19 {left frame center} \
    $topwin(analysis_name).info.frame16.frame1 {left frame center}

  # build widget $topwin(analysis_name).info.frame2
  frame $topwin(analysis_name).info.frame2 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2} \
    -relief {ridge}

  # build widget $topwin(analysis_name).info.frame2.frame14
  frame $topwin(analysis_name).info.frame2.frame14 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  set analysis(sam_option) 1
  # build widget $topwin(analysis_name).info.frame2.frame14.radiobutton1
	radiobutton $topwin(analysis_name).info.frame2.frame14.radiobutton1 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -command {
		if {$bands != 3} {
			set analysis(sam_pct) 0
			$topwin(analysis_name).info.frame2.frame0.checkbutton12 configure -state disabled
		} else {
			$topwin(analysis_name).info.frame2.frame0.checkbutton12 configure -state normal
		}
		set analysis(multi_option) ""; 
		if { $analysis(sam_option) == "" } {
         set analysis(sam_option) 1
         set analysis(sam_percent) "-1"; set analysis(sam_thresh) "-1"
         pack forget $topwin(analysis_name).info.frame2.frame0.frame10
         pack forget $topwin(analysis_name).info.frame2.frame0.frame11
      } elseif { $analysis(sam_option) == "5" } {
         set analysis(sam_percent) "-1"; if { $analysis(sam_thresh) == ""  || $analysis(sam_thresh) == "-1" } {set analysis(sam_thresh) "50"}
         pack forget $topwin(analysis_name).info.frame2.frame0.frame11
         pack forget $topwin(analysis_name).info.frame2.frame0.frame10 
         pack configure $topwin(analysis_name).info.frame2.frame0.frame10 \
			-side {right} \
			-padx {20} 
      } elseif { $analysis(sam_option) == "4" || $analysis(sam_option) == "6" } {
         if { $analysis(sam_percent) == "" || $analysis(sam_percent) == "-1"} {set analysis(sam_percent) "0.70"}
         if { $analysis(sam_thresh) == "" || $analysis(sam_thresh) == "-1"} {set analysis(sam_thresh) "50"}
         pack forget $topwin(analysis_name).info.frame2.frame0.frame11
         pack forget $topwin(analysis_name).info.frame2.frame0.frame10 
         pack configure $topwin(analysis_name).info.frame2.frame0.frame11 \
			-side {right} \
			-padx {20} 
      } else {
         set analysis(sam_percent) "-1"; set analysis(sam_thresh) "-1"
         pack forget $topwin(analysis_name).info.frame2.frame0.frame10
         pack forget $topwin(analysis_name).info.frame2.frame0.frame11
      } \
		set analysis(morph_choice) "" } \
    -relief {flat} \
    -text {Split and Merge} \
    -value {split_merge} \
    -variable {analysis(seg_choice)}

  # pack widget $topwin(analysis_name).info.frame2.frame14
  pack append $topwin(analysis_name).info.frame2.frame14 \
    $topwin(analysis_name).info.frame2.frame14.radiobutton1 {left frame center} 

  # build widget $topwin(analysis_name).info.frame2.frame3
  frame $topwin(analysis_name).info.frame2.frame3 \
    -background "$GuiColor(frame_bg)"

  # build widget $topwin(analysis_name).info.frame2.frame3.frame12
  frame $topwin(analysis_name).info.frame2.frame3.frame12 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2} \
    -width {30} -height {0}

  # build widget $topwin(analysis_name).info.frame2.frame3.frame14
  frame $topwin(analysis_name).info.frame2.frame3.frame14 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame2.frame3.frame14.frame16
  frame $topwin(analysis_name).info.frame2.frame3.frame14.frame16 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame2.frame3.frame14.frame16.radiobutton7
	radiobutton $topwin(analysis_name).info.frame2.frame3.frame14.frame16.radiobutton7 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -command {
} \
    -relief {flat} \
    -text {Pure uniformity} \
    -value {1} \
	 -command {
          set analysis(sam_percent) "-1"; set analysis(sam_thresh) "-1"
    	    pack forget $topwin(analysis_name).info.frame2.frame0.frame10
     	    pack forget $topwin(analysis_name).info.frame2.frame0.frame11
	 } \
    -variable {analysis(sam_option)}

  # build widget $topwin(analysis_name).info.frame2.frame3.frame14.frame16.radiobutton15
	radiobutton $topwin(analysis_name).info.frame2.frame3.frame14.frame16.radiobutton15 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -command {
} \
    -relief {flat} \
    -text {Variance} \
    -value {4} \
	 -command {
         if { $analysis(sam_percent) == "" || $analysis(sam_percent) == "-1" } {set analysis(sam_percent) "0.70"}
         if { $analysis(sam_thresh) == "" || $analysis(sam_thresh) == "-1" } {set analysis(sam_thresh) "50"}
     	   pack forget $topwin(analysis_name).info.frame2.frame0.frame11 
     	   pack configure $topwin(analysis_name).info.frame2.frame0.frame11 \
			-side {right}\
			-padx {20}
    	   pack forget $topwin(analysis_name).info.frame2.frame0.frame10 
    	   pack configure $topwin(analysis_name).info.frame2.frame0.frame10 \
			-side {right}\
			-padx {20}

			$topwin(analysis_name).info.frame2.frame0.frame10.label1 configure -text "Threshold: "
			$topwin(analysis_name).info.frame2.frame0.frame11.label1 configure -text "Percentage: "
			set analysis(sam_percent_max) "2.0"
	 } \
    -variable {analysis(sam_option)}

  # pack widget $topwin(analysis_name).info.frame2.frame3.frame14.frame16
  pack append $topwin(analysis_name).info.frame2.frame3.frame14.frame16 \
    $topwin(analysis_name).info.frame2.frame3.frame14.frame16.radiobutton7 {top frame nw filly} \
    $topwin(analysis_name).info.frame2.frame3.frame14.frame16.radiobutton15 {top frame nw filly}

  # build widget $topwin(analysis_name).info.frame2.frame3.frame14.frame17
  frame $topwin(analysis_name).info.frame2.frame3.frame14.frame17 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame2.frame3.frame14.frame17.radiobutton7
	radiobutton $topwin(analysis_name).info.frame2.frame3.frame14.frame17.radiobutton7 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -relief {flat} \
    -text {Local mean vs. global mean} \
    -value {2} \
	 -command {
          set analysis(sam_percent) "-1"; set analysis(sam_thresh) "-1"
    	    pack forget $topwin(analysis_name).info.frame2.frame0.frame10
     	    pack forget $topwin(analysis_name).info.frame2.frame0.frame11
	 } \
    -variable {analysis(sam_option)}

  # build widget $topwin(analysis_name).info.frame2.frame3.frame14.frame17.radiobutton16
	radiobutton $topwin(analysis_name).info.frame2.frame3.frame14.frame17.radiobutton16 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -relief {flat} \
    -text {Weighted gray level distance} \
    -value {5} \
	 -command {
         set analysis(sam_percent) "-1"
         if { $analysis(sam_thresh) == "" || $analysis(sam_thresh) == "-1" } {set analysis(sam_thresh) "50"}
     	   pack forget $topwin(analysis_name).info.frame2.frame0.frame11 
    	   pack forget $topwin(analysis_name).info.frame2.frame0.frame10 
    	   pack configure $topwin(analysis_name).info.frame2.frame0.frame10 \
			-side {right}\
			-padx {20}
			$topwin(analysis_name).info.frame2.frame0.frame10.label1 configure -text "Threshold: "
	 } \
    -variable {analysis(sam_option)}

  # pack widget $topwin(analysis_name).info.frame2.frame3.frame14.frame17
  pack append $topwin(analysis_name).info.frame2.frame3.frame14.frame17 \
    $topwin(analysis_name).info.frame2.frame3.frame14.frame17.radiobutton7 {top frame nw filly} \
    $topwin(analysis_name).info.frame2.frame3.frame14.frame17.radiobutton16 {top frame nw filly}

  # build widget $topwin(analysis_name).info.frame2.frame3.frame14.frame18
  frame $topwin(analysis_name).info.frame2.frame3.frame14.frame18 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame2.frame3.frame14.frame18.radiobutton7
	radiobutton $topwin(analysis_name).info.frame2.frame3.frame14.frame18.radiobutton7 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -relief {flat} \
    -text {Local std. vs. global mean} \
    -value {3} \
	 -command {
          set analysis(sam_percent) "-1"; set analysis(sam_thresh) "-1"
    	    pack forget $topwin(analysis_name).info.frame2.frame0.frame10
     	    pack forget $topwin(analysis_name).info.frame2.frame0.frame11
	 } \
    -variable {analysis(sam_option)}

  # build widget $topwin(analysis_name).info.frame2.frame3.frame14.frame18.radiobutton17
	radiobutton $topwin(analysis_name).info.frame2.frame3.frame14.frame18.radiobutton17 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -relief {flat} \
    -text {Texture homogeneity} \
    -value {6} \
	 -command {
         if { $analysis(sam_percent) == "" || $analysis(sam_percent) == "-1" } {
				set analysis(sam_percent) "2"
			} else {
				set analysis(sam_percent) [expr int($analysis(sam_percent))];
				if { $analysis(sam_percent)  <= "0" } {set analysis(sam_percent) 2}
			}
         if { $analysis(sam_thresh) == "" || $analysis(sam_thresh) == "-1" } {set analysis(sam_thresh) "50"}
     	   pack forget $topwin(analysis_name).info.frame2.frame0.frame11 
     	   pack configure $topwin(analysis_name).info.frame2.frame0.frame11 \
			-side {right}\
			-padx {20}
    	   pack forget $topwin(analysis_name).info.frame2.frame0.frame10 
    	   pack configure $topwin(analysis_name).info.frame2.frame0.frame10 \
			-side {right}\
			-padx {20}
			$topwin(analysis_name).info.frame2.frame0.frame10.label1 configure -text "Similarity: "
			$topwin(analysis_name).info.frame2.frame0.frame11.label1 configure -text "Pix distance: "
			set analysis(sam_percent_max) "100"
	 } \
    -variable {analysis(sam_option)}

  # pack widget $topwin(analysis_name).info.frame2.frame3.frame14.frame18
  pack append $topwin(analysis_name).info.frame2.frame3.frame14.frame18 \
    $topwin(analysis_name).info.frame2.frame3.frame14.frame18.radiobutton7 {top frame nw filly} \
    $topwin(analysis_name).info.frame2.frame3.frame14.frame18.radiobutton17 {top frame nw filly}

  # pack widget $topwin(analysis_name).info.frame2.frame3.frame14
  pack append $topwin(analysis_name).info.frame2.frame3.frame14 \
    $topwin(analysis_name).info.frame2.frame3.frame14.frame16 {left frame center expand fill} \
    $topwin(analysis_name).info.frame2.frame3.frame14.frame17 {left frame center expand fill} \
    $topwin(analysis_name).info.frame2.frame3.frame14.frame18 {left frame center expand fill}

  # pack widget $topwin(analysis_name).info.frame2.frame3
  pack append $topwin(analysis_name).info.frame2.frame3 \
    $topwin(analysis_name).info.frame2.frame3.frame12 {left frame center fill} \
    $topwin(analysis_name).info.frame2.frame3.frame14 {left frame center expand fill}

  # build widget $topwin(analysis_name).info.frame2.frame0
  frame $topwin(analysis_name).info.frame2.frame0 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame2.frame0.checkbutton12
	checkbutton $topwin(analysis_name).info.frame2.frame0.checkbutton12 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -relief {raised} \
    -text {PCT} \
    -variable {analysis(sam_pct)} \
    -width {5}

  # build widget $topwin(analysis_name).info.frame2.frame0.frame0
  frame $topwin(analysis_name).info.frame2.frame0.frame0 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame2.frame0.frame0.label1
  label $topwin(analysis_name).info.frame2.frame0.frame0.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -text {Entry level: }

  # build widget $topwin(analysis_name).info.frame2.frame0.frame0.value
  entry $topwin(analysis_name).info.frame2.frame0.frame0.value \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
    -textvariable {analysis(sam_level)} \
    -width {4}
  endEntry $topwin(analysis_name).info.frame2.frame0.frame0.value 0 \
  	$topwin(analysis_name).info.frame2.frame0.frame10.value \
  	$topwin(analysis_name).info.frame2.frame0.frame11.value 

  # pack widget $topwin(analysis_name).info.frame2.frame0.frame0
  pack append $topwin(analysis_name).info.frame2.frame0.frame0 \
    $topwin(analysis_name).info.frame2.frame0.frame0.label1 {left frame center fill} \
    $topwin(analysis_name).info.frame2.frame0.frame0.value {left frame center fill}

  # build widget $topwin(analysis_name).info.frame2.frame0.frame10
  frame $topwin(analysis_name).info.frame2.frame0.frame10 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame2.frame0.frame10.label1
  label $topwin(analysis_name).info.frame2.frame0.frame10.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
	 -width {10} \
    -text {Threshold: }

  # build widget $topwin(analysis_name).info.frame2.frame0.frame10.value
  entry $topwin(analysis_name).info.frame2.frame0.frame10.value \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
    -textvariable {analysis(sam_thresh)} \
    -width {4}
  endEntry $topwin(analysis_name).info.frame2.frame0.frame10.value 0 \
  	$topwin(analysis_name).info.frame2.frame0.frame11.value \
  	$topwin(analysis_name).info.frame2.frame0.frame0.value 

  # pack widget $topwin(analysis_name).info.frame2.frame0.frame10
  pack append $topwin(analysis_name).info.frame2.frame0.frame10 \
    $topwin(analysis_name).info.frame2.frame0.frame10.label1 {left frame center fill} \
    $topwin(analysis_name).info.frame2.frame0.frame10.value {left frame center fill}

  # build widget $topwin(analysis_name).info.frame2.frame0.frame11
  frame $topwin(analysis_name).info.frame2.frame0.frame11 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame2.frame0.frame11.label1
  label $topwin(analysis_name).info.frame2.frame0.frame11.label1 \
    -background "$GuiColor(label_bg)" \
	 -width {14} \
    -borderwidth {0} \
    -text {Percentage: }

  # build widget $topwin(analysis_name).info.frame2.frame0.frame11.value
  entry $topwin(analysis_name).info.frame2.frame0.frame11.value \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
    -textvariable {analysis(sam_percent)} \
    -width {4}
  endEntry $topwin(analysis_name).info.frame2.frame0.frame11.value 0 \
  	$topwin(analysis_name).info.frame2.frame0.frame0.value \
  	$topwin(analysis_name).info.frame2.frame0.frame10.value 

  # pack widget $topwin(analysis_name).info.frame2.frame0.frame11
  pack append $topwin(analysis_name).info.frame2.frame0.frame11 \
    $topwin(analysis_name).info.frame2.frame0.frame11.label1 {left frame center fill} \
    $topwin(analysis_name).info.frame2.frame0.frame11.value {left frame center fill}

  # pack widget $topwin(analysis_name).info.frame2.frame0
  pack append $topwin(analysis_name).info.frame2.frame0 \
    $topwin(analysis_name).info.frame2.frame0.checkbutton12 {left frame center padx 100} \
    $topwin(analysis_name).info.frame2.frame0.frame0 {left frame center padx 10 fillx} 

  # pack widget $topwin(analysis_name).info.frame2
  pack append $topwin(analysis_name).info.frame2 \
    $topwin(analysis_name).info.frame2.frame14 {top frame center expand fill} \
    $topwin(analysis_name).info.frame2.frame3 {top frame center expand fill} \
    $topwin(analysis_name).info.frame2.frame0 {top frame center expand fill}

  # build widget $topwin(analysis_name).info.frame4
  frame $topwin(analysis_name).info.frame4 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2} \
    -relief {ridge}

  # build widget $topwin(analysis_name).info.frame4.frame14
  frame $topwin(analysis_name).info.frame4.frame14 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame4.frame14.radiobutton1
	radiobutton $topwin(analysis_name).info.frame4.frame14.radiobutton1 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -command {set analysis(multi_option) ""; set analysis(sam_option) ""
	 if {$analysis(morph_choice) == "" } {set analysis(morph_choice) morph_close 
					if { $analysis(kernel_dim) == "" } {set analysis(kernel_dim) 3}
					if { $analysis(kernel_height) == "" } {set analysis(kernel_height) 3}
					if { $analysis(kernel_width) == "" } {set analysis(kernel_width) 3}
					pack forget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20
					pack append $topwin(analysis_name).info.frame4.frame3.frame14.frame17  \
					$topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19 {bottom frame center fillx}}} \
    -relief {flat} \
    -text {Morphological Filters} \
    -value {filter} \
    -variable {analysis(seg_choice)}

  # pack widget $topwin(analysis_name).info.frame4.frame14
  pack append $topwin(analysis_name).info.frame4.frame14 \
    $topwin(analysis_name).info.frame4.frame14.radiobutton1 {left frame center} 

  # build widget $topwin(analysis_name).info.frame4.frame3
  frame $topwin(analysis_name).info.frame4.frame3 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2} \
    -relief {ridge}

  # build widget $topwin(analysis_name).info.frame4.frame3.frame12
  frame $topwin(analysis_name).info.frame4.frame3.frame12 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2} \
    -width {30} -height {0}

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14
  frame $topwin(analysis_name).info.frame4.frame3.frame14 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2} 

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17
  frame $topwin(analysis_name).info.frame4.frame3.frame14.frame17 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2} 

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19
  frame $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {0} 

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19
  frame $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {0} 

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame4
  frame $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame4 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {0} 

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame4.frame0
  frame $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame4.frame0 \
    -background "$GuiColor(frame_bg)" -borderwidth {0}

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame4.frame0.label1
  label $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame4.frame0.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -text {Kernel type:}

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame4.frame0.value
  entry $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame4.frame0.value \
    -background "$GuiColor(rentry_bg)" \
    -foreground "$GuiColor(rentry_fg)" \
    -relief {sunken} \
    -state {disabled} \
    -textvariable {analysis(kernel_type)} \
    -width {8}

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame4.frame0.menubutton2
	menubutton $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame4.frame0.menubutton2 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)" \
    -text "$GuiMisc(pop_up_text)" -width {2} \
    -width {2}
  # bindings
  bind $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame4.frame0.menubutton2 <Button-1> {MenuPopupPost $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame4.frame0.menubutton2.menu %X %Y}
  bind $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame4.frame0.menubutton2 <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame4.frame0.menubutton2.menu %W}

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame4.frame0.menubutton2.menu
  menu $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame4.frame0.menubutton2.menu \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" 
  $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame4.frame0.menubutton2.menu add command \
    -command {
		set analysis(kernel_type) circle
		pack forget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame6
		pack forget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame7
		$topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame7.frame0.label1\
			configure -text "Width: "
		} \
    -label {circle}
  $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame4.frame0.menubutton2.menu add command \
    -command {
		set analysis(kernel_type) square
		pack append  $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19\
			$topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame6 \
			{left frame center padx 10 fillx}
		$topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame6.frame0.label1\
			configure -text "Side: "
		pack forget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame7
		$topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame7.frame0.label1\
			configure -text "Width: "
		} \
    -label {square}
  $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame4.frame0.menubutton2.menu add command \
    -command {
		pack append  $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19\
			$topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame6 \
				{left frame center fillx}\
			$topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame7\
				{left frame center fillx}
		$topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame6.frame0.label1\
			configure -text "Height: "
		$topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame7.frame0.label1\
			configure -text "Width: "
		set analysis(kernel_type) rectangle} \
    -label {rectangle}
  $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame4.frame0.menubutton2.menu add command \
    -command {
		pack append  $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19\
			$topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame6 \
			{left frame center fillx} \
			$topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame7 \
			{left frame center fillx} 
		$topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame6.frame0.label1\
			configure -text "Thickness: "
		$topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame7.frame0.label1\
			configure -text "Cross size: "
		set analysis(kernel_type) cross} \
    -label {cross}
  bindtags $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame4.frame0.menubutton2.menu {Menu all}
  # bindings
  bind $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame4.frame0.menubutton2.menu <B1-Motion> {MenuPopupMotion $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame4.frame0.menubutton2.menu0 %W %X %Y}
  bind $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame4.frame0.menubutton2.menu <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame4.frame0.menubutton2.menu0 %W}

  # pack widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame4.frame0
  pack append $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame4.frame0 \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame4.frame0.label1 {left frame center fill} \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame4.frame0.value {left frame center fill} \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame4.frame0.menubutton2 {left frame center fill}

  # pack widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame4
  pack append $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame4 \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame4.frame0 {top frame center fillx}

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame5
  frame $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame5 \
    -background "$GuiColor(frame_bg)" -borderwidth {0}

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame5.frame0
  frame $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame5.frame0 \
    -background "$GuiColor(frame_bg)" -borderwidth {0}

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame5.frame0.label1
  label $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame5.frame0.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -text {Kernel size: }

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame5.frame0.value
  entry $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame5.frame0.value \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
    -textvariable {analysis(kernel_dim)} \
    -width {4}
  endEntry $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame5.frame0.value 0 \
  	$topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame6.frame0.value \
  	$topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame7.frame0.value 


  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame5.frame0.menubutton2
	menubutton $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame5.frame0.menubutton2 -padx {1} -pady {5} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)" \
    -text "$GuiMisc(pop_up_text)" -width {2} \
    -width {2}
  # bindings
  bind $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame5.frame0.menubutton2 <Button-1> {MenuPopupPost $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame5.frame0.menubutton2.menu %X %Y}
  bind $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame5.frame0.menubutton2 <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame5.frame0.menubutton2.menu %W}

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame5.frame0.menubutton2.menu
  menu $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame5.frame0.menubutton2.menu \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" 
  $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame5.frame0.menubutton2.menu add command \
    -command {set analysis(kernel_dim) 3} \
    -label {  3  }
  $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame5.frame0.menubutton2.menu add command \
    -command {set analysis(kernel_dim) 5} \
    -label {  5  }
  $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame5.frame0.menubutton2.menu add command \
    -command {set analysis(kernel_dim) 7} \
    -label {  7 }
  $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame5.frame0.menubutton2.menu add command \
    -command {set analysis(kernel_dim) 9} \
    -label {  9 }
  $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame5.frame0.menubutton2.menu add command \
    -command {set analysis(kernel_dim) 11} \
    -label { 11 }
  $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame5.frame0.menubutton2.menu add command \
    -command {set analysis(kernel_dim) 13} \
    -label { 13 }
  bindtags $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame5.frame0.menubutton2.menu {Menu all}
  # bindings
  bind $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame5.frame0.menubutton2.menu <B1-Motion> {MenuPopupMotion $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame5.frame0.menubutton2.menu0 %W %X %Y}
  bind $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame5.frame0.menubutton2.menu <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame5.frame0.menubutton2.menu0 %W}

  # pack widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame5.frame0
  pack append $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame5.frame0 \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame5.frame0.label1 {left frame center fill} \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame5.frame0.value {left frame center fill} \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame5.frame0.menubutton2 {left frame center fill}

  # pack widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame5
  pack append $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame5 \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame5.frame0 {top frame center fillx}

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame6
  frame $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame6 \
    -background "$GuiColor(frame_bg)" -borderwidth {0}

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame6.frame0
  frame $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame6.frame0 \
    -background "$GuiColor(frame_bg)" -borderwidth {0}

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame6.frame0.label1
  label $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame6.frame0.label1 \
    -anchor {e} \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -text {Height: } \
    -width {10}

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame6.frame0.value
  entry $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame6.frame0.value \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
    -textvariable {analysis(kernel_height)} \
    -width {4}
  endEntry $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame6.frame0.value 0 \
  	$topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame7.frame0.value \
  	$topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame5.frame0.value 

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame6.frame0.menubutton2
	menubutton $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame6.frame0.menubutton2 -padx {1} -pady {5} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)" \
    -text "$GuiMisc(pop_up_text)" -width {2} \
    -width {2}
  # bindings
  bind $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame6.frame0.menubutton2 <Button-1> {MenuPopupPost $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame6.frame0.menubutton2.menu %X %Y}
  bind $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame6.frame0.menubutton2 <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame6.frame0.menubutton2.menu %W}

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame6.frame0.menubutton2.menu
  menu $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame6.frame0.menubutton2.menu \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" 
  $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame6.frame0.menubutton2.menu add command \
    -command {set analysis(kernel_height) 1} \
    -label {  1}
  $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame6.frame0.menubutton2.menu add command \
    -command {set analysis(kernel_height) 3} \
    -label {  3}
  $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame6.frame0.menubutton2.menu add command \
    -command {set analysis(kernel_height) 5} \
    -label {  5}
  $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame6.frame0.menubutton2.menu add command \
    -command {set analysis(kernel_height) 7} \
    -label {  7}
  $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame6.frame0.menubutton2.menu add command \
    -command {set analysis(kernel_height) 9} \
    -label {  9}
  $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame6.frame0.menubutton2.menu add command \
    -command {set analysis(kernel_height) 11} \
    -label { 11}
  bindtags $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame6.frame0.menubutton2.menu {Menu all}
  # bindings
  bind $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame6.frame0.menubutton2.menu <B1-Motion> {MenuPopupMotion $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame6.frame0.menubutton2.menu0 %W %X %Y}
  bind $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame6.frame0.menubutton2.menu <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame6.frame0.menubutton2.menu0 %W}

  # pack widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame6.frame0
  pack append $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame6.frame0 \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame6.frame0.label1 {left frame center fill} \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame6.frame0.value {left frame center fill} \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame6.frame0.menubutton2 {left frame center fill}

  # pack widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame6
  pack append $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame6 \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame6.frame0 {top frame center fillx}

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame7
  frame $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame7 \
    -background "$GuiColor(frame_bg)" -borderwidth {0}

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame7.frame0
  frame $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame7.frame0 \
    -background "$GuiColor(frame_bg)" -borderwidth {0}

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame7.frame0.label1
  label $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame7.frame0.label1 \
    -background "$GuiColor(frame_bg)" \
	 -width {12} \
	 -anchor {e} \
    -borderwidth {0} \
    -text {Width: }

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame7.frame0.value
  entry $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame7.frame0.value \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
    -textvariable {analysis(kernel_width)} \
    -width {4}
  endEntry $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame7.frame0.value 0 \
  	$topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame5.frame0.value  \
  	$topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame6.frame0.value 

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame7.frame0.menubutton2
	menubutton $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame7.frame0.menubutton2 -padx {1} -pady {5} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)" \
    -text "$GuiMisc(pop_up_text)" -width {2} \
    -width {2}
  # bindings
  bind $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame7.frame0.menubutton2 <Button-1> {MenuPopupPost $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame7.frame0.menubutton2.menu %X %Y}
  bind $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame7.frame0.menubutton2 <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame7.frame0.menubutton2.menu %W}

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame7.frame0.menubutton2.menu
  menu $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame7.frame0.menubutton2.menu \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" 
  $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame7.frame0.menubutton2.menu add command \
    -command {set analysis(kernel_width) 1} \
    -label {  1 }
  $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame7.frame0.menubutton2.menu add command \
    -command {set analysis(kernel_width) 3} \
    -label {  3 }
  $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame7.frame0.menubutton2.menu add command \
    -command {set analysis(kernel_width) 5} \
    -label {  5 }
  $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame7.frame0.menubutton2.menu add command \
    -command {set analysis(kernel_width) 7} \
    -label {  7 }
  $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame7.frame0.menubutton2.menu add command \
    -command {set analysis(kernel_width) 9} \
    -label {  9 }
  $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame7.frame0.menubutton2.menu add command \
    -command {set analysis(kernel_width) 11} \
    -label { 11 }
  bindtags $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame7.frame0.menubutton2.menu {Menu all}
  # bindings
  bind $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame7.frame0.menubutton2.menu <B1-Motion> {MenuPopupMotion $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame7.frame0.menubutton2.menu0 %W %X %Y}
  bind $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame7.frame0.menubutton2.menu <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame7.frame0.menubutton2.menu0 %W}

  # pack widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame7.frame0
  pack append $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame7.frame0 \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame7.frame0.label1 {left frame center} \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame7.frame0.value {left frame center} \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame7.frame0.menubutton2 {left frame center}

  # pack widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame7
  pack append $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame7 \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame7.frame0 {top frame center fillx}

  # pack widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19
  pack append $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19 \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame4 {left frame center fillx} \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame5 {left frame center fillx} \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame6 {left frame center fillx} \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19.frame7 {left frame center fillx}

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame12
  frame $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame12 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {0}

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame12.label19
  label $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame12.label19 \
    -background "$GuiColor(label_bg)" \
    -text {}

  # pack widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame12
  pack append $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame12 \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame12.label19 {top frame center expand fill}

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame8
  frame $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame8 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {0}

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame8.label19
  label $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame8.label19 \
    -background "$GuiColor(label_bg)" \
    -text {}

  # pack widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame8
  pack append $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame8 \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame8.label19 {top frame center expand fill}

  # pack widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19
  pack append $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19 \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame8 {top frame center expand fill} \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame19 {top frame center expand fillx} \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19.frame12 {top frame center expand fill}

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20
  frame $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20 \
    -background "$GuiColor(frame_bg)" -borderwidth {0}

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame12
  frame $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame12 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {0}

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame12.checkbutton10
	checkbutton $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame12.checkbutton10 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -relief {raised} \
    -text { 8} \
    -variable {morph(set.8)} \
    -width {5}

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame12.checkbutton11
	checkbutton $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame12.checkbutton11 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -relief {raised} \
    -text { 9} \
    -variable {morph(set.9)} \
    -width {5}

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame12.checkbutton12
	checkbutton $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame12.checkbutton12 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -relief {raised} \
    -text {10} \
    -variable {morph(set.10)} \
    -width {5}

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame12.checkbutton13
	checkbutton $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame12.checkbutton13 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -relief {raised} \
    -text {11} \
    -variable {morph(set.11)} \
    -width {5}

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame12.checkbutton14
	checkbutton $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame12.checkbutton14 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -relief {raised} \
    -text {12} \
    -variable {morph(set.12)} \
    -width {5}

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame12.checkbutton15
	checkbutton $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame12.checkbutton15 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -relief {raised} \
    -text {13} \
    -variable {morph(set.13)} \
    -width {5}

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame12.checkbutton16
	checkbutton $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame12.checkbutton16 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -relief {raised} \
    -text {14} \
    -variable {morph(set.14)} \
    -width {5}

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame12.button18
	button $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame12.button18 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -command {TextBox  "      0              0              1              1\n
  0       0      0       1      0       1      1       1\n
  0       0      0       0      0       0      0       0\n
      0              0              0              0\n
(1) E* = 1     (2) E* = 0      (3) E* = 0    (4) E* = 0\n
\n
\n
      1              1              1              0\n
  1       1      1       1      1       1      1       1\n
  1       0      1       0      1       1      0       0\n
      0              1              1              1\n
(5) E* = 0     (6) E* = 0      (7) E* = 1    (8) E* = -2\n
\n
\n
      1              1              1              0\n
  0       1      0       1      1       1      1       1\n
  1       0      0       0      0       0      0       0\n
      0              1              1              0\n
(9) E* = -1  (10) E* = -1    (11) E* = -1   (12) E* = -1\n
\n
\n
      0              1\n
  0       1      0       1\n
  1       0      1       0\n
      0              1\n
(13) E* = -1      (14) E* = -1\n
" "" 450x700 "Available Surrounds"} \
    -text {View Surrounds}

  # pack widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame12
  pack append $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame12 \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame12.button18 {left frame center} \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame12.checkbutton16 {right frame center padx 4} \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame12.checkbutton15 {right frame center padx 4} \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame12.checkbutton14 {right frame center padx 4} \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame12.checkbutton13 {right frame center padx 4} \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame12.checkbutton12 {right frame center padx 4} \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame12.checkbutton11 {right frame center padx 4} \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame12.checkbutton10 {right frame center padx 4}

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame0
  frame $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame0 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {0}

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame0.checkbutton10
	checkbutton $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame0.checkbutton10 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -relief {raised} \
    -text { 1} \
    -variable {morph(set.1)} \
    -width {5}

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame0.checkbutton11
	checkbutton $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame0.checkbutton11 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -relief {raised} \
    -text { 2} \
    -variable {morph(set.2)} \
    -width {5}

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame0.checkbutton12
	checkbutton $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame0.checkbutton12 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -relief {raised} \
    -text { 3} \
    -variable {morph(set.3)} \
    -width {5}

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame0.checkbutton13
	checkbutton $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame0.checkbutton13 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -relief {raised} \
    -text { 4} \
    -variable {morph(set.4)} \
    -width {5}

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame0.checkbutton14
	checkbutton $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame0.checkbutton14 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -relief {raised} \
    -text { 5} \
    -variable {morph(set.5)} \
    -width {5}

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame0.checkbutton15
	checkbutton $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame0.checkbutton15 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -relief {raised} \
    -text { 6} \
    -variable {morph(set.6)} \
    -width {5}

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame0.checkbutton16
	checkbutton $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame0.checkbutton16 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -relief {raised} \
    -text { 7} \
    -variable {morph(set.7)} \
    -width {5}

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame0.button10
	button $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame0.button10 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -command {
	 set temp 0
	 for {set i 1} {$i < 15} {incr i} {
    set temp [expr $temp + $morph(set.$i)]
	 }
	 if {$temp == 14} {
	 	for {set i 1} {$i < 15} {incr i} {
    	set morph(set.$i) 0
		 }
	 } else {
	 	for {set i 1} {$i < 15} {incr i} {
    	set morph(set.$i) 1
		 }
	 }
	 } \
    -relief {flat} \
    -text {Use surrounds:}

  # pack widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame0
  pack append $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame0 \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame0.checkbutton16 {right frame center padx 4} \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame0.checkbutton15 {right frame center padx 4} \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame0.checkbutton14 {right frame center padx 4} \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame0.checkbutton13 {right frame center padx 4} \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame0.checkbutton12 {right frame center padx 4} \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame0.checkbutton11 {right frame center padx 4} \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame0.checkbutton10 {right frame center padx 4} \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame0.button10 {left frame center}

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20
  frame $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20 \
    -background "$GuiColor(frame_bg)" -borderwidth {0}

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.label21
  label $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.label21 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -text {rotation (y/n):}

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.entry22
  entry $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.entry22 \
    -background "$GuiColor(rentry_bg)" \
    -foreground "$GuiColor(rentry_fg)" \
    -relief {sunken} \
    -textvariable {analysis(morph_rot)} \
    -state {disabled} \
    -width {2}
  # bindings
  endEntry $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.entry22 0

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button5
	menubutton $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button5 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)" \
    -text "$GuiMisc(pop_up_text)" -width {2} \
    -width {2}
  # bindings
  bind $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button5 <Button-1> {MenuPopupPost $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button5.menu %X %Y}
  bind $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button5 <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button5.menu %W}

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button5.menu
  menu $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button5.menu
  $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button5.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(morph_rot) y} \
    -label {Yes}
  $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button5.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(morph_rot) n} \
    -label {No}
  bindtags $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button5.menu {Menu all}
  # bindings
  bind $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button5.menu <B1-Motion> {MenuPopupMotion $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button5.menu0 %W %X %Y}
  bind $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button5.menu <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button5.menu0 %W}


  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.label22
  label $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.label22 \
    -background "$GuiColor(label_bg)" \
    -text {, boolean:}

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.entry23
  entry $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.entry23 \
    -background "$GuiColor(rentry_bg)" \
    -foreground "$GuiColor(rentry_fg)" \
    -relief {sunken} \
    -textvariable {morph(boolean.val)} \
	 -state {disabled} \
    -width {5}
  # bindings
  endEntry $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.entry23 0

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button0
	menubutton $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button0 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)" \
    -text "$GuiMisc(pop_up_text)" -width {2} \
    -width {2}
  # bindings
  bind $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button0 <Button-1> {MenuPopupPost $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button0.menu %X %Y}
  bind $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button0 <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button0.menu %W}

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button0.menu
  menu $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button0.menu
  $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button0.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set morph(boolean.val) "0"} \
    -label {   0}
  $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button0.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set morph(boolean.val) "!a"} \
    -label {  !a}
  $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button0.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set morph(boolean.val) "ab"} \
    -label {  ab}
  $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button0.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set morph(boolean.val) "a+b"} \
    -label { a+b}
  $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button0.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set morph(boolean.val) "a^b"} \
    -label { a^b}
  $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button0.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set morph(boolean.val) "(!a)b"} \
    -label {(!a)b}
  bindtags $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button0.menu {Menu all}
  # bindings
  bind $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button0.menu <B1-Motion> {MenuPopupMotion $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button0.menu %W %X %Y}
  bind $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button0.menu <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button0.menu %W}

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.label23
  label $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.label23 \
    -background "$GuiColor(label_bg)" \
    -text {, iteration (0-99):}

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.entry24
  entry $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.entry24 \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
    -textvariable {morph(iteration.val)} \
    -width {2}
  # bindings
  endEntry $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.entry24 0

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button2
	menubutton $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button2 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)" \
    -text "$GuiMisc(pop_up_text)" -width {2} \
    -width {2}
  # bindings
  bind $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button2 <Button-1> {MenuPopupPost $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button2.menu %X %Y}
  bind $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button2 <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button2.menu %W}

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.label24
  label $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.label24 \
    -background "$GuiColor(label_bg)" \
    -text {, subfields (1-3):}

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.entry25
  entry $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.entry25 \
    -background "$GuiColor(rentry_bg)" \
    -foreground "$GuiColor(rentry_fg)" \
    -relief {sunken} \
    -state {disabled} \
    -textvariable {morph(subfields.val)} \
    -width {2}
  # bindings
  endEntry $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.entry25 0


  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button2.menu
  menu $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button2.menu
  $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set morph(subfields.val) 1} \
    -label { 1 }
  $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set morph(subfields.val) 2} \
    -label { 2}
  $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set morph(subfields.val) 3} \
    -label { 3}
  bindtags $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button2.menu {Menu all}
  # bindings
  bind $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button2.menu <B1-Motion> {MenuPopupMotion $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button2.menu %W %X %Y}
  bind $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button2.menu <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button2.menu %W}

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button19
	menubutton $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button19 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)" \
    -text "$GuiMisc(pop_up_text)" -width {2} \
    -width {2}
  # bindings
  bind $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button19 <Button-1> {MenuPopupPost $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button19.menu %X %Y}
  bind $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button19 <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button19.menu %W}

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button19.menu
  menu $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button19.menu
  $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button19.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set morph(iteration.val) 0} \
    -font "$GuiMisc(pop_up_fn)" \
    -label { }
  $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button19.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set morph(iteration.val) 5} \
    -label { 5}
  $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button19.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set morph(iteration.val) 10} \
    -label {10}
  $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button19.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set morph(iteration.val) 20} \
    -label {20}
  $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button19.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set morph(iteration.val) 50} \
    -label {50}
  $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button19.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set morph(iteration.val) 99} \
    -label {99}
  bindtags $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button19.menu {Menu all}
  # bindings
  bind $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button19.menu <B1-Motion> {MenuPopupMotion $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button19.menu0 %W %X %Y}
  bind $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button19.menu <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button19.menu0 %W}

  # pack widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20
  pack append $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20 \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.label21 {left frame center fill} \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.entry22 {left frame center fill} \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button5 {left frame center fill} \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.label22 {left frame center fill} \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.entry23 {left frame center fill} \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button0 {left frame center fill} \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.label23 {left frame center fill} \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.entry24 {left frame center fill} \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button19 {left frame center fill} \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.label24 {left frame center fill} \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.entry25 {left frame center fill} \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20.button2 {left frame center fill}

  # pack widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20
  pack append $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20 \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame20 {top frame center expand fill} \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame0 {top frame center expand fill} \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20.frame12 {top frame center expand fill}

  # pack widget $topwin(analysis_name).info.frame4.frame3.frame14.frame17
  pack append $topwin(analysis_name).info.frame4.frame3.frame14.frame17 \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19 {bottom frame center fillx}

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame16
  frame $topwin(analysis_name).info.frame4.frame3.frame14.frame16 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame16.radiobutton9
	radiobutton $topwin(analysis_name).info.frame4.frame3.frame14.frame16.radiobutton9 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -command { if {$analysis(kernel_dim) == "" } {set analysis(kernel_dim) 3}
					if { $analysis(kernel_height) == "" } {set analysis(kernel_height) 3}
					if { $analysis(kernel_width) == "" } {set analysis(kernel_width) 3}
					pack forget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20
					pack append $topwin(analysis_name).info.frame4.frame3.frame14.frame17  \
					$topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19 {bottom frame center fillx}} \
    -relief {flat} \
    -text {Closing} \
    -value {morph_close} \
    -variable {analysis(morph_choice)}

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame16.radiobutton10
	radiobutton $topwin(analysis_name).info.frame4.frame3.frame14.frame16.radiobutton10 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -command { if {$analysis(kernel_dim) == "" } {set analysis(kernel_dim) 3}
					if { $analysis(kernel_height) == "" } {set analysis(kernel_height) 3}
					if { $analysis(kernel_width) == "" } {set analysis(kernel_width) 3}
					pack forget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20
               pack append $topwin(analysis_name).info.frame4.frame3.frame14.frame17  \
					$topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19 {bottom frame center fillx}} \
    -relief {flat} \
    -text {Opening} \
    -value {morph_open} \
    -variable {analysis(morph_choice)}

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame16.radiobutton11
	radiobutton $topwin(analysis_name).info.frame4.frame3.frame14.frame16.radiobutton11 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -command { if {$analysis(kernel_dim) == "" } {set analysis(kernel_dim) 3}
					if { $analysis(kernel_height) == "" } {set analysis(kernel_height) 3}
					if { $analysis(kernel_width) == "" } {set analysis(kernel_width) 3}
					pack forget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20
               pack append $topwin(analysis_name).info.frame4.frame3.frame14.frame17  \
					$topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19 {bottom frame center fillx}} \
    -relief {flat} \
    -text {Erosion} \
    -value {morph_erode} \
    -variable {analysis(morph_choice)}

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame16.radiobutton12
	radiobutton $topwin(analysis_name).info.frame4.frame3.frame14.frame16.radiobutton12 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -command { if {$analysis(kernel_dim) == "" } {set analysis(kernel_dim) 3}
					if { $analysis(kernel_height) == "" } {set analysis(kernel_height) 3}
					if { $analysis(kernel_width) == "" } {set analysis(kernel_width) 3}
					pack forget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20
               pack append $topwin(analysis_name).info.frame4.frame3.frame14.frame17  \
					$topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19 {bottom frame center fillx}} \
    -relief {flat} \
    -text {Dilation} \
    -value {morph_dilate} \
    -variable {analysis(morph_choice)}

  # build widget $topwin(analysis_name).info.frame4.frame3.frame14.frame16.radiobutton13
	radiobutton $topwin(analysis_name).info.frame4.frame3.frame14.frame16.radiobutton13 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -command {pack forget $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame19
				  pack append $topwin(analysis_name).info.frame4.frame3.frame14.frame17 \
				  $topwin(analysis_name).info.frame4.frame3.frame14.frame17.frame20 {bottom  frame center}} \
    -relief {flat} \
    -text {Iterative Modification (Binary image only)} \
    -value {inter_mod} \
    -variable {analysis(morph_choice)}

  # pack widget $topwin(analysis_name).info.frame4.frame3.frame14.frame16
  pack append $topwin(analysis_name).info.frame4.frame3.frame14.frame16 \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame16.radiobutton9 {left frame center } \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame16.radiobutton10 {left frame center padx 8} \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame16.radiobutton11 {left frame center padx 8} \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame16.radiobutton12 {left frame center padx 8} \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame16.radiobutton13 {left frame center padx 8}

  # pack widget $topwin(analysis_name).info.frame4.frame3.frame14
  pack append $topwin(analysis_name).info.frame4.frame3.frame14 \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame16 {top frame center expand fill} \
    $topwin(analysis_name).info.frame4.frame3.frame14.frame17 {top frame center expand fill}

  # pack widget $topwin(analysis_name).info.frame4.frame3
  pack append $topwin(analysis_name).info.frame4.frame3 \
    $topwin(analysis_name).info.frame4.frame3.frame12 {left frame center fillx} \
    $topwin(analysis_name).info.frame4.frame3.frame14 {left frame center expand fill}

  # pack widget $topwin(analysis_name).info.frame4
  pack append $topwin(analysis_name).info.frame4 \
    $topwin(analysis_name).info.frame4.frame14 {top frame center expand fill} \
    $topwin(analysis_name).info.frame4.frame3 {top frame center expand fill}

  # build widget $topwin(analysis_name).info.frame0
  frame $topwin(analysis_name).info.frame0 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2} \
    -relief {ridge}

  # build widget $topwin(analysis_name).info.frame0.frame1
  frame $topwin(analysis_name).info.frame0.frame1 \
    -borderwidth {0} \
    -highlightthickness {0} \
	 -background "$GuiColor(frame_bg)" \
    -height {30} \
    -width {30}

  # build widget $topwin(analysis_name).info.frame0.frame1.entry
  entry $topwin(analysis_name).info.frame0.frame1.entry \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
    -textvariable {analysis(median_cut_num)} -width {5}
  endEntry $topwin(analysis_name).info.frame0.frame1.entry

  # build widget $topwin(analysis_name).info.frame0.frame1.menubutton7
  menubutton $topwin(analysis_name).info.frame0.frame1.menubutton7 \
	 -activebackground "$GuiColor(btn_act_bg)" -background "$GuiColor(btn_bg)"  \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)" \
    -padx {1} -pady {1} -highlightthickness {0} \
    -text "$GuiMisc(pop_up_text)" -width {2}

  # bindings
  bind $topwin(analysis_name).info.frame0.frame1.menubutton7 <Button-1> { MenuPopupPost $topwin(analysis_name).info.frame0.frame1.menubutton7.menu0 %X %Y}
  bind $topwin(analysis_name).info.frame0.frame1.menubutton7 <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame0.frame1.menubutton7.menu0 %W }

  # build widget $topwin(analysis_name).info.frame0.frame1.menubutton7.menu0
  menu $topwin(analysis_name).info.frame0.frame1.menubutton7.menu0 
  $topwin(analysis_name).info.frame0.frame1.menubutton7.menu0 add command \
    -label {  3} \
	 -command {set analysis(median_cut_num) 3}
  $topwin(analysis_name).info.frame0.frame1.menubutton7.menu0 add command \
    -label {  5} \
	 -command {set analysis(median_cut_num) 5}
  $topwin(analysis_name).info.frame0.frame1.menubutton7.menu0 add command \
    -label { 16} \
	 -command {set analysis(median_cut_num) 16}
  $topwin(analysis_name).info.frame0.frame1.menubutton7.menu0 add command \
    -label { 32} \
	 -command {set analysis(median_cut_num) 32}
  $topwin(analysis_name).info.frame0.frame1.menubutton7.menu0 add command \
    -label { 64} \
	 -command {set analysis(median_cut_num) 64}
  $topwin(analysis_name).info.frame0.frame1.menubutton7.menu0 add command \
    -label {128} \
	 -command {set analysis(median_cut_num) 128}
  $topwin(analysis_name).info.frame0.frame1.menubutton7.menu0 add command \
    -label {256} \
	 -command {set analysis(median_cut_num) 256}

  bind $topwin(analysis_name).info.frame0.frame1.menubutton7.menu0 <B1-Motion> { MenuPopupMotion $topwin(analysis_name).info.frame0.frame1.menubutton7.menu0 %W %X %Y}
  bind $topwin(analysis_name).info.frame0.frame1.menubutton7.menu0 <ButtonRelease-1>  { MenuPopupRelease $topwin(analysis_name).info.frame0.frame1.menubutton7.menu0 %W }

  # pack master $topwin(analysis_name).info.frame0.frame1
  pack configure $topwin(analysis_name).info.frame0.frame1.menubutton7 \
    -fill both \
    -side right
  pack configure $topwin(analysis_name).info.frame0.frame1.entry \
    -fill both \
    -side left

  # build widget $topwin(analysis_name).info.frame0.radiobutton7
	radiobutton $topwin(analysis_name).info.frame0.radiobutton7 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -command {set analysis(multi_option) ""
              set analysis(sam_option) ""; set analysis(morph_choice) ""
				  if {$analysis(median_cut_num) == "" } {set analysis(median_cut_num) 3}} \
    -relief {flat} \
    -text {Median Cut, } \
    -value {median_cut} \
    -variable {analysis(seg_choice)}

  # build widget $topwin(analysis_name).info.frame0.label19
  label $topwin(analysis_name).info.frame0.label19 \
    -background "$GuiColor(label_bg)" \
    -text {with the number of desired colors (2 -1000): }

  # pack widget $topwin(analysis_name).info.frame0
  pack append $topwin(analysis_name).info.frame0 \
    $topwin(analysis_name).info.frame0.radiobutton7 {left frame center} \
    $topwin(analysis_name).info.frame0.label19 {left frame center} \
    $topwin(analysis_name).info.frame0.frame1 {left frame center}

  # build widget $topwin(analysis_name).info.frame17
  frame $topwin(analysis_name).info.frame17 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2} \
    -relief {ridge}

  # build widget $topwin(analysis_name).info.frame17.radiobutton7
	radiobutton $topwin(analysis_name).info.frame17.radiobutton7 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -command {set analysis(multi_option) ""
			     set analysis(sam_option) ""; set analysis(morph_choice) ""
				  if {$analysis(sct_A_color) == ""} { set analysis(sct_A_color) 3}
				  if {$analysis(sct_B_color) == ""} { set analysis(sct_B_color) 3}} \
    -relief {flat} \
    -text {SCT/Center, } \
    -value {sct_split} \
    -variable {analysis(seg_choice)}

  # build widget $topwin(analysis_name).info.frame17.label19
  label $topwin(analysis_name).info.frame17.label19 \
    -background "$GuiColor(label_bg)" \
    -text { and B axis: }

  # build widget $topwin(analysis_name).info.frame17.frame1
  frame $topwin(analysis_name).info.frame17.frame1 \
    -borderwidth {0} \
    -highlightthickness {0} \
	 -background "$GuiColor(frame_bg)" \
    -height {30} \
    -width {30}

  # build widget $topwin(analysis_name).info.frame17.frame1.entry
  entry $topwin(analysis_name).info.frame17.frame1.entry \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
    -textvariable {analysis(sct_A_color)} -width {5}
  # bindings
  endEntry $topwin(analysis_name).info.frame17.frame1.entry 0 \
  	$topwin(analysis_name).info.frame17.frame1.entry  \
  	$topwin(analysis_name).info.frame17.frame1.entry 

  # build widget $topwin(analysis_name).info.frame17.frame1.menubutton7
  menubutton $topwin(analysis_name).info.frame17.frame1.menubutton7 \
	 -activebackground "$GuiColor(btn_act_bg)" -background "$GuiColor(btn_bg)"  \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)" \
    -padx {1} -pady {1} -highlightthickness {0} \
    -text "$GuiMisc(pop_up_text)" -width {2}

  # bindings
  bind $topwin(analysis_name).info.frame17.frame1.menubutton7 <Button-1> { MenuPopupPost $topwin(analysis_name).info.frame17.frame1.menubutton7.menu0 %X %Y}
  bind $topwin(analysis_name).info.frame17.frame1.menubutton7 <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame17.frame1.menubutton7.menu0 %W }

  # build widget $topwin(analysis_name).info.frame17.frame1.menubutton7.menu0
  menu $topwin(analysis_name).info.frame17.frame1.menubutton7.menu0 
  $topwin(analysis_name).info.frame17.frame1.menubutton7.menu0 add command \
    -label {  3} \
	 -command {set analysis(sct_A_color) 3}
  $topwin(analysis_name).info.frame17.frame1.menubutton7.menu0 add command \
    -label {  8} \
	 -command {set analysis(sct_A_color) 8}
  $topwin(analysis_name).info.frame17.frame1.menubutton7.menu0 add command \
    -label { 16} \
	 -command {set analysis(sct_A_color) 16}
  $topwin(analysis_name).info.frame17.frame1.menubutton7.menu0 add command \
    -label { 32} \
	 -command {set analysis(sct_A_color) 32}
  $topwin(analysis_name).info.frame17.frame1.menubutton7.menu0 add command \
    -label { 64} \
	 -command {set analysis(sct_A_color) 64}
  $topwin(analysis_name).info.frame17.frame1.menubutton7.menu0 add command \
    -label {100} \
	 -command {set analysis(sct_A_color) 100}

  bind $topwin(analysis_name).info.frame17.frame1.menubutton7.menu0 <B1-Motion> { MenuPopupMotion $topwin(analysis_name).info.frame17.frame1.menubutton7.menu0 %W %X %Y}
  bind $topwin(analysis_name).info.frame17.frame1.menubutton7.menu0 <ButtonRelease-1>  { MenuPopupRelease $topwin(analysis_name).info.frame17.frame1.menubutton7.menu0 %W }

  # pack master $topwin(analysis_name).info.frame17.frame1
  pack configure $topwin(analysis_name).info.frame17.frame1.menubutton7 \
    -fill both \
    -side right
  pack configure $topwin(analysis_name).info.frame17.frame1.entry \
    -fill both \
    -side left

  # build widget $topwin(analysis_name).info.frame17.label26
  label $topwin(analysis_name).info.frame17.label26 \
    -background "$GuiColor(label_bg)" \
    -text {with the number of  colors along A axis: }

  # build widget $topwin(analysis_name).info.frame17.frame2
  frame $topwin(analysis_name).info.frame17.frame2 \
    -borderwidth {0} \
    -highlightthickness {0} \
	 -background "$GuiColor(frame_bg)" \
    -height {30} \
    -width {30}

  # build widget $topwin(analysis_name).info.frame17.frame2.entry
  entry $topwin(analysis_name).info.frame17.frame2.entry \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
    -textvariable {analysis(sct_B_color)} -width {5}
  # bindings
  endEntry $topwin(analysis_name).info.frame17.frame2.entry 0 \
  	$topwin(analysis_name).info.frame17.frame2.entry \
  	$topwin(analysis_name).info.frame17.frame2.entry 

  # build widget $topwin(analysis_name).info.frame17.frame2.menubutton7
  menubutton $topwin(analysis_name).info.frame17.frame2.menubutton7 \
	 -activebackground "$GuiColor(btn_act_bg)" -background "$GuiColor(btn_bg)"  \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)" \
    -padx {1} -pady {1} -highlightthickness {0} \
    -text "$GuiMisc(pop_up_text)" -width {2}

  # bindings
  bind $topwin(analysis_name).info.frame17.frame2.menubutton7 <Button-1> { MenuPopupPost $topwin(analysis_name).info.frame17.frame2.menubutton7.menu0 %X %Y}
  bind $topwin(analysis_name).info.frame17.frame2.menubutton7 <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame17.frame2.menubutton7.menu0 %W }

  # build widget $topwin(analysis_name).info.frame17.frame2.menubutton7.menu0
  menu $topwin(analysis_name).info.frame17.frame2.menubutton7.menu0 
  $topwin(analysis_name).info.frame17.frame2.menubutton7.menu0 add command \
    -label {  3} \
	 -command {set analysis(sct_B_color) 3}
  $topwin(analysis_name).info.frame17.frame2.menubutton7.menu0 add command \
    -label {  8} \
	 -command {set analysis(sct_B_color) 8}
  $topwin(analysis_name).info.frame17.frame2.menubutton7.menu0 add command \
    -label { 16} \
	 -command {set analysis(sct_B_color) 16}
  $topwin(analysis_name).info.frame17.frame2.menubutton7.menu0 add command \
    -label { 32} \
	 -command {set analysis(sct_B_color) 32}
  $topwin(analysis_name).info.frame17.frame2.menubutton7.menu0 add command \
    -label { 64} \
	 -command {set analysis(sct_B_color) 64}
  $topwin(analysis_name).info.frame17.frame2.menubutton7.menu0 add command \
    -label {100} \
	 -command {set analysis(sct_B_color) 100}

  bind $topwin(analysis_name).info.frame17.frame2.menubutton7.menu0 <B1-Motion> { MenuPopupMotion $topwin(analysis_name).info.frame17.frame2.menubutton7.menu0 %W %X %Y}
  bind $topwin(analysis_name).info.frame17.frame2.menubutton7.menu0 <ButtonRelease-1>  { MenuPopupRelease $topwin(analysis_name).info.frame17.frame2.menubutton7.menu0 %W }

  # pack master $topwin(analysis_name).info.frame17.frame2
  pack configure $topwin(analysis_name).info.frame17.frame2.menubutton7 \
    -fill both \
    -side right
  pack configure $topwin(analysis_name).info.frame17.frame2.entry \
    -fill both \
    -side left

  # pack widget $topwin(analysis_name).info.frame17
  pack append $topwin(analysis_name).info.frame17 \
    $topwin(analysis_name).info.frame17.radiobutton7 {left frame center} \
    $topwin(analysis_name).info.frame17.label26 {left frame center} \
    $topwin(analysis_name).info.frame17.frame1  {left frame center} \
    $topwin(analysis_name).info.frame17.label19 {left frame center} \
    $topwin(analysis_name).info.frame17.frame2  {left frame center}

  # build widget $topwin(analysis_name).info.frame1
  frame $topwin(analysis_name).info.frame1 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2} \
    -relief {ridge}

  # build widget $topwin(analysis_name).info.frame1.frame1
  frame $topwin(analysis_name).info.frame1.frame1 \
    -borderwidth {0} \
    -highlightthickness {0} \
	 -background "$GuiColor(frame_bg)" \
    -height {30} \
    -width {30}

  # build widget $topwin(analysis_name).info.frame1.frame1.entry
  entry $topwin(analysis_name).info.frame1.frame1.entry \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
    -textvariable {analysis(pct_median_num)} -width {5}
  endEntry $topwin(analysis_name).info.frame1.frame1.entry 

  # build widget $topwin(analysis_name).info.frame1.frame1.menubutton7
  menubutton $topwin(analysis_name).info.frame1.frame1.menubutton7 \
	 -activebackground "$GuiColor(btn_act_bg)" -background "$GuiColor(btn_bg)"  \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)" \
    -padx {1} -pady {1} -highlightthickness {0} \
    -text "$GuiMisc(pop_up_text)" -width {2}

  # bindings
  bind $topwin(analysis_name).info.frame1.frame1.menubutton7 <Button-1> { MenuPopupPost $topwin(analysis_name).info.frame1.frame1.menubutton7.menu0 %X %Y}
  bind $topwin(analysis_name).info.frame1.frame1.menubutton7 <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame1.frame1.menubutton7.menu0 %W }

  # build widget $topwin(analysis_name).info.frame1.frame1.menubutton7.menu0
  menu $topwin(analysis_name).info.frame1.frame1.menubutton7.menu0 
  $topwin(analysis_name).info.frame1.frame1.menubutton7.menu0 add command \
    -label {  3} \
	 -command {set analysis(pct_median_num) 3}
  $topwin(analysis_name).info.frame1.frame1.menubutton7.menu0 add command \
    -label {  8} \
	 -command {set analysis(pct_median_num) 8}
  $topwin(analysis_name).info.frame1.frame1.menubutton7.menu0 add command \
    -label { 16} \
	 -command {set analysis(pct_median_num) 16}
  $topwin(analysis_name).info.frame1.frame1.menubutton7.menu0 add command \
    -label { 32} \
	 -command {set analysis(pct_median_num) 32}
  $topwin(analysis_name).info.frame1.frame1.menubutton7.menu0 add command \
    -label { 64} \
	 -command {set analysis(pct_median_num) 64}
  $topwin(analysis_name).info.frame1.frame1.menubutton7.menu0 add command \
    -label {128} \
	 -command {set analysis(pct_median_num) 128}
  $topwin(analysis_name).info.frame1.frame1.menubutton7.menu0 add command \
    -label {256} \
	 -command {set analysis(pct_median_num) 256}
  $topwin(analysis_name).info.frame1.frame1.menubutton7.menu0 add command \
    -label {512} \
	 -command {set analysis(pct_median_num) 512}

  bind $topwin(analysis_name).info.frame1.frame1.menubutton7.menu0 <B1-Motion> { MenuPopupMotion $topwin(analysis_name).info.frame1.frame1.menubutton7.menu0 %W %X %Y}
  bind $topwin(analysis_name).info.frame1.frame1.menubutton7.menu0 <ButtonRelease-1>  { MenuPopupRelease $topwin(analysis_name).info.frame1.frame1.menubutton7.menu0 %W }

  # pack master $topwin(analysis_name).info.frame1.frame1
  pack configure $topwin(analysis_name).info.frame1.frame1.menubutton7 \
    -fill both \
    -side right
  pack configure $topwin(analysis_name).info.frame1.frame1.entry \
    -fill both \
    -side left


  # build widget $topwin(analysis_name).info.frame1.radiobutton7
	radiobutton $topwin(analysis_name).info.frame1.radiobutton7 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -command {set analysis(multi_option) ""
              set analysis(sam_option) ""; set analysis(morph_choice) ""
				  if {$analysis(pct_median_num) == "" } {set analysis(pct_median_num) 3}} \
    -relief {flat} \
    -text {PCT/Median, } \
    -value {pct_median} \
    -variable {analysis(seg_choice)}

  # build widget $topwin(analysis_name).info.frame1.label19
  label $topwin(analysis_name).info.frame1.label19 \
    -background "$GuiColor(label_bg)" \
    -text {with the number of desired colors (2-1000): }

  # pack widget $topwin(analysis_name).info.frame1
  pack append $topwin(analysis_name).info.frame1 \
    $topwin(analysis_name).info.frame1.radiobutton7 {left frame center} \
    $topwin(analysis_name).info.frame1.label19 {left frame center} \
    $topwin(analysis_name).info.frame1.frame1 {left frame center}

  # build widget $topwin(analysis_name).info.frame3
  frame $topwin(analysis_name).info.frame3 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2} \
    -relief {ridge}

  # build widget $topwin(analysis_name).info.frame3.entry20
  entry $topwin(analysis_name).info.frame3.entry20 \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
    -textvariable {analysis(gray_quant_num)} \
    -width {4}
  # bindings
  endEntry $topwin(analysis_name).info.frame3.entry20 0

  # build widget $topwin(analysis_name).info.frame3.radiobutton7
	radiobutton $topwin(analysis_name).info.frame3.radiobutton7 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -command {set analysis(multi_option) ""
              set analysis(sam_option) ""; set analysis(morph_choice) ""
				  if {$analysis(gray_quant_num) == "" } {set analysis(gray_quant_num) 4}} \
    -relief {flat} \
    -text {Gray Level Quantization, } \
    -value {gray_quant} \
    -variable {analysis(seg_choice)}

	radiobutton $topwin(analysis_name).info.frame3.radiobutton8 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -relief {flat} \
    -text {Standard } \
    -value {S} \
	 -anchor {w} \
    -variable {analysis(gray_qmethod)}

	radiobutton $topwin(analysis_name).info.frame3.radiobutton9 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -relief {flat} \
    -text {IGS} \
    -value {I} \
	 -anchor {w} \
    -variable {analysis(gray_qmethod)}

  # build widget $topwin(analysis_name).info.frame3.label19
  label $topwin(analysis_name).info.frame3.label19 \
    -background "$GuiColor(label_bg)" \
    -text {with the number of gray levels: }

  # build widget $topwin(analysis_name).info.frame3.button19
	menubutton $topwin(analysis_name).info.frame3.button19 -padx {1} -pady {5} -highlightthickness {0}  \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)" \
    -text "$GuiMisc(pop_up_text)" -width {2} \
    -width {2}

  # bindings
  bind $topwin(analysis_name).info.frame3.button19 <Button-1> {MenuPopupPost $topwin(analysis_name).info.frame3.button19.menu %X %Y}
  bind $topwin(analysis_name).info.frame3.button19 <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame3.button19.menu %W}

  # build widget $topwin(analysis_name).info.frame3.button19.menu
  menu $topwin(analysis_name).info.frame3.button19.menu
  $topwin(analysis_name).info.frame3.button19.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(gray_quant_num) 2} \
    -label {  2}
  $topwin(analysis_name).info.frame3.button19.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(gray_quant_num) 4} \
    -label {  4}
  $topwin(analysis_name).info.frame3.button19.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(gray_quant_num) 8} \
    -label {  8}
  $topwin(analysis_name).info.frame3.button19.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(gray_quant_num) 16} \
    -label { 16}
  $topwin(analysis_name).info.frame3.button19.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(gray_quant_num) 32} \
    -label { 32}
  $topwin(analysis_name).info.frame3.button19.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(gray_quant_num) 64} \
    -label { 64}
  $topwin(analysis_name).info.frame3.button19.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(gray_quant_num) 128} \
    -label {128}
  bindtags $topwin(analysis_name).info.frame3.button19.menu {Menu all}
  # bindings
  bind $topwin(analysis_name).info.frame3.button19.menu <B1-Motion> {MenuPopupMotion $topwin(analysis_name).info.frame3.button19.menu %W %X %Y}
  bind $topwin(analysis_name).info.frame3.button19.menu <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame3.button19.menu %W}

  # pack widget $topwin(analysis_name).info.frame3
  pack append $topwin(analysis_name).info.frame3 \
    $topwin(analysis_name).info.frame3.radiobutton7 {left frame center} \
    $topwin(analysis_name).info.frame3.label19 {left frame center} \
    $topwin(analysis_name).info.frame3.entry20 {left frame center} \
    $topwin(analysis_name).info.frame3.button19 {left frame center fillx} \
    $topwin(analysis_name).info.frame3.radiobutton8 {top frame w} \
    $topwin(analysis_name).info.frame3.radiobutton9 {top frame w} 

  # pack widget $topwin(analysis_name).info
  pack append $topwin(analysis_name).info \
    $topwin(analysis_name).info.frame16 {top frame center expand fill} \
    $topwin(analysis_name).info.frame3 {top frame center expand fill} \
    $topwin(analysis_name).info.frame14 {top frame center expand fill} \
    $topwin(analysis_name).info.frame0 {top frame center expand fill} \
    $topwin(analysis_name).info.frame1 {top frame center expand fill} \
    $topwin(analysis_name).info.frame17 {top frame center expand fill} \
    $topwin(analysis_name).info.frame6 {top frame center pady 5 expand fill} \
    $topwin(analysis_name).info.frame2 {top frame center pady 5 expand fill} \
    $topwin(analysis_name).info.frame4 {top frame center pady 5 expand fill}

  # build widget $topwin(analysis_name).current_image
  frame $topwin(analysis_name).current_image \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2} \
    -relief {ridge}

  # build widget $topwin(analysis_name).current_image.current
  label $topwin(analysis_name).current_image.current \
    -background "$GuiColor(label_bg)" \
    -text {Current image is :}

  # build widget $topwin(analysis_name).current_image.name
  label $topwin(analysis_name).current_image.name \
    -background "$GuiColor(label_bg)" \
    -foreground {Blue} \
    -text {} \
    -textvariable {current_image}

  # pack widget $topwin(analysis_name).current_image
  pack append $topwin(analysis_name).current_image \
    $topwin(analysis_name).current_image.current {left frame center filly} \
    $topwin(analysis_name).current_image.name {left frame center filly}

  # build widget $topwin(analysis_name).perform
  frame $topwin(analysis_name).perform \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2} \
    -relief {raised}

  # build widget $topwin(analysis_name).perform.perform1
	button $topwin(analysis_name).perform.perform1 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(perf_act_bg)" \
    -background "$GuiColor(perf_bg)" \
    -foreground "$GuiColor(perf_fg)" \
    -command {
		$topwin(analysis_name) configure -cursor watch; update
		if {$current_image == ""} {
			CvipAlert "Please select an image first."
			$topwin(analysis_name) configure -cursor top_left_arrow; update
         return
		}
		switch -exact -- $analysis(seg_choice) {
			hist_thresh { 
				if [catch { set result [cvip $analysis(seg_choice) $current_image]} cresult ] {
					CvipAlert "$cresult"
					$topwin(analysis_name) configure -cursor top_left_arrow; update
				}
				set temp $current_image
				set new_name [cvip rename $result ${current_image}_thresh]
				img_q insert end $new_name
				image_post $new_name
				cvip view $new_name
				CvipMessage "image \"$temp\" is segmented using histogram thresholding"
			}

			fuzzyc {
				if [catch {expr $analysis(fuzzyc_kernel)}] {
					CvipAlert "Numerical values required for the Gaussian kernel: \n gussian_kernel= $analysis(fuzzyc_kernel)"
					$topwin(analysis_name) configure -cursor top_left_arrow; update
					return
				} elseif {$analysis(fuzzyc_kernel) < 0.0 || $analysis(fuzzyc_kernel) > 20.0 } {
					CvipAlert "The Gaussian kernel (=$analysis(fuzzyc_kernel)) should be \n in the range (0.0 - 20.0)"
					$topwin(analysis_name) configure -cursor top_left_arrow; update
					return
				}

				if {$bands == "1"} {
					if [catch { set temp [cvip assemble_bands $current_image $current_image $current_image];\
									set temp1 [cvip fuzzyc $temp $analysis(fuzzyc_kernel)]; \
									cvip dq $temp; set result [cvip extract_band $temp1 1]; cvip dq $temp1 } cresult ] {
						CvipAlert "$cresult"
						$topwin(analysis_name) configure -cursor top_left_arrow; update
					}
				} else {
					if [catch { set result [cvip $analysis(seg_choice) $current_image $analysis(fuzzyc_kernel)]} cresult ] {
						CvipAlert "$cresult"
						$topwin(analysis_name) configure -cursor top_left_arrow; update
					}
				}
				set temp $current_image
				set new_name [cvip rename $result ${current_image}_fuzzyc]
				img_q insert end $new_name
				image_post $new_name
				cvip view $new_name
				CvipMessage "image \"$temp\" is segmented using fuzzy c-means \n\
				with Gaussian kernel size = $analysis(fuzzyc_kernel)"
			}

			gray_quant {
				if [catch {expr $analysis(gray_quant_num)}] {
					CvipAlert "Numerical values required for the Number_of_gray_levels: \n Number_of_gray_levels= $analysis(gray_quant_num)"
					$topwin(analysis_name) configure -cursor top_left_arrow; update
					return
				}
				if { $analysis(gray_quant_num) < 2 || $analysis(gray_quant_num) > 128 || $analysis(gray_quant_num)/2*2 != $analysis(gray_quant_num)} {
					CvipAlert "Number_of_gray_levels should be one of the following values: \n  (2, 4, 8, 16, ..., 128), instead of ($analysis(gray_quant_num))"
					$topwin(analysis_name) configure -cursor top_left_arrow; update
					return
				}
				if { $analysis(gray_qmethod) == "S" } {
					if [catch { set result [cvip $analysis(seg_choice) $current_image $analysis(gray_quant_num)] } cresult ] {
						CvipAlert "$cresult"
						$topwin(analysis_name) configure -cursor top_left_arrow; update
						return
					}
				} else {
					if [catch { set result [cvip igs $current_image $analysis(gray_quant_num)] } cresult ] {
						CvipAlert "$cresult"
						$topwin(analysis_name) configure -cursor top_left_arrow; update
						return
					}
				}
				set temp $current_image
				set new_name [cvip rename $result ${current_image}_GrayQuant$analysis(gray_quant_num)]
				img_q insert end $new_name
				image_post $new_name
				cvip view $new_name
				CvipMessage "image \"$temp\" is segmented using graylevel quantization to $analysis(gray_quant_num) levels"
			}

			median_cut {
				if [catch {expr $analysis(median_cut_num)}] {
					CvipAlert "Numerical values required for the number of desired colors: \n median_cut_num= $analysis(median_cut_num)"
					$topwin(analysis_name) configure -cursor top_left_arrow; update
					return
				} elseif { $analysis(median_cut_num) < 2 || $analysis(median_cut_num) > 1000 } {
					CvipAlert "The Number of Colors (=$analysis(median_cut_num)) should be \n in the range (2 - 1000)"
					$topwin(analysis_name) configure -cursor top_left_arrow; update
					return
				}
				if {$bands == "1"} {
					if [ catch { set temp [cvip assemble_bands $current_image $current_image $current_image];\
									set temp1 [cvip $analysis(seg_choice) $temp $analysis(median_cut_num)];\
									cvip dq $temp; set result [cvip extract_band $temp1 1]; cvip dq $temp1 } cresult ] {
					CvipAlert "$cresult"
					$topwin(analysis_name) configure -cursor top_left_arrow; update
					return
					}
				} else {
					if [catch { set result [cvip $analysis(seg_choice) $current_image $analysis(median_cut_num)] } cresult ] {
						CvipAlert "$cresult"
						$topwin(analysis_name) configure -cursor top_left_arrow; update
						return
					}
				}
				set temp $current_image
				set new_name [cvip rename $result ${current_image}_medCut]
				img_q insert end $new_name
				image_post $new_name
				cvip view $new_name
				CvipMessage "image \"$temp\" is segmented to $analysis(median_cut_num) colors using median cut"
			}

			sct_split {
				if [catch {expr $analysis(sct_A_color) + $analysis(sct_B_color)}] {
					CvipAlert "Numerical values required for the number of desired colors: \n Color_A = $analysis(sct_A_color), Color_B = $analysis(sct_B_color)"
					$topwin(analysis_name) configure -cursor top_left_arrow; update
					return
				} elseif { $analysis(sct_A_color) > 100 || $analysis(sct_A_color) < 1 || $analysis(sct_B_color) > 100 || $analysis(sct_B_color) < 1} {
					CvipAlert "The Number of Colors (Color_A = $analysis(sct_A_color), Color_B = $analysis(sct_B_color))\n should be in the range (1 - 100)"
					$topwin(analysis_name) configure -cursor top_left_arrow; update
					return
				}
				if {$bands == "1"} {
					if [catch { set temp [cvip assemble_bands $current_image $current_image $current_image]; \
									set temp1 [cvip $analysis(seg_choice) $temp $analysis(sct_A_color) $analysis(sct_B_color)];\
									cvip dq $temp; set result [cvip extract_band $temp1 1]; cvip dq $temp1 } cresult ] {
						CvipAlert "$cresult"
						$topwin(analysis_name) configure -cursor top_left_arrow; update
						return
					}
				} else {
					if [catch { set result [cvip $analysis(seg_choice) $current_image $analysis(sct_A_color)\
									$analysis(sct_B_color)] } cresult ] {
						CvipAlert "$cresult"
						$topwin(analysis_name) configure -cursor top_left_arrow; update
						return
					}
				}
				set temp $current_image
				set new_name [cvip rename $result ${current_image}_sct$analysis(sct_A_color)x$analysis(sct_B_color)]
				img_q insert end $new_name
				image_post $new_name
				cvip view $new_name
				CvipMessage "image \"$temp\" is segmented to [expr $analysis(sct_A_color) * $analysis(sct_B_color)] colors\
				using SCT split"
			}

			pct_median {
				set temp2 $current_image
				if [catch {expr $analysis(pct_median_num)}] {
					CvipAlert "Numerical values required for the number of desired colors: \n Number_of_Colors = $analysis(pct_median_num)"
					$topwin(analysis_name) configure -cursor top_left_arrow; update
					return
				} elseif { $analysis(pct_median_num) < 2 || $analysis(pct_median_num) > 1000 } {
					CvipAlert "The Number of Colors (=$analysis(pct_median_num)) should be \n in the range (2 - 1000)"
					$topwin(analysis_name) configure -cursor top_left_arrow; update
					return
				}
				if {$bands == "1"} {
					if [catch {set temp1 [cvip assemble_bands $current_image $current_image $current_image];\
									set temp [cvip pct_median $temp1 $analysis(pct_median_num)]; \
									cvip dq $temp1; set result [cvip extract_band $temp 1]; cvip dq $temp; \
									set new_name [cvip rename $result ${current_image}_pctMed$analysis(pct_median_num)]} cresult ] {
						CvipAlert "$cresult"
						$topwin(analysis_name) configure -cursor top_left_arrow; update
						return
					}
				} else {
					if [catch {set temp [cvip pct_median $current_image $analysis(pct_median_num) ]; \
								set new_name [cvip rename $temp ${current_image}_pctMedian$analysis(pct_median_num)]} cresult ] {
						CvipAlert "$cresult"
						$topwin(analysis_name) configure -cursor top_left_arrow; update
						return
					}
				}
				img_q insert end $new_name
				image_post $new_name
				cvip view $new_name
				CvipMessage "image \"$temp2\" is segmented to $analysis(pct_median_num) colors using pct-median-cut"
			}

			multi_resolution {
				if { $analysis(multi_option) == "4" || $analysis(multi_option) == "5" || $analysis(multi_option) == "6" } {
					if [catch {expr $analysis(multi_thresh)} ] {
						CvipAlert "Numerical values required for the threshold value: \n Threshold = $analysis(multi_thresh)"
					$topwin(analysis_name) configure -cursor top_left_arrow; update
					return
					} elseif { $analysis(multi_thresh) < 0 || $analysis(multi_thresh) > 1000 } {
						CvipAlert "The threshold value should be in the range (0 - 1000): \n Threshold = $analysis(multi_thresh)"
					$topwin(analysis_name) configure -cursor top_left_arrow; update
					return
					}
				}
				if { $analysis(multi_option) == "4" || $analysis(multi_option) == "6" } {
					if [catch {expr $analysis(multi_percent)} ] {
					CvipAlert "Numerical values required for the percentage: \n Percentage = $analysis(multi_percent)"
					$topwin(analysis_name) configure -cursor top_left_arrow; update
					return
					} elseif { $analysis(multi_option) == "6" && ($analysis(multi_percent) < 0.0 || int($analysis(multi_percent)) != $analysis(multi_percent))} {
						CvipAlert "The pixel distance ($analysis(multi_percent)) should be a positive integer" 
					$topwin(analysis_name) configure -cursor top_left_arrow; update
					return
					}
				}
				if { $analysis(multi_option) == "6" } {
					CvipMessage "You have chosen texture homogeneity for muti-resolution, \n\
					which might take a long time\n"
					update
				}
				if [catch {set result [cvip $analysis(seg_choice) $current_image $analysis(multi_option) \
									$analysis(multi_pct) $analysis(multi_thresh) $analysis(multi_percent)]} cresult] {
					CvipAlert "$cresult"
					$topwin(analysis_name) configure -cursor top_left_arrow; update
					return
				}
				set temp $current_image
				set new_name [cvip rename $result ${current_image}_multiRes$analysis(multi_option)]
				img_q insert end $new_name
				image_post $new_name
				cvip view $new_name
				switch -exact -- $analysis(multi_option) {
				1 {set temp1 "pure uniformity"}
				2 {set temp1 "local means vs. global mean"}
				3 {set temp1 "local std. vs. global std."}
				4 {set temp1 "variance"}
				5 {set temp1 "weighted gray level distance"}
				6 {set temp1 "texture homogeneity"}
				}
				#CvipMessage "image \"$temp\" is segmented using multi-resolution \n\
				#according to $temp1"
			}

			split_merge { 
				set temp [expr { $rows > $cols } ? $cols : $rows ]
				if [catch {expr $analysis(sam_level)}] {
				CvipAlert "Numerical values required for the entry level: \n \
				Entry level = $analysis(sam_level)"
					$topwin(analysis_name) configure -cursor top_left_arrow; update
				return
				} elseif [expr $analysis(sam_level) < "0" || $analysis(sam_level) > log($temp) / log(2.0) - 1 ] {
				CvipAlert "The entry level for split and merge should be in the range \
				(0 - [expr floor(log($temp) / log(2.0) - 1)]): \n\
				Entry level = $analysis(sam_level)"
					$topwin(analysis_name) configure -cursor top_left_arrow; update
				return
				}
				if { $analysis(sam_option) == "4" || $analysis(sam_option) == "5" || $analysis(sam_option) == "6" } {
					if [catch {expr $analysis(sam_thresh)} ] {
						CvipAlert "Numerical values required for the threshold/similarity value: \n\
						Threshold/similarity = $analysis(sam_thresh)"
					$topwin(analysis_name) configure -cursor top_left_arrow; update
					return
					} elseif { $analysis(sam_thresh) < 0 || $analysis(sam_thresh) > 1000 } {
						CvipAlert "The threshold/similarity value should be in the range (0 - 1000): \n\
						Threshold/similarity = $analysis(sam_thresh)"
					$topwin(analysis_name) configure -cursor top_left_arrow; update
					return
					}
				}
				if { $analysis(sam_option) == "4" || $analysis(sam_option) == "6" } {
					if [catch {expr $analysis(sam_percent)} ] {
					CvipAlert "Numerical values required for the percentage/pix distance: \n\
					Percentage/pix distance= $analysis(sam_percent)"
					$topwin(analysis_name) configure -cursor top_left_arrow; update
					return
					}  elseif { $analysis(sam_option) == "6" && (int($analysis(sam_percent)) != $analysis(sam_percent) || $analysis(sam_percent) <=0 )} {
						CvipAlert "The pix distance ($analysis(sam_percent)) value should be a positive integer"
					$topwin(analysis_name) configure -cursor top_left_arrow; update
					return
					}
				}
				if { $analysis(sam_option) == "6" } {
					CvipMessage "You have chosen texture homogeneity for split-and-merge, \n\
					which might take a long time"
					update
				}
				if [catch {set result [cvip $analysis(seg_choice) $current_image $analysis(sam_option) \
							$analysis(sam_level) $analysis(sam_pct) $analysis(sam_thresh) $analysis(sam_percent)]} cresult] {
					CvipAlert "$cresult"
					$topwin(analysis_name) configure -cursor top_left_arrow; update
					return
				}
				set temp $current_image
				set new_name [cvip rename $result ${current_image}_sm$analysis(sam_option)]
				img_q insert end $new_name
				image_post $new_name
				cvip view $new_name
				switch -exact -- $analysis(sam_option) {
				1 {set temp1 "pure uniformity"}
				2 {set temp1 "local means vs. global mean"}
				3 {set temp1 "local std. vs. global std."}
				4 {set temp1 "variance"}
				5 {set temp1 "weighted gray level distance"}
				6 {set temp1 "texture homogeneity"}
				}
				CvipMessage "image \"$temp\" is segmented using split and merge \n\
				according to $temp1"
			}

			filter {
				if {$analysis(morph_choice) == "inter_mod"} {
				   foreach i {iteration subfields} {
						if [catch {expr $morph(${i}.val)}] {
							CvipAlert "Numerical values required for $i: \n $i = $morph(${i}.val)"
							$topwin(analysis_name) configure -cursor top_left_arrow; update
							return
						}
						if {$morph(${i}.val) < $morph(${i}.min) || $morph(${i}.val) > $morph(${i}.max)} {
							CvipAlert "The value of  $i (=$morph(${i}.val)) should be \n  in the range ($morph(${i}.min) - $morph(${i}.max))"
							$topwin(analysis_name) configure -cursor top_left_arrow; update
							return
						}
					}
					set morph(set.val) "\{"
					for {set i 1} {$i < 15} {incr i} {
						if {$morph(set.$i) == "1"} {append morph(set.val) "$i,"}
					}
					if {[string index $morph(set.val) [expr [string length $morph(set.val)] - 1]] == "\{"} {
						CvipAlert "Please select surround pattern(s)."
						$topwin(analysis_name) configure -cursor top_left_arrow; update
						return
					} else {
						set morph(set.val) [string range $morph(set.val) 0 [expr [string length $morph(set.val)] - 2]]
						append morph(set.val) "\}"
						if { $GUI_DEBUG == "1" } {
							puts $morph(set.val)
						}
					}
					if {$analysis(morph_rot) != "y" && $analysis(morph_rot) != "n"} {
						CvipAlert "The value of  $i (=$morph(${i}.val)) should be \n  either \"y\" or \"n\""
						$topwin(analysis_name) configure -cursor top_left_arrow; update
						return
					}
					switch -exact -- $morph(boolean.val) {
						0 { set temp 1}
						!a { set temp 2}
						ab { set temp 3}
						a+b { set temp 4}
						a^b { set temp 5}
						(!a)b { set temp 6}
					}
					if [catch {set result [eval "cvip morpho $current_image $morph(set.val) $analysis(morph_rot) $temp  $morph(iteration.val) $morph(subfields.val)"]}] {
						CvipAlert "A two-level image is required for iterative modification, a two \n level image can be created by binary thresholding."
						$topwin(analysis_name) configure -cursor top_left_arrow; update
						return
					}
					set temp $current_image
					set new_name [cvip rename $result ${current_image}_morpho]
					img_q insert end $new_name
					image_post $new_name
					cvip view $new_name
					CvipMessage "iterative modification performed on image \"$temp\""

				} else {
					if [catch {expr $analysis(kernel_dim)}] {
						CvipAlert "Numerical values required for the kernel dimension: \n kernel dimension = $analysis(kernel_dim)"
						$topwin(analysis_name) configure -cursor top_left_arrow; update
						return
					} elseif { $analysis(kernel_dim) <= 0 || $analysis(kernel_dim) > $rows / 8 || $analysis(kernel_dim) > $cols / 8 } {
							CvipAlert "The value of  Kernel dimension (=$analysis(kernel_dim)) should be \n \
							in the range (1--[expr $rows / 8 > $cols / 8 ? $cols / 8 : $rows / 8])"
						$topwin(analysis_name) configure -cursor top_left_arrow; update
							return
					} elseif [catch {expr $analysis(kernel_height)}] { 
						CvipAlert "Numerical values required for the kernel height/side/thickness: \n \
						kernel height/side/thickness = $analysis(kernel_height)"
						$topwin(analysis_name) configure -cursor top_left_arrow; update
						return
					} elseif {$analysis(kernel_height) <= 0 } {
							CvipAlert "The value of  Kernel height/side/thickness (=$analysis(kernel_height)) should be \n \
							a positive number"
						$topwin(analysis_name) configure -cursor top_left_arrow; update
							return
					} elseif [catch {expr $analysis(kernel_width)}] { 
						CvipAlert "Numerical values required for the kernel width: \n \
						kernel width = $analysis(kernel_width)"
						$topwin(analysis_name) configure -cursor top_left_arrow; update
						return
					} elseif {$analysis(kernel_width) <= 0 } {
							CvipAlert "The value of  Kernel width (=$analysis(kernel_width)) should be \n \
							a positive number"
							$topwin(analysis_name) configure -cursor top_left_arrow; update
							return
					} 
					if {$bands != "1"} {
						for { set i 1} { $i <= $bands } { incr i } {
							if [catch { set temp1 [cvip extract_band $current_image $i] } cresult ] {
								CvipAlert "$cresult"
								$topwin(analysis_name) configure -cursor top_left_arrow; update
								return
							}
							set temp "cvip $analysis(morph_choice) $temp1 "
							switch -exact -- $analysis(kernel_type) {
							circle		{append temp "1 $analysis(kernel_dim) -1 -1"}
							square		{append temp "2 $analysis(kernel_dim) $analysis(kernel_height) -1"}
							rectangle	{append temp "3 $analysis(kernel_dim) $analysis(kernel_height) $analysis(kernel_width)"}
							cross			{append temp "4 $analysis(kernel_dim) $analysis(kernel_height) $analysis(kernel_width)"}
							}
							if [catch {set result [eval $temp]} cresult] {
								CvipAlert "$cresult"
								$topwin(analysis_name) configure -cursor top_left_arrow; update
								return
							}
							lappend mid $result; cvip dq $temp1
						}
						if [catch { set result [eval "cvip assemble_bands [join $mid]" ] } cresult ] {
							CvipAlert "$cresult"
							$topwin(analysis_name) configure -cursor top_left_arrow; update
							return
						}
						foreach i [split $mid] {
							cvip dq $i
						}
						unset mid; unset i

					} else {
						set temp "cvip $analysis(morph_choice) $current_image "
						switch -exact -- $analysis(kernel_type) {
						circle		{append temp "1 $analysis(kernel_dim) -1 -1"}
						square		{append temp "2 $analysis(kernel_dim) $analysis(kernel_height) -1"}
						rectangle	{append temp "3 $analysis(kernel_dim) $analysis(kernel_height) $analysis(kernel_width)"}
						cross			{append temp "4 $analysis(kernel_dim) $analysis(kernel_height) $analysis(kernel_width)"}
						}
						if [catch {set result [eval $temp]} cresult] {
							CvipAlert "$cresult"
							$topwin(analysis_name) configure -cursor top_left_arrow; update
							return
						}
					}
					set temp $current_image
					set new_name [cvip rename $result ${current_image}_$analysis(morph_choice)$analysis(kernel_type)]
					img_q insert end $new_name
					image_post $new_name
					cvip view $new_name
					CvipMessage "$analysis(morph_choice) perfomed on image \"$temp\""
				}
			}

		}
		$topwin(analysis_name) configure -cursor top_left_arrow; update
	 } \
    -text {PERFORM}

  # build widget $topwin(analysis_name).perform.dismiss
	button $topwin(analysis_name).perform.dismiss -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(dis_act_bg)" \
    -background "$GuiColor(dis_bg)" \
    -foreground "$GuiColor(dis_fg)" \
    -command {destroy $topwin(analysis_name)} \
    -text {DISMISS} \
    -width {10}

  # pack widget $topwin(analysis_name).perform
  pack append $topwin(analysis_name).perform \
    $topwin(analysis_name).perform.dismiss {left frame sw} \
    $topwin(analysis_name).perform.perform1 {right frame se}

  # pack widget $topwin(analysis_name)
  pack append $topwin(analysis_name) \
    $topwin(analysis_name).menubar {top frame center fill} \
    $topwin(analysis_name).info {top frame center expand fill} \
    $topwin(analysis_name).perform {bottom frame center pady 5 fill} \
    $topwin(analysis_name).current_image {bottom frame center fill}

  if {"[info procs XFEdit]" != ""} {
    catch "XFMiscBindWidgetTree $topwin(analysis_name)"
    after 2 "catch {XFEditSetShowWindows}"
  }
}

