# **********************************************************************
# Computer Vision/Image Processing Tool Project - Dr. Scott Umbaugh SIUE
# **********************************************************************
#
#             File Name: ana_xform.tcl
#           Description: 
#         Related Files: 
#   Initial Coding Date: Wed Aug 28 17:15:21 CDT 1996
#           Portability: TCL script file
#            References:
#             Author(s): Yansheng Wei
#                        Southern Illinois University @ Edwardsville
#
## Copyright (C) 1996 SIUE - by Scott Umbaugh and Yansheng Wei.
##
## Permission to use, copy, modify, and distribute this software and its
## documentation for any purpose and without fee is hereby granted, provided
## that the above copyright notice appear in all copies and that both that
## copyright notice and this permission notice appear in supporting
## documentation.  This software is provided "as is" without express or
## implied warranty.
##
# RCS (Revision Control System) Information
# ...Added automatically by RCS 
#
## $Log: ana_xform.tcl,v $
## Revision 1.33  1997/07/22 20:00:12  wzheng
## DC button raised
##
## Revision 1.32  1997/07/17 14:23:08  yawei
## fixed wavelet bug
##
## Revision 1.31  1997/06/23 22:35:46  yawei
## changed blocksize setup
##
## Revision 1.30  1997/06/19 22:16:37  yawei
## added default size for 0 notch
##
## Revision 1.29  1997/06/19 22:07:11  yawei
## finished configuration
##
## Revision 1.28  1997/06/06 20:47:58  yawei
## anal ==> analysis
##
## Revision 1.27  1997/04/23 14:43:28  yawei
## removed the rows == cols check for wavelet
##
## Revision 1.26  1997/03/15 01:41:05  yawei
## Changed xformfilters to use FILTER_TYPE
##
## Revision 1.25  1997/03/12 00:01:21  yawei
## fixed upgrade bugs
##
## Revision 1.24  1997/03/11 20:55:30  yawei
## Tcl/Tk script upgrading from tcl7.3/tk3.6 to tcl7.6/tk4.2
## compatible. This process is automated with tkboost perl
## script. @ Tue Mar 11 14:51:36 CST 1997
##
## Revision 1.23  1996/12/17 00:19:27  yawei
## Changed background color of the PERFORM button in xform
##
# Revision 1.22  1996/12/14  00:51:54  yawei
# removed "remap" of the "extract_phase button"
#
# Revision 1.21  1996/12/13  22:04:47  yawei
# Added "fft_phase"
#
# Revision 1.20  1996/11/20  16:43:25  mzuke
# added check for square image to wavelet transform
#
# Revision 1.19  1996/10/24  17:16:55  mzuke
# changed one alert box prompt
#
# Revision 1.18  1996/10/23  18:49:27  yawei
# REmoved block_size checking for fft.
#
# Revision 1.17  1996/10/16  15:53:22  mzuke
# moved 'view image' after 'image_post' to correctly label
# displayed images;checked for too-long new image names, none changed
#
# Revision 1.16  1996/10/10  21:37:18  mzuke
# changed 'DISMISS' button color
#
# Revision 1.15  1996/10/08  14:21:22  mzuke
# added check for current_image to transform selection
# radiobuttons
#
# Revision 1.14  1996/10/07  18:16:56  mzuke
# added check for current_image to 'update_size' procedure
# to eliminate Tcl error box when selecting forward transform
# after selecting inverse transform
#
# Revision 1.13  1996/10/03  02:12:00  yawei
# changed the offset value range to 0.01-3.0
#
# Revision 1.12  1996/09/27  18:26:56  mzuke
# made window resizable
#
# Revision 1.11  1996/09/22  21:12:22  yawei
# changed debug info
#
# Revision 1.10  1996/09/03  18:04:54  yawei
# changed anal_ to analysis_
#
# Revision 1.9  1996/08/29  18:34:49  mzuke
#  changed font designation from {symbol} to full listing
#  to restore arrow labels after systemwide font changes
#
# Revision 1.8  1996/08/29  03:10:46  yawei
# removed the trailing 0 of blocksize.
#
# Revision 1.7  1996/08/28  22:16:00  yawei
# no changes, added the forgotten header.
#
#
#****************************************************************************
# procedure to show window $topwin(analysis_name).
proc analysis_xform { } {# xf ignore me 7

  # Window manager configurations
  global tk_version
  global topwin
  global toplevel_position
  global rows
  global analysis
  global {GuiColor}
  global {GuiMisc}
  global {GUI_DEBUG}

  set analysis(xform_dir)  1
  set analysis(xform_dir_wavelet) 1
  set analysis(xform_blocksize) 256
  set analysis(filter_type) Ideal
  if [ winfo exists  $topwin(analysis_name).info] {
     set tempname $topwin(analysis_name);\
destroy $topwin(analysis_name).info;\
set topwin(analysis_name) $tempname;}
  if [ winfo exists  $topwin(analysis_name).current_image] {
     set tempname $topwin(analysis_name);\
destroy $topwin(analysis_name).current_image;\
set topwin(analysis_name) $tempname;}
  if [ winfo exists  $topwin(analysis_name).perform] {
     set tempname $topwin(analysis_name);\
destroy $topwin(analysis_name).perform;\
set topwin(analysis_name) $tempname;}
  wm positionfrom  $topwin(analysis_name) user
  wm sizefrom  $topwin(analysis_name) user

  wm geometry $topwin(analysis_name) $topwin(analysis_transform)[get_pos $topwin(analysis_name)]
  wm maxsize $topwin(analysis_name) [lindex $topwin(analysis_transform_max) 0] \
  	[lindex $topwin(analysis_transform_max) 1]
  wm minsize $topwin(analysis_name) [lindex $topwin(analysis_transform_min) 0] \
  	[lindex $topwin(analysis_transform_min) 1]
  wm minsize  $topwin(analysis_name) 606 454

  wm title  $topwin(analysis_name) {ANALYSIS -> Transforms}

 # bindings
	bind $topwin(analysis_name) <Destroy> {set topwin(analysis_name) NONE}


  # build widget $topwin(analysis_name).current_image
  frame $topwin(analysis_name).current_image \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2} \
    -relief {ridge}

  # build widget $topwin(analysis_name).current_image.label15
  label $topwin(analysis_name).current_image.label15 \
    -background "$GuiColor(label_bg)" \
    -text {Current image: }

  # build widget $topwin(analysis_name).current_image.label16
  label $topwin(analysis_name).current_image.label16 \
    -background "$GuiColor(label_bg)" \
    -foreground {Blue} \
    -text {label16} \
    -textvariable {current_image}

  # pack widget $topwin(analysis_name).current_image
  pack append $topwin(analysis_name).current_image \
    $topwin(analysis_name).current_image.label15 {left frame center} \
    $topwin(analysis_name).current_image.label16 {left frame center}

  # build widget $topwin(analysis_name).perform
  frame $topwin(analysis_name).perform \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2} \
    -relief {raised}

  # build widget $topwin(analysis_name).perform.button17
	button $topwin(analysis_name).perform.button17 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(dis_act_bg)" \
    -background "$GuiColor(dis_bg)" \
    -foreground "$GuiColor(dis_fg)" \
	 -command {destroy $topwin(analysis_name)} \
    -text {DISMISS}

  # build widget $topwin(analysis_name).perform.frame0
  frame $topwin(analysis_name).perform.frame0 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2} \
    -relief {ridge}

  # build widget $topwin(analysis_name).perform.frame0.button2
	button $topwin(analysis_name).perform.frame0.button2 -padx {1} -pady {1} -highlightthickness {0}  \
    -background "$GuiColor(btn_bg)" \
    -activebackground "$GuiColor(btn_act_bg)" \
    -text {Extract phase} \
	 -command {
		$topwin(analysis_name) configure -cursor watch; update
      if {$current_image == ""} {
         CvipAlert "Please select an image first."
			$topwin(analysis_name) configure -cursor top_left_arrow; update
         return
      }  
		if { $data_fmt != "COMPLEX" } {
         CvipAlert "Phase extraction requires a COMPLEX image."
			$topwin(analysis_name) configure -cursor top_left_arrow; update
         return
		}

		set analysis(phase_norm) 1
		set temp "cvip fft_phase $current_image 1 -1"
		if { $analysis(phase_norm) == "0" } {
				if [catch {expr $analysis(phase_K)}] {
		         CvipAlert "Numerical values required for the remap value: \n \
						Remap value = $analysis(phase_K)"
					$topwin(analysis_name) configure -cursor top_left_arrow; update
		         return
     		   } elseif {$analysis(phase_K) <= 0 } {
            	CvipAlert "The remap value (=$analysis(phase_K)) should be a positive number"
					$topwin(analysis_name) configure -cursor top_left_arrow; update
	            return
  		      }
			set temp "cvip fft_phase $current_image 0 $analysis(phase_K)"
		}
		if [catch {set result [eval $temp]} cresult ] {
			CvipAlert "$cresult"
			$topwin(analysis_name) configure -cursor top_left_arrow; update
			return
		} else {
			if { $GUI_DEBUG == "1" } {
				puts $temp
			}
			set temp1 $current_image
			set new_name [cvip rename $result ${current_image}_phase]
			img_q insert end $new_name
			image_post $new_name
			if { $analysis(phase_norm) == "0" } {
				cvip view $new_name
				$topwin(analysis_name) configure -cursor top_left_arrow; update
				return
			}

      	if [catch {set result [cvip fft $new_name 0 -1]} cresult ] {
				CvipAlert "$cresult"
				$topwin(analysis_name) configure -cursor top_left_arrow; update
				return
			} else {
				if { $GUI_DEBUG == "1" } {
					puts "cvip fft $new_name 0 -1"
				}
				set temp [cvip data_remap $result 0]; cvip dq $result
				set new_name [cvip rename $temp ${new_name}_IFFT]
				img_q insert end $new_name
				image_post $new_name
				cvip view $new_name
				CvipMessage "phase image and inverse transform using the phase image performed\n\
					on image \"$temp1\""
       	}
			
       }
		$topwin(analysis_name) configure -cursor top_left_arrow; update
	 }

  # build widget $topwin(analysis_name).perform.frame0.checkbutton3
	checkbutton $topwin(analysis_name).perform.frame0.checkbutton3 -padx {1} -pady {1} -highlightthickness {0}  \
    -background "$GuiColor(btn_bg)" \
    -activebackground "$GuiColor(btn_act_bg)" \
    -relief {groove} \
    -text {normalize} \
    -variable {analysis(phase_norm)} \
	 -command {
	 	if { $analysis(phase_norm) == "0" } {
#  			pack append $topwin(analysis_name).perform.frame0 \
#			    $topwin(analysis_name).perform.frame0.entry4 {left frame center} \
#			    $topwin(analysis_name).perform.frame0.menubutton5 {left frame center} 
	 	} else {
#  			pack forget $topwin(analysis_name).perform.frame0.entry4 
#			pack forget $topwin(analysis_name).perform.frame0.menubutton5
		}
	 }

  # build widget $topwin(analysis_name).perform.frame0.entry4
  entry $topwin(analysis_name).perform.frame0.entry4 \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
    -textvariable {analysis(phase_K)} \
    -width {7}

  # build widget $topwin(analysis_name).perform.frame0.menubutton5
	menubutton $topwin(analysis_name).perform.frame0.menubutton5 -padx {1} -pady {1} -highlightthickness {0}  \
    -background "$GuiColor(btn_bg)" \
    -activebackground "$GuiColor(btn_act_bg)" \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)" \
    -text "$GuiMisc(pop_up_text)" \
	 -width {2}
  # bindings
  bind $topwin(analysis_name).perform.frame0.menubutton5 <Button-1> {MenuPopupPost $topwin(analysis_name).perform.frame0.menubutton5.menu %X %Y}
  bind $topwin(analysis_name).perform.frame0.menubutton5 <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).perform.frame0.menubutton5.menu %W}

  # build widget $topwin(analysis_name).perform.frame0.menubutton5.menu
  menu $topwin(analysis_name).perform.frame0.menubutton5.menu \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"
  $topwin(analysis_name).perform.frame0.menubutton5.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(phase_K) 1} \
    -label {  1 }
  $topwin(analysis_name).perform.frame0.menubutton5.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(phase_K) 2} \
    -label {  2 }
  $topwin(analysis_name).perform.frame0.menubutton5.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(phase_K) 5} \
    -label {  5 }
  $topwin(analysis_name).perform.frame0.menubutton5.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(phase_K) 10} \
    -label { 10 }
  $topwin(analysis_name).perform.frame0.menubutton5.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(phase_K) 100} \
    -label {100 }
  $topwin(analysis_name).perform.frame0.menubutton5.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(phase_K) 200} \
    -label {200 }
  bindtags $topwin(analysis_name).perform.frame0.menubutton5.menu {Menu all}
  # bindings
  bind $topwin(analysis_name).perform.frame0.menubutton5.menu <B1-Motion> {MenuPopupMotion $topwin(analysis_name).perform.frame0.menubutton5.menu0 %W %X %Y}
  bind $topwin(analysis_name).perform.frame0.menubutton5.menu <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).perform.frame0.menubutton5.menu0 %W}

  # pack widget $topwin(analysis_name).perform.frame0
  pack append $topwin(analysis_name).perform.frame0 \
    $topwin(analysis_name).perform.frame0.button2 {left frame center} 
#    $topwin(analysis_name).perform.frame0.checkbutton3 {left frame center} \
#    $topwin(analysis_name).perform.frame0.entry4 {left frame center} \
#    $topwin(analysis_name).perform.frame0.menubutton5 {left frame center}

  # build widget $topwin(analysis_name).perform.button0
	button $topwin(analysis_name).perform.button0 -padx {1} -pady {1} -highlightthickness {0}  \
	 -activebackground "$GuiColor(perf_act_bg)" \
	 -background "$GuiColor(perf_bg)" \
	 -foreground "$GuiColor(perf_fg)" \
    -text {PERFORM} \
	 -command {
		$topwin(analysis_name) configure -cursor watch; update
      if {$current_image == ""} {
         CvipAlert "Please select an image first."
			$topwin(analysis_name) configure -cursor top_left_arrow; update
         return
      }  
		switch -exact  -- $analysis(transform) {
			dct -
			fft -
			haar -
			hadamard -
			walsh {
				set temp [expr { $rows > $cols } ? $cols : $rows ]
				if { $analysis(transform) == "fft" } {
					set temp 2048
				}
				if {$analysis(xform_dir) == "0"} {
					set analysis(xform_blocksize) -1
				} elseif [catch {expr $analysis(xform_blocksize)}] {
		         CvipAlert "Numerical values required for the transform block size: \n \
						blocksize = $analysis(xform_blocksize)"
					$topwin(analysis_name) configure -cursor top_left_arrow; update
		         return
     		   } elseif {$analysis(xform_blocksize) <= 0 || $analysis(xform_blocksize) > $temp} {
            	CvipAlert "The transform block size (=$analysis(xform_blocksize)) should be \n\
					in the range (0 - [expr int(pow(2, floor(log($temp) / log(2.0))))])"
					$topwin(analysis_name) configure -cursor top_left_arrow; update
	            return
  		      }
				if { $GUI_DEBUG == "1" } {
            	puts "cvip $analysis(transform) $current_image $analysis(xform_dir) $analysis(xform_blocksize)"
				}
            if [catch {set result [cvip $analysis(transform) $current_image $analysis(xform_dir) $analysis(xform_blocksize)]} cresult] {
               CvipAlert "$cresult"
					$topwin(analysis_name) configure -cursor top_left_arrow; update
               return
            } else {
					set temp2 $current_image
					if { $analysis(xform_dir) == "1" } {
						set temp3 "" 
						set temp4 "\nwith blocksize = $analysis(xform_blocksize)"
					} else {
						set temp3 inverse
						set temp4 ""
					}
               if {$analysis(xform_dir) == "1"} {set temp1 ""} else {set temp1 "I"}
               set new_name [cvip rename $result ${current_image}_${temp1}$analysis(transform)]
               img_q insert end $new_name
               image_post $new_name
					CvipMessage "$temp3 $analysis(transform) transform performed on image \"$temp2\" $temp4"
               if {$analysis(xform_dir) == "1"} {
						cvip view $new_name l
					} else {
						cvip view $new_name
					}
           }

			}

			wavelet {
#				if {$rows != $cols}  {
#         		CvipAlert "This wavelet transform requires a square image"
#					$topwin(analysis_name) configure -cursor top_left_arrow; update
#					return
#				}
				if {$analysis(xform_dir_wavelet) == "0"} {
					set temp -1
					set temp1 -1
				} elseif {$analysis(xform_wavelet_basis) == "haar"} {
					set temp 1
					set temp1 $analysis(xform_wavelet_decomp)
				} else {
					set temp 2
					set temp1 $analysis(xform_wavelet_decomp)
				}
				set temp2 "" 
				if { $analysis(xform_dir_wavelet) != "1" } {
					set temp2 "inverse" 
				}
				set temp "cvip $analysis(transform) $current_image $analysis(xform_dir_wavelet) $temp $temp1"
            if [catch {set result [eval $temp]} cresult] {
               CvipAlert "$cresult"
					$topwin(analysis_name) configure -cursor top_left_arrow; update
               return
            } else {
               set new_name [cvip rename $result ${current_image}_wav]
               img_q insert end $new_name
               image_post $new_name
					CvipMessage "$temp2 wavelet transform performed on image \
					\"$current_image\" "
               if {$analysis(xform_dir_wavelet) == "1"} {
						cvip view $new_name l
					} else {
						cvip view $new_name
					}
            }
				if { $GUI_DEBUG == "1" } {
            	puts "cvip $analysis(transform) $current_image $analysis(xform_dir_wavelet) $temp $temp1"
				}

			}

			lpf_filter -
			hpf_filter {
				if [catch {expr $analysis(filter_lcf)} ] {
         		CvipAlert "Numerical value required for the cutoff frequency:\n\
					Cutoff frequency = $analysis(filter_lcf)."
					$topwin(analysis_name) configure -cursor top_left_arrow; update
					return
				} elseif {$analysis(filter_lcf) < 0 } {
         		CvipAlert "Cutoff frequency ($analysis(filter_lcf)) should be greater than or equal to 0."
					$topwin(analysis_name) configure -cursor top_left_arrow; update
					return
				}

				if {$analysis(filter_type) == "Ideal"} {
					if { $analysis(transform) == "lpf_filter" } {
						set temp IDEAL_LOW
					} else {
						set temp IDEAL_HIGH
					}
					set temp1 -1
				} else {
					if { $analysis(transform) == "lpf_filter" } {
						set temp BUTTER_LOW
					} else {
						set temp BUTTER_HIGH
					}
					set temp1 $analysis(filter_order)
				}

				set temp2 $current_image
            if [catch {set result [cvip $analysis(transform) $current_image $analysis(filter_dc) $temp $analysis(filter_lcf) $temp1]} cresult] {
               CvipAlert "$cresult"
					$topwin(analysis_name) configure -cursor top_left_arrow; update
               return
            } else {
               set new_name [cvip rename $result ${current_image}_$analysis(transform)]
               img_q insert end $new_name
               image_post $new_name
					cvip view $new_name l
            }
				if { $GUI_DEBUG == "1" } {
					puts "cvip $analysis(transform) $current_image $analysis(filter_dc) $temp $analysis(filter_lcf) $temp1]"
				}
				set temp [cvip history $new_name]
				if { [set temp [Hist_find $temp]] == "-1" }  {
         		CvipAlert "No proper history record for the inverse transform after $analysis(transform)."
					$topwin(analysis_name) configure -cursor top_left_arrow; update
					return
				} else {
					if {$temp == "wavelet" } {
						set temp "cvip $temp $new_name 0 -1 -1" 
					} else {
						set temp "cvip $temp $new_name 0 -1"
					}
					if { $GUI_DEBUG == "1" } {
						puts "$temp"
					}
            	if [catch {set result [eval $temp]} cresult] {
						CvipAlert "$cresult"
						$topwin(analysis_name) configure -cursor top_left_arrow; update
						return
	            } else {
						set new_name [cvip rename $result ${new_name}_Inv]
						img_q insert end $new_name
						image_post $new_name
						cvip view $new_name
						if { $analysis(transform) == "lpf_filter" } {
							CvipMessage "Low pass frequency filtering performed on image \"$temp2\" \n\
							 with cut-off frequency = $analysis(filter_lcf)"
						} else {
							CvipMessage "High pass frequency filtering performed on image \"$temp2\" \n\
							 with cut-off frequency = $analysis(filter_lcf)"
  			         }
					}
				}
			}
			bpf_filter -
			brf_filter {
				set temp2 $current_image
				if [catch {expr $analysis(filter_lcf) + $analysis(filter_hcf)} ] {
         		CvipAlert "Numerical value required for the cutoff frequency:\n\
					Lower cutoff frequency = $analysis(filter_lcf),\n\
					Higher cutoff frequency = $analysis(filter_hcf)."
					$topwin(analysis_name) configure -cursor top_left_arrow; update
					return 
				} elseif {$analysis(filter_lcf) < 0 || $analysis(filter_hcf) < 0} {
         		CvipAlert "Cutoff frequency should be greater than or equal to 0:\
					Lower cutoff frequency = $analysis(filter_lcf),\n\
					Higher cutoff frequency = $analysis(filter_hcf)."
					$topwin(analysis_name) configure -cursor top_left_arrow; update
					return
				} elseif {$analysis(filter_lcf) >= $analysis(filter_hcf)} {
         		CvipAlert "Lower cutoff frequency (= $analysis(filter_lcf)) should be less than \
					higher cutoff frequency = $analysis(filter_hcf)."
					$topwin(analysis_name) configure -cursor top_left_arrow; update
					return
				}
				if {$analysis(filter_type) == "Ideal"} {
					if { $analysis(transform) == "bpf_filter" } {
						set temp IDEAL_BAND
					} else {
						set temp IDEAL_REJECT
					}
					set temp1 -1
				} else {
					if { $analysis(transform) == "bpf_filter" } {
						set temp BUTTER_BAND
					} else {
						set temp BUTTER_REJECT
					}
					set temp1 $analysis(filter_order)
				}
            if [catch {set result [cvip $analysis(transform) $current_image $analysis(filter_dc) $temp $analysis(filter_lcf) $analysis(filter_hcf) $temp1]} cresult] {
               CvipAlert "$cresult"
					$topwin(analysis_name) configure -cursor top_left_arrow; update
               return
            } else {
               set new_name [cvip rename $result ${current_image}_$analysis(transform)]
               img_q insert end $new_name
               image_post $new_name
 		cvip view $new_name l
           }
				if { $GUI_DEBUG == "1" } { puts "cvip $analysis(transform) $current_image $analysis(filter_dc) \
					$temp $analysis(filter_lcf) $analysis(filter_hcf) $temp1"
				}
				set temp [cvip history $new_name]
				if { [set temp [Hist_find $temp]] == "-1" }  {
         		CvipAlert "No proper history record for the inverse transform after $analysis(transform)."
					$topwin(analysis_name) configure -cursor top_left_arrow; update
					return
				} else {
					if {$temp == "wavelet" } {
						set temp "cvip $temp $new_name 0 -1 -1" 
					} else {
						set temp "cvip $temp $new_name 0 -1"
					}
            	if [catch {set result [eval $temp]} cresult] {
						CvipAlert "$cresult"
						$topwin(analysis_name) configure -cursor top_left_arrow; update
						return
	            } else {
						set new_name [cvip rename $result ${new_name}_Inv]
						img_q insert end $new_name
						image_post $new_name
						cvip view $new_name
						if { $analysis(transform) == "bpf_filter" } {
							CvipMessage "Band-pass frequency filtering performed on image \"$temp2\" \n\
							 with lower cut-off = $analysis(filter_lcf), and higher cut-off = $analysis(filter_hcf)"
						} else {
							CvipMessage "Band-reject frequency filtering performed on image \"$temp2\" \n\
							 with lower cut-off = $analysis(filter_lcf), and higher cut-off = $analysis(filter_hcf)"
						}
  	          }
				}
			}
			hfe_filter {
				set temp1 $current_image
				if [catch {expr $analysis(filter_lcf)} ] {
         		CvipAlert "Numerical value required for the cutoff frequency:\n\
					Cutoff frequency = $analysis(filter_lcf)."
					$topwin(analysis_name) configure -cursor top_left_arrow; update
					return
				} elseif {$analysis(filter_lcf) < 0 } {
         		CvipAlert "Cutoff frequency ($analysis(filter_lcf)) should be greater than or equal to 0."
					$topwin(analysis_name) configure -cursor top_left_arrow; update
					return
				}
				if [catch {expr $analysis(filter_offset)} ] {
         		CvipAlert "Numerical value required for the offset value:\n\
					Offset value = $analysis(filter_offset)."
					$topwin(analysis_name) configure -cursor top_left_arrow; update
					return
				} elseif {$analysis(filter_offset) < "0.01" || $analysis(filter_offset) > "3.0"} {
         		CvipAlert "Offset value ($analysis(filter_offset)) should be in the range (0.01-3.0)"
					$topwin(analysis_name) configure -cursor top_left_arrow; update
					return
				}
            if [catch {set result [cvip $analysis(transform) $current_image $analysis(filter_dc) $analysis(filter_lcf) $analysis(filter_offset) $analysis(filter_order)]} cresult] {
               CvipAlert "$cresult"
					$topwin(analysis_name) configure -cursor top_left_arrow; update
               return
            } else {
               set new_name [cvip rename $result ${current_image}_$analysis(transform)]
               img_q insert end $new_name
               image_post $new_name
		cvip view $new_name
            }
				if { $GUI_DEBUG == "1" } { puts "cvip $analysis(transform) $current_image $analysis(filter_dc) \
					$analysis(filter_lcf) $analysis(filter_offset) $analysis(filter_order)"
				}
				set temp [cvip history $new_name]
				if { [set temp [Hist_find $temp]] == "-1" }  {
         		CvipAlert "No proper history record for the inverse transform after $analysis(transform)."
					$topwin(analysis_name) configure -cursor top_left_arrow; update
					return
				} else {
					if {$temp == "wavelet" } {
						set temp "cvip $temp $new_name 0 -1 -1" 
					} else {
						set temp "cvip $temp $new_name 0 -1"
					}
            	if [catch {set result [eval $temp]} cresult] {
						CvipAlert "$cresult"
						$topwin(analysis_name) configure -cursor top_left_arrow; update
						return
	            } else {
						set new_name [cvip rename $result ${new_name}_Inv]
						img_q insert end $new_name
						image_post $new_name
						cvip view $new_name
						CvipMessage "High frequency emphasis filtering performed on image \"$temp2\" \n\
							 with cut-off = $analysis(filter_lcf), and emphasis offset  = $analysis(filter_offset)"
  	          }
				}

			}
			notch {
				if [ catch {set temp [selection get] } ] {
			   } elseif  { [scan $temp "%s %s %s %s %s %s" temp1 temp5 temp2 temp3 temp4 temp6] == "6" && \
                ( $temp1 == "$current_image" || $temp1 == "\[log_power\]$current_image" ) } {
					if { $temp4 > $temp6 } { set temp4 $temp6 }
			      set analysis(filter_row)  [ expr $temp2 + $temp4 / 2 ]
			      set analysis(filter_col)  [ expr $temp3 + $temp4 / 2 ]
			      set analysis(filter_size)  $temp4
					if { $analysis(filter_size) == "0" } { incr analysis(filter_size)}
					selection clear; update } ] {
			   }

				set temp1 $current_image
				if [catch {expr $analysis(filter_row) + $analysis(filter_col) + $analysis(filter_size)} ] {
         		CvipAlert "Numerical value required for the notch filter box:\n\
					Row = $analysis(filter_row), Col = $analysis(filter_col), Size = $analysis(filter_size)."
					$topwin(analysis_name) configure -cursor top_left_arrow; update
					return
				} elseif {$analysis(filter_row) < "0" || $analysis(filter_row) > $rows - 1 } {
         		CvipAlert "The number of row ($analysis(filter_row)) should be in the range (0 - [expr $rows-1])."
					$topwin(analysis_name) configure -cursor top_left_arrow; update
					return
				} elseif {$analysis(filter_col) < 0 || $analysis(filter_col) > $cols -1 } {
         		CvipAlert "The number of col ($analysis(filter_col)) should be in the range (0 - [expr $cols-1])."
					$topwin(analysis_name) configure -cursor top_left_arrow; update
					return
				} elseif {$analysis(filter_size) <= 0} {
         		CvipAlert "The filter size ($analysis(filter_size)) should be a positive number."
					$topwin(analysis_name) configure -cursor top_left_arrow; update
					return
				}
				set temp3 [list $analysis(filter_row) $analysis(filter_col) $analysis(filter_size) ]
				set temp2 [list cvip $analysis(transform) $current_image $temp3]
            if [catch {set result [eval $temp2 ] } cresult] {
               CvipAlert "$cresult"
					$topwin(analysis_name) configure -cursor top_left_arrow; update
               return
            } else {
               set new_name [cvip rename $result ${current_image}_$analysis(transform)]
					cvip view $new_name l
               img_q insert end $new_name
               image_post $new_name
            }
				if { $GUI_DEBUG == "1" } { puts "eval $temp2" }
				set temp [cvip history $new_name]
				if { [set temp [Hist_find $temp]] == "-1" }  {
         		CvipAlert "No proper history record for the inverse transform after $analysis(transform)."
					$topwin(analysis_name) configure -cursor top_left_arrow; update
					return
				} else {
					if {$temp == "wavelet" } {
						set temp "cvip $temp $new_name 0 -1 -1" 
					} else {
						set temp "cvip $temp $new_name 0 -1"
					}
            	if [catch {set result [eval $temp]} cresult] {
						CvipAlert "$cresult"
						$topwin(analysis_name) configure -cursor top_left_arrow; update
						return
	            } else {
						set new_name [cvip rename $result ${new_name}_Inv]
						img_q insert end $new_name
						image_post $new_name
						cvip view $new_name
						CvipMessage "notch filtering performed on image \"$temp2\" at \n\
							row = $analysis(filter_row), col = $analysis(filter_col), and notch size = $analysis(filter_size)"
  	          }
				}

			}
		}

	 $topwin(analysis_name) configure -cursor top_left_arrow; update
	 } \
    -width {9}

  # pack widget $topwin(analysis_name).perform
  pack append $topwin(analysis_name).perform \
    $topwin(analysis_name).perform.button17 {left frame center filly} \
    $topwin(analysis_name).perform.button0 {right frame center filly} \
    $topwin(analysis_name).perform.frame0 {left frame center expand}


  # build widget $topwin(analysis_name).info
  frame $topwin(analysis_name).info \
    -background "$GuiColor(frame_bg)" 

  # build widget $topwin(analysis_name).info.frame3
  frame $topwin(analysis_name).info.frame3 \
    -background "$GuiColor(frame_bg)"  \
    -borderwidth {2} \
    -relief {ridge}

  # build widget $topwin(analysis_name).info.frame3.frame5
  frame $topwin(analysis_name).info.frame3.frame5 \
    -background "$GuiColor(frame_bg)"  \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame3.frame5.radiobutton7
	radiobutton $topwin(analysis_name).info.frame3.frame5.radiobutton7 -padx {1} -pady {1} -highlightthickness {0}  \
    -background "$GuiColor(btn_bg)" \
    -activebackground "$GuiColor(btn_act_bg)" \
    -command {
	 pack forget $topwin(analysis_name).perform.frame0
	 if {$current_image == ""} {
         	AlertBox "Please load an image." "" "350x150" "ALERT!" "DISMISS"
        	 return
      	 }
	 set temp [expr {$rows < $cols} ? $rows : $cols]
	 if {$temp > 0} {
   	 set analysis(xform_blocksize) [ expr int(pow(2, floor(log($temp)/log(2))))]
	 }} \
    -relief {flat} \
    -text {DCT            } \
    -value {dct} \
    -variable {analysis(transform)}

  # build widget $topwin(analysis_name).info.frame3.frame5.radiobutton8
	radiobutton $topwin(analysis_name).info.frame3.frame5.radiobutton8 -padx {1} -pady {1} -highlightthickness {0}  \
    -background "$GuiColor(btn_bg)" \
    -activebackground "$GuiColor(btn_act_bg)" \
    -command {
  	pack append $topwin(analysis_name).perform \
    $topwin(analysis_name).perform.frame0 {left frame center expand}

	 if {$current_image == ""} {
         	AlertBox "Please load an image." "" "350x150" "ALERT!" "DISMISS"
        	 return
      	 }
	 set temp [expr {$rows < $cols} ? $rows : $cols]
	 if {$temp > 0} {
   	 set analysis(xform_blocksize) [ expr int(pow(2, floor(log($temp)/log(2))))]
	 }} \
    -relief {flat} \
    -text {FFT             } \
    -value {fft} \
    -variable {analysis(transform)}

  # build widget $topwin(analysis_name).info.frame3.frame5.radiobutton9
	radiobutton $topwin(analysis_name).info.frame3.frame5.radiobutton9 -padx {1} -pady {1} -highlightthickness {0}  \
    -background "$GuiColor(btn_bg)" \
    -activebackground "$GuiColor(btn_act_bg)" \
    -command {
	 pack forget $topwin(analysis_name).perform.frame0
	 if {$current_image == ""} {
         	AlertBox "Please load an image." "" "350x150" "ALERT!" "DISMISS"
        	 return
      	 }
	 set temp [expr {$rows < $cols} ? $rows : $cols]
	 if {$temp > 0} {
   	 set analysis(xform_blocksize) [ expr int(pow(2, floor(log($temp)/log(2))))]
	 }} \
    -relief {flat} \
    -text {HAAR          } \
    -value {haar} \
    -variable {analysis(transform)}

  # build widget $topwin(analysis_name).info.frame3.frame5.radiobutton10
	radiobutton $topwin(analysis_name).info.frame3.frame5.radiobutton10 -padx {1} -pady {1} -highlightthickness {0}  \
    -background "$GuiColor(btn_bg)" \
    -activebackground "$GuiColor(btn_act_bg)" \
    -command {
	 pack forget $topwin(analysis_name).perform.frame0
	 if {$current_image == ""} {
         	AlertBox "Please load an image." "" "350x150" "ALERT!" "DISMISS"
        	 return
      	 }
	 set temp [expr {$rows < $cols} ? $rows : $cols]
	 if {$temp > 0} {
   	 set analysis(xform_blocksize) [ expr int(pow(2, floor(log($temp)/log(2))))]
	 }} \
    -relief {flat} \
    -text {WALSH        } \
    -value {walsh} \
    -variable {analysis(transform)}

  # build widget $topwin(analysis_name).info.frame3.frame5.radiobutton11
	radiobutton $topwin(analysis_name).info.frame3.frame5.radiobutton11 -padx {1} -pady {1} -highlightthickness {0}  \
    -background "$GuiColor(btn_bg)" \
    -activebackground "$GuiColor(btn_act_bg)" \
    -command {
	 pack forget $topwin(analysis_name).perform.frame0
	 if {$current_image == ""} {
         	AlertBox "Please load an image." "" "350x150" "ALERT!" "DISMISS"
        	 return
      	 }
	 set temp [expr {$rows < $cols} ? $rows : $cols]
	 if {$temp > 0} {
   	 set analysis(xform_blocksize) [ expr int(pow(2, floor(log($temp)/log(2))))]
	 }} \
    -relief {flat} \
    -text {HADAMARD} \
    -value {hadamard} \
    -variable {analysis(transform)}

  # pack widget $topwin(analysis_name).info.frame3.frame5
  pack append $topwin(analysis_name).info.frame3.frame5 \
    $topwin(analysis_name).info.frame3.frame5.radiobutton7 {top frame center} \
    $topwin(analysis_name).info.frame3.frame5.radiobutton8 {top frame center} \
    $topwin(analysis_name).info.frame3.frame5.radiobutton9 {top frame center} \
    $topwin(analysis_name).info.frame3.frame5.radiobutton10 {top frame center} \
    $topwin(analysis_name).info.frame3.frame5.radiobutton11 {top frame center}

  # build widget $topwin(analysis_name).info.frame3.frame6
  frame $topwin(analysis_name).info.frame3.frame6 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame3.frame6.frame13
  frame $topwin(analysis_name).info.frame3.frame6.frame13 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame3.frame6.frame13.radiobutton15
	radiobutton $topwin(analysis_name).info.frame3.frame6.frame13.radiobutton15 -padx {1} -pady {1} -highlightthickness {0}  \
    -background "$GuiColor(btn_bg)" \
    -activebackground "$GuiColor(btn_act_bg)" \
    -command " pack $topwin(analysis_name).info.frame3.frame6.frame14 -side right -padx 30
	 update_size" \
    -relief {flat} \
    -text {Forward} \
    -value {1} \
    -variable {analysis(xform_dir)}

  # build widget $topwin(analysis_name).info.frame3.frame6.frame13.radiobutton16
	radiobutton $topwin(analysis_name).info.frame3.frame6.frame13.radiobutton16 -padx {1} -pady {1} -highlightthickness {0}  \
    -background "$GuiColor(btn_bg)" \
    -activebackground "$GuiColor(btn_act_bg)" \
    -command "
		pack forget $topwin(analysis_name).info.frame3.frame6.frame14 
		set analysis(xform_blocksize) -1" \
    -relief {flat} \
    -text {Inverse } \
    -value {0} \
    -variable {analysis(xform_dir)}

  # pack widget $topwin(analysis_name).info.frame3.frame6.frame13
  pack append $topwin(analysis_name).info.frame3.frame6.frame13 \
    $topwin(analysis_name).info.frame3.frame6.frame13.radiobutton15 {top frame s expand} \
    $topwin(analysis_name).info.frame3.frame6.frame13.radiobutton16 {top frame n expand}

  # build widget $topwin(analysis_name).info.frame3.frame6.frame14
  frame $topwin(analysis_name).info.frame3.frame6.frame14 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame3.frame6.frame14.frame0
  frame $topwin(analysis_name).info.frame3.frame6.frame14.frame0 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame3.frame6.frame14.frame0.label1
  label $topwin(analysis_name).info.frame3.frame6.frame14.frame0.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -text {Blocksize:}

  # build widget $topwin(analysis_name).info.frame3.frame6.frame14.frame0.value
  entry $topwin(analysis_name).info.frame3.frame6.frame14.frame0.value \
    -background "$GuiColor(rentry_bg)" \
    -foreground "$GuiColor(rentry_fg)" \
    -relief {sunken} \
    -state {disabled} \
    -textvariable {analysis(xform_blocksize)} \
    -width {5}

  # build widget $topwin(analysis_name).info.frame3.frame6.frame14.frame0.menubutton2
	menubutton $topwin(analysis_name).info.frame3.frame6.frame14.frame0.menubutton2 -padx {1} -pady {1} -highlightthickness {0}  \
    -background "$GuiColor(btn_bg)" \
    -activebackground "$GuiColor(btn_act_bg)" \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)" \
    -text "$GuiMisc(pop_up_text)" \
	 -width {2}
  # bindings
  bind $topwin(analysis_name).info.frame3.frame6.frame14.frame0.menubutton2 <Button-1> {MenuPopupPost $topwin(analysis_name).info.frame3.frame6.frame14.frame0.menubutton2.menu %X %Y}
  bind $topwin(analysis_name).info.frame3.frame6.frame14.frame0.menubutton2 <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame3.frame6.frame14.frame0.menubutton2.menu %W}

  # build widget $topwin(analysis_name).info.frame3.frame6.frame14.frame0.menubutton2.menu
  menu $topwin(analysis_name).info.frame3.frame6.frame14.frame0.menubutton2.menu \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"
  $topwin(analysis_name).info.frame3.frame6.frame14.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(xform_blocksize) 2} \
    -label {    2 }
  $topwin(analysis_name).info.frame3.frame6.frame14.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(xform_blocksize) 4} \
    -label {    4 }
  $topwin(analysis_name).info.frame3.frame6.frame14.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(xform_blocksize) 8} \
    -label {    8 }
  $topwin(analysis_name).info.frame3.frame6.frame14.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(xform_blocksize) 16} \
    -label {   16 }
  $topwin(analysis_name).info.frame3.frame6.frame14.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(xform_blocksize) 32} \
    -label {   32 }
  $topwin(analysis_name).info.frame3.frame6.frame14.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(xform_blocksize) 64} \
    -label {   64 }
  $topwin(analysis_name).info.frame3.frame6.frame14.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(xform_blocksize) 128} \
    -label {  128 }
  $topwin(analysis_name).info.frame3.frame6.frame14.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(xform_blocksize) 256} \
    -label {  256 }
  $topwin(analysis_name).info.frame3.frame6.frame14.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(xform_blocksize) 512} \
    -label {  512 }
  $topwin(analysis_name).info.frame3.frame6.frame14.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(xform_blocksize) 1024} \
    -label { 1024 }
  bindtags $topwin(analysis_name).info.frame3.frame6.frame14.frame0.menubutton2.menu {Menu all}
  # bindings
  bind $topwin(analysis_name).info.frame3.frame6.frame14.frame0.menubutton2.menu <B1-Motion> {MenuPopupMotion $topwin(analysis_name).info.frame3.frame6.frame14.frame0.menubutton2.menu %W %X %Y}
  bind $topwin(analysis_name).info.frame3.frame6.frame14.frame0.menubutton2.menu <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame3.frame6.frame14.frame0.menubutton2.menu %W}

  # pack widget $topwin(analysis_name).info.frame3.frame6.frame14.frame0
  pack append $topwin(analysis_name).info.frame3.frame6.frame14.frame0 \
    $topwin(analysis_name).info.frame3.frame6.frame14.frame0.label1 {left frame center fill} \
    $topwin(analysis_name).info.frame3.frame6.frame14.frame0.value {left frame center expand fill} \
    $topwin(analysis_name).info.frame3.frame6.frame14.frame0.menubutton2 {right frame center fill}

  # pack widget $topwin(analysis_name).info.frame3.frame6.frame14
  pack append $topwin(analysis_name).info.frame3.frame6.frame14 \
    $topwin(analysis_name).info.frame3.frame6.frame14.frame0 {top frame center expand}

  # pack widget $topwin(analysis_name).info.frame3.frame6
  pack append $topwin(analysis_name).info.frame3.frame6 \
    $topwin(analysis_name).info.frame3.frame6.frame13 {left frame center padx 60 fill} \
    $topwin(analysis_name).info.frame3.frame6.frame14 {right frame center padx 60}

  # pack widget $topwin(analysis_name).info.frame3
  pack append $topwin(analysis_name).info.frame3 \
    $topwin(analysis_name).info.frame3.frame5 {left frame center filly} \
    $topwin(analysis_name).info.frame3.frame6 {right frame center expand fill}

  # build widget $topwin(analysis_name).info.frame4
  frame $topwin(analysis_name).info.frame4 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2} \
    -relief {ridge}

  # build widget $topwin(analysis_name).info.frame4.radiobutton12
	radiobutton $topwin(analysis_name).info.frame4.radiobutton12 -padx {1} -pady {1} -highlightthickness {0}  \
    -background "$GuiColor(btn_bg)" \
    -activebackground "$GuiColor(btn_act_bg)" \
	 -command {
	 	pack forget $topwin(analysis_name).perform.frame0
	 } \
    -padx {5} \
    -relief {flat} \
    -text {WAVELET } \
    -value {wavelet} \
    -variable {analysis(transform)}

  # build widget $topwin(analysis_name).info.frame4.frame13
  frame $topwin(analysis_name).info.frame4.frame13 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame4.frame13.radiobutton15
	radiobutton $topwin(analysis_name).info.frame4.frame13.radiobutton15 -padx {1} -pady {1} -highlightthickness {0}  \
    -background "$GuiColor(btn_bg)" \
    -activebackground "$GuiColor(btn_act_bg)" \
    -command "
		pack $topwin(analysis_name).info.frame4.frame1 -side right
		pack $topwin(analysis_name).info.frame4.frame0 -side right" \
    -relief {flat} \
    -text {Forward} \
    -value {1} \
    -variable {analysis(xform_dir_wavelet)}

  # build widget $topwin(analysis_name).info.frame4.frame13.radiobutton16
	radiobutton $topwin(analysis_name).info.frame4.frame13.radiobutton16 -padx {1} -pady {1} -highlightthickness {0}  \
    -background "$GuiColor(btn_bg)" \
    -activebackground "$GuiColor(btn_act_bg)" \
    -command "
		pack forget $topwin(analysis_name).info.frame4.frame1
		pack forget $topwin(analysis_name).info.frame4.frame0" \
    -relief {flat} \
    -text {Inverse } \
    -value {0} \
    -variable {analysis(xform_dir_wavelet)}

  # pack widget $topwin(analysis_name).info.frame4.frame13
  pack append $topwin(analysis_name).info.frame4.frame13 \
    $topwin(analysis_name).info.frame4.frame13.radiobutton15 {top frame s expand} \
    $topwin(analysis_name).info.frame4.frame13.radiobutton16 {top frame n expand}

  # build widget $topwin(analysis_name).info.frame4.frame0
  frame $topwin(analysis_name).info.frame4.frame0 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame4.frame0.label1
  label $topwin(analysis_name).info.frame4.frame0.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -text {Basis:}

  # build widget $topwin(analysis_name).info.frame4.frame0.value
  entry $topwin(analysis_name).info.frame4.frame0.value \
    -background "$GuiColor(rentry_bg)" \
    -foreground "$GuiColor(rentry_fg)" \
    -relief {sunken} \
    -state {disabled} \
    -textvariable {analysis(xform_wavelet_basis)} \
    -width {6}

  # build widget $topwin(analysis_name).info.frame4.frame0.menubutton2
	menubutton $topwin(analysis_name).info.frame4.frame0.menubutton2 -padx {1} -pady {1} -highlightthickness {0}  \
    -background "$GuiColor(btn_bg)" \
    -activebackground "$GuiColor(btn_act_bg)" \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)" \
    -text "$GuiMisc(pop_up_text)" \
	 -width {2}
  # bindings
  bind $topwin(analysis_name).info.frame4.frame0.menubutton2 <Button-1> {MenuPopupPost $topwin(analysis_name).info.frame4.frame0.menubutton2.menu %X %Y}
  bind $topwin(analysis_name).info.frame4.frame0.menubutton2 <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame4.frame0.menubutton2.menu %W}

  # build widget $topwin(analysis_name).info.frame4.frame0.menubutton2.menu
  menu $topwin(analysis_name).info.frame4.frame0.menubutton2.menu \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"
  $topwin(analysis_name).info.frame4.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(xform_wavelet_basis) haar} \
    -label {haar}
  $topwin(analysis_name).info.frame4.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(xform_wavelet_basis) daub4} \
    -label {daub4}
  bindtags $topwin(analysis_name).info.frame4.frame0.menubutton2.menu  {Menu all}
  # bindings
  bind $topwin(analysis_name).info.frame4.frame0.menubutton2.menu <B1-Motion> {MenuPopupMotion $topwin(analysis_name).info.frame4.frame0.menubutton2.menu %W %X %Y}
  bind $topwin(analysis_name).info.frame4.frame0.menubutton2.menu <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame4.frame0.menubutton2.menu %W}

  # pack widget $topwin(analysis_name).info.frame4.frame0
  pack append $topwin(analysis_name).info.frame4.frame0 \
    $topwin(analysis_name).info.frame4.frame0.label1 {left frame center fill} \
    $topwin(analysis_name).info.frame4.frame0.value {left frame center expand fill} \
    $topwin(analysis_name).info.frame4.frame0.menubutton2 {right frame center fill}

  # build widget $topwin(analysis_name).info.frame4.frame1
  frame $topwin(analysis_name).info.frame4.frame1 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame4.frame1.label1
  label $topwin(analysis_name).info.frame4.frame1.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -relief {sunken} \
    -text {Decompositions:}

  # build widget $topwin(analysis_name).info.frame4.frame1.value
  entry $topwin(analysis_name).info.frame4.frame1.value \
    -background "$GuiColor(rentry_bg)" \
    -foreground "$GuiColor(rentry_fg)" \
    -relief {sunken} \
    -state {disabled} \
    -textvariable {analysis(xform_wavelet_decomp)} \
    -width {4}

  # build widget $topwin(analysis_name).info.frame4.frame1.menubutton2
	menubutton $topwin(analysis_name).info.frame4.frame1.menubutton2 -padx {1} -pady {1} -highlightthickness {0}  \
    -background "$GuiColor(btn_bg)" \
    -activebackground "$GuiColor(btn_act_bg)" \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)" \
    -text "$GuiMisc(pop_up_text)" \
	 -width {2}
  # bindings
  bind $topwin(analysis_name).info.frame4.frame1.menubutton2 <Button-1> {MenuPopupPost $topwin(analysis_name).info.frame4.frame1.menubutton2.menu %X %Y}
  bind $topwin(analysis_name).info.frame4.frame1.menubutton2 <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame4.frame1.menubutton2.menu %W}

  # build widget $topwin(analysis_name).info.frame4.frame1.menubutton2.menu
  menu $topwin(analysis_name).info.frame4.frame1.menubutton2.menu \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"
  $topwin(analysis_name).info.frame4.frame1.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(xform_wavelet_decomp) 1} \
    -label { 1}
  $topwin(analysis_name).info.frame4.frame1.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(xform_wavelet_decomp) 2} \
    -label { 2}
  $topwin(analysis_name).info.frame4.frame1.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(xform_wavelet_decomp) 3} \
    -label { 3}
  $topwin(analysis_name).info.frame4.frame1.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(xform_wavelet_decomp) 4} \
    -label { 4 }
  $topwin(analysis_name).info.frame4.frame1.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(xform_wavelet_decomp) 5} \
    -label { 5 }
  $topwin(analysis_name).info.frame4.frame1.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(xform_wavelet_decomp) 6} \
    -label { 6 }
  $topwin(analysis_name).info.frame4.frame1.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(xform_wavelet_decomp) 7} \
    -label { 7 }
  $topwin(analysis_name).info.frame4.frame1.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(xform_wavelet_decomp) 8} \
    -label { 8 }
  $topwin(analysis_name).info.frame4.frame1.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(xform_wavelet_decomp) 9} \
    -label { 9 }
  bindtags $topwin(analysis_name).info.frame4.frame1.menubutton2.menu  {Menu all}
  # bindings
  bind $topwin(analysis_name).info.frame4.frame1.menubutton2.menu <B1-Motion> {MenuPopupMotion $topwin(analysis_name).info.frame4.frame1.menubutton2.menu %W %X %Y}
  bind $topwin(analysis_name).info.frame4.frame1.menubutton2.menu <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame4.frame1.menubutton2.menu %W}

  # pack widget $topwin(analysis_name).info.frame4.frame1
  pack append $topwin(analysis_name).info.frame4.frame1 \
    $topwin(analysis_name).info.frame4.frame1.label1 {left frame center fill} \
    $topwin(analysis_name).info.frame4.frame1.value {left frame center expand fill} \
    $topwin(analysis_name).info.frame4.frame1.menubutton2 {right frame center fill}

  # pack widget $topwin(analysis_name).info.frame4
  pack append $topwin(analysis_name).info.frame4 \
    $topwin(analysis_name).info.frame4.radiobutton12 {left frame center} \
    $topwin(analysis_name).info.frame4.frame13 {left frame center filly} \
    $topwin(analysis_name).info.frame4.frame1 {right frame center}\
    $topwin(analysis_name).info.frame4.frame0 {right frame center} 

  # build widget $topwin(analysis_name).info.frame5
  frame $topwin(analysis_name).info.frame5 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2} \
    -relief {ridge}

  # build widget $topwin(analysis_name).info.frame5.frame0
  frame $topwin(analysis_name).info.frame5.frame0 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2} \
    -relief {ridge}

  # build widget $topwin(analysis_name).info.frame5.frame0.frame0
  frame $topwin(analysis_name).info.frame5.frame0.frame0 \
    -background "$GuiColor(frame_bg)" \
    -relief {ridge}

  # build widget $topwin(analysis_name).info.frame5.frame0.frame0.radiobutton18
	radiobutton $topwin(analysis_name).info.frame5.frame0.frame0.radiobutton18 -padx {1} -pady {1} -highlightthickness {0}  \
    -background "$GuiColor(btn_bg)" \
    -activebackground "$GuiColor(btn_act_bg)" \
    -relief {flat} \
    -text {Low pass} \
    -value {lpf_filter} \
    -variable {analysis(transform)} \
    -width {8}

  # build widget $topwin(analysis_name).info.frame5.frame0.frame0.radiobutton19
	radiobutton $topwin(analysis_name).info.frame5.frame0.frame0.radiobutton19 -padx {1} -pady {1} -highlightthickness {0}  \
    -background "$GuiColor(btn_bg)" \
    -activebackground "$GuiColor(btn_act_bg)" \
    -relief {flat} \
    -text {High pass} \
    -value {hpf_filter} \
    -variable {analysis(transform)} \
    -width {11}

  # build widget $topwin(analysis_name).info.frame5.frame0.frame0.frame0
  frame $topwin(analysis_name).info.frame5.frame0.frame0.frame0 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame5.frame0.frame0.frame0.label1
  label $topwin(analysis_name).info.frame5.frame0.frame0.frame0.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -text {Cutoff Frequency: }

  # build widget  $topwin(analysis_name).info.frame5.frame0.frame0.frame0.frame1
  frame  $topwin(analysis_name).info.frame5.frame0.frame0.frame0.frame1 \
    -borderwidth {0} \
    -highlightthickness {0} \
	 -background "$GuiColor(frame_bg)" \
    -height {30} \
    -width {30}

  # build widget  $topwin(analysis_name).info.frame5.frame0.frame0.frame0.frame1.entry
  entry  $topwin(analysis_name).info.frame5.frame0.frame0.frame0.frame1.entry \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
    -textvariable {analysis(filter_lcf)} -width {5}
  endEntry $topwin(analysis_name).info.frame5.frame0.frame0.frame0.frame1.entry 

  # build widget  $topwin(analysis_name).info.frame5.frame0.frame0.frame0.frame1.menubutton7
  menubutton  $topwin(analysis_name).info.frame5.frame0.frame0.frame0.frame1.menubutton7 \
	 -activebackground "$GuiColor(btn_act_bg)" -background "$GuiColor(btn_bg)"  \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)" \
    -padx {1} -pady {1} -highlightthickness {0} \
    -text "$GuiMisc(pop_up_text)" -width {2}

  # bindings
  bind  $topwin(analysis_name).info.frame5.frame0.frame0.frame0.frame1.menubutton7 <Button-1> { MenuPopupPost  $topwin(analysis_name).info.frame5.frame0.frame0.frame0.frame1.menubutton7.menu0 %X %Y}
  bind  $topwin(analysis_name).info.frame5.frame0.frame0.frame0.frame1.menubutton7 <ButtonRelease-1> {MenuPopupRelease  $topwin(analysis_name).info.frame5.frame0.frame0.frame0.frame1.menubutton7.menu0 %W }

  # build widget  $topwin(analysis_name).info.frame5.frame0.frame0.frame0.frame1.menubutton7.menu0
  menu  $topwin(analysis_name).info.frame5.frame0.frame0.frame0.frame1.menubutton7.menu0 
   $topwin(analysis_name).info.frame5.frame0.frame0.frame0.frame1.menubutton7.menu0 add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -label {  8} \
	 -command {set analysis(filter_lcf) 8}
   $topwin(analysis_name).info.frame5.frame0.frame0.frame0.frame1.menubutton7.menu0 add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -label { 16} \
	 -command {set analysis(filter_lcf) 16}
   $topwin(analysis_name).info.frame5.frame0.frame0.frame0.frame1.menubutton7.menu0 add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -label { 32} \
	 -command {set analysis(filter_lcf) 32}
   $topwin(analysis_name).info.frame5.frame0.frame0.frame0.frame1.menubutton7.menu0 add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -label { 64} \
	 -command {set analysis(filter_lcf) 64}
   $topwin(analysis_name).info.frame5.frame0.frame0.frame0.frame1.menubutton7.menu0 add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -label {128} \
	 -command {set analysis(filter_lcf) 128}

  bind  $topwin(analysis_name).info.frame5.frame0.frame0.frame0.frame1.menubutton7.menu0 <B1-Motion> { MenuPopupMotion  $topwin(analysis_name).info.frame5.frame0.frame0.frame0.frame1.menubutton7.menu0 %W %X %Y}
  bind  $topwin(analysis_name).info.frame5.frame0.frame0.frame0.frame1.menubutton7.menu0 <ButtonRelease-1>  { MenuPopupRelease  $topwin(analysis_name).info.frame5.frame0.frame0.frame0.frame1.menubutton7.menu0 %W }

  # pack master  $topwin(analysis_name).info.frame5.frame0.frame0.frame0.frame1
  pack configure  $topwin(analysis_name).info.frame5.frame0.frame0.frame0.frame1.menubutton7 \
    -fill both \
    -side right
  pack configure  $topwin(analysis_name).info.frame5.frame0.frame0.frame0.frame1.entry \
    -fill both \
    -side left


  # pack widget $topwin(analysis_name).info.frame5.frame0.frame0.frame0
  pack append $topwin(analysis_name).info.frame5.frame0.frame0.frame0 \
    $topwin(analysis_name).info.frame5.frame0.frame0.frame0.label1 {left frame center fill} \
    $topwin(analysis_name).info.frame5.frame0.frame0.frame0.frame1 {left frame center expand fill} 

  # pack widget $topwin(analysis_name).info.frame5.frame0.frame0
  pack append $topwin(analysis_name).info.frame5.frame0.frame0 \
    $topwin(analysis_name).info.frame5.frame0.frame0.radiobutton18 {left frame center} \
    $topwin(analysis_name).info.frame5.frame0.frame0.radiobutton19 {left frame e padx 4} \
    $topwin(analysis_name).info.frame5.frame0.frame0.frame0 {left frame center}

  # build widget $topwin(analysis_name).info.frame5.frame0.frame12
  frame $topwin(analysis_name).info.frame5.frame0.frame12 \
    -background "$GuiColor(frame_bg)" \
    -relief {ridge}

  # build widget $topwin(analysis_name).info.frame5.frame0.frame12.checkbutton20
	checkbutton $topwin(analysis_name).info.frame5.frame0.frame12.checkbutton20 -padx {1} -pady {1} -highlightthickness {0}  \
    -background "$GuiColor(btn_bg)" \
    -activebackground "$GuiColor(btn_act_bg)" \
    -offvalue {n} \
    -onvalue {y} \
    -relief {raised} \
    -text {Keep DC} \
    -variable {analysis(filter_dc)}

  # build widget $topwin(analysis_name).info.frame5.frame0.frame12.frame1
  frame $topwin(analysis_name).info.frame5.frame0.frame12.frame1 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame5.frame0.frame12.frame1.label1
  label $topwin(analysis_name).info.frame5.frame0.frame12.frame1.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -relief {sunken} \
    -text {Filter type: }

  # build widget $topwin(analysis_name).info.frame5.frame0.frame12.frame1.value
  entry $topwin(analysis_name).info.frame5.frame0.frame12.frame1.value \
    -background "$GuiColor(rentry_bg)" \
    -foreground "$GuiColor(rentry_fg)" \
	 -state {disabled} \
    -relief {sunken} \
    -textvariable {analysis(filter_type)} \
    -width {11}

  # build widget $topwin(analysis_name).info.frame5.frame0.frame12.frame1.menubutton2
	menubutton $topwin(analysis_name).info.frame5.frame0.frame12.frame1.menubutton2 -padx {1} -pady {1} -highlightthickness {0}  \
    -background "$GuiColor(btn_bg)" \
    -activebackground "$GuiColor(btn_act_bg)" \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)" \
    -text "$GuiMisc(pop_up_text)" \
	 -width {2}
  # bindings
  bind $topwin(analysis_name).info.frame5.frame0.frame12.frame1.menubutton2 <Button-1> {MenuPopupPost $topwin(analysis_name).info.frame5.frame0.frame12.frame1.menubutton2.menu %X %Y}
  bind $topwin(analysis_name).info.frame5.frame0.frame12.frame1.menubutton2 <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame5.frame0.frame12.frame1.menubutton2.menu %W}

  # build widget $topwin(analysis_name).info.frame5.frame0.frame12.frame1.menubutton2.menu
  menu $topwin(analysis_name).info.frame5.frame0.frame12.frame1.menubutton2.menu \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"
  $topwin(analysis_name).info.frame5.frame0.frame12.frame1.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command "set analysis(filter_type) Ideal
				  pack forget $topwin(analysis_name).info.frame5.frame0.frame12.frame0
				  pack forget $topwin(analysis_name).info.frame5.frame1.frame13.frame0" \
    -label {    Ideal    }
  $topwin(analysis_name).info.frame5.frame0.frame12.frame1.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command "set analysis(filter_type) Butterworth
				pack $topwin(analysis_name).info.frame5.frame0.frame12.frame0 -side left
				pack $topwin(analysis_name).info.frame5.frame1.frame13.frame0 -side left" \
    -label { Butterworth }
  bindtags $topwin(analysis_name).info.frame5.frame0.frame12.frame1.menubutton2.menu {Menu all}
  # bindings
  bind $topwin(analysis_name).info.frame5.frame0.frame12.frame1.menubutton2.menu <B1-Motion> {MenuPopupMotion $topwin(analysis_name).info.frame5.frame0.frame12.frame1.menubutton2.menu %W %X %Y}
  bind $topwin(analysis_name).info.frame5.frame0.frame12.frame1.menubutton2.menu <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame5.frame0.frame12.frame1.menubutton2.menu %W}

  # pack widget $topwin(analysis_name).info.frame5.frame0.frame12.frame1
  pack append $topwin(analysis_name).info.frame5.frame0.frame12.frame1 \
    $topwin(analysis_name).info.frame5.frame0.frame12.frame1.label1 {left frame center fill} \
    $topwin(analysis_name).info.frame5.frame0.frame12.frame1.value {left frame center expand fill} \
    $topwin(analysis_name).info.frame5.frame0.frame12.frame1.menubutton2 {right frame center fill}

  # build widget $topwin(analysis_name).info.frame5.frame0.frame12.frame0
  frame $topwin(analysis_name).info.frame5.frame0.frame12.frame0 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {0}

  # build widget $topwin(analysis_name).info.frame5.frame0.frame12.frame0.label1
  label $topwin(analysis_name).info.frame5.frame0.frame12.frame0.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -relief {sunken} \
    -text {Filter order: }

  # build widget $topwin(analysis_name).info.frame5.frame0.frame12.frame0.value
  entry $topwin(analysis_name).info.frame5.frame0.frame12.frame0.value \
    -background "$GuiColor(rentry_bg)" \
    -foreground "$GuiColor(rentry_fg)" \
    -relief {sunken} \
    -textvariable {analysis(filter_order)} \
    -width {3}

  # build widget $topwin(analysis_name).info.frame5.frame0.frame12.frame0.menubutton2
	menubutton $topwin(analysis_name).info.frame5.frame0.frame12.frame0.menubutton2 -padx {1} -pady {1} -highlightthickness {0}  \
    -background "$GuiColor(btn_bg)" \
    -activebackground "$GuiColor(btn_act_bg)" \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)" \
    -text "$GuiMisc(pop_up_text)" \
	 -width {2}
  # bindings
  bind $topwin(analysis_name).info.frame5.frame0.frame12.frame0.menubutton2 <Button-1> {MenuPopupPost $topwin(analysis_name).info.frame5.frame0.frame12.frame0.menubutton2.menu %X %Y}
  bind $topwin(analysis_name).info.frame5.frame0.frame12.frame0.menubutton2 <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame5.frame0.frame12.frame0.menubutton2.menu %W}

  # build widget $topwin(analysis_name).info.frame5.frame0.frame12.frame0.menubutton2.menu
  menu $topwin(analysis_name).info.frame5.frame0.frame12.frame0.menubutton2.menu \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"
  $topwin(analysis_name).info.frame5.frame0.frame12.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(filter_order) 1} \
    -label { 1 }
  $topwin(analysis_name).info.frame5.frame0.frame12.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(filter_order) 2} \
    -label { 2 }
  $topwin(analysis_name).info.frame5.frame0.frame12.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(filter_order) 3} \
    -label { 3 }
  $topwin(analysis_name).info.frame5.frame0.frame12.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(filter_order) 4} \
    -label { 4 }
  $topwin(analysis_name).info.frame5.frame0.frame12.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(filter_order) 5} \
    -label { 5 }
  $topwin(analysis_name).info.frame5.frame0.frame12.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(filter_order) 6} \
    -label { 6 }
  $topwin(analysis_name).info.frame5.frame0.frame12.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(filter_order) 7} \
    -label { 7 }
  $topwin(analysis_name).info.frame5.frame0.frame12.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(filter_order) 8} \
    -label { 8 }
  bindtags $topwin(analysis_name).info.frame5.frame0.frame12.frame0.menubutton2.menu {Menu all}
  # bindings
  bind $topwin(analysis_name).info.frame5.frame0.frame12.frame0.menubutton2.menu <B1-Motion> {MenuPopupMotion $topwin(analysis_name).info.frame5.frame0.frame12.frame0.menubutton2.menu %W %X %Y}
  bind $topwin(analysis_name).info.frame5.frame0.frame12.frame0.menubutton2.menu <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame5.frame0.frame12.frame0.menubutton2.menu %W}

  # pack widget $topwin(analysis_name).info.frame5.frame0.frame12.frame0
  pack append $topwin(analysis_name).info.frame5.frame0.frame12.frame0 \
    $topwin(analysis_name).info.frame5.frame0.frame12.frame0.label1 {left frame center fill} \
    $topwin(analysis_name).info.frame5.frame0.frame12.frame0.value {left frame center expand fill} \
    $topwin(analysis_name).info.frame5.frame0.frame12.frame0.menubutton2 {right frame center fill}

  # pack widget $topwin(analysis_name).info.frame5.frame0.frame12
  pack append $topwin(analysis_name).info.frame5.frame0.frame12 \
    $topwin(analysis_name).info.frame5.frame0.frame12.checkbutton20 {left frame e padx 198} \
    $topwin(analysis_name).info.frame5.frame0.frame12.frame1 {left frame center padx 15} 

  # pack widget $topwin(analysis_name).info.frame5.frame0
  pack append $topwin(analysis_name).info.frame5.frame0 \
    $topwin(analysis_name).info.frame5.frame0.frame0 {top frame center expand fill} \
    $topwin(analysis_name).info.frame5.frame0.frame12 {top frame center expand fill}

  # build widget $topwin(analysis_name).info.frame5.frame1
  frame $topwin(analysis_name).info.frame5.frame1 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2} \
    -relief {ridge}

  # build widget $topwin(analysis_name).info.frame5.frame1.frame0
  frame $topwin(analysis_name).info.frame5.frame1.frame0 \
    -background "$GuiColor(frame_bg)" \
    -relief {ridge}

  # build widget $topwin(analysis_name).info.frame5.frame1.frame0.radiobutton18
	radiobutton $topwin(analysis_name).info.frame5.frame1.frame0.radiobutton18 -padx {1} -pady {1} -highlightthickness {0}  \
    -background "$GuiColor(btn_bg)" \
    -activebackground "$GuiColor(btn_act_bg)" \
    -relief {flat} \
    -text {Band pass} \
    -value {bpf_filter} \
    -variable {analysis(transform)} \
    -width {9}

  # build widget $topwin(analysis_name).info.frame5.frame1.frame0.radiobutton1
	radiobutton $topwin(analysis_name).info.frame5.frame1.frame0.radiobutton1 -padx {1} -pady {1} -highlightthickness {0}  \
    -background "$GuiColor(btn_bg)" \
    -activebackground "$GuiColor(btn_act_bg)" \
    -relief {flat} \
    -text {Band reject} \
    -value {brf_filter} \
    -variable {analysis(transform)} \
    -width {11}

  # build widget $topwin(analysis_name).info.frame5.frame1.frame0.frame0
  frame $topwin(analysis_name).info.frame5.frame1.frame0.frame0 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame5.frame1.frame0.frame0.label1
  label $topwin(analysis_name).info.frame5.frame1.frame0.frame0.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -text {Low cutoff Freq: }

  # build widget $topwin(analysis_name).info.frame5.frame1.frame0.frame0.frame1
  frame $topwin(analysis_name).info.frame5.frame1.frame0.frame0.frame1 \
    -borderwidth {0} \
    -highlightthickness {0} \
	 -background "$GuiColor(frame_bg)" \
    -height {30} \
    -width {30}

  # build widget $topwin(analysis_name).info.frame5.frame1.frame0.frame0.frame1.entry
  entry $topwin(analysis_name).info.frame5.frame1.frame0.frame0.frame1.entry \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
    -textvariable {analysis(filter_lcf)} -width {5}
  endEntry $topwin(analysis_name).info.frame5.frame1.frame0.frame0.frame1.entry 0 \
  	$topwin(analysis_name).info.frame5.frame1.frame0.frame3.frame1.entry\
  	$topwin(analysis_name).info.frame5.frame1.frame0.frame3.frame1.entry

  # build widget $topwin(analysis_name).info.frame5.frame1.frame0.frame0.frame1.menubutton7
  menubutton $topwin(analysis_name).info.frame5.frame1.frame0.frame0.frame1.menubutton7 \
	 -activebackground "$GuiColor(btn_act_bg)" -background "$GuiColor(btn_bg)"  \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)" \
    -padx {1} -pady {1} -highlightthickness {0} \
    -text "$GuiMisc(pop_up_text)" -width {2}

  # bindings
  bind $topwin(analysis_name).info.frame5.frame1.frame0.frame0.frame1.menubutton7 <Button-1> { MenuPopupPost $topwin(analysis_name).info.frame5.frame1.frame0.frame0.frame1.menubutton7.menu0 %X %Y}
  bind $topwin(analysis_name).info.frame5.frame1.frame0.frame0.frame1.menubutton7 <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame5.frame1.frame0.frame0.frame1.menubutton7.menu0 %W }

  # build widget $topwin(analysis_name).info.frame5.frame1.frame0.frame0.frame1.menubutton7.menu0
  menu $topwin(analysis_name).info.frame5.frame1.frame0.frame0.frame1.menubutton7.menu0  \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" 

  $topwin(analysis_name).info.frame5.frame1.frame0.frame0.frame1.menubutton7.menu0 add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"  \
    -label {  8} \
	 -command {set analysis(filter_lcf) 8}
  $topwin(analysis_name).info.frame5.frame1.frame0.frame0.frame1.menubutton7.menu0 add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"  \
    -label { 16} \
	 -command {set analysis(filter_lcf) 16}
  $topwin(analysis_name).info.frame5.frame1.frame0.frame0.frame1.menubutton7.menu0 add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"  \
    -label { 32} \
	 -command {set analysis(filter_lcf) 32}
  $topwin(analysis_name).info.frame5.frame1.frame0.frame0.frame1.menubutton7.menu0 add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"  \
    -label { 64} \
	 -command {set analysis(filter_lcf) 64}
  $topwin(analysis_name).info.frame5.frame1.frame0.frame0.frame1.menubutton7.menu0 add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"  \
    -label {128} \
	 -command {set analysis(filter_lcf) 128}

  bind $topwin(analysis_name).info.frame5.frame1.frame0.frame0.frame1.menubutton7.menu0 <B1-Motion> { MenuPopupMotion $topwin(analysis_name).info.frame5.frame1.frame0.frame0.frame1.menubutton7.menu0 %W %X %Y}
  bind $topwin(analysis_name).info.frame5.frame1.frame0.frame0.frame1.menubutton7.menu0 <ButtonRelease-1>  { MenuPopupRelease $topwin(analysis_name).info.frame5.frame1.frame0.frame0.frame1.menubutton7.menu0 %W }

  # pack master $topwin(analysis_name).info.frame5.frame1.frame0.frame0.frame1
  pack configure $topwin(analysis_name).info.frame5.frame1.frame0.frame0.frame1.menubutton7 \
    -fill both \
    -side right
  pack configure $topwin(analysis_name).info.frame5.frame1.frame0.frame0.frame1.entry \
    -fill both \
    -side left



  # pack widget $topwin(analysis_name).info.frame5.frame1.frame0.frame0
  pack append $topwin(analysis_name).info.frame5.frame1.frame0.frame0 \
    $topwin(analysis_name).info.frame5.frame1.frame0.frame0.label1 {left frame center fill} \
    $topwin(analysis_name).info.frame5.frame1.frame0.frame0.frame1 {left frame center expand fill} 

  # build widget $topwin(analysis_name).info.frame5.frame1.frame0.frame3
  frame $topwin(analysis_name).info.frame5.frame1.frame0.frame3 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame5.frame1.frame0.frame3.label1
  label $topwin(analysis_name).info.frame5.frame1.frame0.frame3.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -text {High cutoff Freq: }

  # build widget $topwin(analysis_name).info.frame5.frame1.frame0.frame3.frame1
  frame $topwin(analysis_name).info.frame5.frame1.frame0.frame3.frame1 \
    -borderwidth {0} \
    -highlightthickness {0} \
	 -background "$GuiColor(frame_bg)" \
    -height {30} \
    -width {30}

  # build widget $topwin(analysis_name).info.frame5.frame1.frame0.frame3.frame1.entry
  entry $topwin(analysis_name).info.frame5.frame1.frame0.frame3.frame1.entry \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
    -textvariable {analysis(filter_hcf)} -width {5}
  endEntry $topwin(analysis_name).info.frame5.frame1.frame0.frame3.frame1.entry 0\
  	$topwin(analysis_name).info.frame5.frame1.frame0.frame0.frame1.entry \
  	$topwin(analysis_name).info.frame5.frame1.frame0.frame0.frame1.entry 

  # build widget $topwin(analysis_name).info.frame5.frame1.frame0.frame3.frame1.menubutton7
  menubutton $topwin(analysis_name).info.frame5.frame1.frame0.frame3.frame1.menubutton7 \
	 -activebackground "$GuiColor(btn_act_bg)" -background "$GuiColor(btn_bg)"  \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)" \
    -padx {1} -pady {1} -highlightthickness {0} \
    -text "$GuiMisc(pop_up_text)" -width {2}

  # bindings
  bind $topwin(analysis_name).info.frame5.frame1.frame0.frame3.frame1.menubutton7 <Button-1> { MenuPopupPost $topwin(analysis_name).info.frame5.frame1.frame0.frame3.frame1.menubutton7.menu0 %X %Y}
  bind $topwin(analysis_name).info.frame5.frame1.frame0.frame3.frame1.menubutton7 <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame5.frame1.frame0.frame3.frame1.menubutton7.menu0 %W }

  # build widget $topwin(analysis_name).info.frame5.frame1.frame0.frame3.frame1.menubutton7.menu0
  menu $topwin(analysis_name).info.frame5.frame1.frame0.frame3.frame1.menubutton7.menu0  \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" 

  $topwin(analysis_name).info.frame5.frame1.frame0.frame3.frame1.menubutton7.menu0 add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"  \
    -label { 64} \
	 -command {set analysis(filter_hcf) 64}
  $topwin(analysis_name).info.frame5.frame1.frame0.frame3.frame1.menubutton7.menu0 add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"  \
    -label {128} \
	 -command {set analysis(filter_hcf) 128}
  $topwin(analysis_name).info.frame5.frame1.frame0.frame3.frame1.menubutton7.menu0 add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"  \
    -label {256} \
	 -command {set analysis(filter_hcf) 256}
  $topwin(analysis_name).info.frame5.frame1.frame0.frame3.frame1.menubutton7.menu0 add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"  \
    -label {512} \
	 -command {set analysis(filter_hcf) 512}

  bind $topwin(analysis_name).info.frame5.frame1.frame0.frame3.frame1.menubutton7.menu0 <B1-Motion> { MenuPopupMotion $topwin(analysis_name).info.frame5.frame1.frame0.frame3.frame1.menubutton7.menu0 %W %X %Y}
  bind $topwin(analysis_name).info.frame5.frame1.frame0.frame3.frame1.menubutton7.menu0 <ButtonRelease-1>  { MenuPopupRelease $topwin(analysis_name).info.frame5.frame1.frame0.frame3.frame1.menubutton7.menu0 %W }

  # pack master $topwin(analysis_name).info.frame5.frame1.frame0.frame3.frame1
  pack configure $topwin(analysis_name).info.frame5.frame1.frame0.frame3.frame1.menubutton7 \
    -fill both \
    -side right
  pack configure $topwin(analysis_name).info.frame5.frame1.frame0.frame3.frame1.entry \
    -fill both \
    -side left

  # pack widget $topwin(analysis_name).info.frame5.frame1.frame0.frame3
  pack append $topwin(analysis_name).info.frame5.frame1.frame0.frame3 \
    $topwin(analysis_name).info.frame5.frame1.frame0.frame3.label1 {left frame center fill} \
    $topwin(analysis_name).info.frame5.frame1.frame0.frame3.frame1 {left frame center expand fill} 

  # pack widget $topwin(analysis_name).info.frame5.frame1.frame0
  pack append $topwin(analysis_name).info.frame5.frame1.frame0 \
    $topwin(analysis_name).info.frame5.frame1.frame0.radiobutton18 {left frame center} \
    $topwin(analysis_name).info.frame5.frame1.frame0.radiobutton1 {left frame center} \
    $topwin(analysis_name).info.frame5.frame1.frame0.frame0 {left frame center} \
    $topwin(analysis_name).info.frame5.frame1.frame0.frame3 {left frame center padx 1}

  # build widget $topwin(analysis_name).info.frame5.frame1.frame13
  frame $topwin(analysis_name).info.frame5.frame1.frame13 \
    -background "$GuiColor(frame_bg)" \
    -relief {ridge}

  # build widget $topwin(analysis_name).info.frame5.frame1.frame13.checkbutton20
	checkbutton $topwin(analysis_name).info.frame5.frame1.frame13.checkbutton20 -padx {1} -pady {1} -highlightthickness {0}  \
    -background "$GuiColor(btn_bg)" \
    -activebackground "$GuiColor(btn_act_bg)" \
    -offvalue {n} \
    -onvalue {y} \
    -relief {raised} \
    -text {Keep DC} \
    -variable {analysis(filter_dc)}

  # build widget $topwin(analysis_name).info.frame5.frame1.frame13.frame1
  frame $topwin(analysis_name).info.frame5.frame1.frame13.frame1 \
    -background "$GuiColor(frame_bg)" \
    -relief {flat}

  # build widget $topwin(analysis_name).info.frame5.frame1.frame13.frame1.label1
  label $topwin(analysis_name).info.frame5.frame1.frame13.frame1.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -relief {sunken} \
    -text {Filter type: }

  # build widget $topwin(analysis_name).info.frame5.frame1.frame13.frame1.value
  entry $topwin(analysis_name).info.frame5.frame1.frame13.frame1.value \
    -background "$GuiColor(rentry_bg)" \
    -foreground "$GuiColor(rentry_fg)" \
	 -state {disabled} \
    -relief {sunken} \
    -textvariable {analysis(filter_type)} \
    -width {11}

  # build widget $topwin(analysis_name).info.frame5.frame1.frame13.frame1.menubutton2
	menubutton $topwin(analysis_name).info.frame5.frame1.frame13.frame1.menubutton2 -padx {1} -pady {1} -highlightthickness {0}  \
    -background "$GuiColor(btn_bg)" \
    -activebackground "$GuiColor(btn_act_bg)" \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)" \
    -text "$GuiMisc(pop_up_text)" \
	 -width {2}
  # bindings
  bind $topwin(analysis_name).info.frame5.frame1.frame13.frame1.menubutton2 <Button-1> {MenuPopupPost $topwin(analysis_name).info.frame5.frame1.frame13.frame1.menubutton2.menu %X %Y}
  bind $topwin(analysis_name).info.frame5.frame1.frame13.frame1.menubutton2 <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame5.frame1.frame13.frame1.menubutton2.menu %W}

  # build widget $topwin(analysis_name).info.frame5.frame1.frame13.frame1.menubutton2.menu
  menu $topwin(analysis_name).info.frame5.frame1.frame13.frame1.menubutton2.menu \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"
  $topwin(analysis_name).info.frame5.frame1.frame13.frame1.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command "set analysis(filter_type) Ideal
				  pack forget $topwin(analysis_name).info.frame5.frame0.frame12.frame0
				  pack forget $topwin(analysis_name).info.frame5.frame1.frame13.frame0" \
    -label {    Ideal    }
  $topwin(analysis_name).info.frame5.frame1.frame13.frame1.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command "set analysis(filter_type) Butterworth
				pack $topwin(analysis_name).info.frame5.frame0.frame12.frame0 -side left
				pack $topwin(analysis_name).info.frame5.frame1.frame13.frame0 -side left" \
    -label { Butterworth }
  bindtags $topwin(analysis_name).info.frame5.frame1.frame13.frame1.menubutton2.menu  {Menu all}
  # bindings
  bind $topwin(analysis_name).info.frame5.frame1.frame13.frame1.menubutton2.menu <B1-Motion> {MenuPopupMotion $topwin(analysis_name).info.frame5.frame1.frame13.frame1.menubutton2.menu %W %X %Y}
  bind $topwin(analysis_name).info.frame5.frame1.frame13.frame1.menubutton2.menu <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame5.frame1.frame13.frame1.menubutton2.menu %W}

  # pack widget $topwin(analysis_name).info.frame5.frame1.frame13.frame1
  pack append $topwin(analysis_name).info.frame5.frame1.frame13.frame1 \
    $topwin(analysis_name).info.frame5.frame1.frame13.frame1.label1 {left frame center fill} \
    $topwin(analysis_name).info.frame5.frame1.frame13.frame1.value {left frame center expand fill} \
    $topwin(analysis_name).info.frame5.frame1.frame13.frame1.menubutton2 {right frame center fill}

  # build widget $topwin(analysis_name).info.frame5.frame1.frame13.frame0
  frame $topwin(analysis_name).info.frame5.frame1.frame13.frame0 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {0}

  # build widget $topwin(analysis_name).info.frame5.frame1.frame13.frame0.label1
  label $topwin(analysis_name).info.frame5.frame1.frame13.frame0.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -relief {sunken} \
    -text {Filter order: }

  # build widget $topwin(analysis_name).info.frame5.frame1.frame13.frame0.value
  entry $topwin(analysis_name).info.frame5.frame1.frame13.frame0.value \
    -background "$GuiColor(rentry_bg)" \
    -foreground "$GuiColor(rentry_fg)" \
    -relief {sunken} \
    -textvariable {analysis(filter_order)} \
    -width {3}

  # build widget $topwin(analysis_name).info.frame5.frame1.frame13.frame0.menubutton2
	menubutton $topwin(analysis_name).info.frame5.frame1.frame13.frame0.menubutton2 -padx {1} -pady {1} -highlightthickness {0}  \
    -background "$GuiColor(btn_bg)" \
    -activebackground "$GuiColor(btn_act_bg)" \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)" \
    -text "$GuiMisc(pop_up_text)" \
	 -width {2}
  # bindings
  bind $topwin(analysis_name).info.frame5.frame1.frame13.frame0.menubutton2 <Button-1> {MenuPopupPost $topwin(analysis_name).info.frame5.frame1.frame13.frame0.menubutton2.menu %X %Y}
  bind $topwin(analysis_name).info.frame5.frame1.frame13.frame0.menubutton2 <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame5.frame1.frame13.frame0.menubutton2.menu %W}

  # build widget $topwin(analysis_name).info.frame5.frame1.frame13.frame0.menubutton2.menu
  menu $topwin(analysis_name).info.frame5.frame1.frame13.frame0.menubutton2.menu \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"
  $topwin(analysis_name).info.frame5.frame1.frame13.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(filter_order) 1} \
    -label { 1 }
  $topwin(analysis_name).info.frame5.frame1.frame13.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(filter_order) 2} \
    -label { 2 }
  $topwin(analysis_name).info.frame5.frame1.frame13.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(filter_order) 3} \
    -label { 3 }
  $topwin(analysis_name).info.frame5.frame1.frame13.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(filter_order) 4} \
    -label { 4 }
  $topwin(analysis_name).info.frame5.frame1.frame13.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(filter_order) 5} \
    -label { 5 }
  $topwin(analysis_name).info.frame5.frame1.frame13.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(filter_order) 6} \
    -label { 6 }
  $topwin(analysis_name).info.frame5.frame1.frame13.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(filter_order) 7} \
    -label { 7 }
  $topwin(analysis_name).info.frame5.frame1.frame13.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(filter_order) 8} \
    -label { 8 }
  bindtags $topwin(analysis_name).info.frame5.frame1.frame13.frame0.menubutton2.menu  {Menu all}
  # bindings
  bind $topwin(analysis_name).info.frame5.frame1.frame13.frame0.menubutton2.menu <B1-Motion> {MenuPopupMotion $topwin(analysis_name).info.frame5.frame1.frame13.frame0.menubutton2.menu %W %X %Y}
  bind $topwin(analysis_name).info.frame5.frame1.frame13.frame0.menubutton2.menu <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame5.frame1.frame13.frame0.menubutton2.menu %W}

  # pack widget $topwin(analysis_name).info.frame5.frame1.frame13.frame0
  pack append $topwin(analysis_name).info.frame5.frame1.frame13.frame0 \
    $topwin(analysis_name).info.frame5.frame1.frame13.frame0.label1 {left frame center fill} \
    $topwin(analysis_name).info.frame5.frame1.frame13.frame0.value {left frame center expand fill} \
    $topwin(analysis_name).info.frame5.frame1.frame13.frame0.menubutton2 {right frame center fill}

  # pack widget $topwin(analysis_name).info.frame5.frame1.frame13
  pack append $topwin(analysis_name).info.frame5.frame1.frame13 \
    $topwin(analysis_name).info.frame5.frame1.frame13.checkbutton20 {left frame e padx 198} \
    $topwin(analysis_name).info.frame5.frame1.frame13.frame1 {left frame center padx 15} 

  # pack widget $topwin(analysis_name).info.frame5.frame1
  pack append $topwin(analysis_name).info.frame5.frame1 \
    $topwin(analysis_name).info.frame5.frame1.frame0 {top frame center expand fill} \
    $topwin(analysis_name).info.frame5.frame1.frame13 {top frame center expand fill}

  # build widget $topwin(analysis_name).info.frame5.frame2
  frame $topwin(analysis_name).info.frame5.frame2 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2} \
    -relief {ridge}

  # build widget $topwin(analysis_name).info.frame5.frame2.frame0
  frame $topwin(analysis_name).info.frame5.frame2.frame0 \
    -background "$GuiColor(frame_bg)" \
    -relief {ridge}

  # build widget $topwin(analysis_name).info.frame5.frame2.frame0.radiobutton18
	radiobutton $topwin(analysis_name).info.frame5.frame2.frame0.radiobutton18 -padx {1} -pady {1} -highlightthickness {0}  \
    -background "$GuiColor(btn_bg)" \
    -activebackground "$GuiColor(btn_act_bg)" \
    -relief {flat} \
    -text {High frequency emphasize} \
    -value {hfe_filter} \
    -variable {analysis(transform)} \
    -width {22}

  # build widget $topwin(analysis_name).info.frame5.frame2.frame0.frame0
  frame $topwin(analysis_name).info.frame5.frame2.frame0.frame0 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame5.frame2.frame0.frame0.label1
  label $topwin(analysis_name).info.frame5.frame2.frame0.frame0.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -text {Cutoff Frequency: }

  # build widget $topwin(analysis_name).info.frame5.frame2.frame0.frame0.frame1
  frame $topwin(analysis_name).info.frame5.frame2.frame0.frame0.frame1 \
    -borderwidth {0} \
    -highlightthickness {0} \
	 -background "$GuiColor(frame_bg)" \
    -height {30} \
    -width {30}

  # build widget $topwin(analysis_name).info.frame5.frame2.frame0.frame0.frame1.entry
  entry $topwin(analysis_name).info.frame5.frame2.frame0.frame0.frame1.entry \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
    -textvariable {analysis(filter_lcf)}  -width {5}
  endEntry $topwin(analysis_name).info.frame5.frame2.frame0.frame0.frame1.entry 0\
  	$topwin(analysis_name).info.frame5.frame2.frame14.frame0.value \
  	$topwin(analysis_name).info.frame5.frame2.frame14.frame0.value

  # build widget $topwin(analysis_name).info.frame5.frame2.frame0.frame0.frame1.menubutton7
  menubutton $topwin(analysis_name).info.frame5.frame2.frame0.frame0.frame1.menubutton7 \
	 -activebackground "$GuiColor(btn_act_bg)" -background "$GuiColor(btn_bg)"  \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)" \
    -padx {1} -pady {1} -highlightthickness {0} \
    -text "$GuiMisc(pop_up_text)" -width {2}

  # bindings
  bind $topwin(analysis_name).info.frame5.frame2.frame0.frame0.frame1.menubutton7 <Button-1> { MenuPopupPost $topwin(analysis_name).info.frame5.frame2.frame0.frame0.frame1.menubutton7.menu0 %X %Y}
  bind $topwin(analysis_name).info.frame5.frame2.frame0.frame0.frame1.menubutton7 <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame5.frame2.frame0.frame0.frame1.menubutton7.menu0 %W }

  # build widget $topwin(analysis_name).info.frame5.frame2.frame0.frame0.frame1.menubutton7.menu0
  menu $topwin(analysis_name).info.frame5.frame2.frame0.frame0.frame1.menubutton7.menu0  \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" 

  $topwin(analysis_name).info.frame5.frame2.frame0.frame0.frame1.menubutton7.menu0 add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"  \
    -label { 32} \
	 -command {set analysis(filter_lcf) 32}
  $topwin(analysis_name).info.frame5.frame2.frame0.frame0.frame1.menubutton7.menu0 add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"  \
    -label { 64} \
	 -command {set analysis(filter_lcf) 64}
  $topwin(analysis_name).info.frame5.frame2.frame0.frame0.frame1.menubutton7.menu0 add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"  \
    -label {128} \
	 -command {set analysis(filter_lcf) 128}
  $topwin(analysis_name).info.frame5.frame2.frame0.frame0.frame1.menubutton7.menu0 add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"  \
    -label {256} \
	 -command {set analysis(filter_lcf) 256}

  bind $topwin(analysis_name).info.frame5.frame2.frame0.frame0.frame1.menubutton7.menu0 <B1-Motion> { MenuPopupMotion $topwin(analysis_name).info.frame5.frame2.frame0.frame0.frame1.menubutton7.menu0 %W %X %Y}
  bind $topwin(analysis_name).info.frame5.frame2.frame0.frame0.frame1.menubutton7.menu0 <ButtonRelease-1>  { MenuPopupRelease $topwin(analysis_name).info.frame5.frame2.frame0.frame0.frame1.menubutton7.menu0 %W }

  # pack master $topwin(analysis_name).info.frame5.frame2.frame0.frame0.frame1
  pack configure $topwin(analysis_name).info.frame5.frame2.frame0.frame0.frame1.menubutton7 \
    -fill both \
    -side right
  pack configure $topwin(analysis_name).info.frame5.frame2.frame0.frame0.frame1.entry \
    -fill both \
    -side left

  # pack widget $topwin(analysis_name).info.frame5.frame2.frame0.frame0
  pack append $topwin(analysis_name).info.frame5.frame2.frame0.frame0 \
    $topwin(analysis_name).info.frame5.frame2.frame0.frame0.label1 {left frame center fill} \
    $topwin(analysis_name).info.frame5.frame2.frame0.frame0.frame1 {left frame center expand fill} 

  # pack widget $topwin(analysis_name).info.frame5.frame2.frame0
  pack append $topwin(analysis_name).info.frame5.frame2.frame0 \
    $topwin(analysis_name).info.frame5.frame2.frame0.radiobutton18 {left frame center} \
    $topwin(analysis_name).info.frame5.frame2.frame0.frame0 {left frame center padx 26}

  # build widget $topwin(analysis_name).info.frame5.frame2.frame14
  frame $topwin(analysis_name).info.frame5.frame2.frame14 \
    -background "$GuiColor(frame_bg)" \
    -relief {ridge}

  # build widget $topwin(analysis_name).info.frame5.frame2.frame14.frame0
  frame $topwin(analysis_name).info.frame5.frame2.frame14.frame0 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame5.frame2.frame14.frame0.label1
  label $topwin(analysis_name).info.frame5.frame2.frame14.frame0.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -relief {sunken} \
    -text {Emphasis offset: }

  # build widget $topwin(analysis_name).info.frame5.frame2.frame14.frame0.value
  entry $topwin(analysis_name).info.frame5.frame2.frame14.frame0.value \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
    -textvariable {analysis(filter_offset)} \
    -width {4}
  endEntry $topwin(analysis_name).info.frame5.frame2.frame14.frame0.value 0\
  	$topwin(analysis_name).info.frame5.frame2.frame0.frame0.frame1.entry\
  	$topwin(analysis_name).info.frame5.frame2.frame0.frame0.frame1.entry

  # build widget $topwin(analysis_name).info.frame5.frame2.frame14.frame0.menubutton2
	menubutton $topwin(analysis_name).info.frame5.frame2.frame14.frame0.menubutton2 -padx {1} -pady {1} -highlightthickness {0}  \
    -background "$GuiColor(btn_bg)" \
    -activebackground "$GuiColor(btn_act_bg)" \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)" \
    -text "$GuiMisc(pop_up_text)" \
	 -width {2}
  # bindings
  bind $topwin(analysis_name).info.frame5.frame2.frame14.frame0.menubutton2 <Button-1> {MenuPopupPost $topwin(analysis_name).info.frame5.frame2.frame14.frame0.menubutton2.menu %X %Y}
  bind $topwin(analysis_name).info.frame5.frame2.frame14.frame0.menubutton2 <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame5.frame2.frame14.frame0.menubutton2.menu %W}

  # build widget $topwin(analysis_name).info.frame5.frame2.frame14.frame0.menubutton2.menu
  menu $topwin(analysis_name).info.frame5.frame2.frame14.frame0.menubutton2.menu \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"
  $topwin(analysis_name).info.frame5.frame2.frame14.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(filter_offset) 1.2} \
    -label { 1.2 }
  $topwin(analysis_name).info.frame5.frame2.frame14.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(filter_offset) 1.3} \
    -label { 1.3 }
  $topwin(analysis_name).info.frame5.frame2.frame14.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(filter_offset) 1.5} \
    -label { 1.5 }
  $topwin(analysis_name).info.frame5.frame2.frame14.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(filter_offset) 1.7} \
    -label { 1.7 }
  $topwin(analysis_name).info.frame5.frame2.frame14.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(filter_offset) 1.9} \
    -label { 1.9 }
  $topwin(analysis_name).info.frame5.frame2.frame14.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(filter_offset) 2.0} \
    -label { 2.0 }
  $topwin(analysis_name).info.frame5.frame2.frame14.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(filter_offset) 2.5} \
    -label { 2.5 }
  bindtags $topwin(analysis_name).info.frame5.frame2.frame14.frame0.menubutton2.menu {Menu all}
  # bindings
  bind $topwin(analysis_name).info.frame5.frame2.frame14.frame0.menubutton2.menu <B1-Motion> {MenuPopupMotion $topwin(analysis_name).info.frame5.frame2.frame14.frame0.menubutton2.menu %W %X %Y}
  bind $topwin(analysis_name).info.frame5.frame2.frame14.frame0.menubutton2.menu <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame5.frame2.frame14.frame0.menubutton2.menu %W}

  # pack widget $topwin(analysis_name).info.frame5.frame2.frame14.frame0
  pack append $topwin(analysis_name).info.frame5.frame2.frame14.frame0 \
    $topwin(analysis_name).info.frame5.frame2.frame14.frame0.label1 {left frame center fill} \
    $topwin(analysis_name).info.frame5.frame2.frame14.frame0.value {left frame center expand fill} \
    $topwin(analysis_name).info.frame5.frame2.frame14.frame0.menubutton2 {right frame center fill}

  # build widget $topwin(analysis_name).info.frame5.frame2.frame14.frame1
  frame $topwin(analysis_name).info.frame5.frame2.frame14.frame1 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame5.frame2.frame14.frame1.label1
  label $topwin(analysis_name).info.frame5.frame2.frame14.frame1.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -relief {sunken} \
    -text {Filter order: }

  # build widget $topwin(analysis_name).info.frame5.frame2.frame14.frame1.value
  entry $topwin(analysis_name).info.frame5.frame2.frame14.frame1.value \
    -background "$GuiColor(rentry_bg)" \
    -foreground "$GuiColor(rentry_fg)" \
    -relief {sunken} \
    -textvariable {analysis(filter_order)} \
    -width {3}

  # build widget $topwin(analysis_name).info.frame5.frame2.frame14.frame1.menubutton2
	menubutton $topwin(analysis_name).info.frame5.frame2.frame14.frame1.menubutton2 -padx {1} -pady {1} -highlightthickness {0}  \
    -background "$GuiColor(btn_bg)" \
    -activebackground "$GuiColor(btn_act_bg)" \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)" \
    -text "$GuiMisc(pop_up_text)" \
	 -width {2}
  # bindings
  bind $topwin(analysis_name).info.frame5.frame2.frame14.frame1.menubutton2 <Button-1> {MenuPopupPost $topwin(analysis_name).info.frame5.frame2.frame14.frame1.menubutton2.menu %X %Y}
  bind $topwin(analysis_name).info.frame5.frame2.frame14.frame1.menubutton2 <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame5.frame2.frame14.frame1.menubutton2.menu %W}

  # build widget $topwin(analysis_name).info.frame5.frame2.frame14.frame1.menubutton2.menu
  menu $topwin(analysis_name).info.frame5.frame2.frame14.frame1.menubutton2.menu \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"
  $topwin(analysis_name).info.frame5.frame2.frame14.frame1.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(filter_order) 1} \
    -label { 1 }
  $topwin(analysis_name).info.frame5.frame2.frame14.frame1.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(filter_order) 2} \
    -label { 2 }
  $topwin(analysis_name).info.frame5.frame2.frame14.frame1.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(filter_order) 3} \
    -label { 3 }
  $topwin(analysis_name).info.frame5.frame2.frame14.frame1.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(filter_order) 4} \
    -label { 4 }
  $topwin(analysis_name).info.frame5.frame2.frame14.frame1.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(filter_order) 5} \
    -label { 5 }
  $topwin(analysis_name).info.frame5.frame2.frame14.frame1.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(filter_order) 6} \
    -label { 6 }
  $topwin(analysis_name).info.frame5.frame2.frame14.frame1.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(filter_order) 7} \
    -label { 7 }
  $topwin(analysis_name).info.frame5.frame2.frame14.frame1.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(filter_order) 8} \
    -label { 8 }
  bindtags $topwin(analysis_name).info.frame5.frame2.frame14.frame1.menubutton2.menu {Menu all}
  # bindings
  bind $topwin(analysis_name).info.frame5.frame2.frame14.frame1.menubutton2.menu <B1-Motion> {MenuPopupMotion $topwin(analysis_name).info.frame5.frame2.frame14.frame1.menubutton2.menu %W %X %Y}
  bind $topwin(analysis_name).info.frame5.frame2.frame14.frame1.menubutton2.menu <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame5.frame2.frame14.frame1.menubutton2.menu %W}

  # pack widget $topwin(analysis_name).info.frame5.frame2.frame14.frame1
  pack append $topwin(analysis_name).info.frame5.frame2.frame14.frame1 \
    $topwin(analysis_name).info.frame5.frame2.frame14.frame1.label1 {left frame center fill} \
    $topwin(analysis_name).info.frame5.frame2.frame14.frame1.value {left frame center expand fill} \
    $topwin(analysis_name).info.frame5.frame2.frame14.frame1.menubutton2 {right frame center fill}

  # build widget $topwin(analysis_name).info.frame5.frame2.frame14.checkbutton20
	checkbutton $topwin(analysis_name).info.frame5.frame2.frame14.checkbutton20 -padx {1} -pady {1} -highlightthickness {0}  \
    -background "$GuiColor(btn_bg)" \
    -activebackground "$GuiColor(btn_act_bg)" \
    -offvalue {n} \
    -onvalue {y} \
    -relief {raised} \
    -text {Keep DC} \
    -variable {analysis(filter_dc)}

  # pack widget $topwin(analysis_name).info.frame5.frame2.frame14
  pack append $topwin(analysis_name).info.frame5.frame2.frame14 \
    $topwin(analysis_name).info.frame5.frame2.frame14.checkbutton20 {left frame e padx 197} \
    $topwin(analysis_name).info.frame5.frame2.frame14.frame0 {left frame center} \
    $topwin(analysis_name).info.frame5.frame2.frame14.frame1 {right frame w padx 20}

  # pack widget $topwin(analysis_name).info.frame5.frame2
  pack append $topwin(analysis_name).info.frame5.frame2 \
    $topwin(analysis_name).info.frame5.frame2.frame0 {top frame center expand fill} \
    $topwin(analysis_name).info.frame5.frame2.frame14 {top frame center expand fill}

  # build widget $topwin(analysis_name).info.frame5.frame5
  frame $topwin(analysis_name).info.frame5.frame5 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2} \
    -relief {ridge}

  # build widget $topwin(analysis_name).info.frame5.frame5.frame0
  frame $topwin(analysis_name).info.frame5.frame5.frame0 \
    -background "$GuiColor(frame_bg)" \
    -relief {ridge}

  # build widget $topwin(analysis_name).info.frame5.frame5.frame0.radiobutton18
	radiobutton $topwin(analysis_name).info.frame5.frame5.frame0.radiobutton18 -padx {1} -pady {1} -highlightthickness {0}  \
    -background "$GuiColor(btn_bg)" \
    -activebackground "$GuiColor(btn_act_bg)" \
    -relief {flat} \
    -text {Notch} \
    -value {notch} \
    -variable {analysis(transform)}

  # build widget $topwin(analysis_name).info.frame5.frame5.frame0.frame0
  frame $topwin(analysis_name).info.frame5.frame5.frame0.frame0 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame5.frame5.frame0.frame0.scrollbar11
  scrollbar $topwin(analysis_name).info.frame5.frame5.frame0.frame0.scrollbar11 \
    -activebackground "$GuiColor(menu_act_bg)" \
    -troughcolor "$GuiColor(menu_bg)" \
    -background "$GuiColor(menu_bg)" \
    -borderwidth {1} \
    -command {delta analysis(filter_row)} \
    -relief {sunken} \
    -repeatinterval {80} \
    -width {13}

  # build widget $topwin(analysis_name).info.frame5.frame5.frame0.frame0.label1
  label $topwin(analysis_name).info.frame5.frame5.frame0.frame0.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -text {Row: }

  # build widget $topwin(analysis_name).info.frame5.frame5.frame0.frame0.value
  entry $topwin(analysis_name).info.frame5.frame5.frame0.frame0.value \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
 -xscrollcommand "$topwin(analysis_name).info.frame5.frame5.frame0.frame0.scrollbar11 set" \
    -textvariable {analysis(filter_row)} \
    -width {5}
  endEntry $topwin(analysis_name).info.frame5.frame5.frame0.frame0.value 1\
  	$topwin(analysis_name).info.frame5.frame5.frame0.frame1.value \
  	$topwin(analysis_name).info.frame5.frame5.frame0.frame2.value 
  bind $topwin(analysis_name).info.frame5.frame5.frame0.frame0.value <KeyRelease> { selection clear}

  # build widget $topwin(analysis_name).info.frame5.frame5.frame0.frame0.menubutton2
	menubutton $topwin(analysis_name).info.frame5.frame5.frame0.frame0.menubutton2 -padx {1} -pady {1} -highlightthickness {0}  \
    -background "$GuiColor(btn_bg)" \
    -activebackground "$GuiColor(btn_act_bg)" \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)" \
    -text "$GuiMisc(pop_up_text)" \
	 -width {2}
  # bindings
  bind $topwin(analysis_name).info.frame5.frame5.frame0.frame0.menubutton2 <Button-1> {MenuPopupPost $topwin(analysis_name).info.frame5.frame5.frame0.frame0.menubutton2.menu %X %Y}
  bind $topwin(analysis_name).info.frame5.frame5.frame0.frame0.menubutton2 <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame5.frame5.frame0.frame0.menubutton2.menu %W}

  # build widget $topwin(analysis_name).info.frame5.frame5.frame0.frame0.menubutton2.menu
  menu $topwin(analysis_name).info.frame5.frame5.frame0.frame0.menubutton2.menu \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"
  $topwin(analysis_name).info.frame5.frame5.frame0.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(filter_row) 8; selection clear} \
    -label {   8} \
    -state {active}
  $topwin(analysis_name).info.frame5.frame5.frame0.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(filter_row) 16; selection clear} \
    -label {  16}
  $topwin(analysis_name).info.frame5.frame5.frame0.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(filter_row) 32; selection clear} \
    -label {  32}
  $topwin(analysis_name).info.frame5.frame5.frame0.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(filter_row) 64; selection clear} \
    -label {  64}
  $topwin(analysis_name).info.frame5.frame5.frame0.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(filter_row) 128; selection clear} \
    -label { 128}
  $topwin(analysis_name).info.frame5.frame5.frame0.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(filter_row) 256; selection clear} \
    -label { 256}
  $topwin(analysis_name).info.frame5.frame5.frame0.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(filter_row) 512; selection clear} \
    -label { 512 }
  bindtags $topwin(analysis_name).info.frame5.frame5.frame0.frame0.menubutton2.menu  {Menu all}
  # bindings
  bind $topwin(analysis_name).info.frame5.frame5.frame0.frame0.menubutton2.menu <B1-Motion> {MenuPopupMotion $topwin(analysis_name).info.frame5.frame5.frame0.frame0.menubutton2.menu %W %X %Y}
  bind $topwin(analysis_name).info.frame5.frame5.frame0.frame0.menubutton2.menu <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame5.frame5.frame0.frame0.menubutton2.menu %W}

  # pack widget $topwin(analysis_name).info.frame5.frame5.frame0.frame0
  pack append $topwin(analysis_name).info.frame5.frame5.frame0.frame0 \
    $topwin(analysis_name).info.frame5.frame5.frame0.frame0.label1 {left frame center fill} \
    $topwin(analysis_name).info.frame5.frame5.frame0.frame0.value {left frame center expand fill} \
    $topwin(analysis_name).info.frame5.frame5.frame0.frame0.menubutton2 {right frame center fill} \
    $topwin(analysis_name).info.frame5.frame5.frame0.frame0.scrollbar11 {left frame center fill}

  # build widget $topwin(analysis_name).info.frame5.frame5.frame0.frame1
  frame $topwin(analysis_name).info.frame5.frame5.frame0.frame1 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame5.frame5.frame0.frame1.scrollbar11
  scrollbar $topwin(analysis_name).info.frame5.frame5.frame0.frame1.scrollbar11 \
    -activebackground "$GuiColor(menu_act_bg)" \
    -troughcolor "$GuiColor(menu_bg)" \
    -background "$GuiColor(menu_bg)" \
    -borderwidth {1} \
    -command {delta analysis(filter_col)}\
    -relief {sunken} \
    -repeatinterval {80} \
    -width {13}

  # build widget $topwin(analysis_name).info.frame5.frame5.frame0.frame1.label1
  label $topwin(analysis_name).info.frame5.frame5.frame0.frame1.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -text {Column: }

  # build widget $topwin(analysis_name).info.frame5.frame5.frame0.frame1.value
  entry $topwin(analysis_name).info.frame5.frame5.frame0.frame1.value \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
 -xscrollcommand "$topwin(analysis_name).info.frame5.frame5.frame0.frame1.scrollbar11 set" \
    -textvariable {analysis(filter_col)} \
    -width {5}
  endEntry $topwin(analysis_name).info.frame5.frame5.frame0.frame1.value 1 \
  	$topwin(analysis_name).info.frame5.frame5.frame0.frame2.value \
  	$topwin(analysis_name).info.frame5.frame5.frame0.frame0.value 
  bind $topwin(analysis_name).info.frame5.frame5.frame0.frame1.value <KeyRelease> { selection clear}

  # build widget $topwin(analysis_name).info.frame5.frame5.frame0.frame1.menubutton2
	menubutton $topwin(analysis_name).info.frame5.frame5.frame0.frame1.menubutton2 -padx {1} -pady {1} -highlightthickness {0}  \
    -background "$GuiColor(btn_bg)" \
    -activebackground "$GuiColor(btn_act_bg)" \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)" \
    -text "$GuiMisc(pop_up_text)" \
	 -width {2}
  # bindings
  bind $topwin(analysis_name).info.frame5.frame5.frame0.frame1.menubutton2 <Button-1> {MenuPopupPost $topwin(analysis_name).info.frame5.frame5.frame0.frame1.menubutton2.menu %X %Y}
  bind $topwin(analysis_name).info.frame5.frame5.frame0.frame1.menubutton2 <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame5.frame5.frame0.frame1.menubutton2.menu %W}

  # build widget $topwin(analysis_name).info.frame5.frame5.frame0.frame1.menubutton2.menu
  menu $topwin(analysis_name).info.frame5.frame5.frame0.frame1.menubutton2.menu \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"
  $topwin(analysis_name).info.frame5.frame5.frame0.frame1.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(filter_col) 8; selection clear} \
    -label {   8}
  $topwin(analysis_name).info.frame5.frame5.frame0.frame1.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(filter_col) 16; selection clear} \
    -label {  16}
  $topwin(analysis_name).info.frame5.frame5.frame0.frame1.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(filter_col) 32; selection clear} \
    -label {  32}
  $topwin(analysis_name).info.frame5.frame5.frame0.frame1.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(filter_col) 64; selection clear} \
    -label {  64}
  $topwin(analysis_name).info.frame5.frame5.frame0.frame1.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(filter_col) 128; selection clear} \
    -label { 128}
  $topwin(analysis_name).info.frame5.frame5.frame0.frame1.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(filter_col) 256; selection clear} \
    -label { 256}
  $topwin(analysis_name).info.frame5.frame5.frame0.frame1.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(filter_col) 512; selection clear} \
    -label { 512 }
  bindtags $topwin(analysis_name).info.frame5.frame5.frame0.frame1.menubutton2.menu  {Menu all}
  # bindings
  bind $topwin(analysis_name).info.frame5.frame5.frame0.frame1.menubutton2.menu <B1-Motion> {MenuPopupMotion $topwin(analysis_name).info.frame5.frame5.frame0.frame1.menubutton2.menu %W %X %Y}
  bind $topwin(analysis_name).info.frame5.frame5.frame0.frame1.menubutton2.menu <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame5.frame5.frame0.frame1.menubutton2.menu %W}

  # pack widget $topwin(analysis_name).info.frame5.frame5.frame0.frame1
  pack append $topwin(analysis_name).info.frame5.frame5.frame0.frame1 \
    $topwin(analysis_name).info.frame5.frame5.frame0.frame1.label1 {left frame center fill} \
    $topwin(analysis_name).info.frame5.frame5.frame0.frame1.value {left frame center expand fill} \
    $topwin(analysis_name).info.frame5.frame5.frame0.frame1.menubutton2 {right frame center fill} \
    $topwin(analysis_name).info.frame5.frame5.frame0.frame1.scrollbar11 {left frame center fill}

  # build widget $topwin(analysis_name).info.frame5.frame5.frame0.frame2
  frame $topwin(analysis_name).info.frame5.frame5.frame0.frame2 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(analysis_name).info.frame5.frame5.frame0.frame2.scrollbar11
  scrollbar $topwin(analysis_name).info.frame5.frame5.frame0.frame2.scrollbar11 \
    -activebackground "$GuiColor(menu_act_bg)" \
    -troughcolor "$GuiColor(menu_bg)" \
    -background "$GuiColor(menu_bg)" \
    -borderwidth {1} \
    -command {delta analysis(filter_size)} \
    -relief {sunken} \
    -repeatinterval {80} \
    -width {13}

  # build widget $topwin(analysis_name).info.frame5.frame5.frame0.frame2.label1
  label $topwin(analysis_name).info.frame5.frame5.frame0.frame2.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -text {Size: }

  # build widget $topwin(analysis_name).info.frame5.frame5.frame0.frame2.value
  entry $topwin(analysis_name).info.frame5.frame5.frame0.frame2.value \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
 -xscrollcommand "$topwin(analysis_name).info.frame5.frame5.frame0.frame2.scrollbar11 set" \
    -textvariable {analysis(filter_size)} \
    -width {5}
  endEntry $topwin(analysis_name).info.frame5.frame5.frame0.frame2.value 1\
  	$topwin(analysis_name).info.frame5.frame5.frame0.frame0.value \
  	$topwin(analysis_name).info.frame5.frame5.frame0.frame1.value 
  bind $topwin(analysis_name).info.frame5.frame5.frame0.frame2.value <KeyRelease> { selection clear}

  # build widget $topwin(analysis_name).info.frame5.frame5.frame0.frame2.menubutton2
	menubutton $topwin(analysis_name).info.frame5.frame5.frame0.frame2.menubutton2 -padx {1} -pady {1} -highlightthickness {0}  \
    -background "$GuiColor(btn_bg)" \
    -activebackground "$GuiColor(btn_act_bg)" \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)" \
    -text "$GuiMisc(pop_up_text)" \
	 -width {2}
  # bindings
  bind $topwin(analysis_name).info.frame5.frame5.frame0.frame2.menubutton2 <Button-1> {MenuPopupPost $topwin(analysis_name).info.frame5.frame5.frame0.frame2.menubutton2.menu %X %Y}
  bind $topwin(analysis_name).info.frame5.frame5.frame0.frame2.menubutton2 <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame5.frame5.frame0.frame2.menubutton2.menu %W}

  # build widget $topwin(analysis_name).info.frame5.frame5.frame0.frame2.menubutton2.menu
  menu $topwin(analysis_name).info.frame5.frame5.frame0.frame2.menubutton2.menu \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"
  $topwin(analysis_name).info.frame5.frame5.frame0.frame2.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(filter_size) 1; selection clear} \
    -label {   1}
  $topwin(analysis_name).info.frame5.frame5.frame0.frame2.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(filter_size) 2; selection clear} \
    -label {   2}
  $topwin(analysis_name).info.frame5.frame5.frame0.frame2.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(filter_size) 3; selection clear} \
    -label {   3}
  $topwin(analysis_name).info.frame5.frame5.frame0.frame2.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(filter_size) 4; selection clear} \
    -label {   4}
  $topwin(analysis_name).info.frame5.frame5.frame0.frame2.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(filter_size) 5; selection clear} \
    -label {   5}
  $topwin(analysis_name).info.frame5.frame5.frame0.frame2.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(filter_size) 10; selection clear} \
    -label {  10}
  $topwin(analysis_name).info.frame5.frame5.frame0.frame2.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(filter_size) 20; selection clear} \
    -label {  20}
  $topwin(analysis_name).info.frame5.frame5.frame0.frame2.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(filter_size) 50; selection clear} \
    -label {  50}
  $topwin(analysis_name).info.frame5.frame5.frame0.frame2.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set analysis(filter_size) 100; selection clear} \
    -label { 100 }
  bindtags $topwin(analysis_name).info.frame5.frame5.frame0.frame2.menubutton2.menu  {Menu all}
  # bindings
  bind $topwin(analysis_name).info.frame5.frame5.frame0.frame2.menubutton2.menu <B1-Motion> {MenuPopupMotion $topwin(analysis_name).info.frame5.frame5.frame0.frame2.menubutton2.menu %W %X %Y}
  bind $topwin(analysis_name).info.frame5.frame5.frame0.frame2.menubutton2.menu <ButtonRelease-1> {MenuPopupRelease $topwin(analysis_name).info.frame5.frame5.frame0.frame2.menubutton2.menu %W}

  # pack widget $topwin(analysis_name).info.frame5.frame5.frame0.frame2
  pack append $topwin(analysis_name).info.frame5.frame5.frame0.frame2 \
    $topwin(analysis_name).info.frame5.frame5.frame0.frame2.label1 {left frame center fill} \
    $topwin(analysis_name).info.frame5.frame5.frame0.frame2.value {left frame center expand fill} \
    $topwin(analysis_name).info.frame5.frame5.frame0.frame2.menubutton2 {right frame center fill} \
    $topwin(analysis_name).info.frame5.frame5.frame0.frame2.scrollbar11 {left frame center fill}

  # pack widget $topwin(analysis_name).info.frame5.frame5.frame0
  pack append $topwin(analysis_name).info.frame5.frame5.frame0 \
    $topwin(analysis_name).info.frame5.frame5.frame0.radiobutton18 {left frame center} \
    $topwin(analysis_name).info.frame5.frame5.frame0.frame0 {left frame e padx 88} \
    $topwin(analysis_name).info.frame5.frame5.frame0.frame1 {left frame center padx 20} \
    $topwin(analysis_name).info.frame5.frame5.frame0.frame2 {left frame center padx 20}

  # pack widget $topwin(analysis_name).info.frame5.frame5
  pack append $topwin(analysis_name).info.frame5.frame5 \
    $topwin(analysis_name).info.frame5.frame5.frame0 {top frame center expand fill}

  # pack widget $topwin(analysis_name).info.frame5
  pack append $topwin(analysis_name).info.frame5 \
    $topwin(analysis_name).info.frame5.frame0 {top frame center expand fill} \
    $topwin(analysis_name).info.frame5.frame1 {top frame center expand fill} \
    $topwin(analysis_name).info.frame5.frame2 {top frame center expand fill} \
    $topwin(analysis_name).info.frame5.frame5 {top frame center expand fill}

  # pack widget $topwin(analysis_name).info
  pack append $topwin(analysis_name).info \
    $topwin(analysis_name).info.frame3 {top frame center fillx} \
    $topwin(analysis_name).info.frame4 {top frame center fillx} \
    $topwin(analysis_name).info.frame5 {top frame center fillx}

  # pack widget $topwin(analysis_name).
  pack append $topwin(analysis_name) \
    $topwin(analysis_name).info {top frame center expand fill} \
    $topwin(analysis_name).current_image {top frame center fillx} \
    $topwin(analysis_name).perform {bottom frame center fill}

  if {"[info procs XFEdit]" != ""} {
    catch "XFMiscBindWidgetTree $topwin(analysis_name)."
    after 2 "catch {XFEditSetShowWindows}"
  }
}

proc update_size {} {
global anal
global rows
global cols
global current_image
global {analysis}

if {$current_image == ""} {
         AlertBox "Please load an image." "" "350x150" "ALERT!" "DISMISS"
         return
      }
if {$analysis(xform_blocksize) == -1 || $analysis(xform_blocksize) ==""} {
	if [ catch {expr $rows + $cols} ] {
		set temp 256
	} else {
		set temp [expr {$rows < $cols} ? $rows : $cols]
		if { $temp <= 0 } {
			set temp 256
		}
	}
	set analysis(xform_blocksize) [ expr int(pow(2, floor(log($temp)/log(2))))] 
	}
}

proc Hist_find { str } {
	set hist -1
	while {$str != ""} {
		foreach i [list DCTXFORM FFT HAAR WALSH HADAMARD WAVELET] {
			while { [string first "I_$i" $str] != "-1" } {
				set str [MidTrim $str "I_$i"]
				set str [MidTrim $str "$i"]
			}
		}
		if {[scan $str "%s" b] != "-1" } {
			switch -exact -- $b {
				DCTXFORM		{set hist dct}
				FFT			{set hist fft}
				HAAR			{set hist haar}
				WALSH			{set hist walsh}
				HADAMARD		{set hist hadamard}
				WAVELET		{set hist wavelet}
			}
			set str [MidTrim $str $b]
		} else {
			break
		}
	}
	return $hist
}

proc MidTrim {str1 str2} {
	if { [set i [string first $str2  $str1]] != "-1" } {
		set temp [string range $str1 0 [expr $i - 1]]
		append temp [string range $str1 [expr $i + [string length $str2]] end]
		return $temp
	} else {
		return -1
	}		
}

proc delta {var scroll num units} {
	set temp [string first "(" $var]
	if { $temp > "0" } {
		incr temp -1
		set temp1 [string range $var 0 $temp]
		eval "global $temp1"
	} else {
		eval "global $var"
	}
	set temp "incr $var -$num"
	eval $temp
}
