# **********************************************************************
# Computer Vision/Image Processing Tool Project - Dr. Scott Umbaugh SIUE
# **********************************************************************
#
#             File Name: CVIPInitGlobals.tcl
#           Description: The CVIP specific variable declaration and
#                        default value setup.
#         Related Files: CVIPInitGui.tcl
#   Initial Coding Date: Mon Jul 21 11:50:58 CDT 1997
#           Portability: TCL script file
#            References:
#             Author(s): Yansheng Wei
#                        Southern Illinois University @ Edwardsville
#
## Copyright (C) 1996 SIUE - by Scott Umbaugh and Yansheng Wei.
##
## Permission to use, copy, modify, and distribute this software and its
## documentation for any purpose and without fee is hereby granted, provided
## that the above copyright notice appear in all copies and that both that
## copyright notice and this permission notice appear in supporting
## documentation.  This software is provided "as is" without express or
## implied warranty.
##
# RCS (Revision Control System) Information
# ...Added automatically by RCS 
#
## $Log: CVIPInitGlobals.tcl,v $
## Revision 1.29  1997/08/07 20:11:20  yawei
## fixed morph problem
##
## Revision 1.28  1997/08/07 19:58:52  yawei
## modified the default viewer and size for Windows
##
## Revision 1.27  1997/08/06 00:19:01  yawei
## added window manager selection
##
## Revision 1.26  1997/07/25 17:50:28  wzheng
## Enhancement->Sharp Unsharp masking->upper limit;clip percentage to 0.01
## HIgh freq emphasis, chnage default Filter order to 3
##
## Revision 1.25  1997/07/25 14:56:50  yawei
## changed defaults
##
## Revision 1.24  1997/07/21 17:58:56  yawei
## finished cleanup
##
#
#****************************************************************************

##########################################################
# To users:
#    This file contains all the global variables and the 
#    initial default values. Feel free to change the default
#    values to your preference
#
# To CVIPtools developers:
#   - This file is manually formatted, please don't use
#     any tab keys when you modify this file
#   - Please keep the default values left aligned 
#   - Pay special consideration when adding a global
#     variable
#
##########################################################

proc CvipInitGlobals {} {

# CVIPtools version number
  global {CVIPversion}
  set {CVIPversion} {3.6}

# variable for the message area. This variable usually shouldn't be
# set manually. Please use the CvipMessage procedure to set the value.
  global {message}
  set {message} "CVIPtools Copyright (c) 1996, 1997 SIUE \n \
  \                 Version $CVIPversion"

###################################################
# GUI_DEBUG: control the printout of cviptcl/wish 
# commands in the GUI mode.
# values : 1 for on and 0 for off
#          default to on
###################################################
  global {GUI_DEBUG}
  set {GUI_DEBUG} {1}

###################################################
# CvipAlertControl: control the alert message
# in the GUI mode.
# values : 1 for on and 0 for off
#          default to off
###################################################
  global {CvipAlertControl}
  set {CvipAlertControl} {0}


###################################################
# File | Preference related variable setting goes here
###################################################

  global {env}
  global {tcl_platform}

  # variable to control the position of the Preference window
  global {initial_pref_position}
  set {initial_pref_position} {1}

  global {etc}
  set {etc(viewer)}                       {RamViewer}

  # the following is used when any empty "" viewer is specified
  set {etc(viewer_original)}              "$etc(viewer)"
  set {etc(display_format)}               {Default}
  cvip viewer $etc(viewer) $etc(display_format)

###################################################
# File | Preference Window manager selection
###################################################
  set {etc(wm_name)}                      {OpenWin}
  switch -exact -- $etc(wm_name) {
    OpenWin  { CvipSizeOpenWin }
	 CDE      { CvipSizeCDE }
	 FVWM     { CvipSizeFVWM }
	 IRIX     { CvipSizeIRIX }
	 Win32    { CvipSizeWin32 }
  }

  if { $tcl_platform(platform) == "windows" } {
    set etc(viewer) lviewpro; set etc(display_format) PPM
    cvip viewer $etc(viewer) $etc(display_format)
    CvipSizeWin32
  }

  #Initial popup position of windows
  global {popup_position}
  set {popup_position}                    {ur}


###################################################
# Analysis related variable setting goes here
###################################################

  global {analysis}

# Geometry Window
  set {analysis(geometry)}                {crop}
  set {analysis(crop_ul_col)}             {0}
  set {analysis(crop_ul_row)}             {0}
  set {analysis(util_crop_height)}        {100}
  set {analysis(util_crop_width)}         {100}
                                          
  set {analysis(paste_to_col)}            {128}
  set {analysis(paste_to_row)}            {128}
  set {analysis(paste_transparent)}       {0}
                                          
  set {analysis(resize_height)}           {256}
  set {analysis(resize_width)}            {256}
  set {analysis(resize_linear)}           {0}
                                          
  set {analysis(rotate_degree)}           {90}
                                          
  set {analysis(translate_fill)}          {128}
  set {analysis(translate_lr)}            {right}
  set {analysis(translate_lr_amount)}     {128}
  set {analysis(translate_ud)}            {down}
  set {analysis(translate_ud_amount)}     {128}
  set {analysis(translate_wrap)}          {y}
                                          
  set {analysis(zoom_quadrant)}           {6}
  set {analysis(zoom_ul_col)}             {0}
  set {analysis(zoom_ul_row)}             {0}
  set {analysis(zoom_width)}              {64}
  set {analysis(zoom_height)}             {64}
  set {analysis(zoom_factor)}             {2}
  set {analysis(zoom_method)}             {0 order}

# Edge/Line detection Window
  set {analysis(edge)}                    {kirsch}
  set {analysis(ed_smooth)}               {none}
  set {analysis(ed_thresh)}               {none}
  set {analysis(ed_kernel)}               {3}
  set {analysis(ed_dc)}                   {0}
  set {analysis(rob_lap)}                 {1}
  set {analysis(ed_projection)}           {line subspace}
  set {analysis(ed_threshangle_yn)}       {yes}
  set {analysis(ed_threshangle_value)}    {1.05}
  set {analysis(ed_hough_angles)}         {{90}}
  set {analysis(hough_line)}              {10}
  set {analysis(hough_distance)}          {5}
  set {analysis(link_distance)}           {5}

  set {analysis(bottom_thresh)}           {128}

# Segmentation Window
  set {analysis(seg_choice)}              {hist_thresh}
  set {analysis(fuzzyc_kernel)}           {3}
  set {analysis(gray_quant_num)}          {8}
  set {analysis(gray_qmethod)}            {S}
  set {analysis(median_cut_num)}          {3}
  set {analysis(pct_median_num)}          {3}
  set {analysis(sct_A_color)}             {3}
  set {analysis(sct_B_color)}             {3}
  set {analysis(multi_option)}            {1}
  set {analysis(multi_pct)}               {0}
  set {analysis(multi_thresh)}            {50}
  set {analysis(multi_percent)}           {0.70}
  set {analysis(sam_option)}              {1}
  set {analysis(sam_pct)}                 {0}
  set {analysis(sam_level)}               {3}
  set {analysis(sam_thresh)}              {50}
  set {analysis(sam_percent)}             {0.70}
  set {analysis(sam_percent_max)}         {1.0}
  set {analysis(morph_choice)}            {morph_close}
  set {analysis(kernel_type)}             {rectangle} 
  set {analysis(kernel_dim)}              {3} 
  set {analysis(kernel_height)}           {3} 
  set {analysis(kernel_width)}            {3}  
  set {analysis(morph_rot)}               {y}
  global {morph}
  set {morph(set.val)}                    {1}
  set {morph(boolean.val)}                {(!a)b}
  set {morph(iteration.val)}              {3}
  set {morph(subfields.val)}              {1}
  set {morph(boolean.max)}                {7}
  set {morph(iteration.max)}              {99}
  set {morph(subfields.max)}              {3}
  set {morph(boolean.min)}                {1}
  set {morph(iteration.min)}              {0}
  set {morph(subfields.min)}              {1}
  for {set temp_cont 1} {$temp_cont < 15} {incr temp_cont} {
  set morph(set.$temp_cont) {0}
  }
  set {morph(set.1)}                      {1}
  set {morph(set.7)}                      {1}

# Transform Window
  set {analysis(transform)}               {fft}
  set {analysis(xform_dir)}               {1}
  set {analysis(xform_blocksize)}         {256}
  set {analysis(xform_dir_wavelet)}       {1}
  set {analysis(xform_wavelet_basis)}     {haar}
  set {analysis(xform_wavelet_decomp)}    {1}

  set {analysis(filter)}                  {notch}
  set {analysis(filter_lcf)}              {32}
  set {analysis(filter_hcf)}              {128}
  set {analysis(filter_dc)}               {n}
  set {analysis(filter_type)}             {Ideal}
  set {analysis(filter_order)}            {1}
  set {analysis(filter_offset)}           {1.2}

  set {analysis(filter_col)}              {10}
  set {analysis(filter_row)}              {10}
  set {analysis(filter_size)}             {1}

  # not currently used widgets
  set {analysis(phase_norm)}              {0}
  set {analysis(phase_K)}                 {1}

# Features Window
  set {analysis(feature_fn)}              {}
  set {analysis(feature_class)}           {}
  set {analysis(obj_row)}                 {}
  set {analysis(obj_col)}                 {}
  set {analysis(binary_all_button}        {Select All}
  set {analysis(proj_height)}             {10}
  set {analysis(proj_width)}              {10}
  set {analysis(rst_all_button)}          {Select All}
  set {analysis(his_all_button)}          {Select All}
  set {analysis(text_all_button)}         {Select All}
  set {analysis(texture_dist)}            {2}
  set {analysis(spectral_ring)}           {3}
  set {analysis(spectral_sector)}         {3}

###################################################
# Restoration related variable setting goes here
###################################################

  global {restore}

#Noise window
  set {restore(noise)}                    {negative_exp_noise}
  set {restore(ns_var)}                   {100}
  set {restore(ns_mean)}                  {0}
  set {restore(ns_alpha)}                 {1.25}
  set {restore(ns_pepper)}                {.03}
  set {restore(ns_salt)}                  {.03}

  set {restore(noise_img)}                {current}
  set {restore(black_height)}             {256}
  set {restore(black_width)}              {256}

#Spatial filter window
  set {restore(spatial_filter)}           {median_filter}
  set {restore(spat_mask)}                {3}
  set {restore(spat_trim)}                {0}
  set {restore(spat_order)}               {-5}
  set {restore(mmse_kern)}                {9}
  set {restore(mmse_var)}                 {100}
  set {restore(adap_k1)}                  {0.8}
  set {restore(adap_k2)}                  {1.0}
  set {restore(adap_kern)}                {11}
  set {restore(adap_max)}                 {10}
  set {restore(adap_min)}                 {5}

#Frequency filter window
  global {freq_filt}
  set {freq_filt(type)}                   {inverse_xformfilter}
  set {freq_filt(msk_height)}             {9}
  set {freq_filt(msk_width)}              {9}
  set {freq_filt(degrade_shape)}          {rectangle}
  set {freq_filt(degrade_height)}         {7}
  set {freq_filt(degrade_width)}          {7}
  set {freq_filt(degrade_rad)}            {4}
  set {freq_filt(degrade_weight)}         {line}
  set {freq_filt(param4)}                 {1.0}
  set {freq_filt(cutoff)}                 {32}
  set {freq_filt(handle_zero)}            {1}
  set {freq_filt(noise_fn)}               {}
  set {freq_filt(deg_fn)}                 {image}
  set {freq_filt(original_fn)}            {}
  set {freq_filt(K)}                      {1.0}
  set {freq_filt(gamma)}                  {0}
  set {freq_filt(SNR)}                    {laplacian1}
  set {freq_filt(alpha)}                  {0}

  set {freq_filt(filter_val)}             {}
  set {freq_filt(filter_width)}           {}
  set {freq_filt(filter_height)}          {}
  set {freq_filt(blur_weight)}            {Constant}
  set {freq_filt(blur_weight_val)}        {1}
  set {freq_filt(degrade_ang1)}           {Radius:}
  set {freq_filt(degrade_ang2)}           {10}
  set {freq_filt(degrade_mask)}           {10}
  set {freq_filt(degrade_stdev)}          {10}
  set {freq_filt(h_image)}                {}

  set {freq_filt(notch_transform)}        {fft}
  set {freq_filt(notch_spect)}            ""
  set {freq_filt(notch_mask)}             ""
  set {freq_filt(col)}                    {64}
  set {freq_filt(row)}                    {32}
  set {freq_filt(size)}                   {10}


#Geometric transforms window
  set {restore(mesh_select)}               {enter_new}
  set {restore(mesh_file)}                 {}
  set {restore(mesh_r)}                    {3}
  set {restore(mesh_c)}                    {3}
  set {restore(mesh_point_r)}              {}
  set {restore(mesh_point_c)}              {}
  set {restore(mesh_direct)}               { irregular -> regular}
  set {restore(direction)}                 {1}
  set {restore(mesh_interp)}               { nearest neighbor}
  set {restore(mesh_interpolate)}          {1}
  set {restore(current_point)}             {1}
  set {restore(mesh_points)}               {9}
  #mesh points
  global {restoreCol}
  global {restoreRow}
  global {tempC}
  set {tempC}                              {  45 45 45 45 140 140 140 140 140 }
  global {tempR}
  set {tempR}                              {  45 45 45 45 45 45 45 45 45 }

###################################################
# Enhancement related variable setting goes here
###################################################
  global {enhan} 

#Histogram window
  set {enhan(hist)}                        {histeq}
  set {enhan(hist_band)}                   {0}
  set {enhan(hist_slide_dir)}              {1}
  set {enhan(hist_slide_value)}            {20}
  set {enhan(hist_shrink_lower)}           {0}
  set {enhan(hist_shrink_upper)}           {255}
  set {enhan(hist_l_c)}                    {0.025}
  set {enhan(hist_h_c)}                    {0.025}
  set {enhan(hist_leq_size)}               {4}
  set {enhan(adap_k1)}                     {0.8}
  set {enhan(adap_k2)}                     {1.0}
  set {enhan(adap_kern)}                   {11}
  set {enhan(adap_max)}                    {10}
  set {enhan(adap_min)}                    {5}
  set {enhan(hist_eqr)}                    {sin(0.025*x)}
  set {enhan(hist_eqg)}                    {sin(0.025*x)}
  set {enhan(hist_eqb)}                    {sin(0.025*x)}
  set {enhan(hist_linear_srt)}             {0}
  set {enhan(hist_linear_end)}             {255}
  set {enhan(hist_linear_val)}             {0}
  set {enhan(hist_linear_slp)}             {1.0}
  set {enhan(hist_linear_outer)}           {1}
  set {enhan(mult_clip)}                   {1}
  set {enhan(histo_multiply)}              {2.0}

#Pseudo-Color window
  set {enhan(g_c)}                         {pseudocol_freq}
  set {enhan(pseudo_hcf)}                  {100}
  set {enhan(pseudo_lcf)}                  {10}
  set {enhan(pseudo_bpc)}                  {G}
  set {enhan(pseudo_hpc)}                  {B}
  set {enhan(pseudo_lpc)}                  {R}
  set {enhan(g_c_b)}                       {blue}
  set {enhan(g_c_g)}                       {green}
  set {enhan(g_c_r)}                       {red}
  set {enhan(slice_out)}                   {0}
  set {enhan(slice_10)}                    {0}
  set {enhan(slice_11)}                    {63}
  set {enhan(slice_12)}                    {255}
  set {enhan(slice_13)}                    {0}
  set {enhan(slice_14)}                    {0}
  set {enhan(slice_20)}                    {64}
  set {enhan(slice_21)}                    {127}
  set {enhan(slice_22)}                    {0}
  set {enhan(slice_23)}                    {255}
  set {enhan(slice_24)}                    {0}
  set {enhan(slice_30)}                    {128}
  set {enhan(slice_31)}                    {191}
  set {enhan(slice_32)}                    {0}
  set {enhan(slice_33)}                    {0}
  set {enhan(slice_34)}                    {255}
  set {enhan(slice_40)}                    {192}
  set {enhan(slice_41)}                    {255}
  set {enhan(slice_42)}                    {255}
  set {enhan(slice_43)}                    {255}
  set {enhan(slice_44)}                    {255}

#Sharpening window
  set {enhan(sharp)}                       {high_boost}
  set {enhan(sh_mask_size)}                {3}
  set {enhan(sh_mask_center)}              {15}
  set {enhan(sh_add_result)}               {1}
  set {enhan(unsharp_ll)}                  {0}
  set {enhan(unsharp_ul)}                  {100}
  set {enhan(unsharp_l_c)}                 {0.01}
  set {enhan(unsharp_h_c)}                 {0.01}
  set {enhan(sh_cutoff)}                   {}

  set {enhan(block)}                       {256}
  set {enhan(order)}                       {3}
  set {enhan(sh_DCT_cutoff)}               {32}
  set {enhan(sh_FFT_cutoff)}               {32}

  set {enhan(transform)}                   {fft}
  set {enhan(filter_dc)}                   {n}
  set {enhan(filter_lcf)}                  {64}
  set {enhan(filter_offset)}               {0.5}
  set {enhan(filter_order)}                {3}
  set {enhan(filter_lfg)}                  {0.9}
  set {enhan(filter_hfg)}                  {1.3}

#Smoothing window
  set {enhan(smooth)}                      {mean_filter}
  set {enhan(sm_mean)}                     {3}
  set {enhan(sm_median)}                   {3}
  set {enhan(sm_midpt)}                    {3}
  set {enhan(sm_gauss)}                    {7}
  set {enhan(sm_contraYp)}                 {3}
  set {enhan(sm_contraYp_order)}           {1}
  set {enhan(sm_cutfft)}                   {32}
  set {enhan(sm_cutdct)}                   {32}
  set {enhan(sm_dct_blk)}                  {}
  set {enhan(sm_fft_blk)}                  {}

###################################################
# Compression related variable setting goes here
###################################################
 global {compress}

#Preprocessing window
 set {compress(preproc)}                   {bin_gray}
 set {compress(quant_ht)}                  {32}
 set {compress(quant_wd)}                  {32}
 set {compress(quant_meth)}                {median}
 set {compress(quant_method)}              {2}
 set {compress(gray_quant_num)}            {2}
 set {compress(gray_quant_method)}         {standard}

#Lossless window
 set {compress(lossless)}                  {zvl}
 set {compress(brc_bit_0)}                 {1}
 set {compress(brc_bit_1)}                 {1}
 set {compress(brc_bit_2)}                 {1}
 set {compress(brc_bit_3)}                 {1}
 set {compress(brc_bit_4)}                 {1}
 set {compress(brc_bit_5)}                 {1}
 set {compress(brc_bit_6)}                 {1}
 set {compress(brc_bit_7)}                 {1}
 set {compress(correlation)}               {0.90}
 set {compress(direction)}                 {Horizontal}
 set {compress(bits)}                      {4}

#Lossy window
 set {compress(lossy)}                     {btc}
 set {compress(block_size)}                {8}
 set {compress(jpeg_quality)}              {70}
 set {compress(jpeg_smooth)}               {1}
 set {compress(jpeg_grayscale)}            {0}
 set {compress(jpeg_quant)}                {default}
 set {compress(encoding)}                  {Reconstr. with Lloyd-Max quantizers}
 set {compress(stddev)}                    {10}
 set {compress(zn_mask)}                   {triangle}
 set {compress(zn_ratio)}                  {2}
 set {compress(zn_xform)}                  {DCT}
 set {compress(zn_blk_size)}               {32}
 set {compress(zn_dc)}                     {n}
 set {compress(window_size)}               {10}
 global {luminance}
 global {chrominance}

###################################################
# Utilities related variable setting goes here
###################################################
  global {util}

#Arith/Logic windows
  set {util(arith_log)}                    {}
  set {util(mult)}                         {img}
  set {util(mult_clip)}                    {1}
  set {util(mult_factor)}                  {2.0}

#Convert windows
  set {util(bin_thresh_data)}              {0}
  set {util(bin_thresh_level)}             {128}

  #color transform
  set {util(color)}                        {XYZ}
  set {util(color_dir)}                    {1}
  set {util(color_normal)}                 {1}
  set {util(color_remap)}                  {0}

  #gray level quantization
  set {util(gray_quant)}                   {16}
  set {util(gray_qmethod)}                 {S}

  #gray/binary code
  set {util(g_b)}                          {bin_gray}

  #pseudo-coloring
  set {util(g_c)}                          {pseudocol_freq}
  set {util(pseudo_lcf)}                   {10}
  set {util(pseudo_hcf)}                   {100}
  set {util(pseudo_lpc)}                   {R}
  set {util(pseudo_bpc)}                   {G}
  set {util(pseudo_hpc)}                   {B}
  set {util(g_c_r)}                        {red}
  set {util(g_c_g)}                        {green}
  set {util(g_c_b)}                        {blue}

  #color to gray
  set {util(c_g)}                          {luminance}

  #data type
  set {util(data_remap)}                   {0}

#Create windows
  set {util(img_name)}                     {simple}
  set {util(img_width)}                    {256}
  set {util(img_height)}                   {256}

  #checkerboard
  set {util(img_strtc)}                    {0}
  set {util(img_strtr)}                    {0}
  set {util(img_celw)}                     {32}
  set {util(img_celh)}                     {32}

  #line/rectangle
  set {util(img_box)}                      {}
  set {util(img_endr)}                     {256}
  set {util(img_endc)}                     {256}

  #circle
  set {util(img_cenr)}                     {128}
  set {util(img_cenc)}                     {128}
  set {util(img_rad)}                      {64}

  #sine/cosize/square wave
  set {util(img_freq)}                     {32}
  set {util(img_dir)}                      {horz}

  #extract band
  set {util(extract_band)}                 {1}

  #assemble bands
  set {util(red_src)}                      {}
  set {util(green_src)}                    {}
  set {util(blue_src)}                     {}

  #noise
  set {util(noise)}                        {negative_exp_noise}
  set {util(ns_alpha)}                     {1.25}
  set {util(ns_mean)}                      {0}
  set {util(ns_pepper)}                    {.03}
  set {util(ns_salt)}                      {.03}
  set {util(ns_var)}                       {100}
  set {util(black_width)}                  {256}
  set {util(black_height)}                 {256}
  set {util(noise_img)}                    {current}

#Enhance windows
  #brighten/darken
  set {util(brighten)}                     {1.2}

  #edge enhancement
  set {util(sharp)}                        {hfe_filter}
  set {util(transform)}                    {fft}

  #histogram equalization
  set {util(eq_band)}                      {0}

  #histogram stretch
  set {util(enhan_stretch_hc)}             {0.03}
  set {util(enhan_stretch_hl)}             {255}
  set {util(enhan_stretch_lc)}             {0.03}
  set {util(enhan_stretch_ll)}             {0}

  #pseudo-coloring
  # see convert

#Filter windows
  #mean, media, gaussin
  set {util(remove_method)}                {median_filter}
  set {util(mask_size)}                    {3}

  #specify a filter
  set {util(specify_hl)}                   {low-pass}
  set {util(filter_val)}                   {}

  #specify a blur
  set {util(msk_height)}                   {5}
  set {util(msk_width)}                    {5}
  set {util(degrade_shape)}                {rectangle}
  set {util(degrade_width)}                {5}
  set {util(degrade_height)}               {5}
  set {util(degrade_rad)}                  {5}
  set {util(blur_weight)}                  {Gaussian}
  set {util(param4)}                       {1.0}
  set {util(filter_dim)}                   {3}

#Size windows

  #crop
  set {util(crop_ul_col)}                  {0}
  set {util(crop_ul_row)}                  {0}
  set {util(util_crop_height)}             {100}
  set {util(util_crop_width)}              {100}

  #resize
  set {util(resize_width)}                 {256}
  set {util(resize_height)}                {256}
  set {util(linear)}                       {0}

  #spatial quantization
  set {util(quant_ht)}                     {32}
  set {util(quant_meth)}                   {median}
  set {util(quant_method)}                 {2}
  set {util(quant_wd)}                     {32}

#Miscellaneous  windows
  #compare
  set {util(compare)}                      {subtract}

  #add temporary image path
  global {pathname}
  set {pathname}                           {}

#@@@@ Configuration Ends @@@@@@@@@@@@@@@@@@@@@@@@@@

###################################################
# The following are variables used by the system, 
# Please don't modify anything below this line
###################################################

  # mouse coordinate transfer
  global {coordinate_info}
  set {coordinate_info}                    {}
  global {old_mouse_col}
  set {old_mouse_col}                      {}
  global {old_mouse_row}
  set {old_mouse_row}                      {}
  global {row_coord}
  set {row_coord}                          {}
  global {cname}
  set {cname}                              {}
  global {col_coord}
  set {col_coord}                          {}
  global {coordinates}
  set {coordinates}                        {.}
  global {null1}
  set {null1}                              {selection}
  global {null2}
  set {null2}                              {get}
  global {name}
  set {name}                               {.}

  # images used all the time
  global {current_image}
  set {current_image} {}
  # the index of the current selection in the image queue
  global {ImageSelectIdx} 
  set {ImageSelectIdx} {0}

  global {second_image}
  set {second_image} {}
  global {labeled_image}
  set {labeled_image} {}

  # temporary variables used everythere, named temp, temp1, temp2, ...
  global {temp}
  set {temp} {Deselect All}
  global set {temp1}
  set {temp1} {}

  global {bands}
  set {bands}                              {}
  global {data_fmt}
  set {data_fmt}                           {}
  global {data_type}
  set {data_type}                          {}
  global {img_fmt}
  set {img_fmt}                            {}
  global {new_name}
  set {new_name}                           {}
  global {result}
  set {result}                             {}
  global {rows}
  set {rows}                               {0}
  global {cols}
  set {cols}                               {0}


  global {image_counter}
  set {image_counter} {1}
  global {image_to_load}
  set {image_to_load} {}
}
