# **********************************************************************
# Computer Vision/Image Processing Tool Project - Dr. Scott Umbaugh SIUE
# **********************************************************************
#
#             File Name: CVIPInitGui.tcl
#           Description: The CVIPtools GUI configuration file
#         Related Files: CVIPInitGlobals.tcl
#   Initial Coding Date: Mon Jul 21 11:50:49 CDT 1997
#           Portability: TCL script file
#            References:
#             Author(s): Yansheng Wei
#                        Southern Illinois University @ Edwardsville
#
## Copyright (C) 1996 SIUE - by Scott Umbaugh and Yansheng Wei.
##
## Permission to use, copy, modify, and distribute this software and its
## documentation for any purpose and without fee is hereby granted, provided
## that the above copyright notice appear in all copies and that both that
## copyright notice and this permission notice appear in supporting
## documentation.  This software is provided "as is" without express or
## implied warranty.
##
# RCS (Revision Control System) Information
# ...Added automatically by RCS 
#
## $Log: CVIPInitGui.tcl,v $
## Revision 1.13  1997/08/08 17:29:05  yawei
## moved help window size to WM files
##
## Revision 1.12  1997/08/06 00:19:13  yawei
## *** empty log message ***
##
## Revision 1.11  1997/07/23 17:25:16  yawei
## addd the help window size.
##
## Revision 1.10  1997/07/21 17:59:09  yawei
## put the size configuration to another file and use dynamic
## resolution
##
#
#****************************************************************************

##########################################################
# To users:
#    This file contains the color setup of  the 
#    GUI components of all the windows. Feel free to change the
#    default values to your preference
#
# To CVIPtools developers:
#   - This file is manually formatted, please don't use
#     any tab keys when you modify this file
#   - Please keep the default values left aligned 
#   - Pay special consideration when adding a global
#     variable
#
##########################################################

# initialize the GUI properties
proc CvipInitGui {} {
  global {tcl_platform}

###########################################################
# Initial position of the toplevel windows
###########################################################

  global {toplevel_position}
  set {toplevel_position} {UR}

###########################################################
# Size configuration goes here
###########################################################
# Move to CVIPInitGlobals

###########################################################
# color scheme configuration goes here
###########################################################
  global {GuiColor}

# frames
  set {GuiColor(frame_bg)}         {gray90}

# ordinary buttons
#(include menubuttons and check/radio buttons, scrollbars)
  set {GuiColor(btn_bg)}           {gray90}
  set {GuiColor(btn_act_bg)}       {gray70}
 
  #menu and menu items colors
  set {GuiColor(menu_bg)}          {gray90}
  set {GuiColor(menu_act_bg)}      {gray70}

  #special bg for buttons in the main module
  set {GuiColor(btn_spec_bg)}      {gray95}
  set {GuiColor(btn_fg)}           {black}

  # DISMISS and PERFORM buttons
  set {GuiColor(dis_bg)}           {red}
  set {GuiColor(dis_act_bg)}       {gray70}
  set {GuiColor(dis_fg)}           {black}
  
  set {GuiColor(perf_bg)}          {#13d113}
  set {GuiColor(perf_act_bg)}      {#55ff55}
  set {GuiColor(perf_fg)}          {white}

  # ALERT button (Help, ...)
  set {GuiColor(alert_bg)}         {#ffff80}
  set {GuiColor(alert_act_bg)}     {gray70}
  set {GuiColor(alert_fg)}         {black}

# Ordinary r/w entry boxes
  set {GuiColor(entry_bg)}         {white}
  set {GuiColor(entry_fg)}         {black}

  # Read only entry box
  set {GuiColor(rentry_bg)}        {white}
  set {GuiColor(rentry_fg)}        {blue}

  # Read only entry box
  set {GuiColor(rentry_bg)}        {white}
  set {GuiColor(rentry_fg)}        {blue}

# Ordinary labels
  set {GuiColor(label_bg)}         {gray90}
  set {GuiColor(label_fg)}         {black}

  # Read only labels
  set {GuiColor(rlabel_bg)}        {gray90}
  set {GuiColor(rlabel_fg)}        {blue}

###########################################################
# Miscellaneous configuration goes here
###########################################################
  global {GuiMisc}
  set {GuiMisc(pop_up_fn)} {-adobe-symbol-medium-r-normal--14-140-75-75-p-85-adobe-fontspecific}
  set {GuiMisc(pop_up_text)}                             " \xaf "
  set {GuiMisc(pop_up_relief)}                             "raised"

###########################################################
# configure the XF components
# Please don't modify anything below thie line, unless you 
# really know what your are doing
###########################################################
  global {alertBox}
  set {alertBox(activeBackground)} {white}
  set {alertBox(activeForeground)} {}
  set {alertBox(after)} {0}
  set {alertBox(anchor)} {nw}
  set {alertBox(background)} {white}
  set {alertBox(button)} {0}
  set {alertBox(font)} {}
  set {alertBox(foreground)} {}
  set {alertBox(justify)} {center}
  set {alertBox(toplevelName)} {.alertBox}

  global {fsBox}
  set {fsBox(activeBackground)} {}
  set {fsBox(activeForeground)} {}
  set {fsBox(all)} {0}
  set {fsBox(background)} {}
  set {fsBox(button)} {0}
  set {fsBox(extensions)} {0}
  set {fsBox(font)} {}
  set {fsBox(foreground)} {}
  set {fsBox(internalPath)} {/opt/cvipimages}
  set {fsBox(name)} {}
  set {fsBox(path)} {}
  set {fsBox(pattern)} {*}
  set {fsBox(scrollActiveForeground)} {}
  set {fsBox(scrollBackground)} {}
  set {fsBox(scrollForeground)} {}
  set {fsBox(scrollSide)} {left}
  set {fsBox(showPixmap)} {0}


  global {textBox}
  set {textBox(activeBackground)} {}
  set {textBox(activeForeground)} {}
  set {textBox(background)} {white}
  set {textBox(button)} {0}
  set {textBox(contents)} {}
  set {textBox(font)} {}
  set {textBox(foreground)} {}
  set {textBox(scrollActiveForeground)} {}
  set {textBox(scrollBackground)} {gray90}
  set {textBox(scrollForeground)} {gray80}
  set {textBox(scrollSide)} {left}
  set {textBox(state)} {disabled}
  set {textBox(toplevelName)} {.textBox}

  global {yesNoBox}
  set {yesNoBox(activeBackground)} {red}
  set {yesNoBox(activeForeground)} {}
  set {yesNoBox(activebackground)} {red}
  set {yesNoBox(afterNo)} {0}
  set {yesNoBox(afterYes)} {0}
  set {yesNoBox(anchor)} {n}
  set {yesNoBox(background)} {white}
  set {yesNoBox(button)} {0}
  set {yesNoBox(font)} {*times-bold-r-normal*24*}
  set {yesNoBox(foreground)} {red}
  set {yesNoBox(justify)} {center}

# The following are local configurations to control the position and 
# size of CvipYesNo procedure
  set {yesNoBox(size)} {350x150}
  #use empty for WM management
  set {yesNoBox(position)} {}

# XF internal configurations, DON'T CHANGE ANYTHING AFTER THIS POINT
  global {internalAliasList}
  set {internalAliasList} {
   {img_q .base.image_queue.queue.listbox.listbox1} \
   {current_name .base.image_queue.image_info.name.image_name.entry2} \
   {load_img .base.extras.load_image.frame3.entry5}\
   {to_load .base.extras.load_image.frame3.entry5}\
   {b_band $topwin(util_name).util1.bband}\
   {g_band $topwin(util_name).util1.gband}\
   {r_band $topwin(util_name).util1.rband}\
   {v_band $topwin(util_name).util1.vband}\
   {his_mean $topwin(analysis_name).info.frame16.upper.checkbutton22} \
   {his_dev $topwin(analysis_name).info.frame16.upper.checkbutton23}\
   {his_skew $topwin(analysis_name).info.frame16.upper.checkbutton24}\
   {his_energy $topwin(analysis_name).info.frame16.upper.checkbutton25}\
   {his_entropy $topwin(analysis_name).info.frame16.upper.checkbutton5}\
   {spectral_rings $topwin(analysis_name).frame3.frame6.frame8.frame.entry5}\
   {spectral_sectors $topwin(analysis_name).frame3.frame6.frame8.frame10.entry5}\
   {binary_area $topwin(analysis_name).info.frame15.upper.checkbutton22}\
   {binary_cent $topwin(analysis_name).info.frame15.upper.checkbutton23}\
   {binary_orie $topwin(analysis_name).info.frame15.upper.checkbutton24}\
   {binary_peri $topwin(analysis_name).info.frame15.upper.checkbutton25}\
   {binary_eule $topwin(analysis_name).info.frame15.lower.checkbutton26}\
   {binary_proj $topwin(analysis_name).info.frame15.lower.checkbutton27}\
   {binary_thin $topwin(analysis_name).info.frame15.lower.checkbutton28}\
   {binary_aspe $topwin(analysis_name).info.frame15.lower.checkbutton29}\
  }

  global {autoLoadList}
  set {autoLoadList(GUI.tcl)} {0}
  set {autoLoadList(ana_feature.tcl)} {0}
  set {autoLoadList(ana_edge.tcl)} {0}
  set {autoLoadList(analysis.tcl)} {0}
  set {autoLoadList(compression.tcl)} {0}
  set {autoLoadList(enhancement.tcl)} {0}
  set {autoLoadList(latest_cvip)} {0}
  set {autoLoadList(latest_cvip13G)} {0}
  set {autoLoadList(restoration.tcl)} {0}
  set {autoLoadList(utilities.tcl)} {0}

  global {moduleList}

  global {preloadList}
  set {preloadList(xfInternal)} {}

  global {symbolicName}
  set {symbolicName(current_name)} {.base.image_queue.image_info.name.image_name.entry2}
  set {symbolicName(help_message)} {.help.frame2.frame.text2}
  set {symbolicName(img_q)} {.base.image_queue.queue.listbox.listbox1}
  set {symbolicName(noise_params)} {$topwin(util_name).util6.noise_params}
  set {symbolicName(root)} {.}
  set {symbolicName(view_band)} {.base.image_queue.functions.view_band}

  global {xfWmSetPosition}
  set {xfWmSetPosition} {}

  global {xfWmSetSize}
  set {xfWmSetSize} {. .prefs .enhancement $topwin(util_name)}

  global {xfAppDefToplevels}
  set {xfAppDefToplevels} {}

  global {CvipAlert}
  set {CvipAlert} "bell"

  global {STRANGE}
  set {STRANGE} {1}


}
