#***************************************************************************
# **********************************************************************
# Computer Vision/Image Processing Tool Project - Dr. Scott Umbaugh SIUE
# **********************************************************************
#
#             File Name: GUI.tcl
#           Description: Main module for graphical user interface
#         Related Files: 
#   Initial Coding Date: Mon Jul  8 16:27:17 CDT 1996
#           Portability: TCL script file
#            References:
#             Author(s): Yansheng Wei, Mark Zuke
#                        Southern Illinois University @ Edwardsville
#
## Copyright (C) 1996 SIUE - by Scott Umbaugh, Yansheng Wei, and Mark Zuke.
##
## Permission to use, copy, modify, and distribute this software and its
## documentation for any purpose and without fee is hereby granted, provided
## that the above copyright notice appear in all copies and that both that
## copyright notice and this permission notice appear in supporting
## documentation.  This software is provided "as is" without express or
## implied warranty.
##
# RCS (Revision Control System) Information
# ...Added automatically by RCS 
#
## $Log: CVIPmain.tcl,v $
## Revision 1.161  1997/08/13 20:35:03  smakam
## corrected the path setting for selecting the image from the file select box
##
## Revision 1.160  1997/08/06 00:09:47  yawei
## added window manager selection
##
## Revision 1.159  1997/06/30 19:55:56  yawei
## changed order in the "Save as" menu
##
## Revision 1.158  1997/06/25 22:26:58  yawei
## commented out CCC and BIN
##
## Revision 1.157  1997/06/25 18:33:27  yawei
## *** empty log message ***
##
## Revision 1.156  1997/06/23 20:52:37  yawei
## fixed file delete/selection problem
##
## Revision 1.155  1997/06/16 21:49:48  yawei
## m_anal ==> m_analysis
##
## Revision 1.154  1997/06/09 20:31:59  yawei
## added default images for default users
##
## Revision 1.153  1997/06/09 20:11:34  yawei
## added Alert, Debug to preference window
##
## Revision 1.152  1997/06/06 23:12:07  yawei
## added DEBUG control for the "puts" statements
##
## Revision 1.151  1997/06/04 18:02:44  yawei
## moved the xf procedures to the end.
## We might want to put these procedures in a separate module some
## time later
##
## Revision 1.150  1997/06/03 20:57:50  yawei
## finished configuration
##
## Revision 1.149  1997/05/30 14:00:54  yawei
## changed copyright info.
##
## Revision 1.148  1997/05/30 00:20:40  yawei
## added viewer display format selection
##
## Revision 1.147  1997/05/27 18:20:38  yawei
## changed for configuration.
##
## Revision 1.146  1997/05/14 23:06:07  yawei
## first round GUI configure done.
##
## Revision 1.145  1997/05/13 22:11:49  sumbaug
## changed part of the colors
##
## Revision 1.144  1997/05/13 18:11:52  yawei
## removed InitGlobals
## add CvipInitGui and CvipInitGlobals
##
## Revision 1.143  1997/05/02 16:24:53  yawei
## uniform image wirting.
##
## Revision 1.142  1997/04/29 16:43:52  yawei
## added "SAVE as" jpeg support
##
## Revision 1.141  1997/04/29 16:21:40  yawei
## added defaults for jpeg
##
## Revision 1.140  1997/04/28 23:39:06  yawei
## removed color requirement for GIF writing
##
## Revision 1.139  1997/04/28 23:24:14  yawei
## added GIF support
##
## Revision 1.138  1997/04/25 22:09:15  yawei
## added default for gray_quant method (IGS or standard) for compression/prep
##
## Revision 1.137  1997/04/23 13:59:24  yawei
## added zoom_method default to 0 order holding
##
## Revision 1.136  1997/04/22 22:23:12  yawei
## changed spelling for consistency
##
## Revision 1.135  1997/04/07 22:13:27  yawei
## droppend GIF write support
##
## Revision 1.134  1997/03/31 15:51:24  yawei
## changed wrap to word.
##
## Revision 1.133  1997/03/27 23:42:55  yawei
## Changed wrapping of textBox
##
## Revision 1.132  1997/03/27 23:34:48  yawei
## catch load images for yawei and sumbaug
##
## Revision 1.131  1997/03/27 01:54:17  yawei
## GUI == > CVIPTCL/GUI_SCRIPTS
##
## Revision 1.130  1997/03/26 20:45:47  mzuke
## added autoloading images to mzuke
##
## Revision 1.129  1997/03/25 16:11:09  mzuke
## added global texture_distance
##
## Revision 1.128  1997/03/12 16:13:39  yawei
## changed defaults util(color_normal, color_remap)
##
## Revision 1.127  1997/03/12 16:12:46  yawei
## changed defaults to util_color_norml
##
## Revision 1.126  1997/03/12 15:32:00  yawei
## Changed defaults to zonal
##
## Revision 1.125  1997/03/11 22:36:40  yawei
## corrected view_band and View spectrum problems
##
## Revision 1.124  1997/03/11 21:40:11  yawei
## Removed the first line.
##
## Revision 1.123  1997/03/11 21:01:48  yawei
## 1) Changed the toplevel_position defintion
## 2) Added STRANGE variable
##
## Revision 1.122  1997/03/11 20:55:07  yawei
## Tcl/Tk script upgrading from tcl7.3/tk3.6 to tcl7.6/tk4.2
## compatible. This process is automated with tkboost perl
## script. @ Tue Mar 11 14:51:36 CST 1997
##
## Revision 1.121  1997/02/05 16:15:53  mzuke
## removed mzuke from autoloading images at startup
##
# Revision 1.120  1997/02/04  01:12:08  yawei
# updated to use the new interpretor
#
# Revision 1.119  1997/01/17  21:58:39  yawei
# Changed versio to 3.4
#
# Revision 1.118  1997/01/13  14:19:10  mzuke
# changed CVIPversion from 3.3 to 3.4
#
# Revision 1.117  1996/12/31  00:21:00  yawei
# crop_Image ==> crop
#
# Revision 1.116  1996/12/30  23:45:13  yawei
# Changed version to 3.3
#
# Revision 1.115  1996/12/23  15:20:42  yawei
# changed auto-loading people syntax
#
# Revision 1.114  1996/12/23  15:01:20  akjoele
# Removed akjoele from list of auto-loading people.  Also
# restored absolute pathname of interpreter.
#
# Revision 1.113  1996/12/23  14:09:12  akjoele
# Removed absolute pathname of interpreter (cvipwish) with
# a relative name, to ease portability.
#
# Revision 1.112  1996/12/20  20:59:58  yawei
# added band ImageSelectIdx in view log_remapped images
#
# Revision 1.111  1996/12/13  23:07:16  yawei
# Added "phase" view
#
# Revision 1.110  1996/12/13  18:25:47  akjoele
# changed crop_inner to crop_image()
#
# Revision 1.109  1996/12/09  19:23:18  akjoele
# changed inverse_filter to inverse_xformfilter.
#
# Revision 1.108  1996/12/05  20:01:15  yawei
# Moved help_screen to CVIPhelp.tcl
#
# Revision 1.107  1996/12/05  03:59:07  yawei
# added defaults for util/enhan/hist_stretch
#
# Revision 1.106  1996/12/04  20:37:22  yawei
# changed lzw to zvl
#
# Revision 1.105  1996/12/04  13:38:52  mzuke
# changed button color from 'red' to '#ff5555'
# on help screen DISMISS
#
# Revision 1.104  1996/12/02  19:08:10  yawei
# Changed the laplacian to laplacian1 as the default
#
# Revision 1.103  1996/11/27  19:25:54  mzuke
# added Arve to list of autoloading images
#
# Revision 1.102  1996/11/18  14:30:19  yawei
# Added defaults for gray_level_multiplication (Byte clipping or not).
#
# Revision 1.101  1996/11/17  04:00:47  yawei
# added the default of gray_level quantization (IGs or standard).
#
# Revision 1.100  1996/11/15  02:19:52  yawei
# Added the global variable default for the bin_data for binary thresholding
#
# Revision 1.99  1996/11/13  20:54:51  yawei
# 1) Change version to 3.2
# 2) Addd clipping percentages for hist_stretch
#
# Revision 1.98  1996/11/08  00:36:46  yawei
# changed the default value of H_image weight to 1.0
#
# Revision 1.97  1996/11/06  20:26:02  mzuke
# changed icon name to CVIPHELP
#
# Revision 1.96  1996/11/06  18:22:20  yawei
# Degradation functions begin to work
#
# Revision 1.95  1996/11/03  03:26:54  yawei
# Changed
#
# Revision 1.94  1996/11/03  01:19:03  yawei
# added "add noise to black iamge" to Restore/noise
#
# Revision 1.93  1996/11/02  04:40:21  yawei
# Added homomorphic under ENHANCE/SHARP/FREQ_SHARP
#
# Revision 1.92  1996/10/31  04:22:53  yawei
# Added zonal2
#
# Revision 1.91  1996/10/30  21:37:19  yawei
# Added Frequency_filter window for restoration.
#
# Revision 1.90  1996/10/28  14:24:56  mzuke
# added 'CLEAR' button to rename window
#
# Revision 1.89  1996/10/22  22:40:16  yawei
# regular checkin
#
# Revision 1.88  1996/10/22  02:15:59  yawei
# added choice of choosing viewer in the preference.
#
# Revision 1.87  1996/10/18  14:53:22  yawei
# Added "Utilities | Edge Enhancement"
#
# Revision 1.86  1996/10/18  03:35:17  yawei
# Changed the default ImageSelectIdx for "Gray_Binary"
#
# Revision 1.85  1996/10/18  02:03:55  yawei
# added option of multiplying by a factor or by an image in UTILITIES/AL
#
# Revision 1.84  1996/10/17  23:00:43  yawei
# Added adaptive_contrast.
#
# Revision 1.83  1996/10/17  16:30:32  mzuke
# added links for tcl_help pages
#
# Revision 1.82  1996/10/10  20:26:41  yawei
# Added default kernal size on MMSE to be 9.
#
# Revision 1.81  1996/10/10  03:57:53  yawei
# Added "noise", "black image" to Utilities|Create
#
# Revision 1.80  1996/10/09  20:46:12  yawei
# hooked up zonal
#
# Revision 1.79  1996/10/08  21:31:27  mzuke
# modified image_post to correctly display repeated image names
#
# Revision 1.78  1996/10/08  19:16:05  mzuke
# tried unsuccessfully to correct '_r' name in loading
#
# Revision 1.77  1996/10/08  18:43:55  mzuke
# modified image_post procedure to remove "_r" from
# duplicate images, replacing it with "_#' where '#'
# is the value of the incremented global variable 'image_counter'
#
# Revision 1.76  1996/10/08  14:09:02  mzuke
# added autoloading of two images for users
# yawei and mzuke for ease of testing
#
# Revision 1.75  1996/10/08  02:43:22  yawei
# Added new functions
#
# Revision 1.74  1996/10/07  01:36:49  yawei
# changed the default mask size for High-boost
#
# Revision 1.73  1996/10/07  01:23:55  yawei
# chaned default value of "add original image" in enhance to "1"
#
# Revision 1.72  1996/10/03  03:22:20  yawei
# Added Compress/Pressing
#
# Revision 1.71  1996/10/03  02:13:14  yawei
# changed the offset value range of hfe in enhan/sharp to 0.01 - 3.0
#
# Revision 1.70  1996/10/02  18:07:54  yawei
# added Enhancement | Sharpening windos
#
# Revision 1.69  1996/09/30  00:42:53  yawei
# added restoration | Geometry module
#
# Revision 1.68  1996/09/27  23:38:56  yawei
# changed default values for Enhancement | pseudo_color
#
# Revision 1.67  1996/09/27  18:14:32  mzuke
# eliminated adding extension from 'save as vip' option
#
# Revision 1.66  1996/09/27  13:16:16  yawei
# added global variables for enhancement | pseudo_color
#
# Revision 1.65  1996/09/26  22:25:47  mzuke
# tried to allow entering a button area on root window
# to display contents in message area; not successful
#
# Revision 1.64  1996/09/24  00:29:55  yawei
# added global variables for Enhancement | Smooth
#
# Revision 1.63  1996/09/22  20:24:51  yawei
# changed global variable default values for pseudo_color
# color variables
#
# Revision 1.62  1996/09/22  18:41:34  yawei
# removed minor error
#
# Revision 1.61  1996/09/22  18:39:25  yawei
# added Compress|lossless global variables
#
# Revision 1.60  1996/09/20  21:11:00  yawei
# set the default value of clip in dpcm to "Yes", instead of 8.
#
# Revision 1.59  1996/09/20  18:08:22  mzuke
# added global variables help_short, help_medium,
# help_long for page sizing the help screens
#
# Revision 1.58  1996/09/18  22:50:33  yawei
# minor change
#
# Revision 1.57  1996/09/18  19:35:49  yawei
# added global variables for Compressin-> Lossy
#
# Revision 1.56  1996/09/15  18:33:10  mzuke
# removed Preprocessing from help_screen
#
# Revision 1.55  1996/09/15  15:52:49  mzuke
# removed Preprocessing from toplevel (root) window
#
# Revision 1.54  1996/09/14  03:32:16  yawei
# regular check in
#
# Revision 1.53  1996/09/13  17:05:11  mzuke
# working on help system
#
# Revision 1.52  1996/09/13  13:51:52  mzuke
# changed help window initial size; height now is 300
#
# Revision 1.51  1996/09/11  22:35:41  yawei
# changed to Gray level mapping
#
# Revision 1.50  1996/09/11  19:31:48  yawei
# added gray level remapping
#
# Revision 1.49  1996/09/11  18:24:04  mzuke
# added bindings to main message window to display version number
#
# Revision 1.48  1996/09/10  22:25:30  mzuke
# fixed save as functions
#
# Revision 1.47  1996/09/09  17:08:24  mzuke
# added calls for restoration help screens
#
# Revision 1.46  1996/09/09  15:42:57  mzuke
# modified call to compression toplevel to
# conform with new style of windows
#
# Revision 1.45  1996/09/09  15:14:05  yawei
# fixed multiple image names in teh queue
#
# Revision 1.44  1996/09/04  18:20:58  mzuke
# added icon names to root {CVIPtools} and help {HELP 3.1} windows
#
# Revision 1.43  1996/09/03  18:06:42  yawei
# changed anal_ to analysis_
#
# Revision 1.42  1996/09/01  20:45:44  mzuke
# added help for geometry and edge/line detectors
#
# Revision 1.41  1996/08/30  22:45:02  mzuke
# added modified 'Save as' function to append file type
# to image name during sav save operation
#
# Revision 1.40  1996/08/30  01:33:24  yawei
# check with the initial version of  restore_spatial.tcl
#
# Revision 1.39  1996/08/29  02:59:00  yawei
# changed version to 3.1
#
# Revision 1.38  1996/08/28  15:34:15  mzuke
# work in progress on preprocessing
#
# Revision 1.37  1996/08/28  03:16:22  yawei
# check with the initial restore_noise.tcl module
#
# Revision 1.36  1996/08/27  13:32:26  mzuke
# added menu for utilities help
#
# Revision 1.35  1996/08/22  23:41:49  yawei
# added type, height, width to morphological filters
#
# Revision 1.34  1996/08/22  19:44:46  mzuke
# raised help screen menubar
#
# Revision 1.33  1996/08/22  18:40:21  mzuke
# adjusted time from 10 to 20 seconds for alert boxes
# to automatically disappear
#
# Revision 1.32  1996/08/22  18:32:59  mzuke
# modified 'About this program' in Help screen
# to display timed alert box, which disappears
# after 10 seconds
#
# Revision 1.31  1996/08/22  16:23:28  yawei
# regular check-in.
#
# Revision 1.30  1996/08/21  19:30:33  yawei
# check-in with the first enhancement/histogram module
#
# Revision 1.29  1996/08/20  19:53:02  mzuke
# periodic checkin
#
# Revision 1.28  1996/08/19  19:46:10  mzuke
# in process of adding noise toplevel
#
# Revision 1.27  1996/08/19  12:48:25  mzuke
# added catch to Delete All Images to remove error message
#
# Revision 1.26  1996/08/16  17:11:47  mzuke
# removed 'Dismiss this help screen' from File menu in Help
#
# Revision 1.25  1996/08/16  14:36:43  mzuke
# added Segmentation help
#
# Revision 1.24  1996/08/16  13:44:35  mzuke
# corrected Help screen to display DISMISS button; added
# CVIPtools contributors to help
#
# Revision 1.23  1996/08/16  00:55:44  yawei
# regular check in
#
# Revision 1.21  1996/08/13  22:42:59  yawei
# check-in with the initial version of transforms
#
# Revision 1.20  1996/08/12  19:49:58  yawei
# Added global variables for Edge/Line detection.
#
# Revision 1.18  1996/08/09  22:35:53  yawei
# regular check in.
#
# Revision 1.17  1996/08/09  21:04:38  mzuke
# changed initial message
#
# Revision 1.16  1996/08/09  20:48:55  mzuke
# minor changes to help pages
#
# Revision 1.15  1996/08/09  19:21:47  yawei
# added variables for new utilities
#
# Revision 1.14  1996/08/09  18:52:50  mzuke
# changed help_screen procedure to display new CVIPtools help window
#
# Revision 1.13  1996/08/07  15:24:47  mzuke
# added defaults projection_height and projection_width
#
# Revision 1.12  1996/08/02  00:36:20  yawei
# changed loading image from file ImageSelectIdx box.
# 1) any image can be loaded using the file ImageSelectIdx box
# 2) if the iamge is in the system image path, it will just show the image file name
# 3) if the image is NOT in the system image path, it will also show the file directory
#
# Revision 1.11  1996/07/31  16:52:11  yawei
# regular checkin
#
# Revision 1.10  1996/07/25  13:47:14  yawei
# regular check in
#
# Revision 1.9  1996/07/23  17:09:30  yawei
# regular check in
#
# Revision 1.8  1996/07/18  17:16:55  mzuke
# moved Rename image from file menu to Image queue button
#
# Revision 1.7  1996/07/17  21:07:40  yawei
# changed so it will delete the labeled image in the feature window
#
# Revision 1.6  1996/07/17  17:02:29  yawei
# modifed to allow for future button bars, including global bariables, alias
# lists;
# Added preprocessing button in the main menu.
#
# Revision 1.5  1996/07/10  17:49:12  mzuke
# set global defaults for textBox calls
#
# Revision 1.5  1996/07/10  17:49:12  mzuke
# set global defaults for textBox calls
#
# Revision 1.4  1996/07/10  16:54:40  mzuke
# added new buttons to feature toplevel
#
# Revision 1.3  1996/07/10  15:53:12  mzuke
# added TextBox procedures
#
# Revision 1.2  1996/07/10  13:40:15  mzuke
# removed row and column coordinate defaults
#
# Revision 1.1  1996/07/08  22:07:18  mzuke
# Initial revision
#
#
#****************************************************************************
# Program: GUI
# Tcl version: 7.3 (Tcl/Tk/XF)
# Tk version: 3.6
# XF version: 2.3
#


# module inclusion
global env
global xfMisc
global xfLoadPath
global xfLoadInfo
set xfLoadInfo 0
if {[info exists env(XF_LOAD_PATH)]} {
  if {[string first $env(XF_LOAD_PATH) $env(CVIPHOME)/CVIPTCL/GUI_SCRIPTS] == -1} {
    set xfLoadPath $env(XF_LOAD_PATH):$env(CVIPHOME)/CVIPTCL/GUI_SCRIPTS
  } {
    set xfLoadPath $env(CVIPHOME)/CVIPTCL/GUI_SCRIPTS
  }
} {
  set xfLoadPath $env(CVIPHOME)/CVIPTCL/GUI_SCRIPTS
}

global argc
global argv
global tk_version
set tmpArgv ""
for {set counter 0} {$counter < $argc} {incr counter 1} {
  case [string tolower [lindex $argv $counter]] in {
    {-xfloadpath} {
      incr counter 1
      set xfLoadPath "[lindex $argv $counter]:$xfLoadPath"
    }
    {-xfstartup} {
      incr counter 1
      source [lindex $argv $counter]
    }
    {-xfbindfile} {
      incr counter 1
      set env(XF_BIND_FILE) "[lindex $argv $counter]"
    }
    {-xfcolorfile} {
      incr counter 1
      set env(XF_COLOR_FILE) "[lindex $argv $counter]"
    }
    {-xfcursorfile} {
      incr counter 1
      set env(XF_CURSOR_FILE) "[lindex $argv $counter]"
    }
    {-xffontfile} {
      incr counter 1
      set env(XF_FONT_FILE) "[lindex $argv $counter]"
    }
    {-xfmodelmono} {
      if {$tk_version >= 3.0} {
        tk colormodel . monochrome
      }
    }
    {-xfmodelcolor} {
      if {$tk_version >= 3.0} {
        tk colormodel . color
      }
    }
    {-xfloading} {
      set xfLoadInfo 1
    }
    {-xfnoloading} {
      set xfLoadInfo 0
    }
    {default} {
      lappend tmpArgv [lindex $argv $counter]
    }
  }
}
set argv $tmpArgv
set argc [llength $tmpArgv]
unset counter
unset tmpArgv


# procedure to show window .
proc ShowWindow. {args} {# xf ignore me 7

  # Window manager configurations
  global tk_version
  global CVIPversion
  global GuiColor
  global topwin
  global GUI_DEBUG

  wm positionfrom . user
  wm sizefrom . user
  wm geometry . $topwin(main_geom)
  wm maxsize . [lindex $topwin(main_max) 0] [lindex $topwin(main_max) 1]
  wm minsize . [lindex $topwin(main_min) 0] [lindex $topwin(main_min) 1]
  wm title . {CVIPtools from Southern Illinois University at Edwardsville}
  wm iconname . {CVIPtools}

  # build widget .base
  frame .base \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget .base.menubar
  frame .base.menubar \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2} \
    -relief {raised}

  # build widget .base.menubar.main_menu
  frame .base.menubar.main_menu \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget .base.menubar.main_menu.menubutton1
	menubutton .base.menubar.main_menu.menubutton1 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -menu ".base.menubar.main_menu.menubutton1.menu" \
    -text {File}

  # build widget .base.menubar.main_menu.menubutton1.menu
  menu .base.menubar.main_menu.menubutton1.menu \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" 
  .base.menubar.main_menu.menubutton1.menu add command \
    -command {add_path} \
    -label {Add a temporary path}
  .base.menubar.main_menu.menubutton1.menu add cascade \
    -label {Save as} \
    -menu {.base.menubar.main_menu.menubutton1.menu.save}
  .base.menubar.main_menu.menubutton1.menu add separator
  .base.menubar.main_menu.menubutton1.menu add command \
    -command {prefs} \
    -label {Preferences}
  .base.menubar.main_menu.menubutton1.menu add separator
  .base.menubar.main_menu.menubutton1.menu add command \
    -command {
CvipYesNo "Images that have not been SAVED will be LOST. REALLY EXIT ?"
if {$yesNoBox(button)} {destroy .; exit}} \
    -label {Exit}

  # build widget .base.menubar.main_menu.menubutton1.menu.menu
  menu .base.menubar.main_menu.menubutton1.menu.menu

  # build widget .base.menubar.main_menu.menubutton1.menu.save
  menu .base.menubar.main_menu.menubutton1.menu.save \
    -activebackground {gray70} \
    -background {gray90}
#  .base.menubar.main_menu.menubutton1.menu.save add command \
#    -command {set extension bin
#		if { $current_image == "" } {
#			CvipMessage "no image named  \"$current_image\"" no
#			return
#		}
#		update idletasks
#		set temp $current_image.$extension
#		if { [file exist $temp ] } {
#			if { [file isfile $temp] == 1 } {
#				if { [file readable $temp] && [file writable $temp] } {
#					if { [CvipYesNo "File $temp exists, overwrite it?"] == "yes" } {
#						cvip save $current_image $extension  $temp
#						CvipMessage "Image $current_image is saved as  $temp"
#					} else {
#						CvipMessage "File $temp not saved" "no"
#					}
#				} else {
#					CvipMessage "File $temp exists, permission denied"
#				}
#			} else {
#				CvipMessage "$temp exists, and is not an ordinary file"
#			}
#		} else {
#			cvip save $current_image $extension  $temp
#			CvipMessage "Image $current_image is saved as  $temp"
#		}
#		unset extension
#	} \
#   -label {BIN}
#
#  .base.menubar.main_menu.menubutton1.menu.save add command \
#    -command {
#		if { $current_image == ""} {
#			CvipMessage "no image named  \"$current_image\"" no
#			return
#		}
#	   set extension ccc
#		if {$bands != 3} {CvipAlert "Image : \"$current_image\" is not a 3-band image. \
#		The CCC format is used for color images only."}
#		if {$bands == 3} {
#			set temp $current_image.$extension
#			if { [file exist $temp ] } {
#				if { [file isfile $temp] == 1 } {
#					if { [file readable $temp] && [file writable $temp] } {
#						if { [CvipYesNo "File $temp exists, overwrite it?"] == "yes" } {
#							cvip save $current_image $extension  $temp
#							CvipMessage "Image $current_image is saved as  $temp"
#						} else {
#							CvipMessage "File $temp not saved" "no"
#						}
#					} else {
#						CvipMessage "File $temp exists, permission denied"
#					}
#				} else {
#					CvipMessage "$temp exists, and is not an ordinary file"
#				}
#			} else {
#				cvip save $current_image $extension  $temp
#				CvipMessage "Image $current_image is saved as  $temp"
#			}
#			unset extension
#		}
#	} \
#    -label {CCC}

  .base.menubar.main_menu.menubutton1.menu.save add command \
    -command {set extension ppm
		if { $current_image == ""} {
			CvipMessage "no image named  \"$current_image\"" no
			return
		}
		update idletasks
		set temp_image $current_image
		if {$data_type != "BYTE"} {
			set temp_image [cvip data_remap $current_image BYTE]
		}
		set temp $current_image.$extension
		if { [file exist $temp ] } {
			if { [file isfile $temp] == 1 } {
				if { [file readable $temp] && [file writable $temp] } {
					if { [CvipYesNo "File $temp exists, overwrite it?"] == "yes" } {
						cvip save $temp_image $extension  $temp
						CvipMessage "Image $current_image is saved as  $temp"
					} else {
						CvipMessage "File $temp not saved" "no"
					}
				} else {
					CvipMessage "File $temp exists, permission denied"
				}
			} else {
				CvipMessage "$temp exists, and is not an ordinary file"
			}
		} else {
			cvip save $temp_image $extension  $temp
			CvipMessage "Image $current_image is saved as  $temp"
		}
		if {$data_type != "BYTE"} {
			cvip dq $temp_image
		}
		unset extension
		} \
    -label {PPM}

  .base.menubar.main_menu.menubutton1.menu.save add command \
   -command {set extension ras
		if { $current_image == ""} {
			CvipMessage "no image named  \"$current_image\"" no
			return
		}
		update idletasks
		set temp_image $current_image
		if {$data_type != "BYTE"} {
			set temp_image [cvip data_remap $current_image BYTE]
		}
		set temp $current_image.$extension
		if { [file exist $temp ] } {
			if { [file isfile $temp] == 1 } {
				if { [file readable $temp] && [file writable $temp] } {
					if { [CvipYesNo "File $temp exists, overwrite it?"] == "yes" } {
						cvip save $temp_image $extension  $temp
						CvipMessage "Image $current_image is saved as  $temp"
					} else {
						CvipMessage "File $temp not saved" "no"
					}
				} else {
					CvipMessage "File $temp exists, permission denied"
				}
			} else {
				CvipMessage "$temp exists, and is not an ordinary file"
			}
		} else {
			cvip save $temp_image $extension  $temp
			CvipMessage "Image $current_image is saved as  $temp"
		}
		if {$data_type != "BYTE"} {
			cvip dq $temp_image
		}
		unset extension
		} \
    -label {RAS}

  .base.menubar.main_menu.menubutton1.menu.save add command \
    -command {set extension jpg
		if { $current_image == ""} {
			CvipMessage "no image named  \"$current_image\"" no
			return
		}
		update idletasks
		set  temp_image $current_image
		if {$data_type != "BYTE"} {
			set temp_image [cvip data_remap $current_image BYTE]
		}
		set temp $current_image.$extension
		if { [file exist $temp ] } {
			if { [file isfile $temp] == 1 } {
				if { [file readable $temp] && [file writable $temp] } {
					if { [CvipYesNo "File $temp exists, overwrite it?"] == "yes" } {
						cvip save $temp_image $extension  $temp
						CvipMessage "Image $current_image is saved as  $temp"
					} else {
						CvipMessage "File $temp not saved" "no"
					}
				} else {
					CvipMessage "File $temp exists, permission denied"
				}
			} else {
				CvipMessage "$temp exists, and is not an ordinary file"
			}
		} else {
			cvip save $temp_image $extension  $temp
			CvipMessage "Image $current_image is saved as  $temp"
		}
		if {$data_type != "BYTE"} {
			cvip dq $temp_image
		}
		unset extension
	} \
    -label {JPG}

  .base.menubar.main_menu.menubutton1.menu.save add command \
    -command {set extension tif
		if { $current_image == ""} {
			CvipMessage "no image named  \"$current_image\"" no
			return
		}
		update idletasks
		set temp_image $current_image
		if {$data_type != "BYTE"} {
			set temp_image [cvip data_remap $current_image BYTE]
		}
		set temp $current_image.$extension
		if { [file exist $temp ] } {
			if { [file isfile $temp] == 1 } {
				if { [file readable $temp] && [file writable $temp] } {
					if { [CvipYesNo "File $temp exists, overwrite it?"] == "yes" } {
						cvip save $temp_image $extension  $temp
						CvipMessage "Image $current_image is saved as  $temp"
					} else {
						CvipMessage "File $temp not saved" "no"
					}
				} else {
					CvipMessage "File $temp exists, permission denied"
				}
			} else {
				CvipMessage "$temp exists, and is not an ordinary file"
			}
		} else {
			cvip save $temp_image $extension  $temp
			CvipMessage "Image $current_image is saved as  $temp"
		}
		if {$data_type != "BYTE"} {
			cvip dq $temp_image
		}
		unset extension
	} \
    -label {TIF}

  .base.menubar.main_menu.menubutton1.menu.save add command \
    -command {
		if { $current_image == ""} {
			CvipMessage "no image named  \"$current_image\"" no
			return
		}
		if { $bands != 3 } {CvipMessage "Image: $current_image is converted to color image."}
		set extension gif
		update idletasks
		set temp_image $current_image
		if {$data_type != "BYTE"} {
			set temp_image [cvip data_remap $current_image BYTE]
		}
		set temp $current_image.$extension
		if { [file exist $temp ] } {
			if { [file isfile $temp] == 1 } {
				if { [file readable $temp] && [file writable $temp] } {
					if { [CvipYesNo "File $temp exists, overwrite it?"] == "yes" } {
						cvip save $temp_image $extension  $temp
						CvipMessage "Image $current_image is saved as  $temp"
					} else {
						CvipMessage "File $temp not saved" "no"
					}
				} else {
					CvipMessage "File $temp exists, permission denied"
				}
			} else {
				CvipMessage "$temp exists, and is not an ordinary file"
			}
		} else {
			cvip save $temp_image $extension  $temp
			CvipMessage "Image $current_image is saved as  $temp"
		}
		if {$data_type != "BYTE"} {
			cvip dq $temp_image
		}
		unset extension
	} \
    -label {GIF}

  .base.menubar.main_menu.menubutton1.menu.save add command \
   -command {set extension iris
		if { $current_image == ""} {
			CvipMessage "no image named  \"$current_image\"" no
			return
		}
		update idletasks
		set temp_image $current_image
		if {$data_type != "BYTE"} {
			set temp_image [cvip data_remap $current_image BYTE]
		}
		set temp $current_image.$extension
		if { [file exist $temp ] } {
			if { [file isfile $temp] == 1 } {
				if { [file readable $temp] && [file writable $temp] } {
					if { [CvipYesNo "File $temp exists, overwrite it?"] == "yes" } {
						cvip save $temp_image $extension  $temp
						CvipMessage "Image $current_image is saved as  $temp"
					} else {
						CvipMessage "File $temp not saved" "no"
					}
				} else {
					CvipMessage "File $temp exists, permission denied"
				}
			} else {
				CvipMessage "$temp exists, and is not an ordinary file"
			}
		} else {
			cvip save $temp_image $extension  $temp
			CvipMessage "Image $current_image is saved as  $temp"
		}
		if {$data_type != "BYTE"} {
			cvip dq $temp_image
		}
		unset extension
	} \
    -label {IRIS}

  .base.menubar.main_menu.menubutton1.menu.save add command \
    -command {set extension eps
		if { $current_image == ""} {
			CvipMessage "no image named  \"$current_image\"" no
			return
		}
		update idletasks
		set temp_image $current_image
		if {$data_type != "BYTE"} {
			set temp_image [cvip data_remap $current_image BYTE]
		}
		set temp $current_image.$extension
		if { [file exist $temp ] } {
			if { [file isfile $temp] == 1 } {
				if { [file readable $temp] && [file writable $temp] } {
					if { [CvipYesNo "File $temp exists, overwrite it?"] == "yes" } {
						cvip save $temp_image $extension  $temp
						CvipMessage "Image $current_image is saved as  $temp"
					} else {
						CvipMessage "File $temp not saved" "no"
					}
				} else {
					CvipMessage "File $temp exists, permission denied"
				}
			} else {
				CvipMessage "$temp exists, and is not an ordinary file"
			}
		} else {
			cvip save $temp_image $extension  $temp
			CvipMessage "Image $current_image is saved as  $temp"
		}
		if {$data_type != "BYTE"} {
			cvip dq $temp_image
		}
		unset extension
	} \
    -label {EPS}

  .base.menubar.main_menu.menubutton1.menu.save add command \
    -command {set extension itx
		if { $current_image == ""} {
			CvipMessage "no image named  \"$current_image\"" no
			return
		}
		update idletasks
		set temp_image $current_image
		if {$data_type != "BYTE"} {
			set temp_image [cvip data_remap $current_image BYTE]
		}
		set temp $current_image.$extension
		if { [file exist $temp ] } {
			if { [file isfile $temp] == 1 } {
				if { [file readable $temp] && [file writable $temp] } {
					if { [CvipYesNo "File $temp exists, overwrite it?"] == "yes" } {
						cvip save $temp_image $extension  $temp
						CvipMessage "Image $current_image is saved as  $temp"
					} else {
						CvipMessage "File $temp not saved" "no"
					}
				} else {
					CvipMessage "File $temp exists, permission denied"
				}
			} else {
				CvipMessage "$temp exists, and is not an ordinary file"
			}
		} else {
			cvip save $temp_image $extension  $temp
			CvipMessage "Image $current_image is saved as  $temp"
		}
		if {$data_type != "BYTE"} {
			cvip dq $temp_image
		}
		unset extension
	} \
    -label {ITX}

  .base.menubar.main_menu.menubutton1.menu.save add command \
	-command {set extension vip
		if { $current_image == ""} {
			CvipMessage "no image named  \"$current_image\"" no
			return
		}
		update idletasks
		set temp_image $current_image
		set temp $current_image.$extension
		if { [file exist $temp ] } {
			if { [file isfile $temp] == 1 } {
				if { [file readable $temp] && [file writable $temp] } {
					if { [CvipYesNo "File $temp exists, overwrite it?"] == "yes" } {
						cvip save $temp_image $extension  $temp
						CvipMessage "Image $current_image is saved as  $temp"
					} else {
						CvipMessage "File $temp not saved" "no"
					}
				} else {
					CvipMessage "File $temp exists, permission denied"
				}
			} else {
				CvipMessage "$temp exists, and is not an ordinary file"
			}
		} else {
			cvip save $temp_image $extension  $temp
			CvipMessage "Image $current_image is saved as  $temp"
		}
		unset extension
	} \
    -label {VIP}


  # build widget .base.menubar.main_menu.analysis
	button .base.menubar.main_menu.analysis -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -command {
      if { $topwin(analysis_name) == "NONE" } {
      set topwin(analysis_name) .m_analysis
      toplevel $topwin(analysis_name) -background {gray90}
      bind $topwin(analysis_name) <Destroy> {set topwin(analysis_name) NONE}
      analysis
      }
     } \
    -relief {flat} \
    -text {Analysis}


  # build widget .base.menubar.main_menu.restoration
	button .base.menubar.main_menu.restoration -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -command {
      if { $topwin(restore_name) == "NONE" } {
      set topwin(restore_name) .m_restore
      toplevel $topwin(restore_name) -background {gray90}
      bind $topwin(restore_name) <Destroy> {set topwin(restore_name) NONE}
      restoration
      }
     } \
    -relief {flat} \
    -text {Restoration}

  # build widget .base.menubar.main_menu.enhancement
	button .base.menubar.main_menu.enhancement -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
     -command {
       if { $topwin(enhan_name) == "NONE" } {
       set topwin(enhan_name) .m_enhan
       toplevel $topwin(enhan_name) -background {gray90}
       bind $topwin(enhan_name) <Destroy> {set topwin(enhan_name) NONE}
       enhancement
       }
      } \
    -relief {flat} \
    -text {Enhancement}

  # build widget .base.menubar.main_menu.compression
	button .base.menubar.main_menu.compression -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
     -command {
       if { $topwin(compress_name) == "NONE" } {
       set topwin(compress_name) .m_compress
       toplevel $topwin(compress_name) -background {gray90}
       bind $topwin(compress_name) <Destroy> {set topwin(compress_name) NONE}
       compression
       }
      } \
    -relief {flat} \
    -text {Compression}

  # build widget .base.menubar.main_menu.help
	button .base.menubar.main_menu.help -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(alert_act_bg)" \
    -background "$GuiColor(alert_bg)" \
    -command {help_screen} \
    -relief {flat} \
    -text {Help}



  # build widget .base.menubar.main_menu.utilities
	button .base.menubar.main_menu.utilities -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -command { if {$topwin(util_name) == "NONE"} {
       set utility histeq
       set topwin(util_name) .m_util
       toplevel $topwin(util_name) -background {gray90}
       bind $topwin(util_name) <Destroy> {set topwin(util_name) NONE}
       wm title $topwin(util_name) "Utilities"
       utilities}
     } \
    -relief {flat} \
    -text {Utilities}

  # pack widget .base.menubar.main_menu
  pack append .base.menubar.main_menu \
    .base.menubar.main_menu.menubutton1 {left frame center expand fillx} \
    .base.menubar.main_menu.analysis {left frame center expand fillx} \
    .base.menubar.main_menu.restoration {left frame center expand fillx} \
    .base.menubar.main_menu.enhancement {left frame center expand fillx} \
    .base.menubar.main_menu.compression {left frame center expand fillx} \
    .base.menubar.main_menu.utilities {left frame center expand fillx} \
    .base.menubar.main_menu.help {left frame center fillx}

  # pack widget .base.menubar
  pack append .base.menubar \
    .base.menubar.main_menu {top frame center fillx}

  # build widget .base.image_queue
  frame .base.image_queue \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2} \
    -relief {ridge}

  # build widget .base.image_queue.queue
  frame .base.image_queue.queue \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2} \
    -relief {ridge}

  # build widget .base.image_queue.queue.listbox
  frame .base.image_queue.queue.listbox \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {5}

  # build widget .base.image_queue.queue.listbox.scrollbar2
  scrollbar .base.image_queue.queue.listbox.scrollbar2 \
    -activebackground {gray50} \
    -troughcolor {gray80} \
    -command {.base.image_queue.queue.listbox.listbox1 yview} \
    -background {gray60} \
    -relief {sunken}

  # build widget .base.image_queue.queue.listbox.frame
  frame .base.image_queue.queue.listbox.frame \
    -background "$GuiColor(frame_bg)" \

  # build widget .base.image_queue.queue.listbox.frame.scrollbar3
  scrollbar .base.image_queue.queue.listbox.frame.scrollbar3 \
    -activebackground {gray50} \
    -troughcolor {gray80} \
    -command {.base.image_queue.queue.listbox.listbox1 xview} \
    -background {gray60} \
    -orient {horizontal} \
    -relief {sunken}

  # build widget .base.image_queue.queue.listbox.frame.frame
  frame .base.image_queue.queue.listbox.frame.frame \
    -background "$GuiColor(frame_bg)" \
    -width {19} -height {9}

  # pack widget .base.image_queue.queue.listbox.frame
  pack append .base.image_queue.queue.listbox.frame \
    .base.image_queue.queue.listbox.frame.scrollbar3 {left frame center pady 8 expand fillx} \
    .base.image_queue.queue.listbox.frame.frame {left frame center padx 8}

  # build widget .base.image_queue.queue.listbox.label1
  label .base.image_queue.queue.listbox.label1 \
    -background "$GuiColor(label_bg)" \
    -text {IMAGE QUEUE}

  # build widget .base.image_queue.queue.listbox.listbox1
  listbox .base.image_queue.queue.listbox.listbox1 \
    -background {white} \
    -exportselection {0} \
    -width {14} -height {4} \
    -relief {sunken} \
    -xscrollcommand {.base.image_queue.queue.listbox.frame.scrollbar3 set} \
    -yscrollcommand {.base.image_queue.queue.listbox.scrollbar2 set}
  # bindings
  bind .base.image_queue.queue.listbox.listbox1 <B1-Motion> {%W select anchor [%W nearest %y]}
  bind .base.image_queue.queue.listbox.listbox1 <Button-1> {%W select anchor [%W nearest %y]}
  bind .base.image_queue.queue.listbox.listbox1 <ButtonRelease-1> {set ImageSelectIdx [%W nearest %y]
		set current_image [%W get $ImageSelectIdx]
		set image_to_load $current_image
		CvipMessage "CURRENT IMAGE SELECTION : $image_to_load"
		get_image_info
		update
		to_load delete 0 end
  }
  bind .base.image_queue.queue.listbox.listbox1 <Control-ButtonRelease-1> {set temp [%W nearest %y]
	set second_image [%W get $temp]}
  bind .base.image_queue.queue.listbox.listbox1 <Control-B1-Motion> ""

  bind .base.image_queue.queue.listbox.listbox1 <Shift-ButtonRelease-1>  {set temp [%W nearest %y]
   set second_image [%W get $temp]}
  bind .base.image_queue.queue.listbox.listbox1 <Shift-B1-Motion> ""

  bind .base.image_queue.queue.listbox.listbox1 <Double-Button> {%W select anchor [%W nearest %y]
		set ImageSelectIdx [%W nearest %y]
		set image_to_load [%W get $ImageSelectIdx]
		set msg1 "Viewing $image_to_load"
		update
		unset msg1
		cvip view $image_to_load
		get_image_info
		update
	}
  bind .base.image_queue.queue.listbox.listbox1 <Meta-Button-1> ""
  bind .base.image_queue.queue.listbox.listbox1 <Meta-ButtonRelease-1> ""
  bind .base.image_queue.queue.listbox.listbox1 <Meta-B1-Motion> ""
  bind .base.image_queue.queue.listbox.listbox1 <Alt-Button-1> ""
  bind .base.image_queue.queue.listbox.listbox1 <Alt-ButtonRelease-1> ""
  bind .base.image_queue.queue.listbox.listbox1 <Alt-B1-Motion> ""

  # pack widget .base.image_queue.queue.listbox
  pack append .base.image_queue.queue.listbox \
    .base.image_queue.queue.listbox.label1 {top frame center fillx} \
    .base.image_queue.queue.listbox.frame {bottom frame center fillx} \
    .base.image_queue.queue.listbox.scrollbar2 {right frame center padx 8 filly} \
    .base.image_queue.queue.listbox.listbox1 {top frame center expand fill}

  # pack widget .base.image_queue.queue
  pack append .base.image_queue.queue \
    .base.image_queue.queue.listbox {left frame center expand fill}

  # build widget .base.image_queue.functions
  frame .base.image_queue.functions \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {4} \
    -relief {ridge} \
    -width {100}

  # build widget .base.image_queue.functions.view
	button .base.image_queue.functions.view -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_spec_bg)" \
    -borderwidth {4} \
    -command {
	 	if [catch {cvip view $current_image} ] {
			CvipMessage "No image \"$current_image\" displayed" "no"
		}
	 } \
    -text {View image}

  # build widget .base.image_queue.functions.view_hist
	button .base.image_queue.functions.view_hist -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_spec_bg)" \
    -borderwidth {4} \
    -command {
	 	if [catch {cvip view $current_image h} ] {
			CvipMessage "No histogram image displayed for \"$current_image\"" "no"
		} 
	 } \
    -text {View histogram}

  # build widget .base.image_queue.functions.view_logremap
	menubutton .base.image_queue.functions.view_logremap -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_spec_bg)" \
	 -relief {raised} \
    -borderwidth {4} \
    -text {View spectrum}
#	 -menu ".base.image_queue.functions.view_logremap.menu" \

  bind .base.image_queue.functions.view_logremap <Button-1> {
		MenuPopupPost .base.image_queue.functions.view_logremap.menu  %X %Y }
  bind .base.image_queue.functions.view_logremap <ButtonRelease-1> {
		MenuPopupRelease .base.image_queue.functions.view_logremap.menu %W }

  menu .base.image_queue.functions.view_logremap.menu \
    -activebackground {gray70} \
    -activeforeground {black} \
    -background {Gray95}
  

  .base.image_queue.functions.view_logremap.menu add command \
    -activebackground {gray70} \
	 -background {gray90} \
    -command {
		if { $current_image != "" } {
			if [catch {cvip view $current_image} cresult ] {
				CvipMessage "$cresult" "no"
			}
		} else {
			CvipMessage "No image selected" "no"
		}
	 } \
    -label {Magnitude} \
    -state {active}

  .base.image_queue.functions.view_logremap.menu add cascade \
    -label {Log-remapped} \
    -menu {.base.image_queue.functions.view_logremap.menu.menu}

  menu .base.image_queue.functions.view_logremap.menu.menu \
    -activebackground {gray70} \
    -activeforeground {black} \
    -background {Gray95}

  .base.image_queue.functions.view_logremap.menu.menu add command \
    -activebackground {gray70} \
	 -background {gray90} \
    -command { 
		if { $current_image != "" } {
			if [ catch {cvip view $current_image l} ] {
				CvipMessage "No transform detected in image history" "no"
			}
		} else {
			CvipMessage "No image selected" "no"
		}
	 } \
    -label {All bands} \
    -state {active}

  .base.image_queue.functions.view_logremap.menu.menu add command \
    -activebackground {gray70} \
	 -background {gray90} \
    -command { 
		if { $current_image != "" } {
			if [ catch {cvip view $current_image l 0} ] {
				CvipMessage "No transform detected in image history" "no"
			}
		} else {
			CvipMessage "No image selected" "no"
		}
	 } \
    -label {RED Band} \
    -state {active}

  .base.image_queue.functions.view_logremap.menu.menu add command \
    -activebackground {gray70} \
	 -background {gray90} \
    -command { 
		if { $current_image != "" } {
			if [ catch {cvip view $current_image l 1} ] {
				CvipMessage "No transform detected in image history" "no"
			}
		} else {
			CvipMessage "No image selected" "no"
		}
	 } \
    -label {GREEN Band} \
    -state {active}

  .base.image_queue.functions.view_logremap.menu.menu add command \
    -activebackground {gray70} \
	 -background {gray90} \
    -command { 
		if { $current_image != "" } {
			if [ catch {cvip view $current_image l 2} ] {
				CvipMessage "No transform detected in image history" "no"
			}
		} else {
			CvipMessage "No image selected" "no"
		}
	 } \
    -label {BLUE Band} \
    -state {active}

	bindtags .base.image_queue.functions.view_logremap.menu.menu {Menu all}

  .base.image_queue.functions.view_logremap.menu add command \
    -activebackground {gray70} \
	 -background {gray90} \
    -command {
      if {$current_image == ""} {
			CvipMessage "No image selected" "no"
         return
      }  
		set temp "cvip fft_phase $current_image 0 1"
		if [catch {set result [eval $temp]} cresult ] {
			CvipMessage "Input image should be a complex image" "no"
		} else {
			cvip rename $result ${current_image}_phase
			cvip view ${current_image}_phase
			cvip dq ${current_image}_phase
	 	}
	 } \
    -label {FFT Phase} \
    -state {active}

  bindtags .base.image_queue.functions.view_logremap.menu {Menu all} 

  bind .base.image_queue.functions.view_logremap.menu <B1-Motion> {
		MenuPopupMotion .base.image_queue.functions.view_logremap.menu  %W %X %Y }
 bind .base.image_queue.functions.view_logremap.menu <ButtonRelease-1> {
		MenuPopupRelease .base.image_queue.functions.view_logremap.menu %W }

  # build widget .base.image_queue.functions.save
	button .base.image_queue.functions.save -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_spec_bg)" \
    -borderwidth {4} \
    -command {
		set temp $current_image
		if { $current_image == "" } {
   		CvipMessage "No such image named \"$current_image\"." "no"
			return
		}
		if { [file exist $temp ] } {
			if { [file isfile $temp] == 1 } {
				if { [file readable $temp] && [file writable $temp] } {
					if { [CvipYesNo "File $temp exists, overwrite it?"] == "yes" } {
						cvip save $current_image default  $temp
						CvipMessage "Image $current_image is saved as  $temp" "no"
					} else {
						CvipMessage "File $temp not saved" "no"
					}
				} else {
					CvipMessage "File $temp exists, permission denied"
				}
			} else {
				CvipMessage "$temp exists, and is not an ordinary file"
			}
		} else {
			cvip save $current_image default $temp
			CvipMessage "Image $current_image is saved as  $temp" 
		}
	  } \
    -text {Save image}

  # build widget .base.image_queue.functions.delete
	button .base.image_queue.functions.delete -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_spec_bg)" \
    -borderwidth {4} \
    -command {
       if [catch {cvip dq $current_image}] {
       	CvipMessage "Image \"$current_image\" can not be deleted" "no"
		 } else {
       	CvipMessage "Image $current_image deleted"
         img_q delete $ImageSelectIdx
       	if {![string compare $current_image $second_image]} {
				set second_image ""
			}
       	set current_image ""
		 }
    } \
    -text {Delete image}

  # build widget .base.image_queue.functions.delete_all
	button .base.image_queue.functions.delete_all -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_spec_bg)" \
    -borderwidth {4} \
    -command { \
CvipYesNo "Are you sure you want to remove ALL images?"
if {$yesNoBox(button)} {foreach i [cvip list] {cvip dq $i}
img_q delete 0 end
set current_image "" }} \
    -text {Delete all images}

  # build widget .base.image_queue.functions.view_band
	menubutton .base.image_queue.functions.view_band -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_spec_bg)" \
    -borderwidth {4} \
    -relief {raised} \
    -text {View band ...}

  bind .base.image_queue.functions.view_band <Button-1> {
  		MenuPopupPost .base.image_queue.functions.view_band.menu %X %Y}
  bind .base.image_queue.functions.view_band <ButtonRelease-1> {
		MenuPopupRelease .base.image_queue.functions.view_band.menu %W }

  # build widget .base.image_queue.functions.view_band.menu
  menu .base.image_queue.functions.view_band.menu \
    -activebackground {gray70} \
    -activeforeground {black} \
    -background {Gray95}
  .base.image_queue.functions.view_band.menu add separator
  .base.image_queue.functions.view_band.menu add command \
    -activebackground {#ff7777} \
    -command {
	 	if [catch {cvip view $current_image r} ] {
			CvipMessage "No red band displayed for image \"$current_image\"" "no"
		}
	 } \
    -label {View red band} \
    -state {active}
  .base.image_queue.functions.view_band.menu add command \
    -activebackground {#88ff88} \
    -command {
	 	if [catch {cvip view $current_image g} ] {
			CvipMessage "No green band displayed for image \"$current_image\"" "no"
		}
	 } \
    -label {View green band}
  .base.image_queue.functions.view_band.menu add command \
    -activebackground {#77ccff} \
    -command {
	 	if [catch {cvip view $current_image b} ] {
			CvipMessage "No blue band displayed for image \"$current_image\"" "no"
		}
	 } \
    -label {View blue band}

	bindtags .base.image_queue.functions.view_band.menu {Menu all}

  bind .base.image_queue.functions.view_band.menu <B1-Motion> {
  		MenuPopupMotion .base.image_queue.functions.view_band.menu %W %X %Y}
  bind .base.image_queue.functions.view_band.menu <ButtonRelease-1> {
		MenuPopupRelease .base.image_queue.functions.view_band.menu %W }

 # build widget .base.image_queue.functions.rename
	button .base.image_queue.functions.rename -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_spec_bg)" \
    -borderwidth {4} \
    -command {rename_image} \
    -text {Rename image}

  # pack widget .base.image_queue.functions
  pack append .base.image_queue.functions \
    .base.image_queue.functions.view {top frame center expand fillx} \
    .base.image_queue.functions.view_band {top frame center fillx} \
    .base.image_queue.functions.view_hist {top frame center expand fillx} \
    .base.image_queue.functions.view_logremap {top frame center expand fillx} \
    .base.image_queue.functions.save {top frame center expand fillx} \
    .base.image_queue.functions.delete {top frame center expand fillx} \
    .base.image_queue.functions.delete_all {top frame center expand fillx} \
    .base.image_queue.functions.rename {top frame center expand fillx}
 

  # build widget .base.image_queue.image_info
  frame .base.image_queue.image_info \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget .base.image_queue.image_info.image_name
  frame .base.image_queue.image_info.image_name \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2} \
    -width {250}

  # build widget .base.image_queue.image_info.image_name.name
  label .base.image_queue.image_info.image_name.name \
    -background "$GuiColor(label_bg)" \
    -font {-Adobe-Helvetica-Bold-R-Normal--*-140-*} \
    -text {Current image :} \
    -width {20}

  # pack widget .base.image_queue.image_info.image_name
  pack append .base.image_queue.image_info.image_name \
    .base.image_queue.image_info.image_name.name {top frame center}

  # build widget .base.image_queue.image_info.name
  frame .base.image_queue.image_info.name \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2} \
    -width {250}

  # build widget .base.image_queue.image_info.name.image_name
  frame .base.image_queue.image_info.name.image_name

  # build widget .base.image_queue.image_info.name.image_name.scrollbar1
  scrollbar .base.image_queue.image_info.name.image_name.scrollbar1 \
    -activebackground {gray50} \
    -troughcolor {gray80} \
    -command {.base.image_queue.image_info.name.image_name.entry2 xview} \
    -background {gray60} \
    -orient {horizontal}

  # build widget .base.image_queue.image_info.name.image_name.entry2
  entry .base.image_queue.image_info.name.image_name.entry2 \
    -background "$GuiColor(rentry_bg)" \
    -foreground "$GuiColor(rentry_fg)" \
    -font {-Adobe-Helvetica-Bold-R-Normal--*-140-*} \
	 -state {disabled} \
    -xscrollcommand {.base.image_queue.image_info.name.image_name.scrollbar1 set} \
    -textvariable {current_image}

  # pack widget .base.image_queue.image_info.name.image_name
  pack append .base.image_queue.image_info.name.image_name \
    .base.image_queue.image_info.name.image_name.entry2 {top frame center fill} \
    .base.image_queue.image_info.name.image_name.scrollbar1 {top frame center fillx}

  # pack widget .base.image_queue.image_info.name
  pack append .base.image_queue.image_info.name \
    .base.image_queue.image_info.name.image_name {top frame center fill}

  # build widget .base.image_queue.image_info.labels
  frame .base.image_queue.image_info.labels \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2} \
    -width {110}

  # build widget .base.image_queue.image_info.labels.color_fmt
  label .base.image_queue.image_info.labels.color_fmt \
    -background "$GuiColor(label_bg)" \
    -font {-Adobe-Helvetica-Medium-R-Normal--*-120-*} \
    -text {Color format :}

  # build widget .base.image_queue.image_info.labels.img_fmt
  label .base.image_queue.image_info.labels.img_fmt \
    -background "$GuiColor(label_bg)" \
    -font {-Adobe-Helvetica-Medium-R-Normal--*-120-*} \
    -text {Image format :}

  # build widget .base.image_queue.image_info.labels.data_fmt
  label .base.image_queue.image_info.labels.data_fmt \
    -background "$GuiColor(label_bg)" \
    -font {-Adobe-Helvetica-Medium-R-Normal--*-120-*} \
    -text {Data format :}

  # build widget .base.image_queue.image_info.labels.data_type
  label .base.image_queue.image_info.labels.data_type \
    -background "$GuiColor(label_bg)" \
    -font {-Adobe-Helvetica-Medium-R-Normal--*-120-*} \
    -text {Data type :}

  # build widget .base.image_queue.image_info.labels.bands
  label .base.image_queue.image_info.labels.bands \
    -background "$GuiColor(label_bg)" \
    -font {-Adobe-Helvetica-Medium-R-Normal--*-120-*} \
    -text {Number of bands :}

  # build widget .base.image_queue.image_info.labels.rows
  label .base.image_queue.image_info.labels.rows \
    -background "$GuiColor(label_bg)" \
    -font {-Adobe-Helvetica-Medium-R-Normal--*-120-*} \
    -text {Number of rows :}

  # build widget .base.image_queue.image_info.labels.cols
  label .base.image_queue.image_info.labels.cols \
    -background "$GuiColor(label_bg)" \
    -font {-Adobe-Helvetica-Medium-R-Normal--*-120-*} \
    -text {Number of columns :}

  # pack widget .base.image_queue.image_info.labels
  pack append .base.image_queue.image_info.labels \
    .base.image_queue.image_info.labels.color_fmt {top frame center} \
    .base.image_queue.image_info.labels.img_fmt {top frame center} \
    .base.image_queue.image_info.labels.data_fmt {top frame center} \
    .base.image_queue.image_info.labels.data_type {top frame center} \
    .base.image_queue.image_info.labels.bands {top frame center} \
    .base.image_queue.image_info.labels.rows {top frame center} \
    .base.image_queue.image_info.labels.cols {top frame center}

  # build widget .base.image_queue.image_info.values
  frame .base.image_queue.image_info.values \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2} \
    -width {140}

  # build widget .base.image_queue.image_info.values.color_fmt
  label .base.image_queue.image_info.values.color_fmt \
    -background "$GuiColor(label_bg)" \
    -font {-Adobe-Helvetica-Medium-R-Normal--*-120-*} \
    -text {} \
    -textvariable {color_fmt}

  # build widget .base.image_queue.image_info.values.img_fmt
  label .base.image_queue.image_info.values.img_fmt \
    -background "$GuiColor(label_bg)" \
    -font {-Adobe-Helvetica-Medium-R-Normal--*-120-*} \
    -text {} \
    -textvariable {img_fmt}

  # build widget .base.image_queue.image_info.values.data_fmt
  label .base.image_queue.image_info.values.data_fmt \
    -background "$GuiColor(label_bg)" \
    -font {-Adobe-Helvetica-Medium-R-Normal--*-120-*} \
    -text {} \
    -textvariable {data_fmt}

  # build widget .base.image_queue.image_info.values.data_type
  label .base.image_queue.image_info.values.data_type \
    -background "$GuiColor(label_bg)" \
    -font {-Adobe-Helvetica-Medium-R-Normal--*-120-*} \
    -text {} \
    -textvariable {data_type}

  # build widget .base.image_queue.image_info.values.bands
  label .base.image_queue.image_info.values.bands \
    -background "$GuiColor(label_bg)" \
    -font {-Adobe-Helvetica-Medium-R-Normal--*-120-*} \
    -text {} \
    -textvariable {bands}

  # build widget .base.image_queue.image_info.values.rows
  label .base.image_queue.image_info.values.rows \
    -background "$GuiColor(label_bg)" \
    -font {-Adobe-Helvetica-Medium-R-Normal--*-120-*} \
    -text {} \
    -textvariable {rows}

  # build widget .base.image_queue.image_info.values.cols
  label .base.image_queue.image_info.values.cols \
    -background "$GuiColor(label_bg)" \
    -font {-Adobe-Helvetica-Medium-R-Normal--*-120-*} \
    -text {} \
    -textvariable {cols}

  # pack widget .base.image_queue.image_info.values
  pack append .base.image_queue.image_info.values \
    .base.image_queue.image_info.values.color_fmt {top frame center} \
    .base.image_queue.image_info.values.img_fmt {top frame center fillx} \
    .base.image_queue.image_info.values.data_fmt {top frame center fillx} \
    .base.image_queue.image_info.values.data_type {top frame center fillx} \
    .base.image_queue.image_info.values.bands {top frame center fillx} \
    .base.image_queue.image_info.values.rows {top frame center fillx} \
    .base.image_queue.image_info.values.cols {top frame center}

  # pack widget .base.image_queue.image_info
  pack append .base.image_queue.image_info \
    .base.image_queue.image_info.image_name {top frame center fillx} \
    .base.image_queue.image_info.name {top frame center fillx} \
    .base.image_queue.image_info.labels {left frame center fill} \
    .base.image_queue.image_info.values {right frame center fill}

  # pack widget .base.image_queue
  pack append .base.image_queue \
    .base.image_queue.functions {left frame center filly} \
    .base.image_queue.queue {left frame center expand fill} \
    .base.image_queue.image_info {left frame center filly}

  # build widget .base.extras
  frame .base.extras \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget .base.extras.load_image
  frame .base.extras.load_image \
    -background "$GuiColor(frame_bg)" 

  # build widget .base.extras.load_image.scrollbar7
  scrollbar .base.extras.load_image.scrollbar7 \
    -activebackground {gray50} \
    -troughcolor {gray80} \
    -command {.base.extras.load_image.frame3.entry5 xview} \
    -background {gray60} \
    -orient {horizontal} \
    -relief {raised}

  # build widget .base.extras.load_image.frame3
  frame .base.extras.load_image.frame3 \
    -background "$GuiColor(frame_bg)"

  # build widget .base.extras.load_image.frame3.entry5
  entry .base.extras.load_image.frame3.entry5 \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
 -xscrollcommand {.base.extras.load_image.scrollbar7 set} \
    -textvariable {image_to_load}
  # bindings
  bind .base.extras.load_image.frame3.entry5 <Key-Return> \
 {if [catch {cvip load $image_to_load} result] {
   global message
   CvipMessage "Cannot find $image_to_load -- check spelling or image path."
 } else {
	if [string compare [file tail $result] $result] {
		set temp $result
		while {![catch {cvip info [file tail $temp] } ]} {
			set temp ${temp}_r
			incr image_counter
			if { $GUI_DEBUG } { puts stderr $temp }
		}
		cvip rename $result  [file tail $temp]
		set result $temp
	}
	set image_to_load [file tail $result]
	CvipMessage "LOADING : $image_to_load"
	update idletasks
	get_image_info
	update
	cvip view $image_to_load
	img_q insert end $image_to_load
	load_img delete 0 end
	img_q select clear 0 end  
	img_q select set end end
   set ImageSelectIdx end}
}

  # build widget .base.extras.load_image.frame3.load
	button .base.extras.load_image.frame3.load -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(perf_act_bg)" \
    -background "$GuiColor(perf_bg)" \
	-foreground "$GuiColor(perf_fg)" \
    -command {CvipMessage "Select an image to load from the file select box"
update
FSBox "SELECT AN IMAGE:"
if {$fsBox(name) != ""} {
if [catch {cvip load $fsBox(name)} result] {
   if [catch {cvip load $fsBox(path)$fsBox(name)} result] {
	if [catch {cvip load $fsBox(path)/$fsBox(name)} result] {
   CvipAlert "No such image."
   } else {
        set image_to_load $result
        if [string compare [file tail $result] $result] {
                set temp $result
                while {![catch {cvip info [file tail $temp] } ]} {
                        set temp ${temp}_r
                        incr image_counter
                        if { $GUI_DEBUG } {puts stderr $temp}
                }
                cvip rename $result  [file tail $temp]
                set result $temp
        }
        set image_to_load [file tail $result]
        get_image_info
        update
   cvip view $image_to_load
   CvipMessage "LOADING : $image_to_load"
   img_q insert end  $image_to_load
   load_img delete 0 end
   img_q select clear 0 end
   img_q select set end end
   set ImageSelectIdx end
   }
   } else {
        set image_to_load $result
	if [string compare [file tail $result] $result] {
		set temp $result
		while {![catch {cvip info [file tail $temp] } ]} {
			set temp ${temp}_r
			incr image_counter
			if { $GUI_DEBUG } {puts stderr $temp}
		}
		cvip rename $result  [file tail $temp]
		set result $temp
	}
	set image_to_load [file tail $result]
	get_image_info
	update
   cvip view $image_to_load
   CvipMessage "LOADING : $image_to_load"
   img_q insert end  $image_to_load
   load_img delete 0 end
   img_q select clear 0 end  
   img_q select set end end
   set ImageSelectIdx end
   }
} else {
	if [string compare [file tail $result] $result] {
		set temp $result
		while {![catch {cvip info [file tail $temp] } ]} {
			set temp ${temp}_r
			if { $GUI_DEBUG } {puts stderr $temp}
		}
		cvip rename $result  [file tail $temp]
		set result $temp
	}
	set image_to_load [file tail $result]
	get_image_info
	update
	cvip view $image_to_load
	CvipMessage "LOADING: $fsBox(name)."
	img_q insert end $image_to_load
   load_img delete 0 end
	img_q select clear 0 end  
	img_q select set end end
   set ImageSelectIdx end
}
}
if {$fsBox(name) == ""} {CvipMessage "Image loading cancelled "}
} \
    -text {Load image :}

  # pack widget .base.extras.load_image.frame3
  pack append .base.extras.load_image.frame3 \
    .base.extras.load_image.frame3.load {left frame center} \
    .base.extras.load_image.frame3.entry5 {top frame center expand fill}

  # pack widget .base.extras.load_image
  pack append .base.extras.load_image \
    .base.extras.load_image.frame3 {top frame center fillx} \
    .base.extras.load_image.scrollbar7 {top frame center fillx}

  # pack widget .base.extras
  pack append .base.extras \
    .base.extras.load_image {top frame center fill}

  # build widget .base.message
  frame .base.message \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget .base.message.message
  message .base.message.message \
    -aspect {1500} \
    -background {gray90} \
    -padx {5} \
    -pady {2} \
    -textvariable {message}
#  bindings
  bind .base.message.message <ButtonPress-1> {set temp $message;set message " "; set message "CVIPtools Version ${CVIPversion}"}
  bind .base.message.message <ButtonRelease-1> {set message " ";set message $temp}

  # pack widget .base.message
  pack append .base.message \
    .base.message.message {top frame center expand fillx}

  # pack widget .base
  pack append .base \
    .base.menubar {top frame center fillx} \
    .base.image_queue {top frame center expand fill} \
    .base.extras {top frame center expand fill} \
    .base.message {top frame center expand fillx}

  # pack widget .
  pack append . \
    .base {top frame center expand fill}

}



# Procedure: add_path
proc add_path {} {
##########
# Procedure: add_path
# Author: Mark Zuke (mzuke@ee.siue.edu) May 2, 1996
# Description: adds a temporary image path 
#              
# Arguments: none
# Returns: none
# Sideeffects: none 
#               
##########

 global toplevel_position
 global message
  global GuiColor
  global GUI_DEBUG

  # build widget .add_path
  if {"[info procs XFEdit]" != ""} {
    catch "XFDestroy .add_path"
  } {
    catch "destroy .add_path"
  }
  toplevel .add_path   -background {GRAY90}

  # Window manager configurations
  global tk_version
  wm positionfrom .add_path user
  wm sizefrom .add_path user
  if { $GUI_DEBUG } {puts [top_pos 350x200]}
  wm geometry .add_path [top_pos 350x200]
  wm maxsize .add_path 450 300
  wm minsize .add_path 350 200
  wm title .add_path {ADD TEMPORARY IMAGE PATH}


  # build widget .add_path.entry
  frame .add_path.entry  -background "$GuiColor(frame_bg)" \
  -borderwidth {2}

  # build widget .add_path.entry.new_path
  frame .add_path.entry.new_path  -background "$GuiColor(frame_bg)" 

  # build widget .add_path.entry.new_path.scrollbar7
  scrollbar .add_path.entry.new_path.scrollbar7  -troughcolor {GRAY40}  -command {.add_path.entry.new_path.frame3.entry5 xview}  -background {gray80}  -orient {horizontal}  -relief {raised}  -width {11}

  # build widget .add_path.entry.new_path.frame3
  frame .add_path.entry.new_path.frame3  -background "$GuiColor(frame_bg)"

  # build widget .add_path.entry.new_path.frame3.entry5
  entry .add_path.entry.new_path.frame3.entry5  \
  -background "$GuiColor(entry_bg)" \
  -foreground "$GuiColor(entry_fg)" \
  -relief {sunken} -xscrollcommand {.add_path.entry.new_path.scrollbar7 set}  -textvariable {pathname}

  # build widget .add_path.entry.new_path.frame3.label4
  label .add_path.entry.new_path.frame3.label4  -background {GRAY90}  -padx {2}  -relief {raised}  -text {New path:}

  # pack widget .add_path.entry.new_path.frame3
  pack append .add_path.entry.new_path.frame3  .add_path.entry.new_path.frame3.label4 {left frame center}  .add_path.entry.new_path.frame3.entry5 {top frame center expand fill}

  # pack widget .add_path.entry.new_path
  pack append .add_path.entry.new_path  .add_path.entry.new_path.frame3 {top frame center fillx}  .add_path.entry.new_path.scrollbar7 {top frame center fillx}

  # pack widget .add_path.entry
  pack append .add_path.entry  .add_path.entry.new_path {top frame center fill}

  # build widget .add_path.perform
  frame .add_path.perform  -background "$GuiColor(frame_bg)" \
  -borderwidth {2}

  # build widget .add_path.perform.perform
	button .add_path.perform.perform -padx {1} -pady {1} -highlightthickness {0} \
	-activebackground "$GuiColor(perf_act_bg)" \
	-background "$GuiColor(perf_bg)" \
	-foreground "$GuiColor(perf_fg)" \
	-command {if {$pathname != ""} {cvip add_path $pathname}
if {$pathname == ""} {CvipMessage "You MUST enter a path name first."}}  -text {PERFORM}

  # build widget .add_path.perform.dismiss
	button .add_path.perform.dismiss -padx {1} -pady {1} -highlightthickness {0}\
	-activebackground "$GuiColor(dis_act_bg)"\
	-background "$GuiColor(dis_bg)"\
	-command {destroy .add_path}  -text {DISMISS}  -width {10}

  # pack widget .add_path.perform
  pack append .add_path.perform  .add_path.perform.dismiss {left frame sw expand}  .add_path.perform.perform {left frame se expand}

  # build widget .add_path.message
  frame .add_path.message  \
    -background "$GuiColor(frame_bg)" \
	 -borderwidth {2}

  # build widget .add_path.message.path_message
  label .add_path.message.path_message  -background {gray90}  -text {Enter  image path below}

  # pack widget .add_path.message
  pack append .add_path.message  .add_path.message.path_message {top frame center expand}

  # pack widget .add_path
  pack append .add_path  .add_path.message {top frame center expand fill}  .add_path.entry {top frame center expand fill}  .add_path.perform {bottom frame center fillx}

  if {"[info procs XFEdit]" != ""} {
    catch "XFMiscBindWidgetTree .add_path"
    after 2 "catch {XFEditSetShowWindows}"
  }
}


# Procedure: current_name
proc current_name { args} {
#xf ignore me 4
return [eval ".base.image_queue.image_info.name.image_name.entry2 $args"]
}


# Procedure: get_coordinates
proc get_coordinates {} {
# xf ignore me 7
##########
# Procedure: get_coordinates
# Author: Mark Zuke (mzuke@ee.siue.edu) June 26, 1996
# Description: gets current r,c coordinates
#              
# Arguments: none
# Returns: none
# Sideeffects: none
##########

global image_to_load
global current_image
global toplevel_position
global row_coord
global col_coord
global d_row
global d_col
global cname
global message
global cresult


if [catch {set coordinate_info [selection get]
scan $coordinate_info {%s %s %d %d %d %d} cname null2 row_coord col_coord d_row d_col} cresult] { CvipAlert "Please select coordinates from your image:
1) left mouse button selects a single point
2) center mouse button drags a rectangle"}
}


# Procedure: get_image_info
proc get_image_info {} {
# xf ignore me 7
##########
# Procedure: get_image_info
# Author: Mark Zuke (mzuke@ee.siue.edu) April 16, 1996
# Description: gets image info (img format, color format, bands,
#              rows, columns, data type, data format)
# Arguments: none
# Returns: none
# Sideeffects: none
##########

global image_to_load
global img_fmt
global color_fmt
global bands
global rows
global cols
global data_type
global data_fmt
global current_image
global toplevel_position
global row_coord
global col_coord
global d_row
global d_col
global message




	if {$image_to_load == ""} {CvipMessage "No image selected"; return}
	
	set imageinfo [cvip info $image_to_load]
	scan $imageinfo {%s %s %d %d %d %s %s}  img_fmt color_fmt bands rows cols data_type data_fmt

    
set current_image $image_to_load
current_name delete 0 end
current_name insert end $current_image
if {$bands == 1} {.base.image_queue.functions.view_band configure -state disabled}
if {$bands == 3} {.base.image_queue.functions.view_band configure -state normal}
}


# Procedure: image_post
proc image_post { args} {
# xf ignore me 7
##########
# Procedure: image_post
# Author: Mark Zuke (mzuke@ee.siue.edu) May 31, 1996
# Description: performs some needed postprocessing of returned images;
#              add functions needed by ALL returned images here
# Arguments: image - an image resulting from some operation
# Returns: none
# Sideeffects: none
##########

global image_to_load
global current_image
global second_image
global image_counter
global img_q
global new_name

if { [string range $args [expr [string length $args]-2] end] == "_r"} {
	set temp3 [string trimright $args _r]
	set temp3 [append temp3 _#$image_counter]
	cvip rename $args $temp3
	incr image_counter
	img_q delete end
	img_q insert end $temp3
	set new_name $temp3
	unset temp3
}

}

# Procedure: help_message
proc help_message { args} {
#xf ignore me 4
return [eval ".help.frame2.frame.text2 $args"]
}

# Procedure: img_q
proc img_q { args} {
#xf ignore me 4
return [eval ".base.image_queue.queue.listbox.listbox1 $args"]
}


# Procedure: load_img
proc load_img { args} {
#xf ignore me 4
return [eval ".base.extras.load_image.frame3.entry5 $args"]
}


# Procedure: prefs
proc prefs {} {
##########
# Procedure: prefs
# Author: Mark Zuke (mzuke@ee.siue.edu) May 9, 1996
# Description: displays toplevel preferences screen 
#              
# Arguments: none
# Returns: none
# Sideeffects: none 
##########


global toplevel_position
global initial_pref_position

  toplevel .prefs -background {gray90}
  global GuiColor
  global topwin
  global CvipAlertControl
  global GUI_DEBUG
  global GuiMisc
  global etc

  # Window manager configurations
  global tk_version
  wm positionfrom .prefs user
  wm sizefrom .prefs user
  wm geometry .prefs [top_pos $topwin(preference)]
  wm maxsize .prefs [lindex $topwin(preference_max) 0] [lindex $topwin(preference_max) 1]
  wm minsize .prefs [lindex $topwin(preference_min) 0] [lindex $topwin(preference_min) 1]
  wm title .prefs {PREFERENCES}


  # build widget .prefs.toplevel_position
  frame .prefs.toplevel_position  \
  -background "$GuiColor(frame_bg)" \
  -borderwidth {2} \
  -relief {ridge}

  # build widget .prefs.toplevel_position.label
  frame .prefs.toplevel_position.label \
  -background "$GuiColor(frame_bg)"\
  -borderwidth {2}

  # build widget .prefs.toplevel_position.label.toplevel_position
  label .prefs.toplevel_position.label.toplevel_position \
  -background "$GuiColor(label_bg)"  \
  -text {Default corner position of toplevel windows:}

  # pack widget .prefs.toplevel_position.label
  pack append .prefs.toplevel_position.label  \
  .prefs.toplevel_position.label.toplevel_position {top frame w}
  pack configure .prefs.toplevel_position.label.toplevel_position -padx {20}

  # build widget .prefs.toplevel_position.upper
  frame .prefs.toplevel_position.upper\
  -background "$GuiColor(frame_bg)" \
  -borderwidth {2}

  # build widget .prefs.toplevel_position.upper.upper_left
	radiobutton .prefs.toplevel_position.upper.upper_left\
	-padx {1}\
	-pady {1}\
	-highlightthickness {0}\
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
	-relief {flat}\
	-text {Upper left}\
	-value {ul}\
	-variable {popup_position}

  # build widget .prefs.toplevel_position.upper.upper_right
	radiobutton .prefs.toplevel_position.upper.upper_right -padx {1} -pady {1} -highlightthickness {0}\
   -activebackground "$GuiColor(btn_act_bg)" \
   -background "$GuiColor(btn_bg)" \
	-relief {flat}  -text {Upper right}  -value {ur}  -variable {popup_position}

  # pack widget .prefs.toplevel_position.upper
  pack append .prefs.toplevel_position.upper  \
  .prefs.toplevel_position.upper.upper_left {left frame w expand }\
  .prefs.toplevel_position.upper.upper_right {left frame w expand}

  # build widget .prefs.toplevel_position.lower
  frame .prefs.toplevel_position.lower\
  -background "$GuiColor(frame_bg)" \
  -borderwidth {2}

  # build widget .prefs.toplevel_position.lower.lower_left
	radiobutton .prefs.toplevel_position.lower.lower_left -padx {1} -pady {1} -highlightthickness {0}\
   -activebackground "$GuiColor(btn_act_bg)" \
   -background "$GuiColor(btn_bg)" \
	-relief {flat}  -text {Lower left}  -value {ll}  -variable {popup_position}

  # build widget .prefs.toplevel_position.lower.lower_right
	radiobutton .prefs.toplevel_position.lower.lower_right -padx {1} -pady {1} -highlightthickness {0}\
   -activebackground "$GuiColor(btn_act_bg)" \
   -background "$GuiColor(btn_bg)" \
	-relief {flat}  -text {Lower right}  -value {lr}  -variable {popup_position}

  # pack widget .prefs.toplevel_position.lower
  pack append .prefs.toplevel_position.lower\
   .prefs.toplevel_position.lower.lower_left {left frame w expand}\
	.prefs.toplevel_position.lower.lower_right {left frame w expand}

  # pack widget .prefs.toplevel_position
  pack append .prefs.toplevel_position  \
  .prefs.toplevel_position.label {top frame center fillx}\
  .prefs.toplevel_position.upper {top frame center fillx} \
  .prefs.toplevel_position.lower {top frame center fillx}

  pack configure .prefs.toplevel_position.lower -padx {20}
  pack configure .prefs.toplevel_position.upper -padx {20}

  # build widget .prefs.perform
  frame .prefs.perform  \
  -background "$GuiColor(frame_bg)" \
  -borderwidth {2} \
  -relief {raised} 

  # build widget .prefs.perform.bar
  frame .prefs.perform.bar\
  -background "$GuiColor(frame_bg)" \
  -borderwidth {2}

  # build widget .prefs.perform.bar.perform
	button .prefs.perform.bar.perform -padx {1} -pady {1} -highlightthickness {0}\
	-activebackground "$GuiColor(perf_act_bg)" \
	-background "$GuiColor(perf_bg)" \
	-foreground "$GuiColor(perf_fg)" \
	-command {	
	set _PrefMesg "Preference: Window manager-"
	switch -exact -- $etc(wm_name) {
		OpenWin  { append _PrefMesg "OpenWindows" }
		CDE      { append _PrefMesg "CDE"}
		FVWM     { append _PrefMesg "FVWM"}
		IRIX     { append _PrefMesg "IRIX"}
		Win32    { append _PrefMesg "WindowsNT/95"}
	}
	append _PrefMesg "; Toplevel window position-"
	switch -exact -- $popup_position {
		ul {set toplevel_position UL; append _PrefMesg "Upper-left"}
		ur {set toplevel_position UR; append _PrefMesg "Upper-right"}
		ll {set toplevel_position LL; append _PrefMesg "Lower-left"}
		lr {set toplevel_position LR; append _PrefMesg "Lower-right"}
	}
	destroy .prefs
	if { $etc(viewer) == "" } {
		CvipAlert "No viewer selected, using the original viewer $etc(viewer_original)"
		set etc(viewer) $etc(viewer_original)
		return;
	}
	cvip viewer $etc(viewer) $etc(display_format)
	append _PrefMesg ";\n\t Image viewer-$etc(viewer); Display image format-$etc(display_format)"
	CvipMessage "$_PrefMesg"
	unset _PrefMesg

  }  -text {PERFORM}

  # build widget .prefs.perform.bar.dismiss
	button .prefs.perform.bar.dismiss -padx {1} -pady {1} -highlightthickness {0}  \
	-activebackground "$GuiColor(dis_act_bg)"\
	-background "$GuiColor(dis_bg)"\
	-command {destroy .prefs}  -text {DISMISS}  -width {10}

  # pack widget .prefs.perform.bar
  pack append .prefs.perform.bar  .prefs.perform.bar.dismiss {left frame sw expand}  .prefs.perform.bar.perform {left frame se expand}

  # pack widget .prefs.perform
  pack append .prefs.perform  .prefs.perform.bar {bottom frame center fillx}

  # build widget .prefs.clear_message
  frame .prefs.clear_message\
  -background "$GuiColor(frame_bg)" \
  -borderwidth {2}\
  -relief {ridge}

  # build widget .prefs.clear_message.clear_message
	button .prefs.clear_message.clear_message -padx {1} -pady {1} -highlightthickness {0}\
	-activebackground "$GuiColor(btn_act_bg)" \
	-background "$GuiColor(btn_bg)" \
	-relief {flat} \
	-command {global message
CvipMessage ""}  -text {Clear main window message area}

  # pack widget .prefs.clear_message
  pack append .prefs.clear_message  .prefs.clear_message.clear_message {top frame w}
  pack configure .prefs.clear_message.clear_message -padx {20}

  set etc(viewer_original) $etc(viewer)

  # build widget .prefs.viewer
  frame .prefs.viewer\
  -background "$GuiColor(frame_bg)" \
  -borderwidth {2}\
  -relief {ridge}
 
	radiobutton .prefs.viewer.ram -padx {1} -pady {1} -highlightthickness {0}  \
	-activebackground "$GuiColor(btn_act_bg)" \
	-background "$GuiColor(btn_bg)" \
  -relief {flat}  \
  -text {RAMviewer} \
  -command {set etc(viewer) RamViewer;
  .prefs.viewer.frame1.entry configure -fg $GuiColor(rentry_fg)
  .prefs.viewer.frame1.entry configure -bg $GuiColor(rentry_bg)
  pack forget .prefs.viewer.frame1.frame1
  .prefs.viewer.frame1.entry configure -state {disabled}} \
  -value {RamViewer}  \
  -variable {etc(viewer)}

	radiobutton .prefs.viewer.picture -padx {1} -pady {1} -highlightthickness {0}  \
	-activebackground "$GuiColor(btn_act_bg)" \
	-background "$GuiColor(btn_bg)" \
  -relief {flat}  \
  -text {picture} \
  -command {set etc(viewer) picture
  .prefs.viewer.frame1.entry configure -fg $GuiColor(rentry_fg)
  .prefs.viewer.frame1.entry configure -bg $GuiColor(rentry_bg)
  pack forget .prefs.viewer.frame1.frame1
  .prefs.viewer.frame1.entry configure -state {disabled}} \
  -value {picture}  \
  -variable {etc(viewer)}

	radiobutton .prefs.viewer.others -padx {1} -pady {1} -highlightthickness {0}  \
	-activebackground "$GuiColor(btn_act_bg)" \
	-background "$GuiColor(btn_bg)" \
  -relief {flat}  \
  -text {Other:} \
  -command {
  		.prefs.viewer.frame1.entry configure -state {normal};
  		.prefs.viewer.frame1.entry configure -fg $GuiColor(entry_fg)
  		.prefs.viewer.frame1.entry configure -bg $GuiColor(entry_bg)
      pack append .prefs.viewer.frame1 \
			.prefs.viewer.frame1.frame1 {left frame center fillx}
		} \
  -variable {etc(viewer)}

  # build widget .prefs.viewer.frame1
  frame .prefs.viewer.frame1 \
    -highlightthickness {0} \
    -background "$GuiColor(frame_bg)" 

  entry .prefs.viewer.frame1.entry \
  -background "$GuiColor(rentry_bg)" \
  -foreground "$GuiColor(rentry_fg)" \
  -relief {sunken} \
  -state {disabled} \
  -width {17} \
  -textvariable {etc(viewer)}

  # build widget .prefs.viewer.frame1.frame1
  frame .prefs.viewer.frame1.frame1 \
    -borderwidth {0} \
    -highlightthickness {0} \
    -background "$GuiColor(frame_bg)" 

  # build widget .prefs.viewer.frame1.frame1.label6
  entry .prefs.viewer.frame1.frame1.label6 \
    -background "$GuiColor(rentry_bg)" \
    -foreground "$GuiColor(rentry_fg)" \
    -relief {sunken} \
	 -state {disabled} \
	 -width {7} \
    -textvariable {etc(display_format)}

  # build widget .prefs.viewer.frame1.frame1.menubutton7
  menubutton .prefs.viewer.frame1.frame1.menubutton7 \
    -font {-Adobe-Helvetica-Bold-R-Normal--*-120-*} \
    -padx {1} -pady {1} -highlightthickness {0} \
    -relief {flat} \
    -text { use display format:}

  # bindings
  bind .prefs.viewer.frame1.frame1.menubutton7 <Button-1> { MenuPopupPost .prefs.viewer.frame1.frame1.menubutton7.menu0 %X %Y}
  bind .prefs.viewer.frame1.frame1.menubutton7 <ButtonRelease-1> {MenuPopupRelease .prefs.viewer.frame1.frame1.menubutton7.menu0 %W }

  # build widget .prefs.viewer.frame1.frame1.menubutton7.menu0
  menu .prefs.viewer.frame1.frame1.menubutton7.menu0 

  .prefs.viewer.frame1.frame1.menubutton7.menu0 add command \
    -label {Default} \
	 -command {set etc(display_format) Default}
  .prefs.viewer.frame1.frame1.menubutton7.menu0 add command \
    -label {  PPM } \
	 -command {set etc(display_format) PPM}
  .prefs.viewer.frame1.frame1.menubutton7.menu0 add command \
    -label {  GIF } \
	 -command {set etc(display_format) GIF}
  .prefs.viewer.frame1.frame1.menubutton7.menu0 add command \
    -label {  JPG } \
	 -command {set etc(display_format) JPG}
  .prefs.viewer.frame1.frame1.menubutton7.menu0 add command \
    -label {  RAS } \
	 -command {set etc(display_format) RAS}
  .prefs.viewer.frame1.frame1.menubutton7.menu0 add command \
    -label { IRIS } \
	 -command {set etc(display_format) IRIS}
  .prefs.viewer.frame1.frame1.menubutton7.menu0 add command \
    -label {  TIF } \
	 -command {set etc(display_format) TIF}
  .prefs.viewer.frame1.frame1.menubutton7.menu0 add command \
    -label {  BIN } \
	 -command {set etc(display_format) bin}
  .prefs.viewer.frame1.frame1.menubutton7.menu0 add command \
    -label {  CCC } \
	 -command {set etc(display_format) CCC}
  .prefs.viewer.frame1.frame1.menubutton7.menu0 add command \
    -label {  EPS } \
	 -command {set etc(display_format) EPS}
  .prefs.viewer.frame1.frame1.menubutton7.menu0 add command \
    -label {  ITX } \
	 -command {set etc(display_format) ITX}
  .prefs.viewer.frame1.frame1.menubutton7.menu0 add command \
    -label {  VIP } \
	 -command {set etc(display_format) VIP}

  bind .prefs.viewer.frame1.frame1.menubutton7.menu0 <B1-Motion> { MenuPopupMotion .prefs.viewer.frame1.frame1.menubutton7.menu0 %W %X %Y}
  bind .prefs.viewer.frame1.frame1.menubutton7.menu0 <ButtonRelease-1>  { MenuPopupRelease .prefs.viewer.frame1.frame1.menubutton7.menu0 %W }

  # pack widget .prefs.viewer.frame1.frame1
  pack append .prefs.viewer.frame1.frame1  \
    .prefs.viewer.frame1.frame1.label6 {right frame e fillx} \
    .prefs.viewer.frame1.frame1.menubutton7 {right frame w fillx}

  if { $etc(viewer) == "picture" || $etc(viewer) == "RamViewer" } {
    pack append .prefs.viewer.frame1  \
      .prefs.viewer.frame1.entry {left frame center } 
  } else {
    pack append .prefs.viewer.frame1  \
      .prefs.viewer.frame1.entry {left frame center }  \
      .prefs.viewer.frame1.frame1 {left frame center fillx} 
  }

  # pack widget .prefs.viewer
  pack append .prefs.viewer  \
  .prefs.viewer.frame1 {bottom frame w fillx}  \
  .prefs.viewer.ram {left frame center expand }\
  .prefs.viewer.picture {left frame center expand} \
  .prefs.viewer.others {left frame center expand} 

  pack configure .prefs.viewer.frame1 -padx 20

  # build widget .prefs.message
  frame .prefs.message \
    -highlightthickness {0} \
	 -borderwidth {2} \
    -relief {ridge} \
    -background "$GuiColor(frame_bg)" 

  checkbutton .prefs.message.alert \
   -padx {1} -pady {1} -highlightthickness {0}  \
	-activebackground "$GuiColor(btn_act_bg)"\
	-background "$GuiColor(btn_bg)"\
	-variable {CvipAlertControl}  \
	-command {if { $CvipAlertControl == "1" } {set CvipAlert "bell"} else {set CvipAlert ""} } \
	-text {Alert}

  checkbutton .prefs.message.debug \
   -padx {1} -pady {1} -highlightthickness {0}  \
	-activebackground "$GuiColor(btn_act_bg)"\
	-background "$GuiColor(btn_bg)"\
	-variable {GUI_DEBUG}  \
	-text {Debug}

  # pack widget .prefs.message
  pack append .prefs.message \
    .prefs.message.alert {left frame center expand fillx } \
    .prefs.message.debug {right frame center expand fillx }

  # build widget .prefs.frame1
  frame .prefs.frame1 \
    -borderwidth {2} \
    -highlightthickness {0} \
	 -background "$GuiColor(frame_bg)" \
	 -relief {ridge} \
    -height {30} \
    -width {30}

  label .prefs.frame1.label  \
  -background "$GuiColor(label_bg)"  \
  -padx {20} \
  -text {Window manager name:}

  # build widget .prefs.frame1.entry
  entry .prefs.frame1.entry \
    -background "$GuiColor(rentry_bg)" \
    -foreground "$GuiColor(rentry_fg)" \
	 -state {disabled} \
    -relief {sunken} \
	 -width {10} \
    -textvariable {etc(wm_name)}

  # build widget .prefs.frame1.menubutton7
  menubutton .prefs.frame1.menubutton7 \
	 -activebackground "$GuiColor(btn_act_bg)" -background "$GuiColor(btn_bg)"  \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)" \
    -padx {1} -pady {1} -highlightthickness {0} \
    -text "$GuiMisc(pop_up_text)" -width {2}

  # bindings
  bind .prefs.frame1.menubutton7 <Button-1> { MenuPopupPost .prefs.frame1.menubutton7.menu0 %X %Y}
  bind .prefs.frame1.menubutton7 <ButtonRelease-1> {MenuPopupRelease .prefs.frame1.menubutton7.menu0 %W }

  # build widget .prefs.frame1.menubutton7.menu0
  menu .prefs.frame1.menubutton7.menu0  \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" 

  .prefs.frame1.menubutton7.menu0 add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"  \
    -label {OpenWin} \
	 -command {set etc(wm_name) OpenWin; CvipSizeOpenWin}
  .prefs.frame1.menubutton7.menu0 add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"  \
    -label {CDE} \
	 -command {set etc(wm_name) CDE; CvipSizeCDE}
  .prefs.frame1.menubutton7.menu0 add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"  \
    -label {FVWM} \
	 -command {set etc(wm_name) FVWM; CvipSizeFVWM}
  .prefs.frame1.menubutton7.menu0 add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"  \
    -label {IRIX} \
	 -command {set etc(wm_name) IRIX; CvipSizeIRIX}
  .prefs.frame1.menubutton7.menu0 add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"  \
    -label {Win32} \
	 -command {set etc(wm_name) Win32; CvipSizeWin32}

  bind .prefs.frame1.menubutton7.menu0 <B1-Motion> { MenuPopupMotion .prefs.frame1.menubutton7.menu0 %W %X %Y}
  bind .prefs.frame1.menubutton7.menu0 <ButtonRelease-1>  { MenuPopupRelease .prefs.frame1.menubutton7.menu0 %W }

  # pack master .prefs.frame1
  pack configure .prefs.frame1.label \
    -side left
  pack configure .prefs.frame1.entry \
    -side left
  pack configure .prefs.frame1.menubutton7 \
    -side left

  # pack widget .prefs
  pack append .prefs \
  .prefs.toplevel_position {top frame center fillx} \
  .prefs.viewer {top frame center fillx} \
  .prefs.message {top frame center fillx}  \
  .prefs.clear_message {top frame center fillx}  \
  .prefs.frame1 {top frame center fillx} \
  .prefs.perform {bottom frame center fillx} 

  if {"[info procs XFEdit]" != ""} {
    catch "XFMiscBindWidgetTree .prefs"
    after 2 "catch {XFEditSetShowWindows}"
  }
}


# Procedure: rename_image
proc rename_image {} {
##########
# Procedure: rename_image
# Author: Mark Zuke (mzuke@ee.siue.edu) May 9, 1996
# Description: renames an image 
#              
# Arguments: none
# Returns: none
# Sideeffects: none 
#               
##########

global current_image
global new_name
global toplevel_position
global message

  # build widget .rename_image
  if {"[info procs XFEdit]" != ""} {
    catch "XFDestroy .rename_image"
  } {
    catch "destroy .rename_image"
  }
  toplevel .rename_image   -background {GRAY90}
  global GuiColor

  # Window manager configurations
  global tk_version
  wm positionfrom .rename_image user
  wm sizefrom .rename_image user
  wm geometry .rename_image [top_pos 350x200]
  wm maxsize .rename_image 450 300
  wm minsize .rename_image 350 200
  wm title .rename_image {RENAME}


  # build widget .rename_image.entry
  frame .rename_image.entry \
  -background "$GuiColor(frame_bg)" \
  -borderwidth {2}

  # build widget .rename_image.entry.new_path
  frame .rename_image.entry.new_path  -background "$GuiColor(frame_bg)" 

  # build widget .rename_image.entry.new_path.scrollbar7
  scrollbar .rename_image.entry.new_path.scrollbar7  -troughcolor {GRAY40}  -command {.rename_image.entry.new_path.frame3.entry5 xview}  -background {gray80}  -orient {horizontal}  -relief {raised}  -width {11}

  # build widget .rename_image.entry.new_path.frame3
  frame .rename_image.entry.new_path.frame3  -background "$GuiColor(frame_bg)" 

  # build widget .rename_image.entry.new_path.frame3.entry5
  entry .rename_image.entry.new_path.frame3.entry5  \
  -background "$GuiColor(entry_bg)" \
  -relief {sunken} -xscrollcommand {.rename_image.entry.new_path.scrollbar7 set}  -textvariable {new_name}
  # bindings
#  bind .rename_image.entry.new_path.frame3.entry5 <Key-Return> {if {$new_name == ""} { \
CvipMessage "You MUST enter a new name first."}
#if {$new_name != ""} {cvip rename $current_image $new_name
#set to_delete $ImageSelectIdx
#img_q delete $to_delete
#img_q insert end $new_name
#image_post $new_name
#.rename_image.entry.new_path.frame3.entry5 delete 0 end}}

  # build widget .rename_image.entry.new_path.frame3.label4
  label .rename_image.entry.new_path.frame3.label4  -background {GRAY90}  -padx {2}  -relief {raised}  -text {New name:}

  # pack widget .rename_image.entry.new_path.frame3
  pack append .rename_image.entry.new_path.frame3  .rename_image.entry.new_path.frame3.label4 {left frame center}  .rename_image.entry.new_path.frame3.entry5 {top frame center expand fill}

  # pack widget .rename_image.entry.new_path
  pack append .rename_image.entry.new_path  .rename_image.entry.new_path.frame3 {top frame center fillx}  .rename_image.entry.new_path.scrollbar7 {top frame center fillx}

  # pack widget .rename_image.entry
  pack append .rename_image.entry  .rename_image.entry.new_path {top frame center fill}

  # build widget .rename_image.perform
  frame .rename_image.perform  \
  -background "$GuiColor(frame_bg)" \
  -borderwidth {2}

  # build widget .rename_image.perform.perform
	button .rename_image.perform.perform -padx {1} -pady {1} -highlightthickness {0}  \
	-activebackground "$GuiColor(perf_act_bg)" \
	-background "$GuiColor(perf_bg)" \
	-foreground "$GuiColor(perf_fg)" \
	-command {
		set temp $current_image
      if {$new_name == ""} {CvipAlert "You MUST enter a new name first."}
      if {$new_name != ""} {set current_image [cvip rename $current_image $new_name]
      img_q delete $ImageSelectIdx
      img_q insert end $current_image
      image_post $current_image
		img_q selection set end end
      .rename_image.entry.new_path.frame3.entry5 delete 0 end
		CvipMessage "$temp is renamed to $current_image"
		}
   } \
	-text {PERFORM}

  # build widget .rename_image.perform.dismiss
	button .rename_image.perform.dismiss -padx {1} -pady {1} -highlightthickness {0}  \
	-activebackground "$GuiColor(dis_act_bg)"\
	-background "$GuiColor(dis_bg)"\
	-command {destroy .rename_image}  -text {DISMISS}  -width {10}

  # build widget .rename_image.perform.clear
	button .rename_image.perform.clear -padx {1} -pady {1} -highlightthickness {0}   \
	-activebackground "$GuiColor(alert_act_bg)"\
	-background "$GuiColor(alert_bg)"\
	-command {.rename_image.entry.new_path.frame3.entry5 delete 0 end}  -text {CLEAR}  -width {10}

  # pack widget .rename_image.perform
  pack append .rename_image.perform  .rename_image.perform.dismiss {left frame sw expand}  .rename_image.perform.clear {left frame s expand}  .rename_image.perform.perform {left frame se expand}

  # build widget .rename_image.old_name
  frame .rename_image.old_name \
  -background "$GuiColor(frame_bg)" \
  -borderwidth {2}

  # build widget .rename_image.old_name.old_name
  label .rename_image.old_name.old_name  -background {gray90}  -text {Old name: }

  # build widget .rename_image.old_name.name
  label .rename_image.old_name.name  \
  -background "$GuiColor(rlabel_bg)" \
  -foreground "$GuiColor(rlabel_fg)" \
  -text {cameraman.pgm}  -textvariable {current_image}

  # pack widget .rename_image.old_name
  pack append .rename_image.old_name  .rename_image.old_name.old_name {left frame center}  .rename_image.old_name.name {left frame center fillx}

  # pack widget .rename_image
  pack append .rename_image  .rename_image.old_name {top frame center expand fill}  .rename_image.entry {top frame center expand fill}  .rename_image.perform {bottom frame center fillx}

  if {"[info procs XFEdit]" != ""} {
    catch "XFMiscBindWidgetTree .rename_image"
    after 2 "catch {XFEditSetShowWindows}"
  }
}


# Procedure: to_load
proc to_load { args} {
#xf ignore me 4
return [eval ".base.extras.load_image.frame3.entry5 $args"]
}


# Internal procedures


# Procedure: Alias
if {"[info procs Alias]" == ""} {
proc Alias { args} {
# xf ignore me 7
##########
# Procedure: Alias
# Description: establish an alias for a procedure
# Arguments: args - no argument means that a list of all aliases
#                   is returned. Otherwise the first parameter is
#                   the alias name, and the second parameter is
#                   the procedure that is aliased.
# Returns: nothing, the command that is bound to the alias or a
#          list of all aliases - command pairs. 
# Sideeffects: internalAliasList is updated, and the alias
#              proc is inserted
##########
  global internalAliasList

  if {[llength $args] == 0} {
    return $internalAliasList
  } {
    if {[llength $args] == 1} {
      set xfTmpIndex [lsearch $internalAliasList "[lindex $args 0] *"]
      if {$xfTmpIndex != -1} {
        return [lindex [lindex $internalAliasList $xfTmpIndex] 1]
      }
    } {
      if {[llength $args] == 2} {
        eval "proc [lindex $args 0] {args} {#xf ignore me 4
return \[eval \"[lindex $args 1] \$args\"\]}"
        set xfTmpIndex [lsearch $internalAliasList "[lindex $args 0] *"]
        if {$xfTmpIndex != -1} {
          set internalAliasList [lreplace $internalAliasList $xfTmpIndex $xfTmpIndex "[lindex $args 0] [lindex $args 1]"]
        } {
          lappend internalAliasList "[lindex $args 0] [lindex $args 1]"
        }
      } {
        error "Alias: wrong number or args: $args"
      }
    }
  }
}
}


# Procedure: GetSelection
if {"[info procs GetSelection]" == ""} {
proc GetSelection {} {
# xf ignore me 7
##########
# Procedure: GetSelection
# Description: get current ImageSelectIdx
# Arguments: none
# Returns: none
# Sideeffects: none
##########

  # the save way
  set xfSelection ""
  catch "selection get" xfSelection
  if {"$xfSelection" == "selection doesn't exist or form \"STRING\" not defined"} {
    return ""
  } {
    return $xfSelection
  }
}
}


# Procedure: MenuPopupAdd
if {"[info procs MenuPopupAdd]" == ""} {
proc MenuPopupAdd { xfW xfButton xfMenu {xfModifier ""} {xfCanvasTag ""}} {
# xf ignore me 7
# the popup menu handling is from (I already gave up with popup handling :-):
#
# Copyright 1991,1992 by James Noble.
# Everyone is granted permission to copy, modify and redistribute.
# This notice must be preserved on all copies or derivates.
#
##########
# Procedure: MenuPopupAdd
# Description: attach a popup menu to widget
# Arguments: xfW - the widget
#            xfButton - the button we use
#            xfMenu - the menu to attach
#            {xfModifier} - a optional modifier
#            {xfCanvasTag} - a canvas tagOrId
# Returns: none
# Sideeffects: none
##########
  global tk_popupPriv

  set tk_popupPriv($xfMenu,focus) ""
  set tk_popupPriv($xfMenu,grab) ""
  if {"$xfModifier" != ""} {
    set press "$xfModifier-"
    set motion "$xfModifier-"
    set release "Any-"
  } {
    set press ""
    set motion ""
    set release ""
  }

  bind $xfMenu "<${motion}B${xfButton}-Motion>"  "MenuPopupMotion $xfMenu %W %X %Y"
  bind $xfMenu "<${release}ButtonRelease-${xfButton}>"  "MenuPopupRelease $xfMenu %W"
  if {"$xfCanvasTag" == ""} {
    bind $xfW "<${press}ButtonPress-${xfButton}>"  "MenuPopupPost $xfMenu %X %Y"
    bind $xfW "<${release}ButtonRelease-${xfButton}>"  "MenuPopupRelease $xfMenu %W"
  } {
    $xfW bind $xfCanvasTag "<${press}ButtonPress-${xfButton}>"  "MenuPopupPost $xfMenu %X %Y"
    $xfW bind $xfCanvasTag "<${release}ButtonRelease-${xfButton}>"  "MenuPopupRelease $xfMenu %W"
  }
}
}


# Procedure: MenuPopupMotion
if {"[info procs MenuPopupMotion]" == ""} {
proc MenuPopupMotion { xfMenu xfW xfX xfY} {
# xf ignore me 7
##########
# Procedure: MenuPopupMotion
# Description: handle the popup menu motion
# Arguments: xfMenu - the topmost menu
#            xfW - the menu
#            xfX - the root x coordinate
#            xfY - the root x coordinate
# Returns: none
# Sideeffects: none
##########
  global tk_popupPriv

  if {"[info commands $xfW]" != "" && [winfo ismapped $xfW] &&
      "[winfo class $xfW]" == "Menu" &&
      [info exists tk_popupPriv($xfMenu,focus)] &&
      "$tk_popupPriv($xfMenu,focus)" != "" &&
      [info exists tk_popupPriv($xfMenu,grab)] &&
      "$tk_popupPriv($xfMenu,grab)" != ""} {
    set xfPopMinX [winfo rootx $xfW]
    set xfPopMaxX [expr $xfPopMinX+[winfo width $xfW]]
    if {$xfX >= $xfPopMinX && $xfX <= $xfPopMaxX} {
      $xfW activate @[expr $xfY-[winfo rooty $xfW]]
      if {![catch "$xfW entryconfig @[expr $xfY-[winfo rooty $xfW]] -menu" result]} {
        if {"[lindex $result 4]" != ""} {
          foreach binding [bind $xfMenu] {
            bind [lindex $result 4] $binding [bind $xfMenu $binding]
          }
        }
      }
    } {
      $xfW activate none
    }
  }
}
}


# Procedure: MenuPopupPost
if {"[info procs MenuPopupPost]" == ""} {
proc MenuPopupPost { xfMenu xfX xfY} {
# xf ignore me 7
##########
# Procedure: MenuPopupPost
# Description: post the popup menu
# Arguments: xfMenu - the menu
#            xfX - the root x coordinate
#            xfY - the root x coordinate
# Returns: none
# Sideeffects: none
##########
  global tk_popupPriv

  if {"[info commands $xfMenu]" != ""} {
    if {![info exists tk_popupPriv($xfMenu,focus)]} {
      set tk_popupPriv($xfMenu,focus) [focus]
    } {
      if {"$tk_popupPriv($xfMenu,focus)" == ""} {
        set tk_popupPriv($xfMenu,focus) [focus]
      }
    }
    set tk_popupPriv($xfMenu,grab) $xfMenu

    catch "$xfMenu activate none"
    catch "$xfMenu post $xfX $xfY"
    catch "focus $xfMenu"
    catch "grab -global $xfMenu"
  }
}
}


# Procedure: MenuPopupRelease
if {"[info procs MenuPopupRelease]" == ""} {
proc MenuPopupRelease { xfMenu xfW} {
# xf ignore me 7
##########
# Procedure: MenuPopupRelease
# Description: remove the popup menu
# Arguments: xfMenu - the topmost menu widget
#            xfW - the menu widget
# Returns: none
# Sideeffects: none
##########
  global tk_popupPriv
  global tk_version

  if {"[info commands $xfW]" != "" && [winfo ismapped $xfW] &&
      "[winfo class $xfW]" == "Menu" &&
      [info exists tk_popupPriv($xfMenu,focus)] &&
      "$tk_popupPriv($xfMenu,focus)" != "" &&
      [info exists tk_popupPriv($xfMenu,grab)] &&
      "$tk_popupPriv($xfMenu,grab)" != ""} {
    if {$tk_version >= 3.0} {
      catch "grab release $tk_popupPriv($xfMenu,grab)"
    } {
      catch "grab none"
    }
    catch "focus $tk_popupPriv($xfMenu,focus)"
    set tk_popupPriv($xfMenu,focus) ""
    set tk_popupPriv($xfMenu,grab) ""
    if {"[$xfW index active]" != "none"} {
      $xfW invoke active; catch "$xfMenu unpost"
    }
  }
  catch "$xfMenu unpost"
}
}


# Procedure: NoFunction
if {"[info procs NoFunction]" == ""} {
proc NoFunction { args} {
# xf ignore me 7
##########
# Procedure: NoFunction
# Description: do nothing (especially with scales and scrollbars)
# Arguments: args - a number of ignored parameters
# Returns: none
# Sideeffects: none
##########
}
}



# module load procedure
proc XFLocalIncludeModule {{moduleName ""}} {
  global env
  global xfLoadInfo
  global xfLoadPath
  global xfStatus
  global GuiColor

  foreach p [split $xfLoadPath :] {
    if {[file exists "$p/$moduleName"]} {
      if {![file readable "$p/$moduleName"]} {
        puts stderr "Cannot read $p/$moduleName (permission denied)"
        continue
      }
      if {$xfLoadInfo} {
        puts stdout "Loading $p/$moduleName..."
      }
      source "$p/$moduleName"
      return 1
    }
    # first see if we have a load command
    if {[info exists env(XF_VERSION_SHOW)]} {
      set xfCommand $env(XF_VERSION_SHOW)
      regsub -all {\$xfFileName} $xfCommand $p/$moduleName xfCommand
      if {$xfLoadInfo} {
        puts stdout "Loading $p/$moduleName...($xfCommand)"
      }
      if {[catch "exec $xfCommand" contents]} {
        continue
      } {
        eval $contents
        return 1
      }
    }
    # are we able to load versions from wish ?
    if {[catch "afbind $p/$moduleName" aso]} {
      # try to use xf version load command
      global xfVersion
      if {[info exists xfVersion(showDefault)]} {
        set xfCommand $xfVersion(showDefault)
      } {
	# our last hope
        set xfCommand "vcat -q $p/$moduleName"
      }
      regsub -all {\$xfFileName} $xfCommand $p/$moduleName xfCommand
      if {$xfLoadInfo} {
        puts stdout "Loading $p/$moduleName...($xfCommand)"
      }
      if {[catch "exec $xfCommand" contents]} {
        continue
      } {
        eval $contents
        return 1
      }
    } {
      # yes we can load versions directly
      if {[catch "$aso open r" inFile]} {
        puts stderr "Cannot open $p/[$aso attr af_bound] (permission denied)"
        continue
      }
      if {$xfLoadInfo} {
        puts stdout "Loading $p/[$aso attr af_bound]..."
      }
      if {[catch "read \{$inFile\}" contents]} {
        puts stderr "Cannot read $p/[$aso attr af_bound] (permission denied)"
        close $inFile
        continue
      }
      close $inFile
      eval $contents
      return 1
    }
  }
  puts stderr "Cannot load module $moduleName -- check your xf load path"
  puts stderr "Specify a xf load path with the environment variable:"
  puts stderr "  XF_LOAD_PATH (e.g \"export XF_LOAD_PATH=.\")"
  catch "destroy ."
  catch "exit 0"
}

# application parsing procedure
proc XFLocalParseAppDefs {xfAppDefFile} {
  global xfAppDefaults

  # basically from: Michael Moore
  if {[file exists $xfAppDefFile] &&
      [file readable $xfAppDefFile] &&
      "[file type $xfAppDefFile]" == "link"} {
    catch "file type $xfAppDefFile" xfType
    while {"$xfType" == "link"} {
      if {[catch "file readlink $xfAppDefFile" xfAppDefFile]} {
        return
      }
      catch "file type $xfAppDefFile" xfType
    }
  }
  if {!("$xfAppDefFile" != "" &&
        [file exists $xfAppDefFile] &&
        [file readable $xfAppDefFile] &&
        "[file type $xfAppDefFile]" == "file")} {
    return
  }
  if {![catch "open $xfAppDefFile r" xfResult]} {
    set xfAppFileContents [read $xfResult]
    close $xfResult
    foreach line [split $xfAppFileContents "\n"] {
      # backup indicates how far to backup.  It applies to the
      # situation where a resource name ends in . and when it
      # ends in *.  In the second case you want to keep the *
      # in the widget name for pattern matching, but you want
      # to get rid of the . if it is the end of the name. 
      set backup -2  
      set line [string trim $line]
      if {[string index $line 0] == "#" || "$line" == ""} {
        # skip comments and empty lines
        continue
      }
      set list [split $line ":"]
      set resource [string trim [lindex $list 0]]
      set i [string last "." $resource]
      set j [string last "*" $resource]
      if {$j > $i} { 
        set i $j
        set backup -1
      }
      incr i
      set name [string range $resource $i end]
      incr i $backup
      set widname [string range $resource 0 $i]
      set value [string trim [lindex $list 1]]
      if {"$widname" != "" && "$widname" != "*"} {
        # insert the widget and resourcename to the application
        # defaults list.
        if {![info exists xfAppDefaults]} {
          set xfAppDefaults ""
        }
        lappend xfAppDefaults [list $widname [string tolower $name] $value]
      }
    }
  }
}

# application loading procedure
proc XFLocalLoadAppDefs {{xfClasses ""} {xfPriority "startupFile"} {xfAppDefFile ""}} {
  global env

  if {"$xfAppDefFile" == ""} {
    set xfFileList ""
    if {[info exists env(XUSERFILESEARCHPATH)]} {
      append xfFileList [split $env(XUSERFILESEARCHPATH) :]
    }
    if {[info exists env(XAPPLRESDIR)]} {
      append xfFileList [split $env(XAPPLRESDIR) :]
    }
    if {[info exists env(XFILESEARCHPATH)]} {
      append xfFileList [split $env(XFILESEARCHPATH) :]
    }
    append xfFileList " /usr/lib/X11/app-defaults"
    append xfFileList " /usr/X11/lib/X11/app-defaults"

    foreach xfCounter1 $xfClasses {
      foreach xfCounter2 $xfFileList {
        set xfPathName $xfCounter2
        if {[regsub -all "%N" "$xfPathName" "$xfCounter1" xfResult]} {
          set xfPathName $xfResult
        }
        if {[regsub -all "%T" "$xfPathName" "app-defaults" xfResult]} {
          set xfPathName $xfResult
        }
        if {[regsub -all "%S" "$xfPathName" "" xfResult]} {
          set xfPathName $xfResult
        }
        if {[regsub -all "%C" "$xfPathName" "" xfResult]} {
          set xfPathName $xfResult
        }
        if {[file exists $xfPathName] &&
            [file readable $xfPathName] &&
            ("[file type $xfPathName]" == "file" ||
             "[file type $xfPathName]" == "link")} {
          catch "option readfile $xfPathName $xfPriority"
          if {"[info commands XFParseAppDefs]" != ""} {
            XFParseAppDefs $xfPathName
          } {
            if {"[info commands XFLocalParseAppDefs]" != ""} {
              XFLocalParseAppDefs $xfPathName
            }
          }
        } {
          if {[file exists $xfCounter2/$xfCounter1] &&
              [file readable $xfCounter2/$xfCounter1] &&
              ("[file type $xfCounter2/$xfCounter1]" == "file" ||
               "[file type $xfCounter2/$xfCounter1]" == "link")} {
            catch "option readfile $xfCounter2/$xfCounter1 $xfPriority"
            if {"[info commands XFParseAppDefs]" != ""} {
              XFParseAppDefs $xfCounter2/$xfCounter1
            } {
              if {"[info commands XFLocalParseAppDefs]" != ""} {
                XFLocalParseAppDefs $xfCounter2/$xfCounter1
              }
            }
          }
        }
      }
    }
  } {
    # load a specific application defaults file
    if {[file exists $xfAppDefFile] &&
        [file readable $xfAppDefFile] &&
        ("[file type $xfAppDefFile]" == "file" ||
         "[file type $xfAppDefFile]" == "link")} {
      catch "option readfile $xfAppDefFile $xfPriority"
      if {"[info commands XFParseAppDefs]" != ""} {
        XFParseAppDefs $xfAppDefFile
      } {
        if {"[info commands XFLocalParseAppDefs]" != ""} {
          XFLocalParseAppDefs $xfAppDefFile
        }
      }
    }
  }
}

# application setting procedure
proc XFLocalSetAppDefs {{xfWidgetPath "."}} {
  global xfAppDefaults

  if {![info exists xfAppDefaults]} {
    return
  }
  foreach xfCounter $xfAppDefaults {
    if {"$xfCounter" == ""} {
      break
    }
    set widname [lindex $xfCounter 0]
    if {[string match $widname ${xfWidgetPath}] ||
        [string match "${xfWidgetPath}*" $widname]} {
      set name [string tolower [lindex $xfCounter 1]]
      set value [lindex $xfCounter 2]
      # Now lets see how many tcl commands match the name
      # pattern specified.
      set widlist [info command $widname]
      if {"$widlist" != ""} {
        foreach widget $widlist {
          # make sure this command is a widget.
          if {![catch "winfo id $widget"] &&
              [string match "${xfWidgetPath}*" $widget]} {
            catch "$widget configure -$name $value" 
          }
        }
      }
    }
  }
}



# startup source
proc StartupSrc {args} {

}


# end source
proc EndSrc {} {
global message
global CVIPversion
global toplevel_position
global current_image
global env
img_q delete 0 end
global ImageSelectIdx

if { $env(HOME) == "/home/sumbaug" \
		|| $env(HOME) == "/home/yawei" \
		|| $env(HOME) == "/home/wzheng"\
		|| $env(HOME) == "/home/smakam" } {
	catch {set current_image cam.pgm;	\
		cvip load $current_image; img_q insert end $current_image;\
		set current_image worf.ppm;	\
		cvip load $current_image;\
		img_q insert end $current_image; \
		img_q selection set end end; \
		set ImageSelectIdx 2 \
	}
}
if { $env(HOME) == "/home/mzuke" } {
	set current_image house.ppm	
	cvip load $current_image
	img_q insert end $current_image
	set current_image house_seg5.ppm	
	cvip load $current_image
	img_q insert end $current_image
	set current_image dorian4.ras	
	cvip load $current_image
	img_q insert end $current_image
	set current_image dorian1.ras	
	cvip load $current_image
	img_q insert end $current_image
}

set toplevel_position UR
}

if {"[info procs XFShowHelp]" != ""} {
  XFLocalIncludeModule ana_feature.tcl
}
if {"[info procs XFShowHelp]" != ""} {
  XFLocalIncludeModule ana_edge.tcl
}
if {"[info procs XFShowHelp]" != ""} {
  XFLocalIncludeModule analysis.tcl
}
if {"[info procs XFShowHelp]" != ""} {
  XFLocalIncludeModule compression.tcl
}
if {"[info procs XFShowHelp]" != ""} {
  XFLocalIncludeModule enhancement.tcl
}
if {"[info procs XFShowHelp]" != ""} {
  XFLocalIncludeModule restoration.tcl
}
if {"[info procs XFShowHelp]" != ""} {
  XFLocalIncludeModule utilities.tcl
}
if {"[info procs XFShowHelp]" != ""} {
  XFLocalIncludeModule enhancement.tcl
}


# prepare auto loading
global auto_path
global tk_library
global xfLoadPath
foreach xfElement [eval list [split $xfLoadPath :] $auto_path] {
  if {[file exists $xfElement/tclIndex]} {
    lappend auto_path $xfElement
  }
}

# Procedure: AlertBox
proc AlertBox { {alertBoxMessage "Alert message"} {alertBoxCommand ""} {alertBoxGeometry "350x150"} {alertBoxTitle "Alert box"} args} {
# xf ignore me 5
##########
# Procedure: AlertBox
# Description: show alert box
# Arguments: {alertBoxMessage} - the text to display
#            {alertBoxCommand} - the command to call after ok
#            {alertBoxGeometry} - the geometry for the window
#            {alertBoxTitle} - the title for the window
#            {args} - labels of buttons
# Returns: The number of the selected button, ot nothing
# Sideeffects: none
# Notes: there exist also functions called:
#          AlertBoxFile - to open and read a file automatically
#          AlertBoxFd - to read from an already opened filedescriptor
##########
#
# global alertBox(activeBackground) - active background color
# global alertBox(activeForeground) - active foreground color
# global alertBox(after) - destroy alert box after n seconds
# global alertBox(anchor) - anchor for message box
# global alertBox(background) - background color
# global alertBox(font) - message font
# global alertBox(foreground) - foreground color
# global alertBox(justify) - justify for message box
# global alertBox(toplevelName) - the toplevel name

  global alertBox
  global GuiColor

  # show alert box
  if {[llength $args] > 0} {
    eval AlertBoxInternal "\{$alertBoxMessage\}" "\{$alertBoxCommand\}" "\{$alertBoxGeometry\}" "\{$alertBoxTitle\}" $args
  } {
    AlertBoxInternal $alertBoxMessage $alertBoxCommand $alertBoxGeometry $alertBoxTitle
  }

  if {[llength $args] > 0} {
    # wait for the box to be destroyed
    update idletask
    grab $alertBox(toplevelName)
    tkwait window $alertBox(toplevelName)

    return $alertBox(button)
  }
}

# Procedure: AlertBoxFd
proc AlertBoxFd { {alertBoxInFile ""} {alertBoxCommand ""} {alertBoxGeometry "350x150"} {alertBoxTitle "Alert box"} args} {
# xf ignore me 5
##########
# Procedure: AlertBoxFd
# Description: show alert box containing a filedescriptor
# Arguments: {alertBoxInFile} - a filedescriptor to read. The descriptor
#                               is closed after reading
#            {alertBoxCommand} - the command to call after ok
#            {alertBoxGeometry} - the geometry for the window
#            {alertBoxTitle} - the title for the window
#            {args} - labels of buttons
# Returns: The number of the selected button, ot nothing
# Sideeffects: none
# Notes: there exist also functions called:
#          AlertBox - to display a passed string
#          AlertBoxFile - to open and read a file automatically
##########
#
# global alertBox(activeBackground) - active background color
# global alertBox(activeForeground) - active foreground color
# global alertBox(after) - destroy alert box after n seconds
# global alertBox(anchor) - anchor for message box
# global alertBox(background) - background color
# global alertBox(font) - message font
# global alertBox(foreground) - foreground color
# global alertBox(justify) - justify for message box
# global alertBox(toplevelName) - the toplevel name

  global alertBox
  global GuiColor

  # check file existance
  if {"$alertBoxInFile" == ""} {
    puts stderr "No filedescriptor specified"
    return
  }

  set alertBoxMessage [read $alertBoxInFile]
  close $alertBoxInFile

  # show alert box
  if {[llength $args] > 0} {
    eval AlertBoxInternal "\{$alertBoxMessage\}" "\{$alertBoxCommand\}" "\{$alertBoxGeometry\}" "\{$alertBoxTitle\}" $args
  } {
    AlertBoxInternal $alertBoxMessage $alertBoxCommand $alertBoxGeometry $alertBoxTitle
  }

  if {[llength $args] > 0} {
    # wait for the box to be destroyed
    update idletask
    grab $alertBox(toplevelName)
    tkwait window $alertBox(toplevelName)

    return $alertBox(button)
  }
}


# Procedure: AlertBoxFile
proc AlertBoxFile { {alertBoxFile ""} {alertBoxCommand ""} {alertBoxGeometry "350x150"} {alertBoxTitle "Alert box"} args} {
# xf ignore me 5
##########
# Procedure: AlertBoxFile
# Description: show alert box containing a file
# Arguments: {alertBoxFile} - filename to read
#            {alertBoxCommand} - the command to call after ok
#            {alertBoxGeometry} - the geometry for the window
#            {alertBoxTitle} - the title for the window
#            {args} - labels of buttons
# Returns: The number of the selected button, ot nothing
# Sideeffects: none
# Notes: there exist also functions called:
#          AlertBox - to display a passed string
#          AlertBoxFd - to read from an already opened filedescriptor
##########
#
# global alertBox(activeBackground) - active background color
# global alertBox(activeForeground) - active foreground color
# global alertBox(after) - destroy alert box after n seconds
# global alertBox(anchor) - anchor for message box
# global alertBox(background) - background color
# global alertBox(font) - message font
# global alertBox(foreground) - foreground color
# global alertBox(justify) - justify for message box
# global alertBox(toplevelName) - the toplevel name

  global alertBox
  global GuiColor

  # check file existance
  if {"$alertBoxFile" == ""} {
    puts stderr "No filename specified"
    return
  }

  if {[catch "open $alertBoxFile r" alertBoxInFile]} {
    puts stderr "$alertBoxInFile"
    return
  }

  set alertBoxMessage [read $alertBoxInFile]
  close $alertBoxInFile

  # show alert box
  if {[llength $args] > 0} {
    eval AlertBoxInternal "\{$alertBoxMessage\}" "\{$alertBoxCommand\}" "\{$alertBoxGeometry\}" "\{$alertBoxTitle\}" $args
  } {
    AlertBoxInternal $alertBoxMessage $alertBoxCommand $alertBoxGeometry $alertBoxTitle
  }

  if {[llength $args] > 0} {
    # wait for the box to be destroyed
    update idletask
    grab $alertBox(toplevelName)
    tkwait window $alertBox(toplevelName)

    return $alertBox(button)
  }
}


# Procedure: AlertBoxInternal
proc AlertBoxInternal { alertBoxMessage alertBoxCommand alertBoxGeometry alertBoxTitle args} {
# xf ignore me 6
  global alertBox
  global GuiColor

  set tmpButtonOpt ""
  set tmpFrameOpt ""
  set tmpMessageOpt ""
  if {"$alertBox(activeBackground)" != ""} {
    append tmpButtonOpt "-activebackground \"$alertBox(activeBackground)\" "
  }
  if {"$alertBox(activeForeground)" != ""} {
    append tmpButtonOpt "-activeforeground \"$alertBox(activeForeground)\" "
  }
  if {"$alertBox(background)" != ""} {
    append tmpButtonOpt "-background \"$alertBox(background)\" "
    append tmpFrameOpt "-background \"$alertBox(background)\" "
    append tmpMessageOpt "-background \"$alertBox(background)\" "
  }
  if {"$alertBox(font)" != ""} {
    append tmpButtonOpt "-font \"$alertBox(font)\" "
    append tmpMessageOpt "-font \"$alertBox(font)\" "
  }
  if {"$alertBox(foreground)" != ""} {
    append tmpButtonOpt "-foreground \"$alertBox(foreground)\" "
    append tmpMessageOpt "-foreground \"$alertBox(foreground)\" "
  }

  # start build of toplevel
  if {"[info commands XFDestroy]" != ""} {
    catch {XFDestroy $alertBox(toplevelName)}
  } {
    catch {destroy $alertBox(toplevelName)}
  }
  toplevel $alertBox(toplevelName)  -borderwidth 0
  catch "$alertBox(toplevelName) config $tmpFrameOpt"
  if {[catch "wm geometry $alertBox(toplevelName) $alertBoxGeometry"]} {
    wm geometry $alertBox(toplevelName) 350x150
  }
  wm title $alertBox(toplevelName) $alertBoxTitle
  wm maxsize $alertBox(toplevelName) 1000 1000
  wm minsize $alertBox(toplevelName) 100 100
  # end build of toplevel

  message $alertBox(toplevelName).message1  -anchor "$alertBox(anchor)"  -justify "$alertBox(justify)"  -relief raised  -text "$alertBoxMessage"
  catch "$alertBox(toplevelName).message1 config $tmpMessageOpt"

  set xfTmpWidth  [string range $alertBoxGeometry 0 [expr [string first x $alertBoxGeometry]-1]]
  if {"$xfTmpWidth" != ""} {
    # set message size
    catch "$alertBox(toplevelName).message1 configure  -width [expr $xfTmpWidth-10]"
  } {
    $alertBox(toplevelName).message1 configure  -aspect 1500
  }

  frame $alertBox(toplevelName).frame1  -borderwidth 0  -relief raised
  catch "$alertBox(toplevelName).frame1 config $tmpFrameOpt"

  set alertBoxCounter 0
  set buttonNum [llength $args]
  if {$buttonNum > 0} {
    while {$alertBoxCounter < $buttonNum} {
	button $alertBox(toplevelName).frame1.button$alertBoxCounter -padx {1} -pady {1} -highlightthickness {0}   -text "[lindex $args $alertBoxCounter]"  -command "
          global alertBox
          set alertBox(button) $alertBoxCounter
          if {\"\[info commands XFDestroy\]\" != \"\"} {
            catch {XFDestroy $alertBox(toplevelName)}
          } {
            catch {destroy $alertBox(toplevelName)}
          }"
      catch "$alertBox(toplevelName).frame1.button$alertBoxCounter config $tmpButtonOpt"

      pack append $alertBox(toplevelName).frame1  $alertBox(toplevelName).frame1.button$alertBoxCounter {left fillx expand}

      incr alertBoxCounter
    }
  } {
	button $alertBox(toplevelName).frame1.button0 -padx {1} -pady {1} -highlightthickness {0}   -text "OK"  -command "
        global alertBox
        set alertBox(button) 0
        if {\"\[info commands XFDestroy\]\" != \"\"} {
          catch {XFDestroy $alertBox(toplevelName)}
        } {
          catch {destroy $alertBox(toplevelName)}
        }
        $alertBoxCommand"
    catch "$alertBox(toplevelName).frame1.button0 config $tmpButtonOpt"

    pack append $alertBox(toplevelName).frame1  $alertBox(toplevelName).frame1.button0 {left fillx expand}
  }

  # packing
  pack append $alertBox(toplevelName)  $alertBox(toplevelName).frame1 {bottom fill}  $alertBox(toplevelName).message1 {top fill expand}

  if {$alertBox(after) != 0} {
    after [expr $alertBox(after)*1000]  "catch \"$alertBox(toplevelName).frame1.button0 invoke\""
  }
}


# Procedure: FSBox
proc FSBox { {fsBoxMessage "Select file:"} {fsBoxFileName ""} {fsBoxActionOk ""} {fsBoxActionCancel ""}} {
# xf ignore me 5
##########
# Procedure: FSBox
# Description: show file selector box
# Arguments: fsBoxMessage - the text to display
#            fsBoxFileName - a file name that should be selected
#            fsBoxActionOk - the action that should be performed on ok
#            fsBoxActionCancel - the action that should be performed on cancel
# Returns: the filename that was selected, or nothing
# Sideeffects: none
##########
# 
# global fsBox(activeBackground) - active background color
# global fsBox(activeForeground) - active foreground color
# global fsBox(background) - background color
# global fsBox(font) - text font
# global fsBox(foreground) - foreground color
# global fsBox(extensions) - scan directory for extensions
# global fsBox(scrollActiveForeground) - scrollbar active background color
# global fsBox(scrollBackground) - scrollbar background color
# global fsBox(scrollForeground) - scrollbar foreground color
# global fsBox(scrollSide) - side where scrollbar is located

  global fsBox
  global GuiColor

  set tmpButtonOpt ""
  set tmpFrameOpt ""
  set tmpMessageOpt ""
  set tmpScaleOpt ""
  set tmpScrollOpt ""
  if {"$fsBox(activeBackground)" != ""} {
    append tmpButtonOpt "-activebackground \"$fsBox(activeBackground)\" "
  }
  if {"$fsBox(activeForeground)" != ""} {
    append tmpButtonOpt "-activeforeground \"$fsBox(activeForeground)\" "
  }
  if {"$fsBox(background)" != ""} {
    append tmpButtonOpt "-background \"$fsBox(background)\" "
    append tmpFrameOpt "-background \"$fsBox(background)\" "
    append tmpMessageOpt "-background \"$fsBox(background)\" "
  }
  if {"$fsBox(font)" != ""} {
    append tmpButtonOpt "-font \"$fsBox(font)\" "
    append tmpMessageOpt "-font \"$fsBox(font)\" "
  }
  if {"$fsBox(foreground)" != ""} {
    append tmpButtonOpt "-foreground \"$fsBox(foreground)\" "
    append tmpMessageOpt "-foreground \"$fsBox(foreground)\" "
  }
  if {"$fsBox(scrollActiveForeground)" != ""} {
    append tmpScrollOpt "-activeforeground \"$fsBox(scrollActiveForeground)\" "
  }
  if {"$fsBox(scrollBackground)" != ""} {
    append tmpScrollOpt "-background \"$fsBox(scrollBackground)\" "
  }
  if {"$fsBox(scrollForeground)" != ""} {
    append tmpScrollOpt "-foreground \"$fsBox(scrollForeground)\" "
  }

  if {[file exists [file tail $fsBoxFileName]] &&
      [IsAFile [file tail $fsBoxFileName]]} {
    set fsBox(name) [file tail $fsBoxFileName]
  } {
    set fsBox(name) ""
  }
  if {[file exists $fsBoxFileName] && [IsADir $fsBoxFileName]} {
    set fsBox(path) $fsBoxFileName
  } {
    if {"[file dirname $fsBoxFileName]" != "."} {
      set fsBox(path) [file dirname $fsBoxFileName]
    }
  }
  if {$fsBox(showPixmap)} {
    set fsBox(path) [string trimleft $fsBox(path) @]
  }
  if {"$fsBox(path)" != "" && [file exists $fsBox(path)] &&
      [IsADir $fsBox(path)]} {
    set fsBox(internalPath) $fsBox(path)
  } {
    if {"$fsBox(internalPath)" == "" ||
        ![file exists $fsBox(internalPath)]} {
      set fsBox(internalPath) [pwd]
    }
  }
  # build widget structure

  # start build of toplevel
  if {"[info commands XFDestroy]" != ""} {
    catch {XFDestroy .fsBox}
  } {
    catch {destroy .fsBox}
  }
  toplevel .fsBox  -borderwidth 0
  catch ".fsBox config $tmpFrameOpt"
  wm geometry .fsBox 350x300 
  wm title .fsBox {File select box}
  wm maxsize .fsBox 1000 1000
  wm minsize .fsBox 100 100
  # end build of toplevel

  label .fsBox.message1  -anchor c  -relief raised  -text "$fsBoxMessage"
  catch ".fsBox.message1 config $tmpMessageOpt"

  frame .fsBox.frame1  -borderwidth 0  -relief raised
  catch ".fsBox.frame1 config $tmpFrameOpt"

	button .fsBox.frame1.ok -padx {1} -pady {1} -highlightthickness {0}   -text "OK"  -command "
      global fsBox
      set fsBox(name) \[.fsBox.file.file get\]
      if {$fsBox(showPixmap)} {
        set fsBox(path) @\[.fsBox.path.path get\]
      } {
        set fsBox(path) \[.fsBox.path.path get\]
      }
      set fsBox(internalPath) \[.fsBox.path.path get\]
      $fsBoxActionOk
      if {\"\[info commands XFDestroy\]\" != \"\"} {
        catch {XFDestroy .fsBox}
      } {
        catch {destroy .fsBox}
      }"
  catch ".fsBox.frame1.ok config $tmpButtonOpt"

	button .fsBox.frame1.rescan -padx {1} -pady {1} -highlightthickness {0}   -text "Rescan"  -command {
      global fsBox
      FSBoxFSShow [.fsBox.path.path get]  [.fsBox.pattern.pattern get] $fsBox(all)}
  catch ".fsBox.frame1.rescan config $tmpButtonOpt"

	button .fsBox.frame1.cancel -padx {1} -pady {1} -highlightthickness {0}   -text "Cancel"  -command "
      global fsBox
      set fsBox(name) {}
      set fsBox(path) {}
      $fsBoxActionCancel
      if {\"\[info commands XFDestroy\]\" != \"\"} {
        catch {XFDestroy .fsBox}
      } {
        catch {destroy .fsBox}
      }"
  catch ".fsBox.frame1.cancel config $tmpButtonOpt"

  if {$fsBox(showPixmap)} {
    frame .fsBox.frame2  -borderwidth 0  -relief raised
    catch ".fsBox.frame2 config $tmpFrameOpt"

    scrollbar .fsBox.frame2.scrollbar3  -command {.fsBox.frame2.canvas2 xview}  -orient {horizontal}  -relief {raised}
    catch ".fsBox.frame2.scrollbar3 config $tmpScrollOpt"

    scrollbar .fsBox.frame2.scrollbar1  -command {.fsBox.frame2.canvas2 yview}  -relief {raised}
    catch ".fsBox.frame2.scrollbar1 config $tmpScrollOpt"

    canvas .fsBox.frame2.canvas2  -confine {true}  -relief {raised}  -scrollregion {0c 0c 20c 20c}  -width {100}  -xscrollcommand {.fsBox.frame2.scrollbar3 set}  -yscrollcommand {.fsBox.frame2.scrollbar1 set}
    catch ".fsBox.frame2.canvas2 config $tmpFrameOpt"

    .fsBox.frame2.canvas2 addtag currentBitmap withtag [.fsBox.frame2.canvas2 create bitmap 5 5 -anchor nw]
  }

  frame .fsBox.path  -borderwidth 0  -relief raised
  catch ".fsBox.path config $tmpFrameOpt"

  frame .fsBox.path.paths  -borderwidth 2  -relief raised
  catch ".fsBox.path.paths config $tmpFrameOpt"

	menubutton .fsBox.path.paths.paths -padx {1} -pady {1} -highlightthickness {0}   -borderwidth 0  -menu ".fsBox.path.paths.paths.menu"  -relief flat  -text "Pathname:"
  catch ".fsBox.path.paths.paths config $tmpButtonOpt"

  menu .fsBox.path.paths.paths.menu
  catch ".fsBox.path.paths.paths.menu config $tmpButtonOpt"

  .fsBox.path.paths.paths.menu add command  -label "[string trimright $fsBox(internalPath) {/@}]"  -command "
       global fsBox
       FSBoxFSShow \[.fsBox.path.path get\]  \[.fsBox.pattern.pattern get\] \$fsBox(all)
       .fsBox.path.path delete 0 end
       .fsBox.path.path insert 0 [string trimright $fsBox(internalPath) {/@}]"

  entry .fsBox.path.path  -relief raised
  catch ".fsBox.path.path config $tmpMessageOpt"

  if {![IsADir $fsBox(internalPath)]} {
    set $fsBox(internalPath) [pwd]
  }
  .fsBox.path.path insert 0 $fsBox(internalPath)

  frame .fsBox.pattern  -borderwidth 0  -relief raised
  catch ".fsBox.pattern config $tmpFrameOpt"

  frame .fsBox.pattern.patterns  -borderwidth 2  -relief raised
  catch ".fsBox.pattern.patterns config $tmpFrameOpt"

	menubutton .fsBox.pattern.patterns.patterns -padx {1} -pady {1} -highlightthickness {0}   -borderwidth 0  -menu ".fsBox.pattern.patterns.patterns.menu"  -relief flat  -text "Selection pattern:"
  catch ".fsBox.pattern.patterns.patterns config $tmpButtonOpt"

  menu .fsBox.pattern.patterns.patterns.menu
  catch ".fsBox.pattern.patterns.patterns.menu config $tmpButtonOpt"

  .fsBox.pattern.patterns.patterns.menu add checkbutton  -label "Scan extensions"  -variable fsBox(extensions)  -command {
      global fsBox
      FSBoxFSShow [.fsBox.path.path get]  [.fsBox.pattern.pattern get] $fsBox(all)}

  entry .fsBox.pattern.pattern  -relief raised
  catch ".fsBox.pattern.pattern config $tmpMessageOpt"

  .fsBox.pattern.pattern insert 0 $fsBox(pattern)
  
  frame .fsBox.files  -borderwidth 0  -relief raised
  catch ".fsBox.files config $tmpFrameOpt"

  scrollbar .fsBox.files.vscroll  -relief raised  -command ".fsBox.files.files yview"
  catch ".fsBox.files.vscroll config $tmpScrollOpt"

  scrollbar .fsBox.files.hscroll  -orient horiz  -relief raised  -command ".fsBox.files.files xview"
  catch ".fsBox.files.hscroll config $tmpScrollOpt"

  listbox .fsBox.files.files  -exportselection false  -relief raised  -xscrollcommand ".fsBox.files.hscroll set"  -yscrollcommand ".fsBox.files.vscroll set"
  catch ".fsBox.files.files config $tmpMessageOpt"

  frame .fsBox.file  -borderwidth 0  -relief raised
  catch ".fsBox.file config $tmpFrameOpt"

  label .fsBox.file.labelfile  -relief raised  -text "Filename:"
  catch ".fsBox.file.labelfile config $tmpMessageOpt"

  entry .fsBox.file.file  -relief raised
  catch ".fsBox.file.file config $tmpMessageOpt"

  .fsBox.file.file delete 0 end
  .fsBox.file.file insert 0 $fsBox(name)
  
	checkbutton .fsBox.pattern.all -padx {1} -pady {1} -highlightthickness {0}   -offvalue 0  -onvalue 1  -text "Show all files"  -variable fsBox(all)  -command {
      global fsBox
      FSBoxFSShow [.fsBox.path.path get]  [.fsBox.pattern.pattern get] $fsBox(all)}
  catch ".fsBox.pattern.all config $tmpButtonOpt"

  FSBoxFSShow $fsBox(internalPath) $fsBox(pattern) $fsBox(all)

  # bindings
  bind .fsBox.files.files <Double-Button-1> "
    FSBoxFSFileSelectDouble %W $fsBox(showPixmap) \{$fsBoxActionOk\} %y"
  bind .fsBox.files.files <ButtonPress-1> "
    FSBoxFSFileSelect %W $fsBox(showPixmap) %y"
  bind .fsBox.files.files <Button1-Motion> "
    FSBoxFSFileSelect %W $fsBox(showPixmap) %y"
  bind .fsBox.files.files <Shift-Button1-Motion> "
    FSBoxFSFileSelect %W $fsBox(showPixmap) %y"
  bind .fsBox.files.files <Shift-ButtonPress-1> "
    FSBoxFSFileSelect %W $fsBox(showPixmap) %y"

  bind .fsBox.path.path <Tab> {
    FSBoxFSNameComplete path}
  bind .fsBox.path.path <Return> {
    global tk_version
    global fsBox
    FSBoxFSShow [.fsBox.path.path get]  [.fsBox.pattern.pattern get] $fsBox(all)
    FSBoxFSInsertPath
    if {$tk_version >= 3.0} {
      .fsBox.file.file icursor end
    } {
      .fsBox.file.file cursor end
    }
    focus .fsBox.file.file}
  catch "bind .fsBox.path.path <Up> {}"
  bind .fsBox.path.path <Down> {
    global tk_version
    if {$tk_version >= 3.0} {
      .fsBox.file.file icursor end
    } {
      .fsBox.file.file cursor end
    }
    focus .fsBox.file.file}

  bind .fsBox.file.file <Tab> {
    FSBoxFSNameComplete file}
  bind .fsBox.file.file <Return> "
    global fsBox
    set fsBox(name) \[.fsBox.file.file get\]
    if {$fsBox(showPixmap)} {
      set fsBox(path) @\[.fsBox.path.path get\]
    } {
      set fsBox(path) \[.fsBox.path.path get\]
    }
    set fsBox(internalPath) \[.fsBox.path.path get\]
    $fsBoxActionOk
    if {\"\[info commands XFDestroy\]\" != \"\"} {
      catch {XFDestroy .fsBox}
    } {
      catch {destroy .fsBox}
    }"
  bind .fsBox.file.file <Up> {
    global tk_version
    if {$tk_version >= 3.0} {
      .fsBox.path.path icursor end
    } {
      .fsBox.path.path cursor end
    }
    focus .fsBox.path.path}
  bind .fsBox.file.file <Down> {
    global tk_version
    if {$tk_version >= 3.0} {
      .fsBox.pattern.pattern icursor end
    } {
      .fsBox.pattern.pattern cursor end
    }
    focus .fsBox.pattern.pattern}

  bind .fsBox.pattern.pattern <Return> {
    global fsBox
    FSBoxFSShow [.fsBox.path.path get]  [.fsBox.pattern.pattern get] $fsBox(all)}
  bind .fsBox.pattern.pattern <Up> {
    global tk_version
    if {$tk_version >= 3.0} {
      .fsBox.file.file icursor end
    } {
      .fsBox.file.file cursor end
    }
    focus .fsBox.file.file}
  catch "bind .fsBox.pattern.pattern <Down> {}"

  # packing
  pack append .fsBox.files  .fsBox.files.vscroll "$fsBox(scrollSide) filly"  .fsBox.files.hscroll {bottom fillx}  .fsBox.files.files {left fill expand}
  pack append .fsBox.file  .fsBox.file.labelfile {left}  .fsBox.file.file {left fill expand}
  pack append .fsBox.frame1  .fsBox.frame1.ok {left fill expand}  .fsBox.frame1.rescan {left fill expand}  .fsBox.frame1.cancel {left fill expand}
  pack append .fsBox.path.paths  .fsBox.path.paths.paths {left}
  pack append .fsBox.pattern.patterns  .fsBox.pattern.patterns.patterns {left}
  pack append .fsBox.path  .fsBox.path.paths {left}  .fsBox.path.path {left fill expand}
  pack append .fsBox.pattern  .fsBox.pattern.patterns {left}  .fsBox.pattern.all {right fill}  .fsBox.pattern.pattern {left fill expand}
  if {$fsBox(showPixmap)} {
    pack append .fsBox.frame2  .fsBox.frame2.scrollbar1 {left filly}  .fsBox.frame2.canvas2 {top expand fill}  .fsBox.frame2.scrollbar3 {top fillx} 

    pack append .fsBox  .fsBox.message1 {top fill}  .fsBox.frame1 {bottom fill}  .fsBox.pattern {bottom fill}  .fsBox.file {bottom fill}  .fsBox.path {bottom fill}  .fsBox.frame2 {right fill}  .fsBox.files {left fill expand}
  } {
    pack append .fsBox  .fsBox.message1 {top fill}  .fsBox.frame1 {bottom fill}  .fsBox.pattern {bottom fill}  .fsBox.file {bottom fill}  .fsBox.path {bottom fill}  .fsBox.files {left fill expand}
  }

  if {"$fsBoxActionOk" == "" && "$fsBoxActionCancel" == ""} {
    # wait for the box to be destroyed
    update idletask
    grab .fsBox
    tkwait window .fsBox

    if {"[string trim $fsBox(path)]" != "" ||
        "[string trim $fsBox(name)]" != ""} {
      if {"[string trimleft [string trim $fsBox(name)] /]" == ""} {
        return [string trimright [string trim $fsBox(path)] /]
      } {
        return [string trimright [string trim $fsBox(path)] /]/[string trimleft [string trim $fsBox(name)] /]
      }
    }
  }
}


# Procedure: FSBoxBindSelectOne
proc FSBoxBindSelectOne { fsBoxW fsBoxY} {
# xf ignore me 6

  set fsBoxNearest [$fsBoxW nearest $fsBoxY]
  if {$fsBoxNearest >= 0} {
    $fsBoxW select anchor $fsBoxNearest
    $fsBoxW select set anchor $fsBoxNearest
  }
}


# Procedure: FSBoxFSFileSelect
proc FSBoxFSFileSelect { fsBoxW fsBoxShowPixmap fsBoxY} {
# xf ignore me 6
  global fsBox
  global GuiColor

  FSBoxBindSelectOne $fsBoxW $fsBoxY
  set fsBoxNearest [$fsBoxW nearest $fsBoxY]
  if {$fsBoxNearest >= 0} {
    set fsBoxTmpEntry [$fsBoxW get $fsBoxNearest]
    if {"[string index $fsBoxTmpEntry  [expr [string length $fsBoxTmpEntry]-1]]" == "/" ||
        "[string index $fsBoxTmpEntry  [expr [string length $fsBoxTmpEntry]-1]]" == "@"} {
      set fsBoxFileName [string range $fsBoxTmpEntry 0  [expr [string length $fsBoxTmpEntry]-2]]
      if {![IsADir [string trimright $fsBox(internalPath)/$fsBoxFileName @]] &&
          ![IsASymlink [string trimright $fsBox(internalPath)/$fsBoxFileName @]]} {
        set fsBoxFileName $fsBoxTmpEntry
      }
    } {
      if {"[string index $fsBoxTmpEntry  [expr [string length $fsBoxTmpEntry]-1]]" == "*"} {
        set fsBoxFileName [string range $fsBoxTmpEntry 0  [expr [string length $fsBoxTmpEntry]-2]]
        if {![file executable $fsBox(internalPath)/$fsBoxFileName]} {
          set fsBoxFileName $fsBoxTmpEntry
        }
      } {
        set fsBoxFileName $fsBoxTmpEntry
      }
    }
    if {![IsADir [string trimright $fsBox(internalPath)/$fsBoxFileName @]]} {
      set fsBox(name) $fsBoxFileName
      .fsBox.file.file delete 0 end
      .fsBox.file.file insert 0 $fsBox(name)
      if {$fsBoxShowPixmap} {
        catch ".fsBox.frame2.canvas2 itemconfigure currentBitmap -bitmap \"@$fsBox(internalPath)/$fsBox(name)\""
      }
    }
  }
}


# Procedure: FSBoxFSFileSelectDouble
proc FSBoxFSFileSelectDouble { fsBoxW fsBoxShowPixmap fsBoxAction fsBoxY} {
# xf ignore me 6
  global fsBox
  global GuiColor

  FSBoxBindSelectOne $fsBoxW $fsBoxY
  set fsBoxNearest [$fsBoxW nearest $fsBoxY]
  if {$fsBoxNearest >= 0} {
    set fsBoxTmpEntry [$fsBoxW get $fsBoxNearest]
    if {"$fsBoxTmpEntry" == "../"} {
      set fsBoxTmpEntry [string trimright [string trim $fsBox(internalPath)] "@/"]
      if {"$fsBoxTmpEntry" == ""} {
        return
      }
      FSBoxFSShow [file dirname $fsBoxTmpEntry]  [.fsBox.pattern.pattern get] $fsBox(all)
      .fsBox.path.path delete 0 end
      .fsBox.path.path insert 0 $fsBox(internalPath)
    } {
      if {"[string index $fsBoxTmpEntry  [expr [string length $fsBoxTmpEntry]-1]]" == "/" ||
          "[string index $fsBoxTmpEntry  [expr [string length $fsBoxTmpEntry]-1]]" == "@"} {
        set fsBoxFileName [string range $fsBoxTmpEntry 0  [expr [string length $fsBoxTmpEntry]-2]]
        if {![IsADir [string trimright $fsBox(internalPath)/$fsBoxFileName @]] &&
            ![IsASymlink [string trimright $fsBox(internalPath)/$fsBoxFileName @]]} {
          set fsBoxFileName $fsBoxTmpEntry
        }
      } {
        if {"[string index $fsBoxTmpEntry  [expr [string length $fsBoxTmpEntry]-1]]" == "*"} {
          set fsBoxFileName [string range $fsBoxTmpEntry 0  [expr [string length $fsBoxTmpEntry]-2]]
          if {![file executable $fsBox(internalPath)/$fsBoxFileName]} {
            set fsBoxFileName $fsBoxTmpEntry
          }
        } {
          set fsBoxFileName $fsBoxTmpEntry
        }
      }
      if {[IsADir [string trimright $fsBox(internalPath)/$fsBoxFileName @]]} {
        set fsBox(internalPath) "[string trimright $fsBox(internalPath) {/@}]/$fsBoxFileName"
        FSBoxFSShow $fsBox(internalPath)  [.fsBox.pattern.pattern get] $fsBox(all)
        .fsBox.path.path delete 0 end
        .fsBox.path.path insert 0 $fsBox(internalPath)
      } {
        set fsBox(name) $fsBoxFileName
        if {$fsBoxShowPixmap} {
          set fsBox(path) @$fsBox(internalPath)
        } {
          set fsBox(path) $fsBox(internalPath)
        }
        if {"$fsBoxAction" != ""} {
          eval "global fsBox; $fsBoxAction"
        }
        if {"[info commands XFDestroy]" != ""} {
          catch {XFDestroy .fsBox}
        } {
          catch {destroy .fsBox}
        }
      }
    }
  }
}


# Procedure: FSBoxFSInsertPath
proc FSBoxFSInsertPath {} {
# xf ignore me 6
  global fsBox
  global GuiColor

  set fsBoxLast [.fsBox.path.paths.paths.menu index last]
  set fsBoxNewEntry [string trimright [.fsBox.path.path get] "/@"]
  for {set fsBoxCounter 0} {$fsBoxCounter <= $fsBoxLast} {incr fsBoxCounter 1} {
    if {"$fsBoxNewEntry" ==  "[lindex [.fsBox.path.paths.paths.menu entryconfigure  $fsBoxCounter -label] 4]"} {
      return
    }
  }
  if {$fsBoxLast < 9} {
    .fsBox.path.paths.paths.menu add command  -label "$fsBoxNewEntry"  -command "
        global fsBox
        FSBoxFSShow $fsBoxNewEntry  \[.fsBox.pattern.pattern get\] \$fsBox(all)
        .fsBox.path.path delete 0 end
        .fsBox.path.path insert 0 $fsBoxNewEntry"
  } {
    for {set fsBoxCounter 0} {$fsBoxCounter < $fsBoxLast} {incr fsBoxCounter 1} {
      .fsBox.path.paths.paths.menu entryconfigure  $fsBoxCounter -label  [lindex [.fsBox.path.paths.paths.menu entryconfigure  [expr $fsBoxCounter+1] -label] 4]
      .fsBox.path.paths.paths.menu entryconfigure $fsBoxCounter  -command "
          global fsBox
          FSBoxFSShow [lindex [.fsBox.path.paths.paths.menu entryconfigure  [expr $fsBoxCounter+1] -label] 4]  \[.fsBox.pattern.pattern get\] \$fsBox(all)
          .fsBox.path.path delete 0 end
          .fsBox.path.path insert 0 [lindex  [.fsBox.path.paths.paths.menu entryconfigure  [expr $fsBoxCounter+1] -label] 4]"
    }
    .fsBox.path.paths.paths.menu entryconfigure $fsBoxLast  -label "$fsBoxNewEntry"
    .fsBox.path.paths.paths.menu entryconfigure $fsBoxCounter  -command "
        global fsBox
        FSBoxFSShow \[.fsBox.path.path get\]  \[.fsBox.pattern.pattern get\] \$fsBox(all)
        .fsBox.path.path delete 0 end
        .fsBox.path.path insert 0 $fsBoxNewEntry"
  }
}


# Procedure: FSBoxFSNameComplete
proc FSBoxFSNameComplete { fsBoxType} {
# xf ignore me 6
  global tk_version
  global fsBox
  global GuiColor

  set fsBoxNewFile ""
  if {"$fsBoxType" == "path"} {
    set fsBoxDirName [file dirname [.fsBox.path.path get]]
    set fsBoxFileName [file tail [.fsBox.path.path get]]
  } {
    set fsBoxDirName [file dirname [.fsBox.path.path get]/]
    set fsBoxFileName [file tail [.fsBox.file.file get]]
  }

  set fsBoxNewFile ""
  if {[IsADir [string trimright $fsBoxDirName @]]} {
    catch "glob -nocomplain $fsBoxDirName/${fsBoxFileName}*" fsBoxResult
    foreach fsBoxCounter $fsBoxResult {
      if {"$fsBoxNewFile" == ""} {
        set fsBoxNewFile [file tail $fsBoxCounter]
      } {
        if {"[string index [file tail $fsBoxCounter] 0]" !=
            "[string index $fsBoxNewFile 0]"} {
          set fsBoxNewFile ""
          break
        }
        set fsBoxCounter1 0
        set fsBoxTmpFile1 $fsBoxNewFile
        set fsBoxTmpFile2 [file tail $fsBoxCounter]
        set fsBoxLength1 [string length $fsBoxTmpFile1]
        set fsBoxLength2 [string length $fsBoxTmpFile2]
        set fsBoxNewFile ""
        if {$fsBoxLength1 > $fsBoxLength2} {
          set fsBoxLength1 $fsBoxLength2
        }
        while {$fsBoxCounter1 < $fsBoxLength1} {
          if {"[string index $fsBoxTmpFile1 $fsBoxCounter1]" ==  "[string index $fsBoxTmpFile2 $fsBoxCounter1]"} {
            append fsBoxNewFile [string index $fsBoxTmpFile1 $fsBoxCounter1]
          } {
            break
          }
          incr fsBoxCounter1 1
        }
      }
    }
  }
  if {"$fsBoxNewFile" != ""} {
    if {[IsADir [string trimright $fsBoxDirName/$fsBoxNewFile @]] ||
        ![IsAFile [string trimright $fsBoxDirName/$fsBoxNewFile @]]} {
      if {[IsADir [string trimright $fsBoxDirName/$fsBoxNewFile @]]} {
        if {"$fsBoxDirName" == "/"} {
          .fsBox.path.path delete 0 end
          .fsBox.path.path insert 0 "/[string trimright [string trim $fsBoxNewFile /] @]/"
        } {
          .fsBox.path.path delete 0 end
          .fsBox.path.path insert 0 "[string trimright $fsBoxDirName /]/[string trimright [string trim $fsBoxNewFile /] @]/"
        }
        FSBoxFSShow [.fsBox.path.path get]  [.fsBox.pattern.pattern get] $fsBox(all)
        FSBoxFSInsertPath
      } {
        .fsBox.path.path delete 0 end
        .fsBox.path.path insert 0 "[string trimright $fsBoxDirName /]/[string trimright [string trim $fsBoxNewFile /] @]"
      }
    } {
      .fsBox.path.path delete 0 end
      .fsBox.path.path insert 0 "[string trimright $fsBoxDirName {@/}]/"
      .fsBox.file.file delete 0 end
      .fsBox.file.file insert 0 $fsBoxNewFile
      if {$tk_version >= 3.0} {
        .fsBox.file.file icursor end
      } {
        .fsBox.file.file cursor end
      }
      focus .fsBox.file.file
    }
  }
}


# Procedure: FSBoxFSShow
proc FSBoxFSShow { fsBoxPath fsBoxPattern fsBoxAll} {
# xf ignore me 6
  global fsBox
  global GuiColor

  set tmpButtonOpt ""
  if {"$fsBox(activeBackground)" != ""} {
    append tmpButtonOpt "-activebackground \"$fsBox(activeBackground)\" "
  }
  if {"$fsBox(activeForeground)" != ""} {
    append tmpButtonOpt "-activeforeground \"$fsBox(activeForeground)\" "
  }
  if {"$fsBox(background)" != ""} {
    append tmpButtonOpt "-background \"$fsBox(background)\" "
  }
  if {"$fsBox(font)" != ""} {
    append tmpButtonOpt "-font \"$fsBox(font)\" "
  }
  if {"$fsBox(foreground)" != ""} {
    append tmpButtonOpt "-foreground \"$fsBox(foreground)\" "
  }

  set fsBox(pattern) $fsBoxPattern
  if {[file exists $fsBoxPath] && [file readable $fsBoxPath] &&
      [IsADir $fsBoxPath]} {
    set fsBox(internalPath) $fsBoxPath
  } {
    if {[file exists $fsBoxPath] && [file readable $fsBoxPath] &&
        [IsAFile $fsBoxPath]} {
      set fsBox(internalPath) [file dirname $fsBoxPath]
      .fsBox.file.file delete 0 end
      .fsBox.file.file insert 0 [file tail $fsBoxPath]
      set fsBoxPath $fsBox(internalPath)
    } {
      while {"$fsBoxPath" != "" && "$fsBoxPath" != "/" &&
             ![file isdirectory $fsBoxPath]} {
        set fsBox(internalPath) [file dirname $fsBoxPath]
         set fsBoxPath $fsBox(internalPath)
      }
    }
  }
  if {"$fsBoxPath" == ""} {
    set fsBoxPath "/"
    set fsBox(internalPath) "/"
  }
  .fsBox.path.path delete 0 end
  .fsBox.path.path insert 0 $fsBox(internalPath)

  if {[.fsBox.files.files size] > 0} {
    .fsBox.files.files delete 0 end
  }
  if {$fsBoxAll} {
    if {[catch "exec ls -F -a $fsBoxPath" fsBoxResult]} {
      puts stderr "$fsBoxResult"
    }
  } {
    if {[catch "exec ls -F $fsBoxPath" fsBoxResult]} {
      puts stderr "$fsBoxResult"
    }
  }
  set fsBoxElementList [lsort $fsBoxResult]

  foreach fsBoxCounter [winfo children .fsBox.pattern.patterns.patterns] {
    if {[string length [info commands XFDestroy]] > 0} {
      catch {XFDestroy $fsBoxCounter}
    } {
      catch {destroy $fsBoxCounter}
    }
  }
  menu .fsBox.pattern.patterns.patterns.menu
  catch ".fsBox.pattern.patterns.patterns.menu config $tmpButtonOpt"

  if {$fsBox(extensions)} {
    .fsBox.pattern.patterns.patterns.menu add command  -label "*"  -command {
        global fsBox
        set fsBox(pattern) "*"
        .fsBox.pattern.pattern delete 0 end
        .fsBox.pattern.pattern insert 0 $fsBox(pattern)
        FSBoxFSShow [.fsBox.path.path get] $fsBox(pattern)  $fsBox(all)}
  }

  if {"$fsBoxPath" != "/"} {
    .fsBox.files.files insert end "../"
  }
  foreach fsBoxCounter $fsBoxElementList {
    if {[string match $fsBoxPattern $fsBoxCounter] ||
        [IsADir [string trimright $fsBoxPath/$fsBoxCounter "/@"]]} {
      if {"$fsBoxCounter" != "../" &&
          "$fsBoxCounter" != "./"} {
        .fsBox.files.files insert end $fsBoxCounter
      }
    }

    if {$fsBox(extensions)} {
      catch "file rootname $fsBoxCounter" fsBoxRootName
      catch "file extension $fsBoxCounter" fsBoxExtension
      set fsBoxExtension [string trimright $fsBoxExtension "/*@"]
      if {"$fsBoxExtension" != "" && "$fsBoxRootName" != ""} {
        set fsBoxInsert 1
        set fsBoxLast [.fsBox.pattern.patterns.patterns.menu index last]
        for {set fsBoxCounter1 0} {$fsBoxCounter1 <= $fsBoxLast} {incr fsBoxCounter1 1} {
          if {"*$fsBoxExtension" ==  "[lindex [.fsBox.pattern.patterns.patterns.menu entryconfigure  $fsBoxCounter1 -label] 4]"} {
            set fsBoxInsert 0
          }
        }
	if {$fsBoxInsert} {
          .fsBox.pattern.patterns.patterns.menu add command  -label "*$fsBoxExtension"  -command "
              global fsBox
              set fsBox(pattern) \"*$fsBoxExtension\"
              .fsBox.pattern.pattern delete 0 end
              .fsBox.pattern.pattern insert 0 \$fsBox(pattern)
              FSBoxFSShow \[.fsBox.path.path get\] \$fsBox(pattern)  \$fsBox(all)"
        }
      }
    }
  }
  if {$fsBox(extensions)} {
    .fsBox.pattern.patterns.patterns.menu add separator
  }
  if {$fsBox(extensions) || 
      "[.fsBox.pattern.patterns.patterns.menu index last]" == "none"} {
    .fsBox.pattern.patterns.patterns.menu add checkbutton  -label "Scan extensions"  -variable "fsBox(extensions)"  -command {
        global fsBox
        FSBoxFSShow [.fsBox.path.path get]  [.fsBox.pattern.pattern get] $fsBox(all)}
  }
}


# Procedure: IsADir
proc IsADir { pathName} {
# xf ignore me 5
##########
# Procedure: IsADir
# Description: check if name is a directory (including symbolic links)
# Arguments: pathName - the path to check
# Returns: 1 if its a directory, otherwise 0
# Sideeffects: none
##########
  global GuiColor

  if {[file isdirectory $pathName]} {
    return 1
  } {
    catch "file type $pathName" fileType
    if {"$fileType" == "link"} {
      if {[catch "file readlink $pathName" linkName]} {
        return 0
      }
      catch "file type $linkName" fileType
      while {"$fileType" == "link"} {
        if {[catch "file readlink $linkName" linkName]} {
          return 0
        }
        catch "file type $linkName" fileType
      }
      return [file isdirectory $linkName]
    }
  }
  return 0
}


# Procedure: IsAFile
proc IsAFile { fileName} {
# xf ignore me 5
##########
# Procedure: IsAFile
# Description: check if filename is a file (including symbolic links)
# Arguments: fileName - the filename to check
# Returns: 1 if its a file, otherwise 0
# Sideeffects: none
##########

  if {[file isfile $fileName]} {
    return 1
  } {
    catch "file type $fileName" fileType
    if {"$fileType" == "link"} {
      if {[catch "file readlink $fileName" linkName]} {
        return 0
      }
      catch "file type $linkName" fileType
      while {"$fileType" == "link"} {
        if {[catch "file readlink $linkName" linkName]} {
          return 0
        }
        catch "file type $linkName" fileType
      }
      return [file isfile $linkName]
    }
  }
  return 0
}


# Procedure: IsASymlink
proc IsASymlink { fileName} {
# xf ignore me 5
##########
# Procedure: IsASymlink
# Description: check if filename is a symbolic link
# Arguments: fileName - the path/filename to check
# Returns: none
# Sideeffects: none
##########

  catch "file type $fileName" fileType
  if {"$fileType" == "link"} {
    return 1
  }
  return 0
}


# Procedure: TextBox
proc TextBox { {textBoxMessage "Text message"} {textBoxCommand ""} {textBoxGeometry "350x150"} {textBoxTitle "Text box"} args} {
# xf ignore me 5
##########
# Procedure: TextBox
# Description: show text box
# Arguments: {textBoxMessage} - the text to display
#            {textBoxCommand} - the command to call after ok
#            {textBoxGeometry} - the geometry for the window
#            {textBoxTitle} - the title for the window
#            {args} - labels of buttons
# Returns: The number of the selected button, or nothing
# Sideeffects: none
# Notes: there exist also functions called:
#          TextBoxFile - to open and read a file automatically
#          TextBoxFd - to read from an already opened filedescriptor
##########
#
# global textBox(activeBackground) - active background color
# global textBox(activeForeground) - active foreground color
# global textBox(background) - background color
# global textBox(font) - text font
# global textBox(foreground) - foreground color
# global textBox(scrollActiveForeground) - scrollbar active background color
# global textBox(scrollBackground) - scrollbar background color
# global textBox(scrollForeground) - scrollbar foreground color
# global textBox(scrollSide) - side where scrollbar is located

  global textBox

  # show text box
  if {[llength $args] > 0} {
    eval TextBoxInternal "\{$textBoxMessage\}" "\{$textBoxCommand\}" "\{$textBoxGeometry\}" "\{$textBoxTitle\}" $args
  } {
    TextBoxInternal $textBoxMessage $textBoxCommand $textBoxGeometry $textBoxTitle
  }

  if {[llength $args] > 0} {
    # wait for the box to be destroyed
    update idletask
    grab $textBox(toplevelName)
    tkwait window $textBox(toplevelName)

    return $textBox(button)
  }
}


# Procedure: TextBoxFd
proc TextBoxFd { {textBoxInFile ""} {textBoxCommand ""} {textBoxGeometry "350x150"} {textBoxTitle "Text box"} args} {
# xf ignore me 5
##########
# Procedure: TextBoxFd
# Description: show text box containing a filedescriptor
# Arguments: {textBoxInFile} - a filedescriptor to read. The descriptor
#                              is closed after reading
#            {textBoxCommand} - the command to call after ok
#            {textBoxGeometry} - the geometry for the window
#            {textBoxTitle} - the title for the window
#            {args} - labels of buttons
# Returns: The number of the selected button, ot nothing
# Sideeffects: none
# Notes: there exist also functions called:
#          TextBox - to display a passed string
#          TextBoxFile - to open and read a file automatically
##########
#
# global textBox(activeBackground) - active background color
# global textBox(activeForeground) - active foreground color
# global textBox(background) - background color
# global textBox(font) - text font
# global textBox(foreground) - foreground color
# global textBox(scrollActiveForeground) - scrollbar active background color
# global textBox(scrollBackground) - scrollbar background color
# global textBox(scrollForeground) - scrollbar foreground color
# global textBox(scrollSide) - side where scrollbar is located

  global textBox

  # check file existance
  if {"$textBoxInFile" == ""} {
    puts stderr "No filedescriptor specified"
    return
  }

  set textBoxMessage [read $textBoxInFile]
  close $textBoxInFile

  # show text box
  if {[llength $args] > 0} {
    eval TextBoxInternal "\{$textBoxMessage\}" "\{$textBoxCommand\}" "\{$textBoxGeometry\}" "\{$textBoxTitle\}" $args
  } {
    TextBoxInternal $textBoxMessage $textBoxCommand $textBoxGeometry $textBoxTitle
  }

  if {[llength $args] > 0} {
    # wait for the box to be destroyed
    update idletask
    grab $textBox(toplevelName)
    tkwait window $textBox(toplevelName)

    return $textBox(button)
  }
}


# Procedure: TextBoxFile
proc TextBoxFile { {textBoxFile ""} {textBoxCommand ""} {textBoxGeometry "350x150"} {textBoxTitle "Text box"} args} {
# xf ignore me 5
##########
# Procedure: TextBoxFile
# Description: show text box containing a file
# Arguments: {textBoxFile} - filename to read
#            {textBoxCommand} - the command to call after ok
#            {textBoxGeometry} - the geometry for the window
#            {textBoxTitle} - the title for the window
#            {args} - labels of buttons
# Returns: The number of the selected button, ot nothing
# Sideeffects: none
# Notes: there exist also functions called:
#          TextBox - to display a passed string
#          TextBoxFd - to read from an already opened filedescriptor
##########
#
# global textBox(activeBackground) - active background color
# global textBox(activeForeground) - active foreground color
# global textBox(background) - background color
# global textBox(font) - text font
# global textBox(foreground) - foreground color
# global textBox(scrollActiveForeground) - scrollbar active background color
# global textBox(scrollBackground) - scrollbar background color
# global textBox(scrollForeground) - scrollbar foreground color
# global textBox(scrollSide) - side where scrollbar is located

  global textBox
  global GuiColor

  # check file existance
  if {"$textBoxFile" == ""} {
    puts stderr "No filename specified"
    return
  }

  if {[catch "open $textBoxFile r" textBoxInFile]} {
    puts stderr "$textBoxInFile"
    return
  }

  set textBoxMessage [read $textBoxInFile]
  close $textBoxInFile

  # show text box
  if {[llength $args] > 0} {
    eval TextBoxInternal "\{$textBoxMessage\}" "\{$textBoxCommand\}" "\{$textBoxGeometry\}" "\{$textBoxTitle\}" $args
  } {
    TextBoxInternal $textBoxMessage $textBoxCommand $textBoxGeometry $textBoxTitle
  }

  if {[llength $args] > 0} {
    # wait for the box to be destroyed
    update idletask
    grab $textBox(toplevelName)
    tkwait window $textBox(toplevelName)

    return $textBox(button)
  }
}


# Procedure: TextBoxInternal
proc TextBoxInternal { textBoxMessage textBoxCommand textBoxGeometry textBoxTitle args} {
# xf ignore me 6
  global textBox
  global GuiColor

  set tmpButtonOpt ""
  set tmpFrameOpt ""
  set tmpMessageOpt ""
  set tmpScrollOpt ""
  if {"$textBox(activeBackground)" != ""} {
    append tmpButtonOpt "-activebackground \"$textBox(activeBackground)\" "
  }
  if {"$textBox(activeForeground)" != ""} {
    append tmpButtonOpt "-activeforeground \"$textBox(activeForeground)\" "
  }
  if {"$textBox(background)" != ""} {
    append tmpButtonOpt "-background \"$textBox(background)\" "
    append tmpFrameOpt "-background \"$textBox(background)\" "
    append tmpMessageOpt "-background \"$textBox(background)\" "
  }
  if {"$textBox(font)" != ""} {
    append tmpButtonOpt "-font \"$textBox(font)\" "
    append tmpMessageOpt "-font \"$textBox(font)\" "
  }
  if {"$textBox(foreground)" != ""} {
    append tmpButtonOpt "-foreground \"$textBox(foreground)\" "
    append tmpMessageOpt "-foreground \"$textBox(foreground)\" "
  }
  if {"$textBox(scrollActiveForeground)" != ""} {
    append tmpScrollOpt "-activeforeground \"$textBox(scrollActiveForeground)\" "
  }
  if {"$textBox(scrollBackground)" != ""} {
    append tmpScrollOpt "-background \"$textBox(scrollBackground)\" "
  }
  if {"$textBox(scrollForeground)" != ""} {
    append tmpScrollOpt "-foreground \"$textBox(scrollForeground)\" "
  }

  # start build of toplevel
  if {"[info commands XFDestroy]" != ""} {
    catch {XFDestroy $textBox(toplevelName)}
  } {
    catch {destroy $textBox(toplevelName)}
  }
  toplevel $textBox(toplevelName)  -borderwidth 0
  catch "$textBox(toplevelName) config $tmpFrameOpt"
  if {[catch "wm geometry $textBox(toplevelName) $textBoxGeometry"]} {
    wm geometry $textBox(toplevelName) 350x150
  }
  wm title $textBox(toplevelName) $textBoxTitle
  wm maxsize $textBox(toplevelName) 1000 1000
  wm minsize $textBox(toplevelName) 100 100
  # end build of toplevel

  frame $textBox(toplevelName).frame0  -borderwidth 0  -relief raised
  catch "$textBox(toplevelName).frame0 config $tmpFrameOpt"

#  text $textBox(toplevelName).frame0.text1  -relief raised  -wrap none  -borderwidth 2  -yscrollcommand "$textBox(toplevelName).frame0.vscroll set"
  text $textBox(toplevelName).frame0.text1  -relief raised  -wrap word -borderwidth 2  -yscrollcommand "$textBox(toplevelName).frame0.vscroll set"
  catch "$textBox(toplevelName).frame0.text1 config $tmpMessageOpt"

  scrollbar $textBox(toplevelName).frame0.vscroll  -relief raised  -command "$textBox(toplevelName).frame0.text1 yview"
  catch "$textBox(toplevelName).frame0.vscroll config $tmpScrollOpt"

  frame $textBox(toplevelName).frame1  -borderwidth 0  -relief raised
  catch "$textBox(toplevelName).frame1 config $tmpFrameOpt"

  set textBoxCounter 0
  set buttonNum [llength $args]

  if {$buttonNum > 0} {
    while {$textBoxCounter < $buttonNum} {
	button $textBox(toplevelName).frame1.button$textBoxCounter -padx {1} -pady {1} -highlightthickness {0}   -text "[lindex $args $textBoxCounter]"  -command "
          global textBox
          set textBox(button) $textBoxCounter
          set textBox(contents) \[$textBox(toplevelName).frame0.text1 get 1.0 end\]
          if {\"\[info commands XFDestroy\]\" != \"\"} {
            catch {XFDestroy $textBox(toplevelName)}
          } {
            catch {destroy $textBox(toplevelName)}
          }"
      catch "$textBox(toplevelName).frame1.button$textBoxCounter config $tmpButtonOpt"

      pack append $textBox(toplevelName).frame1  $textBox(toplevelName).frame1.button$textBoxCounter {left fillx expand}

      incr textBoxCounter
    }
  } {
	button $textBox(toplevelName).frame1.button0 -padx {1} -pady {1} -highlightthickness {0}   -text "OK"  -command "
        global textBox
        set textBox(button) 0
        set textBox(contents) \[$textBox(toplevelName).frame0.text1 get 1.0 end\]
        if {\"\[info commands XFDestroy\]\" != \"\"} {
          catch {XFDestroy $textBox(toplevelName)}
        } {
          catch {destroy $textBox(toplevelName)}
        }
        $textBoxCommand"
    catch "$textBox(toplevelName).frame1.button0 config $tmpButtonOpt"

    pack append $textBox(toplevelName).frame1  $textBox(toplevelName).frame1.button0 {left fillx expand}
  }

  $textBox(toplevelName).frame0.text1 insert end "$textBoxMessage"

  $textBox(toplevelName).frame0.text1 config  -state $textBox(state)

  # packing
  pack append $textBox(toplevelName).frame0  $textBox(toplevelName).frame0.vscroll "$textBox(scrollSide) filly"  $textBox(toplevelName).frame0.text1 {left fill expand}
  pack append $textBox(toplevelName)  $textBox(toplevelName).frame1 {bottom fill}  $textBox(toplevelName).frame0 {top fill expand}
}




# Procedure: YesNoBox
proc YesNoBox { {yesNoBoxMessage "Yes/no message"} {yesNoBoxGeometry "350x150"}} {
# xf ignore me 5
##########
# Procedure: YesNoBox
# Description: show yesno box
# Arguments: {yesNoBoxMessage} - the text to display
#            {yesNoBoxGeometry} - the geometry for the window
# Returns: none
# Sideeffects: none
##########
#
# global yesNoBox(activeBackground) - active background color
# global yesNoBox(activeForeground) - active foreground color
# global yesNoBox(anchor) - anchor for message box
# global yesNoBox(background) - background color
# global yesNoBox(font) - message font
# global yesNoBox(foreground) - foreground color
# global yesNoBox(justify) - justify for message box
# global yesNoBox(afterNo) - destroy yes-no box after n seconds.
#                            The no button is activated
# global yesNoBox(afterYes) - destroy yes-no box after n seconds.
#                             The yes button is activated

  global yesNoBox
  global GuiColor

  set tmpButtonOpt ""
  set tmpFrameOpt ""
  set tmpMessageOpt ""
  if {"$yesNoBox(activeBackground)" != ""} {
    append tmpButtonOpt "-activebackground \"$yesNoBox(activeBackground)\" "
  }
  if {"$yesNoBox(activeForeground)" != ""} {
    append tmpButtonOpt "-activeforeground \"$yesNoBox(activeForeground)\" "
  }
  if {"$yesNoBox(background)" != ""} {
    append tmpButtonOpt "-background \"$yesNoBox(background)\" "
    append tmpFrameOpt "-background \"$yesNoBox(background)\" "
    append tmpMessageOpt "-background \"$yesNoBox(background)\" "
  }
  if {"$yesNoBox(font)" != ""} {
    append tmpButtonOpt "-font \"$yesNoBox(font)\" "
    append tmpMessageOpt "-font \"$yesNoBox(font)\" "
  }
  if {"$yesNoBox(foreground)" != ""} {
    append tmpButtonOpt "-foreground \"$yesNoBox(foreground)\" "
    append tmpMessageOpt "-foreground \"$yesNoBox(foreground)\" "
  }

  # start build of toplevel
  if {"[info commands XFDestroy]" != ""} {
    catch {XFDestroy .yesNoBox}
  } {
    catch {destroy .yesNoBox}
  }
  toplevel .yesNoBox  -borderwidth 0
  catch ".yesNoBox config $tmpFrameOpt"
  if {[catch "wm geometry .yesNoBox $yesNoBoxGeometry"]} {
    wm geometry .yesNoBox 350x150
  }
  wm title .yesNoBox {Alert box}
  wm maxsize .yesNoBox 1000 1000
  wm minsize .yesNoBox 100 100
  # end build of toplevel

  message .yesNoBox.message1  -anchor "$yesNoBox(anchor)"  -justify "$yesNoBox(justify)"  -relief raised  -text "$yesNoBoxMessage"
  catch ".yesNoBox.message1 config $tmpMessageOpt"

  set xfTmpWidth  [string range $yesNoBoxGeometry 0 [expr [string first x $yesNoBoxGeometry]-1]]
  if {"$xfTmpWidth" != ""} {
    # set message size
    catch ".yesNoBox.message1 configure  -width [expr $xfTmpWidth-10]"
  } {
    .yesNoBox.message1 configure  -aspect 1500
  }

  frame .yesNoBox.frame1  -borderwidth 0  -relief raised
  catch ".yesNoBox.frame1 config $tmpFrameOpt"

	button .yesNoBox.frame1.button0 -padx {1} -pady {1} -highlightthickness {0}   -text "Yes"  -command "
      global yesNoBox
      set yesNoBox(button) 1
      if {\"\[info commands XFDestroy\]\" != \"\"} {
        catch {XFDestroy .yesNoBox}
      } {
        catch {destroy .yesNoBox}
      }"
  catch ".yesNoBox.frame1.button0 config $tmpButtonOpt"

	button .yesNoBox.frame1.button1 -padx {1} -pady {1} -highlightthickness {0}   -text "No"  -command "
      global yesNoBox
      set yesNoBox(button) 0
      if {\"\[info commands XFDestroy\]\" != \"\"} {
        catch {XFDestroy .yesNoBox}
      } {
        catch {destroy .yesNoBox}
      }"
  catch ".yesNoBox.frame1.button1 config $tmpButtonOpt"

  pack append .yesNoBox.frame1  .yesNoBox.frame1.button0 {left fillx expand}  .yesNoBox.frame1.button1 {left fillx expand}

  # packing
  pack append .yesNoBox  .yesNoBox.frame1 {bottom fill}  .yesNoBox.message1 {top fill expand}

  if {$yesNoBox(afterYes) != 0} {
    after [expr $yesNoBox(afterYes)*1000]  "catch \".yesNoBox.frame1.button0 invoke\""
  }
  if {$yesNoBox(afterNo) != 0} {
    after [expr $yesNoBox(afterNo)*1000]  "catch \".yesNoBox.frame1.button1 invoke\""
  }

  # wait for the box to be destroyed
  update idletask
  grab .yesNoBox
  tkwait window .yesNoBox

  return $yesNoBox(button)
}

catch "unset auto_index"

catch "unset auto_oldpath"

catch "unset auto_execs"

# initialize GUI
CvipInitGui

# initialize global variables
CvipInitGlobals

# startup source
StartupSrc

# display/remove toplevel windows.
ShowWindow.

# load default bindings.
if {[info exists env(XF_BIND_FILE)] &&
    "[info procs XFShowHelp]" == ""} {
  source $env(XF_BIND_FILE)
}

# parse and apply application defaults.
XFLocalLoadAppDefs GUI
XFLocalSetAppDefs

# end source
EndSrc


