# **********************************************************************
# Computer Vision/Image Processing Tool Project - Dr. Scott Umbaugh SIUE
# **********************************************************************
#
#             File Name: CVIPmisc.tcl
#           Description: Utility procedures used in CVIPtools GUI
#         Related Files: 
#   Initial Coding Date: Tue Jun  3 17:05:43 CDT 1997
#           Portability: TCL script file
#            References:
#             Author(s): Yansheng Wei
#                        Southern Illinois University @ Edwardsville
#
## Copyright (C) 1996 SIUE - by Scott Umbaugh and Yansheng Wei.
##
## Permission to use, copy, modify, and distribute this software and its
## documentation for any purpose and without fee is hereby granted, provided
## that the above copyright notice appear in all copies and that both that
## copyright notice and this permission notice appear in supporting
## documentation.  This software is provided "as is" without express or
## implied warranty.
##
# RCS (Revision Control System) Information
# ...Added automatically by RCS 
#
## $Log: CVIPmisc.tcl,v $
## Revision 1.5  1997/08/06 00:31:13  yawei
## put the window position feedback to the GUI_DEBUG control
##
## Revision 1.4  1997/08/06 00:10:03  yawei
## added get_pos and top_pos
##
## Revision 1.3  1997/06/23 21:54:16  yawei
## adjusted teh alert box
##
## Revision 1.2  1997/06/21 15:35:03  yawei
## check in for freezing
##
#
#****************************************************************************

####################################################
# CvipYesNo: CVIP standard YesNo confirmation box
# Parameters -
# msg:    message to show in the alert window
# test  :  test or not? (yes/no), default to yes
# silent:  ring bell? (yes/no), default to no
# Return    -
#      yes/no
####################################################

proc CvipYesNo { msg {test "yes"} {silent "no"} {pos ""} {size ""}} {
	global CvipAlert
	global yesNoBox

	if { $size == "" } {
		set size $yesNoBox(size)
	}
	if { $pos == "" } {
		set pos $yesNoBox(position)
	}
	if { $test == "no" } {
		return "yes"
	} else {
		if { $silent != "yes" } { eval "$CvipAlert" }
		if { [ YesNoBox $msg $size$pos] == "0" } {
			return "no"
		} else {
			return "yes"
		}
	}
}

####################################################
# CvipMessage: CVIP standard message handling routine
# Parameters -
# msg:    message to show in the message area
# silent:  ring bell? (yes/no), default to yes
# Return:  none
####################################################

proc CvipMessage { msg {silent "yes"} } {
	global message
	global CvipAlert

	if { $silent != "yes" } {
		eval $CvipAlert
	} 
	set message ""
	set message $msg
}

####################################################
# get_size_limit: To get the max/min size of a window
# Parameters -
# min_max:  min size or max size ?
# Return:  the computed window size
####################################################

proc get_size_limit { size min_max } {
   global topwin;
   set temp [string first "x" $size]
   incr temp -1
   set width [string range $size 0 $temp]
   incr temp 2
   set height [string range $size $temp end]
 
   if { $min_max == "min" } {
      set width [expr round($width * (1 - $topwin(width_down) )) ]
      set height [expr round($height * (1 - $topwin(height_down) )) ]
   } else {
      set width [expr round($width * (1 + $topwin(width_up) )) ]
      set height [expr round($height * (1 + $topwin(height_up) )) ]
   }
   return [list ${width} ${height}]
}

####################################################
# endEntry: To get the max/min size of a window
# take care of the Return in entry widgets (remove \x13 in older
# versions of tcl/tk, and move the cursor to the end of the entry
# box). 
# Parameters -
#    var      : the entry box widget path
#    flag == 1: clear selection, default to 0; 
#    next     : next widget for traversal 
#    prev     : previous widget for traversal 
####################################################
proc endEntry {var {flag "0"} {next ""} {prev ""} } {
  if { $next != "" } {
	  bind $var <Key-Return> "focus $next ; $next icursor end"
	  bind $var <Tab> "focus $next; $next icursor end"
  }
  if { $prev != "" } {
	  bind $var <Shift-Tab> "focus $prev; $prev icursor end"
	  bind $var <Control-Return> "focus $prev; $prev icursor end"
  }
  if {$flag == 1} {
    bind $var <KeyRelease> { selection clear .  }
  }

}

####################################################
# get_coord: To get the max/min size of a window
# Parameters -
# msg:    message to show in the message area
# silent:  ring bell? (yes/no), default to yes
# Return:  none
####################################################
proc get_coord {} {

global current_image
global row_coord
global col_coord
global d_row
global d_col
global cname
global cresult
global null2

if [catch {set coordinate_info [selection get]
    scan $coordinate_info {%s %s %d %d %d %d} cname null2 row_coord col_coord d_row d_col} cresult] { 
   return 0
} else {
	set temp [string last "]" $cname]
	if { $temp >= "0" } {incr temp}
	set temp1 [string range $cname $temp end]
	set cname $temp1
	return 1
}
}

####################################################
# check_coord: check the validity of the row/col
# coordinate for the current image
# Parameters -
# row,col  : row/col coordinates
# drow,dcol: variance of row/col
# Return:  -
#       1: on success
#       0: on failure
####################################################

proc check_coord {row col drow dcol} {
global current_image
global rows
global cols

if [catch {expr $row + $col + $drow + $dcol} temp] {
   AlertBox "Inappropriate values for $current_image (${cols}x$rows): \n\
   row = $row col = $col\n height = $drow width = $dcol" "" 300x100 "Alert Box" "OK"
   return 0
  }
if { $row >= 0 && $row < $rows && $col >= 0 && $col < $cols && $drow >= 0 && $dcol >= 0 &&
     $row + $drow <= $rows && $col + $dcol <= $cols } {
   return 1
} else {
   AlertBox "Inappropriate values for $current_image (${cols}x$rows): \n\
   row = $row col = $col\n height = $drow width = $dcol" "" 300x100 "Alert Box" "OK"
   return 0
}
}

####################################################
# check_coord: check the validity of the row/col
# coordinate for the current image
# Parameters -
# row,col  : row/col coordinates
# drow,dcol: variance of row/col
# Return:  -
#       1: on success
#       0: on failure
####################################################

proc CvipAlert { mesg } {
	if { $mesg == "" || $mesg == "." } {
		AlertBox  "The function being called has failed, \n\
		please check the parameters and look at the message \n\
		on  stdout for possible reasons." "" "350x150" "Alert Box" "OK"
	} else {
		AlertBox  "$mesg." "" "350x150" "Alert Box" "OK"
	}
}

####################################################
# get_pos: Get the toplevel window position 
# Parameters -
#       win: name of the window
# Return:  -
#       the toplevel window position
####################################################
proc get_pos { win } {
   global GUI_DEBUG
	set temp [wm geometry $win]
	set temp1 [string first "+" $temp]
	set temp2 [string first "-" $temp]
	global {STRANGE}

	if { $temp1 < "0" } {
		set temp3 $temp2
	} elseif { $temp2 < "0" } {
		set temp3 $temp1
	} else {
		set temp3 [expr {$temp1 > $temp2} ? $temp2 : $temp1]
	}
	set temp1 [string last "+" $temp]
	set temp2 [string last "-" $temp]
	set temp4 [expr {$temp1 > $temp2} ? $temp1 : $temp2]

	set temp1 $temp3
	while { [string index $temp $temp3] == "+" || \
		[string index $temp $temp3] == "-"} {incr temp3}
	incr temp3 -1
	set out [string range $temp $temp1 $temp3]

	incr temp3
	while { [string index $temp $temp4] == "+" || \
		[string index $temp $temp4] == "-"} {incr temp4 -1}
	if { $GUI_DEBUG == "1" } {
		puts stderr "$temp $temp3 $temp4"
	}
	set temp1 [string range $temp $temp3 $temp4];
	incr temp1 [expr {$STRANGE} ? 1 : -1]
	append out $temp1

	incr temp4
	set temp3 $temp4
	while { [string index $temp $temp4] == "+" || \
		[string index $temp $temp4] == "-"} {incr temp4}
	incr temp4 -1
	append out [string range $temp $temp3 $temp4]

	incr temp4 
	set temp2 [string range $temp $temp4 end];
	incr temp2 [expr {$STRANGE} ? 1 : -1]
	if { $STRANGE } { set STRANGE 0 } else { set STRANGE 1 } 
	append out $temp2
	return $out
}


####################################################
# top_pos: Get the initial toplevel window position 
# Parameters -
#       size: the size of the window
# Return:  -
#       the toplevel window position
####################################################
proc top_pos { size } {
	global toplevel_position
	set border 10

	set temp [string first "x" $size]
	incr temp -1
	set width [string range $size 0 $temp]
	incr temp 2
	set height [string range $size $temp end]
	set out $size

	switch -exact -- $toplevel_position {
		UL {
			append out "+$border+$border"
			return $out
			}
		UR {
			set temp [winfo screenwidth .]
			append out "+[expr $temp - $width - 2*$border]+$border"
			return $out
			}
		LL {
			set temp [winfo screenheight .]
			append out "+$border+[expr $temp - $height - 5*$border ]"
			return $out
			}
		LR {
			set temp [winfo screenwidth .]
			set temp1 [winfo screenheight .]
			append out "+[expr $temp - $width - 2*$border]+\
				[expr $temp1 - $height - 5*$border]"
			return $out
			}
	}
}
