#***************************************************************************
# **********************************************************************
# Computer Vision/Image Processing Tool Project - Dr. Scott Umbaugh SIUE
# **********************************************************************
#
#             File Name: restoration.tcl
#           Description: Restoration module for graphical user interface
#         Related Files: 
#   Initial Coding Date: Mon Jul  8 16:28:56 CDT 1996
#           Portability: TCL script file
#            References:
#             Author(s): Yansheng Wei, Mark Zuke
#                        Southern Illinois University @ Edwardsville
#
## Copyright (C) 1996 SIUE - by Scott Umbaugh, Yansheng Wei, and Mark Zuke.
##
## Permission to use, copy, modify, and distribute this software and its
## documentation for any purpose and without fee is hereby granted, provided
## that the above copyright notice appear in all copies and that both that
## copyright notice and this permission notice appear in supporting
## documentation.  This software is provided "as is" without express or
## implied warranty.
##
# RCS (Revision Control System) Information
# ...Added automatically by RCS 
#
## $Log: restoration.tcl,v $
## Revision 1.12  1997/06/25 18:37:43  yawei
## removed comments
##
## Revision 1.11  1997/06/25 18:34:00  yawei
## no-opts
##
## Revision 1.10  1997/06/24 20:17:12  yawei
## GUI configuration performed
##
## Revision 1.9  1997/03/11 20:56:21  yawei
## Tcl/Tk script upgrading from tcl7.3/tk3.6 to tcl7.6/tk4.2
## compatible. This process is automated with tkboost perl
## script. @ Tue Mar 11 14:51:36 CST 1997
##
## Revision 1.8  1996/10/30 21:39:39  yawei
## Added frequency-filters
##
# Revision 1.7  1996/09/30  00:44:21  yawei
# Added geometry transform module
#
# Revision 1.6  1996/09/09  17:07:55  mzuke
# changed Help button relief from raised to flat
#
# Revision 1.5  1996/09/04  20:01:38  mzuke
# added icon name RESTORE for toplevel
#
# Revision 1.4  1996/08/30  01:34:11  yawei
# check with the initial version of  restore_spatial.tcl
#
# Revision 1.3  1996/08/28  03:17:33  yawei
# changed to suit modular structure, check in with the initial module of
# restore_noise.tcl
#
# Revision 1.2  1996/08/22  16:56:35  mzuke
# changed toplevel initial dimensions
#
# Revision 1.1  1996/07/08  22:10:09  mzuke
# Initial revision
#
#
#****************************************************************************
# Module: restoration.tcl
# Tcl version: 7.3 (Tcl/Tk/XF)
# Tk version: 3.6
# XF version: 2.3
#

# Procedure: restoration
proc restoration {} {
# xf ignore me 7
##########
# Procedure: restoration
# Author: Mark Zuke (mzuke@ee.siue.edu) May 16, 1996
# Description: calls toplevel screen to allow use of restoration functions
#             
# Arguments: none
# Returns: none
# Sideeffects: none
##########

global toplevel_position
global {topwin}
global {GuiColor}
global {GuiMisc}

  # Window manager configurations
  global tk_version
  wm positionfrom $topwin(restore_name) user
  wm sizefrom $topwin(restore_name) user
  wm geometry $topwin(restore_name) [top_pos $topwin(restore_top)]
  wm maxsize $topwin(restore_name) [lindex [split $topwin(restore_top_max)] 0] \
  	[lindex [split $topwin(restore_top_max)] 1]
  wm minsize $topwin(restore_name) [lindex [split $topwin(restore_top_min)] 0] \
  	[lindex [split $topwin(restore_top_min)] 1]
  wm title $topwin(restore_name) {RESTORATION}
  wm iconname $topwin(restore_name) {RESTORE}


  # build widget $topwin(restore_name).menubar
  frame $topwin(restore_name).menubar  -background "$GuiColor(frame_bg)"  -borderwidth {2}  -relief {raised}

  # build widget $topwin(restore_name).menubar.noise
	button $topwin(restore_name).menubar.noise -padx {1} -pady {1} -highlightthickness {0}   \
  -relief {flat} \
  -activebackground "$GuiColor(btn_act_bg)" \
  -background "$GuiColor(btn_bg)" \
  -command {res_noise} \
  -text {Noise}

  # build widget $topwin(restore_name).menubar.spatial_filters
	button $topwin(restore_name).menubar.spatial_filters -padx {1} -pady {1} -highlightthickness {0}   \
  -activebackground "$GuiColor(btn_act_bg)" \
  -background "$GuiColor(btn_bg)" \
  -relief {flat} \
  -command {spat_filters} \
  -text {Spatial filters}

  # build widget $topwin(restore_name).menubar.frequency_filters
	button $topwin(restore_name).menubar.frequency_filters -padx {1} -pady {1} -highlightthickness {0}  \
  -activebackground "$GuiColor(btn_act_bg)" \
  -background "$GuiColor(btn_bg)" \
  -relief {flat} \
  -text {Frequency filters} \
  -command {freq_filter} 

  # build widget $topwin(restore_name).menubar.geometric_transforms
	button $topwin(restore_name).menubar.geometric_transforms -padx {1} -pady {1} -highlightthickness {0}   \
  -activebackground "$GuiColor(btn_act_bg)" \
  -background "$GuiColor(btn_bg)" \
  -relief {flat} \
  -command {restore_geom} \
  -text {Geometric transforms}

	button $topwin(restore_name).menubar.help -padx {1} -pady {1} -highlightthickness {0}   \
  -activebackground "$GuiColor(alert_act_bg)" \
  -relief {flat} \
  -background "$GuiColor(alert_bg)" \
  -foreground "$GuiColor(alert_fg)" \
  -command {help_screen} \
  -text {Help}

  # pack widget $topwin(restore_name).menubar
  pack append $topwin(restore_name).menubar  \
  $topwin(restore_name).menubar.noise {left frame center expand}  \
  $topwin(restore_name).menubar.spatial_filters {left frame center expand}  \
  $topwin(restore_name).menubar.frequency_filters {left frame center expand}  \
  $topwin(restore_name).menubar.geometric_transforms {left frame center expand} \
  $topwin(restore_name).menubar.help {right frame center expand} 

  # build widget $topwin(restore_name).info
  frame $topwin(restore_name).info  -background "$GuiColor(frame_bg)"  -borderwidth {2}  -width {30} -height {0}  -relief {flat}

  # build widget $topwin(restore_name).perform
  frame $topwin(restore_name).perform  -background "$GuiColor(frame_bg)"  -borderwidth {2}  -relief {flat}

  # build widget $topwin(restore_name).perform.perform
  frame $topwin(restore_name).perform.perform  -background "$GuiColor(frame_bg)"  -borderwidth {2}

  # build widget $topwin(restore_name).perform.perform.dismiss
	button $topwin(restore_name).perform.perform.dismiss -padx {1} -pady {1} -highlightthickness {0}\
	-activebackground "$GuiColor(dis_act_bg)"  -background "$GuiColor(dis_bg)" \
	-foreground "$GuiColor(dis_fg)" \
	-command {destroy $topwin(restore_name)}  -text {DISMISS}  -width {10}

  # pack widget $topwin(restore_name).perform.perform
  pack append $topwin(restore_name).perform.perform  \
  $topwin(restore_name).perform.perform.dismiss {left frame sw expand} 

  # pack widget $topwin(restore_name).perform
  pack append $topwin(restore_name).perform  $topwin(restore_name).perform.perform {bottom frame center fillx}

  # pack widget $topwin(restore_name)
  pack append $topwin(restore_name)  $topwin(restore_name).menubar {top frame center fillx}  $topwin(restore_name).info {top frame center expand fill}  $topwin(restore_name).perform {bottom frame center fillx}

  if {"[info procs XFEdit]" != ""} {
    catch "XFMiscBindWidgetTree $topwin(restore_name)"
    after 2 "catch {XFEditSetShowWindows}"
  }
}

