# **********************************************************************
# Computer Vision/Image Processing Tool Project - Dr. Scott Umbaugh SIUE
# **********************************************************************
#
#             File Name: utilities.tcl
#           Description: 
#         Related Files: 
#   Initial Coding Date: Fri Aug  9 17:34:42 CDT 1996
#           Portability: TCL script file
#            References:
#             Author(s): Yansheng Wei
#                        Southern Illinois University @ Edwardsville
#
## Copyright (C) 1996 SIUE - by Scott Umbaugh and Yansheng Wei.
##
## Permission to use, copy, modify, and distribute this software and its
## documentation for any purpose and without fee is hereby granted, provided
## that the above copyright notice appear in all copies and that both that
## copyright notice and this permission notice appear in supporting
## documentation.  This software is provided "as is" without express or
## implied warranty.
##
# RCS (Revision Control System) Information
# ...Added automatically by RCS 
#
## $Log: utilities.tcl,v $
## Revision 1.26  1997/07/16 16:58:41  yawei
## GUI configuration in place
##
## Revision 1.25  1997/07/15 22:39:43  yawei
## check in changes made by Sreenivas
##
## Revision 1.24  1997/03/11 20:57:26  yawei
## Tcl/Tk script upgrading from tcl7.3/tk3.6 to tcl7.6/tk4.2
## compatible. This process is automated with tkboost perl
## script. @ Tue Mar 11 14:51:36 CST 1997
##
## Revision 1.23  1996/12/05 04:00:05  yawei
## Added histogram stretch to enhancement.
##
# Revision 1.22  1996/11/03  01:47:46  yawei
# Using "luminance" to handle color image in labeling
#
# Revision 1.21  1996/10/31  03:50:08  yawei
# Changed "Gaussian_blur" to "Gaussian_filter"
#
# Revision 1.20  1996/10/25  00:29:18  yawei
# Added Gaussian_blur under Filter
#
# Revision 1.19  1996/10/18  14:55:09  yawei
# Added "Utilities | Edge Enhancement"
#
# Revision 1.18  1996/10/18  02:04:47  yawei
# added option of multiplying by a factor or by an image in UTILITIES/AL
#
# Revision 1.17  1996/10/17  03:14:04  yawei
# changed text
#
# Revision 1.16  1996/10/10  03:32:42  yawei
# Added "noise" and "black image"
#
# Revision 1.15  1996/10/08  02:44:48  yawei
# Added new functions
#
# Revision 1.14  1996/10/04  14:29:08  yawei
# added data_range module
#
# Revision 1.13  1996/09/18  18:55:50  yawei
# Changed "Gray to color " in "Utilities | Enhance" to "Pseudocolor"
#
# Revision 1.12  1996/09/17  18:24:12  yawei
# Changed Utilities | Enhance so it has "Gray to Color"
#
# Revision 1.11  1996/09/15  23:10:33  yawei
# added pseudo color under Utilities | enhance
#
# Revision 1.10  1996/09/03  19:06:01  yawei
# modified wm parameter.
#
# Revision 1.9  1996/08/27  19:18:35  mzuke
# added call of help screen to utility toplevel
#
# Revision 1.8  1996/08/19  21:20:25  mzuke
# changed function call from extract_band to ext_band
#
# Revision 1.7  1996/08/19  20:51:47  mzuke
# added Extract and Assemble band to create menu
#
# Revision 1.6  1996/08/15  15:39:42  mzuke
# added Label segmented image to Convert menu
#
# Revision 1.5  1996/08/13  15:23:16  mzuke
# reversed order of Misc listings in Util toplevel
#
# Revision 1.4  1996/08/09  23:37:58  yawei
# modified so the UTILITIES window can show up even the use has unsuccessfully loaded an image.
#
# Revision 1.3  1996/08/09  22:38:25  yawei
# This is the new version of UTILITIES in the GUI. The previous UTILITIES contains only one file, i.e.,
# utilities.tcl. Now the new UTILITIES has a menu system, and contains 8 files. All other seven files
# come together with the new UTILITIES. They are:
# util_al, util_conv, util_creat, util_enhen, util_filter, util_misc, util_size (tcl).
#
#
#****************************************************************************

# procedure to show window .
proc utilities { } {# xf ignore me 7

 global {topwin}
 global GuiColor
 global toplevel_position
 global message 
 global current_image
 global rows
 global cols
 global bands
 global row_coord
 global col_coord
 global d_row
 global d_col
 global cname
 global cresult

 
  # Window manager configurations
  global tk_version
  wm positionfrom $topwin(util_name) user
  wm sizefrom $topwin(util_name) user

  wm geometry $topwin(util_name) [top_pos $topwin(util_top)]
  wm maxsize $topwin(util_name) [lindex [split $topwin(util_top_max)] 0] \
  	[lindex [split $topwin(util_top_max)] 1]
  wm minsize $topwin(util_name) [lindex [split $topwin(util_top_min)] 0] \
  	[lindex [split $topwin(util_top_min)] 1]

  wm title $topwin(util_name) {UTILITIES}

  # build widget $topwin(util_name).menu
  frame $topwin(util_name).menu \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2} \
    -relief {raised}

  # build widget $topwin(util_name).menu.menubutton1
	menubutton $topwin(util_name).menu.menubutton1 -padx {1} -pady {1} -highlightthickness {0}  \
    -background "$GuiColor(btn_bg)" \
	 -activebackground "$GuiColor(btn_act_bg)" \
    -menu "$topwin(util_name).menu.menubutton1.menu" \
    -text {Arith/Logic}

  # build widget $topwin(util_name).menu.menubutton1.menu
  menu $topwin(util_name).menu.menubutton1.menu \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)" 
  $topwin(util_name).menu.menubutton1.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)"  \
    -label {Add} \
	 -command {util_arith add}\
    -state {active}
  $topwin(util_name).menu.menubutton1.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)"  \
    -label {Subtract} \
	 -command {util_arith subtract}\
    -state {active}
  $topwin(util_name).menu.menubutton1.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)"  \
    -label {Multiply} \
	 -command {util_mult}\
    -state {active}
  $topwin(util_name).menu.menubutton1.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)"  \
    -label {Divide} \
	 -command {util_arith divide}\
    -state {active}
  $topwin(util_name).menu.menubutton1.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)"  \
    -label {AND} \
	 -command {util_arith and}\
    -state {active}
  $topwin(util_name).menu.menubutton1.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)"  \
    -label {NOT} \
	 -command {util_arith not}\
    -state {active}
  $topwin(util_name).menu.menubutton1.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)"  \
    -label {OR} \
	 -command {util_arith or}\
    -state {active}
  $topwin(util_name).menu.menubutton1.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)"  \
    -label {XOR} \
	 -command {util_arith xor}\
    -state {active}

  # build widget $topwin(util_name).menu.menubutton2
	button $topwin(util_name).menu.menubutton2 -padx {1} -pady {1} -highlightthickness {0}  \
    -background "$GuiColor(alert_bg)" \
    -foreground "$GuiColor(alert_fg)" \
	 -activebackground "$GuiColor(alert_act_bg)" \
    -command {help_screen} \
    -relief {flat} \
    -text {Help}

  # build widget $topwin(util_name).menu.menubutton3
	menubutton $topwin(util_name).menu.menubutton3 -padx {1} -pady {1} -highlightthickness {0}  \
    -background "$GuiColor(btn_bg)" \
	 -activebackground "$GuiColor(btn_act_bg)" \
    -menu "$topwin(util_name).menu.menubutton3.menu" \
    -text {Create}

  # build widget $topwin(util_name).menu.menubutton3.menu
  menu $topwin(util_name).menu.menubutton3.menu \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)" 
  $topwin(util_name).menu.menubutton3.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)"  \
	 -command {util_box "Checkerboard"} \
    -label {Checkerboard}
  $topwin(util_name).menu.menubutton3.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)"  \
	 -command {util_circle}\
    -label {Circle}
  $topwin(util_name).menu.menubutton3.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)"  \
	 -command {util_box "Line"} \
    -label {Line}
  $topwin(util_name).menu.menubutton3.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)"  \
	 -command {util_box "Rectangle"} \
    -label {Rectangle}
  $topwin(util_name).menu.menubutton3.menu add separator
  $topwin(util_name).menu.menubutton3.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)"  \
	 -command {util_wave "Cosine wave"} \
    -label {Cosine wave}
  $topwin(util_name).menu.menubutton3.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)"  \
	 -command {util_wave "Sine wave"} \
    -label {Sine wave}
  $topwin(util_name).menu.menubutton3.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)"  \
	 -command {util_wave "Square wave"} \
    -label {Square wave}
  $topwin(util_name).menu.menubutton3.menu add separator
  $topwin(util_name).menu.menubutton3.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)"  \
	 -command {ext_band} \
    -label {Extract band}
  $topwin(util_name).menu.menubutton3.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)"  \
	 -command {assemble_bands} \
    -label {Assemble bands}
  $topwin(util_name).menu.menubutton3.menu add separator
  $topwin(util_name).menu.menubutton3.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)"  \
	 -command {util_black} \
    -label {Black image}
  $topwin(util_name).menu.menubutton3.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)"  \
	 -command {util_noise} \
    -label {Add noise}

  # build widget $topwin(util_name).menu.menubutton5
	menubutton $topwin(util_name).menu.menubutton5 -padx {1} -pady {1} -highlightthickness {0}  \
    -background "$GuiColor(btn_bg)" \
	 -activebackground "$GuiColor(btn_act_bg)" \
    -menu "$topwin(util_name).menu.menubutton5.menu" \
    -text {Convert}

  # build widget $topwin(util_name).menu.menubutton5.menu
  menu $topwin(util_name).menu.menubutton5.menu \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)" 
  $topwin(util_name).menu.menubutton5.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)"  \
    -label {Binary threshold} \
    -command {util_thresh} \
    -state {active}
  $topwin(util_name).menu.menubutton5.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)"  \
    -label {Color space} \
	 -command {util_color} \
	 -state {active}
  $topwin(util_name).menu.menubutton5.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)"  \
	 -command {util_gray_quant} \
    -label {Gray Level Quantization} 
  $topwin(util_name).menu.menubutton5.menu add command \
	 -command {util_gray_bin} \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)"  \
    -label {Gray code-Natural binary} 
  $topwin(util_name).menu.menubutton5.menu add command \
	 -command {util_gray_col} \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)"  \
    -label {Gray to color} 
  $topwin(util_name).menu.menubutton5.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)"  \
    -label {Color to gray} \
	 -command {util_col_grey}
  $topwin(util_name).menu.menubutton5.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)"  \
	 -command {util_dataremap} \
    -label {Data type}
  $topwin(util_name).menu.menubutton5.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)"  \
	 -command {util_label} \
    -label {Label image}

  # build widget $topwin(util_name).menu.menubutton7
	menubutton $topwin(util_name).menu.menubutton7 -padx {1} -pady {1} -highlightthickness {0}  \
    -background "$GuiColor(btn_bg)" \
	 -activebackground "$GuiColor(btn_act_bg)" \
    -menu "$topwin(util_name).menu.menubutton7.menu" \
    -text {Size}

  # build widget $topwin(util_name).menu.menubutton7.menu
  menu $topwin(util_name).menu.menubutton7.menu \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)" 
  $topwin(util_name).menu.menubutton7.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)"  \
	 -command {util_crop} \
    -label {Crop} \
    -state {active}
  $topwin(util_name).menu.menubutton7.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)"  \
	 -command {util_resize} \
    -label {Resize}
  $topwin(util_name).menu.menubutton7.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)"  \
	 -command {util_spa_quant} \
    -label {Spatial Quant}

  # build widget $topwin(util_name).menu.menubutton8
	menubutton $topwin(util_name).menu.menubutton8 -padx {1} -pady {1} -highlightthickness {0}  \
    -background "$GuiColor(btn_bg)" \
	 -activebackground "$GuiColor(btn_act_bg)" \
    -menu "$topwin(util_name).menu.menubutton8.menu" \
    -text {Filter}

  # build widget $topwin(util_name).menu.menubutton8.menu
  menu $topwin(util_name).menu.menubutton8.menu \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)" 
  $topwin(util_name).menu.menubutton8.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)"  \
	 -command {util_filter mean_filter}\
    -label {Mean} \
    -state {active}
  $topwin(util_name).menu.menubutton8.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)"  \
	 -command {util_filter median_filter}\
    -label {Median}
  $topwin(util_name).menu.menubutton8.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)"  \
	 -command {util_specify}\
    -label {Specify a filter}
  $topwin(util_name).menu.menubutton8.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)"  \
	 -command {util_blur}\
    -label {Specify a blur}
  $topwin(util_name).menu.menubutton8.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)"  \
	 -command {util_filter gauss_blur}\
    -label {Gaussian filter} \
    -state {active}

  # build widget $topwin(util_name).menu.menubutton9
	menubutton $topwin(util_name).menu.menubutton9 -padx {1} -pady {1} -highlightthickness {0}  \
    -background "$GuiColor(btn_bg)" \
	 -activebackground "$GuiColor(btn_act_bg)" \
    -menu "$topwin(util_name).menu.menubutton9.menu" \
    -text {Misc}

  # build widget $topwin(util_name).menu.menubutton9.menu
  menu $topwin(util_name).menu.menubutton9.menu \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)" 
  $topwin(util_name).menu.menubutton9.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)"  \
    -label {Compare two images} \
	 -command {util_compare} 
  $topwin(util_name).menu.menubutton9.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)"  \
	 -command {add_path1} \
    -label {Add temporary image path}
  $topwin(util_name).menu.menubutton9.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)"  \
	 -command {util_datarange} \
    -label {Image data range }

  # build widget $topwin(util_name).menu.menubutton9.menu.menu
  menu $topwin(util_name).menu.menubutton9.menu.menu \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)" 
  $topwin(util_name).menu.menubutton9.menu.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)"  \
    -label {By subtraction}
  $topwin(util_name).menu.menubutton9.menu.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)"  \
    -label {By XOR}

  # build widget $topwin(util_name).menu.menubutton0
	menubutton $topwin(util_name).menu.menubutton0 -padx {1} -pady {1} -highlightthickness {0}  \
    -background "$GuiColor(btn_bg)" \
	 -activebackground "$GuiColor(btn_act_bg)" \
    -menu "$topwin(util_name).menu.menubutton0.menu" \
    -text {Enhance}

  # build widget $topwin(util_name).menu.menubutton0.menu
  menu $topwin(util_name).menu.menubutton0.menu \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)" 
  $topwin(util_name).menu.menubutton0.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)"  \
	 -command {util_gray_multi} \
    -label {Brighten/Darken} \
    -state {active}
  $topwin(util_name).menu.menubutton0.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)"  \
	 -command {util_edge} \
    -label {Edge enhancement} \
    -state {active}
  $topwin(util_name).menu.menubutton0.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)"  \
	 -command {util_histeq} \
    -label {Histogram equalization}
  $topwin(util_name).menu.menubutton0.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)"  \
	 -command {util_stretch} \
    -label {Histogram stretch}
  $topwin(util_name).menu.menubutton0.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)"  \
	 -command {util_gray_col
	 wm title $topwin(util_name) "UTILITIES -> Enhance -> Pseudocolor"} \
    -label {Pseudocolor}

  # pack widget $topwin(util_name).menu
  pack append $topwin(util_name).menu \
    $topwin(util_name).menu.menubutton1 {left frame center expand fillx} \
    $topwin(util_name).menu.menubutton5 {left frame center expand fillx} \
    $topwin(util_name).menu.menubutton3 {left frame center expand fillx} \
    $topwin(util_name).menu.menubutton0 {left frame center expand fillx} \
    $topwin(util_name).menu.menubutton8 {left frame center expand fillx} \
    $topwin(util_name).menu.menubutton7 {left frame center expand fillx} \
    $topwin(util_name).menu.menubutton9 {left frame center expand fillx} \
    $topwin(util_name).menu.menubutton2 {left frame center expand fillx}

  # build widget $topwin(util_name).info
  frame $topwin(util_name).info \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2} \
    -width {30} -height {0}

  # build widget $topwin(util_name).perform
  frame $topwin(util_name).perform \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(util_name).perform.button4
	button $topwin(util_name).perform.button4 -padx {1} -pady {1} -highlightthickness {0}  \
    -background "$GuiColor(dis_bg)" \
    -foreground "$GuiColor(dis_fg)" \
	 -activebackground "$GuiColor(dis_act_bg)" \
    -command {destroy $topwin(util_name); set topwin(util_name) NONE} \
    -text {DISMISS}

  # pack widget $topwin(util_name).perform
  pack append $topwin(util_name).perform \
    $topwin(util_name).perform.button4 {left frame center}

  # pack widget $topwin(util_name)
  pack append $topwin(util_name) \
    $topwin(util_name).menu {top frame center fill} \
    $topwin(util_name).info {top frame center expand fill} \
    $topwin(util_name).perform {bottom frame center fillx}

  if {"[info procs XFEdit]" != ""} {
    catch "XFMiscBindWidgetTree ."
    after 2 "catch {XFEditSetShowWindows}"
  }
}

