# **********************************************************************
# Computer Vision/Image Processing Tool Project - Dr. Scott Umbaugh SIUE
# **********************************************************************
#
#             File Name: util_filter.tcl
#           Description: 
#         Related Files: 
#   Initial Coding Date: Fri Aug  9 17:33:57 CDT 1996
#           Portability: TCL script file
#            References:
#             Author(s): Yansheng Wei
#                        Southern Illinois University @ Edwardsville
#
## Copyright (C) 1996 SIUE - by Scott Umbaugh and Yansheng Wei.
##
## Permission to use, copy, modify, and distribute this software and its
## documentation for any purpose and without fee is hereby granted, provided
## that the above copyright notice appear in all copies and that both that
## copyright notice and this permission notice appear in supporting
## documentation.  This software is provided "as is" without express or
## implied warranty.
##
# RCS (Revision Control System) Information
# ...Added automatically by RCS 
#
## $Log: util_filter.tcl,v $
## Revision 1.20  1997/07/28 21:30:36  wzheng
## mean filter setup on mask size.
##
## Revision 1.19  1997/07/23 00:20:36  smakam
## made changes so that that the perform and edit mask button can be used simultaneously in specify blur
##
## Revision 1.18  1997/07/21 18:00:03  yawei
## changed variable name
##
## Revision 1.17  1997/07/17 22:20:21  yawei
## added size configuration
##
## Revision 1.16  1997/07/17 22:15:19  yawei
## GUI configuration is done
##
## Revision 1.15  1997/07/08 00:41:58  smakam
## included specify blur in util_filter
##
## Revision 1.14  1997/03/12 16:27:40  yawei
## changed size, fixed arrow buttons
##
## Revision 1.13  1997/03/11 20:57:06  yawei
## Tcl/Tk script upgrading from tcl7.3/tk3.6 to tcl7.6/tk4.2
## compatible. This process is automated with tkboost perl
## script. @ Tue Mar 11 14:51:36 CST 1997
##
## Revision 1.12  1996/11/17 03:52:51  yawei
## Added convolution with any mask to the filter.
##
# Revision 1.11  1996/11/05  22:03:17  yawei
# added "mask_size" to the global variable list
#
# Revision 1.10  1996/10/31  03:49:38  yawei
# Changed "Gaussian_blur" to "Gaussian_filter"
#
# Revision 1.9  1996/10/30  21:56:53  yawei
# Added entry width so it holds the text "Gaussian blur ..."
#
# Revision 1.8  1996/10/25  00:29:35  yawei
# Added Gaussian_blur under Filter
#
# Revision 1.7  1996/10/22  18:27:21  yawei
# changed "high_pass" central default to N^2.
#
# Revision 1.6  1996/10/18  03:13:16  yawei
# Added "low pass" and "high pass" to the specify a filter.
#
# Revision 1.5  1996/10/16  20:14:34  yawei
# general chagnes
#
# Revision 1.4  1996/10/08  21:30:52  mzuke
# changed filters to show correct image name on display
#
# Revision 1.3  1996/09/27  18:33:44  mzuke
# made window resizable
#
# Revision 1.2  1996/08/15  11:19:49  mzuke
# removed toplevel_position from wm geometry to allow window
# positioning by user
#
# Revision 1.1  1996/08/09  22:37:38  yawei
# Initial revision
#
#
#****************************************************************************
# procedure to show window $topwin(util_name)
proc util_filter { filter } {# xf ignore me 7
  global GuiColor
  global GuiMisc
  global GUI_DEBUG

  global tk_version
  global topwin
  global util
  set util(remove_method) $filter
  global toplevel_position
  if [ winfo exists $topwin(util_name).info] {
     set tempname $topwin(util_name);\
destroy $topwin(util_name).info;\
set topwin(util_name) $tempname;}
  if [ winfo exists $topwin(util_name).current_image] {
     set tempname $topwin(util_name);\
destroy $topwin(util_name).current_image;\
set topwin(util_name) $tempname;}
  if [ winfo exists $topwin(util_name).perform] {
     set tempname $topwin(util_name);\
destroy $topwin(util_name).perform;\
set topwin(util_name) $tempname;}
  wm positionfrom $topwin(util_name) user
  wm sizefrom $topwin(util_name) user
  wm geometry $topwin(util_name) $topwin(util_top)[get_pos $topwin(util_name)]
  wm maxsize $topwin(util_name) [lindex [split $topwin(util_top_max)] 0] \
  	[lindex [split $topwin(util_top_max)] 1]
  wm minsize $topwin(util_name) [lindex [split $topwin(util_top_min)] 0] \
  	[lindex [split $topwin(util_top_min)] 1]
  if {$filter == "median_filter"} {
	set temp Median
  } elseif { $filter == "mean_filter" } {
	set temp Mean
  } else {
	set temp "Gaussian filter"
  }
  wm title $topwin(util_name) "UTILITIES -> Filter -> $temp"

  # build widget $topwin(util_name).info
  frame $topwin(util_name).info \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(util_name).info.frame0
  frame $topwin(util_name).info.frame0 \
    -background "$GuiColor(frame_bg)" 

  # build widget $topwin(util_name).info.frame0.label1
  label $topwin(util_name).info.frame0.label1 \
    -anchor {e} \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -text {Mean filter mask size: } \
    -width {29}
  if {$filter == "median_filter"} {
  	$topwin(util_name).info.frame0.label1 configure -text {Median filter mask size: }
  } elseif { $filter == "gauss_blur" } {
  	$topwin(util_name).info.frame0.label1 configure -text {Gaussian filter kernel size: }
  }
  # build widget $topwin(util_name).info.frame0.value
  entry $topwin(util_name).info.frame0.value \
    -background "$GuiColor(rentry_bg)" \
    -foreground "$GuiColor(rentry_fg)" \
    -relief {sunken} \
    -state {disabled} \
    -textvariable {util(mask_size)} \
    -width {10}
  endEntry $topwin(util_name).info.frame0.value 0
	if { $filter == "gauss_blur" || $filter == "mean_filter"} {
	$topwin(util_name).info.frame0.value configure -state disabled -foreground "$GuiColor(rentry_fg)" -background "$GuiColor(rentry_bg)" 
	} else {
	$topwin(util_name).info.frame0.value configure -state normal -foreground black
	}
  # build widget $topwin(util_name).info.frame0.menubutton2
	menubutton $topwin(util_name).info.frame0.menubutton2 -padx {1} -pady {1} -highlightthickness {0}  \
    -background "$GuiColor(btn_bg)" \
	 -activebackground "$GuiColor(btn_act_bg)" \
    -font "$GuiMisc(pop_up_fn)" \
	 -text "$GuiMisc(pop_up_text)" \
	 -relief "$GuiMisc(pop_up_relief)" \
	 -width {2}
  # bindings
  bind $topwin(util_name).info.frame0.menubutton2 <Button-1> {MenuPopupPost $topwin(util_name).info.frame0.menubutton2.menu %X %Y}
  bind $topwin(util_name).info.frame0.menubutton2 <ButtonRelease-1> {MenuPopupRelease $topwin(util_name).info.frame0.menubutton2.menu %W}

  # build widget $topwin(util_name).info.frame0.menubutton2.menu
  menu $topwin(util_name).info.frame0.menubutton2.menu \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)"
  $topwin(util_name).info.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)" \
    -command {set util(mask_size) 3} \
    -label { 3}
  $topwin(util_name).info.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)" \
    -command {set util(mask_size) 5} \
    -label { 5}
  $topwin(util_name).info.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)" \
    -command {set util(mask_size) 7} \
    -label { 7}
	if { $filter != "gauss_blur" } {
		if { $util(mask_size) == "9" } {
    		set util(mask_size) 7
		}
  $topwin(util_name).info.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)" \
    -command {set util(mask_size) 9} \
    -label { 9}
  $topwin(util_name).info.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
         -background "$GuiColor(menu_bg)" \
    -command {set util(mask_size) 11} \
    -label {11}

  }
  bindtags $topwin(util_name).info.frame0.menubutton2.menu {Menu all}
  # bindings
  bind $topwin(util_name).info.frame0.menubutton2.menu <B1-Motion> {MenuPopupMotion $topwin(util_name).info.frame0.menubutton2.menu %W %X %Y}
  bind $topwin(util_name).info.frame0.menubutton2.menu <ButtonRelease-1> {MenuPopupRelease $topwin(util_name).info.frame0.menubutton2.menu %W}

  # pack widget $topwin(util_name).info.frame0
  pack append $topwin(util_name).info.frame0 \
    $topwin(util_name).info.frame0.label1 {left frame center fill} \
    $topwin(util_name).info.frame0.value {left frame center fill} \
    $topwin(util_name).info.frame0.menubutton2 {right frame center fill}

  # pack widget $topwin(util_name).info
  pack append $topwin(util_name).info \
    $topwin(util_name).info.frame0 {left frame center padx 215}

  # build widget $topwin(util_name).perform
  frame $topwin(util_name).perform \
    -background "$GuiColor(frame_bg)"  \
    -borderwidth {2} \
    -relief {raised}

  # build widget $topwin(util_name).perform.perform1
	button $topwin(util_name).perform.perform1 -padx {1} -pady {1} -highlightthickness {0}  \
    -foreground "$GuiColor(perf_fg)" \
    -background "$GuiColor(perf_bg)" \
	 -activebackground "$GuiColor(perf_act_bg)" \
    -text {PERFORM} \
	 -command {
		set temp [string trimright $util(remove_method) _filter]
		if { $util(remove_method) == "gauss_blur" } {
			set temp1 mask
		} else {
			set temp1 kernel
		}
		$topwin(util_name) configure -cursor watch; update
		if {$current_image == ""} {
			CvipAlert "Please select an image first."
			$topwin(util_name) configure -cursor top_left_arrow; update
			return
		} elseif [catch {expr $util(mask_size)} ] {
			CvipAlert "Numerical value required for $temp1 size.\n $temp1 size = $util(mask_size)"
			$topwin(util_name) configure -cursor top_left_arrow; update
			return
		} elseif { $util(mask_size) < 3 } {
			CvipAlert "$temp1 size shoud be >= 3. \n $temp1 size = $util(mask_size)"
			$topwin(util_name) configure -cursor top_left_arrow; update
			return
		}
		if [catch {set result [cvip $util(remove_method) $current_image $util(mask_size)]} cresult ] {
			CvipAlert "$cresult"
			$topwin(util_name) configure -cursor top_left_arrow; update
			return
		}
		set new_name [cvip rename $result ${current_image}_${temp}_$util(mask_size)]
		img_q insert end $new_name
		image_post $new_name
		cvip view $new_name
		CvipMessage "$util(remove_method) performed on image \"$current_image\""
	$topwin(util_name) configure -cursor top_left_arrow; update
	 }

  # build widget $topwin(util_name).perform.dismiss
	button $topwin(util_name).perform.dismiss -padx {1} -pady {1} -highlightthickness {0}  \
    -foreground "$GuiColor(dis_fg)" \
    -background "$GuiColor(dis_bg)" \
	 -activebackground "$GuiColor(dis_act_bg)" \
    -command {destroy $topwin(util_name); set topwin(util_name) NONE} \
    -text {DISMISS} \
    -width {10}

  # pack widget $topwin(util_name).perform
  pack append $topwin(util_name).perform \
    $topwin(util_name).perform.dismiss {left frame sw filly} \
    $topwin(util_name).perform.perform1 {right frame se filly}

  # build widget $topwin(util_name).current_image
  frame $topwin(util_name).current_image \
    -background "$GuiColor(frame_bg)"  \
    -borderwidth {2} \
    -relief {ridge}

  # build widget $topwin(util_name).current_image.current
  label $topwin(util_name).current_image.current \
    -background "$GuiColor(label_bg)" \
    -text {Current image:}

  # build widget $topwin(util_name).current_image.name
  label $topwin(util_name).current_image.name \
    -background "$GuiColor(rlabel_bg)" \
    -foreground "$GuiColor(rlabel_fg)" \
    -text {} \
    -textvariable {current_image}

  # pack widget $topwin(util_name).current_image
  pack append $topwin(util_name).current_image \
    $topwin(util_name).current_image.current {left frame center} \
    $topwin(util_name).current_image.name {left frame center filly}

  # pack widget $topwin(util_name)
  pack append $topwin(util_name) \
    $topwin(util_name).info {top frame center expand fill} \
    $topwin(util_name).perform {bottom frame center pady 5 fill} \
    $topwin(util_name).current_image {bottom frame center fill}

  if {"[info procs XFEdit]" != ""} {
    catch "XFMiscBindWidgetTree $topwin(util_name)"
    after 2 "catch {XFEditSetShowWindows}"
  }
}


# procedure to show window $topwin(util_name)
proc util_specify { } {# xf ignore me 7
  global GuiColor
  global GuiMisc
  global GUI_DEBUG

  # Window manager configurations
  global tk_version
  global topwin
  global {util}
  global toplevel_position
  if [ winfo exists $topwin(util_name).info] {
     set tempname $topwin(util_name);\
destroy $topwin(util_name).info;\
set topwin(util_name) $tempname;}
  if [ winfo exists $topwin(util_name).current_image] {
     set tempname $topwin(util_name);\
destroy $topwin(util_name).current_image;\
set topwin(util_name) $tempname;}
  if [ winfo exists $topwin(util_name).perform] {
     set tempname $topwin(util_name);\
destroy $topwin(util_name).perform;\
set topwin(util_name) $tempname;}
  wm positionfrom $topwin(util_name) user
  wm sizefrom $topwin(util_name) user
  wm geometry $topwin(util_name) $topwin(util_top)[get_pos $topwin(util_name)]
  wm maxsize $topwin(util_name) [lindex [split $topwin(util_top_max)] 0] \
  	[lindex [split $topwin(util_top_max)] 1]
  wm minsize $topwin(util_name) [lindex [split $topwin(util_top_min)] 0] \
  	[lindex [split $topwin(util_top_min)] 1]
  wm title $topwin(util_name) {UTILITIES -> Filter -> Specify a filter}

  # bindings
  bind $topwin(util_name) <Destroy> {if [ winfo exists .filter] {destroy .filter}; set topwin(util_name) NONE}

  # build widget $topwin(util_name).info
  frame $topwin(util_name).info \
    -background "$GuiColor(frame_bg)"  \
    -borderwidth {2}
  # bindings
  bind $topwin(util_name).info <Button-1> {MenuPopupPost $topwin(util_name).info.menu2 %X %Y}
  bind $topwin(util_name).info <ButtonRelease-1> {MenuPopupRelease $topwin(util_name).info.menu2 %W}

  # build widget $topwin(util_name).info.frame3
  frame $topwin(util_name).info.frame3 \
    -background "$GuiColor(frame_bg)" 

  # build widget $topwin(util_name).info.frame3.label5
  label $topwin(util_name).info.frame3.label5 \
    -background "$GuiColor(label_bg)" \
    -text {Specify a }

  # build widget $topwin(util_name).info.frame3.frame0
  frame $topwin(util_name).info.frame3.frame0 \
    -background "$GuiColor(frame_bg)" 

  # build widget $topwin(util_name).info.frame3.frame0.label1
  label $topwin(util_name).info.frame3.frame0.label1 \
    -anchor {e} \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -relief {sunken} \
    -text { spatial filter of the size: }

  # build widget $topwin(util_name).info.frame3.frame0.value
  entry $topwin(util_name).info.frame3.frame0.value \
    -background "$GuiColor(rentry_bg)" \
	 -foreground "$GuiColor(rentry_fg)" \
    -textvariable {util(specify_hl)} \
    -width {20}
  # bindings
  bind $topwin(util_name).info.frame3.frame0.value <Key-Return> { 
	  set temp [%W get]
	  %W delete 0 end
	  %W insert 0 $temp
	  }

  # build widget $topwin(util_name).info.frame3.frame0.menubutton2
	menubutton $topwin(util_name).info.frame3.frame0.menubutton2 -padx {1} -pady {1} -highlightthickness {0}  \
    -background "$GuiColor(btn_bg)" \
	 -activebackground "$GuiColor(btn_act_bg)" \
    -font "$GuiMisc(pop_up_fn)" \
	 -text "$GuiMisc(pop_up_text)" \
	 -relief "$GuiMisc(pop_up_relief)" \
	 -width {2}
  # bindings
  bind $topwin(util_name).info.frame3.frame0.menubutton2 <Button-1> {MenuPopupPost $topwin(util_name).info.frame3.frame0.menubutton2.menu %X %Y}
  bind $topwin(util_name).info.frame3.frame0.menubutton2 <ButtonRelease-1> {MenuPopupRelease $topwin(util_name).info.frame3.frame0.menubutton2.menu %W}

  # build widget $topwin(util_name).info.frame3.frame0.menubutton2.menu
  menu $topwin(util_name).info.frame3.frame0.menubutton2.menu \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)"
  $topwin(util_name).info.frame3.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)" \
    -command {set util(specify_hl) "low-pass"} \
    -label {Low pass}
  $topwin(util_name).info.frame3.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)" \
    -command {set util(specify_hl) "high-pass"} \
    -label {High pass}
  $topwin(util_name).info.frame3.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)" \
    -command {set util(specify_hl) "center-weight-constant"} \
    -label {Center weighted constant}
  $topwin(util_name).info.frame3.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)" \
    -command {set util(specify_hl) "gaussian"} \
    -label {Gaussian}
  bindtags $topwin(util_name).info.frame3.frame0.menubutton2.menu  {Menu all}
  # bindings
  bind $topwin(util_name).info.frame3.frame0.menubutton2.menu <B1-Motion> {MenuPopupMotion $topwin(util_name).info.frame3.frame0.menubutton2.menu %W %X %Y}
  bind $topwin(util_name).info.frame3.frame0.menubutton2.menu <ButtonRelease-1> {MenuPopupRelease $topwin(util_name).info.frame3.frame0.menubutton2.menu %W}

  # pack widget $topwin(util_name).info.frame3.frame0
  pack append $topwin(util_name).info.frame3.frame0 \
    $topwin(util_name).info.frame3.frame0.value {left frame center} \
    $topwin(util_name).info.frame3.frame0.menubutton2 {left frame center} \
    $topwin(util_name).info.frame3.frame0.label1 {left frame center}

  # pack widget $topwin(util_name).info.frame3
  pack append $topwin(util_name).info.frame3 \
    $topwin(util_name).info.frame3.label5 {left frame center padx 10} \
    $topwin(util_name).info.frame3.frame0 {left frame center}
  # build widget $topwin(util_name).info.frame9
  frame $topwin(util_name).info.frame9 \
    -background "$GuiColor(frame_bg)" 

  # build widget $topwin(util_name).info.frame9.radiobutton10
	radiobutton $topwin(util_name).info.frame9.radiobutton10 -padx {1} -pady {1} -highlightthickness {0}  \
    -background "$GuiColor(btn_bg)" \
	 -activebackground "$GuiColor(btn_act_bg)" \
    -relief {flat} \
	 -command {mat_specify 3 $util(specify_hl) }\
    -text {3 x 3} \
    -value {3} \
    -variable {util(filter_dim)}

  # build widget $topwin(util_name).info.frame9.radiobutton14
	radiobutton $topwin(util_name).info.frame9.radiobutton14 -padx {1} -pady {1} -highlightthickness {0}  \
    -background "$GuiColor(btn_bg)" \
	 -activebackground "$GuiColor(btn_act_bg)" \
    -relief {flat} \
	 -command {mat_specify 5 $util(specify_hl) }\
    -text {5 x 5} \
    -value {5} \
    -variable {util(filter_dim)}

  # build widget $topwin(util_name).info.frame9.radiobutton15
	radiobutton $topwin(util_name).info.frame9.radiobutton15 -padx {1} -pady {1} -highlightthickness {0}  \
    -background "$GuiColor(btn_bg)" \
	 -activebackground "$GuiColor(btn_act_bg)" \
    -relief {flat} \
    -text {7 x 7} \
	 -command {mat_specify 7 $util(specify_hl) }\
    -value {7} \
    -variable {util(filter_dim)}

  # build widget $topwin(util_name).info.frame9.radiobutton16
	radiobutton $topwin(util_name).info.frame9.radiobutton16 -padx {1} -pady {1} -highlightthickness {0}  \
    -background "$GuiColor(btn_bg)" \
	 -activebackground "$GuiColor(btn_act_bg)" \
    -relief {flat} \
    -text {9 x 9} \
	 -command {mat_specify 9 $util(specify_hl) }\
    -value {9} \
    -variable {util(filter_dim)}

  # build widget $topwin(util_name).info.frame9.radiobutton17
	radiobutton $topwin(util_name).info.frame9.radiobutton17 -padx {1} -pady {1} -highlightthickness {0}  \
    -background "$GuiColor(btn_bg)" \
	 -activebackground "$GuiColor(btn_act_bg)" \
    -relief {flat} \
    -text {11 x 11} \
	 -command {mat_specify 11 $util(specify_hl) }\
    -value {11} \
    -variable {util(filter_dim)}

  # pack widget $topwin(util_name).info.frame9
  pack append $topwin(util_name).info.frame9 \
    $topwin(util_name).info.frame9.radiobutton10 {left frame center expand} \
    $topwin(util_name).info.frame9.radiobutton14 {left frame center expand} \
    $topwin(util_name).info.frame9.radiobutton15 {left frame center expand} \
    $topwin(util_name).info.frame9.radiobutton16 {left frame center expand} \
    $topwin(util_name).info.frame9.radiobutton17 {left frame center expand} 

  # pack widget $topwin(util_name).info
  pack append $topwin(util_name).info \
    $topwin(util_name).info.frame3 {top frame center expand fill} \
    $topwin(util_name).info.frame9 {top frame center expand fill}

  # build widget $topwin(util_name).perform
  frame $topwin(util_name).perform \
    -background "$GuiColor(frame_bg)"  \
    -borderwidth {2} \
    -relief {raised}

  # build widget $topwin(util_name).perform.perform1
	button $topwin(util_name).perform.perform1 -padx {1} -pady {1} -highlightthickness {0}  \
    -foreground "$GuiColor(perf_fg)" \
    -background "$GuiColor(perf_bg)" \
	 -activebackground "$GuiColor(perf_act_bg)" \
    -command {
		set new_name ""
		$topwin(util_name) configure -cursor watch; update
      if { $current_image == ""} {
         CvipAlert "Please select an image first."
			$topwin(util_name) configure -cursor top_left_arrow; update
         return
      } elseif {$util(filter_val) == "" } {
         CvipAlert "Please specify the mask matrix first."
			$topwin(util_name) configure -cursor top_left_arrow; update
         return
		}
		if [catch {set result [eval "cvip specify_spatial $current_image $util(filter_dim) $util(filter_dim) $util(filter_val)"]} cresult] {
        CvipAlert "$cresult."
			$topwin(util_name) configure -cursor top_left_arrow; update
        return
		}
     set new_name [cvip rename $result ${current_image}_filter]       
     img_q insert end $new_name
     image_post $new_name
     cvip view $new_name
	  CvipMessage "specify spatial filter performed on image \"$current_image\""
	$topwin(util_name) configure -cursor top_left_arrow; update
	 } \
    -text {PERFORM}

  # build widget $topwin(util_name).perform.dismiss
	button $topwin(util_name).perform.dismiss -padx {1} -pady {1} -highlightthickness {0}  \
    -foreground "$GuiColor(dis_fg)" \
    -background "$GuiColor(dis_bg)" \
	 -activebackground "$GuiColor(dis_act_bg)" \
    -command {if [winfo exist .filter ] {destroy .filter}; destroy $topwin(util_name); set topwin(util_name) NONE} \
    -text {DISMISS} \
    -width {10}

  # pack widget $topwin(util_name).perform
  pack append $topwin(util_name).perform \
    $topwin(util_name).perform.dismiss {left frame sw filly} 
#    $topwin(util_name).perform.perform1 {right frame se filly}

  # build widget $topwin(util_name).current_image
  frame $topwin(util_name).current_image \
    -background "$GuiColor(frame_bg)"  \
    -borderwidth {2} \
    -relief {ridge}

  # build widget $topwin(util_name).current_image.current
  label $topwin(util_name).current_image.current \
    -background "$GuiColor(label_bg)" \
    -text {Current image:}

  # build widget $topwin(util_name).current_image.name
  label $topwin(util_name).current_image.name \
    -foreground "$GuiColor(rlabel_fg)" \
    -background "$GuiColor(rlabel_bg)" \
    -text {} \
    -textvariable {current_image}

  # pack widget $topwin(util_name).current_image
  pack append $topwin(util_name).current_image \
    $topwin(util_name).current_image.current {left frame center} \
    $topwin(util_name).current_image.name {left frame center filly}

  # pack widget $topwin(util_name)
  pack append $topwin(util_name) \
    $topwin(util_name).info {top frame center expand fill} \
    $topwin(util_name).perform {bottom frame center pady 5 fill} \
    $topwin(util_name).current_image {bottom frame center fill}

  if {"[info procs XFEdit]" != ""} {
    catch "XFMiscBindWidgetTree $topwin(util_name)"
    after 2 "catch {XFEditSetShowWindows}"
  }
}

# lp means more than high/low pass, used for backward compatibility
# procedure to show window .m_util
proc mat_specify { mat_dim lp} {# xf ignore me 7
  global GuiColor
  global GuiMisc
  global GUI_DEBUG

  global {util}
  set util(filter_dim) $mat_dim
  if [ winfo exists .filter] {
  destroy .filter
  }
  toplevel .filter  -background {gray90}

  # Window manager configurations
  global tk_version
#  wm positionfrom .filter user
  wm sizefrom .filter user
  wm maxsize .filter 500 360
  wm minsize .filter 0 0
  wm title .filter {Specify filter}

  # bindings
  bind .filter <Destroy> {set util(filter_val) ""}

  # build widget .filter.info
  frame .filter.info \
    -background "$GuiColor(frame_bg)"  \
    -borderwidth {2}

  frame .filter.info.frame \
    -background "$GuiColor(frame_bg)"  \
    -borderwidth {2}

  for {set i 0} {$i <= $mat_dim} {incr i} {
  label .filter.info.frame.label${i} \
    -background "$GuiColor(label_bg)" \
	 -width {5} \
    -text $i
  pack append .filter.info.frame \
    .filter.info.frame.label${i} {left frame center expand} 
  }

  .filter.info.frame.label0 configure -text Mask
  pack append .filter.info \
    .filter.info.frame {top frame center expand fill} 

	if { $lp == "gaussian" } {
		set temp [cvip h_image 3 $mat_dim $mat_dim]
		set llist [cvip data_get $temp]; cvip dq $temp
	} elseif {$lp == "center-weight-constant" } {
		set temp [cvip h_image 2 $mat_dim $mat_dim]
		set llist [cvip data_get $temp]; cvip dq $temp
	}

  for {set i 0} {$i < $mat_dim} {incr i} {
  # build widget .filter.info.frame$i
  frame .filter.info.frame$i \
    -background "$GuiColor(frame_bg)"  \
    -borderwidth {2}

  label .filter.info.frame${i}.label${i} \
    -background "$GuiColor(label_bg)" \
	 -width {5} \
    -text [expr $i + 1]
  pack append .filter.info.frame$i \
	 .filter.info.frame${i}.label${i} {left frame center fill expand}
  for {set j 0} {$j < $mat_dim} {incr j} {

  # build widget .filter.info.frame${i}.entry${j}
  if { $lp == "low-pass" } { set util(filter_${i}_${j}) 1 }  \
  elseif { $lp == "high-pass" } {set util(filter_${i}_${j}) -1 } \
  else {set temp [expr $i*$mat_dim + $j + 2]; set util(filter_${i}_${j}) [lindex $llist $temp] } 
  entry .filter.info.frame${i}.entry${j} \
    -background "$GuiColor(entry_bg)" \
	 -foreground "$GuiColor(entry_fg)" \
    -borderwidth {1} \
    -relief {sunken} \
    -textvariable "util(filter_${i}_${j})" \
    -width {4}
  if { $j == $mat_dim - 1 } {
	if { $i == $mat_dim -1 } {
		endEntry .filter.info.frame${i}.entry${j} 0
	} else {
		set temp [expr $i + 1]
		endEntry .filter.info.frame${i}.entry${j} 0 .filter.info.frame${temp}.entry0
	}
  } else {
		set temp [expr $j + 1]
		endEntry .filter.info.frame${i}.entry${j} 0 .filter.info.frame${i}.entry$temp
  }

  if {$util(filter_${i}_${j}) == ""}  {set util(filter_${i}_${j}) "1"}
  # pack widget .filter.info.frame0
  pack append .filter.info.frame${i} \
    .filter.info.frame${i}.entry${j} {left frame center expand fill}
  }
#end of j
  # pack widget .filter.info
  pack append .filter.info \
    .filter.info.frame$i {top frame center expand fill}
  }
  endEntry .filter.info.frame[expr ${i}-1].entry[expr ${j}-1] 0 .filter.info.frame0.entry0
#end of i
	if { $lp == "high-pass" } {
		set temp "set util(filter_[expr int(floor($mat_dim /2))]_[expr int(floor($mat_dim /2))])\
		[expr $mat_dim * $mat_dim]"
		eval $temp
	} elseif { $lp == "gaussian" || $lp == "center-weight-constant"} { unset llist }
  # build widget .filter.perform
  frame .filter.perform \
    -background "$GuiColor(frame_bg)"  \
	 -relief {raised} 

  # build widget .filter.perform.perform1
	button .filter.perform.perform1 -padx {1} -pady {1} -highlightthickness {0}  \
    -foreground "$GuiColor(perf_fg)" \
    -background "$GuiColor(perf_bg)" \
	 -activebackground "$GuiColor(perf_act_bg)" \
    -command {
	 $topwin(util_name) configure -cursor watch; update
	 set util(filter_val) "\{"
    for {set i 0} {$i < $util(filter_dim)} {incr i} {
    	for {set j 0} {$j < $util(filter_dim)} {incr j} {
	      if [catch "expr $util(filter_${i}_${j})"] {
				CvipAlert "Numerical values required for the matrix element:\n\
				M(${i}, ${j}) = $util(filter_${i}_${j})"
				set util(filter_val) ""; break
			} else {
			append util(filter_val) "$util(filter_${i}_${j}) "
#			unset util(filter_${i}_${j})
			}
		}
		if {$util(filter_val) == ""} break
	 }
	 if {$util(filter_val) != ""} {
		 append util(filter_val) "\}"
	 }
		set new_name ""
      if { $current_image == ""} {
         CvipAlert "Please select an image first."
			$topwin(util_name) configure -cursor top_left_arrow; update
         return
      } elseif {$util(filter_val) == "" } {
         CvipAlert "Please specify the mask matrix first."
			$topwin(util_name) configure -cursor top_left_arrow; update
         return
		}
		if [catch {set result [eval "cvip specify_spatial $current_image $util(filter_dim) $util(filter_dim) $util(filter_val)"]} cresult] {
        CvipAlert "$cresult."
			$topwin(util_name) configure -cursor top_left_arrow; update
        return
		}
     set new_name [cvip rename $result ${current_image}_filter]       
     img_q insert end $new_name
     image_post $new_name
     cvip view $new_name
	  CvipMessage "specify spatial filter performed on image \"$current_image\""
	$topwin(util_name) configure -cursor top_left_arrow; update
	} \
    -text {PERFORM}

  # build widget .filter.perform.dismiss
	button .filter.perform.dismiss -padx {1} -pady {1} -highlightthickness {0}  \
    -foreground "$GuiColor(dis_fg)" \
    -background "$GuiColor(dis_bg)" \
	 -activebackground "$GuiColor(dis_act_bg)" \
    -command {set util(filter_val) "";  destroy .filter} \
    -text {DISMISS} \
    -width {10}

  # pack widget .filter.perform
  pack append .filter.perform \
    .filter.perform.dismiss {left frame sw fill} \
    .filter.perform.perform1 {right frame se fill}

  # pack widget .filter
  pack append .filter \
    .filter.info {top frame center expand fill} \
    .filter.perform {bottom frame center pady 5 fill}

}

# procedure to show window $topwin(util_name)
proc util_blur { } {# xf ignore me 7
  global GuiColor
  global GuiMisc
  global GUI_DEBUG
  # the variable temp_var is used to find whether edit_mask has been performed.if the edit_mask button is clicked,the variable temp_var is set to 1 otherwise the temp_var variable is set to 0
  global temp_var

  # Window manager configurations
  global tk_version
  global topwin
  global {util}
  global toplevel_position
  if [ winfo exists $topwin(util_name).info] {
     set tempname $topwin(util_name);\
destroy $topwin(util_name).info;\
set topwin(util_name) $tempname;}
  if [ winfo exists $topwin(util_name).current_image] {
     set tempname $topwin(util_name);\
destroy $topwin(util_name).current_image;\
set topwin(util_name) $tempname;}
  if [ winfo exists $topwin(util_name).perform] {
     set tempname $topwin(util_name);\
destroy $topwin(util_name).perform;\
set topwin(util_name) $tempname;}
  wm positionfrom $topwin(util_name) user
  wm sizefrom $topwin(util_name) user
  wm geometry $topwin(util_name) $topwin(util_top)[get_pos $topwin(util_name)]
  wm maxsize $topwin(util_name) [lindex [split $topwin(util_top_max)] 0] \
  	[lindex [split $topwin(util_top_max)] 1]
  wm minsize $topwin(util_name) [lindex [split $topwin(util_top_min)] 0] \
  	[lindex [split $topwin(util_top_min)] 1]
  wm title $topwin(util_name) {UTILITIES -> Filter -> Specify a blur}
  set temp_var 0

# bindings
  bind $topwin(util_name) <Destroy> {if [ winfo exists .filter] {destroy .filter}; set topwin(util_name) NONE}

# build widget $topwin(util_name).info
  frame $topwin(util_name).info\
    -background "$GuiColor(frame_bg)"  \
    -borderwidth {2}

  # build widget $topwin(util_name).info.frame10
  frame $topwin(util_name).info.frame10 \
    -background "$GuiColor(frame_bg)"  \
	 -borderwidth {0}

  # build widget $topwin(util_name).info.frame10.frame0
  frame $topwin(util_name).info.frame10.frame0 \
    -background "$GuiColor(frame_bg)"  \
    -borderwidth {2}

  # build widget $topwin(util_name).info.frame10.frame0.frame0
  frame $topwin(util_name).info.frame10.frame0.frame0 \
    -background "$GuiColor(frame_bg)"  \
    -borderwidth {2} \
    -height {1} \
    -width {75}

  # build widget $topwin(util_name).info.frame10.frame0.frame0.label1
  label $topwin(util_name).info.frame10.frame0.frame0.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -text {Blur method:} \
    -width {12}

  # build widget $topwin(util_name).info.frame10.frame0.frame0.value
  entry $topwin(util_name).info.frame10.frame0.frame0.value \
    -background "$GuiColor(rentry_bg)" \
	 -foreground "$GuiColor(rentry_fg)" \
    -relief {sunken} \
    -state {disabled} \
    -textvariable {util(blur_weight)} \
    -width {23}

  # build widget $topwin(util_name).info.frame10.frame0.frame0.menubutton2
	menubutton $topwin(util_name).info.frame10.frame0.frame0.menubutton2 -padx {1} -pady {1} -highlightthickness {0}  \
    -background "$GuiColor(btn_bg)" \
	 -activebackground "$GuiColor(btn_act_bg)" \
    -font "$GuiMisc(pop_up_fn)" \
	 -text "$GuiMisc(pop_up_text)" \
	 -relief "$GuiMisc(pop_up_relief)" \
	 -width {2}
  # bindings
  bind $topwin(util_name).info.frame10.frame0.frame0.menubutton2 <Button-1> {MenuPopupPost $topwin(util_name).info.frame10.frame0.frame0.menubutton2.menu %X %Y}
  bind $topwin(util_name).info.frame10.frame0.frame0.menubutton2 <ButtonRelease-1> {MenuPopupRelease $topwin(util_name).info.frame10.frame0.frame0.menubutton2.menu %W}

  # build widget $topwin(util_name).info.frame10.frame0.frame0.menubutton2.menu
  menu $topwin(util_name).info.frame10.frame0.frame0.menubutton2.menu \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)"
  $topwin(util_name).info.frame10.frame0.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)" \
    -command {pack forget $topwin(util_name).info.frame10.frame0.frame29
pack forget $topwin(util_name).info.frame10.frame0.frame3
set util(blur_weight) Constant
set util(param3) 10
set util(param3_val) 1; set util(filter_val) ""
} \
    -label {Constant}
  $topwin(util_name).info.frame10.frame0.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)" \
    -command {
pack forget $topwin(util_name).info.frame10.frame0.frame29
pack forget $topwin(util_name).info.frame10.frame0.frame3 
set util(blur_weight) {Center-weighted constant}
set util(param3_val) 2; set util(filter_val) ""
} \
    -label {Center-weighted constant}
  $topwin(util_name).info.frame10.frame0.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)" \
    -command {pack forget $topwin(util_name).info.frame10.frame0.frame29 -side left
pack forget $topwin(util_name).info.frame10.frame0.frame3
set util(blur_weight) Gaussian; set util(filter_val) ""
set util(blur_weight_val) 3} \
    -label {Gaussian}
  bindtags $topwin(util_name).info.frame10.frame0.frame0.menubutton2.menu {Menu all}
  # bindings
  bind $topwin(util_name).info.frame10.frame0.frame0.menubutton2.menu <B1-Motion> {MenuPopupMotion $topwin(util_name).info.frame10.frame0.frame0.menubutton2.menu %W %X %Y}
  bind $topwin(util_name).info.frame10.frame0.frame0.menubutton2.menu <ButtonRelease-1> {MenuPopupRelease $topwin(util_name).info.frame10.frame0.frame0.menubutton2.menu %W}

  # pack widget $topwin(util_name).info.frame10.frame0.frame0
  pack append $topwin(util_name).info.frame10.frame0.frame0 \
    $topwin(util_name).info.frame10.frame0.frame0.label1 {left frame center} \
    $topwin(util_name).info.frame10.frame0.frame0.value {left frame center} \
    $topwin(util_name).info.frame10.frame0.frame0.menubutton2 {right frame center}

  # build widget $topwin(util_name).info.frame10.frame0.frame29
  frame $topwin(util_name).info.frame10.frame0.frame29 \
    -background "$GuiColor(frame_bg)"  \
    -borderwidth {2} \
    -height {1} \
    -width {75}

  # build widget $topwin(util_name).info.frame10.frame0.frame29.label1
  label $topwin(util_name).info.frame10.frame0.frame29.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -text {  Mask: }

  # build widget $topwin(util_name).info.frame10.frame0.frame29.value
  entry $topwin(util_name).info.frame10.frame0.frame29.value \
    -background "$GuiColor(entry_bg)" \
	 -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
    -textvariable {util(degrade_mask)} \
    -width {3}

  # pack widget $topwin(util_name).info.frame10.frame0.frame29
  pack append $topwin(util_name).info.frame10.frame0.frame29 \
    $topwin(util_name).info.frame10.frame0.frame29.label1 {left frame center} \
    $topwin(util_name).info.frame10.frame0.frame29.value {left frame center}

  # build widget $topwin(util_name).info.frame10.frame0.frame30
  frame $topwin(util_name).info.frame10.frame0.frame30 \
    -background "$GuiColor(frame_bg)"  \
    -borderwidth {2} \
    -height {1} \
    -width {75}

  # build widget $topwin(util_name).info.frame10.frame0.frame30.label1
  label $topwin(util_name).info.frame10.frame0.frame30.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -text { Weight: }

  # build widget $topwin(util_name).info.frame10.frame0.frame30.value
  entry $topwin(util_name).info.frame10.frame0.frame30.value \
    -background "$GuiColor(entry_bg)" \
	 -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
    -textvariable {util(param4)} \
    -width {7}
	bind $topwin(util_name).info.frame10.frame0.frame30.value \
	<KeyRelease>  { set util(filter_val) ""}

  # build widget $topwin(util_name).info.frame10.frame0.frame30.menubutton2
	menubutton $topwin(util_name).info.frame10.frame0.frame30.menubutton2 -padx {1} -pady {1} -highlightthickness {0}  \
    -background "$GuiColor(btn_bg)" \
	 -activebackground "$GuiColor(btn_act_bg)" \
    -font "$GuiMisc(pop_up_fn)" \
	 -text "$GuiMisc(pop_up_text)" \
	 -relief "$GuiMisc(pop_up_relief)" \
	 -width {2}
  # bindings
  bind $topwin(util_name).info.frame10.frame0.frame30.menubutton2 <Button-1> {MenuPopupPost $topwin(util_name).info.frame10.frame0.frame30.menubutton2.menu %X %Y}
  bind $topwin(util_name).info.frame10.frame0.frame30.menubutton2 <ButtonRelease-1> {MenuPopupRelease $topwin(util_name).info.frame10.frame0.frame30.menubutton2.menu %W}

  # build widget $topwin(util_name).info.frame10.frame0.frame30.menubutton2.menu
  menu $topwin(util_name).info.frame10.frame0.frame30.menubutton2.menu \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)"
  $topwin(util_name).info.frame10.frame0.frame30.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)" \
    -command {set util(param4) 100.0; set util(filter_val) ""} \
    -label {100.0}
  $topwin(util_name).info.frame10.frame0.frame30.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)" \
    -command {set util(param4) 10.0; set util(filter_val) ""} \
    -label {10.00}
  $topwin(util_name).info.frame10.frame0.frame30.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)" \
    -command {set util(param4) 1.0; set util(filter_val) ""} \
    -label {1.000}
  $topwin(util_name).info.frame10.frame0.frame30.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)" \
    -command {set util(param4) 0.1; set util(filter_val) "" } \
    -label {0.100}
  $topwin(util_name).info.frame10.frame0.frame30.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)" \
    -command {set util(param4) 0.01; set util(filter_val) "" } \
    -label {0.010}
  $topwin(util_name).info.frame10.frame0.frame30.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)" \
    -command {set util(param4) 0.001; set util(filter_val) "" } \
    -label {0.001}
  bindtags $topwin(util_name).info.frame10.frame0.frame30.menubutton2.menu {Menu all}
  # bindings
  bind $topwin(util_name).info.frame10.frame0.frame30.menubutton2.menu <B1-Motion> {MenuPopupMotion $topwin(util_name).info.frame10.frame0.frame30.menubutton2.menu0 %W %X %Y}
  bind $topwin(util_name).info.frame10.frame0.frame30.menubutton2.menu <ButtonRelease-1> {MenuPopupRelease $topwin(util_name).info.frame10.frame0.frame30.menubutton2.menu0 %W}

  # pack widget $topwin(util_name).info.frame10.frame0.frame30
  pack append $topwin(util_name).info.frame10.frame0.frame30 \
    $topwin(util_name).info.frame10.frame0.frame30.label1 {left frame center fill} \
    $topwin(util_name).info.frame10.frame0.frame30.value {left frame center fill} \
    $topwin(util_name).info.frame10.frame0.frame30.menubutton2 {right frame center fill}

  # build widget $topwin(util_name).info.frame10.frame0.frame3
  frame $topwin(util_name).info.frame10.frame0.frame3 \
    -background "$GuiColor(frame_bg)"  \
    -borderwidth {2} \
    -height {1} \
    -width {75}

  # build widget $topwin(util_name).info.frame10.frame0.frame3.label1
  label $topwin(util_name).info.frame10.frame0.frame3.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -text { Stdev: }

  # build widget $topwin(util_name).info.frame10.frame0.frame3.value
  entry $topwin(util_name).info.frame10.frame0.frame3.value \
    -background "$GuiColor(entry_bg)" \
	 -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
    -textvariable {util(degrade_stdev)} \
    -width {3}

  # pack widget $topwin(util_name).info.frame10.frame0.frame3
  pack append $topwin(util_name).info.frame10.frame0.frame3 \
    $topwin(util_name).info.frame10.frame0.frame3.label1 {left frame center} \
    $topwin(util_name).info.frame10.frame0.frame3.value {left frame center}

  # pack widget $topwin(util_name).info.frame10.frame0
  pack append $topwin(util_name).info.frame10.frame0 \
    $topwin(util_name).info.frame10.frame0.frame0 {left frame w} \
    $topwin(util_name).info.frame10.frame0.frame30 {left frame w} 
#    $topwin(util_name).info.frame10.frame0.frame29 {left frame center}

  # build widget $topwin(util_name).info.frame10.frame12
  frame $topwin(util_name).info.frame10.frame12 \
    -background "$GuiColor(frame_bg)"  \
    -borderwidth {2}

  # build widget $topwin(util_name).info.frame10.frame12.frame0
  frame $topwin(util_name).info.frame10.frame12.frame0 \
    -background "$GuiColor(frame_bg)"  \
    -borderwidth {2} \
    -height {1} \
    -width {75}

  # build widget $topwin(util_name).info.frame10.frame12.frame0.label1
  label $topwin(util_name).info.frame10.frame12.frame0.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -text {Blur shape:  } \
    -width {12}

  # build widget $topwin(util_name).info.frame10.frame12.frame0.value
  entry $topwin(util_name).info.frame10.frame12.frame0.value \
    -background "$GuiColor(rentry_bg)" \
	 -foreground "$GuiColor(rentry_fg)" \
    -relief {sunken} \
    -state {disabled} \
    -textvariable {util(degrade_shape)} \
    -width {15}

  # build widget $topwin(util_name).info.frame10.frame12.frame0.menubutton2
	menubutton $topwin(util_name).info.frame10.frame12.frame0.menubutton2 -padx {1} -pady {1} -highlightthickness {0}  \
    -background "$GuiColor(btn_bg)" \
	 -activebackground "$GuiColor(btn_act_bg)" \
    -font "$GuiMisc(pop_up_fn)" \
	 -text "$GuiMisc(pop_up_text)" \
	 -relief "$GuiMisc(pop_up_relief)" \
	 -width {2}
  # bindings
  bind $topwin(util_name).info.frame10.frame12.frame0.menubutton2 <Button-1> {MenuPopupPost $topwin(util_name).info.frame10.frame12.frame0.menubutton2.menu %X %Y}
  bind $topwin(util_name).info.frame10.frame12.frame0.menubutton2 <ButtonRelease-1> {MenuPopupRelease $topwin(util_name).info.frame10.frame12.frame0.menubutton2.menu %W}

  # build widget $topwin(util_name).info.frame10.frame12.frame0.menubutton2.menu
  menu $topwin(util_name).info.frame10.frame12.frame0.menubutton2.menu \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)"
  $topwin(util_name).info.frame10.frame12.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)" \
    -command {pack  $topwin(util_name).info.frame10.frame12.frame7 -side left
# pack  $topwin(util_name).info.frame10.frame12.frame29 -side left
# pack  $topwin(util_name).info.frame10.frame12.frame9 -side left
pack forget $topwin(util_name).info.frame10.frame12.frame11
pack forget $topwin(util_name).info.frame10.frame12.frame12
pack forget $topwin(util_name).info.frame10.frame12.frame1 
set util(degrade_shape) circle; set util(filter_val) "" } \
    -label {circle} \
    -state {active}
  $topwin(util_name).info.frame10.frame12.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)" \
    -command {pack forget $topwin(util_name).info.frame10.frame12.frame7
pack forget $topwin(util_name).info.frame10.frame12.frame29
pack forget $topwin(util_name).info.frame10.frame12.frame9
pack  $topwin(util_name).info.frame10.frame12.frame11 -side left
pack  $topwin(util_name).info.frame10.frame12.frame12 -side left
pack  forget $topwin(util_name).info.frame10.frame12.frame1 
set util(degrade_shape) rectangle
set util(param1) 5
set util(param2) 1; set util(filter_val) "" 

} \
    -label {rectangle}
  $topwin(util_name).info.frame10.frame12.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)" \
    -command {pack forget $topwin(util_name).info.frame10.frame12.frame7
pack forget $topwin(util_name).info.frame10.frame12.frame29
pack forget $topwin(util_name).info.frame10.frame12.frame9
pack forget  $topwin(util_name).info.frame10.frame12.frame11
pack forget  $topwin(util_name).info.frame10.frame12.frame12
pack forget  $topwin(util_name).info.frame10.frame12.frame1
set util(degrade_shape) "Horizontal line"
set util(filter_val) "" } \
    -label {Horizontal line}
  $topwin(util_name).info.frame10.frame12.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)" \
    -command {pack forget $topwin(util_name).info.frame10.frame12.frame7
pack forget $topwin(util_name).info.frame10.frame12.frame29
pack forget $topwin(util_name).info.frame10.frame12.frame9
pack forget  $topwin(util_name).info.frame10.frame12.frame11
pack forget  $topwin(util_name).info.frame10.frame12.frame12
pack forget  $topwin(util_name).info.frame10.frame12.frame1
set util(degrade_shape) "Vertical line"; set util(filter_val) "" 
} \
    -label {Vertical line}
  $topwin(util_name).info.frame10.frame12.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)" \
    -command {pack forget $topwin(util_name).info.frame10.frame12.frame7
pack forget $topwin(util_name).info.frame10.frame12.frame29
pack forget $topwin(util_name).info.frame10.frame12.frame9
pack forget  $topwin(util_name).info.frame10.frame12.frame11
pack forget  $topwin(util_name).info.frame10.frame12.frame12
pack forget  $topwin(util_name).info.frame10.frame12.frame1
set util(degrade_shape) "Slash"; set util(filter_val) ""
} \
    -label {Slash}
  $topwin(util_name).info.frame10.frame12.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)" \
    -command {pack forget $topwin(util_name).info.frame10.frame12.frame7
pack forget $topwin(util_name).info.frame10.frame12.frame29
pack forget $topwin(util_name).info.frame10.frame12.frame9
pack forget  $topwin(util_name).info.frame10.frame12.frame11
pack forget  $topwin(util_name).info.frame10.frame12.frame12
pack forget  $topwin(util_name).info.frame10.frame12.frame1
set util(degrade_shape) "Backslash"; set util(filter_val) ""
} \
    -label {Backslash}
  # bindings
  bind $topwin(util_name).info.frame10.frame12.frame0.menubutton2.menu <B1-Motion> {MenuPopupMotion $topwin(util_name).info.frame10.frame12.frame0.menubutton2.menu %W %X %Y}
  bind $topwin(util_name).info.frame10.frame12.frame0.menubutton2.menu <ButtonRelease-1> {MenuPopupRelease $topwin(util_name).info.frame10.frame12.frame0.menubutton2.menu %W}

  # pack widget $topwin(util_name).info.frame10.frame12.frame0
  pack append $topwin(util_name).info.frame10.frame12.frame0 \
    $topwin(util_name).info.frame10.frame12.frame0.label1 {left frame center} \
    $topwin(util_name).info.frame10.frame12.frame0.value {left frame center} \
    $topwin(util_name).info.frame10.frame12.frame0.menubutton2 {right frame center}
  bindtags $topwin(util_name).info.frame10.frame12.frame0.menubutton2.menu  {Menu all}
  # bindings
  bind $topwin(util_name).info.frame10.frame12.frame0.menubutton2.menu <B1-Motion> {MenuPopupMotion $topwin(util_name).info.frame10.frame12.frame0.menubutton2.menu %W %X %Y}
  bind $topwin(util_name).info.frame10.frame12.frame0.menubutton2.menu <ButtonRelease-1> {MenuPopupRelease $topwin(util_name).info.frame10.frame12.frame0.menubutton2.menu %W}

  # pack widget $topwin(util_name).info.frame10.frame12.frame0
  pack append $topwin(util_name).info.frame10.frame12.frame0 \
    $topwin(util_name).info.frame10.frame12.frame0.label1 {left frame center} \
    $topwin(util_name).info.frame10.frame12.frame0.value {left frame center} \
    $topwin(util_name).info.frame10.frame12.frame0.menubutton2 {right frame center}

  # build widget $topwin(util_name).info.frame10.frame12.frame29
  frame $topwin(util_name).info.frame10.frame12.frame29 \
    -background "$GuiColor(frame_bg)"  \
    -borderwidth {2} \
    -height {1} \
    -width {75}

  # build widget $topwin(util_name).info.frame10.frame12.frame29.label1
  label $topwin(util_name).info.frame10.frame12.frame29.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -text {Angle1:} \
    -width {8}

  # build widget $topwin(util_name).info.frame10.frame12.frame29.value
  entry $topwin(util_name).info.frame10.frame12.frame29.value \
    -background "$GuiColor(entry_bg)" \
	 -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
    -textvariable {util(degrade_ang1)} \
    -width {3}

  # pack widget $topwin(util_name).info.frame10.frame12.frame29
  pack append $topwin(util_name).info.frame10.frame12.frame29 \
    $topwin(util_name).info.frame10.frame12.frame29.label1 {left frame center expand fill} \
    $topwin(util_name).info.frame10.frame12.frame29.value {left frame center expand fill}

  # build widget $topwin(util_name).info.frame10.frame12.frame7
  frame $topwin(util_name).info.frame10.frame12.frame7 \
    -background "$GuiColor(frame_bg)"  \
    -borderwidth {2} \
    -height {1} \
    -width {75}

  # build widget $topwin(util_name).info.frame10.frame12.frame7.label1
  label $topwin(util_name).info.frame10.frame12.frame7.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -text {Diameter:} \
    -width {8}

  # build widget $topwin(util_name).info.frame10.frame12.frame7.value
  entry $topwin(util_name).info.frame10.frame12.frame7.value \
    -background "$GuiColor(entry_bg)" \
	 -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
    -textvariable {util(degrade_rad)} \
    -width {3}

	bind $topwin(util_name).info.frame10.frame12.frame7.value \
	<KeyRelease>  { set util(filter_val) "" }

  # pack widget $topwin(util_name).info.frame10.frame12.frame7
  pack append $topwin(util_name).info.frame10.frame12.frame7 \
    $topwin(util_name).info.frame10.frame12.frame7.label1 {left frame center expand fill} \
    $topwin(util_name).info.frame10.frame12.frame7.value {left frame center expand fill}

  # build widget $topwin(util_name).info.frame10.frame12.frame9
  frame $topwin(util_name).info.frame10.frame12.frame9 \
    -background "$GuiColor(frame_bg)"  \
    -borderwidth {2} \
    -height {1} \
    -width {75}

  # build widget $topwin(util_name).info.frame10.frame12.frame9.label1
  label $topwin(util_name).info.frame10.frame12.frame9.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -text {Angle2:} \
    -width {8}

  # build widget $topwin(util_name).info.frame10.frame12.frame9.value
  entry $topwin(util_name).info.frame10.frame12.frame9.value \
    -background "$GuiColor(entry_bg)" \
	 -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
    -textvariable {util(degrade_ang2)} \
    -width {3}

  # pack widget $topwin(util_name).info.frame10.frame12.frame9
  pack append $topwin(util_name).info.frame10.frame12.frame9 \
    $topwin(util_name).info.frame10.frame12.frame9.label1 {left frame center expand fill} \
    $topwin(util_name).info.frame10.frame12.frame9.value {left frame center expand fill}

  # build widget $topwin(util_name).info.frame10.frame12.frame11
  frame $topwin(util_name).info.frame10.frame12.frame11 \
    -background "$GuiColor(frame_bg)"  \
    -borderwidth {2} \
    -height {1} \
    -width {75}

  # build widget $topwin(util_name).info.frame10.frame12.frame11.label1
  label $topwin(util_name).info.frame10.frame12.frame11.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -text {Height:} \
    -width {8}

  # build widget $topwin(util_name).info.frame10.frame12.frame11.value
  entry $topwin(util_name).info.frame10.frame12.frame11.value \
    -background "$GuiColor(entry_bg)" \
	 -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
    -textvariable {util(degrade_height)} \
    -width {3}
	bind $topwin(util_name).info.frame10.frame12.frame11.value \
	<KeyRelease>  { set util(filter_val) "" }

  # pack widget $topwin(util_name).info.frame10.frame12.frame11
  pack append $topwin(util_name).info.frame10.frame12.frame11 \
    $topwin(util_name).info.frame10.frame12.frame11.label1 {left frame center expand fill} \
    $topwin(util_name).info.frame10.frame12.frame11.value {left frame center expand fill}

  # build widget $topwin(util_name).info.frame10.frame12.frame12
  frame $topwin(util_name).info.frame10.frame12.frame12 \
    -background "$GuiColor(frame_bg)"  \
    -borderwidth {2} \
    -height {1} \
    -width {75}

  # build widget $topwin(util_name).info.frame10.frame12.frame12.label1
  label $topwin(util_name).info.frame10.frame12.frame12.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -text {Width:} \
    -width {5}

  # build widget $topwin(util_name).info.frame10.frame12.frame12.value
  entry $topwin(util_name).info.frame10.frame12.frame12.value \
    -background "$GuiColor(entry_bg)" \
	 -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
    -textvariable {util(degrade_width)} \
    -width {3}
	bind $topwin(util_name).info.frame10.frame12.frame12.value \
	<KeyRelease>  { set util(filter_val) "" }

  # pack widget $topwin(util_name).info.frame10.frame12.frame12
  pack append $topwin(util_name).info.frame10.frame12.frame12 \
    $topwin(util_name).info.frame10.frame12.frame12.label1 {left frame center expand fill} \
    $topwin(util_name).info.frame10.frame12.frame12.value {left frame center expand fill}

  # build widget $topwin(util_name).info.frame10.frame12.frame1
  frame $topwin(util_name).info.frame10.frame12.frame1 \
    -background "$GuiColor(frame_bg)"  \
    -borderwidth {2} \
    -height {1} \
    -width {75}

  # build widget $topwin(util_name).info.frame10.frame12.frame1.label1
  label $topwin(util_name).info.frame10.frame12.frame1.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -text {Thickness:} 

  # build widget $topwin(util_name).info.frame10.frame12.frame1.value
  entry $topwin(util_name).info.frame10.frame12.frame1.value \
    -background "$GuiColor(entry_bg)" \
	 -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
    -textvariable {util(degrade_ang1)} \
    -width {3}

  # pack widget $topwin(util_name).info.frame10.frame12.frame1
  pack append $topwin(util_name).info.frame10.frame12.frame1 \
    $topwin(util_name).info.frame10.frame12.frame1.label1 {left frame center expand fill} \
    $topwin(util_name).info.frame10.frame12.frame1.value {left frame center expand fill}

  # pack widget $topwin(util_name).info.frame10.frame12
  pack append $topwin(util_name).info.frame10.frame12 \
    $topwin(util_name).info.frame10.frame12.frame0 {left frame w} \
    $topwin(util_name).info.frame10.frame12.frame11 {left frame center} \
    $topwin(util_name).info.frame10.frame12.frame12 {left frame center} 
#    $topwin(util_name).info.frame10.frame12.frame1 {left frame center}

	pack append $topwin(util_name).info.frame10 \
	$topwin(util_name).info.frame10.frame12 {top frame w}\
	$topwin(util_name).info.frame10.frame0 {top frame w}

  # build widget $topwin(util_name).info.frame0
  frame $topwin(util_name).info.frame0 \
    -background "$GuiColor(frame_bg)"  \
    -borderwidth {0} \
    -width {1} \
    -width {30} -height {0}

  # build widget $topwin(util_name).info.frame1
  frame $topwin(util_name).info.frame1 \
    -background "$GuiColor(frame_bg)"  \
    -borderwidth {2} \
	 -relief {groove}

  # build widget $topwin(util_name).info.frame1.frame0
  frame $topwin(util_name).info.frame1.frame0 \
    -background "$GuiColor(frame_bg)" 
  # bindings
  bind $topwin(util_name).info.frame1.frame0 <Button-1> {MenuPopupPost $topwin(util_name).info.frame1.frame0.menu %X %Y}
  bind $topwin(util_name).info.frame1.frame0 <ButtonRelease-1> {MenuPopupRelease $topwin(util_name).info.frame1.frame0.menu %W}

  # build widget $topwin(util_name).info.frame1.frame0.label1
  label $topwin(util_name).info.frame1.frame0.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -text { Mask height: }

  # build widget $topwin(util_name).info.frame1.frame0.entry2
  entry $topwin(util_name).info.frame1.frame0.entry2 \
    -background "$GuiColor(entry_bg)" \
	 -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
    -textvariable {util(msk_height)} \
    -width {6}
	bind $topwin(util_name).info.frame1.frame0.entry2 \
	<KeyRelease>  { set util(filter_val) "" }

  # build widget $topwin(util_name).info.frame1.frame0.menubutton3
	menubutton $topwin(util_name).info.frame1.frame0.menubutton3 -padx {1} -pady {1} -highlightthickness {0}  \
    -background "$GuiColor(btn_bg)" \
	 -activebackground "$GuiColor(btn_act_bg)" \
    -font "$GuiMisc(pop_up_fn)" \
	 -text "$GuiMisc(pop_up_text)" \
	 -relief "$GuiMisc(pop_up_relief)" \
	 -width {2}
  # bindings
  bind $topwin(util_name).info.frame1.frame0.menubutton3 <Button-1> {MenuPopupPost $topwin(util_name).info.frame1.frame0.menubutton3.menu %X %Y}
  bind $topwin(util_name).info.frame1.frame0.menubutton3 <ButtonRelease-1> {MenuPopupRelease $topwin(util_name).info.frame1.frame0.menubutton3.menu %W}

  # build widget $topwin(util_name).info.frame1.frame0.menubutton3.menu
  menu $topwin(util_name).info.frame1.frame0.menubutton3.menu \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)"
  $topwin(util_name).info.frame1.frame0.menubutton3.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)" \
    -command {set util(msk_height) 3; set util(filter_val) "" } \
    -label {  3 }
  $topwin(util_name).info.frame1.frame0.menubutton3.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)" \
    -command {set util(msk_height) 5; set util(filter_val) "" }\
    -label {  5 }
  $topwin(util_name).info.frame1.frame0.menubutton3.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)" \
    -command {set util(msk_height) 7; set util(filter_val) "" }\
    -label {  7 }
  $topwin(util_name).info.frame1.frame0.menubutton3.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)" \
    -command {set util(msk_height) 9; set util(filter_val) "" }\
    -label {  9 }
  $topwin(util_name).info.frame1.frame0.menubutton3.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)" \
    -command {set util(msk_height) 11; set util(filter_val) "" }\
    -label { 11 }
  $topwin(util_name).info.frame1.frame0.menubutton3.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)" \
    -command {set util(msk_height) 13; set util(filter_val) "" }\
    -label { 13 }
  bindtags $topwin(util_name).info.frame1.frame0.menubutton3.menu  {Menu all}
  # bindings
  bind $topwin(util_name).info.frame1.frame0.menubutton3.menu <B1-Motion> {MenuPopupMotion $topwin(util_name).info.frame1.frame0.menubutton3.menu0 %W %X %Y}
  bind $topwin(util_name).info.frame1.frame0.menubutton3.menu <ButtonRelease-1> {MenuPopupRelease $topwin(util_name).info.frame1.frame0.menubutton3.menu0 %W}

  # pack widget $topwin(util_name).info.frame1.frame0
  pack append $topwin(util_name).info.frame1.frame0 \
    $topwin(util_name).info.frame1.frame0.label1 {left frame center} \
    $topwin(util_name).info.frame1.frame0.entry2 {left frame center} \
    $topwin(util_name).info.frame1.frame0.menubutton3 {left frame center}

  # build widget $topwin(util_name).info.frame1.frame4
  frame $topwin(util_name).info.frame1.frame4 \
    -background "$GuiColor(frame_bg)" 
  # bindings
  bind $topwin(util_name).info.frame1.frame4 <Button-1> {MenuPopupPost $topwin(util_name).info.frame1.frame4.menu %X %Y}
  bind $topwin(util_name).info.frame1.frame4 <ButtonRelease-1> {MenuPopupRelease $topwin(util_name).info.frame1.frame4.menu %W}

  # build widget $topwin(util_name).info.frame1.frame4.label1
  label $topwin(util_name).info.frame1.frame4.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -text { Mask width: }

  # build widget $topwin(util_name).info.frame1.frame4.entry2
  entry $topwin(util_name).info.frame1.frame4.entry2 \
    -background "$GuiColor(entry_bg)" \
	 -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
    -textvariable {util(msk_width)} \
    -width {6}

	bind $topwin(util_name).info.frame1.frame4.entry2 \
	<KeyRelease>  { set util(filter_val) "" }

  # build widget $topwin(util_name).info.frame1.frame4.menubutton3
	menubutton $topwin(util_name).info.frame1.frame4.menubutton3 -padx {1} -pady {1} -highlightthickness {0}  \
    -background "$GuiColor(btn_bg)" \
	 -activebackground "$GuiColor(btn_act_bg)" \
    -font "$GuiMisc(pop_up_fn)" \
	 -text "$GuiMisc(pop_up_text)" \
	 -relief "$GuiMisc(pop_up_relief)" \
	 -width {2}
  # bindings
  bind $topwin(util_name).info.frame1.frame4.menubutton3 <Button-1> {MenuPopupPost $topwin(util_name).info.frame1.frame4.menubutton3.menu %X %Y}
  bind $topwin(util_name).info.frame1.frame4.menubutton3 <ButtonRelease-1> {MenuPopupRelease $topwin(util_name).info.frame1.frame4.menubutton3.menu %W}

  # build widget $topwin(util_name).info.frame1.frame4.menubutton3.menu
  menu $topwin(util_name).info.frame1.frame4.menubutton3.menu \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)"
  $topwin(util_name).info.frame1.frame4.menubutton3.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)" \
    -command {set util(msk_width) 3; set util(filter_val) "" }\
    -label {  3 }
  $topwin(util_name).info.frame1.frame4.menubutton3.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)" \
    -command {set util(msk_width) 5; set util(filter_val) "" }\
    -label {  5 }
  $topwin(util_name).info.frame1.frame4.menubutton3.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)" \
    -command {set util(msk_width) 7; set util(filter_val) "" }\
    -label {  7 }
  $topwin(util_name).info.frame1.frame4.menubutton3.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)" \
    -command {set util(msk_width) 9; set util(filter_val) "" }\
    -label {  9 }
  $topwin(util_name).info.frame1.frame4.menubutton3.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)" \
    -command {set util(msk_width) 11; set util(filter_val) "" }\
    -label { 11 }
  $topwin(util_name).info.frame1.frame4.menubutton3.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
	 -background "$GuiColor(menu_bg)" \
    -command {set util(msk_width) 13; set util(filter_val) "" }\
    -label { 13 }
  bindtags $topwin(util_name).info.frame1.frame4.menubutton3.menu {Menu all}
  # bindings
  bind $topwin(util_name).info.frame1.frame4.menubutton3.menu <B1-Motion> {MenuPopupMotion $topwin(util_name).info.frame1.frame4.menubutton3.menu0 %W %X %Y}
  bind $topwin(util_name).info.frame1.frame4.menubutton3.menu <ButtonRelease-1> {MenuPopupRelease $topwin(util_name).info.frame1.frame4.menubutton3.menu0 %W}

  # pack widget $topwin(util_name).info.frame1.frame4
  pack append $topwin(util_name).info.frame1.frame4 \
    $topwin(util_name).info.frame1.frame4.label1 {left frame center} \
    $topwin(util_name).info.frame1.frame4.entry2 {left frame center} \
    $topwin(util_name).info.frame1.frame4.menubutton3 {left frame center}

  # build widget $topwin(util_name).info.frame1.button5
	button $topwin(util_name).info.frame1.button5 -padx {1} -pady {1} -highlightthickness {0}  \
    -foreground "$GuiColor(alert_fg)" \
    -background "$GuiColor(alert_bg)" \
	 -activebackground "$GuiColor(alert_act_bg)" \
    -text {Edit mask} \
	 -command {
	set temp_var {1}
      $topwin(util_name) configure -cursor watch; update
		if [catch {expr $util(msk_height) + $util(msk_width) } ] {
         CvipAlert "Numerical values required for the degradation image sizes:\n\
			Height = $util(msk_height), Width = $util(msk_width)."
         $topwin(util_name) configure -cursor top_left_arrow; update
         return
		} elseif { $util(msk_height) < "3" || $util(msk_width) < "3" || \
					  $util(msk_height) > "30" || $util(msk_width) > "30" } {
         CvipAlert "The degradation image size ($util(msk_height)x$util(msk_width)) should \
			be in the range \[3-30\]."
         $topwin(util_name) configure -cursor top_left_arrow; update
         return
		}
		if [catch {expr $util(param4)} ] {
         CvipAlert "Positive Numerical value required for weight parameter:\n\
			Weight = $util(param4)."
         $topwin(util_name) configure -cursor top_left_arrow; update
         return
		} elseif {$util(param4) < 0  } {
         CvipAlert "Positive Numerical value required for weight parameter ($util(param4))"
         $topwin(util_name) configure -cursor top_left_arrow; update
         return
		}

		set temp [expr { $util(msk_height) > $util(msk_width) }\
		? $util(msk_width) : $util(msk_height) ]
#		set temp [expr $temp / 2]

		switch -exact -- $util(degrade_shape) {
			"circle" {
				if [catch {expr $util(degrade_rad) }] {
         		CvipAlert "Numerical values required for the circle diameter:\n\
					Diameter = $util(degrade_rad)."
					$topwin(util_name) configure -cursor top_left_arrow; update
					return
				} elseif { $util(degrade_rad) < 1 || $util(degrade_rad) > $temp } {
         		CvipAlert "The circle diameter ($util(degrade_rad)) should be\
					in the range \[1-$temp\]."
					$topwin(util_name) configure -cursor top_left_arrow; update
					return
				}
				set temp [eval "cvip circle $util(msk_width) $util(msk_height) \
									[expr $util(msk_width)/2] [expr $util(msk_height) /2]\
									[expr $util(degrade_rad) / 2]"]
			}
			"rectangle" {
				if [catch {expr $util(degrade_width) + $util(degrade_height)}] {
         		CvipAlert "Numerical values required for the mask sizes:\n\
					Height = $util(degrade_height), Width = $util(degrade_width)."
					$topwin(util_name) configure -cursor top_left_arrow; update
					return
				} elseif {$util(degrade_width) < 1 || $util(degrade_width) > $util(msk_width) ||
					$util(degrade_height) < 1 || $util(degrade_height) > $util(msk_height) } {
         		CvipAlert "The mask size ($util(degrade_height)x$util(degrade_width)) must be \
					positive and no larger than the mask image size\
					($util(msk_height)x$util(msk_width))."
					$topwin(util_name) configure -cursor top_left_arrow; update
					return
				}
				set temp [eval "cvip rectangle $util(msk_width) $util(msk_height) \
					[expr ($util(msk_width) - $util(degrade_width)) /2] \
					[expr ($util(msk_height) - $util(degrade_height)) /2] \
					$util(degrade_width) $util(degrade_height)"]


			}
			"Horizontal line" {
				set temp [eval "cvip line $util(msk_width) $util(msk_height) 0 \
				[expr $util(msk_height) /2 ] [expr $util(msk_width) - 1] \
				[expr $util(msk_height) /2 ]"]
			}
			"Vertical line" {
				set temp [eval "cvip line $util(msk_width) $util(msk_height) \
				[expr $util(msk_width) / 2] 0 \
				[expr $util(msk_width) / 2] [expr $util(msk_height) - 1]"]
			}
			"Slash" {
				set temp [eval "cvip line $util(msk_width) $util(msk_height) \
				0 [expr $util(msk_height) - 1] [expr $util(msk_width) - 1] 0"]
			}
			"Backslash" {
				set temp [eval "cvip line $util(msk_width) $util(msk_height) \
				0 0 [expr $util(msk_width) - 1] [expr $util(msk_height) - 1]"]
			}
		}

		set temp1 [cvip gray_multiply $temp 0.003922 0]; cvip dq $temp
		set temp $temp1

		switch -exact -- $util(blur_weight) {
			Constant {set temp1 [cvip h_image 1 $util(msk_height) $util(msk_width)]}
			"Center-weighted constant" {set temp1 [cvip h_image 2 $util(msk_height) $util(msk_width)]}
			Gaussian {set temp1 [cvip h_image 3 $util(msk_height) $util(msk_width)]}
		}


		set result [cvip multiply $temp $temp1]; cvip dq $temp; cvip dq $temp1
		set temp $result 
		set util(h_image) [cvip gray_multiply $temp $util(param4) 0]; cvip dq $temp
		mask_specify "1" "[cvip data_get $util(h_image)]"
	 }



  # pack widget $topwin(util_name).info.frame1
  pack append $topwin(util_name).info.frame1 \
    $topwin(util_name).info.frame1.frame0 {left frame center expand fillx} \
    $topwin(util_name).info.frame1.frame4 {left frame center expand fillx} \
    $topwin(util_name).info.frame1.button5 {left frame center expand fillx}
  # pack widget $topwin(util_name).info
  
  pack append $topwin(util_name).info \
     $topwin(util_name).info.frame1 {top frame w fillx} \
     $topwin(util_name).info.frame10 {top frame w}  

   
  
  # build widget $topwin(util_name).perform
  frame $topwin(util_name).perform \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2} \
    -relief {raised}

# build widget $topwin(util_name).perform.perform1
	button $topwin(util_name).perform.perform1 -padx {1} -pady {1} -highlightthickness {0}  \
    -foreground "$GuiColor(perf_fg)" \
    -background "$GuiColor(perf_bg)" \
	 -activebackground "$GuiColor(perf_act_bg)" \
    -text {PERFORM} \
	 -command {
	 	if { $temp_var < 1 } {
	 	  $topwin(util_name) configure -cursor watch; update
		if [catch {expr $util(msk_height) + $util(msk_width) } ] {
         CvipAlert "Numerical values required for the degradation image sizes:\n\
			Height = $util(msk_height), Width = $util(msk_width)."
         $topwin(util_name) configure -cursor top_left_arrow; update
         return
		} elseif { $util(msk_height) < "3" || $util(msk_width) < "3" || \
					  $util(msk_height) > "30" || $util(msk_width) > "30" } {
         CvipAlert "The degradation image size ($util(msk_height)x$util(msk_width)) should \
			be in the range \[3-30\]."
         $topwin(util_name) configure -cursor top_left_arrow; update
         return
		}
		if [catch {expr $util(param4)} ] {
         CvipAlert "Positive Numerical value required for weight parameter:\n\
			Weight = $util(param4)."
         $topwin(util_name) configure -cursor top_left_arrow; update
         return
		} elseif {$util(param4) < 0  } {
         CvipAlert "Positive Numerical value required for weight parameter ($util(param4))"
         $topwin(util_name) configure -cursor top_left_arrow; update
         return
		}

		set temp [expr { $util(msk_height) > $util(msk_width) }\
		? $util(msk_width) : $util(msk_height) ]
#		set temp [expr $temp / 2]

		switch -exact -- $util(degrade_shape) {
			"circle" {
				if [catch {expr $util(degrade_rad) }] {
         		CvipAlert "Numerical values required for the circle diameter:\n\
					Diameter = $util(degrade_rad)."
					$topwin(util_name) configure -cursor top_left_arrow; update
					return
				} elseif { $util(degrade_rad) < 1 || $util(degrade_rad) > $temp } {
         		CvipAlert "The circle diameter ($util(degrade_rad)) should be\
					in the range \[1-$temp\]."
					$topwin(util_name) configure -cursor top_left_arrow; update
					return
				}
				set temp [eval "cvip circle $util(msk_width) $util(msk_height) \
									[expr $util(msk_width)/2] [expr $util(msk_height) /2]\
									[expr $util(degrade_rad) / 2]"]
			}
			"rectangle" {
				if [catch {expr $util(degrade_width) + $util(degrade_height)}] {
         		CvipAlert "Numerical values required for the mask sizes:\n\
					Height = $util(degrade_height), Width = $util(degrade_width)."
					$topwin(util_name) configure -cursor top_left_arrow; update
					return
				} elseif {$util(degrade_width) < 1 || $util(degrade_width) > $util(msk_width) ||
					$util(degrade_height) < 1 || $util(degrade_height) > $util(msk_height) } {
         		CvipAlert "The mask size ($util(degrade_height)x$util(degrade_width)) must be \
					positive and no larger than the mask image size\
					($util(msk_height)x$util(msk_width))."
					$topwin(util_name) configure -cursor top_left_arrow; update
					return
				}
				set temp [eval "cvip rectangle $util(msk_width) $util(msk_height) \
					[expr ($util(msk_width) - $util(degrade_width)) /2] \
					[expr ($util(msk_height) - $util(degrade_height)) /2] \
					$util(degrade_width) $util(degrade_height)"]


			}
			"Horizontal line" {
				set temp [eval "cvip line $util(msk_width) $util(msk_height) 0 \
				[expr $util(msk_height) /2 ] [expr $util(msk_width) - 1] \
				[expr $util(msk_height) /2 ]"]
			}
			"Vertical line" {
				set temp [eval "cvip line $util(msk_width) $util(msk_height) \
				[expr $util(msk_width) / 2] 0 \
				[expr $util(msk_width) / 2] [expr $util(msk_height) - 1]"]
			}
			"Slash" {
				set temp [eval "cvip line $util(msk_width) $util(msk_height) \
				0 [expr $util(msk_height) - 1] [expr $util(msk_width) - 1] 0"]
			}
			"Backslash" {
				set temp [eval "cvip line $util(msk_width) $util(msk_height) \
				0 0 [expr $util(msk_width) - 1] [expr $util(msk_height) - 1]"]
			}
		}

		set temp1 [cvip gray_multiply $temp 0.003922 0]; cvip dq $temp
		set temp $temp1

		switch -exact -- $util(blur_weight) {
			Constant {set temp1 [cvip h_image 1 $util(msk_height) $util(msk_width)]}
			"Center-weighted constant" {set temp1 [cvip h_image 2 $util(msk_height) $util(msk_width)]}
			Gaussian {set temp1 [cvip h_image 3 $util(msk_height) $util(msk_width)]}
		}


		set result [cvip multiply $temp $temp1]; cvip dq $temp; cvip dq $temp1
		set temp $result 
		set util(h_image) [cvip gray_multiply $temp $util(param4) 0]; cvip dq $temp
		}
      		set temp_var {0}
		
		$topwin(util_name) configure -cursor watch; update
		if {$current_image == ""} {
			CvipAlert "Please select an image first."
			$topwin(util_name) configure -cursor top_left_arrow; update
			return
		} 
        set result1 [cvip data_get $util(h_image)];
        set result1 [join $result1]
        set result1 [lrange $result1 2 end]
		  set temp2 "\{"; append temp2 [split $result1]; append temp2 "\}"
			
		if [catch {set result [eval "cvip specify_spatial $current_image $util(msk_height) $util(msk_width) $temp2"]} cresult] {
        CvipAlert "$cresult."
			$topwin(util_name) configure -cursor top_left_arrow; update
        return
		}
		set new_name [cvip rename $result ${current_image}_filter]
		img_q insert end $new_name
		image_post $new_name
		cvip view $new_name
	   CvipMessage "image \"$current_image\" is blurred "
	$topwin(util_name) configure -cursor top_left_arrow; update
	 }

  # build widget $topwin(util_name).perform.dismiss
	button $topwin(util_name).perform.dismiss -padx {1} -pady {1} -highlightthickness {0}  \
    -foreground "$GuiColor(dis_fg)" \
    -background "$GuiColor(dis_bg)" \
	 -activebackground "$GuiColor(dis_act_bg)" \
    -command {destroy $topwin(util_name); set topwin(util_name) NONE} \
    -text {DISMISS} \
    -width {10}

  # pack widget $topwin(util_name).perform
  pack append $topwin(util_name).perform \
    $topwin(util_name).perform.dismiss {left frame sw filly} \
    $topwin(util_name).perform.perform1 {right frame se filly}

  # build widget $topwin(util_name).current_image
  frame $topwin(util_name).current_image \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2} \
    -relief {ridge}

  # build widget $topwin(util_name).current_image.current
  label $topwin(util_name).current_image.current \
    -background "$GuiColor(label_bg)" \
    -text {Current image:}

  # build widget $topwin(util_name).current_image.name
  label $topwin(util_name).current_image.name \
    -foreground "$GuiColor(rlabel_fg)" \
    -background "$GuiColor(rlabel_bg)" \
    -text {} \
    -textvariable {current_image}

  # pack widget $topwin(util_name).current_image
  pack append $topwin(util_name).current_image \
    $topwin(util_name).current_image.current {left frame center} \
    $topwin(util_name).current_image.name {left frame center filly}
    
    

  # pack widget $topwin(util_name)
  pack append $topwin(util_name) \
    $topwin(util_name).info {top frame center expand fill} \
    $topwin(util_name).perform {bottom frame center pady 5 fill} \
    $topwin(util_name).current_image {bottom frame center fill}


  

#    $topwin(util_name).info.frame10 {top frame w} 
#    $topwin(util_name).info.frame1 {top frame w} 

 if {"[info procs XFEdit]" != ""} {
    catch "XFMiscBindWidgetTree $topwin(util_name)"
    after 2 "catch {XFEditSetShowWindows}"
  }

}


proc mask_specify {edit arg } {# xf ignore me 7
  global GuiColor
  global GuiMisc
  global GUI_DEBUG

  global {util}
  global {temp}
  if [ winfo exists .mask] {
  destroy .mask
  }
  toplevel .mask  -background {gray90}


  if { $edit == "1" } {grab set .mask}
  # Window manager configurations
  global tk_version
  global topwin
#  wm positionfrom .mask user
  wm sizefrom .mask user
  wm maxsize .mask 500 360
  wm minsize .mask 0 0
  if {$edit != "1" } {wm title .mask "Display degradation mask"} \
  else {wm title .mask "Specify degradation image"}

  # bindings

  if { $edit == "1" } {
  bind .mask <Destroy> { if {$temp == "1" } {set util(filter_val) ""} }
  }

	set llist [eval "list $arg"]
	set util(filter_height) [lindex $llist 0]
	set util(filter_width) [lindex $llist 1]
	
	for {set i 0 } { $i < $util(filter_height) } { incr i } {
		for {set j 0 } { $j < $util(filter_width) } { incr j } {	
			set util(${i}_${j}) [lindex $llist [expr $i * $util(filter_width) + $j + 2]]
#			set util(${i}_${j}) [expr $util(${i}_${j}) * $weight]
		}
	}

  # build widget .mask.info
  frame .mask.info \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  frame .mask.info.frame \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  for {set i 0} {$i <= $util(filter_width)} {incr i} {
  label .mask.info.frame.label${i} \
    -background "$GuiColor(label_bg)" \
	 -width {5} \
    -text $i
  pack append .mask.info.frame \
    .mask.info.frame.label${i} {left frame center expand} 
  }

  .mask.info.frame.label0 configure -text Mask
  pack append .mask.info \
    .mask.info.frame {top frame center expand fill} 

  for {set i 0} {$i < $util(filter_height)} {incr i} {
  # build widget .mask.info.frame$i
  frame .mask.info.frame$i \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  label .mask.info.frame${i}.label${i} \
    -background "$GuiColor(label_bg)" \
	 -width {5} \
    -text [expr $i + 1]
  pack append .mask.info.frame$i \
	 .mask.info.frame${i}.label${i} {left frame center fill expand}

  for {set j 0} {$j < $util(filter_width)} {incr j} {

  # build widget .mask.info.frame${i}.entry${j}
  entry .mask.info.frame${i}.entry${j} \
    -background "$GuiColor(entry_bg)" \
	 -foreground "$GuiColor(entry_fg)" \
    -borderwidth {1} \
    -relief {sunken} \
    -textvariable "util(${i}_${j})" \
    -width {4}
  if { $edit != "1" } {
	.mask.info.frame${i}.entry${j} configure -state disabled
  }
  if { $j == $util(filter_width) - 1 } {
	if { $i == $util(filter_height) -1 } {
		endEntry .mask.info.frame${i}.entry${j} 0
	} else {
		set temp [expr $i + 1]
		endEntry .mask.info.frame${i}.entry${j} 0 .mask.info.frame${temp}.entry0
	}
  } else {
		set temp [expr $j + 1]
		endEntry .mask.info.frame${i}.entry${j} 0 .mask.info.frame${i}.entry$temp
  }

  # pack widget .mask.info.frame0
  pack append .mask.info.frame${i} \
    .mask.info.frame${i}.entry${j} {left frame center expand fill}
  }
#end of j
  # pack widget .mask.info
  pack append .mask.info \
    .mask.info.frame$i {top frame center expand fill}
  }
  endEntry .mask.info.frame[expr ${i}-1].entry[expr ${j}-1] 0 .mask.info.frame0.entry0
#end of i

  # pack widget .mask
  pack append .mask \
    .mask.info {top frame center expand fill} 
  if { $edit == "1" } {
  # build widget .mask.perform
  frame .mask.perform \
    -background "$GuiColor(frame_bg)" \
	 -relief {raised} 

  # build widget .mask.perform.perform1
	button .mask.perform.perform1 -padx {1} -pady {1} -highlightthickness {0}  \
    -foreground "$GuiColor(perf_fg)" \
    -background "$GuiColor(perf_bg)" \
	 -activebackground "$GuiColor(perf_act_bg)" \
    -command {
	 set temp 0
	 set util(filter_val) " "
	 append  util(filter_val) "$util(filter_height) "
	 append  util(filter_val) "$util(filter_width) "
    for {set i 0} {$i < $util(filter_height)} {incr i} {
    	for {set j 0} {$j < $util(filter_width)} {incr j} {
	      if [catch "expr $util(${i}_${j})"] {
				CvipAlert "Numerical values required for the mask element:\n\
				M([expr 1+${i}], [expr 1+${j}]) = $util(${i}_${j})"
				set util(filter_val) ""; set temp 1
  				grab set .mask
			} else {
			append util(filter_val) "$util(${i}_${j}) "
			unset util(${i}_${j})
			}
		}
	 }
	 if {$util(filter_val) != ""} {
		 append util(filter_val) " "
	 }
	 if { $temp == "1" } { set util(filter_val) ""}
	$topwin(util_name) configure -cursor top_left_arrow; update
	set util(filter_val) [string trim $util(filter_val) ]
	if { $temp != "1" } {destroy .mask}
	} \
    -foreground {black} \
    -text {ACCEPT}

  # build widget .mask.perform.dismiss
	button .mask.perform.dismiss -padx {1} -pady {1} -highlightthickness {0}  \
    -foreground "$GuiColor(dis_fg)" \
    -background "$GuiColor(dis_bg)" \
	 -activebackground "$GuiColor(dis_act_bg)" \
    -command {
		$topwin(util_name) configure -cursor top_left_arrow; update
		set util(filter_val) ""
		destroy .mask} \
    -text {DISMISS} \
    -width {10}

  # pack widget .mask.perform
  pack append .mask.perform \
    .mask.perform.dismiss {left frame sw fill} \
    .mask.perform.perform1 {right frame se fill}
  # pack widget .mask
  pack append .mask \
    .mask.perform {bottom frame center pady 5 fill}
	}

}
