/***************************************************************************
* **********************************************************************
* Computer Vision/Image Processing Tool Project - Dr. Scott Umbaugh SIUE
* **********************************************************************
*
*             File Name: display.c
*           Description: Image display related files, moved here from 
*                        ../Image/image.c
*         Related Files: 
*   Initial Coding Date: Sun Jan  5 14:21:17 CST 1997
*           Portability: Standard (ANSI) C
*            References:
*             Author(s): Yansheng Wei
*                        Southern Illinois University @ Edwardsville
*
** Copyright (C) 1996 SIUE - by Scott Umbaugh and Yansheng Wei.
**
** Permission to use, copy, modify, and distribute this software and its
** documentation for any purpose and without fee is hereby granted, provided
** that the above copyright notice appear in all copies and that both that
** copyright notice and this permission notice appear in supporting
** documentation.  This software is provided "as is" without express or
** implied warranty.
**
* RCS (Revision Control System) Information
* ...Added automatically by RCS 
*
** $Log: display.c,v $
** Revision 1.10  1997/06/02 19:32:44  yawei
** modified setDisplay_Image
**
** Revision 1.9  1997/05/30 03:04:34  yawei
** modi for WIN32
**
** Revision 1.8  1997/05/29 22:29:05  yawei
** simplified display_Image(),
** changed prototype for setDisplay_Image()
**
** Revision 1.7  1997/05/21 03:51:34  yawei
** added WIN32 support
**
** Revision 1.6  1997/05/02 19:43:56  yawei
** update image format in display_Image
**
** Revision 1.5  1997/03/28 03:29:25  yawei
** removed "open" as the default viewer for machines other that SUN and SGI
**
** Revision 1.4  1997/03/06 22:20:08  akjoele
**           Heuristics have been changed so that display_Image() will display
**           any format as long as getDisplay_Image() returns EITHER
**           "RamViewer" or "picture".  When "RamViewer" is the display
**           software, display_RAMImage() will be called.  When "picture"
**           is the display software, CVIPtools will spawn a shell and execute
**           the "picture" command to view the image.  This change has been tested..
**
** Revision 1.3  1997/01/16 21:30:51  yawei
** Global Change:  Image_FORMAT ==> IMAGE_FORMAT
**
 * Revision 1.2  1997/01/15  17:19:07  yawei
 * Global Change:  IMAGE ==> Image
 *
 * Revision 1.1  1997/01/05  20:22:11  yawei
 * Initial revision
 *
*
****************************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "CVIPdef.h"           
#include "CVIPimage.h"           
#include "CVIPconvert.h"           

static char disp_prog[80];
extern char * image_format_string[];
extern int display_format;

void
setDisplay_Image(char *viewer, char *format)
{
	char *strPtr = format;

	strcpy(disp_prog, viewer);

	if (strlen(format) == 0 || format == (char *)NULL) {
		display_format = -1;
		return;
	}

	if(strcasecmp(strPtr,"PBM") == 0)
		display_format = (int) PBM;
	else if(strcasecmp(strPtr,"PGM") == 0)
		display_format = (int) PGM;
	else if(strcasecmp(strPtr,"PPM") == 0)
		display_format = (int) PPM;
	else if(strcasecmp(strPtr,"EPS") == 0)
		display_format = (int) EPS;
	else if(strcasecmp(strPtr,"TIF") == 0)
		display_format = (int) TIF;
	else if(strcasecmp(strPtr,"GIF") == 0)
		display_format = (int) GIF;
	else if(strcasecmp(strPtr,"RAS") == 0)
		display_format = (int) RAS;
	else if(strcasecmp(strPtr,"ITX") == 0)
		display_format = (int) ITX;
	else if(strcasecmp(strPtr,"IRIS") == 0)
		display_format = (int) IRIS;
	else if(strcasecmp(strPtr,"CCC") == 0)
		display_format = (int) CCC;
	else if(strcasecmp(strPtr,"BIN") == 0)
		display_format = (int) BIN;
	else if(strcasecmp(strPtr,"JPG") == 0)
		display_format = (int) JPG;
	else if(strcasecmp(strPtr,"VIP") == 0)
		display_format = (int) VIP;
	else if(strcasecmp(strPtr,"DEFAULT") == 0)
		display_format = -1;

}


char *
getDisplay_Image(void)
{

	return disp_prog;
}

/****************************************************************
                   Name: display_Image
          Expanded Name: display image
             Parameters: <image_name> - name of image file to display
			 <format> - file format of image to display
                Returns: nada
            Description: this is a simple routine that issues a shell
			 command to display an image on the current host.
			 As of 5/5/93 displayable fomats are... (Sun and SGI): 
			 PBM, PGM, PPM, GIF and RAS. (NeXT): PBM, PGM, PPM, GIF,
			 RAS, EPS, TIF. NOTE: New formats may be added as better
			 display software is introduced.
	    Diagnostics  cvip_display(...) checks to see if the <format> of the
			 image is supported by the display software of the
			 current system -- if not an error message is displayed
			 and the request can not be satisfied.
              Credit(s): Gregory Hance
		         Southern Illinois University @ Edwardsville
****************************************************************/

void 
display_Image( 
	const char *image_name,
	IMAGE_FORMAT format    /* this parameter is obsolete */
)
{
   	const char *fn = "display_Image";
   	char command[160];

	format = getFormat_CVIP((char*)image_name);
/* Why should we be concerned whether the image is displayable or not
 * given the viewer? Let the user to choose the proper user themselves
 */
/*
#if defined(Sun) || defined(SGI)
      	if( format == PBM || format == PGM || format == PPM || format == GIF ||
        format == RAS || (format==VIP && (strcmp(getDisplay_Image(), "picture")==0)) ) {
#else
      	if( format == PBM || format == PGM || format == PPM || format == GIF ||
        format == RAS || format == EPS || format == TIF ) {

#endif
*/
			#ifndef WIN32
        	sprintf(command,"%s %s 2>&- 1>&- &", getDisplay_Image(), image_name);
        	system( command );
			#else
        	_spawnlp(_P_NOWAIT, getDisplay_Image(),
				getDisplay_Image(), image_name, NULL);
			#endif
/*	}
      	else {
        	cvip_error(fn, "the %s format is not displayable given the display", image_format_string[format]);
         	cvip_error(fn, "software currently in use on the given system.");
         	cvip_error(fn, "display command used - \"%s\".", command);
      	} */
}
         

