/***************************************************************************
* **********************************************************************
* Computer Vision/Image Processing Tool Project - Dr. Scott Umbaugh SIUE
* **********************************************************************
*
*             File Name: view.c
*           Description: 
*         Related Files: 
*   Initial Coding Date: Fri Dec 20 11:16:37 CST 1996
*           Portability: Standard (ANSI) C
*            References:
*             Author(s): Yansheng Wei
*                        Southern Illinois University @ Edwardsville
*
** Copyright (C) 1996 SIUE - by Scott Umbaugh and Yansheng Wei.
**
** Permission to use, copy, modify, and distribute this software and its
** documentation for any purpose and without fee is hereby granted, provided
** that the above copyright notice appear in all copies and that both that
** copyright notice and this permission notice appear in supporting
** documentation.  This software is provided "as is" without express or
** implied warranty.
**
* RCS (Revision Control System) Information
* ...Added automatically by RCS 
*
** $Log: view.c,v $
** Revision 1.16  1997/07/17 13:24:59  yawei
** checked back changes
**
** Revision 1.15  1997/06/13 23:14:48  smakam
** fixed the problem of PGM/PPM display format for external viewer, and also the
** remap fixes
**
** Revision 1.14  1997/06/09 21:34:54  smakam
** Included the remap_Image to byte in the view.c function to facilitate other vcvi
** viewers
**
** Revision 1.13  1997/05/29 22:21:23  yawei
** *** empty log message ***
**
** Revision 1.12  1997/05/29 19:00:40  yawei
** added variable display_format for display purpose
**
** Revision 1.11  1997/05/19 22:11:53  yawei
** typo corrected
**
** Revision 1.10  1997/05/18 07:30:40  yawei
** modifid WIN32 support
**
** Revision 1.9  1997/05/18 07:14:55  yawei
** added Win32 support
**
** Revision 1.8  1997/05/09 16:00:59  yawei
** fixed the file name
**
** Revision 1.7  1997/03/08 17:06:55  yawei
** Swaped CVIP_YES and CVIP_NO
**
** Revision 1.6  1997/03/08 01:00:56  yawei
** Name Changes:
** 	BOOLEAN ==> CVIP_BOOLEAN
** 	FALSE ==> CVIP_YES
** 	TRUE ==> CVIP_NO
** 	BYTE ==> CVIP_BYTE
** 	SHORT ==> CVIP_SHORT
** 	INTEGER ==> CVIP_INTEGER
** 	FLOAT ==> CVIP_FLOAT
** 	DOUBLE ==> CVIP_DOUBLE
** 	TYPE ==> CVIP_TYPE
**
** Revision 1.5  1997/03/06 22:11:34  akjoele
** Changed fn[] in view_Image from display_Image to view_Image
**
** Revision 1.4  1997/01/16 21:30:43  yawei
** Global Change:  Image_FORMAT ==> IMAGE_FORMAT
**
 * Revision 1.3  1997/01/15  17:19:04  yawei
 * Global Change:  IMAGE ==> Image
 *
 * Revision 1.2  1997/01/03  20:01:28  yawei
 * create the temporary directory if not exist.
 *
 * Revision 1.1  1996/12/20  19:23:09  yawei
 * Initial revision
 *
*
****************************************************************************/

#include "CVIPtools.h"
#include "CVIPtoolkit.h"
#include "CVIPconvert.h"
#include "CVIPdef.h"
#include "CVIPimage.h"
#include "CVIPfs.h"
#include "CVIPhisto.h"
#include "CVIPmatrix.h"
#include "CVIPview.h"
#include <sys/types.h>
#include <sys/stat.h>
#ifdef WIN32
#include <io.h>
#include <direct.h>
#else
#include <pwd.h>
#include <dirent.h>
#endif

extern char temp_dir[];
int display_format = -1;

/* This function is a driver for display_Image and dispaly_RAMImage:
 *  - if getDisplay_Image() == RamViewer, call display_RAMImage,
 *  - else call display_Image
 * The reason that getDisplay_Image is not incorporated in 
 * display_Image is the latter one is a lower level function
 */
void 
view_Image( Image *inputImage, char *imagename) {

	const char *fn = "view_Image";
	char command[160];
	IMAGE_FORMAT format = VIP ;/* not needed, be backward compatible */
	CVIP_BOOLEAN remapped = CVIP_NO;

	if (strcmp(temp_dir, "0") == 0) {
		#ifndef WIN32
		struct passwd *pwentry;

		if((pwentry=getpwuid(getuid()))==NULL){/*get home directory of the user*/
			fprintf(stderr,"\nCannot find the user'entry from passwd file.");
			return;
		}
		strcpy(temp_dir,pwentry->pw_dir);
		#endif
		strcat(temp_dir,"/.cvip_temp/");
			/*the directory name is "cvip_temp" */
		if(access(temp_dir,0)==-1)
		mkdir(temp_dir,0755);
			/*if this directory doesn't exist, make one*/
	}

	if (strcmp(getDisplay_Image(), "RamViewer") == 0) {
		#ifndef WIN32
		display_RAMImage(inputImage,imagename);
		#else 
    	error_CVIP("view", "%s\n%s\%s\n", "the RamViewer", 
			" is an X11 based image viewer working on UNIX systems only,",
			" please use your own image viewer by setting setDisplay_Image().");
		#endif
	} else {
		char *image_name = 
			(char *)malloc((strlen(temp_dir)+strlen(imagename)+2)*sizeof(char));
		char *net_name;
		if ( (net_name=strrchr(imagename, '/')) == (char *)NULL) 
			net_name = imagename;
		else 
			net_name++;
		sprintf(image_name, "%s%s%s", temp_dir, "/", net_name);
		if (display_format == -1) {
			write_Image(inputImage, image_name, CVIP_YES, CVIP_NO, format, 1);
		} else {
			if (getDataType_Image(inputImage) != CVIP_BYTE) {
				inputImage = (Image *)remap_Image(inputImage,CVIP_BYTE,0,255);
				remapped = CVIP_YES;
			}
			if (getNoOfBands_Image(inputImage) == 1 && display_format == PPM)	 
				write_Image(inputImage, image_name, CVIP_YES, CVIP_NO, PGM, 1);
			else if (getNoOfBands_Image(inputImage) == 3 && display_format == PGM)
				write_Image(inputImage, image_name, CVIP_YES, CVIP_NO, PPM, 1);
			else 		
				write_Image(inputImage, image_name, CVIP_YES, CVIP_NO,
					(IMAGE_FORMAT)display_format, 1);
		}
		if (remapped == CVIP_YES) delete_Image(inputImage);
		display_Image(image_name, format);
	}
}
