
/***************************************************************************
* ======================================================================
* Computer Vision/Image Processing Tool Project - Dr. Scott Umbaugh SIUE
* ======================================================================
*
*             File Name: xview.c
*           Description: 
*         Related Files: 
*   Initial Coding Date: Mon Sep 18 10:15:56 CDT 1995
*           Portability: Standard (ANSI) C
*            References:
*             Author(s): Kun Luo
*                        Southern Illinois University @ Edwardsville
*
** Copyright (C) 1995  by Kun Luo, all rights reserved.
** Copyright (C) 1995 - by Kun Luo and Dr. Scott Umbaugh.
**
** Permission to use, copy, modify, and distribute this software and its
** documentation without commerical purpose and without fee is hereby granted,
** provided that the above copyright notice appear in all copies and that both
** that copyright notice and this permission notice appear in supporting
** documentation.  This software is provided "as is" without express or
** implied warranty.
**
****************************************************************************/

/***************************************************************************88
$Log: xview.c,v $
Revision 1.46  1997/07/17 14:57:21  yawei
no change

Revision 1.45  1997/07/17 13:58:27  yawei
fixed memory leak in histeq

Revision 1.44  1997/07/17 13:25:14  yawei
- fixed bombing on Linux
- fixed memory leaks

Revision 1.43  1997/06/11 04:42:08  yawei
removed signal

Revision 1.42  1997/06/05 21:59:52  yawei
added signal handler to reap dead child processes (zombie)

Revision 1.41  1997/05/09 16:01:17  yawei
added help message

Revision 1.40  1997/04/22 22:07:57  yawei
changed datatype names

Revision 1.39  1997/03/08 01:00:50  yawei
Name Changes:
	BOOLEAN ==> CVIP_BOOLEAN
	FALSE ==> CVIP_YES
	TRUE ==> CVIP_NO
	BYTE ==> CVIP_BYTE
	SHORT ==> CVIP_SHORT
	INTEGER ==> CVIP_INTEGER
	FLOAT ==> CVIP_FLOAT
	DOUBLE ==> CVIP_DOUBLE
	TYPE ==> CVIP_TYPE

Revision 1.38  1997/01/15 17:18:56  yawei
Global Change:  IMAGE ==> Image

 * Revision 1.37  1996/12/30  18:27:21  yawei
 * histeq_Image ==> histeq
 *
 * Revision 1.36  1996/12/22  21:05:55  akjoele
 * Added be__in_+CVIPtools and temp_dir
 *
 * Revision 1.35  1996/12/22  18:00:17  akjoele
 * Removed termio.h include file.  Also removed
 * be_in_CVIPtools variable and reference to extern struct
 * termio keep_term.
 *
 * Revision 1.34  1996/10/17  03:48:20  yawei
 * changed "copyright" info.
 *
 * Revision 1.33  1996/10/15  03:19:21  yawei
 * Changed the position of the color bars, and texts so they
 * look more balanced.
 *
 * Revision 1.32  1996/10/13  22:24:56  yawei
 * Changed so
 * The image information is the unmapped original image info.
 *
 * Revision 1.31  1996/10/13  21:46:03  yawei
 * Changed arrangement of information"
 *
 * Revision 1.30  1996/10/13  20:30:46  yawei
 * 1) Changed Image format, so it can recognize all image formats
 *    availabel in CVIPtools
 * 2) Added DataRange to the Information window. The Min is the mimimum of all
 *    bands, including real and imaginary pars; The Max is the max of all bands
 *    , including the real and imaginary parts.
 *
 * Revision 1.29  1996/09/13  16:50:46  wli
 * *** empty log message ***
 *
 * Revision 1.28  1996/09/07  21:26:38  yawei
 * added data_remap after hist_eq so it will display correctly for spectrum
 * images
 *
 * Revision 1.28  1996/09/07  21:26:38  yawei
 * added data_remap after hist_eq so it will display correctly for spectrum
 * images
 *
 * Revision 1.27  1996/08/22  16:24:08  yawei
 * changed remap so it works for hist_eq.
 *
 * Revision 1.26  1996/08/20  03:14:01  yawei
 * commented include "CVIPcolor.h"
 *
 * Revision 1.25  1996/08/19  21:50:58  akjoele
 * Adde needed include files.  There is still a conflict betweeen
 * external and internal declarations of typedefs like RED.
 *
 * Revision 1.24  1996/08/19  20:11:23  yawei
 * added histogram equalization to the ramviewer.
 * 1) The command is 'e'
 * 2) for gray level image, histo_eq is carried out
 * 3) for color image, a HSL transform is first carried out, than histo_eq, than inverse HSL
 *
 * Revision 1.23  1996/07/31  17:19:53  yawei
 * change display_RAMImage() to void, and return nothing.
 *
 * Revision 1.22  1996/07/31  16:58:36  akjoele
 * Checked in latest version.
 *
 * Revision 1.21  1996/07/23  02:46:17  wli
 * add cpid into xview.c
 *
 * Revision 1.21  1996/07/23  02:46:17  wli
 * add cpid into xview.c
 *
 * Revision 1.20  1996/07/10  17:54:51  yawei
 * added handler for SelectionClear
 *
 * Revision 1.5  1996/07/02  19:13:03  yawei
 * Modified version of xview.c:
 * 1. Change the message cursor to XC_crosshair;
 * 2. If click the mouse (without moving), the selection box may have strange
 *    behavior. Now, the click will cause the selection box to disappear, and
 *    deselect the area.
 *
 * Revision 1.3  1996/06/19  18:00:47  yawei
 * revised version
 * 1.  Add R, G, B, to the top of the color bars, in case the Red, Green, Blue
 *     colors are not available on some 8bit screens.
 * 2.  Change the caption on the message window, so all texts can be shown in
 *     the window
 * 3.  Modify the selection function, so we a new selection box is make, the
 *     original one will be removed.
 * 4.  Modify the select function, exposure function, and resize function, so
 *     the selection box will regenerate itself when obscured.
 * 5.  Switch the functionality of mouse Button1 and Button2, so their usage is
 *     compatible with "xv".
 *
 * Revision 1.2  1996/06/17  23:08:20  yawei
 * Add new features:
 * 1. Show dynamic color bars in the message window
 * problem:
 * 1. When exposure event happens, selection cann't regenerate itself.
 * 2. the color bars don't work on some 8bit screens. It is necessary to find
 *    some default colors for the bars.
 *
 * Revision 1.1  1996/06/17  19:31:56  yawei
 * Initial revision
 *
 * Revision 1.16  1996/06/11  15:03:54  mzuke
 * periodic checkin
 *
 * Revision 1.15  1996/02/05  17:42:41  kluo
 * change to use condition remap
 *
 * Revision 1.14  1996/01/26  19:18:03  kluo
 * change font
 *
 * Revision 1.13  1995/11/28  21:14:08  kluo
 * add fflush before fork a child
 *
 * Revision 1.12  1995/11/27  19:41:35  kluo
 * ignore SIGQUIT signal if in CVIPtools
 *
 * Revision 1.11  1995/11/09  19:29:59  kluo
 * add check for binary image
 *
 * Revision 1.10  1995/10/23  16:35:06  kluo
 * add one check
 * for directColor when it comes to define colormap
 *
 * Revision 1.9  1995/10/23  16:27:51  kluo
 * change line size
 *
 * Revision 1.8  1995/09/29  01:47:24  kluo
 * change header information
 *
 * Revision 1.7  1995/09/27  18:13:46  kluo
 * check file in
 *
 * Revision 1.6  1995/09/18  18:58:51  kluo
 * add SIUE
 *
 * Revision 1.5  1995/09/18  15:20:26  kluo
 * change header
 *
 * Revision 1.4  1995/09/13  20:10:20  kluo
 * change function name
 *
 * Revision 1.3  1995/09/04  20:36:05  kluo
 * change a prompt
 *
 * Revision 1.2  1995/09/01  18:08:32  kluo
 * change colors name to avoid collision with name definition in segmentation
 *
 * Revision 1.1  1995/09/01  17:13:51  kluo
 * Initial revision
 *
 * Revision 1.23  1995/08/24  14:51:01  kluo
 * add cont_view
 *
 * Revision 1.22  1995/08/07  19:09:09  kluo
 * change a little format and copyright
 *
 * Revision 1.21  1995/08/07  18:56:23  kluo
 * add remap
 *
 * Revision 1.20  1995/08/01  02:45:50  kluo
 * fix problem on image name in message
 *
 * Revision 1.19  1995/08/01  02:18:41  kluo
 * make image format more detail
 *
 * Revision 1.18  1995/08/01  02:14:50  kluo
 * rewrite image format
 *
 * Revision 1.17  1995/08/01  01:43:44  kluo
 * cursor is set different for message window display time
 *
 * Revision 1.16  1995/08/01  01:37:15  kluo
 * add foreground, background color allocation
 *
 * Revision 1.15  1995/07/31  19:30:13  kluo
 * little change of position of image information window
 *
 * Revision 1.14  1995/07/31  19:25:14  kluo
 * display information format is set
 *
 * Revision 1.13  1995/07/28  21:50:31  kluo
 * display data pointer
 *
 * Revision 1.12  1995/07/27  01:52:03  kluo
 * moving cursor is about ok
 *
 * Revision 1.11  1995/07/22  22:24:52  kluo
 * add icon pixmap
 *
 * Revision 1.10  1995/07/20  23:19:12  kluo
 * now change size is ok
 *
 * Revision 1.9  1995/07/20  19:09:28  kluo
 * remap and destroy object is ok
 *
 * Revision 1.8  1995/07/20  16:27:47  kluo
 * add little stuff
 *
 * Revision 1.7  1995/07/20  00:19:11  kluo
 * color image work very good on 8 bit machine(shiva)
 *
 * Revision 1.6  1995/07/19  21:27:33  kluo
 * color image for 8 bit
 *
 * Revision 1.5  1995/07/06  02:49:00  kluo
 * 8-bit gray ok
 *
 * Revision 1.4  1995/06/14  17:54:23  luok
 * fix problem with gray image on 24 bit mode(core )
 *
 * Revision 1.3  1995/06/13  19:02:55  luok
 * now 24-bit works, the problem is caused by put image before xmapraise
 *
 * Revision 1.2  1995/06/13  02:50:39  luok
 * window display, although black, no image for 24
 *
 * Revision 1.1  1995/06/12  18:29:30  luok
 * Initial revision
 *
*****************************************************************************/
#include <sys/types.h>
#include <sys/wait.h>
#include <fcntl.h>
#include <stdlib.h>
#include <signal.h>
#define WEI 
#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/cursorfont.h>
#ifdef WEI
#ifndef XK_MISCELLANY
#define XK_MISCELLANY
#endif
#include <X11/keysymdef.h>
#endif
#include <CVIPimage.h>
#include <CVIPmatrix.h>
#include <CVIPmap.h>
#include <CVIPhisto.h>
#include <CVIPdef.h>
#include <CVIPicon.h> 

#ifndef True
#define True 1
#endif

#ifndef False
#define False 0
#endif

int be_in_CVIPtools=0;
char temp_dir[100] = {'0'};

#define		LOWBIT(x)     ( ( ~(x) + 1) & (x) ) 
#define		NUM_CURSORS	8
#define		FONT1		"-misc-fixed-medium-r-semicondensed--13-120-75-75-c-60-iso8859-1"
#define		MSG_WIDTH	300
#define		MSG_HEIGHT	170

Display  	 *Disp;
Colormap  	 colormap;
int		 default_colormap;
Window    	 image_window, ctrl_window, root_window,message_window;
XImage		 *ximage;
Image		 *xdisp_Image = NULL;
Image 		 *c8Image = NULL;
Image		 *store_Image = NULL;
Image		 *store_8Image = NULL;
byte		 **x_data;
int 		 screen_num;
GC        	 image_gc, ctrl_gc,message_gc;
Cursor		 busy_cursors[NUM_CURSORS];
Cursor		 image_cursor, message_cursor;
#ifdef WEI
Cursor		 grab_cursor;
#endif
unsigned long 	 foreground, background;
int		 alloc_fore = 0, alloc_back = 0;
unsigned long    display_height, display_width;
unsigned long 	 display_cells;
unsigned int	 display_depth;
XFontStruct      *font_info;
Visual		 *visual;
int		 default_visual = True;
int 		 do_remap = CVIP_BYTE;
Pixmap		 icon_pixmap;
int 		 is_gray;
byte		 r_map[256], g_map[256], b_map[256];
unsigned long	 xview_color[256];
unsigned long    color_table[256];
int		 *direct_warning, num_direct;
int 		 color_number = 0;
int		 actual_colors;
int		 message_position;
int 		 bit8mode = True;
pid_t		 cpid = 30000;

#ifdef WEI
#define DATA_LEN   15  /* maximum length */
#define NAME_LEN   50  /* maximum length */
#define MV_NONE    0
#define MV_LEFT    1
#define MV_RIGHT   2
#define MV_UP      3
#define MV_DOWN    4
#define ROT_CLK    0
#define ROT_C_CLK  1
#define FLP_H      2
#define FLP_V      3

typedef enum {NONE, POINT, BOX} selectionType;
selectionType SelType = NONE;
int Selected = False; /* whether there is selected area */
char Image_Name[NAME_LEN];
int M_row_box, M_col_box, M_row_point, M_col_point;
unsigned int M_width_box, M_height_box, M_width_point = 0, M_height_point = 0;
int inputWidth, inputHeight, root_x, root_y;
unsigned int root_width, root_height;
int transMatrix[3][3] = {{1, 0, 0}, {0, 1, 0}, {0, 0, 1}};
int ivsMatrix[3][3] = {{1, 0, 0}, {0, 1, 0}, {0, 0, 1}};
float As_ratio, Row_aspect=1.0, Col_aspect=1.0;
int evenRotate = True;
Image *backImage = NULL;
#endif

static void find_best_visual(void);
static int **find_histogram(Image *input);
static void get_distRGB(int **histogram, int **rmap, int **gmap,int **bmap);
Window CreateIMGwindow(Image * inputImage, char *name);
void Handle_event(void);
XImage * CVIPtoXImage(Image * inputImage);
void Define_direct_color(void);
int bit_shift(unsigned long x);
void Alloc_Gray_Color(int *c_temp);
void Alloc_Color(byte r_map[], byte g_map[], byte b_map[], int num_colors,
		byte *priority );
int * Sort_Image_Gray(Image *inputImage);
byte* Sort_Colormap(Image *c8Image,byte rmap[],byte gmap[],byte bmap[], 
			int num_colors);
void Restore_cursor(Window window);
void CreateBusyCursors(Window window);
void Show_busy(Window window);
Window CreateMSGwindow(char *name);
void ShowData_Image(XButtonEvent *event);
void Load_font(void);
int Define_SetColor(XColor *color);

#ifdef WEI 
void ShowCrop_Frame(XButtonEvent *event);
void Send_Selection(XEvent event);
void Draw_Frame(int x, int y, int width, int height);
int MoveKey(KeySym ks);
void chgSelection(int d_row, int d_col, int shift);
void bellWarn(void);
void chgImgSize(int w, int h, int show, int confirm);
void reRatio(unsigned int *width, unsigned int *height, float ratio, int flag);
Image *cropImage(Image *cvipImage, int row, int col,
            unsigned int *w, unsigned int *h);
void inverseMatrix(int input[][3], int output[][3]);
void multiMatrix(int a[][3], int b[][3]);
Image *rotate90(int flag);
Image *flipHV(int flag);
void initPara(Image *newimage);
static void data_range(Image *imageP,  char *range); 
#endif 


extern Image * Color_Quant(Image *cvipImage,byte r_map[],byte g_map[],
		byte b_map[],int *colors);
extern char * image_format_string[];

void Image_expose(XExposeEvent *event);
void Draw_Image(int x_off, int y_off,int width,int height);
void do_quit(int status);
void Image_keypress(XKeyEvent *event);
static int io_error(Display * display);
void MakeSure_Bin(Image* inputImage);

Image * Resize_Image(Image *cvipImage,int row,int col);
extern Image * colorxform(const Image *rgbImage, COLOR_FORMAT newcspace,
	float *norm, float *refwhite, int dir);



static int io_error(Display * display){
    exit(1);
    return 1;
}

CVIP_BOOLEAN signal_done = CVIP_NO;

void display_RAMImage(Image * inputImage,char *name){
    XWindowAttributes attrib;
    XColor  a_color;
#ifdef WEI
    unsigned int b, d, status;
    Window root_garbage;
	 void justwait(int signum);
	 void notifyParrent(void);
#endif

    fflush(NULL);

    switch(fork()){
	case 0:
	    cpid = getpid();
		 /*atexit(notifyParrent);*/
	    break;
	case -1:
	    fprintf(stderr,"Error:Cannot fork\n");
	    return;
	default:
/*		 if (signal_done == CVIP_NO) { 
			signal(SIGUSR1, justwait);
			signal_done = CVIP_YES;	
		 }  */
	    return;
    }  
	
#ifdef WEI
    strcpy(Image_Name, name);
    backImage = duplicate_Image(inputImage);
#endif
   if(inputImage->bandP[0]->data_type != CVIP_BYTE){
	Image *temp_Image;
	do_remap = inputImage->bandP[0]->data_type;
	temp_Image = (Image*)condRemap_Image(inputImage,CVIP_BYTE,0,255);
	delete_Image(inputImage);	
	inputImage = temp_Image;
    } 

    if(be_in_CVIPtools == 1){
        signal(SIGQUIT,SIG_IGN);/*restore original IO state*/
	signal(SIGINT, SIG_IGN);
    }
    if( (Disp = XOpenDisplay("")) == NULL){
	fprintf(stderr,"Display Error:Cannot open display\n");
	exit(1);
    }
    if(fcntl(ConnectionNumber(Disp),F_SETFD,1) == -1)
	fprintf(stderr,"Warning:child cannot disinherit TCP connection\n");
    xdisp_Image = inputImage;
    store_Image = duplicate_Image(inputImage);
/*         ximage = CVIPtoXImage(xdisp_Image); */
    screen_num = DefaultScreen(Disp);
    colormap = DefaultColormap(Disp,screen_num);
    default_colormap = True;
    visual = DefaultVisual(Disp,screen_num);
    default_visual = True;
    display_depth = DisplayPlanes(Disp,screen_num);
    display_cells = DisplayCells(Disp,screen_num);
    display_width = DisplayWidth(Disp,screen_num);
    display_height = DisplayHeight(Disp,screen_num);
    root_window = RootWindow(Disp,screen_num);
#ifdef WEI
    if(status = XGetGeometry(Disp, root_window, &root_garbage,
                   &root_x, &root_y, &root_width, &root_height, &b, &d));
    else {
     fprintf(stderr, "Cannot get root window geometry, set to default value\n");
     root_x = root_y = 0; root_width = 1152; root_height = 900;
     }
#endif
    find_best_visual();
    Load_font();
    is_gray = (inputImage->bands == 1)?True:False;
    if(inputImage->color_space == BINARY){
	MakeSure_Bin(inputImage);
    }
    a_color.red = 70;
    a_color.green = 190;
    a_color.blue = 220;
    if(Define_SetColor(&a_color)){
	background = a_color.pixel;
	alloc_back = 1;
    }
    else{
	fprintf(stderr,"cannot allocate background color, use the default\n");
	background = WhitePixel(Disp,screen_num);
    }
    a_color.red = a_color.green = a_color.blue = 0;
    if(Define_SetColor(&a_color)){
	alloc_fore = 1;
	foreground = a_color.pixel;
    }
    else{
	fprintf(stderr,"cannot allocate background color, use the default\n");
	foreground = BlackPixel(Disp,screen_num);
    }
    image_window = CreateIMGwindow(inputImage,name);
    if(image_window == 0){
	fprintf(stderr,"cannot create image window");
	exit(1);
    }
    CreateBusyCursors(image_window);
    message_window = CreateMSGwindow(name);
    if(message_window == 0){
	fprintf(stderr,"cannot create message window");
	exit(1);
    }
    if(visual->class != TrueColor && visual->class != DirectColor){
	int *array;
	if(is_gray == True){
	    array = Sort_Image_Gray(inputImage);
	    Alloc_Gray_Color(array);
	    free(array);
	}
	else{
	    byte *prior;
	    color_number = display_cells;
	    c8Image = Color_Quant(inputImage,r_map,g_map,b_map,&color_number);
	    store_8Image = duplicate_Image(c8Image);
	    prior = Sort_Colormap(c8Image,r_map,g_map,b_map,color_number);
	    Alloc_Color(r_map,g_map,b_map,color_number,prior);
	    free(prior);
	}
    }
    ximage = CVIPtoXImage(store_Image);
    XMapRaised(Disp,image_window);
    Handle_event();

    return;
}

void cont_view(Image * inputImage,char *name){
    XWindowAttributes attrib;
    XColor  a_color;
    static int counter = 0;


    if(inputImage->bandP[0]->data_type != CVIP_BYTE){
	Image *temp_Image;
	do_remap = inputImage->bandP[0]->data_type;
	temp_Image = (Image*)remap_Image(inputImage,CVIP_BYTE,0,255);
	delete_Image(inputImage);	
	inputImage = temp_Image;
    }
     
    if(counter == 0){
    	if( (Disp = XOpenDisplay("")) == NULL){
	    fprintf(stderr,"Display Error:Cannot open display\n");
	    exit(1);
        }
    	if(fcntl(ConnectionNumber(Disp),F_SETFD,1) == -1)
	    fprintf(stderr,"Warning:child cannot disinherit TCP connection\n");
    	xdisp_Image = inputImage;
    	screen_num = DefaultScreen(Disp);
    	colormap = DefaultColormap(Disp,screen_num);
    	default_colormap = True;
    	visual = DefaultVisual(Disp,screen_num);
    	default_visual = True;
    	display_depth = DisplayPlanes(Disp,screen_num);
    	display_cells = DisplayCells(Disp,screen_num);
    	display_width = DisplayWidth(Disp,screen_num);
    	display_height = DisplayHeight(Disp,screen_num);
    	root_window = RootWindow(Disp,screen_num);
    	find_best_visual();
    	Load_font();
    	is_gray = (inputImage->bands == 1)?True:False;
    	image_window = CreateIMGwindow(inputImage,name);
    	if(image_window == 0){
	    fprintf(stderr,"cannot create image window");
	    exit(1);
        }
    }
    if(visual->class != TrueColor && visual->class != DirectColor){
	int *array;
	if(is_gray == True){
	    array = Sort_Image_Gray(inputImage);
	    Alloc_Gray_Color(array);
	    free(array);
	}
	else{
	    byte *prior;
	    color_number = display_cells;
	    c8Image = Color_Quant(inputImage,r_map,g_map,b_map,&color_number);
	    store_8Image = duplicate_Image(c8Image);
	    prior = Sort_Colormap(c8Image,r_map,g_map,b_map,color_number);
	    Alloc_Color(r_map,g_map,b_map,color_number,prior);
	    free(prior);
	}
    }
    ximage = CVIPtoXImage(inputImage);
    XMapRaised(Disp,image_window);
    while(counter == 0){
	XEvent event;
	XNextEvent(Disp,&event);
	if(event.type == Expose){
	    Draw_Image(0,0,inputImage->bandP[0]->cols,
		inputImage->bandP[0]->rows);
	    break;
	}
    }
    if(counter)
	Draw_Image(0,0,inputImage->bandP[0]->cols,inputImage->bandP[0]->rows);
    counter++;
    if(bit8mode == True){
	XFreeColors(Disp,colormap,color_table,actual_colors,0);
    }
    return;
}

int Define_SetColor(XColor *color){
   int rmask, gmask, bmask;
   unsigned long rdata, gdata, bdata;

   rmask = visual->red_mask;
   gmask = visual->green_mask;
   bmask = visual->blue_mask;
   if(ximage != NULL){
	/*if(ximage->data != NULL)
	    free(ximage->data); */
	XDestroyImage(ximage);
	ximage = NULL;
   }
   if(visual->class == TrueColor || visual->class == DirectColor){
	rdata = (color->red << bit_shift(rmask)) & rmask;
	gdata = (color->green << bit_shift(gmask)) & gmask;
	bdata = (color->blue << bit_shift(bmask)) & bmask;
	color->pixel = rdata|gdata|bdata;
	return 1;
   }
   else{
	color->red <<= 8;
	color->green <<= 8;
	color->blue <<= 8;
	return XAllocColor(Disp,colormap,color);
   }
}


void CreateBusyCursors(Window window){
    XColor f_color, b_color;
    Pixmap bitmap[NUM_CURSORS];
    Pixmap mask;
    int i;

    f_color.pixel = foreground;
    XQueryColor(Disp,colormap,&f_color);
    b_color.pixel = background;
    XQueryColor(Disp,colormap,&b_color);

    bitmap[0] = XCreateBitmapFromData(Disp,window,(char*)cursor0_bits,
				cursor0_width, cursor0_height);
    bitmap[1] = XCreateBitmapFromData(Disp,window,(char*)cursor1_bits,
				cursor1_width, cursor1_height);    
    bitmap[2] = XCreateBitmapFromData(Disp,window,(char*)cursor2_bits,
					cursor2_width,cursor2_height);
    bitmap[3] = XCreateBitmapFromData(Disp,window,(char*)cursor3_bits,
					cursor3_width,cursor3_height);
    bitmap[4] = XCreateBitmapFromData(Disp,window,(char*)cursor4_bits,
					cursor4_width,cursor4_height);
    bitmap[5] = XCreateBitmapFromData(Disp,window,(char*)cursor5_bits,
					cursor5_width,cursor5_height);
    bitmap[6] = XCreateBitmapFromData(Disp,window,(char*)cursor6_bits,
					cursor6_width,cursor6_height);
    bitmap[7] = XCreateBitmapFromData(Disp,window,(char*)cursor7_bits,
					cursor7_width,cursor7_height);
    mask = XCreateBitmapFromData(Disp,window,(char*)cursor_mask_bits,
				cursor_mask_width,cursor_mask_height);
    for(i = 0; i < NUM_CURSORS; i++){
	busy_cursors[i] = XCreatePixmapCursor(Disp,bitmap[i],mask,&f_color,
				&b_color, cursor0_x_hot, cursor0_y_hot);
    }
    for(i = 0; i < NUM_CURSORS; i++)
	XFreePixmap(Disp,bitmap[i]);
    XFreePixmap(Disp,mask);
}

void Show_busy(Window window){
    static int current_cursor = 0;
 
    current_cursor++;
    if(current_cursor >= NUM_CURSORS)
	current_cursor = 0;
    XDefineCursor(Disp,window,busy_cursors[current_cursor]);
}

void Restore_cursor(Window window){
    if(window == image_window){
	XDefineCursor(Disp,window,image_cursor);
    }
}

void Handle_event(void){
    XEvent  event;
    while(1){
	XNextEvent(Disp,&event);
        switch(event.type){
	    case Expose:{
		XExposeEvent *exp_event = (XExposeEvent*)&event;
	    	if(exp_event->window == image_window){
		    Image_expose(exp_event);
		}
		}
		break;
	    case KeyPress:{
		XKeyEvent *key_event = (XKeyEvent*)&event;
		if(key_event->window == image_window){
		    Image_keypress(key_event);
		}
		}
		break;
	    case ConfigureNotify:{
		XConfigureEvent *conf_event = (XConfigureEvent*)&event;
		if(conf_event->window == image_window){
		     int h, w;
		     h = conf_event->height;
		     w = conf_event->width;
#ifndef WEI
		     if(h != xdisp_Image->bandP[0]->rows ||
			 w != xdisp_Image->bandP[0]->cols){
		 	 if(xdisp_Image == NULL)
			     delete_Image(xdisp_Image);
		         xdisp_Image = Resize_Image(store_Image,h,w);
		         if(bit8mode == True && is_gray == False){
			     if(c8Image != NULL)
				 delete_Image(c8Image);
			     c8Image = Resize_Image(store_8Image,h,w);
			 }
		         ximage = CVIPtoXImage(xdisp_Image);
		         Draw_Image(0,0,-1,-1);
		     }
#else
                 chgImgSize(w, h, True, False);
#endif
		}
		}
		break;    
#ifdef WEI
	    case ButtonRelease: 
#endif
	    case ButtonPress:{
		XButtonEvent *button_event = (XButtonEvent*)&event;
		if(button_event->window == image_window){
#ifndef WEI
		     if(button_event->button == Button1)
		         ShowData_Image(button_event);
#else
		     if(button_event->button == Button1)
			 ShowCrop_Frame(button_event);
		     else if (button_event->button == Button2) 
		         ShowData_Image(button_event);
#endif
		}
		}
		break;
#ifdef WEI
            case SelectionRequest:
                Send_Selection(event);
                break;
            case SelectionClear:
		if (Selected && SelType == BOX){
		   Draw_Frame(M_col_box, M_row_box, M_width_box-1,
		     M_height_box - 1);
                   Selected = False; SelType == NONE;
		   }
		break;
#endif
	    case CirculateNotify:
		break;
  	    case MapNotify:
		Draw_Image(-1,-1,-1,-1);
		break;
  	    case DestroyNotify:
  	    case GravityNotify:
  	    case ReparentNotify:
  	    case UnmapNotify:
	    case ClientMessage:
		break;
        }
    }
}

void ShowData_Image(XButtonEvent *event){
    Window root,child;
    XEvent msg_event;
    int x,y,position,width,height,border,depth;
    int x_point, y_point, is_right = 1,y_inc;
    int start_x,start_y;
    int r,g,b;
    byte **image_r,**image_g,**image_b;
    char string[100];
    unsigned int   message_mask;
     
#ifdef WEI
    int left = 82;
    int left_pos = MSG_WIDTH*3/4, bar_width = MSG_WIDTH/15,
	 top_pos = MSG_HEIGHT*2/5+10, lwd;       
    XGCValues *gcvPtr = malloc(sizeof(XGCValues));
    unsigned long gc_mask = GCForeground|GCBackground|GCLineWidth;
    unsigned long WHITE, RED, BLUE, GREEN, BLACK, foreground, background;
    XColor xcolor, spare;
    Pixmap bar_pixmap;
     
    XGetGCValues(Disp,message_gc,gc_mask, gcvPtr);
    foreground = gcvPtr->foreground;
    background = gcvPtr->background;
    lwd = gcvPtr->line_width;
     
    XAllocNamedColor(Disp, colormap, "White", &xcolor, &spare);
    WHITE = xcolor.pixel;
    XAllocNamedColor(Disp, colormap, "Red", &xcolor, &spare);
    RED = xcolor.pixel;
    if (RED == WHITE) GREEN = BLUE = WHITE;
    else {
        XAllocNamedColor(Disp, colormap, "Green", &xcolor, &spare);
        GREEN = xcolor.pixel;
        XAllocNamedColor(Disp, colormap, "Blue", &xcolor, &spare);
        BLUE = xcolor.pixel;
	}
    XAllocNamedColor(Disp, colormap, "Black", &xcolor, &spare);
    BLACK = xcolor.pixel;
    bar_pixmap=XCreatePixmap(Disp, message_window, bar_width*3+1,64+1,
            display_depth);
#endif

    x_point = event->x;
    y_point = event->y;
     
#ifdef WEI     
    if (event->type == ButtonRelease){ 
            M_row_point = (int)(y_point/Row_aspect);
	    M_col_point = (int)(x_point/Col_aspect);
	    M_width_point = 0; M_height_point = 0;
	    SelType = POINT;
            XSetSelectionOwner(Disp, XA_PRIMARY, image_window, CurrentTime);
            if (XGetSelectionOwner(Disp, XA_PRIMARY) != image_window){
                fprintf(stderr, "failed to own selection\n");
            }
	XSync(Disp, False);
        return;
    }
#endif     
 
    width = xdisp_Image->bandP[0]->cols;
    height = xdisp_Image->bandP[0]->rows;
    XTranslateCoordinates(Disp,image_window,root_window,0,0,&x,&y,&child);
    if(x_point > width/2)
	is_right = 0;
    if((width + x + border + MSG_WIDTH) < display_width)
	start_x = width + x;
    else if((x - MSG_WIDTH - 4) > 0)
	start_x = x - MSG_WIDTH - 4;
    else if(is_right){
	if((width + x + border + MSG_WIDTH) > display_width)
	    start_x = display_width - MSG_WIDTH;
	else
	    start_x = width + x;
    }
    else{
	if((x - MSG_WIDTH - 4) < 0)
	    start_x = 0;
	else
	    start_x = x - MSG_WIDTH - 4;
    }
    if(y_point > height/2)
        start_y = y;
    else
	start_y = y + height - MSG_HEIGHT - 4;
    if(start_y < 0)
	start_y = 0;
    if(start_y >= display_height - MSG_HEIGHT - 4)
	start_y = display_height - 4 - MSG_HEIGHT;
    XDefineCursor(Disp,image_window,message_cursor);
    XMoveWindow(Disp,message_window,start_x,start_y);
    XMapRaised(Disp,message_window);
    XSync(Disp,False);
    XWindowEvent(Disp,message_window,ExposureMask,&msg_event);
#ifndef WEI
    y_inc = (font_info->descent + font_info->ascent) * 2;
#else
    y_inc = (font_info->descent + font_info->ascent);
#endif

    if(is_gray == True){
	image_r = image_g = image_b = getData_Image(xdisp_Image,0);
    }
    else{
	image_r = getData_Image(xdisp_Image,0);
	image_g = getData_Image(xdisp_Image,1);
	image_b = getData_Image(xdisp_Image,2);
    }
    r = image_r[y_point][x_point];
    g = image_g[y_point][x_point];
    b = image_b[y_point][x_point];
#ifndef WEI
    sprintf(string,"Image Height: %4d  Image Width: %4d",height,width);
    XDrawString(Disp,message_window,message_gc,
	(MSG_WIDTH-XTextWidth(font_info,string,strlen(string)))/2,
		message_position,string,strlen(string));
    sprintf(string," (Col=%4d, Row=%4d), R=%4d, G=%4d, B=%4d"
			,x_point,y_point,r,g,b);
    XDrawImageString(Disp,message_window,message_gc,0,
			message_position+y_inc,string,
			strlen(string));
#else 
    sprintf(string,"Image Height: %4d, Width = %4d",
		(int)(height/Row_aspect), (int)(width/Col_aspect)); 
    XDrawString(Disp,message_window,message_gc,
        (MSG_WIDTH-XTextWidth(font_info,string,strlen(string))-left)/2,
                message_position,string,strlen(string));
/*    sprintf(string,"Image  Width: %4d",(int)(width/Col_aspect));
    XDrawString(Disp,message_window,message_gc,
        (MSG_WIDTH-XTextWidth(font_info,string,strlen(string))-left)/2,
                message_position + y_inc,string,strlen(string));  */
#endif 

    while(1){
	if(XQueryPointer(Disp,image_window,&root,&child,&x,&y,&x_point,&y_point,
		&message_mask)){
#ifndef WEI
	    if(!(message_mask & Button1Mask))
#else
	    if(!(message_mask & Button2Mask))
#endif
		 break;
	    if(x_point < 0)
		x_point = 0;
	    if(x_point >= width)
		x_point = width - 1;
	    if(y_point < 0 )
		y_point = 0;
	    if(y_point >=height)
		y_point = height - 1;
   	    r = image_r[y_point][x_point];
    	    g = image_g[y_point][x_point];
    	    b = image_b[y_point][x_point];
#ifndef WEI
	    sprintf(string," (Col=%4d, Row=%4d), R=%4d, G=%4d, B=%4d"
			,x_point,y_point,r,g,b);
	    XDrawImageString(Disp,message_window,message_gc,0,
				message_position+y_inc,string,strlen(string));
#else
            XChangeGC(Disp, message_gc,gc_mask, gcvPtr);
 
            XSetForeground(Disp,message_gc,WHITE);
            XDrawRectangle(Disp, bar_pixmap, message_gc,0, 0,
                                bar_width, 64);
            XDrawRectangle(Disp, bar_pixmap, message_gc,0+bar_width,
                                0, bar_width, 64);
            XDrawRectangle(Disp,bar_pixmap, message_gc,0+2*bar_width,
                                0, bar_width, 64);
 
            XSetForeground(Disp,message_gc,BLACK);
            XFillRectangle(Disp, bar_pixmap, message_gc,0+1,
                                0+1, bar_width-1, 63);
            XFillRectangle(Disp, bar_pixmap,message_gc,0+bar_width+1,
                                0+1, bar_width-1, 63);
            XFillRectangle(Disp,bar_pixmap, message_gc,
                        0+2*bar_width+1, 0+1, bar_width-1, 63);
 
            XSetForeground(Disp,message_gc,RED);
            XFillRectangle(Disp, bar_pixmap, message_gc,0+1,
                                0+64-r/4, bar_width-1, r/4);
            XSetForeground(Disp,message_gc,GREEN);
            XFillRectangle(Disp, bar_pixmap,message_gc,0+bar_width+1,
                                0+64-g/4, bar_width-1, g/4);
            XSetForeground(Disp,message_gc,BLUE);
            XFillRectangle(Disp,bar_pixmap, message_gc,
                        0+2*bar_width+1, 0+64-b/4,bar_width-1,b/4);
 
            XCopyArea(Disp, bar_pixmap, message_window, message_gc, 0, 0,
                bar_width*3+1, 65, left_pos, top_pos);
 
            gcvPtr->foreground = foreground;
            gcvPtr->background = background;
            gcvPtr->line_width = lwd;
            XChangeGC(Disp, message_gc,gc_mask, gcvPtr);
 
            XSetForeground(Disp,message_gc,WHITE);
            sprintf(string," Row=%4d, Col=%4d",(int)(y_point/Row_aspect),
					       (int)(x_point/Col_aspect));
            XDrawImageString(Disp,message_window,message_gc,
                (MSG_WIDTH-XTextWidth(font_info,string,strlen(string))-left)/2,
                message_position+1*y_inc+7,string,strlen(string));
            sprintf(string,"%3d",r);
            XDrawImageString(Disp,message_window,message_gc,
                left_pos+1, message_position+1*y_inc+7,string,strlen(string));
            sprintf(string,"%3d",g);
            XDrawImageString(Disp,message_window,message_gc,
                left_pos+bar_width+2, message_position+1*y_inc+7,
                string,strlen(string));
            sprintf(string,"%3d",b);
            XDrawImageString(Disp,message_window,message_gc,
                    left_pos+2*bar_width+3, message_position+1*y_inc+7,
                string,strlen(string));
            sprintf(string," R ");
            XDrawImageString(Disp,message_window,message_gc,
                left_pos+1, top_pos-4,string,strlen(string));
            sprintf(string," G ");
            XDrawImageString(Disp,message_window,message_gc,
                left_pos+bar_width+2, top_pos-4, string,strlen(string));
            sprintf(string," B ");
            XDrawImageString(Disp,message_window,message_gc,
                    left_pos+2*bar_width+3, top_pos-4, string,strlen(string));
            XSetForeground(Disp,message_gc,foreground);
#endif
	}
    }
    XUnmapWindow(Disp,message_window);  
    XDefineCursor(Disp,image_window,image_cursor);
    XSync(Disp,False);
}
	    
        
    
    

void Image_keypress(XKeyEvent *event){
    KeySym  ks;
    XComposeStatus status;
    char  buffer[200];
#ifndef WEI
    int number;
#else
    int number, shift, ctrl, mata;
    Window child_limbo;
    int imw_x, imw_y;
    shift = event->state & ShiftMask;
    ctrl =  event->state & ControlMask;
    mata = event->state & Mod1Mask;
#endif
    number = XLookupString(event,buffer,200,&ks,&status);
#ifdef WEI
    switch (MoveKey(ks)) {
        case MV_UP: chgSelection(-1, 0, shift);
           break;
        case MV_DOWN: chgSelection(1, 0, shift);
           break;
        case MV_LEFT: chgSelection(0, -1, shift);
           break;
        case MV_RIGHT: chgSelection(0, 1, shift);
           break;
        case MV_NONE: 
           break;
    }

   if(ks == XK_Return ){
     XTextProperty window_name;
	  char *name = (char *)malloc((strlen(Image_Name) + 1) * sizeof(char));
	  strcpy(name, Image_Name);
	  if(Selected)
             Draw_Frame(M_col_box, M_row_box, M_width_box-1, M_height_box - 1);
     delete_Image(store_Image);
	  if (backImage != NULL) store_Image = duplicate_Image(backImage); 
	  initPara(store_Image);
          chgImgSize(inputWidth, inputHeight, True, True);
	  XResizeWindow(Disp, image_window, inputWidth, inputHeight);
     if(XStringListToTextProperty(&name,1,&window_name) == 0){
	  fprintf(stderr,"structure allocation for icon name failed\n");
     return;
	  }
	  XSetWMName(Disp,image_window, &window_name);
	  free(name);
   } else if (ks == XK_Escape) do_quit(0);

     XTranslateCoordinates(Disp, image_window, root_window,
          0, 0, &imw_x, &imw_y, &child_limbo);
#endif
    switch(buffer[0]){
#ifdef WEI
	case '\003':  /* ^C */
#endif
	case 'q':
	case 'Q':
	     do_quit(0);
	     break;
#ifdef WEI
	case 'n': {
     XTextProperty window_name;
	  char *name = (char *)malloc((strlen(Image_Name) + 1) * sizeof(char));
	  strcpy(name, Image_Name);
	  if(Selected)
             Draw_Frame(M_col_box, M_row_box, M_width_box-1, M_height_box - 1);
     delete_Image(store_Image);
	  if (backImage != NULL) store_Image = duplicate_Image(backImage); 
	  initPara(store_Image);
          chgImgSize(inputWidth, inputHeight, True, True);
	  XResizeWindow(Disp, image_window, inputWidth, inputHeight);
     if(XStringListToTextProperty(&name,1,&window_name) == 0){
	  fprintf(stderr,"structure allocation for icon name failed\n");
     return;
	  }
	  XSetWMName(Disp,image_window, &window_name);
	  free(name);
	  }
          break;
	case 'm': {
          chgImgSize(root_width, root_height, False, False); 
	  XMoveResizeWindow(Disp, image_window, root_x, root_y,
	                    root_width, root_height);
	  }break;
	case 'M':{
          unsigned int w, h;
          if (((float)root_width)/root_height>As_ratio){ 
             w = (int) ( root_height * As_ratio );
	     h = root_height;
	  } else {
	     h = (int) (root_width * 1.0 / As_ratio);
	     w = root_width;
	  }
          chgImgSize(w, h, False, False);
          XMoveResizeWindow(Disp, image_window,
	      (root_width - w)/2, (root_height - h)/2, w, h);
          }    break;
	case '>':{
          unsigned int w, h;
          w = (w = 2*xdisp_Image->bandP[0]->cols > root_width) ? root_width :
              2*xdisp_Image->bandP[0]->cols ;
          h = (h = 2*xdisp_Image->bandP[0]->rows > root_height) ? root_height :
              2*xdisp_Image->bandP[0]->rows ;
          chgImgSize(w, h, False, False);
	  if (imw_x+w <= root_width && imw_y+h <= root_height)
               XResizeWindow(Disp, image_window, w, h);
          else XMoveResizeWindow(Disp, image_window,
	                    (root_width-w)/2, (root_height-h)/2, w, h);
          }     break;
	case '<':{
          unsigned int w, h;
	  w = xdisp_Image->bandP[0]->cols/2;
	  h = xdisp_Image->bandP[0]->rows/2;
          chgImgSize(w, h, False, False);
          XResizeWindow(Disp, image_window, w, h);
          }     break;
	case '.':{
          unsigned int w, h;
          w = (unsigned int) ( xdisp_Image->bandP[0]->cols * 1.10);
	  w = (w > root_width) ? root_width : w;
          h = (unsigned int) (xdisp_Image->bandP[0]->rows * 1.10);
	  h = (h > root_height) ? root_height : h;
          chgImgSize(w, h, False, False);
	  if (imw_x+w <= root_width && imw_y+h <= root_height)
               XResizeWindow(Disp, image_window, w, h);
          else XMoveResizeWindow(Disp, image_window,
	                    (root_width-w)/2, (root_height-h)/2, w, h);
          }    break;
	case ',': {
          unsigned int w, h;
          w = (unsigned int) ( xdisp_Image->bandP[0]->cols * 0.90);
          h = (unsigned int) (xdisp_Image->bandP[0]->rows * 0.90);
          chgImgSize(w, h, False, False);
          XResizeWindow(Disp, image_window, w, h);
          }   break;
	case 'a': {
          unsigned int w = xdisp_Image->bandP[0]->cols,
		       h = xdisp_Image->bandP[0]->rows;
          reRatio(&w, &h, As_ratio, False);  
          if (imw_x+w <= root_width && imw_y+h <= root_height)
               XResizeWindow(Disp, image_window, w, h);
          else XMoveResizeWindow(Disp, image_window,
                            (root_width-w)/2, (root_height-h)/2, w, h);
          } break;
	case 'A':{
          unsigned int w = xdisp_Image->bandP[0]->cols,
		       h = xdisp_Image->bandP[0]->rows;
          reRatio(&w, &h, As_ratio, True);  
          if (imw_x+w <= root_width && imw_y+h <= root_height) 
               XResizeWindow(Disp, image_window, w, h); 
          else XMoveResizeWindow(Disp, image_window, 
                            (root_width-w)/2, (root_height-h)/2, w, h); 
          } break; 
	case '4': {
          unsigned int w = xdisp_Image->bandP[0]->cols,
                       h = xdisp_Image->bandP[0]->rows;
          reRatio(&w, &h, 4.0/3.0, False);
          if (imw_x+w <= root_width && imw_y+h <= root_height)
               XResizeWindow(Disp, image_window, w, h);
          else XMoveResizeWindow(Disp, image_window,
                            (root_width-w)/2, (root_height-h)/2, w, h);
          }   break;
	case '$': {
          unsigned int w = xdisp_Image->bandP[0]->cols,
                       h = xdisp_Image->bandP[0]->rows;
          reRatio(&w, &h, 4.0/3.0, True);
          if (imw_x+w <= root_width && imw_y+h <= root_height)
               XResizeWindow(Disp, image_window, w, h);
          else XMoveResizeWindow(Disp, image_window,
                            (root_width-w)/2, (root_height-h)/2, w, h);
          }   break;
	case 't': {
	  unsigned int w, h,
	     wold = xdisp_Image->bandP[0]->cols,
	     hold = xdisp_Image->bandP[0]->rows;
			 if (xdisp_Image) delete_Image(xdisp_Image);
          xdisp_Image = rotate90(ROT_CLK);
	  ximage = CVIPtoXImage(xdisp_Image);
	  w = ximage->width; h = ximage->height;
	  Draw_Image(0,0,-1,-1);
          XMoveResizeWindow(Disp, image_window, (root_width-w)/2,
	     (root_height-h)/2, w, h);
          if (wold == w && hold == h && Selected) 
	    Draw_Frame(M_col_box, M_row_box, M_width_box-1, M_height_box-1);
	} break;
	case 'T': {
          unsigned int w, h,
	     wold = xdisp_Image->bandP[0]->cols,
	     hold = xdisp_Image->bandP[0]->rows;
			 if (xdisp_Image) delete_Image(xdisp_Image);
          xdisp_Image = rotate90(ROT_C_CLK);
          ximage = CVIPtoXImage(xdisp_Image);
	  w = ximage->width; h = ximage->height;
          Draw_Image(0,0,-1,-1);
          XMoveResizeWindow(Disp, image_window, (root_width-w)/2,
	    (root_height-h)/2, w, h);
          if (wold == w && hold == h && Selected) 
	    Draw_Frame(M_col_box, M_row_box, M_width_box-1, M_height_box-1);
        }   break;
	case 'H':
	case 'i':  
	case '?': {
		printf("\n RamViewer Mouse and Keyboard Usage:\n");
		printf(" Button 1 press and drag		\n");
		printf("     --- draw a selection box\n");
		printf(" Button 1 click out of the selection box\n");
		printf("     --- unselect of subimage\n");
		printf(" Button 1 press and drag on selection box\n");
		printf("     ---  move selection box\n");
		printf(" Button 2 press\n");
		printf("     --- display information about the image\n");
		printf("\n");
		printf(" right/left arrow          \n");
		printf("     --- move selection box one pixel right/left\n");
		printf(" up/down arrow             \n");
		printf("     --- move selection box one pixel up/down\n");
		printf(" shift right/left arrow    \n");
		printf("     --- enlarge/shrink selection box one pixel horizontally\n");
		printf(" shift up/down             \n");
		printf("     --- enlarge/shrink selection box one pixel vertically\n");
		printf(" \n");
		printf(" Resizing image:\n");
		printf("    m     maximize image to whole screen\n");
		printf("    M     maximize image and retains original aspect-ratio\n");
		printf("    >     double size image\n");
		printf("    <     half size image\n");
		printf("    .     enlarge image by 10%\n");
		printf("    ,     shrink image by 10%\n");
		printf(" \n");
		printf(" Changing aspect-ratio:\n");
		printf("    a     re-aspect-ratio using shorter side\n");
		printf("    A     re-aspect-ratio using longer side\n");
		printf("    4     re-aspect-ratio to 43 using shorter side\n");
		printf("    $     re-aspect-ratio to 43 using longer side\n");
		printf(" \n");
		printf(" Rotation:\n");
		printf("    t     turn image 90 degrees clockwise\n");
		printf("    T     turn image 90 degrees counter-clockwise\n");
		printf(" \n");
		printf(" Flip:\n");
		printf("    h     horizontal flip\n");
		printf("    v     vertical flip\n");
		printf(" \n");
		printf(" Others:\n");
		printf("    n, <Enter>   revert to original image\n");
		printf("    q,Q,Ctr-C    quit the RamViewer\n");
		printf("    H,?,i         print this help message\n");
		printf("\n");
		} break;
	case 'h': {
          unsigned int w, h;
			if (xdisp_Image) delete_Image(xdisp_Image);
          xdisp_Image = flipHV(FLP_H);
          ximage = CVIPtoXImage(xdisp_Image);
	  w = ximage->width; h = ximage->height;
          Draw_Image(0,0,-1,-1);
          if (Selected )
              Draw_Frame(M_col_box, M_row_box, M_width_box-1, M_height_box-1);
        }   break;
	case 'v': {
          unsigned int w, h;
			 if (xdisp_Image) delete_Image(xdisp_Image);
          xdisp_Image = flipHV(FLP_V);
          ximage = CVIPtoXImage(xdisp_Image);
	  w = ximage->width; h = ximage->height;
          Draw_Image(0,0,-1,-1);
          if (Selected )
	     Draw_Frame(M_col_box, M_row_box, M_width_box-1, M_height_box-1);
        }   break; 

		  case 'e': {
			Image *temp_Image;
    		XTextProperty window_name;
			char *name = (char *)malloc((strlen(Image_Name) + 7) * sizeof(char));
			strcpy(name, Image_Name);
			strcat(name, "(Hist)");
    		if (xdisp_Image->bands == 3) { 
				float *normvec = (float*)malloc(3*sizeof(float));
				normvec[0] = normvec[1] = normvec[2] = 255.0;
				Show_busy(image_window); XFlush(Disp);
				temp_Image = xdisp_Image;
				xdisp_Image =(Image*) remap_Image(xdisp_Image,CVIP_BYTE,0,255);
				delete_Image(temp_Image);
				Show_busy(image_window); XFlush(Disp);
				xdisp_Image = (Image*) colorxform(xdisp_Image,
					(COLOR_FORMAT)HSL,normvec, (float *)NULL, 1);
				Show_busy(image_window); XFlush(Disp);
				temp_Image = xdisp_Image;
				xdisp_Image =(Image*) remap_Image(xdisp_Image,CVIP_BYTE,0,255);
				delete_Image(temp_Image);
				Show_busy(image_window); XFlush(Disp);
				xdisp_Image = (Image*)histeq(xdisp_Image, (int)2);
				Show_busy(image_window); XFlush(Disp);
				temp_Image = xdisp_Image;
				xdisp_Image =(Image*) remap_Image(xdisp_Image,CVIP_BYTE,0,255);
				delete_Image(temp_Image);
				Show_busy(image_window); XFlush(Disp);
				xdisp_Image = (Image*) colorxform(xdisp_Image,
					(COLOR_FORMAT)HSL,normvec, (float *)NULL, -1);
				Show_busy(image_window); XFlush(Disp);
				temp_Image = xdisp_Image;
				xdisp_Image =(Image*) remap_Image(xdisp_Image,CVIP_BYTE,0,255);
				delete_Image(temp_Image);
				Show_busy(image_window); XFlush(Disp);
				temp_Image = xdisp_Image;
				xdisp_Image = (Image*)condRemap_Image(xdisp_Image,CVIP_BYTE,0,255);
				delete_Image(temp_Image);
	
				Show_busy(image_window); XFlush(Disp);
				temp_Image = store_Image;
				store_Image =(Image*) remap_Image(store_Image,CVIP_BYTE,0,255);
				delete_Image(temp_Image);
				Show_busy(image_window); XFlush(Disp);
				store_Image = (Image*) colorxform(store_Image,
					(COLOR_FORMAT)HSL,normvec, (float *)NULL, 1);
				Show_busy(image_window); XFlush(Disp);
				temp_Image = store_Image;
				store_Image =(Image*) remap_Image(store_Image,CVIP_BYTE,0,255);
				delete_Image(temp_Image);
				Show_busy(image_window); XFlush(Disp);
				store_Image = (Image*)histeq(store_Image, (int)2);
				Show_busy(image_window); XFlush(Disp);
				temp_Image = store_Image;
				store_Image = (Image*)remap_Image(store_Image,CVIP_BYTE,0,255);
				delete_Image(temp_Image);
				Show_busy(image_window); XFlush(Disp);
				store_Image = (Image*) colorxform(store_Image,
					(COLOR_FORMAT)HSL,normvec, (float *)NULL, -1);
				Show_busy(image_window); XFlush(Disp);
				temp_Image = store_Image;
				store_Image = (Image*)remap_Image(store_Image,CVIP_BYTE,0,255);
				delete_Image(temp_Image);
				Show_busy(image_window); XFlush(Disp);
				temp_Image = store_Image;
				store_Image = (Image*)condRemap_Image(store_Image,CVIP_BYTE,0,255);
				delete_Image(temp_Image);
	
			} else {
				Show_busy(image_window); XFlush(Disp);
				xdisp_Image = (Image*)histeq(xdisp_Image, (int)0);
				Show_busy(image_window); XFlush(Disp);
				temp_Image = xdisp_Image;
				xdisp_Image = (Image*)condRemap_Image(xdisp_Image,CVIP_BYTE,0,255);
				delete_Image(temp_Image);
				Show_busy(image_window); XFlush(Disp);
				store_Image = (Image*)histeq(store_Image, (int)0);
				Show_busy(image_window); XFlush(Disp);
				temp_Image = store_Image;
				store_Image = (Image*)condRemap_Image(store_Image,CVIP_BYTE,0,255);
				delete_Image(temp_Image);
			}
    		if(XStringListToTextProperty(&name,1,&window_name) == 0){
			fprintf(stderr,"structure allocation for icon name failed\n");
         return;
		   }
			XSetWMName(Disp,image_window, &window_name);
         ximage = CVIPtoXImage(xdisp_Image);
        	Draw_Image(0,0,-1,-1);
			free(name);
		  } break;
/*	case 'c': {
	  unsigned int w = (unsigned int)M_width_box,
		       h = (unsigned int)M_height_box;
          if (!Selected) break;
          store_Image = cropImage(store_Image, M_row_box, M_col_box,
	                 &w, &h); 
	  xdisp_Image = duplicate_Image(store_Image);
	  if (xdisp_Image == NULL) {
             fprintf(stderr, "No crop image obtained.\n");
	     xdisp_Image = duplicate_Image(backImage);
	     break;
	  }
	  initPara(store_Image);
          ximage = CVIPtoXImage(xdisp_Image);
          XResizeWindow(Disp, image_window, w, h);
          Draw_Image(0,0,-1,-1); 
	}  break; 
	case 'u':    break; */
#endif
	default:;
    }
#ifdef WEI
    imw_x = root_width; imw_y = root_height;
#endif
}

void do_quit(int status){
    /*free(ximage->data);*/
    XDestroyImage(ximage);
    delete_Image(xdisp_Image);
    delete_Image(store_Image);
    if(bit8mode == True && is_gray == False){
	delete_Image(c8Image);
	delete_Image(store_8Image);
    }
    XFreeGC(Disp,image_gc);
    if(!default_visual)
	free(visual);
    if(bit8mode == True){
	XFreeColors(Disp,colormap,color_table,actual_colors,0);
	if(alloc_fore)
	    XFreeColors(Disp,colormap,&foreground,1,0);
	if(alloc_back)
	    XFreeColors(Disp,colormap,&background,1,0); 
    }
    else{
	XFreeColormap(Disp,colormap);
    }
    XDestroyWindow(Disp,image_window);
    XFlush(Disp);
    XCloseDisplay(Disp);
    exit(status);
}
    
    

    


void Image_expose(XExposeEvent *event){
    static Region image_region;
    XRectangle rectangle;

    if(image_region == NULL)
	image_region = XCreateRegion();
    rectangle.x=event->x;
    rectangle.y=event->y;
    rectangle.width=event->width;
    rectangle.height=event->height;
    XUnionRectWithRegion(&rectangle,image_region,image_region);
    if(event->count == 0){
    	XClipBox(image_region,&rectangle);
    	XSetClipRectangles(Disp,image_gc,0,0,&rectangle,1,YXBanded);
	XDestroyRegion(image_region);
	image_region = NULL;
	Draw_Image(rectangle.x,rectangle.y,rectangle.width,rectangle.height);
#ifdef WEI
        if (Selected == True && SelType == BOX) 
           Draw_Frame(M_col_box, M_row_box, M_width_box-1, M_height_box - 1);
#endif 
	XSetClipMask(Disp,image_gc,(Pixmap)None);
    }
}

void Draw_Image(int x_off, int y_off,int width,int height){
    if(x_off == -1)
	x_off = 0;
    if(y_off == -1)
	y_off = 0;
    if(width == -1)
        width = ximage->width;
    if(height == -1)
	height = ximage->height;
    XPutImage(Disp,image_window,image_gc,ximage,x_off,y_off,x_off,y_off,
		width,height);
    XSync(Disp,False);
}

   



byte* Sort_Colormap(Image *c8Image,byte rmap[],byte gmap[],byte bmap[], 
			int num_colors){
    long sorted[256];
    byte *array;
    byte **image;
    int rows,cols,y,x;

    rows = c8Image->bandP[0]->rows;
    cols = c8Image->bandP[0]->cols;
    image = getData_Image(c8Image,0);
    array = (byte*)malloc(num_colors);
    for(y=0;y<num_colors;y++)
	array[y] = y;
    memset(sorted,0,256*sizeof(long));
    for(y = 0; y < rows; y++){
	for(x = 0; x < cols; x++){
	     sorted[image[y][x]]++;
	}
    }
    for(y=0;y<num_colors;y++){
	for(x=y;x<num_colors;x++){
	    if(sorted[x] > sorted[y]){
		SWAP(sorted[x],sorted[y]);
		SWAP(array[y],array[x]);
	    }
 	}
    }
    return array;
}  


int * Sort_Image_Gray(Image *inputImage){
    long c_entry[256];
    int *c_temp;
    int  counter = 0;
    long red_c = 0,green_c = 0, blue_c = 0;
    int r_ratio = 0,g_ratio = 0,b_ratio = 0;
    int band, rows, cols,x,y;
    byte **image;
    
    band = inputImage->bands;
    rows = inputImage->bandP[0]->rows;
    cols = inputImage->bandP[0]->cols;
    c_temp = (int*)malloc(256*sizeof(int));
    memset(r_map,-1,256*sizeof(byte));
    memset(g_map,-1,256*sizeof(byte));
    memset(b_map,-1,256*sizeof(byte));
    memset(xview_color,0,256*sizeof(byte));
    for(y = 0; y < 256; y++)
	c_temp[y] = y;
    if(band == 1){
	memset(c_entry,0,256*sizeof(long));
	image = getData_Image(inputImage,0);
	for(y = 0; y < rows; y++){
	    for( x = 0; x < cols; x++){
		c_entry[image[y][x]]++;
	    }
	}
	for(y = 0; y < 256; y++){
	    for(x = y; x < 256; x++){
		if(c_entry[y] < c_entry[x]){
		    SWAP(c_entry[y],c_entry[x]);
		    SWAP(c_temp[y],c_temp[x]);
		}
	    }
	}
	for(y = 0; y < 256; y++){
	    if(c_entry[y] == 0)
		c_temp[y] = -1;
	}
    }
    return c_temp;
}

void Alloc_Color(byte r_map[], byte g_map[], byte b_map[], int num_colors,
			byte *priority){
    XColor color;
    int i,j;
    int warning = -1;
    unsigned long rmask, gmask, bmask;
    XColor *query = NULL;
    float distance,sum;
    int place;

    actual_colors = 0;
    color.flags = DoRed | DoGreen | DoBlue;
    for(i = 0; i < num_colors; i++){
 	color.red = r_map[priority[i]] << 8;
	color.green = g_map[priority[i]] << 8;
	color.blue = b_map[priority[i]] << 8;
 	if(XAllocColor(Disp,colormap,&color)){
	    xview_color[priority[i]] = color.pixel;
	    color_table[actual_colors] = color.pixel;
	    actual_colors ++;
	}
	else{
	    if(query == NULL){
		query = (XColor*)malloc(display_cells*sizeof(XColor));
		for(j=0;j<display_cells;j++){
		    query[j].pixel = j;
		    query[j].pad = 0;
		}
		XQueryColors(Disp,colormap,query,display_cells);
	    }
	    sum = ((float)query[0].red-color.red)*
			((float)query[0].red - color.red) +
	               ((float)query[0].green-color.green)*
			((float)query[0].green-color.green)+
		      ((float)query[0].blue-color.blue)
			*((float)query[0].blue-color.blue); 
	    distance = sum;	
	    place = 0;
	    for(j=1;j<display_cells;j++){
	        sum = ((float)query[j].red-color.red)*
			((float)query[j].red - color.red) +
	               ((float)query[j].green-color.green)*
			((float)query[j].green-color.green)+
		      ((float)query[j].blue-color.blue)
			*((float)query[j].blue-color.blue);
		if(distance > sum){
		    distance = sum;
		    place = j;
		}
	    }
	    xview_color[priority[i]] = place;
	    r_map[priority[i]] = query[place].red >> 8;
	    g_map[priority[i]] = query[place].green >> 8;
	    b_map[priority[i]] = query[place].blue >> 8;	    
	}
    }
}

void Alloc_Gray_Color(int *c_temp){
    XColor color;
    int i,j;
    int warning = -1;
    unsigned long rmask, gmask, bmask;
    XColor *query = NULL;
    float distance,sum;
    int place;
     
    actual_colors = 0;
    color.flags = DoRed | DoGreen | DoBlue;
    for(i = 0; i < 256; i++){
   	if(c_temp[i] != -1){
	    color.red = color.green = color.blue = c_temp[i] << 8;
 	    if(XAllocColor(Disp,colormap,&color)){
		r_map[c_temp[i]]=g_map[c_temp[i]]=b_map[c_temp[i]]=c_temp[i];
		xview_color[c_temp[i]] = color.pixel;
		color_table[actual_colors] = color.pixel;
		actual_colors ++;
	    }
	    else{
		if(query == NULL){
		    query = (XColor*)malloc(display_cells*sizeof(XColor));
		    for(j=0;j<display_cells;j++){
			query[j].pixel = j;
			query[j].pad = 0;
		    }
		    XQueryColors(Disp,colormap,query,display_cells);
		}
	        sum = ((float)query[0].red-color.red)*
			((float)query[0].red - color.red) +
	               ((float)query[0].green-color.green)*
			((float)query[0].green-color.green)+
		      ((float)query[0].blue-color.blue)
			*((float)query[0].blue-color.blue);
		distance = sum;	
		place = 0;
		for(j=1;j<display_cells;j++){
	            sum = ((float)query[j].red-color.red)*
			((float)query[j].red - color.red) +
	               ((float)query[j].green-color.green)*
			((float)query[j].green-color.green)+
		      ((float)query[j].blue-color.blue)
			*((float)query[j].blue-color.blue);
		    if(distance > sum){
			distance = sum;
			place = j;
		    }
		}
		xview_color[c_temp[i]] = place;
		r_map[c_temp[i]] = query[place].red >> 8;
		g_map[c_temp[i]] = query[place].green >> 8;
		b_map[c_temp[i]] = query[place].blue >> 8;	    
	    }
	}
	else{
	    color_number = i;
	}
    }
}

Window CreateIMGwindow(Image * inputImage, char *name){
    XSetWindowAttributes  attrib;
    unsigned long       attrib_mask;
    XWMHints   wm_hints;
    XSizeHints   size_hints;
    XClassHint class_hints;
    XTextProperty window_name,icon_name;
    Pixmap	icon_bitmap;
    XGCValues gc_str;
    int height, width;
    char *copy_name;
    Window window;
 
    copy_name = (char*)malloc(strlen(name)+1);
    strcpy(copy_name,name);

    height = inputImage->bandP[0]->rows;
    width = inputImage->bandP[0]->cols;    
#ifdef WEI
    inputWidth = width;
    inputHeight = height;
    As_ratio = ((float) inputWidth) / inputHeight;
#endif
    size_hints.width = width;
    size_hints.height = height;
    size_hints.flags = PSize;

    class_hints.res_name = "CVIPtools";
    class_hints.res_class = "CVIP";
    
    if(XStringListToTextProperty(&copy_name,1,&window_name) == 0){
	fprintf(stderr,"structure allocation for icon name failed\n");
        return 0;
    }
    if(XStringListToTextProperty(&copy_name,1,&icon_name) == 0){
	fprintf(stderr,"structure allocation for icon name failed\n");
	return 0;
    }

    wm_hints.initial_state = NormalState;
    wm_hints.input = True;
    wm_hints.flags = StateHint | InputHint;

    attrib.colormap = colormap;
    attrib.background_pixel = background;
    attrib.border_pixel = foreground;
    attrib_mask = CWColormap | CWBorderPixel | CWBackPixel;
    window = XCreateWindow(Disp,root_window,0,0,width,height,2,display_depth,
		InputOutput,visual,attrib_mask,&attrib);
    if(window == 0)
	return 0;
    icon_pixmap = XCreateBitmapFromData(Disp,window,(char*)icon_bits,
			icon_width,icon_height);
    wm_hints.icon_pixmap = icon_pixmap;
    wm_hints.flags |= IconPixmapHint;
    XSetWMProperties(Disp,window,&window_name,&icon_name,(char**)0,(int)0,
		&size_hints,&wm_hints,&class_hints);
#ifndef WEI 
    XSelectInput(Disp,window,ExposureMask|ButtonPressMask|ButtonReleaseMask
		|KeyPressMask|KeyReleaseMask|StructureNotifyMask);
#else
    XSelectInput(Disp,window,ExposureMask|ButtonPressMask|ButtonReleaseMask
        |PointerMotionMask|KeyPressMask|KeyReleaseMask|StructureNotifyMask);
#endif
    image_gc = XCreateGC(Disp,window,0,&gc_str);
    free(copy_name);
    XSetIOErrorHandler(io_error);
    image_cursor = XCreateFontCursor(Disp,XC_left_ptr);
    XDefineCursor(Disp,window,image_cursor);
    return window;
}

Window CreateMSGwindow(char *name){
    XSetWindowAttributes  attrib;
    Pixmap		 back_pix;
    unsigned long 	 attrib_mask;
    Window		 window;
    XGCValues 		 gc_str;
    int			 height = MSG_HEIGHT, width = MSG_WIDTH;
    int			 y_inc,position;
    char		 info[200],*ptr;
    int			 str_width;
	 Image *temp_Image;

#ifdef WEI
    int left = 80;
#endif
    attrib.colormap = colormap;
    attrib.border_pixel = foreground;
    attrib.override_redirect = True;
    attrib_mask = CWColormap | CWBorderPixel |CWOverrideRedirect;
    str_width = XTextWidth(font_info," ",1)*45;
    if(str_width > width)
	width = str_width;
    window = XCreateWindow(Disp,root_window,0,0,width,height,2,display_depth,
			InputOutput,visual,attrib_mask,&attrib);
    if(window == 0)
	return 0;
    XSelectInput(Disp,window,ExposureMask);
    gc_str.font = font_info->fid;
    gc_str.background = foreground;
    gc_str.foreground = background;
    gc_str.line_width = 2;
    message_gc = XCreateGC(Disp,window,GCFont|GCBackground|GCForeground
		      	|GCLineWidth,&gc_str);
    y_inc = font_info->ascent + font_info->descent;
    back_pix = XCreatePixmap(Disp,window,width,height,display_depth);
    XFillRectangle(Disp,back_pix,message_gc,0,0,width,height+1);
    XSetBackground(Disp,message_gc,background);
    XSetForeground(Disp,message_gc,foreground);
    strcpy(info,"Information for Image: ");
    ptr = strrchr(name,'/');
    if(ptr == NULL)
	strcat(info,name);
    else
	strcat(info,ptr+1);
    position = y_inc;
    str_width = XTextWidth(font_info,info,strlen(info));
    XDrawString(Disp,back_pix,message_gc,(width-str_width)/2,position,info,
		   strlen(info));
    position += y_inc;
    strcpy(info,"Image Viewer Written By: Kun Luo, Yansheng Wei");
    str_width = XTextWidth(font_info,info,strlen(info));
    XDrawString(Disp,back_pix,message_gc,(width-str_width)/2,position,info,
		   strlen(info));
    position += y_inc;
#ifndef WEI
    strcpy(info,"Copyright (C) 1995 - SIUE by Dr. Scott Umbaugh and Kun Luo");
#else
    strcpy(info,"Copyright (C) 1995,1996 - SIUE,");
#endif
    str_width = XTextWidth(font_info,info,strlen(info));
    XDrawString(Disp,back_pix,message_gc,(width-str_width)/2,position,info,
		   strlen(info));
    position += y_inc;
    strcpy(info,"Scott Umbaugh, Kun Luo, Yansheng Wei");
    str_width = XTextWidth(font_info,info,strlen(info));
    XDrawString(Disp,back_pix,message_gc,(width-str_width)/2,position,info,
		   strlen(info));
    XDrawLine(Disp,back_pix,message_gc,0,4+position,width,4+position);
    position += 4 + y_inc;

    strcpy(info,"Image format: ");
	 strcat(info, image_format_string[backImage->image_format]);

    str_width = XTextWidth(font_info,info,strlen(info));
#ifndef WEI
    XDrawString(Disp,back_pix,message_gc,(width-str_width)/2,position,info,
		   strlen(info));
#else
    XDrawString(Disp,back_pix,message_gc,(width-str_width-left)/2,position,info,
              strlen(info)); 
#endif
    position += 5 + y_inc;
    strcpy(info,"Data Type: ");
    switch(do_remap){
	case CVIP_BYTE:
	   strcat(info,"BYTE");
	   break;
	case CVIP_SHORT:
	   strcat(info,"SHORT");
	   break;
	case CVIP_INTEGER:
	   strcat(info,"INTEGER");
	   break;
	case CVIP_FLOAT:
	   strcat(info,"FLOAT");
	   break;
	case CVIP_DOUBLE:
	   strcat(info,"DOUBLE");
	   break;
    }
    if(do_remap != CVIP_BYTE)
	strcat(info," (remapped to BYTE)");

    str_width = XTextWidth(font_info,info,strlen(info));
#ifndef WEI
    XDrawString(Disp,back_pix,message_gc,(width-str_width)/2,position,info,
		   strlen(info));
#else
    XDrawString(Disp,back_pix,message_gc,(width-str_width-left)/2,position,info,
              strlen(info));
#endif
    position += y_inc;	
 	 strcpy(info,"Data Format: ");
	 if ( getDataFormat_Image(backImage) == COMPLEX ) {
	 	strcat(info,"COMPLEX");
	 } else {
		strcat(info,"REAL");
	 }
    str_width = XTextWidth(font_info,info,strlen(info));
    XDrawString(Disp,back_pix,message_gc,(width-str_width-left)/2,position,info,
              strlen(info));
    position += y_inc;	

	 strcpy(info, "Data Range: ");
	 data_range(backImage, info);
    str_width = XTextWidth(font_info,info,strlen(info));
    XDrawString(Disp,back_pix,message_gc,(width-str_width-left)/2,position,info,
		   strlen(info));
    position += y_inc;	
    if(backImage->bands == 1)
	strcpy(info,"Color Space: GRAY_SCALE, 1 band");
    else
	strcpy(info,"Color Space: COLOR, 3 bands");
    str_width = XTextWidth(font_info,info,strlen(info));
#ifndef WEI
    XDrawString(Disp,back_pix,message_gc,(width-str_width)/2,position,info,
		   strlen(info));
#else
    XDrawString(Disp,back_pix,message_gc,(width-str_width-left)/2,position,info,
              strlen(info));
#endif
    position += y_inc;    
    XSetWindowBackgroundPixmap(Disp,window,back_pix);
#ifndef WEI
    message_cursor = XCreateFontCursor(Disp,XC_cross);
#else
    message_cursor = XCreateFontCursor(Disp,XC_crosshair);
#endif

	temp_Image = (Image*)condRemap_Image(backImage,CVIP_BYTE,0,255);
	delete_Image(backImage);	
	backImage = temp_Image;

    XSync(Disp,False);
    message_position = position;
    return window;
}
    
    

void Load_font(void){
    if((font_info = XLoadQueryFont(Disp,FONT1)) == NULL){
	fprintf(stderr,"Cannot load font");
	exit(1);
    }
}


int bit_shift(unsigned long x){
   int count = 0;
   unsigned long data = LOWBIT(x);
   while( (data%2) == 0){
	count ++;
	data = data >> 1;
   }
   return count;
}

XImage * CVIPtoXImage(Image * inputImage){
   XImage *image;
   byte   *data,**r_input,**g_input,**b_input;
   unsigned short *data16;
   unsigned long  *data32;
   XColor  color;
   int rows, cols,x,y,no_of_bands;
   XWindowAttributes attrib;
   unsigned long rmask, gmask, bmask;
   int b_pixel, b_line, b_order;
   unsigned long rdata,gdata,bdata;
   unsigned long pixel_value;
   int red_shift, green_shift, blue_shift;
   
   no_of_bands = inputImage->bands;
   rows = inputImage->bandP[0]->rows;
   cols = inputImage->bandP[0]->cols;
   
   if(!(XGetWindowAttributes(Disp,image_window,&attrib))){
	fprintf(stderr,"\ncannot get window attributes");
	return NULL;
   }
   rmask = visual->red_mask;
   gmask = visual->green_mask;
   bmask = visual->blue_mask;
   if(ximage != NULL){
	/*if(ximage->data != NULL)
	    free(ximage->data);*/
	XDestroyImage(ximage);
	ximage = NULL;
   }
   if(attrib.visual->class == TrueColor || attrib.visual->class == DirectColor){
	image = XCreateImage(Disp,visual,display_depth,ZPixmap,0,NULL,
			cols,rows,32,0);
	if(image == NULL){
	    fprintf(stderr,"Error in creating X Image");
 	    return NULL;
	}	
	b_pixel = image->bits_per_pixel;
  	b_line = image->bytes_per_line;
	b_order = image->byte_order;
	data = (byte*)malloc(rows*b_line);
	if(data == NULL){
	    fprintf(stderr,"\nError in calloc X image data");
	    return NULL;
        }
	image->data = (char*)data;
	if(no_of_bands != 1){
	    r_input = getData_Image(inputImage,0);
	    g_input = getData_Image(inputImage,1);
	    b_input = getData_Image(inputImage,2);
     	}
	else{
	    r_input = getData_Image(inputImage,0);
	}
	red_shift = bit_shift(rmask);
	green_shift = bit_shift(gmask);
	blue_shift = bit_shift(bmask);
	for(y=0;y<rows;y++){
	    Show_busy(image_window);
	    data = (byte*)(image->data + y*b_line);
	    for(x=0;x<cols;x++){
		if(no_of_bands == 1)
		    rdata = gdata = bdata = r_input[y][x]; 
		else{
		    rdata = r_input[y][x];
		    gdata = g_input[y][x];
		    bdata = b_input[y][x];
		}
		rdata = (rdata << red_shift) & rmask;
		gdata = (gdata << green_shift) & gmask;
		bdata = (bdata << blue_shift) & bmask;
		pixel_value = rdata | gdata | bdata;
  		if(b_pixel == 32){
	   	    if(b_order == MSBFirst){
		    	*data++ = (pixel_value >> 24) & 0xff;
		        *data++ = (pixel_value >> 16) & 0xff;
		 	*data++ = (pixel_value >>  8) & 0xff;
		 	*data++ = (pixel_value	) & 0xff;
		    }
		    else{
		        *data++ = (pixel_value      ) & 0xff;
			*data++ = (pixel_value >>  8) & 0xff;
			*data++ = (pixel_value >> 16) & 0xff;
		        *data++ = (pixel_value >> 24) & 0xff;
	            }
	        }
	        else if(b_pixel == 24){
		    if(b_order == MSBFirst){
			*data++ = (pixel_value >> 16) & 0xff;
			*data++ = (pixel_value >>  8) & 0xff;
			*data++ = (pixel_value      ) & 0xff;
		    }
		    else{
			*data++ = (pixel_value      ) & 0xff;
			*data++ = (pixel_value >>  8) & 0xff;
			*data++ = (pixel_value >> 16) & 0xff;
		    }
		}
		else if(b_pixel == 16){			
		    if(b_order == MSBFirst){
			*data++ = (pixel_value >>  8) & 0xff;
			*data++ = (pixel_value      ) & 0xff;
		    }
		    else{
			*data++ = (pixel_value      ) & 0xff;
			*data++ = (pixel_value >>  8) & 0xff;
		    }
		}
		else{
		    if(b_order == MSBFirst){
			*data++ = (pixel_value      ) & 0xff;
		    }
		    else{
			*data++ = (pixel_value      ) & 0xff;
		    }
		}
	    }
	}
    }
    else{
	image = XCreateImage(Disp,visual,display_depth,ZPixmap,0,NULL,
			cols,rows,32,0);
	if(image == NULL){
	    fprintf(stderr,"Error in creating X Image");
 	    return NULL;
	}	
	b_pixel = image->bits_per_pixel;
  	b_line = image->bytes_per_line;
	b_order = image->byte_order;
	data = (byte*)malloc(rows*b_line);
	if(data == NULL){
	    fprintf(stderr,"\nError in calloc X image data");
	    return NULL;
        }
	image->data = (char*)data;
	if(is_gray == True)
	    r_input = getData_Image(inputImage,0);
	else
	    r_input = getData_Image(c8Image,0);
	switch(display_depth){
	    case 8:
		for(y=0;y<rows;y++){
		    Show_busy(image_window);
		    data = (byte*)(image->data + y*b_line);
		    for(x=0;x<cols;x++){
			*data++ = xview_color[r_input[y][x]];
		    }
		}
		break;
	    case 4:
		if(b_pixel == 4){
		    if(b_order == LSBFirst){
		        for(y=0;y<rows;y++){
			    Show_busy(image_window);
			    data = (byte*)(image->data + y*b_line);
			    for(x=0;x<cols;x++){
			        if(x&1){
				    *data = *data + xview_color[r_input[y][x]] << 4;
				    data++;
			        }
			        else
				    *data = xview_color[r_input[y][x]];
			    }
			} 
		    }
		    else{
		        for(y=0;y<rows;y++){
			    Show_busy(image_window);
			    data = (byte*)(image->data + y*b_line);
			    for(x=0;x<cols;x++){
			        if(x&1){
				    *data = *data + xview_color[r_input[y][x]];
				    data++;
			        }
			        else
				    *data = xview_color[r_input[y][x]] << 4;
			    }
			} 
		    }
		}
		else if(b_pixel == 8){
		    for(y=0;y<rows;y++){
		        data = (byte*)(image->data + y*b_line);
			Show_busy(image_window);
		        for(x=0;x<cols;x++){
			    *data++ = xview_color[r_input[y][x]];
		        }
	 	    }
		}
		break;
	    case 16:
		for(y=0;y<rows;y++){
		    Show_busy(image_window);
		    data16 = (unsigned short*)(image->data + y*b_line);
		    for(x=0;x<cols;x++){
			*data16++ = xview_color[r_input[y][x]]&0xffff;
		    }
	 	}
		break;
	    case 24:
	    case 32:    
		for(y=0;y<rows;y++){
		    Show_busy(image_window);
		    data32 = (unsigned long*)(image->data + y*b_line);
		    for(x=0;x<cols;x++){
			*data16++ = xview_color[r_input[y][x]]&0xffff;
		    }
	 	}
		break;
	    default:
		fprintf(stderr,"Sorry, no way to handle your screen.");
		return NULL;
	}
    } 
    Restore_cursor(image_window);
    return image;
}
    

static void find_best_visual(void){
    XVisualInfo	* vis_info,demand_vis;
    unsigned long flags;
    int choice = -1,num,i,j;

    demand_vis.class = TrueColor;
    demand_vis.screen = screen_num;
    flags = VisualClassMask | VisualScreenMask;

    vis_info = XGetVisualInfo(Disp,flags,&demand_vis,&num);
    if(vis_info){
	for(i=0;i<num;i++){
	    if(vis_info[i].depth >= 24)
		choice = i;
	    if(choice != -1 && vis_info[i].colormap_size > 
			vis_info[choice].colormap_size)
		choice = i;
	}
    }
    if(choice == -1){
        if(vis_info)
	    XFree((char*)vis_info);
        demand_vis.class = DirectColor;
	vis_info = XGetVisualInfo(Disp,flags,&demand_vis,&num);
	if(vis_info){
	    for(i = 0; i < num; i++){
		if(vis_info[i].depth >= 24)
		    choice = i;
		if(choice != -1 && vis_info[i].colormap_size > 
			vis_info[choice].colormap_size)
		    choice = i;
	    }
	}
    }
    if(choice != -1){
	if(vis_info[choice].visualid == XVisualIDFromVisual(visual))
	    return;
   	visual = (Visual*)malloc(sizeof(Visual));
        memcpy(visual,vis_info[choice].visual,sizeof(Visual));
	default_visual = False;
	default_colormap = False;
	bit8mode = False;
	image_gc = NULL;
	screen_num = vis_info[choice].screen;
	root_window = RootWindow(Disp,screen_num);
	display_depth = vis_info[choice].depth;
	display_cells = vis_info[choice].colormap_size;
	colormap = XCreateColormap(Disp,root_window,visual,AllocNone);
	if(vis_info[choice].class == DirectColor)
	    Define_direct_color();
	XSync(Disp,False);
    }
    if(vis_info)
	XFree((char*)vis_info);
}
	
void Define_direct_color(void){
    XColor color;
    int i;
    int cmaplen;
    unsigned long rmask,gmask,bmask;
    int warning = -1;
 
    num_direct = cmaplen = visual->map_entries;
    direct_warning = (int *)calloc(num_direct,sizeof(int));
    rmask = visual->red_mask;
    gmask = visual->green_mask;
    bmask = visual->blue_mask;
    rmask = rmask >> bit_shift(rmask);
    gmask = gmask >> bit_shift(gmask);
    bmask = bmask >> bit_shift(bmask);
    color.flags = DoRed | DoGreen | DoBlue;
    for(i = 0; i < cmaplen; i++){
	color.red = color.green = color.blue = (i * 0xffff) / (cmaplen - 1);
	color.red &= rmask;
	color.green &= gmask;
	color.blue &= bmask;
        if(!XAllocColor(Disp,colormap,&color)){
	     if(i > 1)
     	        direct_warning[i] = (direct_warning[i-1]) ? 
					direct_warning[i-1]:i-1;
	     else
		warning = i;
	}
    }
    if(warning == 0){
	i = 1;
	while(direct_warning[i++]);
	warning = i--;
	while(i >= 0)direct_warning[i--] = warning;
    }
    else if(warning == 1){
	i = 2;
	while(direct_warning[i] !=1 && direct_warning[i] != 0)i++;
	warning = i--;
	while(i > 0)direct_warning[i--] = warning;
    }
}

    
    
    
Image * Resize_Image(Image *cvipImage,int row,int col){
    Image *resultImage;
    int rows, cols, bands, no_of_bands,y,x;
    byte **imaget,**images;
    int total;
    float ratio_x, ratio_y;
    int i,j;

    bands = getNoOfBands_Image(cvipImage);   
    rows =  getNoOfRows_Image(cvipImage);   
    cols =  getNoOfCols_Image(cvipImage);
#ifndef WEI
    ratio_x = (float)cols/col;
    ratio_y = (float)rows/row;
#else
    ratio_x = (float)inputWidth/col;
    ratio_y = (float)inputHeight/row;
#endif
    resultImage = new_Image(cvipImage->image_format,cvipImage->color_space,
			bands,row,col,CVIP_BYTE,cvipImage->bandP[0]->data_format);
    if(resultImage == NULL){
	fprintf(stderr,"Error in allocating new image");
	return NULL;
    }
    for(no_of_bands=0;no_of_bands < bands;no_of_bands++){
	images = getData_Image(cvipImage,no_of_bands);
	imaget = getData_Image(resultImage,no_of_bands);
        for(y=0;y<row;y++){
	    Show_busy(image_window); 
            for(x=0;x<col;x++){
#ifndef WEI
                imaget[y][x] = images[(int)(y*ratio_y)][(int)(x*ratio_x)];
#else
		imaget[y][x] = 
		  images[(int)(y*ratio_y*ivsMatrix[0][0] + x*ratio_x *
		  ivsMatrix[0][1] + ivsMatrix[0][2])][
		  (int)(y*ratio_y*ivsMatrix[1][0] + x*ratio_x * ivsMatrix[1][1]
		  + ivsMatrix[1][2])];
#endif
             }
	}
    }
    if(visual->class != TrueColor && visual->class != DirectColor)
        Restore_cursor(image_window);
    return resultImage;
}

void MakeSure_Bin(Image* inputImage){ int rows, cols, x,y,bands,no_of_bands;
    byte ** image_data;
    int temp;
 
    rows = inputImage->bandP[0]->rows;
    cols = inputImage->bandP[0]->cols;
    bands = inputImage->bands;
    for(no_of_bands=0;no_of_bands<bands;no_of_bands++){
	image_data = getData_Image(inputImage,no_of_bands);
	for(y=0;y<rows;y++){
	    for(x=0;x<cols;x++){
		if(image_data[y][x] != 0)
		     image_data[y][x] = 255;
	    }
	}
    }
}

#ifdef WEI
void ShowCrop_Frame(XButtonEvent *event)
{
    Window root,child;
    XEvent cropEvent;
    unsigned int crop_mask;
    int x, y, left=-1, top=-1, x_point, y_point, bx, by, width=-1, height=-1;
    int BPress, new = True;
    XGCValues *gcvPtr;

    grab_cursor = XCreateFontCursor(Disp,XC_hand2);
    if (event->type == ButtonPress) {
        bx = event->x;
        by = event->y;
        BPress = True;
    }
    while(BPress == True){
        XWindowEvent(Disp,image_window,
            PointerMotionMask|ButtonReleaseMask,&cropEvent);
	if (Selected == True && new == True){
	   Draw_Frame(M_col_box, M_row_box, M_width_box-1, M_height_box - 1); 
           XQueryPointer(Disp,image_window,&root,&child,&x,&y,
                &x_point, &y_point, &crop_mask);
	   x_point = (int) (x_point/Col_aspect); 
	   y_point = (int) (y_point/Row_aspect);
	   if (x_point >= M_col_box && x_point < M_col_box + M_width_box &&
	       y_point >= M_row_box && y_point < M_row_box + M_height_box) {
	       left = M_col_box * Col_aspect;
	       top = M_row_box * Row_aspect;
               width = (M_width_box - 1) * Col_aspect;
               height = (M_height_box - 1) * Row_aspect;
               XDefineCursor(Disp,image_window,grab_cursor);
	       }
	   else Selected = False;
	   }
        if (new == True) new = False;
        else Draw_Frame((int)(left/Col_aspect), (int)(top/Row_aspect),
	    (unsigned int)(width/Col_aspect),(unsigned int)(height/Row_aspect));
        switch (cropEvent.type) {
        case MotionNotify:
            XQueryPointer(Disp,image_window,&root,&child,&x,&y,
                &x_point, &y_point, &crop_mask);
	    if (Selected == False) {
	      if (x_point < bx ) {
		left = (x_point >= 0) ? x_point : 0;
		width = bx - left;
	      }else {
		left = bx;
		width =(x_point>=ximage->width)?ximage->width-bx-1:x_point-bx;
	      }
	      if (y_point < by ) {
		top = (y_point >= 0) ? y_point : 0;
		height = by - top;
	      }else {
	       top = by;
	       height =(y_point>=ximage->height)?ximage->height-by-1:y_point-by;
	      }
	    } else {
              bx = x_point - bx; by = y_point - by;
	      if (left + bx < 0) {left = 0; bellWarn();}
	      else if (left + bx > ximage->width - width - 1) 
		     {left = ximage->width - width - 1; bellWarn();}
              else left += bx;
	      if (top + by < 0) {top = 0; bellWarn();}
	      else if (top + by > ximage->height - height - 1) 
		     {top = ximage->height - height - 1; bellWarn();}
              else top += by;
	      bx = x_point; by = y_point;

	    }
            Draw_Frame((int)(left/Col_aspect), (int)(top/Row_aspect),
	    (unsigned int)(width/Col_aspect),(unsigned int)(height/Row_aspect));
	    XSync(Disp, False);
            break;
        case ButtonRelease:
            XDefineCursor(Disp,image_window,image_cursor);
            BPress = False;
	    if (left == -1 && top == -1 && width == -1 && height == -1) {
	      Selected = False;
              XSetSelectionOwner(Disp, XA_PRIMARY, None, CurrentTime);
	    } else {
	    Selected = True;
            Draw_Frame((int)(left/Col_aspect), (int)(top/Row_aspect),
	    (unsigned int)(width/Col_aspect),(unsigned int)(height/Row_aspect));
	     M_row_box = (int)(top/Row_aspect);M_col_box=(int)(left/Col_aspect);
             M_width_box=(int)(width/Col_aspect) + 1; 
	     M_height_box = (int)(height/Row_aspect) + 1;
	     SelType = BOX;
            XSetSelectionOwner(Disp, XA_PRIMARY, image_window, CurrentTime);
            if (XGetSelectionOwner(Disp, XA_PRIMARY) != image_window){
            fprintf(stderr, "failed to own selection\n");
	    }
        }
            break;
        }
    }
}       
    
void Draw_Frame(int x, int y, int width, int height)
{       
    int anchor = 5;
    static int xx = 1, yy = 1, ww =1,  hh = 1;
    XSetFunction(Disp, image_gc, GXinvert);
    if (height == -1 && width == -1) {
        x = xx; y = yy; width = ww; height = hh;
    }
    x = (int) (Col_aspect * x); y = (int) (Row_aspect * y);
    width = (int) (Col_aspect * width); height = (int) (Row_aspect * height);
    XDrawRectangle(Disp, image_window, image_gc, x, y, width, height);
    if (width > 2*anchor + 2 && height > 2*anchor + 2) {
        XFillRectangle(Disp, image_window, image_gc, x + 1, y + 1,
            anchor, anchor);
        XFillRectangle(Disp, image_window, image_gc,
            x+width-anchor, y + 1, anchor, anchor);
        XFillRectangle(Disp, image_window, image_gc,
            x+1, y+height-anchor, anchor, anchor);
        XFillRectangle(Disp, image_window, image_gc,
        x + width - anchor, y+height-anchor, anchor, anchor);
    }
    XSetFunction(Disp, image_gc, GXcopy);
    yy = y; xx = x; ww = width; hh = height;
    XSync(Disp, False);
}       
    
void Send_Selection(XEvent event){
    int ret;
    char str[NAME_LEN + 5 * DATA_LEN + 1];
    char tmp_str[DATA_LEN];
    XSelectionEvent *ev_ptr = (XSelectionEvent *)
	malloc(sizeof(XSelectionEvent));
    
    strcpy(str, Image_Name); strcat(str, " MOUSE_COORDS ");
    if (SelType == NONE) { 
       sprintf(tmp_str, "%d ", M_row_box); strcat(str, tmp_str);
    } else if (SelType == BOX) {
      sprintf(tmp_str, "%d ", M_row_box); strcat(str, tmp_str);
      sprintf(tmp_str, "%d ", M_col_box); strcat(str, tmp_str);
      sprintf(tmp_str, "%d ", M_height_box); strcat(str, tmp_str);
      sprintf(tmp_str, "%d ", M_width_box); strcat(str, tmp_str);
    } else if (SelType == POINT) {
      sprintf(tmp_str, "%d ", M_row_point); strcat(str, tmp_str);
      sprintf(tmp_str, "%d ", M_col_point); strcat(str, tmp_str);
      sprintf(tmp_str, "%d ", 0); strcat(str, tmp_str);
      sprintf(tmp_str, "%d ", 0); strcat(str, tmp_str);
    }
    ret = XChangeProperty(event.xselectionrequest.display,
    event.xselectionrequest.requestor, event.xselectionrequest.property,
        XA_STRING,8, PropModeReplace, (const unsigned char*) str,
	(int) strlen(str)+1);
    
    ev_ptr->display   = event.xselectionrequest.display;
    ev_ptr->property  = event.xselectionrequest.property;
    ev_ptr->selection = event.xselectionrequest.selection;
    ev_ptr->target    = event.xselectionrequest.target;
    ev_ptr->type      = SelectionNotify;
    ev_ptr->requestor = event.xselectionrequest.requestor;
    ev_ptr->time      = event.xselectionrequest.time;
    ev_ptr->send_event = True;

    ret=XSendEvent(Disp, event.xselectionrequest.requestor,
        True,0L, (XEvent*) ev_ptr);
 
}

int MoveKey(KeySym ks){
  int  i = MV_NONE;
  if (ks==XK_Up || ks==XK_KP_Up) i=MV_UP;

  else if (ks==XK_Down  || ks==XK_KP_Down) i=MV_DOWN;

  else if (ks==XK_Left  || ks==XK_KP_Left) i=MV_LEFT;

  else if (ks==XK_Right || ks==XK_KP_Right) i=MV_RIGHT;
  return i;
}

void chgSelection(int d_row, int d_col, int shift){
  if (Selected == False) {
    bellWarn();
    return;}

  Draw_Frame(M_col_box, M_row_box, M_width_box-1, M_height_box- 1);
  if (shift) {
     if(M_width_box+d_col > 1 && M_col_box+M_width_box+d_col <= inputWidth)
         M_width_box += d_col;
     else bellWarn();
     if(M_height_box+d_row>1&& M_row_box+M_height_box+d_row<= inputHeight)
         M_height_box += d_row;
     else bellWarn();
  }
  else {
     if(M_col_box+d_col >= 0 && M_col_box+d_col + M_width_box <= inputWidth)
        M_col_box += d_col;
     else bellWarn();
     if(M_row_box+d_row>=0 && M_row_box+d_row+M_height_box<=inputHeight)
        M_row_box += d_row;
     else bellWarn();
  }
  Draw_Frame(M_col_box, M_row_box, M_width_box-1, M_height_box - 1);
}

void bellWarn(void){
   XKeyboardControl *kc = (XKeyboardControl *)malloc(sizeof(XKeyboardControl));
   kc->bell_pitch = 12000;
   kc->bell_duration = 80;
   XChangeKeyboardControl(Disp, KBBellPitch|KBBellDuration, kc);
   XBell(Disp,-90);
}

void chgImgSize(int w, int h, int show, int confirm){
if(h != xdisp_Image->bandP[0]->rows || w != xdisp_Image->bandP[0]->cols ||
confirm){
    if(xdisp_Image != NULL) delete_Image(xdisp_Image);
    Row_aspect = (float ) h / inputHeight;
    Col_aspect = (float ) w / inputWidth;
    xdisp_Image = Resize_Image(store_Image,h,w);
    if(bit8mode == True && is_gray == False){
        if(c8Image != NULL) delete_Image(c8Image);
        c8Image = Resize_Image(store_8Image,h,w);
    }
    ximage = CVIPtoXImage(xdisp_Image);
    if (show) Draw_Image(0,0,-1,-1);
    }
}

void reRatio(unsigned int *wid, unsigned int *hei, float ratio, int flag){
   unsigned int width, height;
   int oldw = *wid, oldh = *hei;

   if (((float) oldw)/oldh == ratio) return;
   if (flag) {
      if ( ((float) oldw)/oldh > ratio) {
	 width = (oldw < root_width) ? oldw : root_width;
	 height = (unsigned int) (oldw * 1.0 / ratio);
	 if (height > root_height) {
             height = root_height, 
	     width = (unsigned int)(height * ratio);
	 }
       } else  {
	 height = (oldh < root_height) ? oldh : root_height;
	 width = (unsigned int) (oldh * ratio);
	 if (width > root_width) {
	     width = root_width;
	     height = (unsigned int)(width * 1.0 / ratio);
	 }
       }
   } else {
      if ( ((float) oldw)/oldh > ratio) {
	 height = (oldh < root_height) ? oldh : root_height;
	 width = (unsigned int) (oldh * ratio);
	 if (width > root_width) {
	     width = root_width;
	     height = (unsigned int)(width * 1.0 / ratio);
	 }
       } else  {
	 width = (oldw < root_width) ? oldw : root_width;
	 height = (unsigned int) (oldw * 1.0 / ratio);
	 if (height > root_height) {
             height = root_height, 
	     width = (unsigned int)(height * ratio);
	 }
       }
   }
  Row_aspect = (float) height / inputHeight;
  Col_aspect = (float) width / inputWidth;
   chgImgSize(width, height, False, False);
   *wid = width; *hei = height;
}

Image *rotate90(int flag){
    Image *resultImage;
    int rows, cols, bands, no_of_bands,y,x, cv[3][3];
    unsigned int row, col, mid_in;
    byte **imaget,**images;
    float mid;
    
    bands = getNoOfBands_Image(store_Image);   
    rows =  getNoOfRows_Image(store_Image);   
    cols =  getNoOfCols_Image(store_Image);

    if (evenRotate == True) {
      row = cols*Col_aspect; col = rows*Row_aspect;evenRotate = False;
      }
    else {row = rows*Col_aspect; col = cols*Row_aspect; evenRotate = True;}
    mid = Row_aspect; Row_aspect = Col_aspect; Col_aspect = mid;
    mid_in = inputWidth; inputWidth = inputHeight; inputHeight = mid_in; 
    As_ratio = 1.0/As_ratio;

    cv[0][0] = 0; cv[0][1] = 1 ; cv[0][2] = 0;
    cv[1][0] =-1; cv[1][1] = 0 ; cv[1][2] = inputWidth-1;
    cv[2][0] = 0; cv[2][1] = 0 ; cv[2][2] = 1;

    if (flag == ROT_C_CLK)
       {cv[0][1] = -1; cv[1][0] = 1;
       cv[0][2] = inputHeight-1; cv[1][2] = 0;}

    multiMatrix(cv, transMatrix);
    inverseMatrix(transMatrix, ivsMatrix);

    resultImage = new_Image(store_Image->image_format,store_Image->color_space,
			bands,row,col,CVIP_BYTE,store_Image->bandP[0]->data_format);
    if(resultImage == NULL){
	fprintf(stderr,"Error in allocating new image");
	return NULL;
    }
    for(no_of_bands=0;no_of_bands < bands;no_of_bands++){
	images = getData_Image(store_Image,no_of_bands);
	imaget = getData_Image(resultImage,no_of_bands);
        for(y=0;y<row;y++){
	    Show_busy(image_window);
            for(x=0;x<col;x++){
                imaget[y][x] =
                 images[(int)(y/Row_aspect*ivsMatrix[0][0] + x/Col_aspect*
                 ivsMatrix[0][1] + ivsMatrix[0][2])][
                 (int)(y/Row_aspect*ivsMatrix[1][0]+x/Col_aspect*ivsMatrix[1][1]
                  + ivsMatrix[1][2])];
	    }
	}
    }
    if(Selected) {
    int mid_r, mid_c;
    mid_in = M_width_box; M_width_box = M_height_box; M_height_box = mid_in;
    if (flag == ROT_CLK) {mid_r = M_row_box + M_width_box-1; mid_c = M_col_box;}
    else {mid_r = M_row_box; mid_c = M_col_box + M_height_box - 1;}
    M_row_box = mid_r*cv[0][0] + mid_c*cv[0][1] + cv[0][2];
    M_col_box = mid_r*cv[1][0] + mid_c*cv[1][1] + cv[1][2];

    mid_r = M_row_point; mid_c = M_col_point;
    M_row_point = mid_r*cv[0][0] + mid_c*cv[0][1] + cv[0][2];
    M_col_point = mid_r*cv[1][0] + mid_c*cv[1][1] + cv[1][2];
    }
    if(visual->class != TrueColor && visual->class != DirectColor)
        Restore_cursor(image_window);
    return resultImage;
}

Image *flipHV(int flag){
    Image *resultImage;
    int rows, cols, bands, no_of_bands,y,x, cv[3][3];
    unsigned int row, col;
    byte **imaget,**images;
    
    bands = getNoOfBands_Image(store_Image);   
    rows =  getNoOfRows_Image(store_Image);   
    cols =  getNoOfCols_Image(store_Image);
    col = inputWidth*Col_aspect; row = inputHeight*Row_aspect;
    As_ratio = 1.0/As_ratio;

    cv[0][0] = 1; cv[0][1] = 0 ; cv[0][2] = 0;
    cv[1][0] = 0; cv[1][1] =-1 ; cv[1][2] = inputWidth-1;
    cv[2][0] = 0; cv[2][1] = 0 ; cv[2][2] = 1;

    if (flag == FLP_V)
       {cv[0][0] = -1; cv[0][2] = inputHeight - 1; cv[1][1] = 1; cv[1][2] = 0;}

    multiMatrix(cv, transMatrix);
    inverseMatrix(transMatrix, ivsMatrix);

    resultImage = new_Image(store_Image->image_format,store_Image->color_space,
			bands,row,col,CVIP_BYTE,store_Image->bandP[0]->data_format);
    if(resultImage == NULL){
	fprintf(stderr,"Error in allocating new image");
	return NULL;
    }
    for(no_of_bands=0;no_of_bands < bands;no_of_bands++){
	images = getData_Image(store_Image,no_of_bands);
	imaget = getData_Image(resultImage,no_of_bands);
        for(y=0;y<row;y++){
	    Show_busy(image_window);
            for(x=0;x<col;x++){
                imaget[y][x] =
                 images[(int)(y/Row_aspect*ivsMatrix[0][0] + x/Col_aspect*
                 ivsMatrix[0][1] + ivsMatrix[0][2])][
                 (int)(y/Row_aspect*ivsMatrix[1][0]+x/Col_aspect*ivsMatrix[1][1]
                  + ivsMatrix[1][2])];
	    }
	}
    }
    if(Selected) {
    int mid_r, mid_c;
    if (flag == FLP_V) {mid_r = M_row_box + M_height_box-1; mid_c = M_col_box;}
    else {mid_r = M_row_box; mid_c = M_col_box + M_width_box - 1;}
    M_row_box = mid_r*cv[0][0] + mid_c*cv[0][1] + cv[0][2];
    M_col_box = mid_r*cv[1][0] + mid_c*cv[1][1] + cv[1][2];

    mid_r = M_row_point; mid_c = M_col_point;
    M_row_point = mid_r*cv[0][0] + mid_c*cv[0][1] + cv[0][2];
    M_col_point = mid_r*cv[1][0] + mid_c*cv[1][1] + cv[1][2];
    }
    if(visual->class != TrueColor && visual->class != DirectColor)
        Restore_cursor(image_window);
    return resultImage;
}

Image *cropImage(Image *cvipImage, int row, int col,
            unsigned int *w, unsigned int *h){
    Image *resultImage;
    int rows, cols, bands, no_of_bands,y,x;
    unsigned int width = *w, height = *h;
    byte **imaget,**images;

    bands = getNoOfBands_Image(cvipImage);   
    rows =  getNoOfRows_Image(cvipImage);   
    cols =  getNoOfCols_Image(cvipImage);

/*    width = *w = (width + col > cols) ? cols - col : width;
    height = *h = (height + row > rows) ? rows - row : height; */
    width = *w = width;
    height = *h = height;
    resultImage = new_Image(cvipImage->image_format,cvipImage->color_space,
		bands,height,width,CVIP_BYTE,cvipImage->bandP[0]->data_format);
    if(resultImage == NULL){
	fprintf(stderr,"Error in allocating new image");
	return NULL;
    }
    for(no_of_bands=0;no_of_bands < bands;no_of_bands++){
	images = getData_Image(cvipImage,no_of_bands);
	imaget = getData_Image(resultImage,no_of_bands);
        for(y=0;y<height;y++){
	    Show_busy(image_window);
            for(x=0;x<width;x++)
		imaget[y][x] =
                   images[(y+row)*ivsMatrix[0][0] + (x+col)*ivsMatrix[0][1] +
		   ivsMatrix[0][2]][ (y+row)*ivsMatrix[1][0] +
		   (x+col)*ivsMatrix[1][1] + ivsMatrix[1][2]];
	}
    }
    if(visual->class != TrueColor && visual->class != DirectColor)
        Restore_cursor(image_window);
    return resultImage;
}

void inverseMatrix(int input[][3], int output[][3]){
int D;
D =
  -input[0][2]*input[1][1]*input[2][0] + input[0][1]*input[1][2]*input[2][0]
  + input[0][2]*input[1][0]*input[2][1] - input[0][0]*input[1][2]*input[2][1]
  - input[0][1]*input[1][0]*input[2][2] +input[0][0]*input[1][1]*input[2][2];

output[0][0] = (-(input[1][2]*input[2][1]) + input[1][1]*input[2][2])/(float)D; 
output[0][1] = (input[0][2]*input[2][1] - input[0][1]*input[2][2])/(float)D;
output[0][2] = ((-input[0][2]*input[1][1]) + input[0][1]*input[1][2])/(float)D; 
output[1][0] = (input[1][2]*input[2][0] - input[1][0]*input[2][2])/(float)D; 
output[1][1] = (-(input[0][2]*input[2][0]) + input[0][0]*input[2][2])/(float)D; 
output[1][2] = (input[0][2]*input[1][0] - input[0][0]*input[1][2])/(float)D; 
output[2][0] = (-(input[1][1]*input[2][0]) + input[1][0]*input[2][1])/(float)D; 
output[2][1] = (input[0][1]*input[2][0] - input[0][0]*input[2][1])/(float)D; 
output[2][2] = (-(input[0][1]*input[1][0]) + input[0][0]*input[1][1])/(float)D;
}

void multiMatrix(int a[][3], int b[][3]) {
int i,j,k;
int c[3][3];
for (i = 0; i < 3; i++) 
  for (j = 0; j < 3; j++)
    {
    c[i][j] = 0;
    for (k = 0; k < 3; k++) c[i][j] += a[i][k] * b[k][j];
    }
for (i = 0; i < 3; i++) 
  for (j = 0; j < 3; j++)
    b[i][j] = c[i][j];
}

void initPara(Image *newimage){
int i, j;
inputWidth = newimage->bandP[0]->cols;
inputHeight = newimage->bandP[0]->rows;
M_row_box = M_col_box = M_row_point = M_col_point = 0;
M_width_box = M_height_box = 0;
As_ratio = ((float) inputWidth) / inputHeight;
Selected = False;
evenRotate = True;
for (i = 0; i < 3; i++)
 for (j = 0; j < 3; j++) transMatrix[i][j] = ivsMatrix[i][j] = (i==j) ? 1 : 0;
}
#endif


static void
data_range(Image *imageP,  char *range) {
   CVIP_TYPE     dt;
   register    int i;
   unsigned    no_of_bands, imgsize, rows, cols;
   double      *minP = NULL, *maxP = NULL,
					 min, max;
   void     **imgvecP = NULL, **imgvecPi = NULL;
	char temp[20]={'\0'};
	FORMAT format;

	dt = getDataType_Image(imageP);
	format = getDataFormat_Image(imageP);

   no_of_bands = getNoOfBands_Image(imageP);
   rows = getNoOfRows_Image(imageP);
   cols = getNoOfCols_Image(imageP);
   imgsize = rows*cols;

   minP  = (double *) malloc(sizeof(double)*no_of_bands);
   maxP  = (double *) malloc(sizeof(double)*no_of_bands);
   imgvecP = (void **) malloc(sizeof(void *)*no_of_bands);
   getBandVector_Image((Image *)imageP, imgvecP);

   if(format == COMPLEX) {
		int count = 0;
   	imgvecPi = (void **) malloc(sizeof(void *)*no_of_bands);
		for (count = 0; count < no_of_bands; count ++) 
			imgvecPi[count] = getImagRow_Image(imageP, 0, count);
   }

   for(i=0; i < no_of_bands; i++) {
      minP[i] = findMinVal_Vector(imgvecP[i], imgsize, dt);
      maxP[i] = findMaxVal_Vector(imgvecP[i], imgsize, dt);
		if(format == COMPLEX) {
			min = minP[i];
      	minP[i] = findMinVal_Vector(imgvecPi[i], imgsize, dt);
			minP[i] = (min > minP[i]) ? minP[i] : min;
			max = maxP[i];
			maxP[i] = findMaxVal_Vector(imgvecPi[i], imgsize, dt);
			maxP[i] = (max < maxP[i]) ? maxP[i] : max;
		}
   }  
	min = minP[0]; 
	max = maxP[0]; 
	for (i = 1; i < no_of_bands; i++) {
		min = MIN(min, minP[i]);
		max = MAX(max, maxP[i]);
	}
	strcat(range, "[");
	sprintf(temp, "%.0f, ", min); strcat(range, temp);
	sprintf(temp, "%.0f", max); strcat(range, temp);
	strcat(range,"]");

	free(minP); free(maxP);
	free(imgvecP); 
	if(format == COMPLEX) {
		free(imgvecPi);
	}
}

void justwait(int signum) {
	signal(SIGUSR1, justwait);
	printf("get child\n");
	/*wait( (int *)NULL); */
	return;
}
void notifyParrent(void) {
	kill(getppid(), SIGUSR1);
}



