/***************************************************************************
* =========================================================================
*
*   Computer Vision and Image Processing Lab - Dr. Scott Umbaugh SIUE
* 
* =========================================================================
*
*             File Name: CVIPlab.c 
*           Description: This  is the skeleton program for the Computer Vision
*			 and Image Processing Labs
*   Initial Coding Date: April 23, 1996
*           Portability: Standard (ANSI) C
*             Credit(s): Zhen Li & Kun Luo
*                        Southern Illinois University at Edwardsville
****************************************************************************/

/*
** include header files
*/

#include "CVIPtoolkit.h"
#include "CVIPconvert.h"
#include "CVIPdef.h"
#include "CVIPimage.h"
#define CASE_MAX 2

/* Put the command here, as VIDEO_APP, to run your image acquisition 
application program */

#define VIDEO_APP "SunVideo &"

/* Define the image viewer program here as VIEWER. In CVIPtools under 
** UNIX you can use the disk based viewer called picture, or the RAM-based 
** viewer RamViewer. With Windows95, you can use any disk-based image viewer 
** that will display images from the command line. That is, it will work 
** when run from a system shell and has the format 'viewer <image_file_name>'.
*/

#define VIEWER "picture"

#include "CVIPlab.h"

/*
** function declarations
*/
		    
Image *threshold_Setup(Image *inputImage);
Image *input();

/*
** start main function
*/

void main(){
    IMAGE_FORMAT  format;     	/* the input image format */
    Image	  *cvipImage; 	/* pointer to the CVIP Image structure */
    char	  *outputfile;	/* output file name */	
    int		  choice;
    CVIP_BOOLEAN  done = CVIP_NO; /* end of menu loop variable */

/* Set the image viewer  
** 'RamViewer' is the CVIPtools RAM-based viewer for UNIX systems. 
** 'picture' is the CVIPtools disk-based viewer for UNIX.  
** 'xv' and 'imagetool' are other options on SUN systems.
** Any image viewer will work that can be run from a system shell  
** and has the format 'viewer <image_file_name>'.  When an external
** viewer is used, the second parameter fo setDisplay_Image() is 
** used to specify the display image format, which is selected 
** based on the viewer. For example, if the viewer can only display
** "PPM" images, all images will be converted to PPM format before
** being displayed. VIEWER is defined at the top of the CVIPlab program.   
*/

    setDisplay_Image(VIEWER, "Default");

    print_CVIP("\n\n\n\n***************************************");
    print_CVIP("****************************  ");
    print_CVIP("\n*\t\t Computer Vision and Image Processing Lab\t  *");
    print_CVIP("\n*\t\t\t <Insert your name here> \t\t  *");
    print_CVIP("\n******************************************");
    print_CVIP("*************************\n\n\n");
 
    while(!done) {  
	print_CVIP("\t\t0.\tExit \n\n");
  	print_CVIP("\t\t1.\tGrab and Snap an Image  \n\n");
  	print_CVIP("\t\t2.\tThreshold Operation \n\n");
  	print_CVIP("\n\nCVIPlab>>"); 

	/* 
	** obtain an integer between 0 and CASE_MAX from the user
	*/
 	choice = getInt_CVIP(10, 0, CASE_MAX);

 	switch(choice) {

	    case 0: 
		done=CVIP_YES;
		break;
 
	    case 1: 
		/* invokes the Video Application */
		system(VIDEO_APP); 
		print_CVIP("\n\tThe Video Application is being invoked\n\n");
		print_CVIP("\t\t\tPlease Wait....\n\n");
		sleep(6);/* keeps the system idle for 6 seconds */
		break;
	
	    case 2: 
		/*Get the input image */
		cvipImage = input();
		if(cvipImage == NULL) {
			error_CVIP("main", "could not read input image");
			break;
		}

		/* calls the threshold function */
		cvipImage = threshold_Setup(cvipImage); 
		if (!cvipImage) {
			perror_CVIP("main", "threshold fails");
			break;
		}
		print_CVIP("\n\t\tEnter the Output File Name:  ");
		outputfile = getString_CVIP();

		/*
		** display the resultant image
		*/
		view_Image(cvipImage,outputfile);

		/* 
		** saves the resulting image data out to disk in <outputfile>
		*/	
		format = getFileFormat_Image(cvipImage);
		write_Image(cvipImage, outputfile, CVIP_NO, CVIP_NO, format, 1);

		/*
		**IMPORTANT: free the dynamic allocated memory when it is not 
		** 	     needed 
		*/
		free(outputfile); 
	        break;

   	    default:
		print_CVIP("Sorry ! You Entered a wrong choice ");
		break;
	}
    }
} 
/* 
** end of the function main 
*/
  

/* 
** The following function reads in the image file specified by the user,
** stores the data and other image info. in a CVIPtools Image structure,
** and displays the image.
*/

Image* input(){   
    char  		*inputfile;
    Image		*cvipImage;

    /* 
    ** get the name of the file and stores it in the string 'inputfile ' 
    */	
    print_CVIP("\n\t\tEnter the Input File Name:  "); 	
    inputfile = getString_CVIP();

    /*  
    ** creates the CVIPtools Image structure from the input file 
    */
    cvipImage = read_Image(inputfile, 1);
    if(cvipImage == NULL) {
	error_CVIP("init_Image", "could not read image file");
	free(inputfile);
	return NULL;
    }

    /*
    ** display the source image
    */
    view_Image(cvipImage,inputfile);

    /*
    **IMPORTANT: free the dynamic allocated memory when it is not needed
    */
    free(inputfile);

    return cvipImage;       
}   


/*
** The following setup function asks the threshold value from the user. After
** it gets the threshold value, it will call the threshold_Image() function. 
*/

Image *threshold_Setup(Image *inputImage){
    unsigned int 	threshval;      /* Threshold value */	
	
    /* 
    ** Gets a value between between 0 and 255 for threshsold 
    */
    print_CVIP("\n\t\tEnter the threshold value:  ");
    threshval = getInt_CVIP(10, 0, 255);

    return threshold_lab(inputImage, threshval);
}

