#!/bin/csh -f
# ======================================================================
# Computer Vision/Image Processing Tool Project - Dr. Scott Umbaugh SIUE
# ======================================================================
#
#             File Name: CVIPsetup.csh
#           Description: This script will set up the programming environment
#                        for a CVIPtools developer by examining their host
#                        environment and unbundling the necessary files to get
#			 them started. The script checks to make sure that
#			 the CVIPHOME environment variable is properly set,
#                        the current working directory is writable, the users
#                        path and MANPATH variable are properly set and also
#			 takes the user through the steps necessary to
#			 construct the example program provided.
#         Related Files: newuser.cpio (contains sample source files)
#   Initial Coding Date: Mon Apr 26 21:54:44 CDT 1993
#           Portability: UNIX C Shell script
#             Author(s): Gregory Hance, Arve Kjoelen
#                        Southern Illinois University @ Edwardsville
#
## Copyright (C) 1993 SIUE - by Gregory Hance.
##
## Permission to use, copy, modify, and distribute this software and its
## documentation for any purpose and without fee is hereby granted, provided
## that the above copyright notice appear in all copies and that both that
## copyright notice and this permission notice appear in supporting
## documentation.  This software is provided "as is" without express or
## implied warranty.
##
# RCS (Revision Control System) Information
# ...Added automatically by RCS 
#
# $Log: CVIPsetup.csh,v $
# Revision 1.10  1997/04/12 00:12:40  yawei
# changed to use the CVIPlab_cshrc file
#
# Revision 1.9  1997/04/03 14:08:35  akjoele
# Updated and edited for CVIPtools3.5, including making all paths
# relative to $CVIPHOME, so that only the variable $cviph at the top
# of the file needs edited for other environments.
#
# Revision 1.8  1997/01/10 19:02:26  yawei
# threshold.c ==> threshold_lab.c
#
# Revision 1.7  1997/01/09  23:00:59  yawei
# added
#   CVIPlab.h
#   threshold.c
# file copies
#
# Revision 1.6  1996/11/04  18:44:52  akjoele
# Made changes to better account for special cases of user environments
# and paths.
#
# Revision 1.5  1996/08/19  22:17:13  mzuke
# Added XF_LOAD_PATH so users can run GUI version.
#
# Revision 1.4  1996/08/19  20:44:19  akjoele
# Added TCL, TK parameters for version 3.0
# of CVIPtools.
#
# Revision 1.3  1995/10/26  14:41:58  akjoele
# Changed the most obnoxious error messages.  Also changed script so that
# it will refuse to run if CVIPHOME is already defined.  This will prevent
# a user's .cshrc file from containing several copies of the lines added
# by this script.
#
# Revision 1.1  1995/05/26  20:47:09  root
# Initial revision
#
# Revision 1.1  1993/04/29  06:59:13  hanceg
# Initial revision
#
#

unset noclobber
set umask = 022

#
# check to see if the user has a preferred temporary
# directory... and if not use the venerable "/tmp"
#
set cviph = "/opt/cvipapps/CVIPtools"
if (! $?TMPDIR) set TMPDIR = /tmp

set getans = $TMPDIR/$$GET
set hitret = $TMPDIR/$$RET
set sedscript = $TMPDIR/$$SED
set tmpfile = $TMPDIR/$$TMP
set testimage = "/opt/cvipimages/Gray/cameraman.pgm"
set prog = $0
set target_prog = "CVIPlab"
set error = "${prog:t}-error"
set def = ""

if (! -e $prog:t) then
   echo ""
   echo "${error}: Please run $prog:t in the directory in which it resides."
   echo "${error}: In order for this utility to work it needs access to the"
   echo "${error}: other files in that directory."
   echo ""
   exit 1
endif

set host = `hostname`
cat << EOF > $getans
if ( \$?FASTREAD ) then
   set ans = ""
else
   set ans = "\!"
endif
while ( "\$ans" =~ \!* )
   echo -n "Your answer[\$def]? "
   set ans = "\$<"
   switch( "\$ans" )
   case \!:
      csh
      echo ""
   breaksw
   case \!*:
      csh -c \`expr "\$ans" : '\!\(.*\)\$'\`
      echo ""
   breaksw
   default:
      continue
   endsw
end
if( "\$ans" == "" ) set ans = "\$def"
EOF
chmod +x $getans

cat << EOF > $hitret
#!/bin/csh -f
echo -n "[Hit  RETURN to continue]"
set bogus = "\$<"
EOF
chmod +x $hitret

cat << EOF

~~~~~~~~~
CVIPtools
~~~~~~~~~
	$prog is a script to aid newcomers to CVIPtools in setting up
their progamming environment.  In order for this program to work
it must have write access to the current directory.
If you get stuck on a question, you may use a ! shell escape to start 
a subshell or execute a command. In most cases, if a problem occurs during
the course of the set-up the problem will automatically be taken care
of in an appropriate manner.  Many of the questions also have default answers in square brackets -- typing carriage return gives you the default. A lot of effort was expended to assure that this script will be thorough in its endeavors
to set up your environment so if any problems occur please contact your T.A..
EOF
set def = "Y"
echo ""
echo "continue with script (Y/N)..."
source $getans
if( "$ans" == "n" || "$ans" == "N" ) then
   echo "Aborting CVIPtools setup at user request"
   exit 1
endif


echo ""
echo "~~~~~~~~~~~~~~~~~~~~~~~"
echo "updating environment..."
echo "~~~~~~~~~~~~~~~~~~~~~~~"
echo ""
echo "checking current environment..."
if($?CVIPHOME) then
echo "Your CVIPHOME environment is already set."
echo "Please ask a TA for assistance if you are running this"
echo "setup script to correct current problems.  Manual"
echo "editing of your .cshrc file will be required"
echo ""
echo "Aborting CVIPtools setup..."
exit 1
endif

echo "appending new environment info. to $home/.cshrc."

setenv 	CVIPHOME	$cviph
cat << EOF >> ~/.cshrc
#CVIP_ENV_BEGIN
setenv 	CVIPHOME	$cviph
source ${cviph}/docs/CVIPlab/CVIPlab_cshrc
#CVIP_ENV_END

#SUNC_ENV_BEGIN
setenv 	LD_LIBRARY_PATH /usr/openwin/lib:/usr/lib:/usr/ucblib:${cviph}/lib
#SUNC_ENV_END
EOF

echo "incorporating new path and environment info."
setenv 	LD_LIBRARY_PATH	/opt/apps2/SUNWspro/lib:/usr/openwin/lib
setenv	LM_LICENSE_FILE	/opt/apps2/SUNWspro/SunTech_License/license.dat
setenv 	MANPATH 	${MANPATH}:/opt/apps2/SUNWspro/man:${cviph}/man
source ${CVIPHOME}/docs/CVIPlab/CVIPlab_cshrc
#
# Check to make sure that the environment variable $CVIPHOME is defined
#

echo ""
echo "~~~~~~~~~~~~~~~~~~~~~~~~~"
echo "validating environment..."
echo "~~~~~~~~~~~~~~~~~~~~~~~~~"
echo ""
cat << EOM
It is important that you have an environment variable defined
as \$CVIPHOME that holds the pathname to the CVIPtools installation.
On one of our Sun machines the \$CVIPHOME environment variable should point to...
$cviph
let's check it out...

EOM
$hitret

if (! $?CVIPHOME) then
echo "the environment variable CVIPHOME does not exist! ...panicking"
exit 1
else
   echo ""
   echo "  /"
   echo "\/ ...environment variable CVIPHOME is set to $CVIPHOME."
   echo ""
endif

if ( ! -e ~/cviplab ) then
   echo "It appears that the directory ~/cviplab does not exist, so"
   echo "I'll go ahead and create it for you...."
   mkdir ~/cviplab
   chmod 755 ~/cviplab
   echo "ok."
else if ( ! -d ~/cviplab ) then
   echo "It appears that ~/cviplab is a FILE, not a directory.  You made"
   echo " a mess!  I'll try to clean it up for you..."
   cp ~/cviplab ~/cviplab.old
   mkdir ~/cviplab
   chmod 755 ~/cviplab
   echo "ok...The old cviplab file has been saved as cviplab.old.  Please"
   echo "take a look at it and delete it if you don't need it."
endif
echo ""
cd ~/cviplab

cat << EOM
It is important that you have write access to the
current directory $cwd. 
In order for this script to unbundle your local installation
of set-up files the current directory must be writable.
let's check it out...

EOM
$hitret

if (! -w "$cwd") then

   echo "It appears that you do not have write permission to the"
   echo "current working directory ($cwd). You must have write"
   echo "permission in order to continue with the script."
   echo "If you own the directory and possibly just do not have the "
   echo "attributes set correctly or if your not sure then invoke a"
   echo "subshell at the prompt and check things out... otherwise exit"
   echo "the script."
   echo "Continue with script (Y/N)..."
   set def = "N"
   source $getans
   if (! -w "$cwd" || "$ans" == "n" || "$ans" == "N" ) then
      echo "...later man\!"
      exit 1
   endif
   echo ""
else
   echo ""
   echo "  /"
   echo "\/ ...$cwd appears to be writable."
   echo ""
endif

cat << EOM
Now that we have your environment set up properly let's copy
the necessary files into the current directory to get you started.

EOM
$hitret
#
#  Second check to see if the cpio archive file exists.  If so then
#  extract the files.  Otherwise error out....
#
cp $cviph/docs/CVIPlab/CVIPlab.c ~/cviplab
cp $cviph/docs/CVIPlab/CVIPlab.h ~/cviplab
cp $cviph/docs/CVIPlab/threshold_lab.c ~/cviplab
cp $cviph/docs/CVIPlab/CVIPlab.tmpl ~/cviplab
cp $cviph/docs/CVIPlab/Imakefile ~/cviplab
cp $cviph/docs/CVIPlab/README.cvip ~/cviplab
chmod u+w ~/cviplab/* >& /dev/null
echo "done."
echo ""


cat << EOM
Now for the fun stuff! The files that were just copied
should consist of the following:

    CVIPlab.tmpl   --   imake template file
       Imakefile   --   imake input file
       CVIPlab.c   --   C source file
       CVIPlab.h   --   C header file
 threshold_lab.c   --   C source file
     
CVIPtools uses a C preprocessor interface to the make utility,
known as "imake" to configure a Makefile tuned to a particular system
 -- in this case for the Sun. Imake builds a Makefile from a
template file listing variables and rules, and combines this with
operating system- and site-specific configuration files.

let's go through the steps necessary to construct
a CVIPtools target program...

EOM
$hitret

cat << EOM

The first step is to generate a Makefile using a simple
interface to the imake utility known as xmkmf.
xmkmf runs the imake command with the  appropriate options
to generate a Makefile from an Imakefile in the current directory.

EOM
$hitret
echo ""
echo "${host}> xmkmf"
xmkmf

cat << EOM

The next step involves creating a dependency list of the source
files. This list is used by "make" to associate header files
with source the files that depend on them. If a source file is
"out of date" with respect to a header file that it depends on
then the source file will be recompiled.

EOM
$hitret
echo ""
echo "${host}> make depend"
make depend

cat << EOM

The final step involves constructing the target program 
-- which in this case is a program named $target_prog.

EOM
$hitret
echo ""
echo "${host}> make"
make

echo ""
echo "if all went well... (i.e. your environment is now set up properly)"
echo "then you can test the program -- a threshold should result in"
echo "a binary image being displayed on the monitor..."
echo "use the image (cameraman.pgm) to test your program."
echo ""
$hitret
ln -s $testimage
echo "${host}> $target_prog"
./CVIPlab
echo ""
echo "Did everything go O.K (Y/N)...."
set def = "Y"
source $getans
if( $ans == "y" || $ans == "Y" ) then
   cat << EOM

Good! You're Set!
One last comment... for the changes you made to your .cshrc file 
to be effective in the current shell you will need
to either "source" it (e.g. $host> source ~/.cshrc) or 
log out and back in again... one of the two. Happy programming!

EOM
exit 0
else
   echo "That's not good - get help from someone\!"
   exit 1
endif
