#!/bin/csh -f
# ======================================================================
# Computer Vision/Image Processing Tool Project - Dr. Scott Umbaugh SIUE
# ======================================================================
#
#             File Name: addlibrary.csh
#           Description: This script will set up a directory with the
#			 appropriate files necessary to create a CVIPtools 
#			 library.
#         Related Files: Imakefile, libtest.c
#   Initial Coding Date: Tue Aug 24 17:25:01 CDT 1993
#           Portability: C Shell script
#             Author(s): Gregory Hance
#                        Southern Illinois University @ Edwardsville
#
## Copyright (C) 1993 SIUE - by Gregory Hance.
##
## Permission to use, copy, modify, and distribute this software and its
## documentation for any purpose and without fee is hereby granted, provided
## that the above copyright notice appear in all copies and that both that
## copyright notice and this permission notice appear in supporting
## documentation.  This software is provided "as is" without express or
## implied warranty.
##
unset noclobber
set umask = 022

if( $#argv > 0 ) then
   echo "usage: $0"
   exit 1
endif

echo -n "Enter the library name? "
set name = "$<"
set suffix = `echo $name | tr A-Z a-z`

if (! $?TMPDIR) set TMPDIR = /tmp

set sedscript = $TMPDIR/$$SS

set template1 = $CVIPHOME/CVIPtools1.0/docs/NewLibrary/Imakefile
set template2 = $CVIPHOME/CVIPtools1.0/docs/NewLibrary/test.c

set file1 = Imakefile
set file2 = t$suffix.c
set file3 = CVIP$suffix.h

#
# write out a set of macros in the form of a "sed" script.
#
cat << EOF > $sedscript
s/@date@/`date`/
s/@name@/$name/g
s/@suffix@/$suffix/g
EOF

if( ! -e $file1 ) then
   sed -f $sedscript $template1 > $file1
else
   echo "Imakefile already exists."
endif

if( ! -e $file2 ) then
   sed -f $sedscript $template2 > $file2
else
   echo "$file2 already exists."
endif

if( ! -e $file3 ) then
   addheader.csh -h -rcs -cvip -cr $file3
else
   echo "$file3 already exists."
endif

rm -f $sedscript
exit 0
