#!/bin/csh -f
# ======================================================================
# Computer Vision/Image Processing Tool Project - Dr. Scott Umbaugh SIUE
# ======================================================================
#
#             File Name: addman.csh
#           Description: creates a man page template that is to be filled
#			 in by the user. This script will create man page
#			 templates geared towards man page types (1) and
#			 (3), User Commands and Library Subroutines
#			 respectively. The script has the added bonus that
#			 it will automatically extract description info. of
#			 library functions if the library function contains
#			 a standard CVIPtools function header for each function.
#			 This version is customized for use with CVIPtools.
#         Related Files: CVIPtools man pages w/ ext. .1 and .3
#			 CVIPfcnheader.txt
#   Initial Coding Date: Sun May  2 13:50:21 CDT 1993
#           Portability: C Shell script
#             Credit(s): Gregory Hance,
#                        Southern Illinois University at Edwardsville
#
## Copyright (C) 1995 SIUE - by Scott Umbaugh and Gregory Hance.
##
## Permission to use, copy, modify, and distribute this software and its
## documentation for any purpose and without fee is hereby granted, provided
## that the above copyright notice appear in all copies and that both that
## copyright notice and this permission notice appear in supporting
## documentation.  This software is provided "as is" without express or
## implied warranty.
##
# RCS (Revision Control System) Information
# ...Added automatically by RCS 
#
## $Log: addman.csh,v $
## Revision 1.9  1997/03/26 23:18:08  yawei
## changed man formats
##
## Revision 1.8  1997/01/27 23:02:34  yawei
## minor change
##
# Revision 1.7  1997/01/27  22:59:01  yawei
# corrected bug.
#
#

#
# unset the shell variable noclobber so that we can overwrite
# existing files using ">" and create new files w/ ">>"
# also set umask so that write access is restricted to the owner
#
unset noclobber
set umask = 022

#
# initialize some local shell variables
#
set printusage = FALSE
set prog = $0
set prog = $prog:t

#
# initialize optional arguments
#
set option = TRUE
set sopt = FALSE
set eopt = FALSE
set bopt = FALSE
set mansuffix = "1"

#
# loop through optional arguments and if a argument
# does not match with an optional argument then let's hope
# it is the filename or man page type
#
while("$option" == "TRUE")
   switch("$1")
   case "-s":
      set sopt = TRUE
      shift
   breaksw
   case "-b":
      set bopt = TRUE
      shift
      if( "$1" != "" ) then 
	 set bname = "$1"
         shift
      endif
   breaksw
   case "-e":
      set eopt = TRUE
      shift
   breaksw
   default:
      set option = FALSE
   endsw
end

#
# extract man page suffix (default is 1)
#
if( $#argv > 1 ) then
   set mansuffix = "$1"
   shift
endif

#
# get name of man page or print usage message if the number
# of arguments do not match up.
#
if( $#argv == 1) then
   set mpname = "$1"
else
   echo "${prog}: usage: $prog [-s] [-e] [-b <file name>] [type] <man_page_name>"
   echo "${prog}: OPTIONAL ARGUMENTS:"
   echo "${prog}: -s    Add beginning info. to man page and that's all"
   echo "${prog}: -b    Extract body of man page from the file <filename>..."
   echo "${prog}:       NOTE: this option should be used in conjunction with"
   echo "${prog}:       the -s and -e options."
   echo "${prog}: -e    Add trailing info. to man page and that's all"
   echo "${prog}: type  specify the man page type..."
   echo "${prog}:       (1) - User Commands, (2) - System Calls, (3) Subroutines"
   echo "${prog}:       (4) - Devices, (5) File Formats, (6) Games, (7) Misc."
   echo "${prog}:       (8) - Sys. Admin. ** Defaults to (1) **"
   echo "${prog}: MANDATORY ARGUMENTS:"
   echo "${prog}: man_page_name   name of man page to create or append to."
   exit 1
endif

#
# get the type of system that we are on
#
switch( "`hostname`" )
   case "brahma":
   case "elvis":
   case "krishna":
   case "buddy":
   case	"dylan":
   case "hendrix":
   case "rama":
   case "plato":
   case "shiva":
   case "vlsimain":
   case "vaughan": 
   case "chorale":
   case "cicero":
   case "conductor":
      set system = "sun"
   breaksw
   case "duet":
      set system = "sgi"
   breaksw
   default:
      set system = "next"
   breaksw
endsw

#
# determine the name of the output file
# appropriate extension will be added
#
set outfile = $1:r.$mansuffix
if( "$sopt" == "FALSE" && "$bopt" == "FALSE" && "$eopt" == "FALSE" ) then
   set sopt = "TRUE"
   set bopt = "FALSE"
   set eopt = "TRUE"
endif

#
# set a flag if the output file already exists
# and is not empty
#
if ( -e $outfile && ! -z $outfile ) set existfile

#
# check to see if the user has a preferred temporary
# directory... and if not use the standard "/tmp"
#
if (! $?TMPDIR) set TMPDIR = /tmp


#
# set a variable to the full name of the user
# the name is extracted from the flat file /etc/passwd
# unless we're on the NeXT and using NetInfo
#
set awkfield = '$5'
if( "$system" != "next" ) then
   set name = `awk -F: "/^$user/{print $awkfield}" /etc/passwd`
else
   set name = `nidump passwd / | awk -F: "/^$user/{print $awkfield}"`
endif


#
# set some shell variables to useful info. that can be substituted
# into the man page
#
set date = `date '+%d %h %y'`
set email = "$user@minuet.siue.edu"
set filename = $outfile:t
set filename = $filename:r
set school = "Southern Illinois University at Edwardsville"
set street = "200 University Park"
set city = "Edwardsville, IL  62026-1801"

#
# if the outputfile exists then warn the user that
# the prized man page is about to be overwritten
#
if( $?existfile && "$sopt" == "TRUE" ) then
      while(1)
         echo "${prog}: output file $outfile already exists and is about"
         echo -n "${prog}: to be overwritten... continue? (y/n)?"
         switch("$<")
         case "n":
            exit 1
         case "y":
            break
         endsw
      end
endif

if ( "$sopt" == "TRUE" ) then
cat << EOF
${prog}: ==========================================================
${prog}: INSTRUCTIONS for filling in this UNIX manual page template
${prog}: ==========================================================
${prog}: text inside <>'s (including the <>'s), is to be replaced
${prog}: with the appropriate text as detailed therein.
${prog}: Leave all other text "AS IS" with the exception of changing
${prog}: some of the examples provided.
${prog}: To test try the following: "nroff -man <file> | more"
EOF
endif

if( "$sopt" == "TRUE"  ) then
   cat << EOF > $outfile
.TH $filename $mansuffix "$date"
.IX $filename
.SH NAME
$filename \- <a brief description of the function>
.SH SYNOPSIS
<command usage>
.de Ss
.sp
.ft B
.nf
..
.de Si
.ft I
.sp
..
.de Se
.fi
.ft R
.sp
..
EOF
endif

#
# write out info. for a type (3) man page
#
if( "$sopt" == "TRUE" && "$mansuffix" == 3 ) then
   cat << EOF >> $outfile
.SH PATH
<provide path to this code, relative to \$CVIPHOME/>
.SH DESCRIPTION
<Longer, detailed explanations>
.SH TYPES AND CONSTANTS
.Ss
<list any types or constants defined in library>
<for example:
typedef ... newval;
#define AVAL ...
extern newval that; >
.Se
.SH RETURN VALUES
<description of values returned>
.SH HISTORY
History information recorded: None
.SH BUGS
<this section is optional, keep it when applicable>
.SH EXAMPLE
<add a complete example of the function usage here>
EOF
endif

#
# write out info. for a type (1) man page
#
if( "$sopt" == "TRUE" && "$mansuffix" == 1 ) then
   cat << EOF >> $outfile
.SH OPTIONS
<Describe command line options if any>
.SH DESCRIPTION
<Longer, detailed explanation>
.SH EXAMPLES
<give an example(s) of using the program, with a short explanation>
.SH FILES
<A list of files involved with short descriptions if applicable>
.SH DIAGNOSTICS
<List of error messages and other terminal output and explanations>
.SH BUGS
<this section is optional, keep it when applicable>
EOF
endif

#
# extract info. about the functions in a C source file if
# the user specifies "-sopt" and write them out to $outfile
#

if( "$bopt" == "TRUE" ) then

# initialize some shell variables
set sedscript1 = /tmp/$$S1
set sedscript2 = /tmp/$$S2
set sedscript3 = /tmp/$$S3

# write out a script to be used w/ sed that will isolate
# the appropriate pattern space
cat << EOT > $sedscript1
\?Name:?,\?Expanded Name:?p
\?Parameters:?,\?Author(s):?p
EOT

cat << EOT > $sedscript2
\?Expanded Name:?d
EOT

cat << EOT > $sedscript3
\?Name:?{
i\
.Ss
s/Name://;
}
\?Parameters:?{
i\
.Si
s/Parameters://;
}
\?Returns:?{
i\
.Se
}
EOT

# have at it... yeesh!
sed -n '10,$p' $bname | sed -n -f $sedscript1 | sed -f $sedscript2 | sed -f $sedscript3 | expand | tr -s '\040' '\040' >> $outfile

rm /tmp/$$??

endif

#
# add copyright information
#
if( "$eopt" == "TRUE" ) then
cat << EOF >> $outfile
.SH SEE ALSO
<the library name, other related function names here, e.g.,
 libgeometry, resize(3), crop(3)
>
.SH AUTHOR
Copyright (C) 1996 SIUE - by Scott E. Umbaugh and YOUR NAME or WHOEVER .
.\"
.\"Permission to use, copy, modify, and distribute this software and its
.\"documentation for any purpose and without fee is hereby granted, provided
.\"that the above copyright notice appear in all copies and that both that
.\"copyright notice and this permission notice appear in supporting
.\"documentation.  This software is provided "as is" without express or
.\"implied warranty.
.\"
.\" RCS (Revision Control System) Information
.\"...Added automatically by RCS 
.\"
.\" \$Log\$
.\"
EOF
endif

# beam me out Scotty, I'm done with this script
echo "${prog}: ...that's all folks\!"
exit 0
