#!/bin/csh -f
# **********************************************************************
# Computer Vision/Image Processing Tool Project - Dr. Scott Umbaugh SIUE
# **********************************************************************
#
#             File Name: crt_tree.csh
#           Description: Create the whole CVIPtools source tree from one
#                 directory to another by making symbolic links. This is
#                 useful when different operating systems are sharing the
#                 same set of source files.
#         Related Files: None
#   Initial Coding Date: Tue Jun 17 16:16:45 CDT 1997
#           Portability: C Shell script
#            References:
#             Author(s): Yansheng Wei
#                        Southern Illinois University @ Edwardsville
#
## Copyright (C) 1996 SIUE - by Scott Umbaugh and Yansheng Wei.
##
## Permission to use, copy, modify, and distribute this software and its
## documentation for any purpose and without fee is hereby granted, provided
## that the above copyright notice appear in all copies and that both that
## copyright notice and this permission notice appear in supporting
## documentation.  This software is provided "as is" without express or
## implied warranty.
##
# RCS (Revision Control System) Information
# ...Added automatically by RCS 
#
## $Log: crt_tree.csh,v $
## Revision 1.5  1997/07/25 15:40:42  yawei
## added documentation link
##
## Revision 1.4  1997/07/17 02:25:13  yawei
## removed extraneous stuff
##
## Revision 1.3  1997/07/03 18:24:55  yawei
## added warning message
##
## Revision 1.2  1997/07/03 18:12:14  yawei
## fixed if-then problem
##
## Revision 1.1  1997/06/17 21:18:43  yawei
## Initial revision
##
#
set prog = $0
set prog = $prog:t
set printusage = FALSE
set target_plat = ""

if ( $#argv > 3 || $#argv < 2 ) then
	echo "${prog}: incorrect number of parameters..."
	set printusage = TRUE
else
	if ( $#argv == 3 ) then
		switch ( $3 ) 
			case "SunOS":
			case "IRIS":
			case "FreeBSD":
			case "Linux":
			case "Win32":
				set target_plat = $3		
				breaksw
			default:
				echo "unsupported platform: $3..."
				set printusage = TRUE
		endsw
	else 
		set target_plat = `/bin/uname -s`
	endif
	set src_base = $1
	if ( ! -e $src_base ) then
		echo "source directory: ${src_base} doesn't exist..."
		set printusage = TRUE
	endif
	set dst_base = $2
	if ( ! -e $dst_base ) then
		echo "destination directory: ${dst_base} doesn't exist..."
		set printusage = TRUE
	endif
endif

if ("$printusage" == "TRUE") then
	echo ""
	echo "Usage: ${prog} source_dir dest_dir [target platform]"
	echo "source_dir - the toplevel directory of the source files"
	echo "dest_dir   - the toplevel directory of the destination files"
	echo "target platform:   "
   echo "   - SunOS for SUN solaris systems"
   echo "   - IRIS for SGI IRIS systems"
   echo "   - FreeBSD for FreeBSD systems"
   echo "   - Linux for Linux systems"
   echo "   - Win32 for Miscrosoft Windows NT/95 systems"
	exit
endif

echo "Building CVIPtools source tree..."
echo " from    :$src_base"
echo "   to    :$dst_base"
echo " platform:$target_plat"
#exit

set files="CVIP* *.c *.h *.tcl *.bmp *.xbm configure *.in *.sh *make* *cur *.rc *.ico tclIndex *install*"

echo "Linking files, this takes time, be patient..."
cd $src_base
foreach i (/. `find . -type d -print| sed -e 's/^\.//g' -e '/RCS/d' `)
	set src_dir = ${src_base}$i
	mkdir $dst_base$i
	cd $src_dir
	foreach j (`\ls -l $files | uniq | awk '{print $9}'`)
		if ( -f $j && -r $j) then
			ln -s ${src_dir}/$j ${dst_base}${i}/$j
		endif
	end
end

if ( "$target_plat" != "Win32" ) then
	echo "Linking Libraries..."
	cd  ${dst_base}/lib
	\rm -f *
	foreach i (`\ls -l ${src_base}/lib | sed -n "/^l/p" | awk '{print $11}'`)
		set temp = $i:t
		ln -s $i $temp
	end
endif

echo "Removing documentation to save space..."
if ( -e ${dst_base}/man ) then
	\rm -rf ${dst_base}/man
endif
ln -s ${src_base}/man ${dst_base}/man

if ( -e ${dst_base}/docs ) then
	\rm -rf ${dst_base}/docs
endif
ln -s ${src_base}/docs ${dst_base}/docs

if ( -e ${dst_base}/CVIPTCL/man ) then
	\rm -rf ${dst_base}/CVIPTCL/man
endif
ln -s ${src_base}/CVIPTCL/man ${dst_base}/CVIPTCL/man

if ( "$target_plat" != "Win32" ) then
	echo "Removing unnecessary stuff..."
	\rm -f ${dst_base}/CVIPTCL/tcl7.6/unix/config.log
	\rm -f ${dst_base}/CVIPTCL/tk4.2/unix/config.log
	ln -s  ${src_base}/CVIPTCL/tcl7.6/unix/Makefile ${dst_base}/CVIPTCL/tcl7.6/unix/Makefile 
	ln -s  ${src_base}/CVIPTCL/tk4.2/unix/Makefile ${dst_base}/CVIPTCL/tk4.2/unix/Makefile 
endif

echo "Make a link for the documentation..."
ln -s ${src_base}/CVIPTCL/man ${dst_base}/CVIPTCL/man

if ( "$target_plat" == "Win32" ) then
	echo "Copying RPC files..."
	mkdir ${dst_base}/include/rpc
	cp ${src_base}/../Win32Ext/oncrpc/rpc/*.h ${dst_base}/include/rpc/.
	cp ${src_base}/../Win32Ext/oncrpc/bin/*.dll ${dst_base}/bin/.
	cp ${src_base}/../Win32Ext/oncrpc/bin/*.lib ${dst_base}/lib/.
endif

cd  ${dst_base}
echo "Fixing permissions..."
foreach i (`find . -type d`)
chmod 775 $i
end

echo "Remove makefiles..."
foreach i (`\find . -name Makefile`)
\rm -f $i
end

echo "NOTES: Here we are done\!\!\!"
echo "       Make sure your umask = 002 before building CVIPtools"

