#!/bin/csh -f
# **********************************************************************
# Computer Vision/Image Processing Tool Project - Dr. Scott Umbaugh SIUE
# **********************************************************************
#
#             File Name: dogman.csh
#           Description: UNIX C shell script to create the formatted 
#                   Manual pages, and create the windex database file.
#                   This file is created to put more strict syntax 
#                   checking of the manual pages, and create an entry for
#                   each manual page in the windex database file, which is
#                   not guarantteed by the catman(1) command.
#         Related Files: NONE
#   Initial Coding Date: Sat Jan 25 16:32:41 CST 1997
#           Portability: C Shell script
#            References:
#             Author(s): Yansheng Wei
#                        Southern Illinois University @ Edwardsville
#
## Copyright (C) 1996 SIUE - by Scott Umbaugh and Yansheng Wei.
##
## Permission to use, copy, modify, and distribute this software and its
## documentation for any purpose and without fee is hereby granted, provided
## that the above copyright notice appear in all copies and that both that
## copyright notice and this permission notice appear in supporting
## documentation.  This software is provided "as is" without express or
## implied warranty.
##
# RCS (Revision Control System) Information
# ...Added automatically by RCS 
#
## $Log: dogman.csh,v $
## Revision 1.9  1997/08/05 13:34:18  yawei
## checked in for freezing
##
## Revision 1.8  1997/07/25 16:31:12  yawei
## removed cd CVIPHOME
##
## Revision 1.7  1997/06/18 20:42:25  yawei
## fixed target directory
##
## Revision 1.6  1997/06/18 18:47:19  yawei
## removed debug message
##
## Revision 1.5  1997/06/15 00:50:14  yawei
## fixed small bug
##
## Revision 1.4  1997/06/03 02:23:25  yawei
## tried to fix the bug
##
## Revision 1.3  1997/05/15 15:37:39  yawei
## added cleanup stuff
##
## Revision 1.2  1997/02/04 02:39:19  yawei
## Change the windex file permission to 664.
##
# Revision 1.1  1997/01/25  22:40:02  yawei
# Initial revision
#
#
# **********************************************************************

unset noclobber
#
# set virables for later use
#
set prog = $0
set prog = $prog:t
set cdir = "."
set subdirs = FALSE
set tempdir = /tmp/
set upindex = TRUE
set catonly = TRUE
set printusage = FALSE
set options = TRUE

if ($#argv > 2 ) then
	echo "${prog}: too many arguments on command line, exit..."
	set printusage = TRUE
else if ( $#argv > 0 ) then
	while ( "$options" == "TRUE" )
		switch ( "$1" )
			case "-c":
				set upindex = FALSE
				breaksw
			case "-u":
				set catonly = FALSE
				breaksw
			case "-h":
				set printusage = TRUE
				breaksw
			case "":
				set options = FALSE
				breaksw
			default:
				set cdir = $1
				set options = FALSE
		endsw
		if ( "$options" == "TRUE" ) then
			shift
		endif
	end 
endif


if ("$printusage" == "TRUE") then
	echo "Usage: ${prog} <options> [directory]"
	echo "      By default, ${prog} will create the windex file, and add the missing "
	echo "      entries for the current directory. Additional options can be "
	echo "      added as:"
	echo "         -u only update the missing windex file entries"
	echo "         -c execute catman only"
	echo "         -h print this help message"
	echo "         directory : the directory to work on"
	exit
endif

cd $cdir
#echo $cdir

if ("$catonly" == "TRUE") then
	echo "Running catman, please wait..."
	foreach i (`\ls -l $cdir | \grep "^d" | \awk '{print $9}' | \grep cat `)
		echo "   Removing $i..."
		\rm -rf $i
	end
	foreach i (`\ls -l $cdir | \grep "^d" | \awk '{print $9}' | \grep man `)
		echo "   Removing unnecessary files in $i..."
		\rm -f $i/*.bak $i/*% $i/mid
		switch ( "$i" )
			case "man1":
				cd $i
				foreach k (`\ls -l | awk '{print $9}' | sed -e '/.1/d' -e '/RCS/d' `)
					echo "\tremoving $k"
					\rm -f $k
				end
				cd ..
				breaksw
			case "man3":
				cd $i
				foreach k (`\ls -l | awk '{print $9}' | sed -e '/.3/d' -e '/RCS/d' `)
					echo "\tremoving $k"
					\rm -f $k
				end
				cd ..
				breaksw
		endsw

	end
	\catman -M$cdir
endif 

if ("$upindex" == "FALSE") then
	echo "Here we are done"
	exit
endif

#cd $CVIPHOME/man

if ( ! -f windex ) then
	echo "No man directories found, exit..."
	exit
else if (  ! (-r windex && -w windex) ) then
	echo "Windex file permission denied..."
	exit
endif

echo ""
echo "Add missing entries to the windex file, this may take a long time..."
set oldnum = `wc -l < windex`
set nofound = "0"

cp windex ${tempdir}windex$$
foreach i (`\ls -l | \grep cat | \awk '{print $9}'`)
	echo "updateing missing entries for directory: $cdir/$i"
	if ( -d $i) then
		foreach j (`\ls -l $i | \sed -e '/^[^l]/d' | \awk '{print $9}'`)
			set source = $j:t; set source = $source:r
			set target = `\ls -l $i/$j | \awk '{print $11}'`
			set target = $target:t; set target = $target:r; set targtab = "$target	"
			set newentry = `\sed -n "/^$targtab/p" windex`
			if ( "$newentry" == "" ) then
				set targtab = "$target "
				set newentry = `\sed -n "/^$targtab/p" windex`
				if ( "$newentry" == "" ) then
					echo "Cannot update: $source,  $target not found"
					set nofound = `expr $nofound + 1`
				endif
			endif 

			if ( "$newentry" != "" ) then
				\sed -n "/^$targtab/p" windex | sed "s/^$target/$source/" >> ${tempdir}windex$$
			endif
		end
	endif
end

sort -u ${tempdir}windex$$ windex -o ${tempdir}windex$$ windex
\mv ${tempdir}windex$$ windex
chmod 664 windex

set newnum = `wc -l < windex`

echo ""
echo " ---------------------- Summary ----------------------"
echo " Number of  manual  pages of  incorrect  format: $nofound"
echo " Number of entries added to the windex database: `expr $newnum - $oldnum`"
echo " We are done at `date` \!\!\!"
