#!/bin/csh -f
# **********************************************************************
# Computer Vision/Image Processing Tool Project - Dr. Scott Umbaugh SIUE
# **********************************************************************
#
#             File Name: dogman.csh
#           Description: UNIX C shell script to create the HTML 
#                   Manual pages from formatted man pages.
#         Related Files: NONE
#   Initial Coding Date: Tue Jun 17 16:49:59 CDT 1997
#           Portability: C Shell script
#            References:
#             Author(s): Yansheng Wei
#                        Southern Illinois University @ Edwardsville
#
## Copyright (C) 1996, 1997 SIUE - by Scott Umbaugh and Yansheng Wei.
##
## Permission to use, copy, modify, and distribute this software and its
## documentation for any purpose and without fee is hereby granted, provided
## that the above copyright notice appear in all copies and that both that
## copyright notice and this permission notice appear in supporting
## documentation.  This software is provided "as is" without express or
## implied warranty.
##
## $Log: htmlman.csh,v $
## Revision 1.3  1997/07/17 16:19:03  yawei
## removed extra linkds
##
## Revision 1.2  1997/07/15 19:53:09  yawei
## modified so the index file will have a shot description of the function.
##
## Revision 1.1  1997/06/18 20:42:59  yawei
## Initial revision
##
#
# **********************************************************************

unset noclobber
#
# set virables for later use
#
set prog = $0
set prog = $prog:t
set cdir = `pwd`
set subdirs = FALSE
set tempdir = /tmp/
set upindex = TRUE
set htmlonly = TRUE
set printusage = FALSE
set options = TRUE
set indexfile = index.html
set PID=$$
set nolink = TRUE

if ($#argv > 2 ) then
	echo "${prog}: too many arguments on command line, exit..."
	set printusage = TRUE
else if ( $#argv > 0 ) then
	while ( "$options" == "TRUE" )
		switch ( "$1" )
			case "-c":
				set upindex = FALSE
				breaksw
			case "-u":
				set htmlonly = FALSE
				breaksw
			case "-h":
				set printusage = TRUE
				breaksw
			case "":
				set options = FALSE
				breaksw
			default:
				set cdir = $1
				set options = FALSE
		endsw
		if ( "$options" == "TRUE" ) then
			shift
		endif
	end 
endif


if ("$printusage" == "TRUE") then
	echo "Usage: ${prog} <options> [directory]"
	echo "      By default, ${prog} will create the index.html file"
	echo "      Additional options can be added as:"
	echo "         -u only update the index.html file"
	echo "         -c execute man2html only"
	echo "         -h print this help message"
	echo "         directory : the directory to work on"
	exit
endif

cd $cdir
#echo $cdir

if ("$htmlonly" == "TRUE") then
	echo "Create html pages, wait for quite a while..."
	foreach i (`\ls -l $cdir | \grep "^d" | \awk '{print $9}' | \grep html `)
		echo "   Removing $i..."
		\rm -rf $i
	end
	foreach i (`\ls -l $cdir | \grep "^d" | \awk '{print $9}' | \grep cat `)
		echo "   Creating HTML pages in $i..."
		set htmldir = `echo $i | sed 's/cat/html/'`
		mkdir $htmldir
		cd $i
		foreach j (`\ls -l | \grep "^-" | \awk '{print $9}'`)
			man2html -sun < $j | uniq > ../$htmldir/$j:r.html
		end
		cd ..
	end
endif 

if ("$upindex" == "FALSE") then
	echo "Here we are done"
	exit
endif

cd $cdir

if (  -f $indexfile && ! (-r $indexfile && -w $indexfile ) ) then
	echo "$indexfile file permission denied..."
	exit
endif

\rm -f $indexfile

echo ""
echo "Create $indexfile, this may take a while..."
cd $cdir

cat << EOF > $indexfile
<HTML>
 
<TITLE>
CVIPtools Man Pages
</TITLE>
  
<body>
<H1>CVIPtools Man Pages</H1>
<HR>
EOF

set section = "1"
set newsection = "1"
set sectiondir = html1
cat << EOF >> $indexfile
<H2>Commands</H2>
<UL>
EOF

if ( "$nolink" == "TRUE" ) then
	\mv $cdir/windex $cdir/windex.sav
	catman -w -M $cdir
endif

\cat  $cdir/windex | sed "s/[	 ]*\-/%/" | awk -F% '{print $1}' |  \awk '{print $3";"$1";"$2}' > /tmp/f1$PID
\cat  $cdir/windex | sed "s/[	 ]*\-/%/" | awk -F% '{print $2}' | sed "s/[	 ]/%/g"> /tmp/f2$PID
paste -d";" /tmp/f1$PID /tmp/f2$PID |sort| \sed "s/^(\([0-9]\))/\1/" >/tmp/tmp$PID
\rm -f /tmp/f1$PID
\rm -f /tmp/f2$PID

sed "s/\*/@/g" < /tmp/tmp$PID > /tmp/f$PID
\rm -f /tmp/tmp$PID

echo "section $newsection begins"
foreach i ( `/bin/cat /tmp/f$PID` )
#foreach i ( `/bin/cat /tmp/f3131` )
	set newsection = `echo $i | sed 's/^\([0-9]\).*/\1/'`
	if ( "$newsection" != "$section") then
		echo "section $section has finished"
		echo "section $newsection begins"
		echo '</UL>' >> $indexfile
		echo "" >> $indexfile
		set section = "$newsection"
		switch ( "$section" )
		case "2":
			set sectiondir = html2
			echo '<H2>System Calls</H2>' >> $indexfile
			breaksw
		case "3":
			set sectiondir = html3
			echo '<H2>Library Functions</H2>' >> $indexfile
			breaksw
		endsw
		echo '<UL>' >> $indexfile
	endif
	echo '<LI>' >> $indexfile
	echo $i | sed -e "s/^[0-9];\([^;]*\);\([^;]*\);\(.*\)/<A HREF=${sectiondir}\/\2.html>\1($section)<\/A> - \3/" -e 's/%/ /g' -e 's/@/*/g' >> $indexfile
end
cat << EOF >> $indexfile
</UL>
</body>
</HTML>
EOF
\rm -f /tmp/f$PID

if ( "$nolink" == "TRUE" ) then
	\mv $cdir/windex.sav $cdir/windex
endif

echo " We are done at `date` \!\!\!"
