#!/bin/sh
# **********************************************************************
# Computer Vision/Image Processing Tool Project - Dr. Scott Umbaugh SIUE
# **********************************************************************
#
#             File Name: makeftp.sh
#           Description: Bourne shell script to generate the files for 
#                        the CDROM and FTP distribution
#         Related Files: NONE
#   Initial Coding Date: Tue Aug  5 09:13:33 CDT 1997
#           Portability: Bourne Shell scipt
#            References:
#             Author(s): Yansheng Wei
#                        Southern Illinois University @ Edwardsville
#
## Copyright (C) 1996 SIUE - by Scott Umbaugh and Yansheng Wei.
##
## Permission to use, copy, modify, and distribute this software and its
## documentation for any purpose and without fee is hereby granted, provided
## that the above copyright notice appear in all copies and that both that
## copyright notice and this permission notice appear in supporting
## documentation.  This software is provided "as is" without express or
## implied warranty.
##
# RCS (Revision Control System) Information
# ...Added automatically by RCS 
#
## $Log: makeftp.sh,v $
## Revision 1.17  1997/08/11 23:10:42  smakam
## made small change
##
## Revision 1.16  1997/08/11 21:04:34  smakam
## changed the compressed tar file for the source distribution
##
## Revision 1.15  1997/08/11 14:47:43  smakam
## included the include directory for tar library distribution
##
## Revision 1.14  1997/08/08 21:21:05  yawei
## fixed the directory permissions after CDROM prepare
##
## Revision 1.13  1997/08/08 19:59:45  yawei
## added support modules for Win32 library and bin distribution
##
## Revision 1.12  1997/08/08 17:43:50  yawei
## convert README.win32 to dos format
##
## Revision 1.11  1997/08/08 16:51:10  yawei
## change directory permission to 700 in the beginning
##
## Revision 1.10  1997/08/08 16:13:34  yawei
## modify the Makefiles for Win32
##
## Revision 1.9  1997/08/08 16:04:06  yawei
## finialize the CDROM creation .
##
## Revision 1.8  1997/08/07 22:33:57  yawei
## added Win32 extra stuff
##
## Revision 1.7  1997/08/07 21:56:06  yawei
## turned off -f switch
##
## Revision 1.6  1997/08/07 17:28:01  yawei
## create the toplevel directory first.
##
## Revision 1.5  1997/08/07 17:24:14  yawei
## added the book images
##
## Revision 1.4  1997/08/05 18:44:42  yawei
## added Win32 makefiles backup
##
## Revision 1.3  1997/08/05 15:48:19  yawei
## added README files
##
## Revision 1.2  1997/08/05 15:41:12  yawei
## modified cp -f
##
## Revision 1.1  1997/08/05 14:14:36  yawei
## Initial revision
##
#

#!/bin/sh
masksave=`umask`
umask 022
tempfile=/tmp/filelist$$
version=CVIPtools3.6
CDROM=/home/cvip/cvipapps/CDROM

echo "Removing old archive...."
if /bin/test -d $CDROM;
then
	find $CDROM -type d -exec chmod 755 {} \;
	\rm -rf $CDROM;
fi
mkdir $CDROM;

#common source files
echo "Archiving the common source files...."
srcdir=/home/cvip/cvipapps/$version
targetdir=$CDROM/src

if /bin/test ! -d $targetdir;
then
	mkdir $targetdir;
fi

cd $srcdir

echo "==>creating the directory structure...."
find . -type d | sed '/RCS/d' > $tempfile
for i in `cat $tempfile`
do
      if /bin/test ! -d $targetdir/$i;
      then
         mkdir $targetdir/$i;
		fi
done
\rm -f $tempfile

echo "==>updating the source files ...."
find . \( -name "CVIP*" -o -name "*.[ch]" -o -name "*.tcl" \
			 -o -name "*.bmp" -o -name "*.xbm" -o -name configure \
			 -o -name "*.in" -o -name "*.sh" -o -name "*make*" \
			 -o -name "*cur" -o -name "*.rc" -o -name "*.ico" \
			 -o -name "*install*" -o -name "mkLinks" -o -name "READ" \
			 -o -name "*license*" -o -name "*[Rr][Ee][Aa][Dd][Mm][Ee]*" \) \
			 -a -type f -print | sed '/RCS/d' > $tempfile
for i in `cat $tempfile`
do
      if /bin/test ! -f $targetdir/$i;
      then
         cp $i $targetdir/$i;
      elif /bin/test  $targetdir/$i -ot $i;
		then
			\rm -f $targetdir/$i; cp $i $targetdir/$i;
		fi
done
\rm -f $tempfile
cp /home/cvip/cvipapps/Win32Ext/bin/* $targetdir/bin/.

#rpc for Win32
if /bin/test -d $targetdir/include/rpc;
then
	\rm -rf $targetdir/include/rpc;
fi
cp /home/cvip/Win32/bin/*.dll $targetdir/bin/.
cp /home/cvip/Win32/lib/oncrpc.lib $targetdir/lib/.
cp /home/cvip/Win32/lib/tcl76.lib $targetdir/lib/.
cp /home/cvip/Win32/lib/tk42.lib $targetdir/lib/.
cp -r /home/cvip/cvipapps/Win32Ext/oncrpc/rpc $targetdir/include
\rm -f $targetdir/CONVERSION/TIFF/g3states.h

echo "==>updating the makefiles for Win32 platform...."
cd $targetdir
find . -name Makefile -exec rm -f {} \;
cd /home/cvip/Win32
for i in ` find . -name makefile `
do
	j=`echo $i|sed 's/makefile/Makefile/g'`
	cp $i $targetdir/$j
done
cd $targetdir
for i in ` find . -name Makefile `
do
	sed 's/Z:\/cvip\/Win32/$(CVIPTOP)/g' < $i > /tmp/cvipMkTmp
	mv /tmp/cvipMkTmp $i
done

echo "==>copying the files in the ./docs direcotry ...."
srcdir=/home/cvip/cvipapps/$version
targetdir=$CDROM/src
rm -rf $targetdir/docs/*
cp -r $srcdir/docs $targetdir
cp -r $srcdir/CVIPTCL/tcl7.6/doc $targetdir/CVIPTCL/tcl7.6
cp -r $srcdir/CVIPTCL/tk4.2/doc $targetdir/CVIPTCL/tk4.2

echo "Archiving the documentation ...."
srcdir=/home/cvip/cvipapps/$version/man
targetdir=$CDROM/src/man
if /bin/test ! -d $targetdir;
then
	mkdir $targetdir;
fi
cd $srcdir

echo "==>creating the man directory structure...."
find . -type d | sed '/RCS/d' > $tempfile
for i in `cat $tempfile`
do
      if /bin/test ! -d $targetdir/$i;
      then
         mkdir $targetdir/$i;
		fi
done
\rm -f $tempfile

echo "==>copying the book images ...."
cp -r /home/sumbaug/BookImgs $CDROM/Images
find $CDROM/Images -type d -exec chmod 755 {} \;

echo "==>updating the manual page  files ...."
find .  -type f -print | sed -e '/RCS/d' -e '/cat[13]/d' -e '/html[13]/d' \
		-e '/tcl_help/d' > $tempfile
for i in `cat $tempfile`
do
      if /bin/test ! -f $targetdir/$i;
      then
         cp $i $targetdir/$i;
      elif /bin/test  $targetdir/$i -ot $i;
		then
			\rm -f $targetdir/$i; cp $i $targetdir/$i;
		fi
done
\rm -f $tempfile
cp /home/cvip/cvipapps/$version/README* $CDROM/.
cp /home/cvip/cvipapps/$version/install* $CDROM/.
if  /bin/test -f $targetdir/README.win32;
then
	unix2dos $targetdir/README.win32 > /tmp/tmpRead;
	mv /tmp/tmpRead $targetdir/README.win32
fi

cd $CDROM/src/man
echo "==>formatting the man pages, run this on shiva and takes time  ...."
/home/cvip/cvipapps/$version/docs/Scripts/cathlp.csh
/home/cvip/cvipapps/$version/docs/Scripts/dogman.csh
/home/cvip/cvipapps/$version/docs/Scripts/htmlman.csh

for j in Solaris FreeBSD Linux IRIX
do
	echo "Archiving the binaries and libraries for $j ...."
	if [ $j = "Solaris" ] 
	then 
		srcdir=/home/cvip/cvipapps/$version
	elif [ $j = "FreeBSD" ]
	then
		srcdir=/net/cvip-pc2/usr/opt/cvipapps/CVIPtools
	elif [ $j = "Linux" ]
	then
		srcdir=/net/cvip-pc/usr/cvipapps/CVIPtools
	else
		srcdir=/net/duet/opt/cvipapps/CVIPtools
	fi

	targetdir=$CDROM/$j
	cd $srcdir
	
	if /bin/test ! -d $targetdir;
	then
		mkdir $targetdir;
	fi
	if /bin/test ! -d $targetdir/bin;
	then
		mkdir $targetdir/bin;
	fi
	if /bin/test ! -d $targetdir/lib;
	then
		mkdir $targetdir/lib;
	fi
	for i in `find . \( -type f -a -name "lib*.a" \) | sed '/RCS/d' `
	do
		cp $i $targetdir/lib/.;
	done
	cp /home/cvip/cvipapps/$version/bin/CVIPtools $targetdir/bin/.;
	cp $srcdir/CVIPTCL/cviptcl $targetdir/bin/.;
	cp $srcdir/CVIPTCL/cvipwish $targetdir/bin/.;
	cp $srcdir/DISPLAY/picture $targetdir/bin/.;
done

j=Win32
echo "Archiving the binaries and libraries for $j ...."

srcdir=/home/cvip/Win32
targetdir=$CDROM/$j

if /bin/test ! -d $targetdir;
then
	mkdir $targetdir;
fi
if /bin/test ! -d $targetdir/bin;
then
	mkdir $targetdir/bin;
fi
if /bin/test ! -d $targetdir/lib;
then
	mkdir $targetdir/lib;
fi

cd $srcdir
for i in `find . \( -type f -a -name "*.lib" \) | sed '/RCS/d' `
do
	cp $i $targetdir/lib/.;
done
for i in `find . \( -type f -a -name "*.dll" \) | sed '/RCS/d' `
do
	cp $i $targetdir/bin/.;
done
#cp /home/cvip/cvipapps/$version/bin/CVIPtools.bat $targetdir/bin/.;
cp /home/cvip/cvipapps/Win32Ext/bin/* $targetdir/bin/.
cp $srcdir/CVIPTCL/cviptcl.exe $targetdir/bin/.;
cp $srcdir/CVIPTCL/cvipwish.exe $targetdir/bin/.;

echo "Creating archives for the FTP site ...."
cd $CDROM/src
echo "==>make tar file for the source distribution...."
tar -cf ../src.tar *
tar -cf ../bin.tar ./READ* ./install* ./man ./CVIPTCL/GUI_SCRIPTS ./CVIPTCL/lib/tcl7.6 ./CVIPTCL/lib/tk4.2
tar -cf ../lib.tar ./READ* ./install* ./man ./CVIPTCL/GUI_SCRIPTS ./CVIPTCL/lib ./docs ./include
cd ..
echo "==>compress tar file for the source distribution...."
cd /home/cvip/cvipapps/Win32Ext
tar -uf $CDROM/src.tar ./bin
gzip $CDROM/src.tar

cd $CDROM
for j in Solaris FreeBSD Linux IRIX Win32
do
	cp bin.tar bin$j.tar
	cp lib.tar lib$j.tar
	cd $j
	echo "==>make the tar files for $j binary distribution...."
		tar -uf ../bin$j.tar ./bin
	echo "==>make the tar files for $j library distribution...."
		tar -uf ../lib$j.tar ./bin ./lib
	cd ..
	echo "==>compress the tar files for $j binary and library distribution...."
	if [ $j != "Win32" ] 
	then
		gzip bin$j.tar
		gzip lib$j.tar
	fi
done
rm bin.tar lib.tar

echo "==>compress the tar files for Win32 binary and library distribution...."
cd /home/cvip/cvipapps/Win32Ext
tar -uf $CDROM/binWin32.tar ./bin
tar -uf $CDROM/libWin32.tar ./bin
cd $CDROM
gzip binWin32.tar
gzip libWin32.tar

find $CDROM -type d -exec chmod 755 {} \;
umask $masksave

