#!/bin/csh -f
# ======================================================================
# Computer Vision/Image Processing Tool Project - Dr. Scott Umbaugh SIUE
# ======================================================================
#
#             File Name: newp
#           Description: Adds header information to a file. The amount of
#			 information added can be controlled by setting
#			 command line flags. The comment style will vary
#			 depending on the type of file operated upon. The type
#			 of file can be specified explicitly on the command
#			 line or can be interpreted from the file extension.
#			 See usage message below for more detailed info. on 
#			 command line options.
#			 
#         Related Files: None
#   Initial Coding Date: Mon Aug 23 14:08:18 CDT 1993
#           Portability: C Shell script
#             Author(s): Gregory Hance
#                        Southern Illinois University @ Edwardsville
#
## Copyright (C) 1993 SIUE - by Gregory Hance.
##
## Permission to use, copy, modify, and distribute this software and its
## documentation for any purpose and without fee is hereby granted, provided
## that the above copyright notice appear in all copies and that both that
## copyright notice and this permission notice appear in supporting
## documentation.  This software is provided "as is" without express or
## implied warranty.
##
# RCS (Revision Control System) Information
# ...Added automatically by RCS 
#
## $Log: addheader.csh,v $
# Revision 1.2  1995/11/02  23:10:27  mzuke
# chnged copyright date to 1996 and
# changed exit message from That's All Folks to
# Header has been added
#
# Revision 1.1  1995/11/02  22:59:54  mzuke
# Initial revision
#
#

#
# unset the shell variable noclobber so that we can overwrite
# existing files using ">" and create new files w/ ">>"
# also set umask so that write access is restricted to the owner
#
unset noclobber
set umask = 022

#
# initialize some local shell variables
#
set printusage = FALSE
set prog = $0
set prog = $prog:t

#
# parse the first command argument to identify the type of file
# that we are dealing with
#
switch( "$1" )
case "-c":
   set mainarg = 'c'
   set ext = '.c'
breaksw
case "-h":
   set mainarg = 'h'
   set ext = '.h'
breaksw
case "-im":
   set mainarg = 'im'
   set ext
breaksw
case "-m":
   set mainarg = 'm'
   set ext
breaksw
case "-man":
   set mainarg = 'man'
   set ext = '.man'
breaksw
case "-sh":
   set mainarg = 'sh'
   set ext = '.sh'
breaksw
case "-csh":
   set mainarg = 'csh'
   set ext = '.csh'
breaksw
default:
   set mainarg
   echo "${prog}: no mandatory option specified."
   echo "${prog}: ...attempting to use filename extension to identify file type"
endsw

#
# if there is more than one argument and a mandatory argument
# was specified then shift the arguments and check for optional
# arguments
#
if($#argv >= 1 && $?ext ) shift

# initialize optional arguments
set option = TRUE
set cvipopt = FALSE
set cropt = FALSE
set rcsopt = FALSE

#
# loop through remaining optional arguments and if an argument
# does not match with an optional argument then let's hope
# it is the filename
#
while("$option" == "TRUE")
   switch("$1")
   case "-cvip":
      set cvipopt = TRUE
      shift
   breaksw
   case "-cr":
      set cropt = TRUE
      shift
   breaksw
   case "-rcs":
      set rcsopt = TRUE
      shift
   breaksw
   default:
      set option = FALSE
   endsw
end

#
# the only argument left at this point should be the filename
# with one exception... if a Makefile or Imakefile was specified
# then we know what the name should be
#
if( $#argv > 1 ) then
   echo "${prog}: too many arguments on command line."
   set printusage = TRUE
else if( $#argv < 1 && "$mainarg" != "m" && "$mainarg" != "im" ) then
   echo "${prog}: too few arguments on command line."
   set printusage = TRUE
endif

#
# if a mandatory argument wasn't specified then i'll be nice
# to the user and try to figure out the file type based on the
# filename extension or possibly the filename itself in the case
# of a Makefile or Imakefile
#
if( "$1:e" != "" && ! $?ext ) then
   set ext = ".$1:e"
   switch("$ext")
   case ".c":
      set mainarg = 'c'
   breaksw
   case ".h":
      set mainarg = 'h'
   breaksw
   case ".man":
      set mainarg = 'man'
   breaksw
   case ".sh":
      set mainarg = 'sh'
   breaksw
   case ".csh":
      set mainarg = 'csh'
   breaksw
   default:
      echo "${prog}: do not recognize file name extension $1:e."
      exit 1
      set printusage = "TRUE"
   endsw
else if ( "$1:e" == "" && ! $?ext ) then
   switch("$1")
   case [mM]akefile:
      set mainarg = 'm'
      set ext
   breaksw
   case [iI]makefile:
      set mainarg = 'im'
      set ext
   breaksw
   default:
      echo "${prog}: no file name extension specified thus file type"
      echo "${prog}: can not be determined for $1."
      set printusage = "TRUE"
   endsw
endif

#
# finally, if the script didn't like something the user did
# and the printusage variable was set then print a
# "usage:..." message
#
if("$printusage" == "TRUE") then
   echo "${prog}:"
   echo "${prog}: usage: newp -c|-h|-im|-m|-man|-[c]sh [-cvip] [-cr]"
   echo "${prog}:        [-head] [-prepend] filename"
   echo "${prog}: MANDATORY ARGUMENTS:"
   echo "${prog}: one of..."
   echo "${prog}:       -c    C language source file (~.c)"
   echo "${prog}:       -h    C language header file (~.h)"
   echo "${prog}:      -im    UNIX Imakefile"
   echo "${prog}:       -m    UNIX Makefile"
   echo "${prog}:     -man    UNIX manual page (~.man)"
   echo "${prog}:     -csh    UNIX C shell script (~.csh)"
   echo "${prog}:      -sh    UNIX Bourne shell script (~.sh)"
   echo "${prog}: and..."
   echo "${prog}: filename    name of output file (extension not necessary)"
   echo "${prog}: OPTIONAL ARGUMENTS:"
   echo "${prog}:    -cvip    Add CVIPtools standard headers"
   echo "${prog}:      -cr    Add Copyright Information"
   echo "${prog}:     -rcs    Add RCS (Revision Control System) information"
   exit 1
endif

#
# we made it this far... bitchin'
# now let's initialize some file type dependent variables
# ( such as comment syntax ) so they can be used as macros 
# w/ "sed" to make a nice header w/ appropriate information
# relative to each file type
#
switch( "$mainarg" )
case "c":
   set comment = '*'
   set coms = '/***************************************************************************'
   set come = '****************************************************************************/'
   set port = 'Standard (ANSI) C'
breaksw
case "h":
   set comment = '*'
   set coms = '/***************************************************************************'
   set come = '****************************************************************************/'
   set port = 'Standard (ANSI) C'
breaksw
case "im":
   set comment = '*'
   set coms = '/***************************************************************************'
   set come = '****************************************************************************/'
   set port = 'UNIX\/X imake config. file'
breaksw
case "m":
   set comment = '#'
   set coms = ""
   set come = ""
   set port = 'UNIX make config. file'
breaksw
case "man":
   set comment = '.\"'
   set coms = ""
   set come = ""
   set port = 'UNIX Manual page'
breaksw
case "sh":
   set comment = '#'
   set coms = ""
   set come = ""
   set port = 'Bourne Shell scipt'
breaksw
case "csh":
   set comment = '#'
   set coms = ""
   set come = ""
   set port = 'C Shell script'
breaksw
default:
   echo "${prog}: no mandatory option specified."
   echo "${prog}: ...attempting to use filename extension to identify file type"
endsw

#
# determine the name of the output file
# appropriate extension will be added
#
if ( "$mainarg" == "m" ) then
   set outfile = "Makefile"
else if ( "$mainarg" == "im" ) then
   set outfile = "Imakefile"
else
   set outfile = $1:r$ext
endif

#
# set a flag if the output file already exists
# and is not empty
#
if ( -e $outfile && ! -z $outfile ) set existfile

#
# check to see if the user has a preferred temporary
# directory... and if not use the standard "/tmp"
#
if (! $?TMPDIR) set TMPDIR = /tmp

#
# set local variables to identify temporary files to be used
# within the script
#
set sedscript = $TMPDIR/$$SS
set header = $TMPDIR/$$HH
set body = $TMPDIR/$$BB
set tail = $TMPDIR/$$TT
set tmpfile = $TMPDIR/$$FF

#
# set a variable to the full name of the user
# the name is extracted from the flat file /etc/passwd
# unless we're on the NeXT and using Larry's database
#
set awkfield = '$5'
set name = `awk -F: "/^$user/{print $awkfield}" /etc/passwd`

#
# write out a set of macros in the form of a "sed" script
# to be used within the header, etcetera.
#
cat << EOF > $sedscript
s/@date@/`date`/
s/@name@/$name/
s/@email@/$user@minuet.siue.edu/
s/@filename@/$outfile/
s/@school@/Southern Illinois University @ Edwardsville/
s/@street@/200 University Park/
s/@city@/Edwardsville, IL  62026-1801/
s/@portability@/$port/
EOF

#
# Begin writing contents of header out to temporary file
# to be concatenated later with rest of file
#
switch( "$mainarg" )
case "sh":
   echo "#\!/bin/sh -f" > $header
   breaksw
case "csh":
   echo "#\!/bin/csh -f" > $header
   breaksw
default:
   echo "$coms" > $header
endsw

#
# add CVIPtools header with name of the almighty Umbaugh
# ...silence naive
#
if( "$cvipopt" == "TRUE" ) then
cat << EOF >> $header
$comment ======================================================================
$comment Computer Vision/Image Processing Tool Project - Dr. Scott Umbaugh SIUE
$comment ======================================================================
EOF
endif

#
# add main header info -- name, date, etc...
#
cat << EOF >> $header
$comment
$comment             File Name: @filename@
$comment           Description: 
$comment         Related Files: 
$comment   Initial Coding Date: @date@
$comment           Portability: @portability@
$comment            References:
$comment             Author(s): @name@
$comment                        @school@
$comment
EOF

#
# add copyright info. to scare off weenies :o)
#
if( "$cropt" == "TRUE" ) then
cat << EOF >> $header
$comment$comment Copyright (C) 1996 SIUE - by $name.
$comment$comment
$comment$comment Permission to use, copy, modify, and distribute this software and its
$comment$comment documentation for any purpose and without fee is hereby granted, provided
$comment$comment that the above copyright notice appear in all copies and that both that
$comment$comment copyright notice and this permission notice appear in supporting
$comment$comment documentation.  This software is provided "as is" without express or
$comment$comment implied warranty.
$comment$comment
EOF
endif

#
# add information about RCS if requested by the user
#
if( "$rcsopt" == "TRUE" ) then
set rcslog = '$Log: addheader.csh,v $
# Revision 1.2  1995/11/02  23:10:27  mzuke
# chnged copyright date to 1996 and
# changed exit message from That's All Folks to
# Header has been added
#
# Revision 1.1  1995/11/02  22:59:54  mzuke
# Initial revision
#'
cat << EOF >> $header
$comment RCS (Revision Control System) Information
$comment ...Added automatically by RCS 
$comment
$comment$comment $rcslog
$comment
EOF
endif

#
# add closing comment syntax to the header and clap your hands :-)
#
echo "$come" >> $header


#
# if the header option is FALSE and the output file already exists
# warn the user that his file is about to be overwritten
#
if( $?existfile ) then
      while(1)
         echo "${prog}: output file $outfile already exists and is about"
         echo -n "${prog}: to be overwritten... continue? (y/n)?"
         switch("$<")
         case "n":
            exit 1
         case "y":
            break
         endsw
      end
endif

#
# if the header option is TRUE and the output file exists
# make a temporary back up that will be concatenated with
# the header.
#
if( $?existfile ) then
   cp $outfile $tmpfile
   if ( $status != 0 ) then
      echo "${prog}: can not create temporary file $tmpfile."
      echo "${prog}: ...terminating script"
      exit 1
   endif
endif

#
# parse for macros using "sed" and write the result to outfile
#
sed -f $sedscript $header > $outfile

#
# if the header option is true then concatenate the user
# specified file with it
#
if( $?existfile ) then
   cat $tmpfile >> $outfile
else if ( -e $1 && ! -z $1 && $1 != $outfile ) then
   cat $1 >> $outfile 
endif

echo "${prog}: Header has been added."
   
# delete the temporary files
rm $TMPDIR/$$??

# beam me out Scotty, I'm done with this script
exit 0
