/*
*	Bilinear Interpolation
*	Arve Kjoelen - akjoele@ee.siue.edu
*	Mon Oct 23 09:26:08 CDT 1995
*
*/
#include <stdio.h>
#include <CVIPimage.h>
#include <CVIPdef.h>
#include "bilinear.h"

Image *bilinear_setup(Image *inImg) {
	float sf;
	char *mystr=malloc(32);
	fprintf(stdout, "Please enter a scaling factor;\n");
	fprintf(stdout, "A number greater than 1 will enlarge\n");
	fprintf(stdout, "The image, while a number less than 1 will\n");
	fprintf(stdout, "shrink it.\n");
	sf=(float)atof(fgets(mystr, 31, stdin));

	return(Image *)bilinear_interp(inImg, sf);
}

/*
 * Bilinear Interpolation.  Takes a floating-point
 * number as input.  This scaling factor is used to 
 * enlarge or shrink the image.
 */
Image *bilinear_interp(Image *inImg, float factor) {
	int rows, cols;
	int i,j,k;		/* Counters */
	int *int_map;		/* Map of old coords to new coords */
	float *delta_map;	/* Map of interpolation coordinates */
	Image *outIm;

	rows=inImg->image_ptr[0]->rows;
	cols=inImg->image_ptr[0]->cols;

	i=(int)(float)(0.5+factor*(float)rows);
	j=(int)(float)(0.5+factor*(float)cols);
	int_map=(int *)malloc(sizeof(int)*(1+MAX(i,j)));
	delta_map=(float *)malloc(sizeof(float)*(1+MAX(i,j)));
	outIm=new_Image(inImg->image_format, inImg->color_space, inImg->bands, i, j, inImg->image_ptr[0]->data_type, inImg->image_ptr[0]->data_format);

	/* Calculate mappings */ 
	for(k=0; k<=MAX(i,j); k++) {
	   int_map[k]=(int)(float)((float)k/factor);
	   delta_map[k]=(float)(k/factor - (float)int_map[k]);
	}
	/* adjust limits so we don't overrun bounds */
	i--;j--;

	for(k=0; k<inImg->bands; k++) {
	   for(rows=0; rows<i; rows++) {
	      switch(inImg->image_ptr[0]->data_type) {
		 case CVIP_BYTE:
		    LOOP_BILINEAR(byte,rows,j,int_map,delta_map,(byte **)inImg->image_ptr[k]->rptr,(byte *)outIm->image_ptr[k]->rptr[rows])
	 	 break;
		 case CVIP_SHORT:
                    LOOP_BILINEAR(short,rows,j,int_map,delta_map,(short **)inImg->image_ptr[k]->rptr,(short *)outIm->image_ptr[k]->rptr[rows]);
		 break;
		 case CVIP_INTEGER:
                    LOOP_BILINEAR(int,rows,j,int_map,delta_map,(int **)inImg->image_ptr[k]->rptr,(int *)outIm->image_ptr[k]->rptr[rows]);
		 break;
		 case CVIP_FLOAT:
                    LOOP_BILINEAR(float,rows,j,int_map,delta_map,(float **)inImg->image_ptr[k]->rptr,(float *)outIm->image_ptr[k]->rptr[rows]);
	  	 break;
		 case CVIP_DOUBLE:
                    LOOP_BILINEAR(double,rows,j,int_map,delta_map,(double **)inImg->image_ptr[k]->rptr,(double *)outIm->image_ptr[k]->rptr[rows]);
		 break;
	      } /* switch */
	   } /* for (rows.. */
	} /* k */
	delete_Image(inImg);
	return(outIm);
}
