.TH cvip_segmentation
.SH wm geometry .help $help_long    
.SH help_message delete 1.0 end
.SH help_message insert 1.0  {
.SH Analysis: Segmentation

.SS FOR ALL FUNCTIONS:
After setting the desired parameters, press the 'PERFORM' 
button at lower right to perform the operation.

The current image is automatically selected, and can be 
changed by clicking mouse button 1 on an image in the 
main window's image queue.

Select a segmentation method by clicking on the button 
to the left of the desired function, then change the 
default values as desired. These values can be entered directly from the
keyboard into the entry box, or selected from the pop-up menu that
appears when the downward pointing arrow is selected.
	
.SS Fuzzy C Mean:
To use, select a value from the pop-up menu or enter a value from keyboard
(typically 0 - 20) for the Gaussian kernel variance. In general, the larger
the number, the coarser the segmentation, with fewer gray levels.

.SS Gray Level Quantization:
Select the desired number of gray levels from the
pop-up menu, or type directly into the entry box.
The number must be a power of 2. Select Standard or
IGS quantization. IGS minimizes false contouring that
can occur in areas that appear to be uniform, but also
adds a dithered effect to the image which may be not
be desired for segmentation. If IGS is selected, it is
recommended that morphological filtering is performed,
for example erosion, before using the segmented image
for feature extraction.

.SS Histogram Thresholding:
No parameters to enter. The algorithm determines the
number of 'good' peaks in the histogram to use for
the segmentation.	   

.SS Median Cut:
To use, select a value from the pop-up menu or enter a value from keyboard 
for the desired number of colors. 
This algorithm is based on an algorithm originally developed
to map 24-bit color images to 8-bit color. It works by finding
the maximum spread along the red, green or blue axes, and then
dividing the color space with the median value along that axis. This
division of the color space continues until the number of desired
colors is reached. At this point, all the color vectors in a given
subdivision of the color space are used to find an average color for that
subdivision. After all the average colors are found, the algorithm goes back
and maps each of the original color vectors to the closest one.


.SS PCT/Median:
To use, select a value from the pop-up menu or enter a value from keyboard 
for the desired number of colors. 
This algorithm performs the Principal Components Transform on the
3-D color space, and then performs the Median Cut described above.

.SS SCT/Center:
To use, select values from the pop-up menus or enter values from keyboard 
for the desired number of colors along the A and B axis.
The algorithm performs the Spherical Coordinate Transform, and divides the
color space by splitting the 2-D space created by angles A and B. This
division is done by a simple center split - that is, evenly dividing along each
angle axis based on the numbers specified. Note that only the subspace 
contained in the image is used (this means the maximums and minimums are found
along each axis and these limit the subspace used for the division). 
After this is done, the average is found for each color in each subdivision,
and the original color vectors are mapped to these.

.SS Multi_Resolution:
Multi_resolution is similar to the 'Split and Merge'
(below), with entry level fixed at a value of zero.
This starts the process with the entire image as the
initial region.

.SS Split and Merge:
All selections require an 'Entry level' parameter;
the lower the entry level, the larger the initial 
region. 

There are six tests for homogeneity. The tests and 
.TP 4
their required parameters are:

.RS
.TP 4
\f31)\f1
Pure Uniformity: region considered homogeneous if 
completely uniform, i.e. all the pixels have the 
same gray value. No extra parameters required. 

.TP 4
\f32)\f1
Local Mean vs. Global Mean: region is considered
homogeneous if the local mean is greater than the 
global mean. No extra parameters required.

.TP 4
\f33)\f1
Local Standard Deviation vs. Global Mean: region
is considered homogeneous if the local standard 
deviation is less than 10% of the global mean.
No extra parameters required.

.TP 4
\f34)\f1
Variance test: region is considered homogeneous if
at least a given percentage of the pixels are within 
2 sigma of the local mean, unless the standard 
deviation exceeds a maximum threshold. 

Parameters:
.RS
.TP 4
-Percentage (in decimal format; typically .70-.98)
the percentage of pixels in a region that
must be within 2 standard deviations of the
mean for the region to be homogeneous
.TP 4
-Threshold
maximum allowed value for standard deviation;
if exceeded, area is non-homogeneous
.RE
.TP 4
\f35)\f1
Weighted gray level distance: region is considered 
homogeneous if a computed value is less than the 
chosen threshold. The weighted gray level value is
found:
.SS " "
.SS "                                |     # pixels at Gi         | "
.SS "        value = SUM(i=1 to max) |Gi* ----------------  - Mode| "
.SS "                                |   # pixels in region       | "
.SS " "

.RS
.RS
where Gi is a gray level in the region, and Mode is
the most frequent gray level in the region.
Parameter:
.TP 4
-Threshold
maximum value for weighted gray level value
.RE
.TP 4
\f36)\f1
Texture homogeneity: A region is divided into 4 equal-sized
sections called quadrants. A region is considered homogeneous
if computed texture features of all quadrants of the
region are within the given similarity measure. 
The features used are: energy (also called angular second moment or ASM), 
inertia, correlation, inverse difference, and entropy.
The five features are calculated for four orientations: 0, 
45, 90 and 135 degrees, and then the average of the results from
these four orientations are used.
The textural features and similarity measure are defined in
Pattern Recognition Engineering, Nadler and Smith, Wiley, 1993, pp.262-270.

Parameters:
.RS
.TP 4
-Similarity
maximum allowable value for similarity of 
quadrants of the region.
.TP 4
-Pix distance (usually 1 or 2)
distance between pixels, used in finding GLDM
(Gray Level Dependence Matrix), a matrix used
in texture feature determination

NOTE: Texture homogeneity test is EXTREMELY SLOW.
.RE
.RE

.TP 4
PCT option:
If the image being segmented is color, user may elect
to perform a Principal Components Transform (PCT)  
on the image before segmentation is run. A PCT will be  
performed if the square on the left side of the 'PCT' 
button is dark; no PCT is performed if the square is 
light. You toggle between the two choices by pressing 
the 'PCT' button with a mouse click.

Note: If you have selected a color image for Split and 
Merge or Multi_Resolution segmentation and the
PCT button is not available, reselect your
segmentation method to activate the button.
		


.SS Morphological Filtering:

.TP 4
Closing, Opening, Erosion, Dilation functions:

These functions are normally 
used on images that have undergone segmentation, but can be used on
original images for various filtering effects. They will work on binary, gray-level, and color images. They work on multi-band (color)
images by treating each individual band as a gray-level image.
With gray-level images, the brighter of two adjacent objects is 
assumed to be the "object" for morphological purposes.

To use, select a kernel type from the pop-up menu. 
Parameters for kernel type:
.TP 4
circle:
enter kernel size from keyboard or select 
from the pop-up menu
.TP 4
square:
enter kernel size from keyboard or select 
from the pop-up menu; enter length of side
from keyboard or select from pop-up menu
.TP 4
rectangle:
enter kernel size from keyboard or select 
from the pop-up menu; enter height, width
from keyboard or select from pop-up menu
.TP 4
cross:
enter kernel size from keyboard or select 
from the pop-up menu; enter thickness (of
each arm of cross) and (overall) cross size
from keyboard or select from pop-up menu

.TP 4
Iterative Modification function:
For binary images only. A binary image can be created from 
any image by using the button at the bottom of the 
Analysis->Edge/Line detection window, or by using 
Utilties->Convert->Binary threshold.   

Parameters:
.TP 4
-rotation (y/n)
Consider all rotations of the surrounds, or only their initial rotation
shown with the View Surrounds button?
.TP 4
-boolean
Select the desired expression from the 
pop-up menu for the binary operation to be performed. Note that 'a' is
true only if the pixel's surround is in the set selected, 
and 'b' is the current pixel value.
.TP 4
-iteration
Select the desired number of iterations from
the pop-up menu or type directly into the
entry box. A 0 (zero) is used to represent infinity, 
which performs the function until no further
change is detected. Any value from 0 to 99
can be typed into the box.
.TP 4
-subfields
Select the desired number of subfields from
the pop-up menu or type directly into the
entry box. Must be 1, 2, or 3. This divides the image into subfields
that are operated on sequentially. When 1 is selected the operation 
is performed on the entire image in (simulated) parallel during 
each iteration.
.TP 4
-surrounds
Push the button for each surround that you
want applied to the image. You can see the
surrounds by pressing the 'View Surrounds'
button.
.SH }
