/***************************************************************************
* ======================================================================
* Computer Vision/Image Processing Tool Project - Dr. Scott Umbaugh SIUE
* ======================================================================
*
*             File Name: lin_xform.c
*           Description: 
*         Related Files: 
*   Initial Coding Date: Sun Sep 12 23:07:49 CDT 1993
*           Portability: Standard (ANSI) C
*            References:
*             Author(s): Gregory Hance
*                        Southern Illinois University @ Edwardsville
*
** Copyright (C) 1993 SIUE - by Gregory Hance.
**
** Permission to use, copy, modify, and distribute this software and its
** documentation for any purpose and without fee is hereby granted, provided
** that the above copyright notice appear in all copies and that both that
** copyright notice and this permission notice appear in supporting
** documentation.  This software is provided "as is" without express or
** implied warranty.
**
* RCS (Revision Control System) Information
* ...Added automatically by RCS 
*
** $Log: lin_xform.c,v $
** Revision 1.1  1994/10/30 17:58:29  hanceg
** CVIPtools1.0 source file - G. Hance
**
*
****************************************************************************/
#include "CVIPmap.h"

/****************************************************************
                   Name: linxform
          Expanded Name: linear transform
             Parameters: <cvecP> - pointer to an array of band vectors
                         <no_of_vecs> - number of vectors
			 <vdim> - dimension of each vector (number of pixels)
                         <transmatrix> - transformation matrix
                Returns: nada
            Description: linearly transforms a set of band vectors <cvecP>
			 through a transformation matrix <transmatrix>. The
			 transformation matrix must be have a dimension of
			 <no_of_vecs> by <no_of_vecs>.
	    Diagnostics: none
              Credit(s): Gregory Hance
		         Southern Illinois University @ Edwardsville
****************************************************************/
 
void linearTrans_Image(float **cvecP, unsigned int no_of_vecs, unsigned int vdim, double **transmatrix)
{
register int i=0, j, k;
float *tP;
double *dP;
const char *fn = "linxform";

msg_CVIP(fn, "performing linear color transform...\n");
/*
if( ... ) {
   fprintf(stderr, "%s-ERROR: transformation matrix must be %dx%d.\n", fn, no_of_vecs, no_of_vecs);
   return;
}
*/
msg_CVIP(fn, "transformation vectors being used:\n");
for ( j=0; j < no_of_vecs; j++ ) {
   msg_CVIP(fn, "[t%d] = ", j);
   dP = transmatrix[j];
   for( k=0; k < no_of_vecs; k++, dP++)
      print_CVIP(" %10.4f(x%d) ", *dP, k);
   print_CVIP("\n");
}

tP = (float *) malloc(no_of_vecs*sizeof(float));

for(; i < vdim; i++) {
   (void) memset((char *)tP, 0, (size_t) no_of_vecs*sizeof(float));
   for (j=0; j < no_of_vecs; j++) {
      dP = transmatrix[j];
      for(k=0; k < no_of_vecs; k++, dP++)
         tP[j] += (float) (*dP * (*(cvecP[k] + i)));
   }
   for(k=0; k < no_of_vecs; k++)
      *(cvecP[k] + i) = tP[k];
}

free(tP);
}
   
