/*****************************************************************************
 * ======================================================================    *
 * Image Processing/Computer Vision Tool Project - Dr. Scott Umbaugh SIUE    *
 * ======================================================================    *
 *                                                                           *
 * PROGRAM NAME: skeleton.c                                                  *                    
 *                                                                           *
 *                                                                           *
 *              DESCRIPTION: This is a skeleton program for processing an    *
 * image in the NeXT UNIX environment. The program takes a raw image file as *
 * input and requires the following information to be specified at run time: *
 * REQUIRED:                                                                 *
 * -i <inputfile>  - the name of the raw image input file with extension .bin*
 * -o <outputfile> - the name of the desired output file name with .bin ext. *
 * color format of image - graylevel => -m, RGB => -r                        *
 * OPTIONAL:                                                                 *
 * -f => put output in a displayable format in addition to the .bin format   *
 * -d => display the processed result at run time                            *
 * -s => suppress messages                                                   *
 *                                                                           *
 * NOTE TO IP/CV Tools DEVELOPERS:                                           * 
 * Certain commands such as those mentioned above will be implemented at the *    
 * command line at run time but any other input needed from the user, etc.   *
 * must be specified within the respective function. That is, any modules    *      
 * or functions added must be self contained and independent of the calling  * 
 * function if they are to be linked and used with IP/CV Tools. If you are   *
 * uncertain of what I mean please let me know.                              *  
 *                                                                           *                                                                          
 *                   AUTHOR: Gregory Hance                                   *
 *                                                                           *
 *            RELATED FILES: mydefine.h, get.h, matrix.h, image.h            * 
 *                           matrix.c, image.c                               *                                     
 *                                                                           *                                                             
 *              CODING DATE: 1/5/92                                          *
 *  LAST MODIFICATIONS MADE: 4/11/92                                         *
 *                                                                           *
 *****************************************************************************

RCS (Revision Control System) Information - Added automatically by RCS) 

$Author: yawei $                                                          
$Date: 1997/03/28 20:11:02 $                                                           
$Source: /opt/cvipapps/CVIPtools3.5/MatrixAlgebra/RCS/tma.c,v $                                                        
$Header: /opt/cvipapps/CVIPtools3.5/MatrixAlgebra/RCS/tma.c,v 1.5 1997/03/28 20:11:02 yawei Exp $

*******************************************************************************/
/*typedef enum {CVIP_NO, CVIP_YES} CVIP_BOOLEAN;*/
 
/* Define Preprocessor statements */
#include <stdio.h>
#include <stdlib.h>
#include "CVIPcomplex.h"
#include "CVIPimage.h"
#include <sys/file.h>
#include <sys/types.h>
#include <sys/uio.h>

/*
int
readproc(char *iohandle, char *buf, int len)
{
	return(read((int) *iohandle, buf, len));
}


int
writeproc(char *iohandle, char *buf, int len)
{
	return(write((int) *iohandle, buf, len));
}
*/

extern int abs_Matrix(Matrix *matP);

main(argc,argv)                  /* Begin main() */
int argc;
char *argv[];
{

/* Declare variables local to main() */

Matrix *RA, *RB ,*CA, *CB, *RESULT, *mP, nP;
FILE *fileP;
XDR xdri, xdro;
int x, y, *rowP;
float c, d;
double a ,b;
long i = 2001, j;
const char fn[] = {"matrix_test"};
typedef short type1;
typedef double type2;

#define COMPLEX_DEFINED

mP = new_Matrix(3,3,CVIP_FLOAT,COMPLEX);
for(y=0; y < 3; y++)
	for(x=0; x < 3; x++) {
		setRealVal_Matrix(mP,float,x,y,x+y);
#ifdef COMPLEX_DEFINED
		setImagVal_Matrix(mP,float,x,y,x+y);
#endif
	}


RA = new_Matrix(3,3,CVIP_FLOAT,COMPLEX);
for(y=0; y < 3; y++)
	for(x=0; x < 3; x++) {
		setRealVal_Matrix(RA,float,x,y,x-y);
#ifdef COMPLEX_DEFINED
		setImagVal_Matrix(RA,float,x,y,x+y);
#endif
	}

print_Matrix(RA);
print_Matrix(mP);

(void) abs_Matrix(RA);
print_Matrix(RA); 

(void) divide_Matrix(&RESULT, RA, mP, RA);

print_Matrix(RESULT);

exit(0);

write_Matrix(mP, "xdr.txt");
read_Matrix(&nP, "xdr.txt");

fprintf(stderr, "\n  type = %d\n", nP.data_type);
fprintf(stderr, "format = %d\n", nP.data_format);
fprintf(stderr, "  rows = %u\n", nP.rows);
fprintf(stderr, "  cols = %u", nP.cols);
print_Matrix(&nP);
/*
for(y=0; y < 3; y++)
	for(x=0; x < 3; x++)
		printf("nP[%d][%d] = %d\n", y, x,
		getVal_Matrix(&nP, int, x, y) );
*/

exit(0);

RA = new_Matrix(3, 3, CVIP_SHORT, REAL);
RB = new_Matrix(3, 3, CVIP_DOUBLE, REAL);
for(x=0;x<3;x++)
   for(y=0;y<3;y++) {
      ((type1 **) RA->rptr)[x][y] = x+y;
      ((type2 **) RB->rptr)[x][y] = x-y;
   }
((type1 **) RA->rptr)[0][0] += 2;
((type1 **) RA->rptr)[1][0] += 2;
((type1 **) RA->rptr)[2][0] += 3;

fprintf(stderr,"%s: TESTING REAL MATRICES:\n",fn);
fprintf(stderr,"%s: =====================\n",fn);
fprintf(stderr,"\n%s: Matrix A:\n",fn);
print_Matrix(RA);
fprintf(stderr,"\n%s: Matrix B:\n",fn);
print_Matrix(RB);
fprintf(stderr,"\n%s: testing... crop_Matrix(A,1,1,2,2)\n",fn);
RESULT = crop_Matrix(RA, 1, 1, 2, 2);
print_Matrix(RESULT);
fprintf(stderr,"\n%s: testing... det_Matrix(A)\n",fn);
fprintf(stderr,"determinat = %g\n", det_Matrix(RA));
fprintf(stderr,"\n%s: testing... invert_Matrix(A)\n",fn);
RESULT = invert_Matrix(RA);
print_Matrix(RESULT);
fprintf(stderr,"\n%s: testing... mult_Matrix(A,B)\n",fn);
RESULT = mult_Matrix(RA,RB);
print_Matrix(RESULT);
fprintf(stderr,"\n%s: testing... add_Matrix(A,B)\n",fn);
RESULT = add_Matrix(RA,RB);
print_Matrix(RESULT);
fprintf(stderr,"\n%s: testing... sub_Matrix(A,B)\n",fn);
RESULT = sub_Matrix(RA,RB);
print_Matrix(RESULT);
fprintf(stderr,"\n%s: testing... multPWise_Matrix(A,B)\n",fn);
RESULT = multPWise_Matrix(RA,RB);
print_Matrix(RESULT);
fprintf(stderr,"\n%s: testing... scale_Matrix(A)\n",fn);
RESULT = scale_Matrix(RA,2.0);
print_Matrix(RESULT);
fprintf(stderr,"\n%s: testing... matrix_trans(A)\n",fn);
RESULT = transpose_Matrix(RA);
print_Matrix(RESULT);
fprintf(stderr,"\n%s: testing... copy_Matrix(A,RESULT,CVIP_NO)\n",fn);
copy_Matrix(RA,RESULT);
print_Matrix(RESULT);
fprintf(stderr,"\n%s: testing... fastCopy_Matrix(B,RESULT)\n",fn);
fastCopy_Matrix(RA,RESULT);
print_Matrix(RESULT);
fprintf(stderr,"\n%s: testing... duplicate_Matrix(A,A->data_format)\n",fn);
RESULT = duplicate_Matrix(RA,RA->data_format);
print_Matrix(RESULT);
fprintf(stderr,"\n%s: testing... makeComplex_Matrix(A,CVIP_YES)\n",fn);
makeComplex_Matrix(RA,CVIP_YES);
print_Matrix(RA);
fprintf(stderr,"\n%s: testing... makeReal_Matrix(A,CVIP_YES)\n",fn);
makeReal_Matrix(RA,CVIP_YES);
print_Matrix(RA);


CA = new_Matrix(3, 3, CVIP_SHORT, COMPLEX);
CB = new_Matrix(3, 3, CVIP_DOUBLE, COMPLEX);
CASSIGN(CA,type1,0,0,2,3);CASSIGN(CA,type1,0,1,5,0);CASSIGN(CA,type1,0,2,6,7);
CASSIGN(CA,type1,1,0,5,1);CASSIGN(CA,type1,1,1,2,-3);CASSIGN(CA,type1,1,2,7,6);
CASSIGN(CA,type1,2,0,2,-1);CASSIGN(CA,type1,2,1,1,0);CASSIGN(CA,type1,2,2,6,-7);
CASSIGN(CB,type2,0,0,1,-2);CASSIGN(CB,type2,0,1,1,0);CASSIGN(CB,type2,0,2,3,4);
CASSIGN(CB,type2,1,0,2,1);CASSIGN(CB,type2,1,1,2,3);CASSIGN(CB,type2,1,2,1,5);
CASSIGN(CB,type2,2,0,2,-4);CASSIGN(CB,type2,2,1,1,3);CASSIGN(CB,type2,2,2,6,7);

fprintf(stderr,"%s: TESTING COMPLEX MATRICES:\n",fn);
fprintf(stderr,"%s: ========================\n",fn);
fprintf(stderr,"\n%s: Matrix A:\n",fn);
print_Matrix(CA);
fprintf(stderr,"\n%s: Matrix B:\n",fn);
print_Matrix(CB);
fprintf(stderr,"\n%s: testing... crop_Matrix(A,1,1,2,2)\n",fn);

   fprintf(stderr,"CA->rptr[0]=%ld\n",CA->rptr[0]);         
   fprintf(stderr,"CA->iptr[0]=%ld\n",CA->iptr[0]);

RESULT = crop_Matrix(CA, 1, 1, 2, 2);
fprintf(stderr,"Hello made it\n");
print_Matrix(RESULT);
fprintf(stderr,"\n%s: testing... invert_Matrix(A)\n",fn);
RESULT = invert_Matrix(CA);
print_Matrix(RESULT);
fprintf(stderr,"\n%s: testing... mult_Matrix(A,B)\n",fn);
RESULT = mult_Matrix(CA,CB);
print_Matrix(RESULT);
fprintf(stderr,"\n%s: testing... add_Matrix(A,B)\n",fn);
RESULT = add_Matrix(CA,CB);
print_Matrix(RESULT);
fprintf(stderr,"\n%s: testing... sub_Matrix(A,B)\n",fn);
RESULT = sub_Matrix(CA,CB);
print_Matrix(RESULT);
fprintf(stderr,"\n%s: testing... multPWise_Matrix(A,B)\n",fn);
RESULT = multPWise_Matrix(CA,CB);
print_Matrix(RESULT);
fprintf(stderr,"\n%s: testing... scale_Matrix(A)\n",fn);
RESULT = scale_Matrix(CA,2.0);
print_Matrix(RESULT);
fprintf(stderr,"\n%s: testing... matrix_trans(A)\n",fn);
RESULT = transpose_Matrix(CA);
print_Matrix(RESULT);
fprintf(stderr,"\n%s: testing... copy_Matrix(A,RESULT,CVIP_NO)\n",fn);
copy_Matrix(CA,RESULT);
print_Matrix(RESULT);
fprintf(stderr,"\n%s: testing... fastCopy_Matrix(B,RESULT)\n",fn);
fastCopy_Matrix(CA,RESULT);
print_Matrix(RESULT);
fprintf(stderr,"\n%s: testing... duplicate_Matrix(A,A->data_format)\n",fn);
RESULT = duplicate_Matrix(CA,CA->data_format);
print_Matrix(RESULT);

fprintf(stderr,"%s: TESTING COMBINATION MATRICES:\n",fn);
fprintf(stderr,"%s: ========================\n",fn);
fprintf(stderr,"\n%s: Matrix RA:\n",fn);
print_Matrix(RA);
fprintf(stderr,"\n%s: Matrix CB:\n",fn);
print_Matrix(CB);
fprintf(stderr,"\n%s: testing... mult_Matrix(RA,CB)\n",fn);
RESULT = mult_Matrix(RA,CB);
print_Matrix(RESULT);
fprintf(stderr,"\n%s: testing... add_Matrix(RA,CB)\n",fn);
RESULT = add_Matrix(RA,CB);
print_Matrix(RESULT);
fprintf(stderr,"\n%s: testing... sub_Matrix(RA,CB)\n",fn);
RESULT = sub_Matrix(RA,CB);
print_Matrix(RESULT);
fprintf(stderr,"\n%s: testing... multPWise_Matrix(RA,CB)\n",fn);
RESULT = multPWise_Matrix(RA,CB);
print_Matrix(RESULT);
fprintf(stderr,"\n%s: testing... copy_Matrix(RA,CB,CVIP_YES)\n",fn);
copy_Matrix(RA,CB);
print_Matrix(CB);
fprintf(stderr,"\n%s: testing... fastCopy_Matrix(RA,RESULT)\n",fn);
fastCopy_Matrix(RA,RESULT);
print_Matrix(RA);
fprintf(stderr,"\n%s: testing... duplicate_Matrix(RA,COMPLEX)\n",fn);
RESULT = duplicate_Matrix(RA,COMPLEX);
print_Matrix(RESULT);


delete_Matrix(RA);
delete_Matrix(RB);
delete_Matrix(CA);
delete_Matrix(CB);

exit(0);

}
