/***************************************************************************
* ======================================================================
* Computer Vision/Image Processing Tool Project - Dr. Scott Umbaugh SIUE
* ======================================================================
*
*             File Name: tobject.c
*           Description: Test program for functions in libobject.a.
*         Related Files: Imakefile, libobject.a
*   Initial Coding Date: Tue Aug 24 19:59:57 CDT 1993
*           Portability: Standard (ANSI) C
*             Credit(s): Gregory Hance
*                        Southern Illinois University @ Edwardsville
*
* RCS (Revision Control System) Information
* ...Added automatically by RCS 
*
** $Log: tobject.c,v $
** Revision 1.5  1997/03/08 17:03:11  yawei
** Swaped CVIP_YES and CVIP_NO
**
** Revision 1.4  1997/03/08 00:52:51  yawei
** Name Changes:
** 	BOOLEAN ==> CVIP_BOOLEAN
** 	FALSE ==> CVIP_YES
** 	TRUE ==> CVIP_NO
** 	BYTE ==> CVIP_BYTE
** 	SHORT ==> CVIP_SHORT
** 	INTEGER ==> CVIP_INTEGER
** 	FLOAT ==> CVIP_FLOAT
** 	DOUBLE ==> CVIP_DOUBLE
** 	TYPE ==> CVIP_TYPE
**
** Revision 1.3  1997/01/16 21:01:26  yawei
** Global Change:  Image_FORMAT ==> IMAGE_FORMAT
**
 * Revision 1.2  1997/01/15  17:13:44  yawei
 * Global Change:  IMAGE ==> Image
 *
 * Revision 1.1  1994/10/30  17:58:51  hanceg
 * CVIPtools1.0 source file - G. Hance
 *
*
****************************************************************************/
/* Define Preprocessor statements */
#include "CVIPtoolkit.h"
#include "CVIPconvert.h"
#include "CVIPdef.h"
#include "CVIPobject.h"


main(int argc, char *argv[])
{
/* Declare variables local to main() */
Image *cvipImage, *labelP, *imageP;
IMAGE_FORMAT format;
char inputfile[50], outputfile[50], flabel[80], fprop[80], fcontour[80];
const char *usage = "[-display] [infile] [outfile]", *fn = argv[0];
int argn=1, *xP, *yP, img_size, ray_x, ray_y;
CVIP_BOOLEAN display = CVIP_NO, label = CVIP_NO, properties = CVIP_NO, bb = CVIP_NO, contour = CVIP_NO, trim=CVIP_NO, file_flag = CVIP_NO;
ObjectList listP;
Object *objP;
ROI *roiP;
Color pixel = {1,1,1};
ChainCode *ccP;

/* initialize global command options */
cvip_init( &argc, argv );

/* process command line options */
while ( argn < argc && argv[argn][0] == '-' && argv[argn][1] != '\0' ) {
   if ( cvip_keymatch( argv[argn], "-display", 2 ) )
      display = CVIP_YES;
   else if( cvip_keymatch( argv[argn], "-label", 2 ) ) {
      ++argn;
      label = CVIP_YES;
      if ( argn == argc || sscanf( argv[argn], "%s", flabel ) != 1 )
         usage_CVIP( fn, usage );
   }
   else if( cvip_keymatch( argv[argn], "-properties", 2 ) ) {
      ++argn;
      properties = CVIP_YES;
      if ( argn == argc || sscanf( argv[argn], "%s", fprop ) != 1 )
         usage_CVIP( fn, usage );
   }
   else if ( cvip_keymatch( argv[argn], "-bb", 2 ) )
      bb = CVIP_YES;
   else if ( cvip_keymatch( argv[argn], "-trim", 2 ) )
      trim = CVIP_YES;
   else if ( cvip_keymatch( argv[argn], "-file", 3 ) )
      file_flag = CVIP_YES;
   else if ( cvip_keymatch( argv[argn], "-contour", 2 ) ) {
      ++argn;
      contour = CVIP_YES;
      if ( argn == argc || sscanf( argv[argn], "%s", fcontour ) != 1 )
         usage_CVIP( fn, usage );
   }

   else
      cvip_usage( fn, usage );
   argn++;
}

/* decide where to read input from and where to send output */
if(argc > argn) {
   if ( (argv[argn][0] == '-' && argv[argn][1] != '\0') || sscanf( argv[argn], "%s", inputfile ) != 1 )
      cvip_usage( fn, usage );
   argn++;
   if ( argc > argn ) {
      if ( (argv[argn][0] == '-' && argv[argn][1] != '\0') || sscanf( argv[argn], "%s", outputfile ) != 1 )
         cvip_usage( fn, usage );
      argn++;
   }
   else
      (void) strcpy(outputfile, "-");
   if( argc != argn )
      cvip_usage( fn, usage );
}
else {
   (void) strcpy(inputfile, "-");
   (void) strcpy(outputfile, "-");
}



format = CVIPgetformat(inputfile);
cvipImage = CVIPreadimage(inputfile, format, 1);


if(label) {

	labelP = setUpLabel_Objects( cvipImage );
   	image_free(cvipImage);
	cvipImage = labelP;

}
   
if(properties) {
   if(file_flag) {
   	listP = new_LL();
   	readLabel_Objects(listP, fprop);
   }
   getProp_Objects(listP, cvipImage);
   printProp_Objects(listP, fprop);
}

if(trim) {
	trimList_Objects(listP, getNoOfCols_Image(cvipImage), 
	getNoOfRows_Image(cvipImage), 0.0075, 0.9, 0.2, 1.0, -90.0, 90.0, 0.5, 0.5, CVIP_YES );
	fprintf(stderr,"\n!!!! Trimmed List !!!!\n");
	printProp_Objects(listP, "+");
	/*
	labelP = new_Image( PGM, GRAY_SCALE, 1, getNoOfRows_Image(cvipImage),
	getNoOfCols_Image(cvipImage), CVIP_INTEGER, REAL);
	head_LL(listP);
	objP = retrieve_LL(listP);
	selectCopy_Image(cvipImage, labelP, &objP->label, 1);
	delete_Image(cvipImage);
	cvipImage = labelP;
	*/
}


if(contour) {
	imageP = new_Image( PBM, BINARY, 1, getNoOfRows_Image(cvipImage), 	
	getNoOfCols_Image(cvipImage), CVIP_BYTE, REAL);

	head_LL(listP);

	for (;;) {
		objP = retrieve_LL(listP);
		shootRay(cvipImage, objP->label, &ray_x, &ray_y, objP->x_min, 
		objP->y_min, objP->x_max, objP->y_max);
		ccP = new_ChainCode(ray_y,ray_x,objP->label);
		build_ChainCode(ccP,cvipImage,objP->x_min,objP->y_min, 	
		objP->x_max, objP->y_max);
		if(file_flag) {
			print_ChainCode(ccP, fcontour);
			delete_ChainCode(ccP);
			ccP = read_ChainCode(fcontour);
		}
		draw_ChainCode(ccP, imageP);
		if (istail_LL(listP)) break;
		next_LL(listP);
	}

	delete_Image(cvipImage);
	cvipImage = imageP;
}


if(bb) {
    cvipImage = remap_Image(cvipImage, CVIP_BYTE, 0, 255);
    drawBB_Objects(cvipImage, listP);
}

if(getDataType_Image(cvipImage) != CVIP_BYTE)
	cvipImage = remap_Image(cvipImage, CVIP_BYTE, 0, 255);
CVIPwriteimage(cvipImage, outputfile, CVIP_NO, CVIP_NO, format, 1);


if( display && strcmp(outputfile,"-") )
   cvip_display( outputfile, format );
else if( display )
   cvip_error(fn, "can not display an image directed to \"stdout\".");

image_free(cvipImage);
exit(0);
} /* End Main */
