/***************************************************************************
* ======================================================================
* Computer Vision/Image Processing Tool Project - Dr. Scott Umbaugh SIUE
* ======================================================================
*
*             File Name: tmanager.c
*           Description: Test program for functions in libmanager.a.
*         Related Files: Imakefile, libmanager.a
*   Initial Coding Date: Tue Aug 24 20:12:21 CDT 1993
*           Portability: Standard (ANSI) C
*             Credit(s): Gregory Hance
*                        Southern Illinois University @ Edwardsville
*
* RCS (Revision Control System) Information
* ...Added automatically by RCS 
*
** $Log: tmanager.c,v $
** Revision 1.6  1997/05/21 14:15:25  yawei
** added binary fopen() for WIN32
**
** Revision 1.5  1997/03/08 17:07:32  yawei
** Swaped CVIP_YES and CVIP_NO
**
** Revision 1.4  1997/03/08 01:02:02  yawei
** Name Changes:
** 	BOOLEAN ==> CVIP_BOOLEAN
** 	FALSE ==> CVIP_YES
** 	TRUE ==> CVIP_NO
** 	BYTE ==> CVIP_BYTE
** 	SHORT ==> CVIP_SHORT
** 	INTEGER ==> CVIP_INTEGER
** 	FLOAT ==> CVIP_FLOAT
** 	DOUBLE ==> CVIP_DOUBLE
** 	TYPE ==> CVIP_TYPE
**
** Revision 1.3  1997/01/16 21:32:49  yawei
** Global Change:  Image_FORMAT ==> IMAGE_FORMAT
**
 * Revision 1.2  1997/01/15  17:20:10  yawei
 * Global Change:  IMAGE ==> Image
 *
 * Revision 1.1  1994/10/30  17:59:07  hanceg
 * CVIPtools1.0 source file - G. Hance
 *
*
****************************************************************************/
/* Define Preprocessor statements */
#include "CVIPtoolkit.h"
#include "CVIPconvert.h"
#include "CVIPdef.h"
#include "CVIPmanager.h"

struct dogType {
   char myName[100];
   char myBark[100];
   int nBarks;
};

typedef struct dogType dog;

static dog *new_Dog(void);
static void describe_Dog(dog *this, char *newName, int newN, char *newBark);
static void bark_Dog(void *this, FILE *fileP);
static void delete_Dog(void *this);
static int match_Dog(void *this, void *lookforP);

typedef dlinked_list *ImagePathList;

ImagePathList 	new_ImagePathList(void);
void 		print_ImagePathList(ImagePathList iplP, FILE *fileP);
int 		init_ImagePathList(ImagePathList iplP, const char *name);
char *		find_ImagePathList(ImagePathList iplP, const char *fname);
void 		addtail_ImagePathList(ImagePathList iplP, const char *path);


main(int argc, char *argv[])
{
/* Declare variables local to main() */
Image *cvipImage;
IMAGE_FORMAT format;
char inputfile[50], outputfile[50], name[50], *strP;
const char *usage = "[-display] [infile] [outfile]", *fn = argv[0];
int argn=1;
register int i;
CVIP_BOOLEAN display = CVIP_NO, ll=CVIP_NO, dll = CVIP_NO,stack_flag=CVIP_NO, ipl = CVIP_NO;
dog *golden, *silky, *boxer;
dlinked_list *list;
linked_list *list1;
Stack StackP;
ImagePathList	iplP;

/* initialize global command options */
cvip_init( &argc, argv );

/* process command line options */
while ( argn < argc && argv[argn][0] == '-' && argv[argn][1] != '\0' ) {
   if ( cvip_keymatch( argv[argn], "-display", 2 ) )
      display = CVIP_YES;
   else if ( cvip_keymatch( argv[argn], "-ll", 2 ) )
      ll = CVIP_YES;
   else if ( cvip_keymatch( argv[argn], "-ipl", 2 ) ) {
      ++argn;
      ipl = CVIP_YES;
      if ( argn == argc || sscanf( argv[argn], "%s", name ) != 1 )
         usage_CVIP( fn, usage );
   }
   else if ( cvip_keymatch( argv[argn], "-dll", 2 ) )
      dll = CVIP_YES;
   else if ( cvip_keymatch( argv[argn], "-stack", 2 ) )
      stack_flag = CVIP_YES;
   else
      cvip_usage( fn, usage );
   argn++;
}


/* decide where to read input from and where to send output */
if(argc > argn) {
   if ( (argv[argn][0] == '-' && argv[argn][1] != '\0') || sscanf( argv[argn], "%s", inputfile ) != 1 )
      cvip_usage( fn, usage );
   argn++;
   if ( argc > argn ) {
      if ( (argv[argn][0] == '-' && argv[argn][1] != '\0') || sscanf( argv[argn], "%s", outputfile ) != 1 )
         cvip_usage( fn, usage );
      argn++;
   }
   else
      (void) strcpy(outputfile, "-");
   if( argc != argn )
      cvip_usage( fn, usage );
}
else {
   (void) strcpy(inputfile, "-");
   (void) strcpy(outputfile, "-");
}

if(ipl) {

	iplP = new_ImagePathList();
	(void) init_ImagePathList(iplP, "CVIP_IMGPATH");
	addtail_ImagePathList(iplP,"/usr/images/EE_courses/ee438");
	print_ImagePathList(iplP, stderr);
	if((strP=find_ImagePathList(iplP, name)) == NULL)
		fprintf(stderr,"couldn't find it!\n");
	else
		fprintf(stderr,"path = %s\n", strP);
}



if(stack_flag) {
  
   golden = new_Dog();
   silky = new_Dog();
   boxer = new_Dog();

   describe_Dog(golden, "Golden", 5, "Woof!");
   describe_Dog(silky, "Silky", 3, "Yipe!");
   describe_Dog(boxer, "Boxer", 1, "Arf!");

   StackP = new_Stack();

   push_Stack(StackP, golden);
   push_Stack(StackP, silky);
   push_Stack(StackP, boxer);

   while( !isempty_Stack(StackP) ) bark_Dog(pop_Stack(StackP), stderr);

}




if(dll) {
   fprintf(stderr," Testing double list!!!\n");
   fprintf(stderr," ======================\n\n");

   golden = new_Dog();
   silky = new_Dog();
   boxer = new_Dog();

   describe_Dog(golden, "Golden", 5, "Woof!");
   describe_Dog(silky, "Silky", 3, "Yipe!");
   describe_Dog(boxer, "Boxer", 1, "Arf!");

   list = new_DLL();

   addhead_DLL(list, golden);
   addhead_DLL(list, silky);
   addhead_DLL(list, boxer);
   addtail_DLL(list, boxer);
   addnext_DLL(list, golden);
   addnext_DLL(list, silky);

   print_DLL(list, bark_Dog, stderr);

   fprintf(stderr,"no. of dogs = %d.\n", size_DLL(list) );
   fprintf(stderr,"\n");


   find_DLL(list, match_Dog, "Boxer" );
   bark_Dog(retrieve_DLL(list), stderr);
   find_DLL(list, match_Dog, "Silky" );
   bark_Dog(retrieve_DLL(list), stderr);
   find_DLL(list, match_Dog, "Golden" );
   bark_Dog(retrieve_DLL(list), stderr);

   head_DLL(list);

   fprintf(stderr,"\n");
   promote_DLL(list);
   next_DLL(list);
   promote_DLL(list);
   tail_DLL(list);
   promote_DLL(list);
   print_DLL(list, bark_Dog, stderr);

   removetail_DLL(list);
   removehead_DLL(list);
   next_DLL(list);
   next_DLL(list);
   removecurr_DLL(list);

   fprintf(stderr,"\n");
   print_DLL(list, bark_Dog, stderr);
   fprintf(stderr,"no. of dogs = %d.\n", size_DLL(list) );

   fprintf(stderr,"\n");
   print_reverse_DLL(list, bark_Dog, stderr);
   fprintf(stderr,"no. of dogs = %d.\n", size_DLL(list) );

   delete_DLL(list, delete_Dog);


}

if(ll) {
   fprintf(stderr," Testing single list!!!\n");
   fprintf(stderr," ======================\n\n");

   golden = new_Dog();
   silky = new_Dog();
   boxer = new_Dog();

   describe_Dog(golden, "Golden", 5, "Woof!");
   describe_Dog(silky, "Silky", 3, "Yipe!");
   describe_Dog(boxer, "Boxer", 1, "Arf!");

   list1 = new_LL();

   addhead_LL(list1, golden);
   addhead_LL(list1, silky);
   addhead_LL(list1, boxer);
   addnext_LL(list1, golden);
   next_LL(list1);
   next_LL(list1);
   addnext_LL(list1, boxer);
   addnext_LL(list1, silky);


   print_LL(list1, bark_Dog, stderr);

   fprintf(stderr,"no. of dogs = %d.\n", size_LL(list1) );
   fprintf(stderr,"\n");


   find_LL(list1, match_Dog, "Boxer" );
   bark_Dog(retrieve_LL(list1), stderr);
   find_LL(list1, match_Dog, "Silky" );
   bark_Dog(retrieve_LL(list1), stderr);
   find_LL(list1, match_Dog, "Golden" );
   bark_Dog(retrieve_LL(list1), stderr);

   head_LL(list1);

   fprintf(stderr,"\n");
   promote_LL(list1);
   next_LL(list1);
   promote_LL(list1);
   tail_LL(list1);
   promote_LL(list1);
   print_LL(list1, bark_Dog, stderr);

   removehead_LL(list1);
   removenext_LL(list1);
   removenext_LL(list1);

   fprintf(stderr,"\n");
   print_LL(list1, bark_Dog, stderr);
   fprintf(stderr,"no. of dogs = %d.\n", size_LL(list1) );

   delete_LL(list1, delete_Dog);

}

#if 0
format = CVIPgetformat(inputfile);
cvipImage = CVIPreadimage(inputfile, format, 1);

CVIPwriteimage(cvipImage, outputfile, CVIP_NO, CVIP_NO, format, 1);

if( display && strcmp(outputfile,"-") )
   cvip_display( outputfile, format );
else if( display )
   cvip_error(fn, "can not display an image directed to \"stdout\".");

image_free(cvipImage);
#endif

exit(0);
} /* End Main */


static dog *new_Dog(void)
{
   return (dog *) malloc(sizeof(dog));
}

static void describe_Dog(dog *this, char *newName, int newN, char *newBark)
{
   this->nBarks = newN;
   strcpy(this->myName, newName);
   strcpy(this->myBark, newBark);
}

static void bark_Dog(void *this, FILE *fileP)
{  
   dog *bark = this;
   int n;
   printf("%s goes\n", bark->myName);
   for(n=0; n < bark->nBarks; n++) printf(" %s\n", bark->myBark);
}

static void delete_Dog(void *this)
{
   free(this);
}
      
static int match_Dog(void *this, void *lookforP)
{
   dog *dogP = this;

   return( !strcmp(dogP->myName,lookforP) );

}


ImagePathList
new_ImagePathList(void)
{
	return new_DLL();
}
      
int
init_ImagePathList(
	ImagePathList	iplP,
	const char 	*name
)
{
	register int	i;
	char 		*valP, **pnP, *pP;
	int		n;

	valP = getEnv_CVIP(name);
	if(valP == NULL) return 0;

	pnP = seperateStrings_CVIP(valP, ':', &n);

	head_DLL(iplP);

	for(i=0; i < n; i++) {
		pP = malloc(strlen(pnP[i])+1);
		(void) strcpy(pP, pnP[i]);
   		addtail_DLL(iplP, pP);
	}

	free(*pnP);
	free(pnP);
	free(valP);

	return 1;
}

void
addtail_ImagePathList(
	ImagePathList	iplP,
	const char	*path
)
{
	char		*strP;

	strP = malloc(strlen(path)+1);
	(void) strcpy(strP, path);
	addtail_DLL(iplP, strP);
}

	
	

static void
printIt(
	void	*objectP,
	FILE	*fileP
)
{
	fprintf(fileP,"%s\n",(char *) objectP);
}

void
print_ImagePathList(
	ImagePathList	iplP,
	FILE		*fileP
)
{
	print_DLL(iplP, &(printIt), fileP);
}


char *
find_ImagePathList(
	ImagePathList	iplP,
	const char	*fname
)
{
	char 	buf[160], *strP;

	head_DLL(iplP);

	for(;;) {
 		strP = retrieve_DLL(iplP);

		strcpy(buf, strP);
		strcat(buf, "/");
		strcat(buf, fname);

		#ifndef WIN32
		if( fopen(buf, "r") ) return strP;
		#else
		if( fopen(buf, "rb") ) return strP;
		#endif
		if( istail_DLL(iplP) ) break;
      		next_DLL(iplP);
	}

	return NULL;
}
