/***************************************************************************
* ======================================================================
* Computer Vision/Image Processing Tool Project - Dr. Scott Umbaugh SIUE
* ======================================================================
*    
*             File Name: spatial_filter.c
*           Description:  contain spatial filters
*         Related Files: 
*   Initial Coding Date: Sun April 17, 15:54:12 CDT 1995
*           Portability: Standard (ANSI) C
*             Credit(s): Kun Luo, John  Creighton
*                        Southern Illinois University @ Edwardsville
*
** Copyright (C) 1995 SIUE - by  Kun Luo, John Creighton
**
** Permission to use, copy, modify, and distribute this software and its
** documentation for any purpose and without fee is hereby granted, provided
** that the above copyright notice appear in all copies and that both that
** copyright notice and this permission notice appear in supporting
** documentation.  This software is provided "as is" without express or
** implied warranty.
**
** $Log: spatial_filter.c,v $
** Revision 1.11  1997/07/26 17:48:27  smakam
** corrected the code for midpoint filter
**
** Revision 1.10  1997/03/08 01:03:20  yawei
** Name Changes:
** 	BOOLEAN ==> CVIP_BOOLEAN
** 	FALSE ==> CVIP_YES
** 	TRUE ==> CVIP_NO
** 	BYTE ==> CVIP_BYTE
** 	SHORT ==> CVIP_SHORT
** 	INTEGER ==> CVIP_INTEGER
** 	FLOAT ==> CVIP_FLOAT
** 	DOUBLE ==> CVIP_DOUBLE
** 	TYPE ==> CVIP_TYPE
**
** Revision 1.9  1997/01/15 17:21:08  yawei
** Global Change:  IMAGE ==> Image
**
 * Revision 1.8  1996/12/24  18:42:15  yawei
 * chaged function name:
 *   alfa_filter ==> alpha_filter
 *
 * Revision 1.7  1996/01/23  18:26:11  kluo
 * check in
 *
 * Revision 1.6  1995/07/31  17:33:54  kluo
 * add header
 *
 * Revision 1.5  1995/06/16  05:28:52  luok
 * get rid of abs function in YP mean
 *
 * Revision 1.4  1995/06/12  19:18:26  luok
 * modify the left over =
 *
 * Revision 1.3  1995/06/12  14:00:52  luok
 * fix a bug in new-added filters, not use = in the outer loop.
 *
 * Revision 1.2  1995/06/09  02:56:35  luok
 * rewrite alapha-trim, harmonic, YPmean, contra-harmonic filters
 * completely
 *
 * Revision 1.1  1995/06/06  23:55:38  luok
 * Initial revision
 *
 * Revision 1.1  1995/04/17  21:23:27  luok
 * Initial revision
 *
****************************************************************************/
#include <CVIPspfltr.h>
#include <float.h>


Image* minimum_setup(Image *cvipImage)      
  {      
     int mask_size;      
printf(" ***************   THE MINIMUM FILTER   ***********\n");             
      
do{      
	printf("\nType in an integer value [3,5,7,9 or 11] for mask size:");      
	scanf("%d",&mask_size);      
      
if(mask_size == 3) break;      
if(mask_size == 5) break;      
if(mask_size == 7) break;      
if(mask_size == 9) break;      
if(mask_size == 11) break;      
else printf("\nInvalid Selection !!!!!  Try Again !!!!!\n");      
}while(1);      
      
return minimum_filter(cvipImage,mask_size);      
}      
      
Image* minimum_filter(Image *imageP, int mask_size)      
       {      
	Image  *imageW;
        int N;  
	register int    i, j, k, J, I,x;      
	unsigned bands, rows, cols;      
      
	bands = getNoOfBands_Image(imageP);      
	rows =  getNoOfRows_Image(imageP);      
	cols =  getNoOfCols_Image(imageP);      
             
	N=mask_size;      
 	imageW = duplicate_Image(imageP);
	switch(imageP->bandP[0]->data_type){
	   case CVIP_BYTE:{
        	register byte   *bP, *bP1;      
	        int minimum[121];    
		register int smin;
		for(i=0; i < bands; i++)      
 	        {      
  	         for(j=N/2; j < (rows-N/2); j++,bP++)      
   	         {      						      				         
    	         bP=(byte*)getRow_Image(imageW,j,i); 

     	        for(k=N/2; k < (cols-N/2); k++,bP1++)
	     	 {	      
       	          x=0, smin=255; 

        	        for(J = -N/2; J <= N/2 ; J++)
         	          {	
	                     bP1=(byte*)getRow_Image(imageP,j+J,i);
							          
  	                     for(I=-N/2;I<=N/2;I++,x++)
   	                       { 
    	                        minimum[x] = *(bP1+k+I);
     	                      }
      	             }                 
			
   	               for(J=0;J<N*N;J++)
    	               {
     	              if(minimum[J] < smin)
      	              smin = minimum[J];
	      	       } 
                                 					                                       
		*(bP + k)=(byte)(smin);

		}      
     	   	 }      
		} 
	  }
	  break;     
	  case CVIP_SHORT:{  
        	register short   *bP, *bP1;      
	        int minimum[121];    
		register int smin;
		for(i=0; i < bands; i++)      
 	        {      
  	         for(j=N/2; j < (rows-N/2); j++,bP++)      
   	         {      						      				         
    	         bP=getRow_Image(imageW,j,i); 

     	        for(k=N/2; k < (cols-N/2); k++,bP1++)
	     	 {	      
       	          x=0, smin=SHRT_MAX; 

        	        for(J = -N/2; J <= N/2 ; J++)
         	          {	
	                     bP1=getRow_Image(imageP,j+J,i);
							          
  	                     for(I=-N/2;I<=N/2;I++,x++)
   	                       { 
    	                        minimum[x] = *(bP1+k+I);
     	                      }
      	             }                 
			
   	               for(J=0;J<N*N;J++)
    	               {
     	              if(minimum[J] < smin)
      	              smin = minimum[J];
	      	       } 
                                 					                                       
		*(bP + k)=(smin);

		}      
     	   	 }      
		} 
	  }
	  break;     	
	case CVIP_INTEGER:{
        	register long   *bP, *bP1;      
	        long minimum[121];    
		register long smin;
		for(i=0; i < bands; i++)      
 	        {      
  	         for(j=N/2; j < (rows-N/2); j++,bP++)      
   	         {      						      				         
    	         bP=getRow_Image(imageW,j,i); 

     	        for(k=N/2; k < (cols-N/2); k++,bP1++)
	     	 {	      
       	          x=0, smin=LONG_MAX; 

        	        for(J = -N/2; J <= N/2 ; J++)
         	          {	
	                     bP1=getRow_Image(imageP,j+J,i);
							          
  	                     for(I=-N/2;I<=N/2;I++,x++)
   	                       { 
    	                        minimum[x] = *(bP1+k+I);
     	                      }
      	             }                 
			
   	               for(J=0;J<N*N;J++)
    	               {
     	              if(minimum[J] < smin)
      	              smin = minimum[J];
	      	       } 
                                 					                                       
		*(bP + k)=(smin);

		}      
     	   	 }      
		} 
	  }
	  break;  
	case CVIP_FLOAT:{
        	register float   *bP, *bP1;      
	        float minimum[121];    
		register float smin;
		for(i=0; i < bands; i++)      
 	        {      
  	         for(j=N/2; j < (rows-N/2); j++,bP++)      
   	         {      						      				         
    	         bP=getRow_Image(imageW,j,i); 

     	        for(k=N/2; k < (cols-N/2); k++,bP1++)
	     	 {	      
       	          x=0, smin=FLT_MAX; 

        	        for(J = -N/2; J <= N/2 ; J++)
         	          {	
	                     bP1=getRow_Image(imageP,j+J,i);
							          
  	                     for(I=-N/2;I<=N/2;I++,x++)
   	                       { 
    	                        minimum[x] = *(bP1+k+I);
     	                      }
      	             }                 
			
   	               for(J=0;J<N*N;J++)
    	               {
     	              if(minimum[J] < smin)
      	              smin = minimum[J];
	      	       } 
                                 					                                       
		*(bP + k)=(smin);

		}      
     	   	 }      
		} 
	  }
	  break;    
	case CVIP_DOUBLE:{
        	register double   *bP, *bP1;      
	        double minimum[121];    
		register double smin;
		for(i=0; i < bands; i++)      
 	        {      
  	         for(j=N/2; j < (rows-N/2); j++,bP++)      
   	         {      						      				         
    	         bP=getRow_Image(imageW,j,i); 

     	        for(k=N/2; k < (cols-N/2); k++,bP1++)
	     	 {	      
       	          x=0, smin=DBL_MAX; 

        	        for(J = -N/2; J <= N/2 ; J++)
         	          {	
	                     bP1=getRow_Image(imageP,j+J,i);
							          
  	                     for(I=-N/2;I<=N/2;I++,x++)
   	                       { 
    	                        minimum[x] = *(bP1+k+I);
     	                      }
      	             }                 
			
   	               for(J=0;J<N*N;J++)
    	               {
     	              if(minimum[J] < smin)
      	              smin = minimum[J];
	      	       } 
                                 					                                       
		*(bP + k)=(smin);

		}      
     	   	 }      
		} 
	  }
	  break;     	 
     }	   	
     delete_Image(imageP);
     return imageW;
}     
  
Image * mean_setup(Image *cvipImage)  
  {  
     int mask_size;  
   
 printf(" **********  THE MEAN FILTER  **********\n");           
  
do{  
	printf("\nType in an integer value [3,5,7,9 or 11] for mask size:");  
	scanf("%d",&mask_size);  
  
if(mask_size == 3) break;  
if(mask_size == 5) break;  
if(mask_size == 7) break;  
if(mask_size == 9) break;  
if(mask_size == 11) break;  
else printf("\nInvalid Selection !!!!!  Try Again !!!!!\n");  
}while(1);  
  
return mean_filter(cvipImage,mask_size);  
}  
  
Image * mean_filter(Image *imageW, int mask_size)  
       {  
	Image *imageP;
        byte   *bP1, *bP;  
	int    i, j, k;  
	int I, J, N, sum,mean[121],a,M;  	   
	unsigned bands, rows, cols;  
	float  *fP1, *fP;
	float 	fsum,fmean[121];


	bands = getNoOfBands_Image(imageW);  
	rows =  getNoOfRows_Image(imageW);  
	cols =  getNoOfCols_Image(imageW);  
  
	N=mask_size;  
      
      if(imageW->bandP[0]->data_type != CVIP_BYTE){
	   if(imageW->bandP[0]->data_type != CVIP_FLOAT && cast_Image(imageW,CVIP_FLOAT)){
		fprintf(stderr,"Image is cast into float \n");
	   }
	   else if(imageW->bandP[0]->data_type != CVIP_FLOAT ){
		fprintf(stderr,"cast_Image failed..\n");
		delete_Image(imageW);
		return NULL;
	   }
	   imageP = duplicate_Image(imageW);
      	   for(i=0; i < bands; i++)  
		{  
		for(j=N/2; j < (rows-N/2); j++,fP++)  
		 {    

                   fP=(float*)getRow_Image(imageW,j,i); 

  		   for(k=N/2; k < (cols-N/2); k++, fP1++)   
  	           {  		      	                      
                                M=0,fsum=0;
                               				
                               for(J=-N/2;J<=N/2;J++)
  			         {                                  					                                                                                                                                                                                                         
                                     fP1=(float*)getRow_Image(imageP,j+J,i);
						               						  
                                    for(a=-N/2;a<=N/2;a++,M++)
                                    { 
                                      fmean[M]=*(fP1+k+a);
                                      }
                                   }    

                        for(I = 0; I < N*N; I++) 
                            {
                           fsum += fmean[I];
                             }
                        

                *(fP+k) = (fsum/(N*N));
       }
      }  
    }  
      }
      else{
	   imageP = duplicate_Image(imageW);
      	   for(i=0; i < bands; i++)  
		{  
		for(j=N/2; j < (rows-N/2); j++,bP++)  
		 {    

                   bP=(byte*)getRow_Image(imageW,j,i); 

  		   for(k=N/2; k < (cols-N/2); k++, bP1++)   
  	           {  		      	                      
                                M=0,sum=0;
                               				
                               for(J=-N/2;J<=N/2;J++)
  			         {                                  					                                                                                                                                                                                                         
                                     bP1=(byte*)getRow_Image(imageP,j+J,i);
						               						  
                                    for(a=-N/2;a<=N/2;a++,M++)
                                    { 
                                      mean[M]=*(bP1+k+a);
                                      }
                                   }    

                        for(I = 0; I < N*N; I++) 
                            {
                           sum += mean[I];
                             }
                        

                *(bP+k) = (byte)(sum/(N*N));
       }
      }  
    }  
  }  
  delete_Image(imageP);
  return imageW;
}  

  
Image *alpha_setup(Image *cvipImage){   
   int mask_size, p;   
   printf(" ***********   THE ALFA-TRIMMED MEAN FILTER  **********\n");  
   do{   
	printf("\nType in an integer value [3,5,7,9 or 11] for mask size:");   
	scanf("%d",&mask_size);      
	if(mask_size == 3) break;   
	if(mask_size == 5) break;   
	if(mask_size == 7) break;   
	if(mask_size == 9) break;   
	if(mask_size == 11) break;   
	else printf("\nInvalid Selection !!!!!  Try Again !!!!!\n");   
    }while(1);   
   
    do{   
	printf("\nType in an integer value [0-%d] for the trim size:",
		mask_size*mask_size/2);   
	scanf("%d",&p);
	if(p >= 0 && p <= mask_size*mask_size)
	    break;
	else 
	    printf("\nInvalid Selection !!!!!  Try Again !!!!!\n");   
    }while(1);   
      
    return alpha_filter(cvipImage,mask_size,p);   
}   
   
Image* alpha_filter(Image *imageW, int mask_size, int p){
   Image *imageP;
   byte   **bP,**bP1;   
   int    no_of_bands,i, j,J, I,x,area;           
   int  A, AR[121], sum;     
   unsigned bands, rows, cols; 
   float  **fP,**fP1;
   float  fAR[121],fsum,fA;

   bands = getNoOfBands_Image(imageW);   
   rows =  getNoOfRows_Image(imageW);   
   cols =  getNoOfCols_Image(imageW);   
   area = mask_size * mask_size;

   if(imageW->bandP[0]->data_type != CVIP_BYTE ){
	if(imageW->bandP[0]->data_type != CVIP_FLOAT && cast_Image(imageW,CVIP_FLOAT)){
	    fprintf(stderr,"Image is cast into float \n");
	}
	else if(imageW->bandP[0]->data_type != CVIP_FLOAT ){
	    fprintf(stderr,"cast_Image failed..\n");
		 delete_Image(imageW);
	    return NULL;
	}
	imageP = duplicate_Image(imageW);
     	for(no_of_bands=0; no_of_bands < bands; no_of_bands++){   
	    fP = getData_Image(imageP,no_of_bands);
	    fP1 = getData_Image(imageW,no_of_bands);
	    for(i = mask_size/2; i < rows - mask_size/2; i++){
		for(j = mask_size/2; j < cols - mask_size/2; j++){
		     x = 0;
		     for(I = i - mask_size/2; I <= i + mask_size/2; I++)
			 for(J = j - mask_size/2; J <= j + mask_size/2;J++)
				fAR[x++] = fP[I][J];
		     for(I = 0; I < area; I++){
			 for(J = I; J < area; J++){
			     if(fAR[I] > fAR[J]){
				  fA = fAR[J];
				  fAR[J] = fAR[I];
				  fAR[I] = fA;
			     }
			 }
		     }
		     fsum = 0.0;
		     for(I = p; I < area-p; I++)
			 fsum += fAR[I];
		     fsum /= (area - 2*p);
		     fP1[i][j] = fsum;
		}
	    }
	}	 
   }
   else{
	imageP = duplicate_Image(imageW);
     	for(no_of_bands=0; no_of_bands < bands; no_of_bands++){   
	    bP = getData_Image(imageP,no_of_bands);
	    bP1 = getData_Image(imageW,no_of_bands);
	    for(i = mask_size/2; i < rows - mask_size/2; i++){
		for(j = mask_size/2; j < cols - mask_size/2; j++){
		     x = 0;
		     for(I = i - mask_size/2; I <= i + mask_size/2; I++)
			 for(J = j - mask_size/2; J <= j + mask_size/2;J++)
				AR[x++] = bP[I][J];
		     for(I = 0; I < area; I++){
			 for(J = I; J < area; J++){
			     if(AR[I] > AR[J]){
				  A = AR[J];
				  AR[J] = AR[I];
				  AR[I] = A;
			     }
			 }
		     }
		     sum = 0;
		     for(I = p; I < area-p; I++)
			 sum += AR[I];
		     sum /= (area - 2*p);
		     bP1[i][j] = sum;
		}
	    }
	}		    
   }
   delete_Image(imageP);
   return imageW;
}   
      
Image* contra_setup(Image *cvipImage) { 
   int mask_size, p; 
   printf(" ***********  THE CONTRA - HARMONIC MEAN FILTER  **********\n");    
   do{ 
	printf("\nType in an integer value [3,5,7,9 or 11] for mask size:"); 
	scanf("%d",&mask_size); 
	if(mask_size == 3) break; 
	if(mask_size == 5) break; 
	if(mask_size == 7) break; 
	if(mask_size == 9) break; 
	if(mask_size == 11) break; 
	else printf("\nInvalid Selection !!!!!  Try Again !!!!!\n"); 
    }while(1); 
 
    do{   
  	fflush(stdin);
	printf("\nType in an integer value [(-5) to (+5)] for the desired filter order:");   
	scanf("%d",&p);   
	if(abs(p) == 1) break;   
	if(abs(p) == 2) break;   
	if(abs(p) == 3) break;  	
	if(abs(p) == 4) break;   
	if(abs(p) == 5) break; 
	else printf("\nInvalid Selection !!!!!  Try Again !!!!!\n");   
    }while(1);   

    return   contra_filter(cvipImage,mask_size,p); 
} 
 
Image * contra_filter(Image *imageW, int mask_size, int p) {
   Image *imageP;
   byte   **bP,**bP1;   
   int    no_of_bands,i, j,J, I,x,area;           
   unsigned bands, rows, cols; 
   float  **fP,**fP1;
   float  AR[121],sum,sum1,A;

   bands = getNoOfBands_Image(imageW);   
   rows =  getNoOfRows_Image(imageW);   
   cols =  getNoOfCols_Image(imageW);   
   area = mask_size * mask_size;

   if(imageW->bandP[0]->data_type != CVIP_BYTE ){
	if(imageW->bandP[0]->data_type != CVIP_FLOAT && cast_Image(imageW,CVIP_FLOAT)){
	    fprintf(stderr,"Image is cast into float \n");
	}
	else if(imageW->bandP[0]->data_type != CVIP_FLOAT ){
	    fprintf(stderr,"cast_Image failed..\n");
		 delete_Image(imageW);
	    return NULL;
	}
	imageP = duplicate_Image(imageW);
     	for(no_of_bands=0; no_of_bands < bands; no_of_bands++){   
	    fP = getData_Image(imageP,no_of_bands);
	    fP1 = getData_Image(imageW,no_of_bands);
	    for(i = mask_size/2; i < rows - mask_size/2; i++){
		for(j = mask_size/2; j < cols - mask_size/2; j++){
		     x = 0;
		     sum = sum1 = 0.0;
		     for(I = i - mask_size/2; I <= i + mask_size/2; I++)
			 for(J = j - mask_size/2; J <= j + mask_size/2;J++)
				AR[x++] = fP[I][J];
		     for(I = 0; I < area; I++){
			 sum += pow((double)AR[I],(double)(p+1));
			 sum1 +=pow((double)AR[I],(double)p);
		     }
		     if(sum1)
		         fP1[i][j] = sum / sum1;
		     else
			 fP1[i][j] = 0.0;
		}
	    }
	}	 
   }
   else{
	imageP = duplicate_Image(imageW);
     	for(no_of_bands=0; no_of_bands < bands; no_of_bands++){   
	    bP = getData_Image(imageP,no_of_bands);
	    bP1 = getData_Image(imageW,no_of_bands);
	    for(i = mask_size/2; i < rows - mask_size/2; i++){
		for(j = mask_size/2; j < cols - mask_size/2; j++){
		     sum1=sum=0.0;
		     x = 0;
		     for(I = i - mask_size/2; I <= i + mask_size/2; I++)
			 for(J = j - mask_size/2; J <= j + mask_size/2;J++)
				AR[x++] = (float)(bP[I][J]);
		     for(I = 0; I < area; I++){
			 sum += pow((double)AR[I],(double)(p+1));
			 sum1 +=pow((double)AR[I],(double)p);
		     }
		     if(sum1)
		         bP1[i][j] = (byte) (sum / sum1);
		     else
			 bP1[i][j] = 0;
		}
	    }
	}		    
   }
   delete_Image(imageP);
   return imageW;
}               

             

Image* geometric_setup(Image *cvipImage)                   
  {                   
     int mask_size;                   
                    
 printf(" **********  THE GEOMETRIC MEAN FILTER  **********\n");                            
                   
do{                   
	printf("\nType in an integer value [3,5,7,9 or 11] for mask size:");                   
	scanf("%d",&mask_size);                   
                   
if(mask_size == 3) break;                   
if(mask_size == 5) break;                   
if(mask_size == 7) break;                   
if(mask_size == 9) break;                   
if(mask_size == 11) break;                   
else printf("\nInvalid Selection !!!!!  Try Again !!!!!\n");                   
}while(1);                   
                   
return geometric_filter(cvipImage,mask_size);                   
   }                   
                   
Image* geometric_filter(Image *imageW, int mask_size)                   
       {    
	Image *imageP;               
	byte   *bP, *bP1;                   
	int    i, j, k, J, I, A;                   
	int mask[121], Z, N, index, x;                   
	float PRODUCT, TAR[121];    
	float *fP, *fP1;
	float fmask[121],fZ;                  
	unsigned bands, rows, cols;                   
     
	bands = getNoOfBands_Image(imageW);                   
	rows =  getNoOfRows_Image(imageW);                   
	cols =  getNoOfCols_Image(imageW);                   
                   
	N=mask_size;                   
       if(imageW->bandP[0]->data_type != CVIP_BYTE){
	   if(imageW->bandP[0]->data_type != CVIP_FLOAT && cast_Image(imageW,CVIP_FLOAT)){
		fprintf(stderr,"Image is cast into float \n");
	   }
	   else if(imageW->bandP[0]->data_type != CVIP_FLOAT ){
		fprintf(stderr,"cast_Image failed..\n");
		delete_Image(imageW);
		return NULL;
	   }
	imageP = duplicate_Image(imageW);
	for(i=0; i < bands; i++)                   
        {                   
         for(j=N/2; j < (rows-N/2); j++,fP++)                   
	 {                   
          fP=(float*)getRow_Image(imageW,j,i); 

             for(k=N/2; k < (cols-N/2); k++, fP1++)       
	   {       
       
	    x= 0;
            for(J=-N/2; J<= N/2; J++) 
             { 
               fP1 = (float *) getRow_Image(imageP, J+j, i);         
		            
	      for(I=-N/2; I<= N/2; I++, x++)       
                {              
	         fmask[x] = *(fP1 + k +I);      
	         }       
	      }                           

	         for(J=0; J <= N*N-1; J++)                   
		  {                   
		    if(fmask[J] <= 0)                   
		      {                   
		        fmask[J] = 1;                   
	               }                   
		        TAR[J] = pow((double)fmask[J],  
                              (double)(1.0/(float)(N*N)));                 			                 				                             				 
                   }

                     PRODUCT = 1.0;                   			              
		    for(J=0; J <= N*N-1; J++)                   
		      PRODUCT *= TAR[J];                   
			                        
                      *(fP + k ) = PRODUCT;                   
                   
             }                   
        }                   
    }
	}
	else{
           
        imageP = duplicate_Image(imageW);          
	for(i=0; i < bands; i++)                   
        {                   
         for(j=N/2; j < (rows-N/2); j++,bP++)                   
	 {                   
          bP=(byte*)getRow_Image(imageW,j,i); 

             for(k=N/2; k < (cols-N/2); k++, bP1++)       
	   {       
       
	    x= 0;
            for(J=-N/2; J<= N/2; J++) 
             { 
               bP1 = (byte *) getRow_Image(imageP, J+j, i);         
		            
	      for(I=-N/2; I<= N/2; I++, x++)       
                {              
	         mask[x] = *(bP1 + k +I);      
	         }       
	      }                           

	         for(J=0; J <= N*N-1; J++)                   
		  {                   
		    if(mask[J] <= 0)                   
		      {                   
		        mask[J] = 1;                   
	               }                   
		        TAR[J] = pow((double)mask[J],  
                              (double)(1.0/(float)(N*N)));                 			                 				                             				 
                   }

                     PRODUCT = 1.0;                   			              
		    for(J=0; J <= N*N-1; J++)                   
		      PRODUCT *= TAR[J];                   
			                        
		      if(PRODUCT > 255)                   
		         *(bP+k) = 255;                   
		           else                   
				   
                    	    *(bP + k ) = (byte) (unsigned char)PRODUCT;                   
                   
             }                   
        }                   
    }         
    }          
    delete_Image(imageP);
    return imageW;
}                   

Image * harmonic_setup(Image *cvipImage){ 
   int mask_size; 
   printf(" ***********  THE HARMONIC MEAN FILTER  **********\n");          
   do{ 
	printf("\nType in an integer value [3,5,7,9 or 11] for mask size:"); 
	scanf("%d",&mask_size);  
	if(mask_size == 3) break; 
	if(mask_size == 5) break; 
	if(mask_size == 7) break; 
	if(mask_size == 9) break; 
	if(mask_size == 11) break; 
	else printf("\nInvalid Selection !!!!!  Try Again !!!!!\n"); 
   }while(1);  
   return harmonic_filter(cvipImage,mask_size); 
} 
 
Image * harmonic_filter(Image *imageW, int mask_size){
   Image *imageP;
   byte   **bP,**bP1;   
   int    no_of_bands,i, j,J, I,x,area,count;           
   unsigned bands, rows, cols; 
   float  **fP,**fP1;
   float  AR[121],sum,A;

   bands = getNoOfBands_Image(imageW);   
   rows =  getNoOfRows_Image(imageW);   
   cols =  getNoOfCols_Image(imageW);   
   area = mask_size * mask_size;

   if(imageW->bandP[0]->data_type != CVIP_BYTE ){
	if(imageW->bandP[0]->data_type != CVIP_FLOAT && cast_Image(imageW,CVIP_FLOAT)){
	    fprintf(stderr,"Image is cast into float \n");
	}
	else if(imageW->bandP[0]->data_type != CVIP_FLOAT ){
	    fprintf(stderr,"cast_Image failed..\n");
		 delete_Image(imageW);
	    return NULL;
	}
	imageP = duplicate_Image(imageW);
     	for(no_of_bands=0; no_of_bands < bands; no_of_bands++){   
	    fP = getData_Image(imageP,no_of_bands);
	    fP1 = getData_Image(imageW,no_of_bands);
	    for(i = mask_size/2; i < rows - mask_size/2; i++){
		for(j = mask_size/2; j < cols - mask_size/2; j++){
		     x = 0;
		     sum = 0.0;
	 	     count = area;
		     for(I = i - mask_size/2; I <= i + mask_size/2; I++){
			 for(J = j - mask_size/2; J <= j + mask_size/2;J++){
				if(fP[I][J])
				    sum += 1.0 / fP[I][J];
				else
				    count--;
			 }
		     }
		     if(sum)
		         fP1[i][j] = (float) count / sum;
		     else
			 fP1[i][j] = 0.0;
		}
	    }
	}	 
   }
   else{
	imageP = duplicate_Image(imageW);
     	for(no_of_bands=0; no_of_bands < bands; no_of_bands++){   
	    bP = getData_Image(imageP,no_of_bands);
	    bP1 = getData_Image(imageW,no_of_bands);
	    for(i = mask_size/2; i < rows - mask_size/2; i++){
		for(j = mask_size/2; j < cols - mask_size/2; j++){
		     x = 0;
		     sum = 0.0;
	 	     count = area;
		     for(I = i - mask_size/2; I <= i + mask_size/2; I++){
			 for(J = j - mask_size/2; J <= j + mask_size/2;J++){
				if(bP[I][J])
				    sum += 1.0 / (float)(bP[I][J]);
				else
				    count--;
			 }
		     }
		     if(sum)
		         bP1[i][j] = (byte) (count / sum);
		     else
			 bP1[i][j] = 0;
		}
	    }
	}		    
   }
   delete_Image(imageP);
   return imageW;
}                              

Image* Ypmean_setup(Image *cvipImage){ 
   int mask_size, p; 
   printf(" **********  THE Yp MEAN FILTER  **********\n");          
   do{ 
	printf("\nType in an integer value [3,5,7,9 or 11] for mask size:"); 
	scanf("%d",&mask_size);  
	if(mask_size == 3) break;  
	if(mask_size == 5) break; 
	if(mask_size == 7) break; 
	if(mask_size == 9) break; 
	if(mask_size == 11) break; 
	else printf("\nInvalid Selection !!!!!  Try Again !!!!!\n");  
   }while(1); 
 
   do{ 
	printf("\nType in an integer value from [-5 0) to (0 5] for the desired filter order:"); 
	scanf("%d",&p); 
	if(p >= -5 && p <= 5 && p != 0) break; 
	else printf("\nInvalid Selection !!!!!  Try Again !!!!!\n"); 
   }while(1); 
    
   return Ypmean_filter(cvipImage,mask_size,p); 
} 
 
Image * Ypmean_filter(Image *imageW, int mask_size, int p){ 
   Image *imageP;
   byte   **bP,**bP1;   
   int    no_of_bands,i, j,J, I,x,area;           
   unsigned bands, rows, cols; 
   float  **fP,**fP1;
   float  AR[121],sum,A;

   bands = getNoOfBands_Image(imageW);   
   rows =  getNoOfRows_Image(imageW);   
   cols =  getNoOfCols_Image(imageW);   
   area = mask_size * mask_size;

   if(imageW->bandP[0]->data_type != CVIP_BYTE ){
	if(imageW->bandP[0]->data_type != CVIP_FLOAT && cast_Image(imageW,CVIP_FLOAT)){
	    fprintf(stderr,"Image is cast into float \n");
	}
	else if(imageW->bandP[0]->data_type != CVIP_FLOAT ){
	    fprintf(stderr,"cast_Image failed..\n");
		 delete_Image(imageW);
	    return NULL;
	}
	imageP = duplicate_Image(imageW);
     	for(no_of_bands=0; no_of_bands < bands; no_of_bands++){   
	    fP = getData_Image(imageP,no_of_bands);
	    fP1 = getData_Image(imageW,no_of_bands);
	    for(i = mask_size/2; i < rows - mask_size/2; i++){
		for(j = mask_size/2; j < cols - mask_size/2; j++){
		     x = 0;
		     sum = 0.0;
		     for(I = i - mask_size/2; I <= i + mask_size/2; I++){
			 for(J = j - mask_size/2; J <= j + mask_size/2;J++){
				sum +=  pow((double)fP[I][J],(double)p);
			 }
		     }
		     sum /= area;
		     sum = abs(sum);
		     fP1[i][j] = pow((double)sum,(double)1/p);
		}
	    }
	}	 
   }
   else{
	imageP = duplicate_Image(imageW);
     	for(no_of_bands=0; no_of_bands < bands; no_of_bands++){   
	    bP = getData_Image(imageP,no_of_bands);
	    bP1 = getData_Image(imageW,no_of_bands);
	    for(i = mask_size/2; i < rows - mask_size/2; i++){
		for(j = mask_size/2; j < cols - mask_size/2; j++){
		     x = 0;
		     sum = 0.0;
		     for(I = i - mask_size/2; I <= i + mask_size/2; I++){
			 for(J = j - mask_size/2; J <= j + mask_size/2;J++){
				sum +=  pow((double)bP[I][J],(double)p);
			 }
		     }
		     sum /= area;
		     sum = (sum < 0)? -sum : sum;
		     bP1[i][j] = pow((double)sum,(double)1/p);
		}
	    }
	}		    
   }
   delete_Image(imageP);
   return imageW;
}  


Image* midpoint_setup(Image *cvipImage)       
  {       
     int mask_size;       
       
printf(" ************  THE MIDPOINT FILTER  **********\n");              
       
do{       
	printf("\nType in an integer value [3,5,7,9 or 11] for mask size:");       
	scanf("%d",&mask_size);       
       
if(mask_size == 3) break;       
if(mask_size == 5) break;       
if(mask_size == 7) break;       
if(mask_size == 9) break;       
if(mask_size == 11) break;       
else printf("\nInvalid Selection !!!!!  Try Again !!!!!\n");       
}while(1);       
       
return midpoint_filter(cvipImage,mask_size);       
   }       
       
Image *midpoint_filter(Image *imageW, int mask_size)       
       {       
	Image  *imageP;
        byte   *bP1, *bP2;       
	int    i, j, k, J, I, x, smin, smax, N;       
	unsigned bands, rows, cols;       
        int Pixel[121];
	float *fP1, *fP2;
	float fPixel[121];
	float fsmin, fsmax;

	bands = getNoOfBands_Image(imageW);       
	rows =  getNoOfRows_Image(imageW);       
	cols =  getNoOfCols_Image(imageW);       

  	N=mask_size;       
       if(imageW->bandP[0]->data_type != CVIP_BYTE ){
	   if(imageW->bandP[0]->data_type != CVIP_FLOAT && cast_Image(imageW,CVIP_FLOAT)){
		fprintf(stderr,"Image is cast into float \n");
	   }
	   else if(imageW->bandP[0]->data_type != CVIP_FLOAT ){
		fprintf(stderr,"cast_Image failed..\n");
		delete_Image(imageW);
		return NULL;
	   }
	imageP = duplicate_Image(imageW);
	for(i=0; i < bands; i++)       
	  {       
		for(j=N/2; j < (rows-N/2); j++,bP1++)       
	          {        		  
                     fP1 = (float *) getRow_Image(imageW, j, i);
                        for(k=N/2; k < (cols-N/2); k++,bP2++)       
			{       
       
			 fsmin=FLT_MAX; fsmax=FLT_MIN; x= 0;
                          for(J=-N/2; J<= N/2; J++) 
                           {
                            fP2 = (float *) getRow_Image(imageP, J+j, i);         		            
			 for(I=-N/2; I<= N/2;I++, x++)       
			  {              
			    fPixel[x] = *(fP2 + k +I);      
			   }       
                         }


			 for(J=0; J < N; J++)       
			    {       
			     if(fPixel[J] < fsmin)       
				 fsmin = fPixel[J];       
				       
			     }       
			  for(I=0; I < N; I++)       
			   {       
		            if(fPixel[I] > fsmax)       
				 fsmax = fPixel[I];       
				       
			     }       
       
			 *(fP1 + k) =((fsmin+fsmax)/2+0.5);       
                 }
       }       
      }
	}
	/*cast_Image(imageW,CVIP_BYTE);*/
	else{
	imageP = duplicate_Image(imageW);
	for(i=0; i < bands; i++)       
	  {       
		for(j=N/2; j < (rows-N/2); j++,bP1++)       
	          {        		  
                     bP1 = (byte *) getRow_Image(imageW, j, i);
                        for(k=N/2; k < (cols-N/2); k++,bP2++)       
			{       
       
			 smin=255, smax=0, x= 0;
                          for(J=-N/2; J<= N/2; J++) 
                           {
                            bP2 = (byte *) getRow_Image(imageP, J+j, i);         		            
			 for(I=-N/2; I<= N/2;I++, x++)       
			  {              
			    Pixel[x] = *(bP2 + k +I);      
			   }       
                         }


			 for(J=0; J < N; J++)       
			    {       
			     if(Pixel[J] < smin)       
				 smin = Pixel[J];       
				       
			     }       
			  for(I=0; I < N; I++)       
			   {       
		            if(Pixel[I] > smax)       
				 smax = Pixel[I];       
				       
			     }       
       
			 *(bP1 + k) = (byte) ((smin+smax)/2+0.5);       
                 }
       }       
      } 
   }    
   delete_Image(imageP);
   return imageW;
}                                

Image* maximum_setup(Image *cvipImage)   
  {   
     int mask_size;   
   
printf("   ************  THE MAXIMUM FILTER  **********\n");          
   
do{   
	printf("\nType in an integer value [3,5,7,9 or 11] for mask size:");   
	scanf("%d",&mask_size);   
   
if(mask_size == 3) break;   
if(mask_size == 5) break;   
if(mask_size == 7) break;   
if(mask_size == 9) break;   
if(mask_size == 11) break;   
else printf("\nInvalid Selection !!!!!  Try Again !!!!!\n");   
}while(1);   
   
   return maximum_filter(cvipImage,mask_size);   
   }   
   
Image *maximum_filter(Image *imageW, int mask_size)   
       {   
	Image *imageP;
        byte   *bP1, *bP;   
	int    i, j, k;   
        int J, I, smax, max[121],M,a,N;   
	float  *fP1, *fP;
	float fsmax,fmax[121];   
	unsigned bands, rows, cols;   
   
	bands = getNoOfBands_Image(imageW);   
	rows =  getNoOfRows_Image(imageW);   
	cols =  getNoOfCols_Image(imageW);    
	N = mask_size;
       if(imageW->bandP[0]->data_type != CVIP_BYTE ){
	   if(imageW->bandP[0]->data_type != CVIP_FLOAT && cast_Image(imageW,CVIP_FLOAT)){
		fprintf(stderr,"Image is cast into float \n");
	   }
	   else if(imageW->bandP[0]->data_type != CVIP_FLOAT ){
		fprintf(stderr,"cast_Image failed..\n");
		delete_Image(imageW);
		return NULL;
	   }
	   imageP = duplicate_Image(imageW);
            for(i=0; i < bands; i++)   
		{   
		for(j=N/2; j < (rows-N/2); j++,bP++)   
			{   
		     
                         fP=(float*)getRow_Image(imageW,j,i); 

                        for(k=N/2; k < cols-N/2; k++,bP1++)
				{
			        M=0;
                                fsmax=FLT_MIN;
				for(J=-N/2;J<=N/2;J++)
  			         {
                                   					                                                                                                                                                                                                         
                                     fP1=(float*)getRow_Image(imageP,j+J,i);
						               						  
                                    for(a=-N/2;a<=N/2;a++,M++)
                                    { fmax[M]=*(fP1+k+a);}
                                        
                                 }    

				for(J=0;J<N;J++)
                                  {
                                   if(fmax[J] > fsmax)
                                    fsmax = fmax[J];
	                           }                                  					                                                                                        				                 
                           *(fP+k) = fsmax;

}
}
}

	}
	else{
	   imageP = duplicate_Image(imageW);
            for(i=0; i < bands; i++)   
		{   
		for(j=N/2; j < (rows-N/2); j++,bP++)   
			{   
		     
                         bP=(byte*)getRow_Image(imageW,j,i); 

                        for(k=N/2; k < cols-N/2; k++,bP1++)
				{
			        M=0;
                                smax=0;
				for(J=-N/2;J<=N/2;J++)
  			         {
                                   					                                                                                                                                                                                                         
                                     bP1=(byte*)getRow_Image(imageP,j+J,i);
						               						  
                                    for(a=-N/2;a<=N/2;a++,M++)
                                    { max[M]=*(bP1+k+a);}
                                        
                                 }    

				for(J=0;J<N;J++)
                                  {
                                   if(max[J] > smax)
                                    smax = max[J];
	                           }                                  					                                                                                        				                 
                           *(bP+k) = (byte)smax;

}
}
}
}
delete_Image(imageP);
return imageW;
}				   
	
