/***************************************************************************
* ======================================================================
* Computer Vision/Image Processing Tool Project - Dr. Scott Umbaugh SIUE
* ======================================================================
*
*             File Name: tt.c
*           Description: test program for functions contained in the CVIPtools
*			 core toolbox -- libtransform.a
*         Related Files: Imakefile, libtransform.a
*   Initial Coding Date: Tue Jul  6 08:34:13 CDT 1993
*           Portability: Standard (ANSI) C
*             Credit(s): Gregory Hance
*                        Southern Illinois University @ Edwardsville
*
** Copyright (C) 1993 SIUE - by Gregory Hance.
**
** Permission to use, copy, modify, and distribute this software and its
** documentation for any purpose and without fee is hereby granted, provided
** that the above copyright notice appear in all copies and that both that
** copyright notice and this permission notice appear in supporting
** documentation.  This software is provided "as is" without express or
** implied warranty.
**
* RCS (Revision Control System) Information
* ...Added automatically by RCS 
*
* $Log: tt.c,v $
* Revision 1.6  1997/04/14 23:42:17  yawei
* removed the second parameter in read_Image
*
* Revision 1.5  1997/03/08 17:05:19  yawei
* Swaped CVIP_YES and CVIP_NO
*
* Revision 1.4  1997/03/08 00:57:38  yawei
* Name Changes:
* 	BOOLEAN ==> CVIP_BOOLEAN
* 	FALSE ==> CVIP_NO
* 	TRUE ==> CVIP_YES
* 	BYTE ==> CVIP_BYTE
* 	SHORT ==> CVIP_SHORT
* 	INTEGER ==> CVIP_INTEGER
* 	FLOAT ==> CVIP_FLOAT
* 	DOUBLE ==> CVIP_DOUBLE
* 	TYPE ==> CVIP_TYPE
*
* Revision 1.3  1997/01/16 21:20:42  yawei
* Global Change:  Image_FORMAT ==> IMAGE_FORMAT
*
 * Revision 1.2  1997/01/15  17:16:31  yawei
 * Global Change:  IMAGE ==> Image
 *
 * Revision 1.1  1994/11/15  19:26:03  luok
 * Initial revision
 *
 * Revision 1.1  1994/10/30  17:58:45  hanceg
 * CVIPtools1.0 source file - G. Hance
 *
*
*
****************************************************************************/

/* Define Preprocessor statements */
#include "CVIPtoolkit.h"
#include "dct.h"

main(argc,argv)
int argc;
char *argv[];
{
/* Declare variables local to main() */
Image *cvipImage;
IMAGE_FORMAT format;
char inputfile[50], outputfile[50];
const char *usage = "[-display] [infile] [outfile]", *fn = argv[0];
int argn=1;
CVIP_BOOLEAN display = CVIP_NO, remap;

cvip_init( &argc, argv );

while ( argn < argc && argv[argn][0] == '-' && argv[argn][1] != '\0' ) {
   if ( cvip_keymatch( argv[argn], "-display", 2 ) )
      display = CVIP_YES;
   else
      cvip_usage( fn, usage );
   argn++;
}

if(argc > argn) {
   if ( (argv[argn][0] == '-' && argv[argn][1] != '\0') || sscanf( argv[argn], "%s", inputfile ) != 1 )
      cvip_usage( fn, usage );
   argn++;
   if ( argc > argn ) {
      if ( (argv[argn][0] == '-' && argv[argn][1] != '\0') || sscanf( argv[argn], "%s", outputfile ) != 1 )
         cvip_usage( fn, usage );
      argn++;
   }
   else
      (void) strcpy(outputfile, "-");
   if( argc != argn )
      cvip_usage( fn, usage );
}
else {
   (void) strcpy(inputfile, "-");
   (void) strcpy(outputfile, "-");
}

format = getFormat_CVIP(inputfile);
cvipImage = read_Image(inputfile, 1);

cvipImage = dct_setup( cvipImage, &remap );
cvipImage = dct_setup( cvipImage, &remap );

cvipImage->bandP = bandcast( cvipImage->bandP, cvipImage->bands, CVIP_BYTE);

write_Image(cvipImage, outputfile, CVIP_NO, CVIP_NO, format, 1);

if( display && strcmp(outputfile,"-") )
   cvip_display( outputfile, format );
else if( display )
   cvip_error(fn, "can not display an image directed to \"stdout\".");

image_free(cvipImage);
exit(0);
} /* End Main */

     
         





   
      
