/***************************************************************************
* ======================================================================
* Computer Vision/Image Processing Tool Project - Dr. Scott Umbaugh SIUE
* ======================================================================
*
*             File Name: tvector.c
*           Description: Test program for functions in libvector.a.
*         Related Files: Imakefile, libvector.a
*   Initial Coding Date: Sun Aug 29 18:54:06 CDT 1993
*           Portability: Standard (ANSI) C
*             Credit(s): Gregory Hance
*                        Southern Illinois University @ Edwardsville
*
* RCS (Revision Control System) Information
* ...Added automatically by RCS 
*
** $Log: tvector.c,v $
** Revision 1.5  1997/03/08 17:07:35  yawei
** Swaped CVIP_YES and CVIP_NO
**
** Revision 1.4  1997/03/08 01:02:29  yawei
** Name Changes:
** 	BOOLEAN ==> CVIP_BOOLEAN
** 	FALSE ==> CVIP_YES
** 	TRUE ==> CVIP_NO
** 	BYTE ==> CVIP_BYTE
** 	SHORT ==> CVIP_SHORT
** 	INTEGER ==> CVIP_INTEGER
** 	FLOAT ==> CVIP_FLOAT
** 	DOUBLE ==> CVIP_DOUBLE
** 	TYPE ==> CVIP_TYPE
**
** Revision 1.3  1997/01/16 21:35:31  yawei
** Global Change:  Image_FORMAT ==> IMAGE_FORMAT
**
 * Revision 1.2  1997/01/15  17:20:26  yawei
 * Global Change:  IMAGE ==> Image
 *
 * Revision 1.1  1994/10/30  17:59:03  hanceg
 * CVIPtools1.0 source file - G. Hance
 *
*
****************************************************************************/
/* Define Preprocessor statements */
#include "CVIPtoolkit.h"
#include "CVIPconvert.h"
#include "CVIPdef.h"
#include "CVIPvector.h"

main(int argc, char *argv[])
{
/* Declare variables local to main() */
Image *cvipImage;
IMAGE_FORMAT format;
char inputfile[50], outputfile[50];
const char *usage = "[-display] [infile] [outfile]", *fn = argv[0];
int argn=1, rows, cols;
register int i;
unsigned length;
CVIP_BOOLEAN display = CVIP_NO, sub = CVIP_NO, absolute = CVIP_NO;
float a[5] = {0.5,-1,9.3,9,-2};
float test[10] = {0.5,1,9.3,9,2,0.69,7,8.0,2.9,.4}, *result;
byte *bP;
int ihisto[256];
double dhisto[256];
double filter[5] = {0.2,0.2,0.2,0.2,0.2};
double deriv[2] = {-1.0, 1.0};


/* initialize global command options */
cvip_init( &argc, argv );


/* process command line options */
while ( argn < argc && argv[argn][0] == '-' && argv[argn][1] != '\0' ) {
   if ( cvip_keymatch( argv[argn], "-display", 2 ) )
      display = CVIP_YES;
   else if ( cvip_keymatch( argv[argn], "-absolute", 3 ) )
      absolute = CVIP_YES;
   else if ( cvip_keymatch( argv[argn], "-sub", 2 ) )
      sub = CVIP_YES;
   else
      cvip_usage( fn, usage );
   argn++;
}

/* decide where to read input from and where to send output */
if(argc > argn) {
   if ( (argv[argn][0] == '-' && argv[argn][1] != '\0') || sscanf( argv[argn], "%s", inputfile ) != 1 )
      cvip_usage( fn, usage );
   argn++;
   if ( argc > argn ) {
      if ( (argv[argn][0] == '-' && argv[argn][1] != '\0') || sscanf( argv[argn], "%s", outputfile ) != 1 )
         cvip_usage( fn, usage );
      argn++;
   }
   else
      (void) strcpy(outputfile, "-");
   if( argc != argn )
      cvip_usage( fn, usage );
}
else {
   (void) strcpy(inputfile, "-");
   (void) strcpy(outputfile, "-");
}

if(sub) {
	result = subSample_Vector(test, 10, CVIP_FLOAT, 2, &length);
	for(i=0; i < length; i++)
		fprintf(stderr,"result[%d] = %.3f\n", i, result[i]);
	exit(0);
}

if(absolute) {
	(void) printVector(a, CVIP_FLOAT, 5, stderr);
	(void) absVector(a, CVIP_FLOAT, 5);
	(void) printVector(a, CVIP_FLOAT, 5, stderr);

	exit(0);
}


format = CVIPgetformat(inputfile);
cvipImage = CVIPreadimage(inputfile, format, 1);

/*
xprism2 -np1 3 -xsmin 0.0 -xsmax 255.0 -xsint 10 -xtics 3 -legend 0 -i11 histo.txt -i12 histo.txt -i13 histo.txt
fprintf(stderr,"min = %3.1f\n", findMinVal_Vector(a, 5, CVIP_FLOAT));
fprintf(stderr,"max = %3.1f\n", findMaxVal_Vector(a, 5, CVIP_FLOAT));
*/
getBandVector_Image(cvipImage, &bP);
rows = getNoOfRows_Image(cvipImage);
cols = getNoOfCols_Image(cvipImage);
if(!findHisto_Vector(bP, rows*cols, dhisto, 0, 255, CVIP_DOUBLE)) {
	error_CVIP("main", "could not find histogram!\n");
	exit(1);
}
#if 0
convolve_Vector(dhisto, filter, 256, 5);
convolve_Vector(dhisto, deriv, 256, 2);
#endif
printHisto_Vector("histo.txt", dhisto, CVIP_DOUBLE, 0, 255, 0);
/*
for(i=0;i<256;i++)
	fprintf(stderr,"%d %g\n",i, ihisto[i]);
*/

CVIPwriteimage(cvipImage, outputfile, CVIP_NO, CVIP_NO, format, 1);

if( display && strcmp(outputfile,"-") )
   cvip_display( outputfile, format );
else if( display )
   cvip_error(fn, "can not display an image directed to \"stdout\".");

image_free(cvipImage);
exit(0);
} /* End Main */

     
         





   
      
