/*
 * create_blur.c - Create blur mask.
 *
 * Description: This file contains routines that create images suitable
 * for use as blur masks.  These images can be categorized as follows
 * (not all categories are implemented initially):
 *
 * - Shape of the blur (int):
 *	1 = circular, 2 = rectangular, 3=elliptic
 * - Parameter #1 of blur function (int, cannot be zero)
 *      if shape is circle, its radius.
 *      if shape is rectangle, its height.
 * - Parameter #2 of blur function (int)
 *	if shape is circle, its arc (0-360)
 *	if shape is rectangle, its width.
 * - Parameter #3 of blur function (int)
 *	if shape is circle, the beginning angle of its arc (the rest of the
 *		arc wil be constructed by a counterclockwise sweep).
 *	if shape is rectangle, its angle.
 * - Parameter #4 of blur function - blur weighting type (int)
 *	1=constant blur, 2=select from set of fixed blur masks, 3=gaussian
 *		(center-weighted).
 * - Parameter #5 of blur function (float)
 *	If constant blur, this is 0.  
 *	If select from set of masks, this specifies which mask (int portion)
 * 	If gaussian, this specifies standard deviation.
 * - Parameter #6 of blur function (scaling factor). (float)
 *	All blur functions are normalized, then multiplied by this
 *	scaling factor.
 *
 * NOTE:  All Images are only as large as they need to be, with zeroes
 *	  filled in in order to make the image rectangular.  Other
 *	  functions should use zero-padding if the image needs extended.
 *	  This is done to facilitate the use of the same blur on multiple
 *	  images of different sizes, as well as to reduce the already
 *	  bloated number of parameters which must be passed to create_blur().
 */
 
Image *create_blur(int shape, int size1, int size2, int orient, int wtype, \
	float param, int scale) {
	Image *newImg;
}
