/***************************************************************************
* ======================================================================
* Computer Vision/ImageRec Processing Tool Project - Dr. Scott Umbaugh SIUE
* ======================================================================
*
*             File Name: cvipImg.c
*           Description: 
*         Related Files: 
*   Initial Coding Date: Tue Nov 28 21:50:13 CST 1995
*           Portability: Standard (ANSI) C
*            References:
*             Author(s): Kun Luo, Zhen Li
*                        Southern Illinois University @ Edwardsville
*
** Copyright (C) 1996 SIUE - by Kun Luo, Zhen Li, Scott Umbaugh
**
** Permission to use, copy, modify, and distribute this software and its
** documentation for any purpose and without fee is hereby granted, provided
** that the above copyright notice appear in all copies and that both that
** copyright notice and this permission notice appear in supporting
** documentation.  This software is provided "as is" without express or
** implied warranty.
**
* RCS (Revision Control System) Information
* ...Added automatically by RCS 
*
** $Log: cvipImg.c,v $
** Revision 1.199  1997/08/06 19:30:45  wzheng
** avoid the image type change in 6 edge_detector command
**
** Revision 1.198  1997/08/06 16:59:50  wzheng
** label error return for the too many objects
** snr and rms memory leak
**
** Revision 1.197  1997/07/28 19:03:10  wzheng
** add command for temp_filename
** modify the label command on error msg return
**
** Revision 1.196  1997/07/10 18:30:00  smakam
** prevented bombing when we take the inverse transform on the image in the spatial domain (exceot dct)
**
** Revision 1.195  1997/07/01 18:28:08  smakam
** included duplicate_Image for rms_error.
**
** Revision 1.194  1997/06/15 23:22:29  wzheng
** modified JPEG so the quantfile won't be got form the temp directory.
**
** Revision 1.193  1997/06/13 18:43:20  yawei
** duplicate Image before snr
**
** Revision 1.192  1997/06/13 04:39:32  wzheng
** add quantfile parameter for jpeg
**
** Revision 1.191  1997/06/02 20:01:03  wzheng
** add grayscale to jpeg command
**
** Revision 1.190  1997/05/30 03:49:57  yawei
** version changed to 1.6
**
** Revision 1.189  1997/05/29 22:28:23  yawei
** added display format selection to cvipImg.c
**
** Revision 1.188  1997/05/21 14:12:08  yawei
** added WIN32 binary fopen()
**
** Revision 1.187  1997/05/19 22:37:43  yawei
** added strcasecmp() macro, and related restriction on "viewer" selection for
** WIN32 flexibility
**
** Revision 1.186  1997/05/19 02:18:40  yawei
** added WIN32 support
**
** Revision 1.185  1997/05/15 00:16:45  yawei
** added another parameter for save.
**
** Revision 1.184  1997/05/03 00:19:22  yawei
** changed FFT
** 1) for inverse, if no FFT found, used image size to do ifft
**
** Revision 1.183  1997/04/29 15:29:21  yawei
** added JPEG support
**
** Revision 1.182  1997/04/26 00:26:19  yawei
** modified SNR message
**
** Revision 1.181  1997/04/26 00:20:05  yawei
** *** empty log message ***
**
** Revision 1.180  1997/04/21 16:38:26  akjoele
** Removed upper limit of <512 on blocksize for walsh, hadamard and haar.
**
** Revision 1.179  1997/04/18 20:05:52  yawei
** changed image format to VIP when saved as VIOP
**
** Revision 1.178  1997/04/18 19:39:53  yawei
** remapped to BYTE for ZON/ZON2
**
** Revision 1.177  1997/04/10 20:26:24  yawei
** switched from rows, cols to 512, 512
**
** Revision 1.176  1997/04/09 22:49:02  yawei
** changed 512 to the actual size
**
** Revision 1.175  1997/04/08 22:31:44  yawei
** no remap before rotate
**
** Revision 1.174  1997/03/29 01:25:47  yawei
** changed versin to 1.5
**
** Revision 1.173  1997/03/29 00:30:50  yawei
** checked output of fft
**
** Revision 1.172  1997/03/21 01:46:37  yawei
** Added copy_paste
**
** Revision 1.171  1997/03/18 15:30:44  yawei
** 1) fixed typo in FILTER_TYPEs.
** 2) fixed seg error in bpf_filter
**
** Revision 1.170  1997/03/15 01:28:25  yawei
** using FILTER_TYPE enumerated type.
**
** Revision 1.169  1997/03/15 00:57:41  yawei
** restored back
**
** Revision 1.168  1997/03/11 18:32:43  yawei
** Changed the "info" return message to "BYTE", instead of "CVIP_BYTE"
**
** Revision 1.167  1997/03/08 17:06:39  yawei
** Swaped CVIP_YES and CVIP_NO
**
** Revision 1.166  1997/03/08 02:34:22  yawei
** update TYPE changes
**
** Revision 1.165  1997/03/08 01:00:13  yawei
** Name Changes:
** 	BOOLEAN ==> CVIP_BOOLEAN
** 	FALSE ==> CVIP_YES
** 	TRUE ==> CVIP_NO
** 	BYTE ==> CVIP_BYTE
** 	SHORT ==> CVIP_SHORT
** 	INTEGER ==> CVIP_INTEGER
** 	FLOAT ==> CVIP_FLOAT
** 	DOUBLE ==> CVIP_DOUBLE
** 	TYPE ==> CVIP_TYPE
**
** Revision 1.164  1997/01/17 22:07:54  yawei
** Changed version 1.3 ==> 1.4
**
 * Revision 1.163  1997/01/17  20:58:52  yawei
 * power_spect_equal ==> power_spect_eq
 *
 * Revision 1.162  1997/01/16  21:29:35  yawei
 * Global Change:  Image_FORMAT ==> IMAGE_FORMAT
 *
 * Revision 1.161  1997/01/15  17:18:17  yawei
 * Global Change:  IMAGE ==> Image
 *
 * Revision 1.160  1997/01/15  16:35:37  yawei
 * updated changes
 *
 * Revision 1.159  1997/01/04  23:31:34  sumbaug
 * modified LIBSPATIALFLTR to LIBSPATIALFILTER
 *
 * Revision 1.158  1997/01/03  22:08:05  yawei
 * added pct_median_segment
 *
 * Revision 1.157  1997/01/03  19:48:13  yawei
 * function name changes
 *
 * Revision 1.156  1996/12/31  20:17:21  yawei
 * function name change
 *
 * Revision 1.155  1996/12/31  00:17:32  yawei
 * function name changes
 *
 * Revision 1.154  1996/12/30  17:35:52  yawei
 * wiener_filter ==> wiener
 *
 * Revision 1.153  1996/12/30  17:15:01  yawei
 * homomorphic_filter ==> homomorphic
 *
 * Revision 1.152  1996/12/30  15:46:57  yawei
 * corner_setup ==> nonfft_xformfilter
 *
 * Revision 1.151  1996/12/29  05:20:38  yawei
 * wht2d ==> walhad_transform
 *
 * Revision 1.150  1996/12/29  04:55:57  yawei
 * changed wavelet function names
 *
 * Revision 1.149  1996/12/28  06:25:51  yawei
 * blockwise_fft ==> fft_transform
 * inverse_fft   ==> ifft_transform
 *
 * Revision 1.148  1996/12/27  05:53:02  yawei
 * changed function names, get rid of unnecessary function prototype
 * declaration
 *
 * Revision 1.147  1996/12/24  00:46:57  yawei
 * *** empty log message ***
 *
 * Revision 1.146  1996/12/23  19:03:46  yawei
 * Changed IGS ==> igs_segment
 *
 * Revision 1.145  1996/12/23  16:25:15  yawei
 * Changed funcion name:
 *   SCTsplit_Image ==> sct_split_segment
 *
 * Revision 1.144  1996/12/23  15:33:45  yawei
 * changed split_merge
 *
 * Revision 1.143  1996/12/23  15:05:26  yawei
 * *** empty log message ***
 *
 * Revision 1.142  1996/12/22  17:31:59  akjoele
 * CVIPxformfilter was included twice, so the second include
 * statement was removed.
 *
 * Revision 1.141  1996/12/22  16:58:18  yawei
 * Changed function names
 *
 * Revision 1.140  1996/12/20  19:21:33  yawei
 * Use "view_Image" to display images.
 *
 * Revision 1.139  1996/12/18  02:14:09  sumbaug
 * chnaged usage note on parametric_wiener, from 'simple' to 'parametric'
 *
 * Revision 1.138  1996/12/17  22:52:48  yawei
 * Finished the clean up of the geometry library
 *
 * Revision 1.137  1996/12/17  00:08:46  yawei
 * fixed minor bug
 *
 * Revision 1.136  1996/12/16  23:47:38  yawei
 * 1) Changed version to 3.2
 * 2) Changed output of snr and rms to return back
 *
 * Revision 1.135  1996/12/16  23:39:23  yawei
 * Changed function names
 *
 * Revision 1.134  1996/12/16  22:04:09  akjoele
 * fixed small crop bug.
 *
 * Revision 1.133  1996/12/13  18:24:48  akjoele
 * changed crop_inner() to crop_Image() .
 *
 * Revision 1.132  1996/12/13  14:23:48  akjoele
 * Changed version number from 1.1 to 1.2
 *
 * Revision 1.131  1996/12/13  14:06:07  akjoele
 * Changed version number to 1.2 from 1.1
 *
 * Revision 1.130  1996/12/13  14:05:08  akjoele
 * Added rms_error and snr
 *
 * Revision 1.129  1996/12/09  19:20:32  akjoele
 * changed inverse_filter to inverse_xformfilter.
 * l;
 *
 * Revision 1.128  1996/12/06  15:44:05  akjoele
 * Eliminated 4-6 compiler warnings by straightening out function
 * prototypes and return types.
 *
 * Revision 1.127  1996/12/06  15:31:20  akjoele
 * Added power_spect_eq() and geometric_mean()
 *
 * Revision 1.126  1996/12/06  00:19:35  wli
 * add new function fft_phase() into cviptcl.
 *
 * Revision 1.125  1996/12/05  19:05:19  akjoele
 * Changed inverse filter call in accordance with changes in
 * CVIPxformfilter.h.  The inverse Filter C routine is now called
 * inverse_filter, and takes fewer variables.
 *
 * Revision 1.124  1996/12/05  03:51:39  wli
 * modify cxform() usage note.
 *
 * Revision 1.123  1996/12/05  03:12:41  wli
 * modify cxform, make it accomodate LUV, LAB, SCT and CCT
 * inverse transforms.
 *
 * Revision 1.122  1996/12/05  00:07:16  yawei
 * Changed the compression drivers, so temporary files won't be
 * overwritten
 *
 * Revision 1.121  1996/12/04  22:01:14  yawei
 * Changed zonal, and zonal2
 *
 * Revision 1.120  1996/12/04  21:51:00  yawei
 * changed zvl and zonal
 *
 * Revision 1.119  1996/12/04  04:02:18  yawei
 * Changed huffman compress function
 *
 * Revision 1.118  1996/12/03  23:48:31  yawei
 * changed the glr_ functions
 *
 * Revision 1.117  1996/11/30  18:04:10  yawei
 * changed DPCM to dpc, and the function names as well
 *
 * Revision 1.116  1996/11/30  16:26:20  yawei
 * Fixed btc3 bug
 *
 * Revision 1.115  1996/11/30  02:34:05  yawei
 * changed names for btc2, btc3, bug exists for btc3
 *
 * Revision 1.114  1996/11/27  19:37:47  yawei
 * Changed COMR_FORMAT to CMPR_FORMAT
 *
 * Revision 1.113  1996/11/27  16:08:18  akjoele
 * Changed all WIENER definitions to WIENER_FILTER, after
 * change of typedefs in CVIPtools.h
 *
 * Revision 1.112  1996/11/26  23:08:04  yawei
 * changed to hist_spec, but not fix the problem
 *
 * Revision 1.111  1996/11/26  17:28:27  akjoele
 * Changed name of simple_wiener_backend() to simple_wiener()
 *
 * Revision 1.110  1996/11/26  16:51:53  akjoele
 * Changed function names of least_squares to be consistent
 *
 * Revision 1.109  1996/11/23  22:12:31  sumbaug
 * added read,write, gray_binary to libconverter list.
 * added LIBBAND: extract_band and assembles_bands to list
 *
 * Revision 1.108  1996/11/22  03:35:17  wli
 * change the name of 'setUpLabel_Objects()' to be 'label()'.
 *
 * Revision 1.107  1996/11/21  17:37:08  akjoele
 * - Commented out hardcoding of laplacian image in least_squares
 * - Added wiener_filter()
 * - Added parametric_wiener()
 *
 * Revision 1.106  1996/11/20  18:45:36  yawei
 * Removed break to check for multiple compressed files
 *
 * Revision 1.105  1996/11/20  17:56:09  yawei
 * Changed the hard coded image extension of compressed images to a
 * global array.
 *
 * Revision 1.104  1996/11/20  03:40:11  wli
 * Move 'label' to libfeature and rm libobject.
 *
 * Revision 1.103  1996/11/13  04:51:25  wli
 * change IGS usage note.
 *
 * Revision 1.102  1996/11/12  02:37:13  wli
 * change "IGS" to be "igs" in cvip list.
 *
 * Revision 1.101  1996/11/08  05:09:19  wli
 * add IGS.c into cviptcl.
 *
 * Revision 1.100  1996/11/07  23:39:22  yawei
 * Changed "data_get" so it return large results in a more elegent way.
 *
 * Revision 1.99  1996/11/07  16:01:59  akjoele
 * Modified inverse_filter to duplicate all images before
 * calling filter function (to prevent trashing original images).
 *
 * Revision 1.98  1996/11/05  21:59:31  yawei
 * Changed "filter_mask" to "h_image"
 * changed "data_mask " to "data_h_image"
 *
 * Revision 1.97  1996/11/05  16:39:10  yawei
 * In "data_get", made the following changes:
 * 1) puts the result to interp->result, instead of stdout
 * 2) If the required space is larger than the preallocated space, try to
 *    enlarge it.
 * 3) The maximum allowable image size is 900 pixels.
 * 4) The interp->result pointer is altered.
 *
 * Revision 1.96  1996/11/05  04:17:46  yawei
 * changed "data_get" to puts the output to stdout
 *
 * Revision 1.95  1996/11/04  19:25:33  yawei
 * Changed "data_get" so it handle any data_types.
 *
 * Revision 1.94  1996/11/04  01:00:34  yawei
 * Added "data_mask" function
 *
 * Revision 1.93  1996/11/04  00:30:08  yawei
 * Added filter_mask and "data_get" functions.
 *
 * Revision 1.92  1996/10/30  13:57:37  akjoele
 * Bug fixes in least_squares()
 * Also added cvip zonal (wli), using zonal2.c
 *
 * Revision 1.91  1996/10/18  13:10:42  akjoele
 * Added least_squares; there are still bugs, but progress.  Also
 * eliminated 8-10 compiler warnings due to undefined functions
 * .  This happens when 1) function not defined in include file.
 *
 * Revision 1.90  1996/10/17  06:38:27  wli
 * add two more parameters to adaptive_contrast and modify usage note.
 *
 * Revision 1.89  1996/10/17  00:40:42  wli
 * remove 'paint_cell' off list(function is still there).
 *
 * Revision 1.88  1996/10/15  20:40:41  akjoele
 * changed least_squares_filter() function name to be shorter.
 * Still some debugging to do.
 *
 * Revision 1.87  1996/10/15  17:47:09  akjoele
 * Added definition for least_squares_filter.
 * Also removed at least a dozen unnecessary casts of function
 * call return types to (Image *).
 *
 * Revision 1.86  1996/10/14  18:21:40  akjoele
 * Removed some old-style function declarations, included header
 * files for dct,fft.  Modified inverse fft towards being able to
 * to an IFFT on a spatial-domain image.  this is still not complete.
 * Also documented fft.
 *
 * Revision 1.85  1996/10/11  19:38:32  yawei
 * Changed "datarange"
 * 1) Since it doesnm
 *    doesn't change the data, it won't copy the image form the queue,
 *    it only reads on the original image
 * 2) use getImgData_Matrix, and no need to create new image
 *
 * Revision 1.84  1996/10/11  18:15:46  akjoele
 * Modified inverse_filter, and added simple_wiener().  Debuigging
 * still needed, this check-in is so that others may have a chance to add functions.
 *
 * Revision 1.83  1996/10/10  16:30:16  yawei
 * added black image
 *
 * Revision 1.82  1996/10/09  02:21:09  yawei
 * commented out history_copy in the datarange.
 *
 * Revision 1.81  1996/10/08  19:57:28  akjoele
 * Added wiener filter.
 *
 * Revision 1.80  1996/10/07  01:47:26  yawei
 * changed the order in which "datarange" will appear in the "cvip" command output.
 *
 * Revision 1.79  1996/10/04  07:07:45  wli
 * add new functions adaptive_contrast()
 *  mmse_filter(), gray_binary() and zonal()
 * into cviptcl
 *
 * Revision 1.78  1996/09/20  01:52:58  yawei
 * added command "datarange", which will get the datarange of the image
 *
 * Revision 1.77  1996/09/17  21:13:04  akjoele
 * Added support for Hadamard transform filtering by changing the
 * same functions as were changed in Revision 1.72
 *
 * Revision 1.76  1996/09/07  22:51:08  sumbaug
 * corrected signon header (missing C, in CVIPTCL -> VIPTCL)
 *
 * Revision 1.75  1996/09/06  23:36:49  yawei
 * changed display function to write data in to temp dir in the
 * correct format.
 *
 * Revision 1.74  1996/09/04  22:22:28  yawei
 * added choice to viewer
 *
 * Revision 1.73  1996/09/04  20:06:00  yawei
 * added function "cvip viewer" to let the user to choose the viewer.
 * The viewer can be "picture" or "RamViewer"
 *
 * Revision 1.72  1996/08/26  13:23:23  akjoele
 * Added support for Haar transform filtering by changing the following functions:
 * S_ImageHistory(), check_xform_history(), S_ImageHPF_Filter(), S_ImageLPF_Filter(),
 * S_ImageBPF_Filter(), S_ImageBRF_Filter, S_ImageHFE_Filter().
 *
 * Revision 1.72  1996/08/26  13:23:23  akjoele
 * Added support for Haar transform filtering by changing the following functions:
 * S_ImageHistory(), check_xform_history(), S_ImageHPF_Filter(), S_ImageLPF_Filter(),
 * S_ImageBPF_Filter(), S_ImageBRF_Filter, S_ImageHFE_Filter().
 *
 * Revision 1.71  1996/08/21  19:21:17  yawei
 * commented condRemap_Image before image is displayed. First the RamViewer will do the remap,
 * and secondly, the RamViewer message window will also display the original data type by
 * saying "data type:... (remaped to CVIP_BYTE)".
 *
 * Revision 1.70  1996/08/20  21:02:48  sumbaug
 * modified hist_spec usage note to clarify multiband
 *
 * Revision 1.69  1996/08/14  22:35:25  wli
 * modify frei_chen usage
 *
 * Revision 1.68  1996/08/13  21:04:42  yawei
 * Changed wavelet so it duplicates the image before calling
 * transform. (akjoele/yawei).
 *
 * Revision 1.67  1996/08/13  20:32:59  akjoele
 * Moved assemble_bands and extract_band from LIBBAND to
 * UTILITIES.  Deleted LIBBAND.
 *
 * Revision 1.66  1996/08/12  16:18:08  akjoele
 * Added extract_band and assmeble_bands
 *
 * Revision 1.65  1996/08/10  10:22:22  wli
 * lots of usage changes
 *
 * Revision 1.64  1996/08/08  03:54:06  wli
 * put gray_multiply
 * change declarations of some functions to ANSI C style
 *
 * Revision 1.63  1996/08/03  01:15:31  wli
 * change GUI_Texture_Setup() to be split_merge()
 *
 * Revision 1.62  1996/08/03  00:57:29  wli
 * modify 'split_merge' to call the new function
 * move 'split_merge' from libgeometry to libsegment
 *
 * Revision 1.61  1996/07/31  14:43:12  mzuke
 * minor change to usage message for pseudocol_freq
 * added commas to RGB
 *
 * Revision 1.60  1996/07/31  05:39:56  wli
 * modify 'projection' to call size_norm_projection
 * modify 'gray_linear' to call new GrayLin_Image()
 * modify 'morpho' usage
 *
 * Revision 1.59  1996/07/26  13:18:49  mzuke
 * corrected spelling of boolean in morpho usage message
 *
 * Revision 1.58  1996/07/22  15:10:36  akjoele
 * Modified check_xform_history() function to use getlast_Hist() when
 * checkinbg for transforms performed.  This ensures that the LAST
 * transform performed on the image is returned.
 *
 * Revision 1.57  1996/07/17  22:30:57  wli
 * for pseudocol_freq
 *
 * Revision 1.56  1996/07/16  06:30:15  wli
 * put frequency_pseudocolor() into CVIPTCL
 *
 * Revision 1.55  1996/07/13  08:53:00  wli
 * add texture and rst_invariant
 *
 * Revision 1.54  1996/07/08  07:27:24  wli
 * add hist_feature() and spectral_feature(),
 * change alsm to orientation
 * modify projection()
 *
 * Revision 1.53  1996/07/02  19:12:46  akjoele
 * Defined M_FILE_DEFINED to the compiler, so compilation
 * does not bomb on other systems.  I really dislike all these
 * special define directives to get around compiling problems.
 *
 * Revision 1.52  1996/07/01  07:48:09  wli
 * add area, aspect, alsm, centroid and projection;
 *
 * Revision 1.51  1996/06/26  16:39:15  mzuke
 * changed usage message for bilinear_interp factor
 * from (0.1-1.0) to reflect numbers over 1 are allowed
 *
 * Revision 1.50  1996/06/25  21:02:49  akjoele
 * Added LOCAL_HISTEQ
 *
 * Revision 1.49  1996/06/07  21:46:33  wli
 * change lum_ave to lum_average
 *
 * Revision 1.48  1996/06/07  20:31:24  wli
 * add lum_average() into LIBCOLOR
 *
 * Revision 1.47  1996/06/07  12:49:34  akjoele
 * Added bilinear_interp
 *
 * Revision 1.46  1996/05/30  00:00:41  wli
 * add crop_inner()
 *
 * Revision 1.45  1996/05/29  13:51:53  akjoele
 * Added bilinear_interp
 *
 * Revision 1.44  1996/05/23  21:46:41  wli
 * modify S_ImageHomomorphic() according to mark's opinion
 *
 * Revision 1.43  1996/05/20  20:45:03  wli
 * move *_noise functions to LIBNOISE
 *
 * Revision 1.42  1996/05/16  23:53:57  wli
 * irregular, euler, thiness, perimeter are moved to LIBFEATURE,
 * homomorphic & inverse_filter to LIBXFORMFILTER, pct to LIBCOLOR,
 * hough to LIBSPATIALFLTR
 *
 * Revision 1.41  1996/05/06  00:00:17  zli
 * change the name of blur_spatial to gauss_blur
 *
 * Revision 1.40  1996/04/30  19:43:40  mzuke
 * changed histeq prompt from 1=b,2=g
 * to 1=g,2=b
 *
 * Revision 1.39  1996/04/26  22:35:25  zli
 * check file into RCS
 *
 * Revision 1.38  1996/04/11  19:30:26  kluo
 * fix problem with notch
 *
 * Revision 1.37  1996/04/11  18:25:59  kluo
 * fix arument requirement for notch.
 *
 * Revision 1.36  1996/04/10  02:39:32  kluo
 * add wavelet
 *
 * Revision 1.35  1996/04/07  22:35:41  kluo
 * add info utility
 *
 * Revision 1.34  1996/04/05  18:16:31  kluo
 * add save option
 *
 * Revision 1.33  1996/04/05  03:37:02  kluo
 * add history and add_path
 *
 * Revision 1.32  1996/04/04  19:30:32  kluo
 * fix to avoid repeating name
 *
 * Revision 1.31  1996/04/02  17:51:30  kluo
 * change menus
 *
 * Revision 1.30  1996/04/02  16:35:19  kluo
 * change menu
 *
 * Revision 1.29  1996/03/27  03:12:36  kluo
 * sort routines in library order
 *
 * Revision 1.28  1996/03/27  01:58:39  kluo
 * add in new function for warp.
 *
 * Revision 1.27  1996/03/20  04:14:28  zli
 * add commands for creating simple image
 *
 * Revision 1.26  1996/03/20  02:58:30  kluo
 * add transform filters
 *
 * Revision 1.25  1996/03/13  04:02:48  zli
 * data_remap		remap_Image()  & cast_Image()
 * gamma_noise		gamma_noise()
 * gaussian_noise		gaussian_noise()
 * negative_exp_noise	neg_exp_noise()
 * rayleigh_noise		rayleigh_noise()
 * speckle_noise		speckle_noise()
 * uniform_noise		uniform_noise()
 * edge_link		link_Image()
 * create_checkboard
 *
 * Revision 1.24  1996/03/13  03:19:07  kluo
 * add some new features
 *
 * Revision 1.23  1996/02/27  19:59:19  zli
 * comment out some functions
 *
 * Revision 1.22  1996/02/24  00:31:30  kluo
 * add new option for view
 *
 * Revision 1.21  1996/02/21  03:27:01  zli
 *  add comands of gray_linear, median_filter, median_cut, hist_slide, hist_shrink, hist_stretch, add, subtract, multiply, divide, and, or, xor, not
 *
 * Revision 1.20  1996/02/21  01:39:41  kluo
 * add zoom, rotate, label, scale, morpho
 *
 * Revision 1.19  1996/02/16  04:34:44  kluo
 * fix problem with compression
 *
 * Revision 1.18  1996/02/14  03:51:39  zli
 * add comands of thinness, irregularity, unsharp, spatial_quant, minimum_filter, mean_filter, alpha_filter, contra_filter, geometric_filter, harmonic_filter, ypmean_filter, midpoint_filter, maximum_filter
 *
 * Revision 1.17  1996/02/14  02:40:29  kluo
 * add compression routines
 *
 * Revision 1.16  1996/02/07  03:54:35  kluo
 * fix problem of load image
 *
 * Revision 1.15  1996/02/07  03:48:58  zli
 * add commands of "gray_quant", "euler", "perimeter"
 *
 * Revision 1.14  1996/02/07  03:13:07  kluo
 * add nightvision
 *
 * Revision 1.13  1996/02/07  01:58:11  zli
 * add command of "specify_spatial"
 *
 * Revision 1.12  1996/02/06  21:09:20  kluo
 * add transform, hough
 *
 * Revision 1.11  1996/02/05  18:12:08  kluo
 * add color transform dq.
 *
 * Revision 1.10  1996/02/02  02:43:31  kluo
 * add single filter
 *
 * Revision 1.9  1996/01/31  04:09:11  zli
 *  add commands of cut_paste, multi_resolution, blur_spatial, diff_spatial, lowpass_spatial, laplacian_spatial, highpass_spatial
 *
 * Revision 1.8  1996/01/31  02:00:20  kluo
 * add specification merge_split
 *
 * Revision 1.7  1996/01/30  16:37:58  kluo
 * add new feature such as rename, hidden directory
 *
 * Revision 1.6  1996/01/24  03:48:07  zli
 * add commands of morph_dilate, morph_erode, morph_open, morph_close, enlarge, translate
 *
 * Revision 1.5  1996/01/24  02:51:31  kluo
 * add edge detectors
 *
 * Revision 1.4  1996/01/19  22:49:11  kluo
 * fix bomb. If you type cvip only, bomb out. Now it will prints out
 * available commands.
 *
 * Revision 1.3  1996/01/19  22:33:45  kluo
 * add IMG_PATH enviroment variable
 *
 * Revision 1.2  1996/01/16  17:45:22  kluo
 * get rid of old name definition for Image pointer, matrix pointer, etc.
 *
 * Revision 1.1  1996/01/16  16:30:49  kluo
 * Initial revision
 *
*
****************************************************************************/
#define M_FILE_DEFINED
#include <tcl.h>

#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <limits.h>
#include "cvipImg.h"
#include "CVIPtools.h"
#include "CVIPtoolkit.h"
#include "CVIPdef.h"
#include "CVIPimage.h"
#include "CVIPmanager.h"
#include "CVIPconvert.h"
#include "CVIPvector.h"
#include "CVIPcolor.h"
#include "DermVis.h"
#include <CVIPmorph.h>
#include "bilinear.h"
#include "CVIPgeometry.h"
#include "CVIPtransform.h"
#include "CVIPfs.h"
#include "CVIPxformfilter.h"
#include "CVIPcompress.h"
#include "CVIPmap.h"
#include "CVIPsegment.h"
#include "cvipUtil.h"
#include "CVIPspfltr.h"
#include "CVIPhisto.h"
#include "CVIPdrawimg.h"
#include "histogram.h"

#ifdef WIN32
#ifndef strcasecmp
#define strcasecmp(a,b) stricmp(a,b)
#endif
#endif

typedef dlinked_list *ImagePathList;
extern char temp_dir[];
extern char *Cmpr_Name[];
extern ImagePathList	CVIPTCL_iplP;
extern void quite_CVIP(void);
extern ImagePathList CVIPTCL_iplP;

Image * gray_binary(Image *, int);
Image * fft_phase(Image * fftImage, int remap_norm, float k);
extern    char *Cmpr_Name[];

static void
getTempFileName(Tcl_Interp *interp, char imageName[], CMPR_FORMAT format,
		char *filename, char *result_name); 

/*
** ------------------------
** 	Prototypes.
** ------------------------
*/

CMD(CvipImage_Cmd);
CMD(S_WAITFOR);

CMD(S_ImageRead);
CMD(S_ImageWrite);
CMD(S_ImageRename);
CMD(S_ImageObjects);
CMD(S_ImageHisteq);
CMD(S_ImageDisplay);
CMD(S_ImageGrayMorph);
CMD(S_ImageSegmentPct);
CMD(S_ImageTransform);
CMD(S_ImageHashList);
CMD(S_TempFilename);

CMD(S_ImageFrei_Chen);
CMD(S_ImagePrewitt);
CMD(S_ImageLaplacian);
CMD(S_ImageRoberts);
CMD(S_ImageSobel);
CMD(S_ImageKirsch);
CMD(S_ImageRobinson);
CMD(S_ImagePyramid);
CMD(S_ImageMorpho);
CMD(S_ImageViewer);
CMD(S_ImageRange);
CMD(S_ImageDataGet);

CMD(S_Imagezonal);
CMD(S_Imagezonal2);
CMD(S_Imagegray_binary);
CMD(S_Imagemmse_filter);
CMD(S_Imageadaptive_contrast);
CMD(S_ImageIGS);
CMD(S_Imagefft_phase);

ECMD(S_ImageSmooth);
ECMD(S_ImageSharp);
ECMD(S_ImageRasterFilter);
ECMD(S_ImageBinthresh);

ECMD(S_ImageMORPH_DILATE);
ECMD(S_ImageMORPH_ERODE);
ECMD(S_ImageMORPH_OPEN);
ECMD(S_ImageMORPH_CLOSE);
ECMD(S_ImageENLARGE);
ECMD(S_ImageTRANSLATE);

ECMD(S_ImageCOPY_PASTE);
ECMD(S_ImageMULTIRES);
ECMD(S_ImageBLUR_SPATIAL);
ECMD(S_ImageDIFFERENCE_SPATIAL);
ECMD(S_ImageLOWPASS_SPATIAL);
ECMD(S_ImageLAPLACIAN_SPATIAL);
ECMD(S_ImageHIGHPASS_SPATIAL);


CMD(S_ImageConvert);
CMD(S_ImageSplitMerge);
CMD(S_ImageHistoSpec);
CMD(S_ImageSingle_Filter);
CMD(S_ImageColorTransform);
CMD(S_ImageDequeue);

CMD(S_ImageFFT);
CMD(S_ImageWALSH);
CMD(S_ImageHADAMARD);
CMD(S_ImageHAAR);
CMD(S_ImageDCT);
CMD(S_ImageHOUGH);
CMD(S_ImageSCTSPLIT);
CMD(S_ImageHISTTHRESH);
CMD(S_ImageNightVision);
CMD(S_ImageWavelet);

ECMD(S_ImageSPECIFY_SPATIAL);
ECMD(S_ImageGRAY_QUANT);
ECMD(S_ImageEULER);
ECMD(S_ImagePERIMETER);

ECMD(S_ImageAREA);
ECMD(S_ImageCENTROID);
ECMD(S_ImageORIENTATION);
ECMD(S_ImageASPECT);
ECMD(S_ImagePROJECTION);
ECMD(S_ImageHIST_FEATURE);
ECMD(S_ImageSPECTRAL_FEATURE);
ECMD(S_ImageTEXTURE);
ECMD(S_Imagerst_invariant);

CMD(S_ImageBTC);
CMD(S_ImageBTC2);
CMD(S_ImageBTC3);
CMD(S_ImageDPCM);
CMD(S_ImageZVL);
CMD(S_Imagerms_error);
CMD(S_Imagesnr);
CMD(S_ImageGray_RLC);
CMD(S_ImageBitplane_RLC);
CMD(S_ImageHuffman);
CMD(S_ImageJPEG);

ECMD(S_ImageTHINNESS);
ECMD(S_ImageIRREGULARITY);
ECMD(S_ImageUNSHARP_SPATIAL);
ECMD(S_ImageSPATIAL_QUANT);
ECMD(S_ImageMINIMUM_FILTER);
ECMD(S_ImageMEAN_FILTER);
ECMD(S_ImageALPHA_FILTER);
ECMD(S_ImageCONTRA_FILTER);
ECMD(S_ImageGEOMETRIC_FILTER);
ECMD(S_ImageHARMONIC_FILTER);
ECMD(S_ImageYP_MEAN_FILTER);
ECMD(S_ImageMIDPOINT_FILTER);
ECMD(S_ImageMAXIMUM_FILTER);

CMD(S_ImageZoom);
CMD(S_ImageRotate);
CMD(S_ImageShrink);
CMD(S_ImageBilinear);
CMD(S_rcCrop);
CMD(S_ImageCrop);
CMD(S_ImageLabel);
CMD(S_ImageFuzzyC);
CMD(S_ImageLOCAL_HISTEQ);

ECMD(S_GRAY_LIN_MOD);
ECMD(S_GRAY_MULTIPLY);
ECMD(S_MEDIAN_FILTER);
ECMD(S_ImageMEDIANCUT);
ECMD(S_HIST_SLIDE);
ECMD(S_HIST_SHRINK);
ECMD(S_HIST_STRETCH);
ECMD(S_ImageADD);
ECMD(S_ImageMULTIPLY);
ECMD(S_ImageSUBTRACT);
ECMD(S_ImageDIVIDE);
ECMD(S_ImageAND);
ECMD(S_ImageOR);
ECMD(S_ImageXOR);
ECMD(S_ImageNOT);


CMD(S_ImagePCT);
CMD(S_ImagePctMedian);
CMD(S_ImageLuminance);
CMD(S_Imagelum_average);
CMD(S_Imagefrequency_pseudocolor);
CMD(S_ImageHomomorphic);
CMD(S_ImageInverseFilter);
CMD(S_ImageSimpleWiener);
CMD(S_Imageleast_squares_filter);
CMD(S_Imagewiener_filter);
CMD(S_Imageparametric_wiener);
CMD(S_Imagepower_spect_eq);
CMD(S_Imagegeometric_mean);
CMD(S_ImageNotch);

ECMD(S_DATA_REMAP);
ECMD(S_ImageGAMMA_NOISE);
ECMD(S_ImageGAUSSIAN_NOISE);
ECMD(S_ImageNEGATIVE_NOISE);
ECMD(S_ImageRAYLEIGH_NOISE);
ECMD(S_ImageSALT_NOISE);
ECMD(S_ImageUNIFORM_NOISE);
ECMD(S_ImageEDGE_LINK);
ECMD(S_CHECKBOARD);

CMD(S_ImageHPF_Filter);
CMD(S_ImageLPF_Filter);
CMD(S_ImageBPF_Filter);
CMD(S_ImageBRF_Filter);
CMD(S_ImageDATA_H_Image);
CMD(S_ImageH_Image);
CMD(S_ImageHFE_Filter);

CMD(S_BLACK);
ECMD(S_CIRCLE);
ECMD(S_LINE);
ECMD(S_RECTANGLE);
ECMD(S_SINE_WAVE);
ECMD(S_COSINE_WAVE);
ECMD(S_SQUARE_WAVE);
ECMD(S_Display_Regular);
ECMD(S_Area_Points);

CMD(S_SolveBilinear);
CMD(S_Create_Mesh_Sine);
CMD(S_Mesh_Warp);
CMD(S_Keyboard_Mesh);

CMD(S_AddImagePath);
CMD(S_ImageHistory);
CMD(S_ImageInfo);

CMD(S_ImageEXTRACT_BAND);
CMD(S_ImageASSEMBLE_BANDS);

static char * CVIPTCL_find_ImagePathList(ImagePathList,char*);
static void draw_histo(Image *inputImage,char *image_name);
static void compare_find_max(float **map, float **map1,int dim, int len);
static int chk_bin(Image *inputImage);
static Image* draw_picture(float **map,int len,int dim);
static void view_rgb(Image *resultimage, char *name, char option);

int User_Init( Tcl_Interp *interp );

/*
** ----------------------------
** 			Globals.
** ----------------------------
*/


/*
 * Built-in commands, and the procedures associated with them:
 */

static CmdInfo S_Cmds[] = {
    /*
     * Commands in the CVIP image extension
     */
	{"LIBARITHLOGIC",	S_WAITFOR},
	{"add",			S_ImageADD},
	{"and",			S_ImageAND},
	{"divide",		S_ImageDIVIDE},
	{"multiply",		S_ImageMULTIPLY},
	{"not",			S_ImageNOT},
	{"or",			S_ImageOR},
	{"subtract",		S_ImageSUBTRACT},
	{"xor",			S_ImageXOR},
	{"LIBAND",		S_WAITFOR},
	{"assemble_bands",	S_ImageASSEMBLE_BANDS},
	{"extract_band",	S_ImageEXTRACT_BAND},
	{"LIBCOLOR",		S_WAITFOR},
	{"cxform",		S_ImageColorTransform},
	{"pseudocol_freq", 	S_Imagefrequency_pseudocolor},
	{"luminance",		S_ImageLuminance},
        {"lum_average",         S_Imagelum_average},
        {"pct",                 S_ImagePCT},
	{"LIBCOMPRESS",		S_WAITFOR},
	{"bitplane_rlc",	S_ImageBitplane_RLC},
	{"btc",			S_ImageBTC},
	{"btc2",		S_ImageBTC2},
	{"btc3",		S_ImageBTC3},
	{"dpc",		S_ImageDPCM},
	{"gray_rlc",		S_ImageGray_RLC},
	{"huffman",		S_ImageHuffman},
	{"jpeg",		S_ImageJPEG},
	{"zonal", 		S_Imagezonal},
	{"zonal2",		S_Imagezonal2},
	{"zvl",			S_ImageZVL},
	{"rms_error",		S_Imagerms_error},
	{"snr",			S_Imagesnr},
	{"LIBCONVERTER",	S_WAITFOR},
	{"convert",		S_ImageConvert},
	{"gray_binary",		S_Imagegray_binary},
	{"read",		S_ImageWrite},
	{"write",		S_ImageRead},
        {"LIBFEATURE",          S_WAITFOR},
	{"area",		S_ImageAREA},
	{"aspect",		S_ImageASPECT},
	{"centroid",		S_ImageCENTROID},
        {"euler",               S_ImageEULER},
	{"hist_feature",	S_ImageHIST_FEATURE},
        {"irregular",           S_ImageIRREGULARITY},
        {"label",               S_ImageLabel},
        {"orientation",         S_ImageORIENTATION},
        {"perimeter",           S_ImagePERIMETER},
	{"projection",		S_ImagePROJECTION},
	{"rst_invariant",	S_Imagerst_invariant},
	{"spectral_feature",	S_ImageSPECTRAL_FEATURE},
	{"texture",		S_ImageTEXTURE},
        {"thinness",            S_ImageTHINNESS},
	{"LIBGEOMETRY",		S_WAITFOR},
	{"bilinear_interp",	S_ImageBilinear},
	{"black",		S_BLACK},
	{"circle",		S_CIRCLE},
	{"copy_paste",		S_ImageCOPY_PASTE},
	{"cosine_wave",		S_COSINE_WAVE},
	{"create_checkboard",	S_CHECKBOARD},
	{"create_mesh",		S_Create_Mesh_Sine},
	{"crop",		S_ImageCrop},	
	{"display_mesh",	S_Display_Regular},
	{"enlarge",		S_ImageENLARGE},
	{"keyboard_mesh",	S_Keyboard_Mesh},
	{"line",		S_LINE},
/*	{"paint_cell",		S_Area_Points},		*/
/*	{"rc_crop",		S_rcCrop}, */
	{"rectangle",		S_RECTANGLE},
	{"rotate",		S_ImageRotate},
	{"shrink",		S_ImageShrink},
	{"sine_wave",		S_SINE_WAVE},
	{"solve_bilinear",	S_SolveBilinear},
	{"spatial_quant",	S_ImageSPATIAL_QUANT},
	{"square_wave",		S_SQUARE_WAVE},
	{"translate",		S_ImageTRANSLATE},
	{"warp",		S_Mesh_Warp},
	{"zoom",		S_ImageZoom},
	{"LIBHISTOGRAM",	S_WAITFOR},
	{"gray_linear",		S_GRAY_LIN_MOD},
	{"gray_multiply",	S_GRAY_MULTIPLY}, 
	{"hist_shrink",		S_HIST_SHRINK},
	{"hist_slide", 		S_HIST_SLIDE},
	{"hist_stretch",	S_HIST_STRETCH},
	{"histeq", 		S_ImageHisteq},
	{"hist_spec", 		S_ImageHistoSpec},
	{"local_histeq",	S_ImageLOCAL_HISTEQ},
	{"LIBMORPH",		S_WAITFOR},
	{"morph_close",		S_ImageMORPH_CLOSE},
	{"morph_dilate",	S_ImageMORPH_DILATE},
	{"morph_erode",		S_ImageMORPH_ERODE},
	{"morph_open",		S_ImageMORPH_OPEN},
	{"morpho",		S_ImageMorpho},
        {"LIBNOISE",            S_WAITFOR},
        {"gamma_noise",         S_ImageGAMMA_NOISE},
        {"gaussian_noise",      S_ImageGAUSSIAN_NOISE},
        {"negative_exp_noise",  S_ImageNEGATIVE_NOISE},
        {"rayleigh_noise",      S_ImageRAYLEIGH_NOISE},
        {"speckle_noise",       S_ImageSALT_NOISE},
        {"uniform_noise",       S_ImageUNIFORM_NOISE},
/*	{"LIBOBJECT",		S_WAITFOR},
	{"label",		S_ImageLabel}, */
	{"LIBSEGMENT",		S_WAITFOR},
	{"fuzzyc",		S_ImageFuzzyC},
	{"gray_quant",		S_ImageGRAY_QUANT},
	{"hist_thresh",		S_ImageHISTTHRESH},
	{"igs",			S_ImageIGS},
	{"median_cut", 		S_ImageMEDIANCUT},
	{"multi_resolution",	S_ImageMULTIRES},
	{"pct_median", 		S_ImagePctMedian},
	{"sct_split",		S_ImageSCTSPLIT},
        {"split_merge",         S_ImageSplitMerge},
	{"threshold",		S_ImageBinthresh},
	{"LIBSPATIALFILTER",	S_WAITFOR},
	{"adaptive_contrast", 	S_Imageadaptive_contrast},
	{"alpha_filter",	S_ImageALPHA_FILTER},
	{"gauss_blur",		S_ImageBLUR_SPATIAL},
	{"contra_filter",	S_ImageCONTRA_FILTER},
	{"diff_spatial",	S_ImageDIFFERENCE_SPATIAL},
	{"edge_link",		S_ImageEDGE_LINK},
	{"frei_chen",		S_ImageFrei_Chen},
	{"geometric_filter",	S_ImageGEOMETRIC_FILTER},
	{"harmonic_filter",	S_ImageHARMONIC_FILTER},
	{"highpass_spatial",	S_ImageHIGHPASS_SPATIAL},
        {"hough",               S_ImageHOUGH},
	{"kirsch",		S_ImageKirsch},
	{"laplacian",		S_ImageLaplacian},
	{"laplacian_spatial",	S_ImageLAPLACIAN_SPATIAL},
	{"lowpass_spatial",	S_ImageLOWPASS_SPATIAL},
	{"maximum_filter",		S_ImageMAXIMUM_FILTER},   
	{"mean_filter",		S_ImageMEAN_FILTER},
	{"median_filter",	S_MEDIAN_FILTER},
	{"midpoint_filter",	S_ImageMIDPOINT_FILTER},
	{"minimum_filter",	 	S_ImageMINIMUM_FILTER},
	{"mmse_filter",		S_Imagemmse_filter},
	{"nightvision",		S_ImageNightVision},
	{"prewitt",		S_ImagePrewitt},
	{"pyramid",		S_ImagePyramid},
	{"raster_deblur",	S_ImageRasterFilter},
	{"roberts",		S_ImageRoberts},
	{"robinson",		S_ImageRobinson},
	{"sharp",	      	S_ImageSharp},
        {"single_filter",       S_ImageSingle_Filter},
	{"smooth",		S_ImageSmooth},
	{"sobel",		S_ImageSobel},
	{"specify_spatial",	S_ImageSPECIFY_SPATIAL},
	{"unsharp",		S_ImageUNSHARP_SPATIAL},
	{"ypmean_filter",	S_ImageYP_MEAN_FILTER},
	{"LIBTRANSFORM",	S_WAITFOR},
	{"dct",			S_ImageDCT},
	{"fft",			S_ImageFFT},
	{"fft_phase", 		S_Imagefft_phase},
	{"haar",		S_ImageHAAR},
	{"hadamard",		S_ImageHADAMARD},
	{"walsh",		S_ImageWALSH},
	{"wavelet",		S_ImageWavelet},
	{"LIBXFORMFILTER",	S_WAITFOR},
	{"bpf_filter",		S_ImageBPF_Filter},
	{"brf_filter",		S_ImageBRF_Filter},
	{"data_h_image",	S_ImageDATA_H_Image},
	{"h_image",	S_ImageH_Image},
	{"hfe_filter",		S_ImageHFE_Filter},
        {"homomorphic",         S_ImageHomomorphic},
	{"hpf_filter",		S_ImageHPF_Filter},
        {"inverse_xformfilter",      S_ImageInverseFilter},
	{"simple_wiener",	S_ImageSimpleWiener},
	{"wiener",	S_Imagewiener_filter},
	{"parametric_wiener",	S_Imageparametric_wiener},
	{"least_squares",	S_Imageleast_squares_filter},
	{"power_spect_eq",	S_Imagepower_spect_eq},
	{"geometric_mean",	S_Imagegeometric_mean},
	{"lpf_filter",		S_ImageLPF_Filter},
	{"notch",		S_ImageNotch},
	{"UTILITY",		S_WAITFOR},
	{"add_path",		S_AddImagePath},
	{"assemble_bands",	S_ImageASSEMBLE_BANDS},
	{"datarange",     S_ImageRange},
	{"data_get",     S_ImageDataGet},
	{"data_remap",		S_DATA_REMAP},
	{"display", 		S_ImageDisplay},
	{"dq",			S_ImageDequeue},
	{"extract_band",	S_ImageEXTRACT_BAND},
	{"fft_phase",		S_Imagefft_phase},
	{"gray_binary", 	S_Imagegray_binary},
	{"history",		S_ImageHistory},
	{"info",		S_ImageInfo},
	{"list",	 	S_ImageHashList},
	{"load",		S_ImageRead},
	{"read",		S_ImageRead},
        {"temp_filename",       S_TempFilename},
	{"rename",		S_ImageRename},
	{"rms_error",		S_Imagerms_error},
	{"save",		S_ImageWrite},
	{"snr",			S_Imagesnr},
	{"view",		S_ImageDisplay},
	{"viewer",		S_ImageViewer},
	{"write",               S_ImageWrite},

	{NULL,			(Tcl_CmdProc *) NULL}

};


static Tcl_HashTable S_CmdTable;

/*
** ImageRec hash table.
*/
static Tcl_HashTable S_ImageTable;

static Image * convertImage_Queue(Image *inputImage, 
			CVIP_TYPE *desir_type, int num_type){
    Image * copyImage;
    int counter;
    char *name[] ={"CVIP_BYTE","CVIP_SHORT","CVIP_INTEGER","CVIP_FLOAT","CVIP_DOUBLE"};

    for(counter = 0; counter < num_type; counter++){
	if(inputImage->bandP[0]->data_type == desir_type[counter]){
	     return (Image*)duplicate_Image(inputImage);
	}
    }
    if(inputImage->bandP[0]->data_type > desir_type[0]){
	fprintf(stderr,"Image is remapped into %s\n",name[desir_type[0]]);
	switch(desir_type[0]){
	    case CVIP_BYTE:
     		copyImage = (Image*)remap_Image(inputImage,CVIP_BYTE,0,255);
		break;
	    case CVIP_SHORT:
		copyImage =(Image*)remap_Image(inputImage,CVIP_SHORT,
						SHRT_MIN,SHRT_MAX);
		break;
	    case CVIP_INTEGER:
		copyImage =(Image*)remap_Image(inputImage,CVIP_INTEGER,
						LONG_MIN,LONG_MAX);
		break;
	    default:
		fprintf(stderr,"Is is a double precision image?\n");
	}
    }
    else{
	copyImage = duplicate_Image(inputImage);
	if(cast_Image(copyImage,desir_type[0]))
 	    fprintf(stderr,"Image is cast into %s\n",name[desir_type[0]]);
	else
	    return NULL;
    }

    return copyImage;
}

static int 
chk_bin(Image *inputImage)
{
	byte 		**image; 	/* 2-d matrix data pointer */
	int 		y,		/* row index */
			x, 		/* column index */
			bands;		/* band index */
			
	unsigned int 	rows, 		/* number of rows in image */
			cols,		/* number of columns in image */
			no_of_bands,	/* number of image bands */
			level[2];	/* gray levels */


	/*
	** Gets the number of image bands (planes)
	*/
	no_of_bands = getNoOfBands_Image(inputImage);

	/* 
	** Gets the number of rows in the input image  
	*/
	rows =  getNoOfRows_Image(inputImage);

	/* 
	** Gets the number of columns in the input image  
	*/
  	cols =  getNoOfCols_Image(inputImage);
	
	level[1]=256;
	
	/* 
	** compares the pixel value at the location (y,x)
	** with the threshold value. If it is greater than 
 	** the threshold value it writes 255 at the location
	** else 0
	*/
	for(bands=0; bands < no_of_bands; bands++) {
		/*
		** reference each band of image data in 2-d matrix form
		*/
  		image = getData_Image(inputImage, bands);
		if(bands == 0)
		    level[0]=image[0][0];
  		for(y=0; y < rows; y++) {
     			for(x=0; x < cols; x++) {
				if((image[y][x] != image[0][0])&&(level[1]==256)) 
            				level[1] = image[y][x];
				if((image[y][x]!=level[0])&&(image[y][x]!=level[1]))
	     				return 0;
         		}
     		}
	}
	return 1;
}


/*
** ---------------
** 	 Init
** ---------------
*/

int
CvipImage_Init( Tcl_Interp *interp )
{

	CmdInfo *cmd;

	quite_CVIP();

	/*
	** Create table for sub-commands
	*/
	printf("\n\n   ****************************************************");
	printf("\n   *                CVIPTCL/CVIPWISH                  *");
	printf("\n   *                   Version: 1.6                   *");
	printf("\n   *          Scott Umbaugh, Kun Luo, Zhen Li         *");
	printf("\n   *              Arve Kjoelen, Mark Zuke             *");
	printf("\n   *              Wenxing Li, Yansheng Wei            *");
	printf("\n   *         Electrical Engineering Department        *");
	printf("\n   *   Southern Illinois University at Edwardsville   *");
	printf("\n   ****************************************************\n");

	Tcl_InitHashTable(&S_CmdTable, TCL_STRING_KEYS);
	for (cmd = S_Cmds; cmd->Name != NULL; cmd++) {
		int newEntry;
		Tcl_HashEntry *entry;
		entry = Tcl_CreateHashEntry(&S_CmdTable, cmd->Name, &newEntry);
		Tcl_SetHashValue(entry, cmd->Proc);
	}

	/*
	** Create hash table for images.
	*/
	Tcl_InitHashTable(&S_ImageTable, TCL_STRING_KEYS);

	CREATE("cvip", CvipImage_Cmd);

	return TCL_OK;
}

/*
** ----------------------
** 	  Functions.
** ----------------------
*/

static int
S_ImageHistory(ClientData clientData, Tcl_Interp *interp, int argc, char **argv )
{
	Image *imageP;
	int result = 0;

	if(argc != 3 && argc != 4){
		sprintf(interp->result,"Usage: cvip history <image> [PROGRAM]");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
	
	if(argc == 3){
		Tcl_ResetResult(interp);

		if(history_check(FFT,imageP))
			Tcl_AppendResult(interp, "FFT\n",(char *)NULL);
		if(history_check(I_FFT,imageP))
			Tcl_AppendResult(interp, "I_FFT\n",(char *)NULL);
		if(history_check(PCT,imageP))
			Tcl_AppendResult(interp, "PCT\n",(char *)NULL);
		if(history_check(I_PCT,imageP))
			Tcl_AppendResult(interp, "I_PCT\n",(char *)NULL);
		if(history_check(WALSH,imageP))
			Tcl_AppendResult(interp, "WALSH\n",(char *)NULL);
		if(history_check(I_WALSH,imageP))
			Tcl_AppendResult(interp, "I_WALSH\n",(char *)NULL);
		if(history_check(HAAR,imageP))
			Tcl_AppendResult(interp, "HAAR\n",(char *)NULL);
		if(history_check(I_HAAR,imageP))
			Tcl_AppendResult(interp, "I_HAAR\n",(char *)NULL);
		if(history_check(HADAMARD,imageP))
			Tcl_AppendResult(interp, "HADAMARD\n",(char *)NULL);
		if(history_check(I_HADAMARD,imageP))
			Tcl_AppendResult(interp, "I_HADAMARD\n",(char *)NULL);
		if(history_check(HAAR,imageP))
			Tcl_AppendResult(interp, "HAAR\n",(char *)NULL);
		if(history_check(I_HAAR,imageP))
			Tcl_AppendResult(interp, "I_HAAR\n",(char *)NULL);
		if(history_check(DCTXFORM,imageP))
			Tcl_AppendResult(interp, "DCTXFORM\n",(char *)NULL);
		if(history_check(I_DCTXFORM,imageP))
			Tcl_AppendResult(interp, "I_DCTXFORM\n",(char *)NULL);
		if(history_check(WAVELET,imageP))
			Tcl_AppendResult(interp, "WAVELET\n",(char *)NULL);
		if(history_check(I_WAVELET,imageP))
			Tcl_AppendResult(interp, "I_WAVELET\n",(char *)NULL);
		if(history_check(XFORMFILTER,imageP))
			Tcl_AppendResult(interp, "XFORMFILTER\n",(char *)NULL);
		if(history_check(INVERSEFILTER,imageP))
			Tcl_AppendResult(interp, "INVERSEFILTER\n",(char *)NULL);
		if(history_check(WIENER_FILTER,imageP))
			Tcl_AppendResult(interp, "WIENER_FILTER\n",(char *)NULL); 
		if(history_check(ZERO_PAD,imageP))
			Tcl_AppendResult(interp, "ZERO_PAD\n",(char *)NULL);
		if(history_check(MEDIAN_FILTER,imageP))
			Tcl_AppendResult(interp, "MEDIAN_FILTER\n",(char *)NULL);
	}
	if(argc == 4){
		if(strcasecmp(argv[3],"FFT")==0)
			result = history_check(FFT,imageP);
		else if(strcasecmp(argv[3],"I_FFT")==0)
			result = history_check(I_FFT,imageP);
		else if(strcasecmp(argv[3],"PCT")==0)
			result = history_check(PCT,imageP);
		else if(strcasecmp(argv[3],"I_PCT")==0)
			result = history_check(I_PCT,imageP);
		else if(strcasecmp(argv[3],"WALSH")==0)
			result = history_check(WALSH,imageP);
		else if(strcasecmp(argv[3],"I_WALSH")==0)
			result = history_check(I_WALSH,imageP);
		else if(strcasecmp(argv[3],"HADAMARD")==0)
			result = history_check(HADAMARD,imageP);
		else if(strcasecmp(argv[3],"I_HADAMARD")==0)
			result = history_check(I_HADAMARD,imageP);
		else if(strcasecmp(argv[3],"HAAR")==0)
			result = history_check(HAAR,imageP);
		else if(strcasecmp(argv[3],"I_HAAR")==0)
			result = history_check(I_HAAR,imageP);
		else if(strcasecmp(argv[3],"DCTXFORM")==0)
			result = history_check(DCTXFORM,imageP);
		else if(strcasecmp(argv[3],"I_DCTXFORM")==0)
			result = history_check(I_DCTXFORM,imageP);
		else if(strcasecmp(argv[3],"WAVELET")==0)
			result = history_check(WAVELET,imageP);
		else if(strcasecmp(argv[3],"I_WAVELET")==0)
			result = history_check(I_WAVELET,imageP);
		else if(strcasecmp(argv[3],"XFORMFILTER")==0)
			result = history_check(XFORMFILTER,imageP);
		else if(strcasecmp(argv[3],"INVERSEFILTER")==0)
			result = history_check(INVERSEFILTER,imageP);
		else if(strcasecmp(argv[3],"WIENER_FILTER")==0)
			result = history_check(WIENER_FILTER,imageP); 
		else if(strcasecmp(argv[3],"ZERO_PAD")==0)
			result = history_check(ZERO_PAD,imageP);
		else if(strcasecmp(argv[3],"MEDIAN_FILTER")==0)
			result = history_check(MEDIAN_FILTER,imageP);
		
		if(result){
			sprintf(interp->result,"YES");
		}
		else{
			sprintf(interp->result,"NO");
		}
	}



	return TCL_OK;

}

extern char *color_format_string[];
extern char * image_format_string[];

static int
S_ImageInfo(ClientData clientData, Tcl_Interp *interp, int argc, char **argv )
{
	int row, col, bands;
	IMAGE_FORMAT format;
	COLOR_FORMAT color;
	char *data_type[] = {"BYTE", "SHORT", "INTEGER", "FLOAT", "DOUBLE"};
	char *data_format[] = {"REAL", "COMPLEX"};
	Image *imageP;

	if(argc != 3){
		sprintf(interp->result,"Usage: cvip info <image>");
		return TCL_ERROR;
	}
	
	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
	
	format = imageP->image_format;
	color  = imageP->color_space;
	bands  = imageP->bands;
	row    = imageP->bandP[0]->rows;
	col    = imageP->bandP[0]->cols;

	sprintf(interp->result,"%s %s %d %d %d %s %s",
			image_format_string[format],color_format_string[color],
			bands, row, col,
			data_type[(int)(imageP->bandP[0]->data_type)],
			data_format[(int)(imageP->bandP[0]->data_format)]);
	
	return TCL_OK;
}
	
		
static int
S_AddImagePath(ClientData clientData, Tcl_Interp *interp, int argc, char **argv )
{
	char *strP;

	if(argc != 3){
		sprintf(interp->result,"Usage: cvip add_path <path>");
		return TCL_ERROR;
	}
	
	strP = (char*)malloc(strlen(argv[2])+1);
	strcpy(strP, argv[2]);
	addtail_DLL(CVIPTCL_iplP,strP);

	return TCL_OK;
}




static int
S_WAITFOR(ClientData clientData, Tcl_Interp *interp, int argc, char **argv )
{
	char *name = NULL;
	int i=0;
	int counter=0, pos;
	char space[21];

	memset(space,' ',sizeof(space));
	space[20] = '\0';

	while(NULL != (name = S_Cmds[i++].Name)){
		if(*name >='a')
			continue;
		else if( strcmp(name,argv[1]) == 0)
			break;
	}

	Tcl_ResetResult(interp);
	Tcl_AppendResult(interp, "Available for ", argv[1], ":\n", (char *)NULL);
	while(NULL != (name = S_Cmds[i++].Name)){
		counter++;
		if(name[0] < 'a'){
			break;
		}
		else if(counter == 4){
			Tcl_AppendResult(interp, name, "\n", NULL);
			counter = 0;
		}
		else{
			pos = strlen(name);
			Tcl_AppendResult(interp, name,&space[pos], NULL);
		}
	}

	return TCL_OK;
}
		

static int
CvipImage_Cmd ( ClientData clientData, Tcl_Interp *interp, int argc, char **argv )
{
	Tcl_HashEntry *	entry;
	Tcl_CmdProc *		proc;

	USE(clientData);
	USE(interp);
	USE(argc);
	USE(argv);

	if(argc != 1){
	    entry = Tcl_FindHashEntry(&S_CmdTable, argv[1]);
	    if (NULL != entry) {
		proc = (Tcl_CmdProc *) Tcl_GetHashValue(entry);
		return (*proc)(clientData, interp, argc, argv);
	    }
	}
	Tcl_ResetResult(interp);
	Tcl_AppendResult(interp, "   Available for ", argv[0], ":\n", (char *)NULL);
	Util_AppendCommands(interp, S_Cmds);

	return TCL_OK;
}

int
S_ImageAddToHash(Tcl_Interp *interp, Image * imageP, const char *name) {
	static unsigned int id = 0;
	int new;
	char *fname = (char*)name;
	Tcl_HashEntry *entryPtr;
	Image *tempImage;

	do {
		if(name) {
			while(tempImage = S_ImageGetFromHash(interp,
				 fname))
				strcat(fname,"_r");
			sprintf(interp->result, "%s", name);
		} else {
			sprintf(interp->result, "image%u", id);
		}
		id++;
		entryPtr = Tcl_CreateHashEntry(&S_ImageTable, interp->result, &new);
	} while (!new);
	
	Tcl_SetHashValue(entryPtr, imageP);
	
	return TCL_OK;
}

Image * 
S_ImageGetFromHash(Tcl_Interp *interp, const char *name)
{
	Tcl_HashEntry *entryPtr;
	
	entryPtr = Tcl_FindHashEntry(&S_ImageTable, (char *)name);

	if (entryPtr == NULL) {
		Tcl_AppendResult(interp, "no image named \"", name, "\"", (char *)NULL);
		return NULL;
	}
	return (Image * ) Tcl_GetHashValue(entryPtr);
}

int  
S_ImageDeleteFromHash(Tcl_Interp *interp, const char *name)
{
	Tcl_HashEntry *entryPtr;
	
	entryPtr = Tcl_FindHashEntry(&S_ImageTable, (char *)name);

	if (entryPtr == NULL) {
		Tcl_AppendResult(interp, "no image named \"", name, "\"", (char *)NULL);
		return 0;
	}
	Tcl_DeleteHashEntry(entryPtr);
	
	return 1;
}



/*
** -----------------------------
** 		Sub Commands.
** -----------------------------
*/

static int
S_ImageRead(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
        Image * image = (NULL);
	FILE *fptr;
	char filename[200], *strP=NULL;

	if (argc != 3) {
	    sprintf(interp->result, "Usage: cvip read <image>");
	    return TCL_ERROR;
	}
	
	#ifndef WIN32
	if((fptr=fopen(argv[2],"r")) == NULL){
	#else
	if((fptr=fopen(argv[2],"rb")) == NULL){
	#endif
	    if((strP=CVIPTCL_find_ImagePathList(CVIPTCL_iplP, argv[2]))
				 == NULL){
		sprintf(interp->result, "Cannot find image %s",argv[2]);
		return TCL_ERROR;
	    }
	}	
	
	if(strP != NULL){
	    strcpy(filename,strP);
	    strcat(filename,"/");
	    strcat(filename,argv[2]);
	}
	else 
	    strcpy(filename,argv[2]);
	image = CVIPreadimage(filename,0,0);
	
	Tcl_ResetResult(interp);

	(void) S_ImageAddToHash(interp, image, argv[2]);
	
	return TCL_OK;
}

static int
S_ImageSingle_Filter(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
        Image * imageP = (NULL),*resultImage;
	int kernel;
	char **split;
	int number;
	float *data;
	CVIP_TYPE	required_type[5];
	int	num_type;

	required_type[0] = CVIP_BYTE;
	num_type = 1;

	if (argc != 12) {
	    sprintf(interp->result, "Usage: cvip single_filter <image> <horizontal sizing factor> <vertical sizing factor> <x coord for new center> < y coord for new center> <angle of rotation> <beta> <kernel size(3,5,7)> <kernel arary(enclosed by curly brackets) <filter operation(1-4)>");
	    return TCL_ERROR;
	}

	kernel = atoi(argv[9]);
	data = (float*)calloc(kernel*kernel+1,sizeof(float));
	     
	if(data == NULL){
	    sprintf(interp->result,"Fatal Error: malloc()");
	    return TCL_ERROR;
	}
	if(atof(argv[8]) != 0){
	    if(Tcl_SplitList(interp, argv[10], &number, &split)
		 == TCL_ERROR){
		sprintf(interp->result,"Error in interpreting kernel array");
		return TCL_ERROR;
	    }
	    if(number != kernel*kernel){
 		sprintf(interp->result,
				"Insufficient arguments in kernel array");
		return TCL_ERROR;
	    }	
	    number--; 
	    for(;number>=0;number--)
		data[number+1] = atof(split[number]);
	}
	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
	imageP = convertImage_Queue(imageP,required_type,num_type);

        resultImage = single_filter(imageP,atof(argv[3]),atof(argv[4]),
			atoi(argv[5]), atoi(argv[6]), atof(argv[7]), 
			atof(argv[8]),atoi(argv[9]),data,atoi(argv[11]));
	if(resultImage == NULL)
	    return TCL_ERROR;
	S_ImageAddToHash(interp,resultImage,NULL);
	free(split);
  	free(data);
     	return TCL_OK;
}


S_TempFilename(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
      if (argc != 3) {
               sprintf(interp->result, "Usage: cvip temp_filename <filename>");
               return TCL_ERROR;
       }
 
       sprintf(interp->result,"%s/%s", temp_dir, argv[2]);
       return TCL_OK;
}



static int
S_ImageRename(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
   Image * imageP = (NULL);
	char file_name[150], old_path[150], new_path[150];
	CMPR_FORMAT cformat;
	#ifndef WIN32
	struct stat buf;
	#else
	struct _stat buf;
	#endif

	if (argc != 4) {
	    sprintf(interp->result, "Usage: cvip rename <old-name> <new-name>");
	    return TCL_ERROR;
	}	
	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
	
	
	if(!S_ImageDeleteFromHash(interp,argv[2])) return TCL_ERROR;


	(void) S_ImageAddToHash(interp, imageP, argv[3]);
	
	/* rename the compressed file in the temporary directory as well */
	for (cformat = Btc; cformat < TOP; cformat++) {
		/* old compressed file name */
		sprintf(file_name, "%s__%s", argv[2], Cmpr_Name[cformat]);
		sprintf(old_path, "%s/%s", temp_dir, file_name);
		#ifndef WIN32
		if (stat(old_path, &buf) == 0) {
		#else
		if (stat(old_path, &buf) == 0) {
		#endif
			sprintf(file_name, "%s__%s", argv[3], Cmpr_Name[cformat]);
			sprintf(new_path, "%s/%s", temp_dir, file_name);
			rename(old_path, new_path);
			printf("Temporary file :%s \nis renamed to: %s\n", old_path, new_path);
		}
	}

	return TCL_OK;
}

static int
S_ImageDequeue(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
        Image * imageP = (NULL);

	if (argc != 3) {
	    sprintf(interp->result, "Usage: cvip dq <image-name>");
	    return TCL_ERROR;
	}	
	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
	
	
	if(!S_ImageDeleteFromHash(interp,argv[2])) return TCL_ERROR;
	
	delete_Image(imageP);

	return TCL_OK;

}

static int
S_ImageWrite(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image *imageP = (NULL);
	IMAGE_FORMAT   format;

	if (argc != 3 && argc != 4 && argc != 5) {
		sprintf(interp->result, "Usage: cvip write <image> [format] [New name]");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	format = imageP->image_format;

	if(argc == 3 || strcasecmp(argv[3], "DEFAULT") == 0){
		if(imageP->bandP[0]->data_type != CVIP_BYTE) {
			if(format == ZON || format == ZON2 ) {
				imageP = remap_Image(imageP, CVIP_BYTE, 0, 255);
			} else {
				format = VIP; 
				imageP->image_format = VIP;
			}
		}
	}
	else{
		if(strcasecmp(argv[3],"PBM") == 0)
			format = PBM;
		else if(strcasecmp(argv[3],"PGM") == 0)
			format = PGM;
		else if(strcasecmp(argv[3],"PPM") == 0)
			format = PPM;
		else if(strcasecmp(argv[3],"EPS") == 0)
			format = EPS;
		else if(strcasecmp(argv[3],"TIF") == 0)
			format = TIF;
		else if(strcasecmp(argv[3],"GIF") == 0)
			format = GIF;
		else if(strcasecmp(argv[3],"RAS") == 0)
			format = RAS;
		else if(strcasecmp(argv[3],"ITX") == 0)
			format = ITX;
		else if(strcasecmp(argv[3],"IRIS") == 0)
			format = IRIS;
		else if(strcasecmp(argv[3],"CCC") == 0)
			format = CCC;
		else if(strcasecmp(argv[3],"BIN") == 0)
			format = BIN;
		else if(strcasecmp(argv[3],"JPG") == 0)
			format = JPG;
		else if(strcasecmp(argv[3],"VIP") == 0)
			format = VIP;
	}
	if (argc <= 4)
		write_Image(imageP,argv[2],CVIP_YES,CVIP_NO,format,1);
	else 
		write_Image(imageP,argv[4],CVIP_YES,CVIP_NO,format,1);

	Tcl_SetResult(interp, "Writing image ...", TCL_VOLATILE);

	return TCL_OK;
}

static void draw_histo(Image *inputImage,char *image_name){
 	byte 		**image;
	Image      *map_image,*map_image1=NULL;
        char name1[100];
	char direct[100];
	register int 	i, j,temp;
	unsigned int 	rows, cols, bands,no_of_bands;
	float 		scale = 255.0, **map,**map_i=NULL, max;
	char *software;

   	strcpy(name1,image_name);
	rows 	= getNoOfRows_Image(inputImage);
	cols 	= getNoOfCols_Image(inputImage);
	no_of_bands = getNoOfBands_Image(inputImage);
        map=(float**)malloc(no_of_bands*sizeof(float*)); 

     
        for(bands=0; bands < no_of_bands; bands++){
           map[bands]=(float*)calloc(256,sizeof(float));
           image = getData_Image(inputImage, bands);
           for( i = 0; i < cols; i++ ){
               for( j = 0; j < rows; j++){
                   map[bands][image[j][i]]+=1;
               }
           }
        }
	if(getDataFormat_Image(inputImage) == COMPLEX) {
         map_i=(float**)malloc(no_of_bands*sizeof(float*)); 
         for(bands=0; bands < no_of_bands; bands++){
           map_i[bands]=(float*)calloc(256,sizeof(float));
           image = getImagData_Image(inputImage, bands);
           for( i = 0; i < cols; i++ ){
               for( j = 0; j < rows; j++){
                   map_i[bands][image[j][i]]+=1;
               }
           }

         }
        }
	compare_find_max(map,map_i,no_of_bands,256);
	map_image = draw_picture(map,256,no_of_bands);
	if(map_i != NULL)
	    map_image1 = draw_picture(map_i,256,no_of_bands);
	/*free dynamic memory allocated in this function*/
        for(bands=0; bands < no_of_bands; bands++)
	   free(map[bands]);
	free(map);
	if(map_i != NULL){
           for(bands=0; bands < no_of_bands; bands++)
	       free(map_i[bands]);
	   free(map_i);
	}

	if(map_image1 !=NULL)
	    strcat(image_name,"-REAL");

	view_Image(map_image, image_name);
	delete_Image(map_image);

	if(map_image1 != NULL){
	   view_Image(map_image1, name1);
	   delete_Image(map_image1);
	}
}

static void compare_find_max(float **map, float **map1,int dim, int len){
   int i,j;
   float max;
   max = map[0][0];

   for(j=0;j<dim;j++){
      for(i=0;i<len;i++){
          if(max<map[j][i])
             max=map[j][i];
      }
   }
   if(map1 != NULL){
    for(j=0;j<dim;j++){
      for(i=0;i<len;i++){
          if(max<map[j][i])
             max=map[j][i];
      }
    }
   }

   for(j=0;j<dim;j++){
      for(i=0;i<len;i++){
         map[j][i]/=max;               
      }
   }
   if(map1 != NULL){
    for(j=0;j<dim;j++){
      for(i=0;i<len;i++){
         map1[j][i]/=max;               
      }
    }
   }
}

static Image* draw_picture(float **map,int len,int dim){ 
   Image *map_image;
   int x,y,i,j, bands;
   float max;
   int limit,bound;
   byte **data;

   if(dim==1)
      map_image=new_Image(PGM,GRAY_SCALE,1,len,len,CVIP_BYTE,REAL);
   if(dim==3)
      map_image=new_Image(PPM,RGB,3,len*3,len,CVIP_BYTE,REAL);
   if(map_image==NULL){
      fprintf(stderr,"Fatal Error:cannot create a new image\n");
      return NULL;
   }
   for(bands=0;bands<dim;bands++){
      data=getData_Image(map_image,bands);
      bound=bands*len;
      for(x=0;x<len;x++){
         limit=len-map[bands][x]*len;
         for(y=bound;y<len*(bands+1);y++){
             if((y-bound)>=limit)
                data[y][x]=255;
             else
                data[y][x]=0;
         }
      }
   }
   return map_image;
}

static void view_rgb(Image *resultimage, char *name, char option){
   int band, bandpos, height, width,h,w;
   byte **datar, **datai;

   if(option=='r')
      band=0;
   else if(option=='g')
      band=1;
   else if(option=='b')
      band=2;
   else
      band=3;
   height = resultimage->bandP[0]->rows;
   width = resultimage->bandP[0]->cols;
   for(bandpos=0;bandpos<resultimage->bands;bandpos++){
      datar=(byte**)resultimage->bandP[bandpos]->rptr;
      datai=(byte**)resultimage->bandP[bandpos]->iptr;
      if(bandpos!=band){
         for(h=0;h<height;h++){
   	    for(w=0;w<width;w++){
	       datar[h][w]=0;
	       if(datai!=NULL)
		   datai[h][w]=0;
	    }
	 }	
      }
   }

   view_Image(resultimage,name);
}  

static int
S_ImageDisplay(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP = (NULL);
	int do_delete = 0, band = -1;
	char display_name[150];
	char option;

	if (argc != 3 && argc != 4 && argc != 5) {
		sprintf(interp->result, "Usage: cvip display <image> [rgblh] [band (for l only)]");
		return TCL_ERROR;
	}
	
	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	if(argc == 3){
		view_Image(imageP, argv[2]);
	}
	else{
	    strcpy(display_name, argv[2]);
	    switch(argv[3][0]){
		case 'r':; case 'R':
		    strcat(display_name,"-RED");
		    option = 'r';
		    break;
		case 'g':; case 'G':
		    strcat(display_name,"-GREEN");
		    option = 'g';
		    break;	
		case 'b':; case 'B':
		    strcat(display_name,"-BLUE");
		    option = 'b';
		    break;
		case 'h':; case 'H':
		    strcat(display_name,"-HIST");
		    option = 'h';
		    break;
		case 'l':; case 'L':
		    option = 'l';
			 if (argc == 5) band = atoi(argv[4]);
		    break;
		default:
		    sprintf(interp->result, 
				"Usage: cvip display <image> [rgblh]");
		    return TCL_ERROR;
	   } /* switch */

		if(option == 'l'){
			imageP = duplicate_Image(imageP);
			if ( (imageP = logMap_Image(imageP, band)) == (Image *)NULL) {
				return TCL_ERROR;	
			}
			if (band >= 0)
				sprintf(display_name, "[log_power]%s-band%d", argv[2], band);
			else 
				sprintf(display_name, "[log_power]%s", argv[2]);

			view_Image(imageP, display_name);
			do_delete = 1;
		} else if(option == 'h'){
			if(imageP->bandP[0]->data_type != CVIP_BYTE){
				imageP = (Image*)remap_Image_C(imageP,CVIP_BYTE,0,255);
				Tcl_AppendElement(interp, "Image is remapped to CVIP_BYTE");
				do_delete = 1;
			}
			draw_histo(imageP,display_name);
		} else{ /* rgb option*/
			if(imageP->bandP[0]->data_type != CVIP_BYTE){
				imageP = (Image*)condRemap_Image(imageP,CVIP_BYTE,0,255);
				Tcl_AppendElement(interp, "Image is remapped to CVIP_BYTE");
	    	}	else imageP = duplicate_Image(imageP);
			do_delete = 1;
			view_rgb(imageP,display_name, option);
		}
			
	}   

	if(do_delete == 1)
	    delete_Image(imageP);
	
	return TCL_OK;
}

#include "CVIPmesh.h"

static int
S_SolveBilinear(ClientData clientData, Tcl_Interp *interp,int argc,char **argv)
{
	struct mesh_node tie[2][4];
	float c[8];
	int number, count, i;
	char **split_x, **split_y;
	int pos=0;
	
	if(argc != 6){
		sprintf(interp->result, "Usage: cvip solve_bilinear {X_in[0] X_in[1] X_in[2] X_in[3]} {Y_in[0] Y_in[1] Y_in[2] Y_in[3]} {X_out[0] X_out[1] X_out[2] X_out[3]} {Y_out[0] Y_out[1] Y_out[2] Y_out[3]}");
	    return TCL_ERROR;
	}
	memset(c, 0, sizeof(c));

	for(count=2;count < 6; count++){
	    if(Tcl_SplitList(interp, argv[count], &number, &split_x) 
				== TCL_ERROR){
		sprintf(interp->result,"Error in interpreting");
		return TCL_ERROR;
	    }
	    if(number != 4){
		sprintf(interp->result,"Error in interpreting");
		free(split_x);
		return TCL_ERROR;
	    }
	    for(number=0;number<4;number++)
		tie[(count/2)-1][number].x = atoi(split_x[number]);
	    free(split_x);
	    count++;
	    if(Tcl_SplitList(interp, argv[count], &number, &split_y) 
				== TCL_ERROR){
		sprintf(interp->result,"Error in interpreting");
		return TCL_ERROR;
	    }
	    if(number != 4){
		sprintf(interp->result,"Error in interpreting");
		free(split_y);
		return TCL_ERROR;
	    }
	    for(number=0;number<4;number++)
		tie[(count/2)-1][number].y = atoi(split_y[number]);
	    free(split_y);
	}
	solve_c(tie[0],tie[1],c);
	for(i=0;i<8;i++){
	    sprintf(interp->result+pos,"%f ",c[i]);
	    pos = strlen(interp->result);
	}
	return TCL_OK;
}

static int
S_Keyboard_Mesh(ClientData clientData, Tcl_Interp *interp,int argc,char **argv)
{
	struct mesh *outmesh;
	int cols, rows, x, y,number;
	char **split;
	char *filename;
	Image *cvipImage;

	if(argc != 7){
	    sprintf(interp->result,"cvip keyboard_mesh <mesh_file_name> <col> <row> <data for the col mesh matrix COL by row enclosed in curly brackets> <data for the ROW mesh matrix col by row enclosed in curly brackets>");
	    return TCL_ERROR;
	}

	filename = argv[2];
	cols = atoi(argv[3]);
	rows = atoi(argv[4]);
	if(Tcl_SplitList(interp, argv[5], &number, &split) == TCL_ERROR){
	    sprintf(interp->result,"Error in interpreting");
	    return TCL_ERROR;
	}
	if(number != rows * cols){
	    sprintf(interp->result,"cannot get enough COL data to fill in matrix");
	    return TCL_ERROR;
	}

	outmesh = (struct mesh*)malloc(sizeof(struct mesh));
	outmesh->nodes 
	= (struct mesh_node**)malloc(sizeof(struct mesh_node*)*cols);
	outmesh->width = cols;
	outmesh->height = rows;
	for(x=0;x<cols;x++)
		outmesh->nodes[x]
		= (struct mesh_node*)calloc(rows,sizeof(struct mesh_node));

	for(y=0;y<rows;y++)
	    for(x=0;x<cols;x++)
		outmesh->nodes[x][y].x = atoi(split[y*cols+x]);

	free(split);
	if(Tcl_SplitList(interp, argv[6], &number, &split) == TCL_ERROR){
	    sprintf(interp->result,"Error in interpreting");
	    return TCL_ERROR;
	}
	if(number != rows * cols){
	    sprintf(interp->result,"cannot get enough ROW data to fill in matrix");
	    return TCL_ERROR;
	}
	for(y=0;y<rows;y++)
	    for(x=0;x<cols;x++)
		outmesh->nodes[x][y].y = atoi(split[y*cols+x]);
	
	if((mesh_to_file(outmesh,filename)) == 0)
		return TCL_ERROR;
	cvipImage = create_black(512,512);
	display_mesh(cvipImage,outmesh,filename);
	
	free(split);
	for(x=0;x<cols;x++)
		free(outmesh->nodes[x]);
	free(outmesh->nodes);
	free(outmesh);

	return TCL_OK;

}

static int
S_Create_Mesh_Sine(ClientData clientData, Tcl_Interp *interp,int argc,char **argv)
{
	Image* outputImage;
	struct mesh *outmesh;
	char* mesh_file,*outputfile;
	IMAGE_FORMAT 	format1;
	int cols,rows,x,y,oldx,newx,oldy,newy,width,height;
	width = 511;
	height = 511;

	if(argc != 5){
	    sprintf(interp->result,"cvip create_mesh <matrix row> <matrix col> <mesh_file_name>");
	    return TCL_ERROR;
	}
	cols = atoi(argv[3]);
	rows = atoi(argv[2]);
	outmesh = (struct mesh*)malloc(sizeof(struct mesh));
	outmesh->nodes 
	= (struct mesh_node**)malloc(sizeof(struct mesh_node*)*cols);
	outmesh->width = cols;
	outmesh->height = rows;
	for(x=0;x<cols;x++)
		outmesh->nodes[x]
		= (struct mesh_node*)calloc(rows,sizeof(struct mesh_node));
	oldy = 0;
	for(y=0; y<rows; y++) {
		oldx = 0;
		newy = oldy + abs((int)(25*cos(y)+0.5));
		if(newy>=height)
			newy = height;
		for(x=0; x<cols; x++) {
			newx = oldx + abs((int)(25*sin(x)+0.5));
			if(newx >= width)
				newx = width;
			outmesh->nodes[x][y].x = newx;
			oldx = newx;
			outmesh->nodes[x][y].y = newy;
		}
		oldy = newy;
	}
        mesh_file = argv[4];
	outputImage = create_black(512, 512);
	display_mesh(outputImage,outmesh,mesh_file);
	mesh_to_file(outmesh,mesh_file);

	return TCL_OK;
}

static int
S_Mesh_Warp(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image *outputImage;
	IMAGE_FORMAT format;
	int method,source,direction;
	char *mesh_file,*outputfile;
	struct mesh *inmesh;
	CVIP_BOOLEAN zero_out= CVIP_YES;

        Image * imageP;
	CVIP_TYPE	required_type[5];
	int	num_type;

	if(argc != 6 && argc != 7){
		Tcl_AppendResult(interp,"Usage: cvip warp <image> <mesh_file_name>\n");
		Tcl_AppendResult(interp,
			"<direction (either (1)from irregular mesh to a regular mesh or (2)from regular mesh to an irregular mesh)>\n");
		Tcl_AppendResult(interp,
			"<Gray-value interpolation method(1.Nearest neighbor 2.Bilinear interpolation 3.Neighbor average) >\n");
		Tcl_AppendResult(interp,
			"<zero background (1/0 for yes/no), only significant from irregular to regular, default to yes", "\n", (char *)NULL);
		return TCL_ERROR;
	}

	required_type[0] = CVIP_BYTE;
	num_type = 1;
	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
	imageP = convertImage_Queue(imageP,required_type,num_type);

	mesh_file = argv[3];
	inmesh = file_to_mesh(mesh_file);
	if(inmesh == NULL){
		sprintf(interp->result,"Error in reading mesh file");
		delete_Image(imageP);
		return TCL_ERROR;
	}
	direction = atoi(argv[4]);
	method = atoi(argv[5]);
	if (argc == 7 &&  atoi(argv[6]) == 0) zero_out = CVIP_NO;;

	if(direction == 1){
		outputImage = mesh_warping(imageP, inmesh, method);
	}
	else if(direction == 2){
		outputImage = mesh_warping_ri(imageP, inmesh, method, zero_out);
	}
	
	if(outputImage == NULL) return TCL_ERROR;

	S_ImageAddToHash(interp,outputImage,NULL);

	return TCL_OK;
}

static int
S_ImageNotch(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
        Image * imageP = (NULL),*resultImage;
	char **split;
	int number, count;
	CVIP_TYPE	required_type[5];
	int	num_type;
	NOTCH_ZONE  *zone;

	required_type[0] = CVIP_FLOAT;
	num_type = 1;

	if (argc < 4) {
	    sprintf(interp->result, "Usage: cvip notch <image> {<row> <col> <size>}...");
	    return TCL_ERROR;
	}
	count = argc-3;
	zone = (NOTCH_ZONE*)malloc(sizeof(NOTCH_ZONE)*count);
	argc--;
	while(argc >= 3){
	    if(Tcl_SplitList(interp, argv[argc], &number, &split)
		 == TCL_ERROR){
		sprintf(interp->result,"Error in interpreting kernel array");
		return TCL_ERROR;
	    }
	    if(number != 3){
		sprintf(interp->result,"Error in interpreting kernel array");
		free(zone);
		free(split);
		return TCL_ERROR;
	    } 
	    else{
		zone[argc-3].y = atoi(split[0]);
		zone[argc-3].x = atoi(split[1]);
		zone[argc-3].radius = atoi(split[2]);
		zone[argc-3].radius /=2;
		free(split);
	    }
	    argc--;
	}
	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
	imageP = convertImage_Queue(imageP,required_type,num_type);

        resultImage = notch(imageP, NULL, zone, count, CVIP_NO);
	if(resultImage == NULL)
	    return TCL_ERROR;
	S_ImageAddToHash(interp,resultImage,NULL);

     	return TCL_OK;
}

static int
S_ImageEXTRACT_BAND(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
        Image * imageP = (NULL),*resultImage;
 
        if (argc != 4) {
            sprintf(interp->result, "Usage: cvip extract_band <image> <bandno>");
            return TCL_ERROR;
        }
        if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
 
	     imageP=duplicate_Image(imageP);
        resultImage = extract_band(imageP, atoi(argv[3]));
        if(resultImage == NULL)
            return TCL_ERROR;
        S_ImageAddToHash(interp,resultImage,NULL);
 
        return TCL_OK;
}

static int
S_ImageASSEMBLE_BANDS(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
        Image **imageP = (NULL),*resultImage;
        int count;
 
        if (argc < 4) {
            sprintf(interp->result, "Usage: cvip assemble_bands <image1> <image2>...<imagen>");
            return TCL_ERROR;
        }

	/*
	 * set up double pointer from the passed images.  If images
	 * are of different data types, they will all be converted to
	 * the type image 0 has.
	 */
	imageP=(Image **)malloc(sizeof(Image *)*(argc-2));
	for(count=2; count<argc; count++) {
           if(!(imageP[count-2] = S_ImageGetFromHash(interp, argv[count]))) return TCL_ERROR;
	   if(count>2) 
              imageP[count-2] = convertImage_Queue(imageP[count-2],&imageP[0]->bandP[0]->data_type,1);
	}
 
        resultImage = assemble_bands(imageP, argc-2);
        if(resultImage == NULL)
            return TCL_ERROR;
        S_ImageAddToHash(interp,resultImage,NULL);
 
        return TCL_OK;
}


static int
S_ImageObjects(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	char *list[] = {"red", "green", "blue", NULL};
	char **item = list;
	CVIP_TYPE	required_type[5];
	int	num_type;

   USE(clientData);
   USE(interp);
   USE(argc);
   USE(argv);

   required_type[0] = CVIP_BYTE;
   num_type = 1;

   if (argc != 3) {
      sprintf(interp->result, "Usage: cvip objects <image>");
      return TCL_ERROR;
   }

   Tcl_ResetResult(interp);

   do {
         Tcl_AppendElement(interp, *item);
	 Tcl_AppendElement(interp, "\t\t");
   } while ( *(++item) != NULL );

   return TCL_OK;
}


static int
S_ImageHisteq(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP;
	CVIP_TYPE	required_type[5];
	int	num_type;
	int band = 0;

	required_type[0] = CVIP_BYTE;
	num_type = 1;

	if (argc != 3 && argc != 4) {
		sprintf(interp->result, "Usage: cvip histeq <image> [0=r,1=g, 2=b]");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	imageP = convertImage_Queue(imageP,required_type,num_type);
	if(argc == 4)
	    band = atoi(argv[3]);
	imageP = histeq(imageP,band);

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}

static int
S_ImageLOCAL_HISTEQ(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image *imageP;
	CVIP_TYPE required_type[5];
	int num_type;

	required_type[0] = CVIP_BYTE;
	num_type = 1;

	if (argc !=5) {
		sprintf(interp->result, "Usage: cvip local_histeq <image> [0=r,1=g, 2=b] <blocksize>");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
 
        imageP = convertImage_Queue(imageP,required_type,num_type);
        imageP = local_histeq(imageP,atoi(argv[4]), atoi(argv[3]));
 
        (void) S_ImageAddToHash(interp, imageP, (char *)NULL);
 
        return TCL_OK;
}


extern Image * label( const Image *imageP );

static int
S_ImageLabel(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP;
	CVIP_TYPE	required_type[5];
	int	num_type;

	required_type[0] = CVIP_BYTE;
	num_type = 1;

	if (argc != 3) {
		sprintf(interp->result, "Usage: cvip label <image>");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	imageP = convertImage_Queue(imageP,required_type,num_type);

	imageP = label(imageP);
	if(imageP == NULL) {
		sprintf(interp->result, "Too many objects on labeling. Be sure the segmented image you specifed is correct" );
		return TCL_ERROR;
	}

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}

static int
S_ImageHomomorphic(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP;

	if (argc != 6) {
		sprintf(interp->result, "Usage: cvip homomorphic <image> <upper frequency gain> <lower frequency gain> <cutoff frequency>");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	imageP = duplicate_Image(imageP);

	imageP = homomorphic(imageP,atof(argv[3]), atof(argv[4]),
			 atoi(argv[5]));

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}



static int
S_ImageZoom(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP;
	CVIP_TYPE	required_type[5];
	int	num_type;

	required_type[0] = CVIP_BYTE;
	num_type = 1;

	if (argc != 9) {
		sprintf(interp->result, "Usage: cvip zoom <image> <quadrant(1-6)> <zoom_factor> <col offset> <row offset> <width> <height>\nFor argument(s) you don't need, pass -1 ");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	imageP = convertImage_Queue(imageP,required_type,num_type);
	imageP = zoom(imageP,atoi(argv[3]),atoi(argv[5]),
			atoi(argv[6]),atoi(argv[7]),atoi(argv[8]), 
			atof(argv[4]));
	if(imageP == NULL)
		return TCL_ERROR;
	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}


static int
S_ImageRotate(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP;
	CVIP_TYPE	required_type[5];
	int	num_type;

	required_type[0] = CVIP_BYTE;
	num_type = 1;

	if (argc != 4) {
		sprintf(interp->result, "Usage: cvip rotate <image> <degree(1-360)>");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	/*imageP = convertImage_Queue(imageP,required_type,num_type); */
	imageP = rotate(imageP,atof(argv[3]));
	if(imageP == NULL)
		return TCL_ERROR;
	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}
/*
static int
S_rcCrop(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP;
	int choice;

	if (argc != 6) {
		sprintf(interp->result, "Usage: cvip rc_crop <image> <row/col(1/0)>  <first line> <last line>"); 
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
	imageP=duplicate_Image(imageP);
	choice = atoi(argv[3]);
	if(choice)
	    imageP = crop(imageP,atoi(argv[4]), 0, (atoi(argv[5])-atoi(argv[4])+1), imageP->bandP[0]->cols);
	else
	    imageP = crop(imageP,0, atoi(argv[4]), imageP->bandP[0]->rows, (atoi(argv[5])-atoi(argv[4])+1));

	if(imageP == NULL)
		return TCL_ERROR;
	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
} */

static int
S_ImageCrop(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP;

        if (argc != 7) {
                sprintf(interp->result, "Usage: cvip crop <image> <row coordi>  <col coordi>  <row size> <col size>");
                return TCL_ERROR;
        }

        if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

        imageP=duplicate_Image(imageP);
 	imageP = crop(imageP, atoi(argv[3]), atoi(argv[4]), atoi(argv[5]), atoi(argv[6]));

        if(imageP == NULL)  return TCL_ERROR; 
  
        (void) S_ImageAddToHash(interp, imageP, (char *)NULL);

        return TCL_OK;
}

static int
S_ImageShrink(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP;
	CVIP_TYPE	required_type[5];
	int	num_type;

	required_type[0] = CVIP_BYTE;
	num_type = 1;

	if (argc != 4) {
		sprintf(interp->result, "Usage: cvip scale <image> <factor(0.1-1.0)>");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	imageP = convertImage_Queue(imageP,required_type,num_type);
	imageP = shrink(imageP,atof(argv[3]));
	if(imageP == NULL)
		return TCL_ERROR;
	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}

static int
S_ImageBilinear(ClientData clientData, Tcl_Interp *interp, int argc, char **argv) {
	Image *imageP, *copyP;

	if(argc!=4) {
	       sprintf(interp->result, "Usage: cvip bilinear_interp <image> <factor ( > 1 enlarges, < 1 shrinks)>");
                return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	copyP = duplicate_Image(imageP);
	copyP = bilinear_interp(copyP, atof(argv[3]));
	if(copyP == NULL)
                return TCL_ERROR;
        (void) S_ImageAddToHash(interp, copyP, (char *)NULL);
 
        return TCL_OK;
}

static int
S_ImageBTC(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP;
	CVIP_TYPE	required_type[5];
	int	num_type, block_size = 8;
	CVIP_BOOLEAN	remap;
	char *filename, *result_name; 
	filename = (char *)malloc(sizeof(char) * 150);
	result_name = (char *)malloc(sizeof(char) * 150); 

	if (filename == (char *)NULL || result_name == (char *)NULL) {
		sprintf(interp->result, "Memory allocation error");
		return TCL_ERROR;
	} 

	required_type[0] = CVIP_BYTE;
	num_type = 1;

	if (argc != 3 && argc != 4) {
		sprintf(interp->result, "Usage: cvip btc <image> block_size");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	imageP = convertImage_Queue(imageP,required_type,num_type);
	getTempFileName(interp, argv[2], Btc, filename, result_name); 
	if (argc == 3) 
		print_CVIP("Use default 16 for blocksize\n");
	else 
		block_size = atoi(argv[3]);
	

	if (btc_compress(imageP, filename, block_size) < 0) {
		 free(filename); free(result_name);
	    return TCL_ERROR;
	}
	if ( (imageP = btc_decompress(filename)) == (Image *)NULL)  {
		 free(filename); free(result_name);
	    return TCL_ERROR;
	}

	(void) S_ImageAddToHash(interp, imageP, (char *)result_name);
	free(filename); free(result_name);

	return TCL_OK;
}

static int
S_ImageBTC2(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP;
	CVIP_TYPE	required_type[5];
	int	num_type;
	int	remap;
	char *filename, *result_name; 
	filename = (char *)malloc(sizeof(char) * 150);
	result_name = (char *)malloc(sizeof(char) * 150); 

	if (filename == (char *)NULL || result_name == (char *)NULL) {
		sprintf(interp->result, "Memory allocation error");
		return TCL_ERROR;
	} 

	required_type[0] = CVIP_BYTE;
	num_type = 1;

	if (argc != 4) {
		sprintf(interp->result, "Usage: cvip btc2 <image> <block_size>");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	imageP = convertImage_Queue(imageP,required_type,num_type);
	getTempFileName(interp, argv[2], Btc2, filename, result_name); 

	if (btc2_compress(imageP, filename, atoi(argv[3])) < 0) {
		 free(filename); free(result_name);
	    return TCL_ERROR;
	}
	if ( (imageP = btc2_decompress(filename)) == (Image *)NULL) {
		 free(filename); free(result_name);
	    return TCL_ERROR;
	}

	(void) S_ImageAddToHash(interp, imageP, (char *)result_name);
	free(filename); free(result_name);
	return TCL_OK;
}


static int
S_ImageBTC3(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP;
	CVIP_TYPE	required_type[5];
	int	num_type;
	int	remap;
	char *filename, *result_name; 
	filename = (char *)malloc(sizeof(char) * 150);
	result_name = (char *)malloc(sizeof(char) * 150); 

	if (filename == (char *)NULL || result_name == (char *)NULL) {
		sprintf(interp->result, "Memory allocation error");
		return TCL_ERROR;
	} 

	required_type[0] = CVIP_BYTE;
	num_type = 1;

	if (argc != 4) {
		sprintf(interp->result, "Usage: cvip btc3 <image> <block_size>");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	imageP = convertImage_Queue(imageP,required_type,num_type);
	getTempFileName(interp, argv[2], Btc3, filename, result_name); 

	if ( btc3_compress(imageP, filename, atoi(argv[3])) < 0) {
		 free(filename); free(result_name);
	    return TCL_ERROR;
	}
	if ( (imageP = btc3_decompress(filename)) == (Image *)NULL){
		 free(filename); free(result_name);
	    return TCL_ERROR;
	}	
	(void) S_ImageAddToHash(interp, imageP, (char *)result_name);
	free(filename); free(result_name);

	return TCL_OK;
}

/*
extern Image *DPCM_decompression(char *filename);
extern int DPCM_compress(Image *inputImage, char *filename,float ratio,int num,int c, int d, int or, float dnum);
*/
static int
S_ImageDPCM(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP, *tempImage;
	CVIP_TYPE	required_type[5];
	int	num_type;

	char *filename, *result_name; 
	filename = (char *)malloc(sizeof(char) * 150);
	result_name = (char *)malloc(sizeof(char) * 150); 

	if (filename == (char *)NULL || result_name == (char *)NULL) {
		sprintf(interp->result, "Memory allocation error");
		return TCL_ERROR;
	} 

	required_type[0] = CVIP_BYTE;
	num_type = 1;

	if (argc != 9) {
		sprintf(interp->result, "Usage: cvip dpc <image> <ratio(0~1)> <do_clipping(1/0)> <number_bits(1-8)> <vertical_scan(1/0)> <original/reconstructed/llyod_max(1/2/3)> <deviation>");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	imageP = convertImage_Queue(imageP,required_type,num_type);
	getTempFileName(interp, argv[2], Dpcm, filename, result_name); 

	if(dpc_compress(imageP,filename,atof(argv[3]),atoi(argv[5]),
			atoi(argv[4]),atoi(argv[6]),atoi(argv[7]), atof(argv[8]))== -1) {
		free(filename); free(result_name);
		return TCL_ERROR;
	}
   if ( (imageP = dpc_decompress(filename)) == (Image *)NULL) {
		free(filename); free(result_name);
		return TCL_ERROR;
	}
	(void) S_ImageAddToHash(interp, imageP, (char *)result_name);
	free(filename); free(result_name);

	return TCL_OK;
}

static int
S_ImageZVL(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP;
	CVIP_TYPE	required_type[5];
	int	num_type;
	
	char *filename, *result_name; 
	filename = (char *)malloc(sizeof(char) * 150);
	result_name = (char *)malloc(sizeof(char) * 150); 

	if (filename == (char *)NULL || result_name == (char *)NULL) {
		sprintf(interp->result, "Memory allocation error");
		return TCL_ERROR;
	} 

	required_type[0] = CVIP_BYTE;
	num_type = 1;

	if (argc != 3) {
		sprintf(interp->result, "Usage: cvip zvl <image>");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	imageP = convertImage_Queue(imageP,required_type,num_type);
	getTempFileName(interp, argv[2], Zvl, filename, result_name); 

   if (zvl_compress(imageP, filename) < 0)  {
		free(filename); free(result_name);
	   return TCL_ERROR;
	}
   if ( (imageP=zvl_decompress(filename)) == (Image *)NULL) {
		free(filename); free(result_name);
	   return TCL_ERROR;
	}
	(void) S_ImageAddToHash(interp, imageP, (char *)result_name);
	free(filename); free(result_name);

	return TCL_OK;
}

static int
S_ImageGray_RLC(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{ 
	Image * imageP;
	CVIP_TYPE	required_type[5];
	int	num_type;
	int 	remap;
	char *fname;
        int  bands, cols, rows;

	char *filename, *result_name; 
	filename = (char *)malloc(sizeof(char) * 150);
	result_name = (char *)malloc(sizeof(char) * 150); 

	if (filename == (char *)NULL || result_name == (char *)NULL) {
		sprintf(interp->result, "Memory allocation error");
		return TCL_ERROR;
	} 

	required_type[0] = CVIP_BYTE;
	num_type = 1;

	if (argc != 4) {
		sprintf(interp->result, "Usage: cvip gray_rlc <image> <window_length>");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	imageP = convertImage_Queue(imageP,required_type,num_type);
	getTempFileName(interp, argv[2], Glr, filename, result_name); 

   if (glr_compress(imageP, filename, atoi(argv[3])) < 0)  {
		free(filename); free(result_name);
	    return TCL_ERROR;
	}
   if ( (imageP=glr_decompress(filename)) == (Image *)NULL) {
		free(filename); free(result_name);
	    return TCL_ERROR;
	}
	(void) S_ImageAddToHash(interp, imageP, (char *)result_name);
	free(filename); free(result_name);

	return TCL_OK;
}

static int
S_ImageBitplane_RLC(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP;
	CVIP_TYPE	required_type[5];
	int	num_type, i, word;
	byte sect = 0;

	char *filename, *result_name; 
	filename = (char *)malloc(sizeof(char) * 150);
	result_name = (char *)malloc(sizeof(char) * 150); 

	if (filename == (char *)NULL || result_name == (char *)NULL) {
		sprintf(interp->result, "Memory allocation error");
		return TCL_ERROR;
	} 

	required_type[0] = CVIP_BYTE;
	num_type = 1;

	if (argc != 4) {
		sprintf(interp->result, "Usage: cvip bitplane_rlc <image> <retained_bitplane(s) (enclosed in curly brackets)>");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	imageP = convertImage_Queue(imageP,required_type,num_type);
	getTempFileName(interp, argv[2], Brc, filename, result_name); 


	word = '0';
	for(i=0;i<8;i++,word++) 
		if(strchr(argv[3],word))
			sect |= (1<<i);

	if ( bit_compress(imageP, filename, sect) < 0) {
		free(filename); free(result_name);
		return TCL_ERROR;
	}
	if ( (imageP=bit_decompress(filename)) == (Image *) NULL) {
		free(filename); free(result_name);
		return TCL_ERROR;
	}
	(void) S_ImageAddToHash(interp, imageP, (char *)result_name);
	free(filename); free(result_name);

	return TCL_OK;
}

static int
S_ImageHuffman(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP;
	CVIP_TYPE	required_type[5];
	int	num_type;

	char *filename, *result_name; 
	filename = (char *)malloc(sizeof(char) * 150);
	result_name = (char *)malloc(sizeof(char) * 150); 

	if (filename == (char *)NULL || result_name == (char *)NULL) {
		sprintf(interp->result, "Memory allocation error");
		return TCL_ERROR;
	} 

	required_type[0] = CVIP_BYTE;
	num_type = 1;

	if (argc != 3) {
		sprintf(interp->result, "Usage: cvip huffman <image>");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	imageP = convertImage_Queue(imageP,required_type,num_type);

	getTempFileName(interp, argv[2], Huf, filename, result_name); 

	if (huf_compress(imageP, filename) < 0) {
		free(filename); free(result_name);
		return TCL_ERROR;
	}
	else if ( (imageP = huf_decompress(filename)) == (Image *) NULL) {
		free(filename); free(result_name);
		return TCL_ERROR;
	}
	(void) S_ImageAddToHash(interp, imageP, (char *)result_name);
	free(filename); free(result_name);

	return TCL_OK;
}

static int
S_ImageJPEG(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP;
	CVIP_TYPE	required_type[5];
	int	num_type, quality, smooth, grayscale;
	char *filename, *result_name;
	char *quantfile, *quant_name;


	required_type[0] = CVIP_BYTE;
	num_type = 1;

	if (argc != 7) {
		sprintf(interp->result, "Usage: cvip jpeg <image> <quality(integer)> <smooth(1/0)> <grayscale(1/0)> <quantfile>");
		return TCL_ERROR;
	}
	
	quality = atoi(argv[3]);
	if ( quality > 100 || quality < 0) {
		sprintf(interp->result, "quality(integer) should be between 0 and 100");
		return TCL_ERROR;
	}
	smooth = atoi(argv[4]);
	if ( smooth != 0 && smooth != 1) {
		sprintf(interp->result, "smooth can only be 0 or 1");
		return TCL_ERROR;
	}
	grayscale = atoi(argv[5]);
	if ( grayscale != 0 && grayscale != 1) {
		sprintf(interp->result, "grayscale can only be 0 or 1");
		return TCL_ERROR;
	}	

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
	
	imageP = convertImage_Queue(imageP,required_type,num_type);

        filename = (char *)malloc(sizeof(char) * 150);
        result_name = (char *)malloc(sizeof(char) * 150);
        quantfile = (char *)malloc(sizeof(char) * 150);
        quant_name = (char *)malloc(sizeof(char) * 150);

	getTempFileName(interp, argv[2], Jpg, filename, result_name);
	/*getTempFileName(interp, argv[6], Jpg, quantfile, quant_name);*/
 
	if(jpg_compress(imageP,filename,quality,grayscale,CVIP_NO,smooth,CVIP_YES,argv[6]) <0) {
		free(filename); free(result_name);
		/*free(quantfile);*/
		free(quant_name);
		return TCL_ERROR;
	} else if (!(imageP=jpg_decompress(filename,256,CVIP_NO,CVIP_NO,CVIP_NO,CVIP_YES))) {
		free(filename); free(result_name);
		/*free(quantfile); */ free(quant_name);
		return TCL_ERROR;
	}
	(void) S_ImageAddToHash(interp, imageP, (char *)result_name);

	free(filename); free(result_name);
	/*free(quantfile); */ free(quant_name);
	return TCL_OK; 
}

static int
S_Imagerms_error(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP, *imageP1;
        float *error;
        int i;
	char temp[50];
	if (argc != 4) {
		sprintf(interp->result, "Usage: cvip rms_error <image1> <image2>");
		return TCL_ERROR;
	}
	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
	if(!(imageP1 = S_ImageGetFromHash(interp, argv[3]))) return TCL_ERROR;
	imageP = duplicate_Image(imageP);
	imageP1 = duplicate_Image(imageP1);
	
	error=rms_error(imageP, imageP1);
   if(error==NULL) return(TCL_ERROR);
	sprintf(interp->result, "rms_error:\n==============\n");
   for(i=0; i<imageP->bands; i++) {
	  sprintf(temp, "band %d: %5.4f\n", i+1, error[i]);
     strcat(interp->result, temp);
	}
   free(error);
   if (imageP) delete_Image(imageP);
   if (imageP1) delete_Image(imageP1);
	return TCL_OK;
}

static int
S_Imagesnr(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP, *imageP1;
        float *error;
        int i;
	char temp[50];

	if (argc != 4) {
		sprintf(interp->result, "Usage: cvip snr <image1> <image2>");
		return TCL_ERROR;
	}
	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
	if(!(imageP1 = S_ImageGetFromHash(interp, argv[3]))) return TCL_ERROR;
	imageP = duplicate_Image(imageP);
	imageP1 = duplicate_Image(imageP1);
	error=snr(imageP, imageP1);
     if(error==NULL) return(TCL_ERROR);
	sprintf(interp->result, "Signal-to-Noise Ratio:\n==============\n");
   for(i=0; i<imageP->bands; i++) {
		if (error[i] >= 0) sprintf(temp, "band %d: %5.4f\n", i+1, error[i]);
		else sprintf(temp, "band %d: SNR=infinite (no error)\n", i+1);
      strcat(interp->result, temp);
	}

   free(error);
   if (imageP1) delete_Image(imageP1);
   if (imageP) delete_Image(imageP);
	return TCL_OK;
}

static int
S_ImageSCTSPLIT(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP;
	CVIP_TYPE	required_type[5];
	int	num_type;
	int 	y_split, x_split;

	required_type[0] = CVIP_BYTE;
	num_type = 1;

	if (argc != 5) {
		sprintf(interp->result, "Usage: cvip sct_split <color_image> <num_color on A axis> <num_color on B axis>");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	if(imageP->bands != 3){
		sprintf(interp->result, "Input image not a color image");
		return TCL_ERROR;
	}
	

	imageP = convertImage_Queue(imageP,required_type,num_type);
	
	y_split = atoi(argv[3]);
	x_split = atoi(argv[4]);

	imageP=sct_split_segment(imageP,y_split, x_split);

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}

static int
S_ImageFuzzyC(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP;
	CVIP_TYPE	required_type[5];
	int	num_type;

	required_type[0] = CVIP_BYTE;
	num_type = 1;

	if (argc != 4) {
		sprintf(interp->result, "Usage: cvip fuzzyc <color_image> <gaussian kernel vriance(0.0-20.0)");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	if(imageP->bands != 3){
		sprintf(interp->result, "Input image not a color image");
		return TCL_ERROR;
	}
	

	imageP = convertImage_Queue(imageP,required_type,num_type);

	if ( (imageP = fuzzyc_segment(imageP,atof(argv[3]))) == (Image *)NULL)
	return TCL_ERROR;
	
	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}


static int
S_ImagePCT(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP;
	CVIP_TYPE	required_type[5];
	int	num_type;

	required_type[0] = CVIP_FLOAT;
	num_type = 1;

	if (argc != 4) {
		sprintf(interp->result, "Usage: cvip pct <color_image> <forward/inverse(1/2)>");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	if(imageP->bands != 3){
		sprintf(interp->result, "Input image not a color image");
		return TCL_ERROR;
	}

	imageP = convertImage_Queue(imageP,required_type,num_type);

	if( (imageP=pct_color(imageP,CVIP_NO,NULL, atoi(argv[3])))==(Image *)NULL)
		return TCL_ERROR;
	
	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}

static int
S_ImagePctMedian(ClientData clientData, Tcl_Interp *interp,
	int argc, char **argv)
{
	Image * imageP;

	if (argc != 4) {
		sprintf(interp->result,
			"Usage: cvip pct_median <color_image> <number of colors>");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	if(imageP->bands != 3){
		sprintf(interp->result, "Input image not a color image");
		return TCL_ERROR;
	}

	imageP=duplicate_Image(imageP);
	if( (imageP=pct_median_segment(imageP,(unsigned)atoi(argv[3])))
		== (Image *)NULL) {
		sprintf(interp->result, "Pct_Median_cut error.");
		return TCL_ERROR;
	}
	
	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}

static int
S_ImageLuminance(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP;
	CVIP_TYPE	required_type[5];
	int	num_type;

	required_type[0] = CVIP_BYTE;
	num_type = 1;

	if (argc != 3) {
		sprintf(interp->result, "Usage: cvip luminance <color_image>");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	if(imageP->bands != 3){
		sprintf(interp->result, "Input image not a color image");
		return TCL_ERROR;
	}

	imageP = convertImage_Queue(imageP,required_type,num_type);

	imageP = luminance_Image(imageP);

	if(imageP == NULL)
		return TCL_ERROR;
	else	
		(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}


extern Image * lum_average(Image * );

static int
S_Imagelum_average(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
        Image * imageP;

        if (argc != 3) {
                sprintf(interp->result, "Usage: cvip lum_average <grey or color image>");
                return TCL_ERROR;
        }

        if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

        imageP = lum_average(imageP);

        if(imageP == NULL) return TCL_ERROR;
        else               (void) S_ImageAddToHash(interp, imageP, (char *)NULL);
 
        return TCL_OK;
}


static int
S_Imagefrequency_pseudocolor(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
        Image * imageP;

        if (argc != 8) {
                sprintf(interp->result, "Usage: cvip pseudocol_freq <grey image> <lowpass cutoff> <bandpass upper cutoff> <lp color(0,1,2=R,G,B)> <bp color> <hp color>");
                return TCL_ERROR;
        }

        if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
	if (imageP->bands > 1) { printf("not a gray image\n"); return TCL_ERROR;}

	if (*(argv[5])>50||*(argv[5])<48||*(argv[6])>50||*(argv[6])<48||*(argv[7])>50||*(argv[7])<48)
	  { sprintf(interp->result, "<lp color(0,1,2=RGB)> or <bp color> or <hp color> can only be 0 or 1 or 2");  return TCL_ERROR;} 

		  imageP = duplicate_Image(imageP);
        imageP = pseudocol_freq(imageP, atoi(argv[3]), atoi(argv[4]),
			atoi(argv[5]), atoi(argv[6]),atoi(argv[7]));

        if(imageP == NULL) return TCL_ERROR;
        else  (void) S_ImageAddToHash(interp, imageP, (char *)NULL);

        return TCL_OK;
}


static int
S_Imagegray_binary(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
        Image * imageP;

        if (argc != 4) {
                sprintf(interp->result, "Usage: cvip gray_binary <grey or color image> <direction(1/0)>");
                return TCL_ERROR;
        }

        if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

        imageP = (Image *)gray_binary(imageP, atoi(argv[3]));

        if(imageP == NULL) return TCL_ERROR;
        else               (void) S_ImageAddToHash(interp, imageP, (char *)NULL);

        return TCL_OK;
}


static int
S_Imagemmse_filter(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
        Image * imageP;

        if (argc != 5) {
                sprintf(interp->result, "Usage: cvip mmse_filter <grey or color image> <noise variance> <kernel size(odd number)>");
                return TCL_ERROR;
        }

        if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

        imageP = mmse_filter(imageP, atof(argv[3]), atoi(argv[4]));

        if(imageP == NULL) return TCL_ERROR;
        else               (void) S_ImageAddToHash(interp, imageP, (char *)NULL);

        return TCL_OK;
}


static int
S_ImageIGS(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
        Image * imageP;
 
        if (argc != 4) {
                sprintf(interp->result, "Usage: cvip igs <grey or color image> <gray level(2,4,8,16,32,64,128,256)>");
                return TCL_ERROR;
        }
 
        if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
 
        imageP = igs_segment(imageP, atoi(argv[3]));
 
        if(imageP == NULL) return TCL_ERROR;
        else               (void) S_ImageAddToHash(interp, imageP, (char *)NULL);
 
        return TCL_OK;
}


static int
S_Imageadaptive_contrast(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
        Image * imageP;

        if (argc != 8) {
                sprintf(interp->result, "Usage: cvip adaptive_contrast <grey or color image> <k1, local gain factor multiplier> <k2, local mean multiplier> <kernel size(odd number)> <local gain factor Min> <local gain factor Max>");
                return TCL_ERROR;
        }

        if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

        imageP = adaptive_contrast_filter(imageP, atof(argv[3]), atof(argv[4]), atoi(argv[5]), atof(argv[6]), atof(argv[7]));

        if(imageP == NULL) return TCL_ERROR;
        else               (void) S_ImageAddToHash(interp, imageP, (char *)NULL);

        return TCL_OK;
}


static int
S_Imagezonal(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
   Image * imageP, *tempImage;
   CVIP_TYPE    required_type[5];
   int     num_type;

	char *filename, *result_name; 
	filename = (char *)malloc(sizeof(char) * 150);
	result_name = (char *)malloc(sizeof(char) * 150); 

	if (filename == (char *)NULL || result_name == (char *)NULL) {
		sprintf(interp->result, "Memory allocation error");
		return TCL_ERROR;
	} 

        required_type[0] = CVIP_BYTE;
        num_type = 1;

        if (argc != 7) {
           Tcl_AppendResult(interp, "Usage: cvip zonal <gray or color image>",
			" <block size(power of 2)> <transform type(1-4)> <kernel type(1-3)>",
			" <compression ratio>\nwhere\ttransform:",
			" 1:FFT 2:DCT 3:Walsh 4:Hadamard\n\t",
			" kernel type: 1:triangle 2:square 3:circle\n\t", 
			"compression ratio is from 1.0(min) to ",
			"(block_size*block_size/4)(max) for all kinds of transform", "\n",
			(char*)NULL);
         return TCL_ERROR;
        }

   if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
	getTempFileName(interp, argv[2], Zon, filename, result_name); 

   if(zon_compress(imageP, filename,
		 	atoi(argv[3]),atoi(argv[4]),atoi(argv[5]), atof(argv[6])) < 0) {
		free(filename); free(result_name);
      return TCL_ERROR;
	}
   if ( (imageP = zon_decompress(filename)) == (Image *)NULL) {
		free(filename); free(result_name);
      return TCL_ERROR;
	}

   (void) S_ImageAddToHash(interp, imageP, (char *)result_name);
	free(filename); free(result_name);
   return TCL_OK;
}


static int
S_Imagezonal2(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
   Image * imageP, *tempImage;
   CVIP_TYPE    required_type[5];
   int     num_type;

	char *filename, *result_name; 
	filename = (char *)malloc(sizeof(char) * 150);
	result_name = (char *)malloc(sizeof(char) * 150); 

	if (filename == (char *)NULL || result_name == (char *)NULL) {
		sprintf(interp->result, "Memory allocation error");
		return TCL_ERROR;
	} 

        required_type[0] = CVIP_BYTE;
        num_type = 1;

        if (argc != 7) {
           Tcl_AppendResult(interp, "Usage: cvip zonal2 <gray or color image>",
			  " <block size(power of 2)> <transform type(1-4)>",
			  " <kernel type(1-3)> <compression ratio>\nwhere\t", 
			  " transform: 1:FFT 2:DCT 3:Walsh 4:Hadamard\n\t",
			  "kernel type: 1:triangle 2:square 3:circle\n\t",
			  "compression ratio is from 1.0(min) to (block_size*block_size/4)(max)",
			  " for all kinds of transform", "\n", (char *)NULL);
                return TCL_ERROR;
        }

   if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
	getTempFileName(interp, argv[2], Zon2, filename, result_name); 

   if(zon2_compress(imageP, filename, atoi(argv[3]),atoi(argv[4]),
			atoi(argv[5]), atof(argv[6])) < 0) {
		free(filename); free(result_name);
      return TCL_ERROR;
	}
   if ( (imageP = zon2_decompress(filename)) == (Image *)NULL)  {
		free(filename); free(result_name);
      return TCL_ERROR;
	}
   (void) S_ImageAddToHash(interp, imageP, (char *)result_name);
	free(filename); free(result_name);
   return TCL_OK;
}


static int
S_ImageInverseFilter(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP, *imageP1,*resultImage;

	if (argc != 6) {
		sprintf(interp->result, "Usage: cvip inverse_xformfilter <G image> <H image> < F(u,v) = G(u,v)?(1/2)> <cut-off frequency> ");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
	imageP=duplicate_Image(imageP);
	if(!(imageP1 = S_ImageGetFromHash(interp, argv[3]))) return TCL_ERROR;
	imageP1=duplicate_Image(imageP1);

	resultImage=inverse_xformfilter(imageP, imageP1, atoi(argv[4]),atof(argv[5]));
	if(resultImage == NULL)
		return TCL_ERROR;
	else
		(void) S_ImageAddToHash(interp, resultImage, (char *)NULL);

	return TCL_OK;
}

static int
S_ImageSimpleWiener(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
        Image *imageP, *imageP1,*resultImage;


        if (argc != 5) {
                sprintf(interp->result, "Usage: cvip simple_wiener <G image> <H image> <gamma>");
                return TCL_ERROR;
        }

        if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
        if(!(imageP1 = S_ImageGetFromHash(interp, argv[3]))) return TCL_ERROR;
	imageP=duplicate_Image(imageP);
	imageP1=duplicate_Image(imageP1);

	resultImage=simple_wiener(imageP, imageP1, atof(argv[4])); 
        if(resultImage == NULL)
                return TCL_ERROR;
        else
                (void) S_ImageAddToHash(interp, resultImage, (char *)NULL);

        return TCL_OK;
}

static int
S_Imagewiener_filter(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
        Image *imageP, *imageP1, *imageP2, *imageP3, *resultImage;


        if (argc != 8) {
                sprintf(interp->result, "Usage: cvip wiener <G image> <H image> <Noise Power Image> <Original Power Image> <F(u,v) = G(u,v)?(1/2)> <cut-off frequency>");
                return TCL_ERROR;
        }

        if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
        if(!(imageP1 = S_ImageGetFromHash(interp, argv[3]))) return TCL_ERROR;
        if(!(imageP2 = S_ImageGetFromHash(interp, argv[4]))) return TCL_ERROR;
        if(!(imageP3 = S_ImageGetFromHash(interp, argv[5]))) return TCL_ERROR;
        imageP=duplicate_Image(imageP);
        imageP1=duplicate_Image(imageP1);
        imageP2=duplicate_Image(imageP2);
        imageP3=duplicate_Image(imageP3);

        resultImage=wiener(imageP, imageP1, imageP2, imageP3, atoi(argv[6]), atoi(argv[7]));
        if(resultImage == NULL)
                return TCL_ERROR;
        else
                (void) S_ImageAddToHash(interp, resultImage, (char *)NULL);

        return TCL_OK;
}

static int
S_Imageparametric_wiener(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
        Image *imageP, *imageP1, *imageP2, *imageP3, *resultImage;


        if (argc != 9) {
                sprintf(interp->result, "Usage: cvip parametric_wiener <G image> <H image> <Noise Power Image> <Original Power Image> <gamma> <F(u,v) = G(u,v)?(1/2)> <cut-off frequency>");
                return TCL_ERROR;
        }

        if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
        if(!(imageP1 = S_ImageGetFromHash(interp, argv[3]))) return TCL_ERROR;
        if(!(imageP2 = S_ImageGetFromHash(interp, argv[4]))) return TCL_ERROR;
        if(!(imageP3 = S_ImageGetFromHash(interp, argv[5]))) return TCL_ERROR;
        imageP=duplicate_Image(imageP);
        imageP1=duplicate_Image(imageP1);
        imageP2=duplicate_Image(imageP2);
        imageP3=duplicate_Image(imageP3);

        resultImage=parametric_wiener(imageP, imageP1, imageP2, imageP3, atof(argv[6]), atoi(argv[7]), atoi(argv[8]));
        if(resultImage == NULL)
                return TCL_ERROR;
        else
                (void) S_ImageAddToHash(interp, resultImage, (char *)NULL);

        return TCL_OK;
}

static int
S_Imagepower_spect_eq(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
        Image *imageP, *imageP1, *imageP2, *imageP3, *resultImage;


        if (argc != 8) {
                sprintf(interp->result, "Usage: cvip power_spect_eq <G image> <H image> <Noise Power Image> <Original Power Image> <F(u,v) = G(u,v)?(1/2)> <cut-off frequency>");
                return TCL_ERROR;
        }

        if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
        if(!(imageP1 = S_ImageGetFromHash(interp, argv[3]))) return TCL_ERROR;
        if(!(imageP2 = S_ImageGetFromHash(interp, argv[4]))) return TCL_ERROR;
        if(!(imageP3 = S_ImageGetFromHash(interp, argv[5]))) return TCL_ERROR;
        imageP=duplicate_Image(imageP);
        imageP1=duplicate_Image(imageP1);
        imageP2=duplicate_Image(imageP2);
        imageP3=duplicate_Image(imageP3);

        resultImage=power_spect_eq(imageP, imageP1, imageP2, imageP3, atoi(argv[6]), atoi(argv[7]));
        if(resultImage == NULL)
                return TCL_ERROR;
        else
                (void) S_ImageAddToHash(interp, resultImage, (char *)NULL);

        return TCL_OK;
}

static int
S_Imagegeometric_mean(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
        Image *imageP, *imageP1, *imageP2, *imageP3, *resultImage;


        if (argc != 10) {
                sprintf(interp->result, "Usage: cvip geometric_mean <G image> <H image> <Noise Power Image> <Original Power Image> <gamma> <alpha> <F(u,v) = G(u,v)?(1/2)> <cut-off frequency>");
                return TCL_ERROR;
        }

        if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
        if(!(imageP1 = S_ImageGetFromHash(interp, argv[3]))) return TCL_ERROR;
        if(!(imageP2 = S_ImageGetFromHash(interp, argv[4]))) return TCL_ERROR;
        if(!(imageP3 = S_ImageGetFromHash(interp, argv[5]))) return TCL_ERROR;
        imageP=duplicate_Image(imageP);
        imageP1=duplicate_Image(imageP1);
        imageP2=duplicate_Image(imageP2);
        imageP3=duplicate_Image(imageP3);

        resultImage=geometric_mean(imageP, imageP1, imageP2, imageP3, atof(argv[6]), atof(argv[7]), atoi(argv[8]), atoi(argv[9]));
        if(resultImage == NULL)
                return TCL_ERROR;
        else
                (void) S_ImageAddToHash(interp, resultImage, (char *)NULL);

        return TCL_OK;
}

static int
S_Imageleast_squares_filter(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
        Image *imageP, *imageP1,*imageP2;
	int i,x,y;


        if (argc != 8) {
                sprintf(interp->result, "Usage: cvip least_squares <G image> <Himage> <SNR Image> <gamma> <choice when h=0 (1:F=G, 2:F=0)> <filter cutoff>");
                return TCL_ERROR;
        }

        if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
        if(!(imageP1 = S_ImageGetFromHash(interp, argv[3]))) return TCL_ERROR;
        if(!(imageP2 = S_ImageGetFromHash(interp, argv[4]))) return TCL_ERROR;
        imageP=duplicate_Image(imageP);
        imageP1=duplicate_Image(imageP1);
	imageP2=duplicate_Image(imageP2);

        imageP=least_squares(imageP, imageP1, imageP2, atof(argv[5]), atoi(argv[6]), atoi(argv[7]));
        if(imageP == NULL)
                return TCL_ERROR;
        else
                (void) S_ImageAddToHash(interp, imageP, (char *)NULL);

        return TCL_OK;
}


static int
S_ImageHISTTHRESH(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP, *tempImage;
	CVIP_TYPE	required_type[5];
	int	num_type;

	required_type[0] = CVIP_BYTE;
	num_type = 1;

	if (argc != 3){
		sprintf(interp->result,
			"Usage: cvip hist_thresh <image> ");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	imageP = convertImage_Queue(imageP,required_type,num_type);
 	imageP = hist_thresh_segment(imageP);

	if(imageP == NULL)
		return TCL_ERROR;
	else
		(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}
	
static int
S_ImageNightVision(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP;
	CVIP_TYPE	required_type[5];
	int	num_type;
	int 	y_split, x_split;

	required_type[0] = CVIP_BYTE;
	required_type[1] = CVIP_SHORT;
	num_type = 2;

	if (argc != 6) {
		sprintf(interp->result, "Usage: cvip nightvision <image> <nightvision/acuity(y/n)> <threshold> <acuity>\nFor argument you don't need, pass -1 ");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	imageP = convertImage_Queue(imageP,required_type,num_type);
	imageP = acuity_nightvision_filter(imageP,argv[3][0],atoi(argv[4]),
		atoi(argv[5]));

	if(imageP == NULL)
		return TCL_ERROR;
	else
		(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}


static int
S_ImageHOUGH(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP;

	if (argc != 6) {
		sprintf(interp->result, "Usage: cvip hough <binary image> <degrees(enclosed in curly brackets)> <line threshold> <connection threshold>");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	imageP = duplicate_Image(imageP);
	imageP = hough_filter(imageP,argv[2],argv[3],atoi(argv[4]), atoi(argv[5]),0);
	if(imageP == NULL)
		return TCL_ERROR;
	else
	   	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}

extern   char *color_format_string[];

static int
S_ImageColorTransform(ClientData clientData, Tcl_Interp *interp, int argc, char **argv){
	Image * imageP, *remapImage=NULL;
	CVIP_TYPE	required_type[5];
	int	num_type;
	int   color_format=0;
	float *normvec = NULL;
	float *refwhite = NULL;
	int choice = 1;
	float **normout = NULL;
	int size,i;

	required_type[0] = CVIP_BYTE;
	num_type = 1;

	if (argc != 7) {
		sprintf(interp->result, "Usage: cvip cxform <image> <color_space> <forward/inverse(1/0)> <normalize(1/0)> <do_remap(1/0)>\nwhere\tFor LUV, LAB, CCT and SCT, when do forward transform, <do_remap> can't be 1; if it is inverse transform, <do_remap> can be 1 or 0.\n\t For conditions you don't need, pass -1");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	if(imageP->bands != 3){
		sprintf(interp->result,"The input image should be color image");
		return TCL_ERROR;
	}

	for(;color_format_string[color_format];color_format++){
	    if(strcasecmp(color_format_string[color_format],argv[3])==0)
		break;
	}
	if(color_format_string[color_format] == NULL){
	    sprintf(interp->result,"Unknown Color Format %s",argv[3]);
	    return TCL_ERROR;
	}
	if(atoi(argv[4]) == 0)
	    choice = -1;
	if(atoi(argv[5]) == 1){
	    normvec = (float*)malloc(3*sizeof(float));
	    normvec[0] = normvec[1] = normvec[2] = 255.0;
	}

	if ((COLOR_FORMAT)color_format == HSL ||
	    (COLOR_FORMAT)color_format == HSV)
	  imageP = convertImage_Queue(imageP,required_type,num_type); 
	else imageP = duplicate_Image(imageP);
	
	switch((COLOR_FORMAT)color_format){
	    case LUV:
	    case LAB:
		refwhite = (float*)malloc(3*sizeof(float));
		refwhite[0] = refwhite[1] = refwhite[2] = 1.0;
	    case HSL:
	    case HSV:
	    case XYZ:
	    case SCT:
	    case CCT:
		imageP = colorxform(imageP,(COLOR_FORMAT)color_format,normvec,
				refwhite,choice);
		break;
	    default:
	        sprintf(interp->result,"Unsupported Color Format %s",argv[3]);
	        return TCL_ERROR;
	}

	if(normvec) free(normvec);
	if(refwhite) free(refwhite);

	if(strcmp(argv[6],"1")==0){
	    remapImage = imageP;
	    imageP = remap_Image(imageP,CVIP_BYTE,0,255);
	    delete_Image(remapImage);
	}
	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);
	return TCL_OK;
}


extern CVIP_BOOLEAN zero(CVIP_BOOLEAN a, CVIP_BOOLEAN b);
extern CVIP_BOOLEAN not_b(CVIP_BOOLEAN a, CVIP_BOOLEAN b);
extern CVIP_BOOLEAN and(CVIP_BOOLEAN a, CVIP_BOOLEAN b);
extern CVIP_BOOLEAN or(CVIP_BOOLEAN a, CVIP_BOOLEAN b);
extern CVIP_BOOLEAN not_and(CVIP_BOOLEAN a, CVIP_BOOLEAN b);
extern CVIP_BOOLEAN or_not_and(CVIP_BOOLEAN a, CVIP_BOOLEAN b);
extern CVIP_BOOLEAN xor(CVIP_BOOLEAN a, CVIP_BOOLEAN b);


int
S_ImageMorpho(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP;
   int *surrchoice, n=0, no_of_iter=-1, subf=0, i, j, k, *sP, boolf, add_sur=0;
   Matrix **surrounds;
   CVIP_BOOLEAN (*bF)(CVIP_BOOLEAN a,CVIP_BOOLEAN b);
   byte *surrP[] =   {(byte*)HEXLAT1, (byte*)HEXLAT2, (byte*)HEXLAT3, 
                      (byte*)HEXLAT4, (byte*)HEXLAT5, (byte*)HEXLAT6, 
		      (byte*)HEXLAT7, (byte*)HEXLAT8, (byte*)HEXLAT9, 
		      (byte*)HEXLAT10,(byte*)HEXLAT11,(byte*)HEXLAT12,
		      (byte*)HEXLAT13,(byte*)HEXLAT14};


	if (argc != 8) {
		sprintf(interp->result, "Usage: cvip morpho <image> {<set>} <rotation> <boolean> <iteration> <subfields>\nwhere\t{<set>} is a set of numbers from 1~14 for surrounds\n\t   (numbers separated by spaces, commas or semicolons, e.g. {1 5 10});\n\t<rotation> is y/n;\n\t<boolean> is a logic function of (a,b) from 1-7;\n\t<iteration> is a number between 1 and infinity (use 0 for infinity);\n\t<subfields> is a number of subfields(1-3);");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	if(chk_bin(imageP) == 0){
		sprintf(interp->result, "Input image MUST be two-level only, a two level image can be created from a gray level image by thresholding.");
		return TCL_ERROR;
	}

	surrchoice = (int*)parse2ints(argv[3], " ,;:\n",&n);

	if(argv[4][0] == 'y' || argv[4][0] == 'Y'){
      	    for(i=0; i < n; i++) 
	 	if(surrchoice[i]>1 && surrchoice[i]<7) add_sur+=5;
	 	else if(surrchoice[i]>8 && surrchoice[i]<13) add_sur+=5;
	 	else if(surrchoice[i] == 8) add_sur++;
	 	else if(surrchoice[i]==13 || surrchoice[i]==14) add_sur+=2;
	 	else {/*Change Nothing*/}
   	}
	k = n;
        surrounds = (MATRIX **) calloc(n+add_sur,sizeof(MATRIX *));
  	for(i=0, sP=surrchoice; i < n; i++, sP++)
             surrounds[i] = array2MATRIX(surrP[*sP-1], CVIP_BYTE, 3, 3);
	if(argv[4][0] == 'y' || argv[4][0] == 'Y'){
      	    for(i=0; i < n; i++) 
	 if( (surrchoice[i]>1 && surrchoice[i]<7) || (surrchoice[i]>8 && surrchoice[i]<13) ) 
	    for(j=0; j < 5; j++, k++)
	       surrounds[k] = hexrotate(surrounds[i], SIX_NWSE, (j+1)*60);
	 else if(surrchoice[i] == 8) 
	    surrounds[k++] = hexrotate(surrounds[i], SIX_NWSE, 60);
	 else if(surrchoice[i]==13 || surrchoice[i]==14)
	    for(j=0; j < 2; j++, k++)
	       surrounds[k] = hexrotate(surrounds[i], SIX_NWSE, (j+1)*60);
	 else {/*Change Nothing*/}
   	}

	boolf = atoi(argv[5]);
	switch(boolf){
      		case 1: bF = &zero; break;
      		case 2: bF = &not_b; break;
      		case 3: bF = &and; break;
      		case 4: bF = &or; break;
      		case 5: bF = &xor; break;
      		case 6: bF = &not_and; break;
      		default: bF = &or_not_and; break;
	}
	no_of_iter = atoi(argv[6]);
	subf = atoi(argv[7]);
	imageP = image_duplicate(imageP);
	imageP = (Image*)morphIterMod_Image(imageP, (const MATRIX **)surrounds, bF, n+add_sur, SIX_NWSE, no_of_iter, subf);
	if(imageP == NULL)
		return TCL_ERROR;

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}




static int
S_ImageHistoSpec(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP;
	CVIP_TYPE	required_type[5];
	int	num_type;
	int     no_of_bands, i;
	char **args;

	if(argc < 3){
		sprintf(interp->result, "Usage: cvip hist_spec <image> {<spec eqn band_0>} {<spec eqn band_1>} ... {<spec eqn band_N>}");
		return TCL_ERROR;
	}
	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) 
		return TCL_ERROR;

	no_of_bands = imageP->bands;
	if( (argc-no_of_bands) != 3){
		sprintf(interp->result, "Usage: cvip hist_spec <image>");
		while(no_of_bands>0){
			strcat(interp->result, " <spec eq.>");
			no_of_bands --;
		}
		return TCL_ERROR;
	}

	args = (char **) malloc(sizeof(char*) * (argc-2));
	for (i = 0; i < argc-3; i++) {
		args[i] = (char *)malloc(sizeof(char) * (strlen(argv[i+3]) + 1));
		strcpy(args[i], argv[i+3]);
	}
	args[i] = (char *)NULL;

	required_type[0] = CVIP_BYTE;
	num_type = 1;
	imageP = convertImage_Queue(imageP,required_type,num_type);
	imageP = hist_spec(imageP, 0, args);
	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);
	for (i = 0; i < argc-3; i++) {
		free(args[i]);
	}
	free(args);

	return TCL_OK;
}



static int
S_ImageSplitMerge(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
        Image * imageP;
        CVIP_TYPE    required_type = CVIP_BYTE;
        int     num_type = 1;
 
        int     choice;
        void *  parameter = (void *)malloc(2*sizeof(float));
        CVIP_BOOLEAN Run_PCT;
 
        if (argc != 8) {  
                sprintf(interp->result, "Usage: cvip split_merge <image, color or gray> <choice(1-6)> <level> <PCT (1/0=y/n)> <threshold/similarity> <percentage/pix distance>\n\tFor choice 1,2 & 3: <threshold> & <percentage> are -1\n\tFor choice 5:       <percentage> is -1\n\tFor choice 4 & 6:   no -1");
                return TCL_ERROR;
        }
        
        choice = atoi(argv[3]);
        Run_PCT = atoi(argv[5]);
        *((float *)parameter) = atof(argv[6]);
        *((float *)parameter + 1) = atof(argv[7]);
 
        if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
 
        imageP = convertImage_Queue(imageP,&required_type,num_type);
        imageP = split_merge_segment(imageP, atoi(argv[4]), choice,
		 	 parameter, Run_PCT);
 
        free((char *)parameter);

        if(imageP == NULL)
                return TCL_ERROR;
        else
                (void) S_ImageAddToHash(interp, imageP, (char *)NULL);

        return TCL_OK;
}

static int 
S_ImageFrei_Chen(ClientData clientData, Tcl_Interp *interp,int argc,char **argv)
{
	Image * imageP;
	Image * tempImage;
	CVIP_TYPE	required_type[5];
	int	num_type;

	required_type[0] = CVIP_SHORT;
	num_type = 1;

	if (argc != 9) {
		sprintf(interp->result, "Usage: cvip frei_chen <image> <smooth_filter(1-4)> <keep_DC(0/1)> <projection(1-3)> <threshold_projection(choice 1-3)> <threshold> <radian_threshold>\nFor argument(s) you don't need, pass -1.");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	/*imageP = convertImage_Queue(imageP,required_type,num_type);*/
	tempImage = duplicate_Image(imageP);
	imageP = edge_detect_filter(tempImage,EDGE_FREI,atoi(argv[3]),0,
			atoi(argv[4]),atoi(argv[7]),atoi(argv[5]),
			atoi(argv[6]),atoi(argv[8]));
	if(imageP == NULL)
	    return TCL_ERROR;

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}

static int
S_ImageWavelet(ClientData clientData, Tcl_Interp *interp,int argc,char **argv)
{
   Image *image, *imageP;
   MATRIX **matrix;
   char *answer;
   int i, rows, isign, **hptr;
   int choice, fchoice, lowband;
   float *hn, *hi, *htrans, *hitrans;  /* Filters */
   HISTORY stuff;

	
	if(argc != 6){
		sprintf(interp->result, "Usage: cvip wavelet <image> <forward/inverse(1/0)> <basis wavelet> <decomposition level>\nFor argument(s) you don't need, pass -1.");
		return TCL_ERROR;
	}
   if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
	imageP=duplicate_Image((const Image *)imageP);

   choice = atoi(argv[4]);
   fchoice = atoi(argv[5]);
   if(argv[3][0] = '1')
	isign = 1;
   else
	isign = -1;
   
   image = imageP;
   if(image->image_ptr[0]->data_type !=CVIP_FLOAT)
      (void) cast_Image(image, CVIP_FLOAT);


   answer = (char *)malloc(32);
   matrix = (MATRIX **)image->image_ptr;
   rows = image->image_ptr[0]->rows;

   /* Check to see if forward transform has already been performed */
   if((history_check(WAVELET, image)) && (!history_check(I_WAVELET, image))) {
      stuff=history_get(image, WAVELET);
      hptr = (int **)stuff->packetP->dptr;
      choice=hptr[0][0];
      fchoice=hptr[1][0];
      isign=-1;
   }

   switch(choice) {
      case 1 : lowband = (matrix[0]->rows)/ (int)(pow(2,(fchoice-1))+0.5);
               image = wavhaar_transform(image, isign, lowband);
               break;

      case 2 : lowband = (matrix[0]->rows)/ (int)(pow(2,(fchoice-1))+0.5);
               image = wavdaub4_transform(image, isign, lowband);
               break; 
   }

   if(choice>10) {
      fprintf(stdout, "choice=%d\n", choice);
      fprintf(stdout, "Invalid selection\n");
   }
   if((choice>2)&&(choice<11)) 
	return TCL_ERROR;
  
   /* Add history information */
   stuff=(HISTORY)NULL;
   stuff=(HISTORY)malloc(sizeof(struct history));
   stuff->packetP = (PACKET *)malloc(sizeof(PACKET));
   stuff->packetP->dsize=2;
   stuff->packetP->dtype=(CVIP_TYPE *)malloc(sizeof(CVIP_TYPE)*2);
   stuff->packetP->dtype[0]=CVIP_INTEGER;
   stuff->packetP->dtype[1]=CVIP_INTEGER;
   hptr=(int **)NULL;
   hptr=(int **)malloc(sizeof(int *)*2);
   hptr[0]=(int *)malloc(sizeof(int));
   hptr[1]=(int *)malloc(sizeof(int));
   hptr[0][0]=(int)choice;
   hptr[1][0]=(int)fchoice;
   stuff->packetP->dptr=(void **)hptr;
   stuff->next=(HISTORY)NULL;
   if(isign==1) stuff->ftag=WAVELET;
   else if(isign==-1) stuff->ftag=I_WAVELET;
   history_add(image, stuff);

   (void) S_ImageAddToHash(interp, image, (char *)NULL);

   return TCL_OK;

}
	
/*
 * Some heuristics used for the FFT:
 *
 * The inverse FFT will utilize the block size stored in the history
 * of the image.  If no block size is found, it will use the passed
 * block size.  the inverse FFT will work on a spatial-domain image,
 * and will result in a complex image.  What actually happens is
 * that the forward FFT routine is called.
 *
 * Arve Kjoelen (akjoele@ee.siue.edu) - Mon Oct 14 13:03:38 CDT 1996
 */
static int
S_ImageFFT(ClientData clientData, Tcl_Interp *interp,int argc,char **argv)
{
	Image * imageP;
	int choice = 1, block_size, n = 1;
	HISTORY stuff = NULL;
	int *hptr;
	PROGRAMS progs[2]={FFT,I_FFT};

	if(argc != 5){
		sprintf(interp->result, "Usage: cvip fft <image> <forward/inverse(1/0)> <block_size(2^n)> \nFor argument(s) you don't need, pass -1.");
		return TCL_ERROR;
	}
	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	if(atoi(argv[3]) == 0)
	    choice = 0;
	block_size = atoi(argv[4]);
	
	if(choice == 1)
	    while(n < block_size) n = n << 1;
	else
	    n = block_size;
	
	if(n != block_size){
	    sprintf(interp->result,"Error: block size %d not exponential of 2", block_size);
	    return TCL_ERROR;
	}
	
	imageP = duplicate_Image(imageP);

	if(choice){
	    imageP = fft_transform(imageP, block_size);
		if(imageP == NULL) return TCL_ERROR;
   	 stuff=(HISTORY)malloc(sizeof(struct history));
  	    stuff->packetP = (PACKET *)malloc(sizeof(PACKET));
 	    stuff->packetP->dsize=1;
  	    stuff->packetP->dtype=(CVIP_TYPE *)malloc(sizeof(CVIP_TYPE));
    	    stuff->packetP->dtype[0]=CVIP_INTEGER;
    	    hptr=(int *)malloc(sizeof(int));
   	    hptr[0]=block_size;
   	    stuff->packetP->dptr = (void**)malloc(sizeof(void*));
   	    stuff->packetP->dptr[0]=(int *)hptr;
  	    stuff->packetP->dsize = 1;
   	    stuff->next=(HISTORY)NULL;
   	    stuff->ftag=FFT;
            history_add(imageP, stuff);  
	}
	else{
   	    if(history_check(FFT, imageP)) {
	        stuff=history_get(imageP, FFT);
          	GET_HIST_DATA(stuff,0,block_size);
		imageP = ifft_transform(imageP, block_size);
		if(imageP == NULL) return TCL_ERROR;
  	    }
	    else {
		int rows = getNoOfRows_Image(imageP),
		    cols = getNoOfCols_Image(imageP);
		block_size = 1;
		rows = (rows > cols) ? rows : cols;
		while (block_size < rows) block_size = block_size << 1;
		fprintf(stderr,"No FFT found on the input image, use blocksize = %d\n",
			block_size);
		imageP=ifft_transform(imageP, block_size);
		if(imageP == NULL) return TCL_ERROR;
	    }

	    stuff=(HISTORY)malloc(sizeof(struct history));
            stuff->packetP = (PACKET *)malloc(sizeof(PACKET));
            stuff->packetP->dsize=1;
            stuff->packetP->dtype=(CVIP_TYPE *)malloc(sizeof(CVIP_TYPE));
            stuff->packetP->dtype[0]=CVIP_INTEGER;
            hptr=(int *)malloc(sizeof(int));
            hptr[0]=block_size;
            stuff->packetP->dptr = (void**)malloc(sizeof(void*));
            stuff->packetP->dptr[0]=(int *)hptr;
            stuff->packetP->dsize = 1;
            stuff->next=(HISTORY)NULL;
            stuff->ftag=I_FFT;
            history_add(imageP, stuff); 
	}         	    
	
	if(imageP == NULL)
		return TCL_ERROR;
	else
		(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}	

int
S_Imagefft_phase(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
        Image * imageP;

        if (argc != 5) {
                sprintf(interp->result, "Usage: cvip fft_phase <complex image> <remap_norm(1/0)> <k>\nwhere\twhen remap_norm is 0, remaps the phase data; when 1, normalizes the magnitude\nFor argument(s) you don't need, pass -1");
                return TCL_ERROR;
        }

        if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

        imageP = fft_phase(imageP, atoi(argv[3]), atof(argv[4]));

	if(imageP == NULL) return TCL_ERROR;
        else  (void) S_ImageAddToHash(interp, imageP, (char *)NULL);
 
        return TCL_OK;
}


static int
S_ImageWALSH(ClientData clientData, Tcl_Interp *interp,int argc,char **argv)
{
	Image * imageP;
	int choice = 1, block_size, n = 1;
	HISTORY stuff = NULL;
	int *hptr, n_rows, n_cols;

	if(argc != 5){
		sprintf(interp->result, "Usage: cvip walsh <image> <forward/inverse(1/0)> <block_size(2^n)> \nFor argument(s) you don't need, pass -1.");
		return TCL_ERROR;
	}
	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	if(atoi(argv[3]) == 0)
	    choice = 0;
	block_size = atoi(argv[4]);
	
	if(choice == 1)
	    while(n < block_size) n = n << 1;
	else
	    n = block_size;
	
	if(n != block_size){
	    sprintf(interp->result,"Error: block size %d not exponential of 2", block_size);
	    return TCL_ERROR;
	}
	imageP = duplicate_Image(imageP);

	if(choice){
            if(get_zpad_dims(imageP, &n_rows, &n_cols, block_size)==CVIP_YES)
                  zero_pad(imageP, n_rows, n_cols);
	    imageP = walhad_transform(imageP,1, block_size);
   	    stuff=(HISTORY)malloc(sizeof(struct history));
  	    stuff->packetP = (PACKET *)malloc(sizeof(PACKET));
 	    stuff->packetP->dsize=1;
  	    stuff->packetP->dtype=(CVIP_TYPE *)malloc(sizeof(CVIP_TYPE));
    	    stuff->packetP->dtype[0]=CVIP_INTEGER;
    	    hptr=(int *)malloc(sizeof(int));
   	    hptr[0]=block_size;
   	    stuff->packetP->dptr = (void**)malloc(sizeof(void*));
   	    stuff->packetP->dptr[0]=(int *)hptr;
  	    stuff->packetP->dsize = 1;
   	    stuff->next=(HISTORY)NULL;
   	    stuff->ftag=WALSH;
            history_add(imageP, stuff);  
	}
	else{
   	    if(history_check(WALSH, imageP)) {
	        stuff=history_get(imageP, WALSH);
          	GET_HIST_DATA(stuff,0,block_size);
   	    	imageP = walhad_transform(imageP,0,block_size);
                if(imageP == NULL) return TCL_ERROR;
  	    }
	    else {
                int rows = getNoOfRows_Image(imageP),
                    cols = getNoOfCols_Image(imageP);
                block_size = 1;
                rows = (rows > cols) ? rows : cols;
                while (block_size < rows) block_size = block_size << 1;
                fprintf(stderr,"No WALSH found on the input image, use blocksize = %d\n",block_size);
   	    	imageP = walhad_transform(imageP,0,block_size);
                if(imageP == NULL) return TCL_ERROR;
            }

	    stuff=(HISTORY)malloc(sizeof(struct history));
            stuff->packetP = (PACKET *)malloc(sizeof(PACKET));
            stuff->packetP->dsize=1;
            stuff->packetP->dtype=(CVIP_TYPE *)malloc(sizeof(CVIP_TYPE));
            stuff->packetP->dtype[0]=CVIP_INTEGER;
            hptr=(int *)malloc(sizeof(int));
            hptr[0]=block_size;
            stuff->packetP->dptr = (void**)malloc(sizeof(void*));
            stuff->packetP->dptr[0]=(int *)hptr;
            stuff->packetP->dsize = 1;
            stuff->next=(HISTORY)NULL;
            stuff->ftag=I_WALSH;
            history_add(imageP, stuff); 
	}         	    
	
	if(imageP == NULL)
		return TCL_ERROR;
	else
		(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}	
	
static int
S_ImageHADAMARD(ClientData clientData, Tcl_Interp *interp,int argc,char **argv)
{
	Image * imageP;
	int choice = 1, block_size, n = 1;
	HISTORY stuff = NULL;
	int *hptr, n_rows, n_cols;

	if(argc != 5){
		sprintf(interp->result, "Usage: cvip hadamard <image> <forward/inverse(1/0)> <block_size(2^n)> \nFor argument(s) you don't need, pass -1.");
		return TCL_ERROR;
	}
	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	if(atoi(argv[3]) == 0)
	    choice = 0;
	block_size = atoi(argv[4]);
	
	if(choice == 1)
	    while(n < block_size) n = n << 1;
	else
	    n = block_size;
	
	if(n != block_size){
	    sprintf(interp->result,"Error: block size %d not exponential of 2", block_size);
	    return TCL_ERROR;
	}
	imageP = duplicate_Image(imageP);

	if(choice){
            if(get_zpad_dims(imageP, &n_rows, &n_cols, block_size)==CVIP_YES)
                  zero_pad(imageP, n_rows, n_cols);
	    imageP = walhad_transform(imageP,3, block_size);
   	    stuff=(HISTORY)malloc(sizeof(struct history));
  	    stuff->packetP = (PACKET *)malloc(sizeof(PACKET));
 	    stuff->packetP->dsize=1;
  	    stuff->packetP->dtype=(CVIP_TYPE *)malloc(sizeof(CVIP_TYPE));
    	    stuff->packetP->dtype[0]=CVIP_INTEGER;
    	    hptr=(int *)malloc(sizeof(int));
   	    hptr[0]=block_size;
   	    stuff->packetP->dptr = (void**)malloc(sizeof(void*));
   	    stuff->packetP->dptr[0]=(int *)hptr;
  	    stuff->packetP->dsize = 1;
   	    stuff->next=(HISTORY)NULL;
   	    stuff->ftag=HADAMARD;
            history_add(imageP, stuff);  
	}
	else{
   	    if(history_check(HADAMARD, imageP)) {
	        stuff=history_get(imageP, HADAMARD);
          	GET_HIST_DATA(stuff,0,block_size);
   	    	imageP = walhad_transform(imageP,2,block_size);
                if(imageP == NULL) return TCL_ERROR;
  	    }
	     else {
                int rows = getNoOfRows_Image(imageP),
                    cols = getNoOfCols_Image(imageP);
                block_size = 1;
                rows = (rows > cols) ? rows : cols;
                while (block_size < rows) block_size = block_size << 1;
                fprintf(stderr,"No HADAMARD found on the input image, use blocksize = %d\n",block_size);
                imageP = walhad_transform(imageP,0,block_size);
                if(imageP == NULL) return TCL_ERROR;
            }

	    stuff=(HISTORY)malloc(sizeof(struct history));
            stuff->packetP = (PACKET *)malloc(sizeof(PACKET));
            stuff->packetP->dsize=1;
            stuff->packetP->dtype=(CVIP_TYPE *)malloc(sizeof(CVIP_TYPE));
            stuff->packetP->dtype[0]=CVIP_INTEGER;
            hptr=(int *)malloc(sizeof(int));
            hptr[0]=block_size;
            stuff->packetP->dptr = (void**)malloc(sizeof(void*));
            stuff->packetP->dptr[0]=(int *)hptr;
            stuff->packetP->dsize = 1;
            stuff->next=(HISTORY)NULL;
            stuff->ftag=I_HADAMARD;
            history_add(imageP, stuff); 
	}         	    
	
	if(imageP == NULL)
		return TCL_ERROR;
	else
		(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}

static int
S_ImageHAAR(ClientData clientData, Tcl_Interp *interp,int argc,char **argv)
{
	Image * imageP;
	int choice = 1, block_size, n = 1;
	HISTORY stuff = NULL;
	int *hptr, n_rows, n_cols;

	if(argc != 5){
		sprintf(interp->result, "Usage: cvip haar <image> <forward/inverse(1/0)> <block_size(2^n)> \nFor argument(s) you don't need, pass -1.");
		return TCL_ERROR;
	}
	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	if(atoi(argv[3]) == 0)
	    choice = 0;
	block_size = atoi(argv[4]);
	
	if(choice == 1)
	    while(n < block_size) n = n << 1;
	else
	    n = block_size;
	
	if(n != block_size){
	    sprintf(interp->result,"Error: block size %d not exponential of 2", block_size);
	    return TCL_ERROR;
	}
	imageP = duplicate_Image(imageP);

	if(choice){
            if(get_zpad_dims(imageP, &n_rows, &n_cols, block_size)==CVIP_YES)
                  zero_pad(imageP, n_rows, n_cols);
	    imageP = haar_transform(imageP,1, block_size);
   	    stuff=(HISTORY)malloc(sizeof(struct history));
  	    stuff->packetP = (PACKET *)malloc(sizeof(PACKET));
 	    stuff->packetP->dsize=1;
  	    stuff->packetP->dtype=(CVIP_TYPE *)malloc(sizeof(CVIP_TYPE));
    	    stuff->packetP->dtype[0]=CVIP_INTEGER;
    	    hptr=(int *)malloc(sizeof(int));
   	    hptr[0]=block_size;
   	    stuff->packetP->dptr = (void**)malloc(sizeof(void*));
   	    stuff->packetP->dptr[0]=(int *)hptr;
  	    stuff->packetP->dsize = 1;
   	    stuff->next=(HISTORY)NULL;
   	    stuff->ftag=HAAR;
            history_add(imageP, stuff);  
	}
	else{
   	    if(history_check(HAAR, imageP)) {
	        stuff=history_get(imageP, HAAR);
          	GET_HIST_DATA(stuff,0,block_size);
   	    	imageP = haar_transform(imageP,0,block_size);
                if(imageP == NULL) return TCL_ERROR;
  	    }
	    else {
                int rows = getNoOfRows_Image(imageP),
                    cols = getNoOfCols_Image(imageP);
                block_size = 1;
                rows = (rows > cols) ? rows : cols;
                while (block_size < rows) block_size = block_size << 1;
                fprintf(stderr,"No HAAR found on the input image, use blocksize = %d\n",block_size);
                imageP = haar_transform(imageP,0,block_size);
                if(imageP == NULL) return TCL_ERROR;
            }

	    stuff=(HISTORY)malloc(sizeof(struct history));
            stuff->packetP = (PACKET *)malloc(sizeof(PACKET));
            stuff->packetP->dsize=1;
            stuff->packetP->dtype=(CVIP_TYPE *)malloc(sizeof(CVIP_TYPE));
            stuff->packetP->dtype[0]=CVIP_INTEGER;
            hptr=(int *)malloc(sizeof(int));
            hptr[0]=block_size;
            stuff->packetP->dptr = (void**)malloc(sizeof(void*));
            stuff->packetP->dptr[0]=(int *)hptr;
            stuff->packetP->dsize = 1;
            stuff->next=(HISTORY)NULL;
            stuff->ftag=I_HAAR;
            history_add(imageP, stuff); 
	}         	    
	
	if(imageP == NULL)
		return TCL_ERROR;
	else
		(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}

static int
S_ImageDCT(ClientData clientData, Tcl_Interp *interp,int argc,char **argv)
{
	Image * imageP;
	int choice = 1, block_size, n = 1;
	HISTORY stuff = NULL;
	int *hptr, n_rows, n_cols;

	if(argc != 5){
		sprintf(interp->result, "Usage: cvip dct <image> <forward/inverse(1/0)> <block_size(2^n)> \nFor argument(s) you don't need, pass -1.");
		return TCL_ERROR;
	}
	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	if(atoi(argv[3]) == 0)
	    choice = 0;
	block_size = atoi(argv[4]);
	
	if(choice == 1)
	    while(n < block_size) n = n << 1;
	else
	    n = block_size;
	
	if(n != block_size){
	    sprintf(interp->result,"Error: block size %d not exponential of 2", block_size);
	    return TCL_ERROR;
	}
	imageP = duplicate_Image(imageP);

	if(choice){
            if(get_zpad_dims(imageP, &n_rows, &n_cols, block_size)==CVIP_YES)
                  zero_pad(imageP, n_rows, n_cols);
	    imageP = dct_transform(imageP, block_size);
   	    stuff=(HISTORY)malloc(sizeof(struct history));
  	    stuff->packetP = (PACKET *)malloc(sizeof(PACKET));
 	    stuff->packetP->dsize=1;
  	    stuff->packetP->dtype=(CVIP_TYPE *)malloc(sizeof(CVIP_TYPE));
    	    stuff->packetP->dtype[0]=CVIP_INTEGER;
    	    hptr=(int *)malloc(sizeof(int));
   	    hptr[0]=block_size;
   	    stuff->packetP->dptr = (void**)malloc(sizeof(void*));
   	    stuff->packetP->dptr[0]=(int *)hptr;
  	    stuff->packetP->dsize = 1;
   	    stuff->next=(HISTORY)NULL;
   	    stuff->ftag=DCTXFORM;
            history_add(imageP, stuff);  
	}
	else{
   	    if(history_check(DCTXFORM, imageP)) {
	        stuff=history_get(imageP, DCTXFORM);
          	GET_HIST_DATA(stuff,0,block_size);
   	    	imageP = idct_transform(imageP,block_size);
                if(imageP == NULL) return TCL_ERROR;
  	    }
	      else {
                int rows = getNoOfRows_Image(imageP),
                    cols = getNoOfCols_Image(imageP);
                block_size = 1;
                rows = (rows > cols) ? rows : cols;
                while (block_size < rows) block_size = block_size << 1;
                fprintf(stderr,"No DCT found on the input image, use blocksize = %d\n",block_size);
                imageP = idct_transform(imageP,block_size);
                if(imageP == NULL) return TCL_ERROR;
            }

	    stuff=(HISTORY)malloc(sizeof(struct history));
            stuff->packetP = (PACKET *)malloc(sizeof(PACKET));
            stuff->packetP->dsize=1;
            stuff->packetP->dtype=(CVIP_TYPE *)malloc(sizeof(CVIP_TYPE));
            stuff->packetP->dtype[0]=CVIP_INTEGER;
            hptr=(int *)malloc(sizeof(int));
            hptr[0]=block_size;
            stuff->packetP->dptr = (void**)malloc(sizeof(void*));
            stuff->packetP->dptr[0]=(int *)hptr;
            stuff->packetP->dsize = 1;
            stuff->next=(HISTORY)NULL;
            stuff->ftag=I_DCTXFORM;
            history_add(imageP, stuff); 
	}         	    
	
	if(imageP == NULL)
		return TCL_ERROR;
	else
		(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}
	
static int
S_ImagePrewitt(ClientData clientData, Tcl_Interp *interp,int argc,char **argv)
{
	Image * imageP;
	Image * tempImage;
	CVIP_TYPE	required_type[5];
	int	num_type;

	required_type[0] = CVIP_SHORT;
	num_type = 1;

	if (argc != 7) {
		sprintf(interp->result, "Usage: cvip prewitt <image> <smooth_filter(1-4)> <keep_DC(0/1)> <kernel> <threshold>\nFor argument(s) you don't need, pass -1.");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	/*imageP = convertImage_Queue(imageP,required_type,num_type);*/
	tempImage = duplicate_Image(imageP);
	imageP = edge_detect_filter(tempImage,EDGE_PREWITT,atoi(argv[3]),
			atoi(argv[5]),atoi(argv[4]),atoi(argv[6]),0,0,0);
	if(imageP == NULL)
	    return TCL_ERROR;

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}

static int
S_ImageLaplacian(ClientData clientData, Tcl_Interp *interp,int argc,char **argv)
{
	Image * imageP;
	Image * tempImage;
	CVIP_TYPE	required_type[5];
	int	num_type;

	required_type[0] = CVIP_SHORT;
	num_type = 1;

	if (argc != 7) {
		sprintf(interp->result, "Usage: cvip laplacian <image> <smooth_filter(1-4)> <keep_DC(0/1)> <laplacian mask(1,2)> <threshold>\nFor argument(s) you don't need, pass -1.");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	/*imageP = convertImage_Queue(imageP,required_type,num_type);*/
	tempImage = duplicate_Image(imageP);
	imageP = edge_detect_filter(tempImage,EDGE_LAPLACIAN,atoi(argv[3]),
			atoi(argv[5]),atoi(argv[4]),atoi(argv[6]),0,0,0);
	if(imageP == NULL)
	    return TCL_ERROR;

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}

static int
S_ImageRoberts(ClientData clientData, Tcl_Interp *interp,int argc,char **argv)
{
	Image * imageP;
	Image * tempImage;
	CVIP_TYPE	required_type[5];
	int	num_type;

	required_type[0] = CVIP_SHORT;
	num_type = 1;

	if (argc != 7) {
		sprintf(interp->result, "Usage: cvip roberts <image> <smooth_filter(1-4)> <keep_DC(0/1)> <detector type(1,2)> <threshold>\nFor argument(s) you don't need, pass -1.");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	/*imageP = convertImage_Queue(imageP,required_type,num_type);*/
	tempImage = duplicate_Image(imageP);
	imageP = edge_detect_filter(tempImage,EDGE_ROBERTS,atoi(argv[3]),
			atoi(argv[5]),atoi(argv[4]),atoi(argv[6]),0,0,0);
	if(imageP == NULL)
	    return TCL_ERROR;

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}

static int 
S_ImageSobel(ClientData clientData, Tcl_Interp *interp,int argc,char **argv)
{
	Image * imageP;
	Image * tempImage;
	CVIP_TYPE	required_type[5];
	int	num_type;

	required_type[0] = CVIP_SHORT;
	num_type = 1;

	if (argc != 7) {
		sprintf(interp->result, "Usage: cvip sobel <image> <smooth_filter(1-4)> <keep_DC(0/1)> <kernel(3,5,7)> <threshold>\nFor argument(s) you don't need, pass -1.");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	/*imageP = convertImage_Queue(imageP,required_type,num_type);*/
	tempImage = duplicate_Image(imageP);
	imageP = edge_detect_filter(tempImage,EDGE_SOBEL,atoi(argv[3]),
			atoi(argv[5]),atoi(argv[4]),atoi(argv[6]),0,0,0);
	if(imageP == NULL)
	    return TCL_ERROR;

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}

static int 
S_ImageKirsch(ClientData clientData, Tcl_Interp *interp,int argc,char **argv)
{
	Image * imageP;
	Image * tempImage;
	CVIP_TYPE	required_type[5];
	int	num_type;

	required_type[0] = CVIP_SHORT;
	num_type = 1;

	if (argc != 5) {
		sprintf(interp->result, "Usage: cvip kirsch <image> <smooth_filter(1-4)> <threshold>\nFor argument(s) you don't need, pass -1.");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	/*imageP = convertImage_Queue(imageP,required_type,num_type);*/
	tempImage = duplicate_Image(imageP);
	imageP = edge_detect_filter(tempImage,EDGE_KIRSCH,atoi(argv[3]),
			0,0,atoi(argv[4]),0,0,0);
	if(imageP == NULL)
	    return TCL_ERROR;

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}

static int
S_ImageRobinson(ClientData clientData, Tcl_Interp *interp,int argc,char **argv)
{
	Image * imageP;
	Image * tempImage;
	CVIP_TYPE	required_type[5];
	int	num_type;

	required_type[0] = CVIP_SHORT;
	num_type = 1;

	if (argc != 5) {
		sprintf(interp->result, "Usage: cvip robinson <image> <smooth_filter(1-4)> <threshold>\nFor argument(s) you don't need, pass -1.");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	/*imageP = convertImage_Queue(imageP,required_type,num_type);*/
	tempImage = duplicate_Image(imageP);
	imageP = edge_detect_filter(tempImage,EDGE_ROBINSON,atoi(argv[3]),
			0,0,atoi(argv[4]),0,0,0);
	if(imageP == NULL)
	    return TCL_ERROR;

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}


static int
S_ImagePyramid(ClientData clientData, Tcl_Interp *interp,int argc,char **argv)
{
	Image * imageP;
	Image * tempImage;
	CVIP_TYPE	required_type[5];
	int	num_type;

	required_type[0] = CVIP_SHORT;
	num_type = 1;

	if (argc != 5) {
		sprintf(interp->result, "Usage: cvip pyramid <image> <smooth_filter(1-4)> <threshold>\nFor argument(s) you don't need, pass -1.");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	/*imageP = convertImage_Queue(imageP,required_type,num_type);*/
	tempImage = duplicate_Image(imageP);
	imageP = edge_detect_filter(tempImage,EDGE_PYRAMID,atoi(argv[3]),
			0,0,atoi(argv[4]),0,0,0);
	if(imageP == NULL)
	    return TCL_ERROR;

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}

extern  char * image_format_string[];

static int
S_ImageConvert(ClientData clientData, Tcl_Interp *interp,int argc,char **argv)
{
	Image *imageP;
	int pos = 0;

        if(argc != 4){
		sprintf(interp->result, "Usage: cvip convert <image> <new_format>");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	imageP = duplicate_Image(imageP);

	for(; image_format_string[pos]; pos++){
		if(strcasecmp(image_format_string[pos],argv[3]) == 0)
			break;
	}
	
	if(image_format_string[pos] == NULL){
		sprintf(interp->result,"Unknown Image Fomrat: %s",argv[3]);
		return TCL_ERROR;
	}

	imageP->image_format = (IMAGE_FORMAT) pos;	

	(void)S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}


	

static int
S_ImageHashList(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP;
	Tcl_HashEntry *entryP;
	Tcl_HashSearch search;

	if (argc != 2) {
		sprintf(interp->result, "Usage: cvip list");
		return TCL_ERROR;
	}

	for(entryP = Tcl_FirstHashEntry(&S_ImageTable, &search); entryP != NULL; entryP = Tcl_NextHashEntry(&search)) {
		Tcl_AppendResult(interp, Tcl_GetHashKey(&S_ImageTable, entryP), " \n", NULL);
	}

	return TCL_OK;
}

static int
S_ImageGrayMorph(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image * imageP = (NULL), *tmpP = (NULL);
	Matrix * matP = (NULL);
	double ksize = (0);
	COLOR_FORMAT colorSpace;

	if (argc != 4) {
		sprintf(interp->result, "Usage: cvip morph.gray <image> <kernel_size>");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	if (Tcl_GetDouble( interp, argv[3], &ksize) != TCL_OK) {
		return TCL_ERROR;
	}

	imageP = image_duplicate(imageP);

	colorSpace = getColorSpace_Image(imageP);

	if( (BINARY != colorSpace) && (GRAY_SCALE != colorSpace) ) {
		tmpP = imageP;
		imageP = color2index_Image(imageP);
		delete_Image(tmpP);
	}

	if(!disk_SE(&matP, CVIP_INTEGER, -1.0, 0.0, (float) ksize)) {
		sprintf(interp->result, "could not form structuring element");
		return TCL_ERROR;
	}

	if(!MorphOpen_Image(imageP, matP, 0, 0, 0)) {
		sprintf(interp->result, "could not perform gray-scale morphological opening");
		return TCL_ERROR;
	}

	if(!MorphClose_Image(imageP, matP, 0, 0, 0)) {
		sprintf(interp->result, "could not perform gray-scale morphological closing");
		return TCL_ERROR;
	}

	delete_Matrix(matP);

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}


static char * CVIPTCL_find_ImagePathList(
	ImagePathList	iplP,
	char	*fname
){
	char 	buf[160], *strP;

	if(!size_DLL(iplP)) return NULL;

	head_DLL(iplP);

	for(;;) {
 		strP = retrieve_DLL(iplP);

		strcpy(buf, strP);
		strcat(buf, "/");
		strcat(buf, fname);

		#ifndef WIN32
		if( fopen(buf, "r") ) return strP;
		#else
		if( fopen(buf, "rb") ) return strP;
		#endif
		if( istail_DLL(iplP) ) break;
      		next_DLL(iplP);
	}

	return NULL;
}


int check_xform_history(Image *in_Image, unsigned filter, PROGRAMS *transform){
      PROGRAMS i, progs[6];
      HISTORY h_story;

      progs[0]=FFT; progs[1]=DCTXFORM; progs[2]=WALSH; progs[3]=WAVELET; progs[4]=HAAR; progs[5]=HADAMARD;
      if((i=getlast_Hist(in_Image, progs, 6))==0)
         return i;	

      /* Add information to image history */
      if((filter>0) && (filter<6)) {
         h_story=(HISTORY)malloc(sizeof(struct history));
         h_story->next=(HISTORY)NULL;
         h_story->packetP=(PACKET *)malloc(sizeof(PACKET));
	 h_story->packetP->dtype=(CVIP_TYPE*)malloc(sizeof(CVIP_TYPE));
	 *(h_story->packetP->dtype)=CVIP_INTEGER;
	 h_story->packetP->dsize=1;
	 h_story->packetP->dptr = (void **)malloc(sizeof(void*));
 	 h_story->packetP->dptr[0]=(int *)malloc(sizeof(int));
	 memcpy((void *)h_story->packetP->dptr[0], (void *)&filter, 
			sizeof(int));
         h_story->ftag=XFORMFILTER;
         history_add(in_Image, h_story);
      }
      *transform = i;

      return 1;     
}



static int
S_ImageHPF_Filter(ClientData clientData, Tcl_Interp *interp, int argc, char **argv){
	Image * imageP = (NULL), *tmpP = (NULL);
	CVIP_TYPE	required_type[5];
	int	num_type;
	int dc=0, dcpos, block_size,order, cut;
	FILTER_TYPE filtertype;
	PROGRAMS transform;
	HISTORY stuff;

	required_type[0] = CVIP_FLOAT;
	num_type = 1;

	if (argc != 7) {
		sprintf(interp->result, "Usage: cvip hpf_filter <image> <keep_dc(y/n)> <IDEAL_HIGH/BUTTER_HIGH> <cutoff_frequency(1-block_size)> <Butterworth filter order(1-8)>\n For argument(s) you don't need, pass -1");
		return TCL_ERROR;
	}

	if (!strcmp(argv[4],"IDEAL_HIGH")) 
    	filtertype = IDEAL_HIGH;
	else if (!strcmp(argv[4],"BUTTER_HIGH"))
    	filtertype = BUTTER_HIGH;
	else 	{
		sprintf(interp->result, "filter type can only be IDEAL_HIGH or BUTTER_HIGH");
		return TCL_ERROR;
	}


	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
	
	imageP = convertImage_Queue(imageP,required_type,num_type);


	if(!check_xform_history(imageP,1,&transform)){
		sprintf(interp->result,"Input image doesn't have appropriate transform history");
		delete_Image(imageP);
		return TCL_ERROR;
	}

	if(argv[3][0] == 'y' || argv[3][0] == 'Y')
		dc = 1;
  switch(transform) {
    case WALSH:
       stuff=history_get(imageP, WALSH);
          GET_HIST_DATA(stuff,0,block_size);
       dcpos = 2; /* corner */
       break;
    case HADAMARD:
       stuff=history_get(imageP, HADAMARD);
          GET_HIST_DATA(stuff,0,block_size);
       dcpos = 2; 
       break;
    case FFT:
       stuff=history_get(imageP, FFT);
          GET_HIST_DATA(stuff,0,block_size);
       dcpos = 1;
       break;  
    case DCTXFORM:
       stuff=history_get(imageP, DCTXFORM);
          GET_HIST_DATA(stuff,0,block_size);
       dcpos=2;
       break;
    case HAAR:
       stuff=history_get(imageP, HAAR);
          GET_HIST_DATA(stuff,0,block_size);
       dcpos=2;
       break;
    default:
       dcpos=2;
       block_size=imageP->image_ptr[0]->rows;
       break;
  }

    cut = atoi(argv[5]);
    order = atoi(argv[6]);
  switch(filtertype) {
    case IDEAL_HIGH:  /* Ideal High Pass */ 
      if ( dcpos == 1) { /* dc in center */
        imageP =(Image *) Ideal_High (imageP, block_size, dc,cut);
      }
      else { /* 0 for filter order since this is not Butterworth */
        imageP=(Image *)nonfft_xformfilter(imageP,block_size,dc,filtertype,cut,0,0);
      }
        break;
    case BUTTER_HIGH: /* Butterworth High Pass */
        if ( dcpos == 1) {
          imageP=(Image *)Butterworth_High(imageP,block_size,dc,cut,order);
        }
        else {
          imageP =(Image *)nonfft_xformfilter(imageP,block_size,dc,filtertype,cut, 0,order);
        }
        break;
    }
	S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;


	
}


static int
S_ImageLPF_Filter(ClientData clientData, Tcl_Interp *interp, int argc, char **argv){
	Image * imageP = (NULL), *tmpP = (NULL);
	CVIP_TYPE	required_type[5];
	int	num_type;
	int dc=0, dcpos, block_size,order, cut;
	FILTER_TYPE filtertype;
	PROGRAMS transform;
	HISTORY stuff;

	required_type[0] = CVIP_FLOAT;
	num_type = 1;

	if (argc != 7) {
		sprintf(interp->result, "Usage: cvip lpf_filter <image> <keep_dc(y/n)> <IDEAL_LOW/BUTTER_LOW> <cutoff_frequency(1-block_size)> <Butterworth filter order(1-8)>\n For argument(s) you don't need, pass -1");
		return TCL_ERROR;
	}

	if (!strcmp(argv[4],"IDEAL_LOW")) 
    	filtertype = IDEAL_LOW;
	else if (!strcmp(argv[4],"BUTTER_LOW"))
    	filtertype = BUTTER_LOW;
	else 	{
		sprintf(interp->result, "filter type can only be IDEAL_LOW or BUTTER_LOW");
		return TCL_ERROR;
	}


	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
	
	imageP = convertImage_Queue(imageP,required_type,num_type);


	if(!check_xform_history(imageP,2,&transform)){
		sprintf(interp->result,"Input image doesn't have appropriate transform history");
		delete_Image(imageP);
		return TCL_ERROR;
	}

	if(argv[3][0] == 'y' || argv[3][0] == 'Y')
		dc = 1;
  switch(transform) {
    case WALSH:
       stuff=history_get(imageP, WALSH);
          GET_HIST_DATA(stuff,0,block_size);
       dcpos = 2; /* corner */
       break;
    case HADAMARD:
       stuff=history_get(imageP, HADAMARD);
          GET_HIST_DATA(stuff,0,block_size);
       dcpos = 2; 
       break;
    case FFT:
       stuff=history_get(imageP, FFT);
          GET_HIST_DATA(stuff,0,block_size);
       dcpos = 1;
       break;  
    case DCTXFORM:
       stuff=history_get(imageP, DCTXFORM);
       GET_HIST_DATA(stuff,0,block_size);
       dcpos=2;
       break;
    case HAAR:
       stuff=history_get(imageP, HAAR);
       GET_HIST_DATA(stuff,0,block_size);
       dcpos=2;
       break;
    default:
       dcpos=2;
       block_size=imageP->image_ptr[0]->rows;
       break;
  }

    cut = atoi(argv[5]);
    order = atoi(argv[6]);
  switch(filtertype) {
    case IDEAL_LOW:  /* Ideal Low Pass */ 
      if ( dcpos == 1) { /* dc in center */
        imageP =(Image *) Ideal_Low (imageP, block_size, dc,cut);
      }
      else { /* 0 filter order since this is not Butterworth */
        imageP=(Image *)nonfft_xformfilter(imageP,block_size,dc,filtertype,cut,0,0);
      }
        break;
    case BUTTER_LOW: /* Butterworth Low Pass */
        if ( dcpos == 1) {
          imageP=(Image *)Butterworth_Low(imageP,block_size,dc,cut,order);
        }
        else {
          imageP =(Image *)nonfft_xformfilter(imageP,block_size,dc,filtertype,cut, 0,order);
        }
    }
	S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;


	
}


static int
S_ImageBPF_Filter(ClientData clientData, Tcl_Interp *interp, int argc, char **argv){
	Image * imageP = (NULL), *tmpP = (NULL);
	CVIP_TYPE	required_type[5];
	int	num_type;
	int dc=0, dcpos, block_size,order;
	FILTER_TYPE filtertype;
	int inner=0, outer=0;
	PROGRAMS transform;
	HISTORY stuff;

	required_type[0] = CVIP_FLOAT;
	num_type = 1;

	if (argc != 8) {
	Tcl_AppendResult(interp, "Usage: cvip bpf_filter <image> <keep_dc(y/n)> <IDEAL_BAND/BUTTER_BAND> <lower cutoff_frequency(1-block_size)> <higher cutoff_frequency(1-block_size)> <Butterworth filter order(1-8)>\n For argument(s) you don't need, pass -1", (char *)NULL);
		return TCL_ERROR;
	}

	if (!strcmp(argv[4],"IDEAL_BAND")) 
    	filtertype = IDEAL_BAND;
	else if (!strcmp(argv[4],"BUTTER_BAND"))
    	filtertype = BUTTER_BAND;
	else 	{
		sprintf(interp->result, "filter type can only be IDEAL_BAND or BUTTER_BAND");
		return TCL_ERROR;
	}


	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
	
	imageP = convertImage_Queue(imageP,required_type,num_type);


	if(!check_xform_history(imageP,3,&transform)){
		sprintf(interp->result,"Input image doesn't have appropriate transform history");
		delete_Image(imageP);
		return TCL_ERROR;
	}

	if(argv[3][0] == 'y' || argv[3][0] == 'Y')
		dc = 1;
  switch(transform) {
    case WALSH:
       stuff=history_get(imageP, WALSH);
          GET_HIST_DATA(stuff,0,block_size);
       dcpos = 2; /* corner */
       break;
    case HADAMARD:
       stuff=history_get(imageP, HADAMARD);
          GET_HIST_DATA(stuff,0,block_size);
       dcpos = 2; 
       break;
    case FFT:
       stuff=history_get(imageP, FFT);
          GET_HIST_DATA(stuff,0,block_size);
       dcpos = 1;
       break;  
    case DCTXFORM:
       stuff=history_get(imageP, DCTXFORM);
          GET_HIST_DATA(stuff,0,block_size);
       dcpos=2;
       break;
    case HAAR:
       stuff=history_get(imageP, HAAR);
       GET_HIST_DATA(stuff,0,block_size);
       dcpos=2;
       break;
    default:
       dcpos=2;
       block_size=imageP->image_ptr[0]->rows;
       break;
  }

    inner = atoi(argv[5]);
    outer = atoi(argv[6]);
    order = atoi(argv[7]);
  switch(filtertype) {
    case IDEAL_BAND:  /* Ideal Band Pass */ 
      if ( dcpos == 1) { /* dc in center */
        imageP=(Image *)Ideal_Band_Pass(imageP,block_size,dc,inner,outer);
      }
      else { /* 0 filter order since this is not Butterworth */
        imageP=(Image *)nonfft_xformfilter(imageP,block_size,dc,filtertype,inner,outer,0);
      }
        break;
    case BUTTER_BAND: /* Butterworth Band Pass */
        if ( dcpos == 1) {
          imageP=(Image *)Butterworth_Band_Pass(imageP,block_size,dc,inner,outer,order);
        }
        else {
          imageP =(Image *)nonfft_xformfilter(imageP,block_size,dc,filtertype,inner, outer,order);
        }
    }
	S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;


	
}


static int
S_ImageBRF_Filter(ClientData clientData, Tcl_Interp *interp, int argc, char **argv){
	Image * imageP = (NULL), *tmpP = (NULL);
	CVIP_TYPE	required_type[5];
	int	num_type;
	int dc=0, dcpos, block_size,order;
	FILTER_TYPE filtertype;
	int inner=0, outer=0;
	PROGRAMS transform;
	HISTORY stuff;

	required_type[0] = CVIP_FLOAT;
	num_type = 1;

	if (argc != 8) {
		sprintf(interp->result, "Usage: cvip brf_filter <image> <keep_dc(y/n)> <IDEAL_REJECT/BUTTER_REJECT> <lower cutoff_frequency(1-block_size)> <higher cutoff_frequency(1-block_size)> <Butterworth filter order(1-8)>\n For argument(s) you don't need, pass -1");
		return TCL_ERROR;
	}

	if (!strcmp(argv[4],"IDEAL_REJECT")) 
    	filtertype = IDEAL_REJECT;
	else if (!strcmp(argv[4],"BUTTER_REJECT"))
    	filtertype = BUTTER_REJECT;
	else 	{
		sprintf(interp->result, "filter type can only be IDEAL_REJECT or BUTTER_REJECT");
		return TCL_ERROR;
	}


	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
	
	imageP = convertImage_Queue(imageP,required_type,num_type);


	if(!check_xform_history(imageP,4,&transform)){
		sprintf(interp->result,"Input image doesn't have appropriate transform history");
		delete_Image(imageP);
		return TCL_ERROR;
	}

	if(argv[3][0] == 'y' || argv[3][0] == 'Y')
		dc = 1;
  switch(transform) {
    case WALSH:
       stuff=history_get(imageP, WALSH);
          GET_HIST_DATA(stuff,0,block_size);
       dcpos = 2; /* corner */
       break;
    case HADAMARD:
       stuff=history_get(imageP, HADAMARD);
          GET_HIST_DATA(stuff,0,block_size);
       dcpos = 2; 
       break;
    case FFT:
       stuff=history_get(imageP, FFT);
          GET_HIST_DATA(stuff,0,block_size);
       dcpos = 1;
       break;  
    case DCTXFORM:
       stuff=history_get(imageP, DCTXFORM);
          GET_HIST_DATA(stuff,0,block_size);
       dcpos=2;
       break;
    case HAAR:
       stuff=history_get(imageP, HAAR);
       GET_HIST_DATA(stuff,0,block_size);
       dcpos=2;
       break;
    default:
       dcpos=2;
       block_size=imageP->image_ptr[0]->rows;
       break;
  }

    inner = atoi(argv[5]);
    outer = atoi(argv[6]);
    order = atoi(argv[7]);
  switch(filtertype) {
    case IDEAL_REJECT:  /* Ideal Band Reject */ 
      if ( dcpos == 1) { /* dc in center */
        imageP=(Image *)Ideal_Band_Reject(imageP,block_size,dc,inner,outer);
      }
      else { /* 0 filter order since this is not Butterworth */
        imageP=(Image *)nonfft_xformfilter(imageP,block_size,dc,filtertype,inner,outer,0);
      }
        break;
    case BUTTER_REJECT: /* Butterworth Band Reject */
        if ( dcpos == 1) {
          imageP=(Image *)Butterworth_Band_Reject(imageP,block_size,dc,inner,outer,order);
        }
        else {
          imageP =(Image *)nonfft_xformfilter(imageP,block_size,dc,filtertype,inner, outer,order);
        }
    }
	S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;


	
}

static int
S_ImageHFE_Filter(ClientData clientData, Tcl_Interp *interp, int argc, char **argv){
	Image * imageP = (NULL), *tmpP = (NULL);
	CVIP_TYPE	required_type[5];
	int	num_type;
	int dc=0, dcpos, block_size,order, cut;
	float alfa;
	PROGRAMS transform;
	HISTORY stuff;

	required_type[0] = CVIP_FLOAT;
	num_type = 1;

	if (argc != 7) {
		sprintf(interp->result, "Usage: cvip hfe_filter <image> <keep_dc(y/n)> <cutoff_frequency(1-block_size)> <Emphasis offset Value> <Butterworth filter order(1-8)>\n For argument(s) you don't need, pass -1");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
	
	imageP = convertImage_Queue(imageP,required_type,num_type);


	if(!check_xform_history(imageP,5,&transform)){
		sprintf(interp->result,"Input image doesn't have appropriate transform history");
		delete_Image(imageP);
		return TCL_ERROR;
	}

	if(argv[3][0] == 'y' || argv[3][0] == 'Y')
		dc = 1;
  switch(transform) {
    case WALSH:
       stuff=history_get(imageP, WALSH);
          GET_HIST_DATA(stuff,0,block_size);
       dcpos = 2; /* corner */
       break;
    case HADAMARD:
       stuff=history_get(imageP, HADAMARD);
          GET_HIST_DATA(stuff,0,block_size);
       dcpos = 2; 
       break;
    case FFT:
       stuff=history_get(imageP, FFT);
          GET_HIST_DATA(stuff,0,block_size);
       dcpos = 1;
       break;  
    case DCTXFORM:
       stuff=history_get(imageP, DCTXFORM);
          GET_HIST_DATA(stuff,0,block_size);
       dcpos=2;
       break;
    case HAAR:
       stuff=history_get(imageP, HAAR);
       GET_HIST_DATA(stuff,0,block_size);
       dcpos=2;
       break;
    default:
       dcpos=2;
       block_size=imageP->image_ptr[0]->rows;
       break;
  }

    cut = atoi(argv[4]);
    alfa = atof(argv[5]);
    order = atoi(argv[6]);
  if ( dcpos == 1) { /* dc in center */
    imageP=(Image *)High_Freq_Emphasis(imageP,block_size,dc,cut,alfa,order);
  }
  else { 
    imageP=(Image *)nonfft_xformfilter(imageP,block_size,dc,HIGH_FREQ_EMPHASIS,
				cut,alfa,order);
  }
	S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;


	
}

static int
S_ImageViewer(ClientData clientData, Tcl_Interp *interp, int argc, char **argv )
{
	int row, col, bands;
	IMAGE_FORMAT format;
	COLOR_FORMAT color;
	char *data_type[] = {"CVIP_BYTE", "CVIP_SHORT", "CVIP_INTEGER", "CVIP_FLOAT", "CVIP_DOUBLE"};
	char *data_format[] = {"REAL", "COMPLEX"};
	Image *imageP;

	if(argc > 4){
		sprintf(interp->result,"Usage: cvip viewer [picture / RamViewer] [display format]");
		return TCL_ERROR;
	} else if (argc >= 3) {
		if (strcmp("picture", argv[2]) == 0) {
			setDisplay_Image(argv[2], "Default");
		} else if (strcmp("RamViewer", argv[2]) == 0) {
			setDisplay_Image(argv[2], "Default");
		} else if (argc == 4) {
			printf("User supplied viewer: %s selected, using %s as the display format\n",
				argv[2], argv[3]);
			setDisplay_Image(argv[2], argv[3]);
		} else if (argc == 3) {
			printf("User supplied viewer: %s selected, using default image format to display\n",
				argv[2]);
			setDisplay_Image(argv[2], "Default");
		}

	}
	sprintf(interp->result,"%s",getDisplay_Image());
	return TCL_OK;
}


static int
S_ImageRange(ClientData clientData, Tcl_Interp *interp, int argc, char **argv )
{
   Image       *imageP = NULL;
   CVIP_TYPE     dt;
   HISTORY  h_story, old_story;
   register    int i;
   unsigned    no_of_bands, imgsize, rows, cols;
   double      *minP = NULL, *maxP = NULL;
   void     **imgvecP = NULL, **imgvecPi = NULL;
   Image    rImage;
	char temp[20]={'\0'};
	FORMAT format;


	if (argc != 3) {
		sprintf(interp->result, "Usage: cvip datarange <image>");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;

	dt = getDataType_Image(imageP);
	format = getDataFormat_Image(imageP);

   no_of_bands = getNoOfBands_Image(imageP);
   rows = getNoOfRows_Image(imageP);
   cols = getNoOfCols_Image(imageP);
   imgsize = rows*cols;

   minP  = (double *) malloc(sizeof(double)*no_of_bands);
   maxP  = (double *) malloc(sizeof(double)*no_of_bands);
   imgvecP = (void **) malloc(sizeof(void *)*no_of_bands);
   getBandVector_Image((Image *)imageP, imgvecP);

   if(format == COMPLEX) {
		int count = 0;
   	imgvecPi = (void **) malloc(sizeof(void *)*no_of_bands);
		for (count = 0; count < no_of_bands; count ++) 
			imgvecPi[count] = getImagRow_Image(imageP, 0, count);
   }

   for(i=0; i < no_of_bands; i++) {
      minP[i] = findMinVal_Vector(imgvecP[i], imgsize, dt);
      maxP[i] = findMaxVal_Vector(imgvecP[i], imgsize, dt);
		sprintf(temp, "%.0f %.0f ", minP[i], maxP[i]);
		strcat(interp->result, temp);
		if(format == COMPLEX) {
      	minP[i] = findMinVal_Vector(imgvecPi[i], imgsize, dt);
			maxP[i] = findMaxVal_Vector(imgvecPi[i], imgsize, dt);
			sprintf(temp, "%.0f %.0f ", minP[i], maxP[i]);
			strcat(interp->result, temp);
		}
   }  
	free(minP); free(maxP);
	free(imgvecP); 
	if(format == COMPLEX) {
		free(imgvecPi);
	}

	return TCL_OK;
}

static int
S_BLACK(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
	Image 		*imageP;

	if (argc == 3) {
		if (atoi(argv[2]) < 1) {
			sprintf(interp->result,
				"Image size should be greater than or equal to 1");
			return TCL_ERROR;
		}
		imageP = create_black(atoi(argv[2]), atoi(argv[2]));
	} else if (argc == 4) {
		if (atoi(argv[2]) < 1 || atoi(argv[3]) < 1) {
			sprintf(interp->result,
				"Image size should be greater than or equal to 1");
			return TCL_ERROR;
		}
		imageP = create_black(atoi(argv[2]), atoi(argv[3]));
	} else {
		sprintf(interp->result,
			"Usage: cvip black <width> [<height>, if omitted, square image]");
		return TCL_ERROR;
	}

	
	if(imageP == NULL)
		return TCL_ERROR;

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}

static int
S_ImageH_Image(ClientData clientData,
		Tcl_Interp *interp, int argc, char **argv) {
	Image 		*imageP;

	if (argc != 5) {
		sprintf(interp->result,
		"Usage: cvip h_image <type(1/2/3 - constant, fixed mask, gaussian)> <height> <width> ");
		return TCL_ERROR;
	}

	if (atoi(argv[3]) < 3 || atoi(argv[4]) < 3) {
		sprintf(interp->result,
			"Improper mask sizes: height=%s, width=%s", argv[3], argv[4]);
		return TCL_ERROR;
	}

	if ( (imageP=h_image(atoi(argv[2]),atoi(argv[3]),atoi(argv[4])))
			== (Image *) NULL) {
		sprintf(interp->result,
			"Improper parameters: type=%s, height=%s, width=%s",
			argv[2], argv[3], argv[4]);
		return TCL_ERROR;
	}
	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);

	return TCL_OK;
}

static int
S_ImageDataGet(ClientData clientData, Tcl_Interp *interp,int argc,char **argv )
{
   Image       *imageP = NULL;
   CVIP_TYPE     dt;
   unsigned    no_of_bands, rows, cols, r, c;
	FORMAT format;
	float **data;
	char temp[20], *new_result = 0;
	char out_format[10];
	CVIP_TYPE	required_type[5];
	int	num_type, size;
	static int first_free = 1;
	
	required_type[0] = CVIP_FLOAT;
	num_type = 1;


	if (argc != 3) {
		sprintf(interp->result, "Usage: cvip data_get <single band real image>");
		return TCL_ERROR;
	}

	if(!(imageP = S_ImageGetFromHash(interp, argv[2]))) return TCL_ERROR;
   no_of_bands = getNoOfBands_Image(imageP);
   rows = getNoOfRows_Image(imageP);
   cols = getNoOfCols_Image(imageP);
	format = getDataFormat_Image(imageP);
   if(format == COMPLEX || no_of_bands != 1) {
		sprintf(interp->result,
			"This function only works on single band real image");
		return TCL_ERROR;
   }
	if (rows*cols > 912 ) {
		sprintf(interp->result,
		"The largest allowable image size is 900 pixels");
		return TCL_ERROR;
	}
	

	dt = getDataType_Image(imageP);
	switch (dt) {
		case CVIP_FLOAT: 
			strcpy(out_format, "%10.3f ");
			size = rows*cols*11;
			break;
		case CVIP_BYTE:
			strcpy(out_format, "%6.0f ");
			size = rows*cols*7;
			break;
		case CVIP_SHORT:
			strcpy(out_format, "%9.0f ");
			size = rows*cols*10;
			break;
		case CVIP_INTEGER:
			strcpy(out_format, "%15.0f ");
			size = rows*cols*16;
			break;
	}
	imageP = convertImage_Queue(imageP,required_type,num_type);
	data = (float **)getData_Image(imageP, 0);
	size = size + 12;

	/* The result buffer is enlarged, might cause problems */
/*	if (size > TCL_RESULT_SIZE) {
		new_result = (char*) malloc(sizeof(char) *(size + 1));
		if (new_result == NULL) {
			sprintf(interp->result,
			"Memory allocation error, image size too large");
			return TCL_ERROR;
		} else {
			if (first_free != 1) free(interp->result);
			else  {
				strcpy(new_result, interp->result); 
				first_free = 0;
			}
			interp->result = new_result;
		}
	} */
	sprintf(temp, "%5.0f %5.0f ", (float) rows, (float) cols);
	Tcl_AppendResult(interp, temp, (char *)NULL);
	for (r = 0; r < rows; r++) 
		for (c = 0; c < cols; c++) {
			sprintf(temp, out_format, data[r][c]);
			Tcl_AppendResult(interp, temp, (char *)NULL); 
	}
	return TCL_OK;
}

static int
S_ImageDATA_H_Image(ClientData clientData,
		Tcl_Interp *interp, int argc, char **argv) {
	Image 		*imageP;
	float **data;
	int rows, cols, r, c;

	if (argc < 5) {
		sprintf(interp->result,
		"Usage: cvip data_h_image <height> <width> <height x width data items> ");
		return TCL_ERROR;
	}

	rows = atoi(argv[2]); cols = atoi(argv[3]);
	if (rows <  1 || cols <  1) {
		sprintf(interp->result, "The image size should be at least 1 by 1");
		return TCL_ERROR;
	}

	if (argc != rows*cols + 4) {
		sprintf(interp->result, "There should be %d data items", rows*cols+2);
		return TCL_ERROR;
	}

	imageP = new_Image(PGM, GRAY_SCALE, 1, rows, cols, CVIP_FLOAT, REAL);
	data = (float **)getData_Image(imageP, 0);
	for (r = 0; r < rows; r++) 
		for (c = 0; c < cols; c++) {
			data[r][c] = (float) atof(argv[r*cols + c + 4]);
	}

	(void) S_ImageAddToHash(interp, imageP, (char *)NULL);
	return TCL_OK;
}

/* this function is used to generate the proper temporary file name
 * and the image name for compression algorithms
 */
static void
getTempFileName(Tcl_Interp *interp, char imageName[], CMPR_FORMAT format,
		char *filename, char *result_name) {
	Image *tempImage;

	sprintf(result_name,"%s_", Cmpr_Name[format]);
	strcat(result_name,imageName);

	while(tempImage = S_ImageGetFromHash(interp, result_name)) 
		strcat(result_name,"_r"); 

	strcpy(filename,temp_dir);
	strcat(filename,"/");
	strcat(filename,result_name);
	strcat(filename,"__"); strcat(filename, Cmpr_Name[format]);
	return;
}
