# **********************************************************************
# Computer Vision/Image Processing Tool Project - Dr. Scott Umbaugh SIUE
# **********************************************************************
#
#             File Name: restore_freq.tcl
#           Description: 
#         Related Files: 
#   Initial Coding Date: Wed Oct 30 15:13:20 CST 1996
#           Portability: TCL script file
#            References:
#             Author(s): Mark Zuke and Yansheng Wei
#                        Southern Illinois University @ Edwardsville
#
## Copyright (C) 1996 SIUE - by Scott Umbaugh, Mark Zuke  and Yansheng Wei.
##
## Permission to use, copy, modify, and distribute this software and its
## documentation for any purpose and without fee is hereby granted, provided
## that the above copyright notice appear in all copies and that both that
## copyright notice and this permission notice appear in supporting
## documentation.  This software is provided "as is" without express or
## implied warranty.
##
# RCS (Revision Control System) Information
# ...Added automatically by RCS 
#
## $Log: restore_freq.tcl,v $
## Revision 1.28  1997/08/06 00:09:27  yawei
## fixed scrollbar problem
##
## Revision 1.27  1997/07/02 16:47:28  yawei
## Gui configuration in place
##
## Revision 1.26  1997/06/16 21:16:54  yawei
## removed 3.1 bitmap file
##
## Revision 1.25  1997/06/03 00:38:51  smakam
## Removed the display of the temporary image of the laplacian mask in the least_squares function
##
## Revision 1.24  1997/03/27 01:58:03  yawei
## GUI ==> CVIPTCL/GUI_SCRIPTS
##
## Revision 1.23  1997/03/12 00:44:26  yawei
## Fixed upgrade bugs
##
## Revision 1.22  1997/03/11 20:56:27  yawei
## Tcl/Tk script upgrading from tcl7.3/tk3.6 to tcl7.6/tk4.2
## compatible. This process is automated with tkboost perl
## script. @ Tue Mar 11 14:51:36 CST 1997
##
## Revision 1.21  1997/01/17 22:03:10  yawei
## small change
##
# Revision 1.20  1997/01/17  21:00:33  yawei
# power_spect_equal ==> power_spect_eq
#
# Revision 1.19  1997/01/08  18:22:39  mzuke
# changed suffix names of Wiener filters
#
# Revision 1.18  1996/12/31  22:28:40  yawei
# log-power ==> log_power
#
# Revision 1.17  1996/12/30  20:02:21  yawei
# square ==> rectangle
#
# Revision 1.16  1996/12/30  19:44:44  yawei
# power_spect_eqize ==> power_spect_eq
#
# Revision 1.15  1996/12/30  17:25:45  yawei
# wiener_filter ==> wiener
#
# Revision 1.14  1996/12/09  20:12:53  akjoele
# corrected spelling error: freq_filter(alpha) -> freq_filt(alpha)
#
# Revision 1.13  1996/12/09  19:24:20  akjoele
# Added geometric_mean()
#
# Revision 1.12  1996/12/09  18:13:21  akjoele
# Added power_spect_eq() and changed alpha for future geometric_mean()
#
# Revision 1.11  1996/12/06  01:09:08  yawei
# Changed:
#  "let F(u,v) = G(u,v)" ==> "Let R(u,v) = 1"
#  "Let F(u,v) = 0" ==> "Let R(u,v) = 0"
#
# Revision 1.10  1996/12/05  03:39:07  yawei
# removed the "masks" display on execution.
#
# Revision 1.9  1996/11/29  23:28:34  yawei
# Added parametric_wiener and wiener filters
#
# Revision 1.8  1996/11/26  21:56:16  yawei
# minor changes
#
# Revision 1.7  1996/11/08  00:00:18  yawei
# Removed boundary restrinctions on Gamma for LCS,
# and added more defaults for the weight of h_image
#
# Revision 1.6  1996/11/07  22:58:13  yawei
# Everything about the h_image is working fine now
#
# Revision 1.5  1996/11/06  19:51:46  yawei
# fixed problem of notch filter brought by degradation functions.
#
# Revision 1.4  1996/11/06  18:21:52  yawei
# degradation functions begins to work
#
# Revision 1.3  1996/11/04  03:07:05  yawei
# Incomplete degradation function generations
#
# Revision 1.2  1996/11/03  03:26:28  yawei
# Changed parameters for masks
#
# Revision 1.1  1996/10/30  21:39:56  yawei
# Initial revision
#
#
#****************************************************************************
proc notch_trans { transform }  {
	global {freq_filt}
	global {topwin}
	global {GUI_DEBUG}
	global {current_image}
	$topwin(restore_name) configure -cursor watch; update
	if {$current_image == ""} {
		CvipAlert "Please select an image first."
		$topwin(restore_name) configure -cursor top_left_arrow; update
		return
	}  

	scan [cvip info $current_image] "%s %s %s %s %s %s %s" temp temp temp rows cols temp temp
	set temp [expr { $rows > $cols } ? $cols : $rows ]
	set temp1 [expr int(pow(2, floor(log($temp) / log(2.0))))]

	if { $freq_filt(notch_spect) == "" } {
		if [catch {set temp [cvip $transform $current_image 1 $temp1]} cresult] {
			CvipAlert "$cresult"
			$topwin(restore_name) configure -cursor top_left_arrow; update
			return
		} else {
			set new_name [cvip rename $temp ${current_image}_$transform]
			set freq_filt(notch_spect) $new_name
			cvip view $new_name l
		}
	}
	set freq_filt(notch_mask) ""
	if { $GUI_DEBUG == 1 } {
		puts stderr "cvip $transform $current_image 1 $temp1"
	}
	$topwin(restore_name) configure -cursor top_left_arrow; update
}

proc get_rc { } {
	global {freq_filt}
	global {topwin}
	global {current_image}
	global {GUI_DEBUG}
	if {$freq_filt(notch_spect) == ""} {
		CvipAlert "Spectrum image doesn't exist."
		return
	}
	
	scan [cvip info $current_image] "%s %s %s %s %s %s %s" temp temp temp height width temp temp

	if [ catch {set temp [selection get] } ] {
	} elseif  { [scan $temp "%s %s %s %s %s %s" temp1 temp4 temp2 temp3 temp5 temp6] == "6" && \
					$temp1 == "\[log_power\]$freq_filt(notch_spect)" } {

		if [catch {expr $temp2 + $temp3 + $freq_filt(size) } ] {
			CvipAlert "Numerical values required for the notch filter parameters:\n\
			Row = $temp2, Column = $temp3, Size = $freq_filt(size)."
			return
		} elseif { $temp2 < 0 || $temp2 > [expr $height - 1]} {
			CvipAlert "The row position of the notch filter ($temp2) should be in the range:\
			\[0 - [expr $height - 1]\]."
			return
		} elseif {$temp3 < 0 || $temp3 > [expr $width - 1] } {
			CvipAlert "The column position of the notch filter ($temp3) should be in the range:\
			\[0 - [expr $width - 1]\]."
			return
		}
		set freq_filt(row)  $temp2	
		set freq_filt(col)  $temp3	
		set temp [ expr { $temp5 > $temp6 } ? $temp6 : $temp5 ];
		if { $temp > 0 } { set freq_filt(size) $temp } else { set freq_filt(size) 1 }
	}
	append freq_filt(notch_mask)\
		"\{$freq_filt(row), $freq_filt(col), $freq_filt(size)\} "

}


# procedure to show window .
proc freq_filter { } {# xf ignore me 7

  # Window manager configurations
  global env;
  global tk_version
  global toplevel_position
	global {GUI_DEBUG}

  global {topwin}
  global {GuiColor}
  global {GuiMisc}
  global {restore}
  if [ winfo exists $topwin(restore_name).info] {
     set tempname $topwin(restore_name);\
destroy $topwin(restore_name).info;\
set topwin(restore_name) $tempname;}
  if [ winfo exists $topwin(restore_name).current_image] {
     set tempname $topwin(restore_name);\
destroy $topwin(restore_name).current_image;\
set topwin(restore_name) $tempname;}
  if [ winfo exists $topwin(restore_name).perform] {
     set tempname $topwin(restore_name);\
destroy $topwin(restore_name).perform;\
set topwin(restore_name) $tempname;}
  bind $topwin(restore_name) <Destroy> {set topwin(restore_name) NONE ; \
	if { $freq_filt(notch_spect) != "" } {cvip dq $freq_filt(notch_spect); set freq_filt(notch_spect) "" } }

  wm positionfrom $topwin(restore_name) user
  wm sizefrom $topwin(restore_name) user

  wm geometry $topwin(restore_name) $topwin(restore_freqency)[get_pos $topwin(restore_name)]
  wm maxsize $topwin(restore_name) [lindex $topwin(restore_freqency_max) 0] \
  	[lindex $topwin(restore_freqency_max) 1]
  wm minsize $topwin(restore_name) [lindex $topwin(restore_freqency_min) 0] \
  	[lindex $topwin(restore_freqency_min) 1]
  wm minsize $topwin(restore_name) 706 606

  wm title $topwin(restore_name) {RESTORATION -> Frequency filters}

  # build widget $topwin(restore_name).info
  frame $topwin(restore_name).info \
    -background "$GuiColor(frame_bg)"

  # build widget $topwin(restore_name).info.frame11
  frame $topwin(restore_name).info.frame11 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {3} \
    -relief {ridge}

  # build widget $topwin(restore_name).info.frame11.info
  frame $topwin(restore_name).info.frame11.info \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(restore_name).info.frame11.info.frame0
  frame $topwin(restore_name).info.frame11.info.frame0 \
    -background "$GuiColor(frame_bg)" \
    -relief {ridge}

  # build widget $topwin(restore_name).info.frame11.info.frame0.radiobutton18
	radiobutton $topwin(restore_name).info.frame11.info.frame0.radiobutton18 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -command {set freq_filt(notch_spect) ""} \
    -relief {flat} \
    -text {Notch} \
    -value {notch} \
    -variable {freq_filt(type)}

  # build widget $topwin(restore_name).info.frame11.info.frame0.frame5
  frame $topwin(restore_name).info.frame11.info.frame0.frame5 \
    -background "$GuiColor(frame_bg)"

  # build widget $topwin(restore_name).info.frame11.info.frame0.frame5.frame0
  frame $topwin(restore_name).info.frame11.info.frame0.frame5.frame0 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(restore_name).info.frame11.info.frame0.frame5.frame0.scrollbar11
  scrollbar $topwin(restore_name).info.frame11.info.frame0.frame5.frame0.scrollbar11 \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -borderwidth {1} \
    -command {delta freq_filt(row)} \
    -relief {sunken} \
    -repeatinterval {80} \
    -width {13}

  # build widget $topwin(restore_name).info.frame11.info.frame0.frame5.frame0.label1
  label $topwin(restore_name).info.frame11.info.frame0.frame5.frame0.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -text {Row: }

  # build widget $topwin(restore_name).info.frame11.info.frame0.frame5.frame0.value
  entry $topwin(restore_name).info.frame11.info.frame0.frame5.frame0.value \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
 -xscrollcommand "$topwin(restore_name).info.frame11.info.frame0.frame5.frame0.scrollbar11 set" \
    -textvariable {freq_filt(row)} \
    -width {4}
  endEntry $topwin(restore_name).info.frame11.info.frame0.frame5.frame0.value 1 \
  	 $topwin(restore_name).info.frame11.info.frame0.frame5.frame1.value \
  	 $topwin(restore_name).info.frame11.info.frame0.frame5.frame2.value 
	
  # build widget $topwin(restore_name).info.frame11.info.frame0.frame5.frame0.menubutton2
	menubutton $topwin(restore_name).info.frame11.info.frame0.frame5.frame0.menubutton2 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)"\
    -text "$GuiMisc(pop_up_text)" \
	 -width {2}
  # bindings
  bind $topwin(restore_name).info.frame11.info.frame0.frame5.frame0.menubutton2 <Button-1> {MenuPopupPost $topwin(restore_name).info.frame11.info.frame0.frame5.frame0.menubutton2.menu %X %Y}
  bind $topwin(restore_name).info.frame11.info.frame0.frame5.frame0.menubutton2 <ButtonRelease-1> {MenuPopupRelease $topwin(restore_name).info.frame11.info.frame0.frame5.frame0.menubutton2.menu %W}

  # build widget $topwin(restore_name).info.frame11.info.frame0.frame5.frame0.menubutton2.menu
  menu $topwin(restore_name).info.frame11.info.frame0.frame5.frame0.menubutton2.menu \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"
  $topwin(restore_name).info.frame11.info.frame0.frame5.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(row) 8; selection clear} \
    -label {   8}
  $topwin(restore_name).info.frame11.info.frame0.frame5.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(row) 16; selection clear} \
    -label {  16}
  $topwin(restore_name).info.frame11.info.frame0.frame5.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(row) 32; selection clear} \
    -label {  32}
  $topwin(restore_name).info.frame11.info.frame0.frame5.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(row) 64; selection clear} \
    -label {  64}
  $topwin(restore_name).info.frame11.info.frame0.frame5.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(row) 128; selection clear} \
    -label { 128}
  $topwin(restore_name).info.frame11.info.frame0.frame5.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(row) 256; selection clear} \
    -label { 256}
  $topwin(restore_name).info.frame11.info.frame0.frame5.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(row) 512; selection clear} \
    -label { 512 }
  bindtags $topwin(restore_name).info.frame11.info.frame0.frame5.frame0.menubutton2.menu {Menu all}
  # bindings
  bind $topwin(restore_name).info.frame11.info.frame0.frame5.frame0.menubutton2.menu <B1-Motion> {MenuPopupMotion $topwin(restore_name).info.frame11.info.frame0.frame5.frame0.menubutton2.menu %W %X %Y}
  bind $topwin(restore_name).info.frame11.info.frame0.frame5.frame0.menubutton2.menu <ButtonRelease-1> {MenuPopupRelease $topwin(restore_name).info.frame11.info.frame0.frame5.frame0.menubutton2.menu %W}

  # pack widget $topwin(restore_name).info.frame11.info.frame0.frame5.frame0
  pack append $topwin(restore_name).info.frame11.info.frame0.frame5.frame0 \
    $topwin(restore_name).info.frame11.info.frame0.frame5.frame0.label1 {left frame center fill} \
    $topwin(restore_name).info.frame11.info.frame0.frame5.frame0.value {left frame center expand fill} \
    $topwin(restore_name).info.frame11.info.frame0.frame5.frame0.menubutton2 {right frame center fill} \
    $topwin(restore_name).info.frame11.info.frame0.frame5.frame0.scrollbar11 {left frame center fill}

  # build widget $topwin(restore_name).info.frame11.info.frame0.frame5.frame1
  frame $topwin(restore_name).info.frame11.info.frame0.frame5.frame1 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(restore_name).info.frame11.info.frame0.frame5.frame1.scrollbar11
  scrollbar $topwin(restore_name).info.frame11.info.frame0.frame5.frame1.scrollbar11 \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -borderwidth {1} \
    -command {delta freq_filt(col)} \
    -relief {sunken} \
    -repeatinterval {80} \
    -width {13}

  # build widget $topwin(restore_name).info.frame11.info.frame0.frame5.frame1.label1
  label $topwin(restore_name).info.frame11.info.frame0.frame5.frame1.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -text {Column: }

  # build widget $topwin(restore_name).info.frame11.info.frame0.frame5.frame1.value
  entry $topwin(restore_name).info.frame11.info.frame0.frame5.frame1.value \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
 -xscrollcommand "$topwin(restore_name).info.frame11.info.frame0.frame5.frame1.scrollbar11 set" \
    -textvariable {freq_filt(col)} \
    -width {4}
  endEntry $topwin(restore_name).info.frame11.info.frame0.frame5.frame1.value 1 \
  	 $topwin(restore_name).info.frame11.info.frame0.frame5.frame2.value \
  	 $topwin(restore_name).info.frame11.info.frame0.frame5.frame0.value 

  # build widget $topwin(restore_name).info.frame11.info.frame0.frame5.frame1.menubutton2
	menubutton $topwin(restore_name).info.frame11.info.frame0.frame5.frame1.menubutton2 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)"\
    -text "$GuiMisc(pop_up_text)" \
	 -width {2}
  # bindings
  bind $topwin(restore_name).info.frame11.info.frame0.frame5.frame1.menubutton2 <Button-1> {MenuPopupPost $topwin(restore_name).info.frame11.info.frame0.frame5.frame1.menubutton2.menu %X %Y}
  bind $topwin(restore_name).info.frame11.info.frame0.frame5.frame1.menubutton2 <ButtonRelease-1> {MenuPopupRelease $topwin(restore_name).info.frame11.info.frame0.frame5.frame1.menubutton2.menu %W}

  # build widget $topwin(restore_name).info.frame11.info.frame0.frame5.frame1.menubutton2.menu
  menu $topwin(restore_name).info.frame11.info.frame0.frame5.frame1.menubutton2.menu \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"
  $topwin(restore_name).info.frame11.info.frame0.frame5.frame1.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(col) 8; selection clear} \
    -label {   8}
  $topwin(restore_name).info.frame11.info.frame0.frame5.frame1.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(col) 16; selection clear} \
    -label {  16}
  $topwin(restore_name).info.frame11.info.frame0.frame5.frame1.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(col) 32; selection clear} \
    -label {  32}
  $topwin(restore_name).info.frame11.info.frame0.frame5.frame1.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(col) 64; selection clear} \
    -label {  64}
  $topwin(restore_name).info.frame11.info.frame0.frame5.frame1.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(col) 128; selection clear} \
    -label { 128}
  $topwin(restore_name).info.frame11.info.frame0.frame5.frame1.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(col) 256; selection clear} \
    -label { 256}
  $topwin(restore_name).info.frame11.info.frame0.frame5.frame1.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(col) 512; selection clear} \
    -label { 512 }
  bindtags $topwin(restore_name).info.frame11.info.frame0.frame5.frame1.menubutton2.menu {Menu all}
  # bindings
  bind $topwin(restore_name).info.frame11.info.frame0.frame5.frame1.menubutton2.menu <B1-Motion> {MenuPopupMotion $topwin(restore_name).info.frame11.info.frame0.frame5.frame1.menubutton2.menu %W %X %Y}
  bind $topwin(restore_name).info.frame11.info.frame0.frame5.frame1.menubutton2.menu <ButtonRelease-1> {MenuPopupRelease $topwin(restore_name).info.frame11.info.frame0.frame5.frame1.menubutton2.menu %W}

  # pack widget $topwin(restore_name).info.frame11.info.frame0.frame5.frame1
  pack append $topwin(restore_name).info.frame11.info.frame0.frame5.frame1 \
    $topwin(restore_name).info.frame11.info.frame0.frame5.frame1.label1 {left frame center fill} \
    $topwin(restore_name).info.frame11.info.frame0.frame5.frame1.value {left frame center expand fill} \
    $topwin(restore_name).info.frame11.info.frame0.frame5.frame1.menubutton2 {right frame center fill} \
    $topwin(restore_name).info.frame11.info.frame0.frame5.frame1.scrollbar11 {left frame center fill}

  # build widget $topwin(restore_name).info.frame11.info.frame0.frame5.frame2
  frame $topwin(restore_name).info.frame11.info.frame0.frame5.frame2 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(restore_name).info.frame11.info.frame0.frame5.frame2.scrollbar11
  scrollbar $topwin(restore_name).info.frame11.info.frame0.frame5.frame2.scrollbar11 \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -borderwidth {1} \
    -command {delta freq_filt(size)} \
    -relief {sunken} \
    -repeatinterval {80} \
    -width {13}

  # build widget $topwin(restore_name).info.frame11.info.frame0.frame5.frame2.label1
  label $topwin(restore_name).info.frame11.info.frame0.frame5.frame2.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -text {Size: }

  # build widget $topwin(restore_name).info.frame11.info.frame0.frame5.frame2.value
  entry $topwin(restore_name).info.frame11.info.frame0.frame5.frame2.value \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
 -xscrollcommand "$topwin(restore_name).info.frame11.info.frame0.frame5.frame2.scrollbar11 set" \
    -textvariable {freq_filt(size)} \
    -width {3}
  endEntry $topwin(restore_name).info.frame11.info.frame0.frame5.frame2.value 1 \
  	 $topwin(restore_name).info.frame11.info.frame0.frame5.frame0.value \
  	 $topwin(restore_name).info.frame11.info.frame0.frame5.frame1.value 

  # build widget $topwin(restore_name).info.frame11.info.frame0.frame5.frame2.menubutton2
	menubutton $topwin(restore_name).info.frame11.info.frame0.frame5.frame2.menubutton2 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)"\
    -text "$GuiMisc(pop_up_text)" \
	 -width {2}
  # bindings
  bind $topwin(restore_name).info.frame11.info.frame0.frame5.frame2.menubutton2 <Button-1> {MenuPopupPost $topwin(restore_name).info.frame11.info.frame0.frame5.frame2.menubutton2.menu %X %Y}
  bind $topwin(restore_name).info.frame11.info.frame0.frame5.frame2.menubutton2 <ButtonRelease-1> {MenuPopupRelease $topwin(restore_name).info.frame11.info.frame0.frame5.frame2.menubutton2.menu %W}

  # build widget $topwin(restore_name).info.frame11.info.frame0.frame5.frame2.menubutton2.menu
  menu $topwin(restore_name).info.frame11.info.frame0.frame5.frame2.menubutton2.menu \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"
  $topwin(restore_name).info.frame11.info.frame0.frame5.frame2.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(size) 1; selection clear} \
    -label {   1}
  $topwin(restore_name).info.frame11.info.frame0.frame5.frame2.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(size) 2; selection clear} \
    -label {   2}
  $topwin(restore_name).info.frame11.info.frame0.frame5.frame2.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(size) 3; selection clear} \
    -label {   3}
  $topwin(restore_name).info.frame11.info.frame0.frame5.frame2.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(size) 4; selection clear} \
    -label {   4}
  $topwin(restore_name).info.frame11.info.frame0.frame5.frame2.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(size) 5; selection clear} \
    -label {   5}
  $topwin(restore_name).info.frame11.info.frame0.frame5.frame2.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(size) 10; selection clear} \
    -label {  10}
  $topwin(restore_name).info.frame11.info.frame0.frame5.frame2.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(size) 20; selection clear} \
    -label {  20}
  $topwin(restore_name).info.frame11.info.frame0.frame5.frame2.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(size) 50; selection clear} \
    -label {  50}
  $topwin(restore_name).info.frame11.info.frame0.frame5.frame2.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(size) 100; selection clear} \
    -label { 100 }
  bindtags $topwin(restore_name).info.frame11.info.frame0.frame5.frame2.menubutton2.menu  {Menu all}
  # bindings
  bind $topwin(restore_name).info.frame11.info.frame0.frame5.frame2.menubutton2.menu <B1-Motion> {MenuPopupMotion $topwin(restore_name).info.frame11.info.frame0.frame5.frame2.menubutton2.menu %W %X %Y}
  bind $topwin(restore_name).info.frame11.info.frame0.frame5.frame2.menubutton2.menu <ButtonRelease-1> {MenuPopupRelease $topwin(restore_name).info.frame11.info.frame0.frame5.frame2.menubutton2.menu %W}

  # pack widget $topwin(restore_name).info.frame11.info.frame0.frame5.frame2
  pack append $topwin(restore_name).info.frame11.info.frame0.frame5.frame2 \
    $topwin(restore_name).info.frame11.info.frame0.frame5.frame2.label1 {left frame center fill} \
    $topwin(restore_name).info.frame11.info.frame0.frame5.frame2.value {left frame center expand fill} \
    $topwin(restore_name).info.frame11.info.frame0.frame5.frame2.menubutton2 {right frame center fill} \
    $topwin(restore_name).info.frame11.info.frame0.frame5.frame2.scrollbar11 {left frame center fill}

  # build widget $topwin(restore_name).info.frame11.info.frame0.frame5.button6
	button $topwin(restore_name).info.frame11.info.frame0.frame5.button6 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(alert_act_bg)" \
    -background "$GuiColor(alert_bg)" \
    -foreground "$GuiColor(alert_fg)" \
	 -command {get_rc
	if { $GUI_DEBUG == 1 } {puts stderr $freq_filt(notch_mask)} } \
    -text {Get row-col}

  # build widget $topwin(restore_name).info.frame11.info.frame0.frame5.button7
	button $topwin(restore_name).info.frame11.info.frame0.frame5.button7 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(alert_act_bg)" \
    -background "$GuiColor(alert_bg)" \
    -foreground "$GuiColor(alert_fg)" \
	 -command { set freq_filt(notch_mask) "" } \
    -text {Redo all}

  # pack widget $topwin(restore_name).info.frame11.info.frame0.frame5
  pack append $topwin(restore_name).info.frame11.info.frame0.frame5 \
    $topwin(restore_name).info.frame11.info.frame0.frame5.frame0 {left frame center expand fillx} \
    $topwin(restore_name).info.frame11.info.frame0.frame5.frame1 {left frame center expand} \
    $topwin(restore_name).info.frame11.info.frame0.frame5.frame2 {left frame center expand} \
    $topwin(restore_name).info.frame11.info.frame0.frame5.button6 {left frame center expand} \
    $topwin(restore_name).info.frame11.info.frame0.frame5.button7 {left frame center expand}

  # build widget $topwin(restore_name).info.frame11.info.frame0.frame8
  frame $topwin(restore_name).info.frame11.info.frame0.frame8 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {4}

  # build widget $topwin(restore_name).info.frame11.info.frame0.frame8.menubutton2
	menubutton $topwin(restore_name).info.frame11.info.frame0.frame8.menubutton2 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -pady {4} \
    -relief {raised} \
    -text {Select a transform}
  # bindings
  bind $topwin(restore_name).info.frame11.info.frame0.frame8.menubutton2 <Button-1> {MenuPopupPost $topwin(restore_name).info.frame11.info.frame0.frame8.menubutton2.menu %X %Y}
  bind $topwin(restore_name).info.frame11.info.frame0.frame8.menubutton2 <ButtonRelease-1> {MenuPopupRelease $topwin(restore_name).info.frame11.info.frame0.frame8.menubutton2.menu %W}

  # build widget $topwin(restore_name).info.frame11.info.frame0.frame8.menubutton2.menu
  menu $topwin(restore_name).info.frame11.info.frame0.frame8.menubutton2.menu \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"
  $topwin(restore_name).info.frame11.info.frame0.frame8.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {
	if { $freq_filt(notch_spect) != "" } {cvip dq $freq_filt(notch_spect); set freq_filt(notch_spect) "" }
#	 cvip load $current_image
	set freq_filt(type) notch
	 set freq_filt(notch_transform) fft; set freq_filt(notch_mask) ""
	 notch_trans fft
	 pack $topwin(restore_name).info.frame11.info.frame0.frame5 -side left} \
    -label {FFT}

  $topwin(restore_name).info.frame11.info.frame0.frame8.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {
	if { $freq_filt(notch_spect) != "" } {cvip dq $freq_filt(notch_spect); set freq_filt(notch_spect) "" }
#	 cvip load $current_image 
	set freq_filt(type) notch;  set freq_filt(notch_mask) ""
	 set freq_filt(notch_transform) dct
	 notch_trans dct
	 pack $topwin(restore_name).info.frame11.info.frame0.frame5 -side left} \
    -label {DCT}

  $topwin(restore_name).info.frame11.info.frame0.frame8.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {
	if { $freq_filt(notch_spect) != "" } {cvip dq $freq_filt(notch_spect); set freq_filt(notch_spect) "" }
#	 cvip load $current_image 
	set freq_filt(type) notch;  set freq_filt(notch_mask) ""
	 set freq_filt(notch_transform) haar
	 notch_trans haar
	 pack $topwin(restore_name).info.frame11.info.frame0.frame5 -side left} \
    -label {HAAR}

  $topwin(restore_name).info.frame11.info.frame0.frame8.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {
	if { $freq_filt(notch_spect) != "" } {cvip dq $freq_filt(notch_spect); set freq_filt(notch_spect) "" }
#	 cvip load $current_image 
	 set freq_filt(notch_transform) walsh
	set freq_filt(type) notch;  set freq_filt(notch_mask) ""
	 notch_trans walsh
	 pack $topwin(restore_name).info.frame11.info.frame0.frame5 -side left} \
    -label {WALSH}

  $topwin(restore_name).info.frame11.info.frame0.frame8.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {
	if { $freq_filt(notch_spect) != "" } {cvip dq $freq_filt(notch_spect); set freq_filt(notch_spect) "" }
#	 cvip load $current_image 
	 set freq_filt(notch_transform) hadamard
	set freq_filt(type) notch;  set freq_filt(notch_mask) ""
	 notch_trans hadamard
	 pack $topwin(restore_name).info.frame11.info.frame0.frame5 -side left} \
    -label {HADAMARD}
  bindtags $topwin(restore_name).info.frame11.info.frame0.frame8.menubutton2.menu {Menu all}
  # bindings
  bind $topwin(restore_name).info.frame11.info.frame0.frame8.menubutton2.menu <B1-Motion> {MenuPopupMotion $topwin(restore_name).info.frame11.info.frame0.frame8.menubutton2.menu %W %X %Y}
  bind $topwin(restore_name).info.frame11.info.frame0.frame8.menubutton2.menu <ButtonRelease-1> {MenuPopupRelease $topwin(restore_name).info.frame11.info.frame0.frame8.menubutton2.menu %W}

  # pack widget $topwin(restore_name).info.frame11.info.frame0.frame8
  pack append $topwin(restore_name).info.frame11.info.frame0.frame8 \
    $topwin(restore_name).info.frame11.info.frame0.frame8.menubutton2 {left frame e padx 10 pady 6}

  # pack widget $topwin(restore_name).info.frame11.info.frame0
  pack append $topwin(restore_name).info.frame11.info.frame0 \
    $topwin(restore_name).info.frame11.info.frame0.radiobutton18 {left frame w} \
    $topwin(restore_name).info.frame11.info.frame0.frame8 {left frame center expand fillx} 
#    $topwin(restore_name).info.frame11.info.frame0.frame5 {left frame center}

  # pack widget $topwin(restore_name).info.frame11.info
  pack append $topwin(restore_name).info.frame11.info \
    $topwin(restore_name).info.frame11.info.frame0 {top frame w expand fillx}

  # pack widget $topwin(restore_name).info.frame11
  pack append $topwin(restore_name).info.frame11 \
    $topwin(restore_name).info.frame11.info {top frame center expand fillx}

  # build widget $topwin(restore_name).info.frame13
  frame $topwin(restore_name).info.frame13 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {3} \
    -relief {ridge}

  # build widget $topwin(restore_name).info.frame13.frame1
  frame $topwin(restore_name).info.frame13.frame1 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2} \
    -relief {ridge}

  # build widget $topwin(restore_name).info.frame13.frame1.frame6
  frame $topwin(restore_name).info.frame13.frame1.frame6 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(restore_name).info.frame13.frame1.frame6.radiobutton16
	radiobutton $topwin(restore_name).info.frame13.frame1.frame6.radiobutton16 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -command {
set freq_filt(alpha) 0
pack forget $topwin(restore_name).info.frame13.frame5.frame1
pack forget $topwin(restore_name).info.frame13.frame5.frame2
pack forget $topwin(restore_name).info.frame13.frame5.frame6
pack forget $topwin(restore_name).info.frame13.frame2.frame0.frame1
pack forget $topwin(restore_name).info.frame13.frame4.frame1
pack forget $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame4.frame3
pack forget $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame2.frame1
pack forget $topwin(restore_name).info.frame11.info.frame0.frame5
pack forget $topwin(restore_name).info.frame13.frame1.frame0.frame4
pack forget $topwin(restore_name).info.frame13.frame1.frame0.frame3
pack $topwin(restore_name).info.frame13.frame5.frame0 -side left
pack $topwin(restore_name).info.frame13.frame1.frame0.frame0 -side left 
pack $topwin(restore_name).info.frame13.frame1.frame0.frame1 -side left
} \
    -relief {flat} \
    -text {Practical Wiener} \
    -value {simple_wiener} \
    -variable {freq_filt(type)}

  # build widget $topwin(restore_name).info.frame13.frame1.frame6.radiobutton15
	radiobutton $topwin(restore_name).info.frame13.frame1.frame6.radiobutton15 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -command {
set freq_filt(alpha) 0
set freq_filt(gamma) 1
pack forget $topwin(restore_name).info.frame13.frame5.frame0
pack forget $topwin(restore_name).info.frame13.frame5.frame1
pack forget $topwin(restore_name).info.frame13.frame5.frame2
pack forget $topwin(restore_name).info.frame13.frame5.frame6
pack forget $topwin(restore_name).info.frame13.frame2.frame0.frame1
pack forget $topwin(restore_name).info.frame11.info.frame0.frame5 
pack forget $topwin(restore_name).info.frame13.frame1.frame0.frame1
pack $topwin(restore_name).info.frame13.frame4.frame1
pack $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame4.frame3
pack $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame2.frame1
pack $topwin(restore_name).info.frame13.frame1.frame0.frame0 -side left
pack $topwin(restore_name).info.frame13.frame1.frame0.frame4 -side left
pack $topwin(restore_name).info.frame13.frame1.frame0.labelnull -side right



} \
    -relief {flat} \
    -text {Wiener} \
    -value {wiener} \
    -variable {freq_filt(type)}

  # build widget $topwin(restore_name).info.frame13.frame1.frame6.radiobutton17
	radiobutton $topwin(restore_name).info.frame13.frame1.frame6.radiobutton17 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -command {
set freq_filt(alpha) 0
pack forget $topwin(restore_name).info.frame13.frame5.frame0
pack forget $topwin(restore_name).info.frame13.frame5.frame1
pack forget $topwin(restore_name).info.frame13.frame5.frame2
pack forget $topwin(restore_name).info.frame13.frame2.frame0.frame1
pack forget $topwin(restore_name).info.frame11.info.frame0.frame5 
pack forget $topwin(restore_name).info.frame13.frame1.frame0.frame1
pack forget $topwin(restore_name).info.frame13.frame1.frame0.frame4
pack $topwin(restore_name).info.frame13.frame5.frame6 -side left
pack  $topwin(restore_name).info.frame13.frame4.frame1
pack  $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame4.frame3
pack  $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame2.frame1
pack $topwin(restore_name).info.frame13.frame1.frame0.labelnull -side right
} \
    -relief {flat} \
    -text {Parametric Wiener} \
    -value {parametric_wiener} \
    -variable {freq_filt(type)}

  # pack widget $topwin(restore_name).info.frame13.frame1.frame6
  pack append $topwin(restore_name).info.frame13.frame1.frame6 \
    $topwin(restore_name).info.frame13.frame1.frame6.radiobutton15 {top frame w padx 4} \
    $topwin(restore_name).info.frame13.frame1.frame6.radiobutton16 {top frame w padx 14} \
    $topwin(restore_name).info.frame13.frame1.frame6.radiobutton17 {top frame w}

  # build widget $topwin(restore_name).info.frame13.frame1.frame19
  frame $topwin(restore_name).info.frame13.frame1.frame19 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(restore_name).info.frame13.frame1.frame19.radiobutton16
	radiobutton $topwin(restore_name).info.frame13.frame1.frame19.radiobutton16 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -command {
set freq_filt(alpha) 0
set freq_filt(gamma) 0
pack forget $topwin(restore_name).info.frame13.frame5.frame0
pack forget $topwin(restore_name).info.frame13.frame5.frame1
pack forget $topwin(restore_name).info.frame13.frame5.frame2
pack forget $topwin(restore_name).info.frame13.frame5.frame6
pack forget $topwin(restore_name).info.frame13.frame4.frame1
pack forget $topwin(restore_name).info.frame11.info.frame0.frame5
pack forget $topwin(restore_name).info.frame13.frame1.frame0.frame1
pack $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame4.frame3
pack $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame2.frame1
pack $topwin(restore_name).info.frame13.frame1.frame0.frame0 -side left
pack $topwin(restore_name).info.frame13.frame1.frame0.frame4 -side left
pack $topwin(restore_name).info.frame13.frame1.frame0.labelnull -side right} \
    -relief {flat} \
    -text {Inverse} \
    -value {inverse_xformfilter} \
    -variable {freq_filt(type)}

  # build widget $topwin(restore_name).info.frame13.frame1.frame19.radiobutton15
	radiobutton $topwin(restore_name).info.frame13.frame1.frame19.radiobutton15 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -command {
set freq_filt(alpha) 0.5
set freq_filt(gamma) 1
pack forget $topwin(restore_name).info.frame13.frame5.frame0
pack forget $topwin(restore_name).info.frame13.frame5.frame1
pack forget $topwin(restore_name).info.frame13.frame5.frame2
pack forget $topwin(restore_name).info.frame13.frame5.frame6
pack forget $topwin(restore_name).info.frame13.frame2.frame0.frame1
pack forget $topwin(restore_name).info.frame11.info.frame0.frame5 
pack forget $topwin(restore_name).info.frame13.frame1.frame0.frame1
pack $topwin(restore_name).info.frame13.frame4.frame1
pack $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame4.frame3
pack $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame2.frame1
pack $topwin(restore_name).info.frame13.frame1.frame0.frame0 -side left
pack $topwin(restore_name).info.frame13.frame1.frame0.frame4 -side left
pack $topwin(restore_name).info.frame13.frame1.frame0.labelnull -side right

} \
    -relief {flat} \
    -text {Power Spectrum Equalization} \
    -value {power_spect_eq} \
    -variable {freq_filt(type)}

  # build widget $topwin(restore_name).info.frame13.frame1.frame19.radiobutton17
	radiobutton $topwin(restore_name).info.frame13.frame1.frame19.radiobutton17 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -command {
set freq_filt(alpha) 0
pack forget $topwin(restore_name).info.frame13.frame5.frame0
pack forget $topwin(restore_name).info.frame13.frame5.frame2
pack forget $topwin(restore_name).info.frame13.frame2.frame0.frame1
pack forget $topwin(restore_name).info.frame13.frame4.frame1
pack forget $topwin(restore_name).info.frame11.info.frame0.frame5
pack forget $topwin(restore_name).info.frame13.frame1.frame0.frame4 
pack forget $topwin(restore_name).info.frame13.frame1.frame0.frame1
pack $topwin(restore_name).info.frame13.frame1.frame0.frame0 -side left
pack  $topwin(restore_name).info.frame13.frame5.frame6 -side left
pack  $topwin(restore_name).info.frame13.frame5.frame1 -side left
pack  $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame4.frame3
pack  $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame2.frame1
pack $topwin(restore_name).info.frame13.frame1.frame0.labelnull -side right


} \
    -relief {flat} \
    -text {Constrained Least Squares} \
    -value {least_squares} \
    -variable {freq_filt(type)}

  # build widget $topwin(restore_name).info.frame13.frame1.frame19.radiobutton18
	radiobutton $topwin(restore_name).info.frame13.frame1.frame19.radiobutton18 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -command {
pack forget $topwin(restore_name).info.frame13.frame5.frame0
pack forget $topwin(restore_name).info.frame13.frame5.frame1
pack forget $topwin(restore_name).info.frame13.frame2.frame0.frame1
pack forget $topwin(restore_name).info.frame11.info.frame0.frame5
pack forget $topwin(restore_name).info.frame13.frame1.frame0.frame0 
pack forget $topwin(restore_name).info.frame13.frame1.frame0.frame4 
pack forget $topwin(restore_name).info.frame13.frame1.frame0.frame1
pack $topwin(restore_name).info.frame13.frame4.frame1
pack $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame4.frame3
pack $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame2.frame1
pack $topwin(restore_name).info.frame13.frame5.frame2 -side left
pack $topwin(restore_name).info.frame13.frame5.frame6 -side left
pack $topwin(restore_name).info.frame13.frame1.frame0.labelnull -side right
} \
    -relief {flat} \
    -text {Geometric Mean} \
    -value {geometric_mean} \
    -variable {freq_filt(type)}

  # pack widget $topwin(restore_name).info.frame13.frame1.frame19
  pack append $topwin(restore_name).info.frame13.frame1.frame19 \
    $topwin(restore_name).info.frame13.frame1.frame19.radiobutton16 {top frame w} \
    $topwin(restore_name).info.frame13.frame1.frame19.radiobutton18 {top frame w} \
    $topwin(restore_name).info.frame13.frame1.frame19.radiobutton15 {top frame w} \
    $topwin(restore_name).info.frame13.frame1.frame19.radiobutton17 {top frame w}

  # build widget $topwin(restore_name).info.frame13.frame1.frame7
  frame $topwin(restore_name).info.frame13.frame1.frame7 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(restore_name).info.frame13.frame1.frame7.label8
  label $topwin(restore_name).info.frame13.frame1.frame7.label8 \
    -background "$GuiColor(label_bg)" \
    -text {RESTORATION FILTERS}

  # pack widget $topwin(restore_name).info.frame13.frame1.frame7
  pack append $topwin(restore_name).info.frame13.frame1.frame7 \
    $topwin(restore_name).info.frame13.frame1.frame7.label8 {top frame center}

  # build widget $topwin(restore_name).info.frame13.frame1.frame0
  frame $topwin(restore_name).info.frame13.frame1.frame0 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2} \
    -relief {ridge}

  # build widget $topwin(restore_name).info.frame13.frame1.frame0.frame0
  frame $topwin(restore_name).info.frame13.frame1.frame0.frame0 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(restore_name).info.frame13.frame1.frame0.frame0.label8
  label $topwin(restore_name).info.frame13.frame1.frame0.frame0.label8 \
    -background "$GuiColor(label_bg)" \
    -foreground {blue} \
    -text {0} \
    -textvariable {freq_filt(alpha)}

  # build widget $topwin(restore_name).info.frame13.frame1.frame0.frame0.label2
  label $topwin(restore_name).info.frame13.frame1.frame0.frame0.label2 \
    -background "$GuiColor(label_bg)" \
    -font {-adobe-symbol-medium-r-normal--14-100-100-100-p-85-adobe-fontspecific} \
    -text {a=}

  # pack widget $topwin(restore_name).info.frame13.frame1.frame0.frame0
  pack append $topwin(restore_name).info.frame13.frame1.frame0.frame0 \
    $topwin(restore_name).info.frame13.frame1.frame0.frame0.label2 {left frame center} \
    $topwin(restore_name).info.frame13.frame1.frame0.frame0.label8 {left frame center}

  # build widget $topwin(restore_name).info.frame13.frame1.frame0.frame4
  frame $topwin(restore_name).info.frame13.frame1.frame0.frame4 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(restore_name).info.frame13.frame1.frame0.frame4.label8
  label $topwin(restore_name).info.frame13.frame1.frame0.frame4.label8 \
    -background "$GuiColor(label_bg)" \
    -foreground {blue} \
    -text {0} \
    -textvariable {freq_filt(gamma)}

  # build widget $topwin(restore_name).info.frame13.frame1.frame0.frame4.label2
  label $topwin(restore_name).info.frame13.frame1.frame0.frame4.label2 \
    -background "$GuiColor(label_bg)" \
    -text {Gamma=}

  # pack widget $topwin(restore_name).info.frame13.frame1.frame0.frame4
  pack append $topwin(restore_name).info.frame13.frame1.frame0.frame4 \
    $topwin(restore_name).info.frame13.frame1.frame0.frame4.label2 {left frame center} \
    $topwin(restore_name).info.frame13.frame1.frame0.frame4.label8 {left frame center}

  # build widget $topwin(restore_name).info.frame13.frame1.frame0.frame1
  frame $topwin(restore_name).info.frame13.frame1.frame0.frame1 \
    -background "$GuiColor(frame_bg)"

  # build widget $topwin(restore_name).info.frame13.frame1.frame0.frame1.label8
  label $topwin(restore_name).info.frame13.frame1.frame0.frame1.label8 \
    -background "$GuiColor(label_bg)" \
    -bitmap "@$env(CVIPHOME)/CVIPTCL/GUI_SCRIPTS/bmp/K1.bmp" \
    -foreground {blue} \
    -text {0}

  # build widget $topwin(restore_name).info.frame13.frame1.frame0.frame1.label2
  label $topwin(restore_name).info.frame13.frame1.frame0.frame1.label2 \
    -background "$GuiColor(label_bg)" \
    -text {K=}

  # pack widget $topwin(restore_name).info.frame13.frame1.frame0.frame1
  pack append $topwin(restore_name).info.frame13.frame1.frame0.frame1 \
    $topwin(restore_name).info.frame13.frame1.frame0.frame1.label2 {left frame center} \
    $topwin(restore_name).info.frame13.frame1.frame0.frame1.label8 {left frame center}

  # build widget $topwin(restore_name).info.frame13.frame1.frame0.frame3
  label $topwin(restore_name).info.frame13.frame1.frame0.labelnull \
    -background "$GuiColor(label_bg)" \
    -borderwidth {2} \
    -height {4} \
	 -text {}

  # pack widget $topwin(restore_name).info.frame13.frame1.frame0
  pack append $topwin(restore_name).info.frame13.frame1.frame0 \
    $topwin(restore_name).info.frame13.frame1.frame0.labelnull {right frame center} \
    $topwin(restore_name).info.frame13.frame1.frame0.frame0 {left frame center} \
    $topwin(restore_name).info.frame13.frame1.frame0.frame4 {left frame center}

  # pack widget $topwin(restore_name).info.frame13.frame1
  pack append $topwin(restore_name).info.frame13.frame1 \
    $topwin(restore_name).info.frame13.frame1.frame7 {top frame center fillx} \
    $topwin(restore_name).info.frame13.frame1.frame19 {top frame center fillx} \
    $topwin(restore_name).info.frame13.frame1.frame6 {top frame center fillx} \
    $topwin(restore_name).info.frame13.frame1.frame0 {bottom frame center fillx}

  # build widget $topwin(restore_name).info.frame13.frame2
  frame $topwin(restore_name).info.frame13.frame2 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2} \
    -relief {ridge}

  # build widget $topwin(restore_name).info.frame13.frame2.frame8
  frame $topwin(restore_name).info.frame13.frame2.frame8 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(restore_name).info.frame13.frame2.frame8.label20
  label $topwin(restore_name).info.frame13.frame2.frame8.label20 \
    -background "$GuiColor(label_bg)" \
    -text {Degraded image: }

  # build widget $topwin(restore_name).info.frame13.frame2.frame8.label21
  label $topwin(restore_name).info.frame13.frame2.frame8.label21 \
    -background "$GuiColor(label_bg)" \
    -foreground {Blue} \
    -textvariable {current_image}

  # build widget $topwin(restore_name).info.frame13.frame2.frame8.label22
  label $topwin(restore_name).info.frame13.frame2.frame8.label22 \
    -background "$GuiColor(label_bg)" \
    -text {(the current image)}

  # pack widget $topwin(restore_name).info.frame13.frame2.frame8
  pack append $topwin(restore_name).info.frame13.frame2.frame8 \
    $topwin(restore_name).info.frame13.frame2.frame8.label20 {left frame center} \
    $topwin(restore_name).info.frame13.frame2.frame8.label21 {left frame center} \
    $topwin(restore_name).info.frame13.frame2.frame8.label22 {left frame center}

  # build widget $topwin(restore_name).info.frame13.frame2.frame9
  frame $topwin(restore_name).info.frame13.frame2.frame9 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(restore_name).info.frame13.frame2.frame9.frame5
  frame $topwin(restore_name).info.frame13.frame2.frame9.frame5 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(restore_name).info.frame13.frame2.frame9.frame5.label23
  label $topwin(restore_name).info.frame13.frame2.frame9.frame5.label23 \
    -background "$GuiColor(label_bg)" \
    -text {Degradation function: }

  # pack widget $topwin(restore_name).info.frame13.frame2.frame9.frame5
  pack append $topwin(restore_name).info.frame13.frame2.frame9.frame5 \
    $topwin(restore_name).info.frame13.frame2.frame9.frame5.label23 {left frame center}

  # build widget $topwin(restore_name).info.frame13.frame2.frame9.frame6
  frame $topwin(restore_name).info.frame13.frame2.frame9.frame6 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(restore_name).info.frame13.frame2.frame9.frame6.frame8
  frame $topwin(restore_name).info.frame13.frame2.frame9.frame6.frame8 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(restore_name).info.frame13.frame2.frame9.frame6.frame8.radiobutton24
	radiobutton $topwin(restore_name).info.frame13.frame2.frame9.frame6.frame8.radiobutton24 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -command { $topwin(restore_name).info.frame13.frame2.frame9.frame6.frame8.frame.entry2 configure -bg white -state normal
pack forget $topwin(restore_name).info.frame13.frame2.frame28.frame10
pack forget $topwin(restore_name).info.frame13.frame2.frame28.frame1
} \
    -relief {flat} \
    -text {Use image:} \
    -value {image} \
    -variable {freq_filt(deg_fn)}

  # build widget $topwin(restore_name).info.frame13.frame2.frame9.frame6.frame8.frame
  frame $topwin(restore_name).info.frame13.frame2.frame9.frame6.frame8.frame \
    -background "$GuiColor(frame_bg)" \
    -width {46}

  # build widget $topwin(restore_name).info.frame13.frame2.frame9.frame6.frame8.frame.scrollbar1
  scrollbar $topwin(restore_name).info.frame13.frame2.frame9.frame6.frame8.frame.scrollbar1 \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -command { $topwin(restore_name).info.frame13.frame2.frame9.frame6.frame8.frame.entry2 xview} \
    -orient {horizontal} \
    -width {9}

  # build widget $topwin(restore_name).info.frame13.frame2.frame9.frame6.frame8.frame.entry2
  entry $topwin(restore_name).info.frame13.frame2.frame9.frame6.frame8.frame.entry2 \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
 -xscrollcommand "$topwin(restore_name).info.frame13.frame2.frame9.frame6.frame8.frame.scrollbar1 set" \
    -textvariable {second_image} \
    -width {25}
  endEntry $topwin(restore_name).info.frame13.frame2.frame9.frame6.frame8.frame.entry2 

  # pack widget $topwin(restore_name).info.frame13.frame2.frame9.frame6.frame8.frame
  pack append $topwin(restore_name).info.frame13.frame2.frame9.frame6.frame8.frame \
    $topwin(restore_name).info.frame13.frame2.frame9.frame6.frame8.frame.entry2 {top frame center fill} \
    $topwin(restore_name).info.frame13.frame2.frame9.frame6.frame8.frame.scrollbar1 {top frame center fillx}

  # pack widget $topwin(restore_name).info.frame13.frame2.frame9.frame6.frame8
  pack append $topwin(restore_name).info.frame13.frame2.frame9.frame6.frame8 \
    $topwin(restore_name).info.frame13.frame2.frame9.frame6.frame8.radiobutton24 {left frame center} \
    $topwin(restore_name).info.frame13.frame2.frame9.frame6.frame8.frame {left frame center filly}

  # build widget $topwin(restore_name).info.frame13.frame2.frame9.frame6.frame9
  frame $topwin(restore_name).info.frame13.frame2.frame9.frame6.frame9 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(restore_name).info.frame13.frame2.frame9.frame6.frame9.radiobutton26
	radiobutton $topwin(restore_name).info.frame13.frame2.frame9.frame6.frame9.radiobutton26 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -command { $topwin(restore_name).info.frame13.frame2.frame9.frame6.frame8.frame.entry2 configure -state disabled
pack append $topwin(restore_name).info.frame13.frame2.frame28\
$topwin(restore_name).info.frame13.frame2.frame28.frame1 {top frame w fillx} \
$topwin(restore_name).info.frame13.frame2.frame28.frame10 {top frame w} 
set freq_filt(degrade_shape) rectangle
set freq_filt(blur_weight_val) 1

} \
    -relief {flat} \
    -text {Specify the function} \
    -value {specify} \
    -variable {freq_filt(deg_fn)}

  # pack widget $topwin(restore_name).info.frame13.frame2.frame9.frame6.frame9
  pack append $topwin(restore_name).info.frame13.frame2.frame9.frame6.frame9 \
    $topwin(restore_name).info.frame13.frame2.frame9.frame6.frame9.radiobutton26 {left frame w padx 25}

  # pack widget $topwin(restore_name).info.frame13.frame2.frame9.frame6
  pack append $topwin(restore_name).info.frame13.frame2.frame9.frame6 \
    $topwin(restore_name).info.frame13.frame2.frame9.frame6.frame8 {top frame w expand} \
    $topwin(restore_name).info.frame13.frame2.frame9.frame6.frame9 {top frame w}

  # pack widget $topwin(restore_name).info.frame13.frame2.frame9
  pack append $topwin(restore_name).info.frame13.frame2.frame9 \
    $topwin(restore_name).info.frame13.frame2.frame9.frame5 {left frame center} \
    $topwin(restore_name).info.frame13.frame2.frame9.frame6 {left frame center}

  # build widget $topwin(restore_name).info.frame13.frame2.frame28
  frame $topwin(restore_name).info.frame13.frame2.frame28 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.frame10
  frame $topwin(restore_name).info.frame13.frame2.frame28.frame10 \
    -background "$GuiColor(frame_bg)" \
	 -borderwidth {0}

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0
  frame $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame0
  frame $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame0 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2} \
    -height {1} \
    -width {75}

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame0.label1
  label $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame0.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -text {Blur method:} \
    -width {12}

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame0.value
  entry $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame0.value \
    -background "$GuiColor(rentry_bg)" \
    -foreground "$GuiColor(rentry_fg)" \
    -relief {sunken} \
    -state {disabled} \
    -textvariable {freq_filt(blur_weight)} \
    -width {23}

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame0.menubutton2
	menubutton $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame0.menubutton2 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)"\
    -text "$GuiMisc(pop_up_text)" \
	 -width {2}
  # bindings
  bind $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame0.menubutton2 <Button-1> {MenuPopupPost $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame0.menubutton2.menu %X %Y}
  bind $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame0.menubutton2 <ButtonRelease-1> {MenuPopupRelease $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame0.menubutton2.menu %W}

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame0.menubutton2.menu
  menu $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame0.menubutton2.menu \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"
  $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {pack forget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame29
pack forget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame3
set freq_filt(blur_weight) Constant
set freq_filt(filter_val) ""
} \
    -label {Constant}
  $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {
pack forget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame29
pack forget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame3 
set freq_filt(blur_weight) {Center-weighted constant}
set freq_filt(filter_val) ""
} \
    -label {Center-weighted constant}
  $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {pack forget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame29 -side left
pack forget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame3
set freq_filt(blur_weight) Gaussian; set freq_filt(filter_val) ""
set freq_filt(blur_weight_val) 3} \
    -label {Gaussian}
  bindtags $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame0.menubutton2.menu {Menu all}
  # bindings
  bind $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame0.menubutton2.menu <B1-Motion> {MenuPopupMotion $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame0.menubutton2.menu %W %X %Y}
  bind $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame0.menubutton2.menu <ButtonRelease-1> {MenuPopupRelease $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame0.menubutton2.menu %W}

  # pack widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame0
  pack append $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame0 \
    $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame0.label1 {left frame center} \
    $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame0.value {left frame center} \
    $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame0.menubutton2 {right frame center}

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame29
  frame $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame29 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2} \
    -height {1} \
    -width {75}

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame29.label1
  label $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame29.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -text {  Mask: }

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame29.value
  entry $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame29.value \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
    -textvariable {freq_filt(degrade_mask)} \
    -width {3}

  # pack widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame29
  pack append $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame29 \
    $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame29.label1 {left frame center} \
    $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame29.value {left frame center}

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame30
  frame $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame30 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2} \
    -height {1} \
    -width {75}

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame30.label1
  label $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame30.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -text { Weight: }

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame30.value
  entry $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame30.value \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
    -textvariable {freq_filt(param4)} \
    -width {7}
  endEntry $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame30.value 0\
  	 $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame4.frame3.entry6 \
  	 $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame12.frame1.entry
	bind $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame30.value \
	<KeyRelease>  { set freq_filt(filter_val) ""}

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame30.menubutton2
	menubutton $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame30.menubutton2 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)"\
    -text "$GuiMisc(pop_up_text)" \
	 -width {2}
  # bindings
  bind $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame30.menubutton2 <Button-1> {MenuPopupPost $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame30.menubutton2.menu %X %Y}
  bind $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame30.menubutton2 <ButtonRelease-1> {MenuPopupRelease $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame30.menubutton2.menu %W}

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame30.menubutton2.menu
  menu $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame30.menubutton2.menu \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"
  $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame30.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(param4) 100.0; set freq_filt(filter_val) ""} \
    -label {100.0}
  $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame30.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(param4) 10.0; set freq_filt(filter_val) ""} \
    -label {10.00}
  $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame30.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(param4) 1.0; set freq_filt(filter_val) ""} \
    -label {1.000}
  $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame30.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(param4) 0.1; set freq_filt(filter_val) "" } \
    -label {0.100}
  $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame30.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(param4) 0.01; set freq_filt(filter_val) "" } \
    -label {0.010}
  $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame30.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(param4) 0.001; set freq_filt(filter_val) "" } \
    -label {0.001}
  bindtags $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame30.menubutton2.menu {Menu all}
  # bindings
  bind $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame30.menubutton2.menu <B1-Motion> {MenuPopupMotion $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame30.menubutton2.menu0 %W %X %Y}
  bind $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame30.menubutton2.menu <ButtonRelease-1> {MenuPopupRelease $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame30.menubutton2.menu0 %W}

  # pack widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame30
  pack append $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame30 \
    $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame30.label1 {left frame center fill} \
    $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame30.value {left frame center fill} \
    $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame30.menubutton2 {right frame center fill}

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame3
  frame $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame3 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2} \
    -height {1} \
    -width {75}

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame3.label1
  label $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame3.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -text { Stdev: }

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame3.value
  entry $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame3.value \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
    -textvariable {freq_filt(degrade_stdev)} \
    -width {3}

  # pack widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame3
  pack append $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame3 \
    $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame3.label1 {left frame center} \
    $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame3.value {left frame center}

  # pack widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0
  pack append $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0 \
    $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame0 {left frame w} \
    $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame30 {left frame w} 
#    $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame29 {left frame center}

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12
  frame $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame0
  frame $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame0 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2} \
    -height {1} \
    -width {75}

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame0.label1
  label $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame0.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -text {Blur shape:  } \
    -width {12}

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame0.value
  entry $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame0.value \
    -background "$GuiColor(rentry_bg)" \
    -foreground "$GuiColor(rentry_fg)" \
    -relief {sunken} \
    -state {disabled} \
    -textvariable {freq_filt(degrade_shape)} \
    -width {15}

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame0.menubutton2
	menubutton $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame0.menubutton2 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)"\
    -text "$GuiMisc(pop_up_text)" \
	 -width {2}
  # bindings
  bind $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame0.menubutton2 <Button-1> {MenuPopupPost $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame0.menubutton2.menu %X %Y}
  bind $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame0.menubutton2 <ButtonRelease-1> {MenuPopupRelease $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame0.menubutton2.menu %W}

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame0.menubutton2.menu
  menu $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame0.menubutton2.menu \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"
  $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {pack  $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame7 -side left
# pack  $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame29 -side left
# pack  $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame9 -side left
pack forget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame11
pack forget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame12
pack forget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame1 
set freq_filt(degrade_shape) circle; set freq_filt(filter_val) "" } \
    -label {circle} \
    -state {active}
  $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {pack forget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame7
pack forget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame29
pack forget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame9
pack  $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame11 -side left
pack  $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame12 -side left
pack  forget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame1 
set freq_filt(degrade_shape) rectangle
set freq_filt(filter_val) "" 

} \
    -label {rectangle}
  $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {pack forget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame7
pack forget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame29
pack forget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame9
pack forget  $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame11
pack forget  $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame12
pack forget  $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame1
set freq_filt(degrade_shape) "Horizontal line"
set freq_filt(filter_val) "" } \
    -label {Horizontal line}
  $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {pack forget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame7
pack forget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame29
pack forget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame9
pack forget  $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame11
pack forget  $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame12
pack forget  $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame1
set freq_filt(degrade_shape) "Vertical line"; set freq_filt(filter_val) "" 
} \
    -label {Vertical line}
  $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {pack forget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame7
pack forget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame29
pack forget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame9
pack forget  $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame11
pack forget  $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame12
pack forget  $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame1
set freq_filt(degrade_shape) "Slash"; set freq_filt(filter_val) ""
} \
    -label {Slash}
  $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame0.menubutton2.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {pack forget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame7
pack forget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame29
pack forget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame9
pack forget  $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame11
pack forget  $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame12
pack forget  $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame1
set freq_filt(degrade_shape) "Backslash"; set freq_filt(filter_val) ""
} \
    -label {Backslash}
  # bindings
  bind $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame0.menubutton2.menu <B1-Motion> {MenuPopupMotion $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame0.menubutton2.menu %W %X %Y}
  bind $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame0.menubutton2.menu <ButtonRelease-1> {MenuPopupRelease $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame0.menubutton2.menu %W}

  # pack widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame0
  pack append $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame0 \
    $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame0.label1 {left frame center} \
    $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame0.value {left frame center} \
    $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame0.menubutton2 {right frame center}
  bindtags $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame0.menubutton2.menu  {Menu all}
  # bindings
  bind $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame0.menubutton2.menu <B1-Motion> {MenuPopupMotion $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame0.menubutton2.menu %W %X %Y}
  bind $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame0.menubutton2.menu <ButtonRelease-1> {MenuPopupRelease $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame0.menubutton2.menu %W}

  # pack widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame0
  pack append $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame0 \
    $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame0.label1 {left frame center} \
    $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame0.value {left frame center} \
    $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame0.menubutton2 {right frame center}

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame29
  frame $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame29 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2} \
    -height {1} \
    -width {75}

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame29.label1
  label $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame29.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -text {Angle1:} \
    -width {8}

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame29.value
  entry $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame29.value \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
    -textvariable {freq_filt(degrade_ang1)} \
    -width {3}

  # pack widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame29
  pack append $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame29 \
    $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame29.label1 {left frame center expand fill} \
    $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame29.value {left frame center expand fill}

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame7
  frame $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame7 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2} \
    -height {1} \
    -width {75}

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame7.label1
  label $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame7.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -text {Diameter:} \
    -width {8}

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame7.frame1
  frame $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame7.frame1 \
    -borderwidth {0} \
    -highlightthickness {0} \
	 -background "$GuiColor(frame_bg)" \
    -height {30} \
    -width {30}

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame7.frame1.entry
  entry $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame7.frame1.entry \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
    -textvariable {freq_filt(degrade_rad)} -width {3}
	bind $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame7.frame1.entry \
	<KeyRelease>  { set freq_filt(filter_val) "" }

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame7.frame1.menubutton7
  menubutton $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame7.frame1.menubutton7 \
	 -activebackground "$GuiColor(btn_act_bg)" -background "$GuiColor(btn_bg)"  \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)" \
    -padx {1} -pady {1} -highlightthickness {0} \
    -text "$GuiMisc(pop_up_text)" -width {2}

  # bindings
  bind $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame7.frame1.menubutton7 <Button-1> { MenuPopupPost $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame7.frame1.menubutton7.menu0 %X %Y}
  bind $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame7.frame1.menubutton7 <ButtonRelease-1> {MenuPopupRelease $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame7.frame1.menubutton7.menu0 %W }

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame7.frame1.menubutton7.menu0
  menu $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame7.frame1.menubutton7.menu0  \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"

  $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame7.frame1.menubutton7.menu0 add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"  \
    -label { 3 } \
	 -command {set freq_filt(degrade_rad) 3}
  $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame7.frame1.menubutton7.menu0 add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"  \
    -label { 4 } \
	 -command {set freq_filt(degrade_rad) 4}
  $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame7.frame1.menubutton7.menu0 add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"  \
    -label { 5 } \
	 -command {set freq_filt(degrade_rad)  5}
  $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame7.frame1.menubutton7.menu0 add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"  \
    -label { 6 } \
	 -command {set freq_filt(degrade_rad) 6}
  $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame7.frame1.menubutton7.menu0 add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"  \
    -label { 7 } \
	 -command {set freq_filt(degrade_rad) 7}

  bind $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame7.frame1.menubutton7.menu0 <B1-Motion> { MenuPopupMotion $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame7.frame1.menubutton7.menu0 %W %X %Y}
  bind $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame7.frame1.menubutton7.menu0 <ButtonRelease-1>  { MenuPopupRelease $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame7.frame1.menubutton7.menu0 %W }

  # pack master $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame7.frame1
  pack configure $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame7.frame1.menubutton7 \
    -fill both \
    -side right
  pack configure $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame7.frame1.entry \
    -fill both \
    -side left

  # pack widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame7
  pack append $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame7 \
    $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame7.label1 {left frame center expand fill} \
    $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame7.frame1 {left frame center expand fill}

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame9
  frame $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame9 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2} \
    -height {1} \
    -width {75}

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame9.label1
  label $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame9.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -text {Angle2:} \
    -width {8}

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame9.value
  entry $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame9.value \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
    -textvariable {freq_filt(degrade_ang2)} \
    -width {3}

  # pack widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame9
  pack append $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame9 \
    $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame9.label1 {left frame center expand fill} \
    $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame9.value {left frame center expand fill}

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame11
  frame $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame11 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2} \
    -height {1} \
    -width {75}

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame11.label1
  label $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame11.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -text {Height:} \
    -width {8}

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame11.frame1
  frame $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame11.frame1 \
    -borderwidth {0} \
    -highlightthickness {0} \
	 -background "$GuiColor(frame_bg)" \
    -height {30} \
    -width {30}

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame11.frame1.entry
  entry $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame11.frame1.entry \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
    -textvariable {freq_filt(degrade_height)} -width {3}
  endEntry $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame11.frame1.entry 0 \
  	 $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame12.frame1.entry \
  	 $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame4.entry2
	bind $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame11.frame1.entry \
	<KeyRelease>  { set freq_filt(filter_val) "" }

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame11.frame1.menubutton7
  menubutton $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame11.frame1.menubutton7 \
	 -activebackground "$GuiColor(btn_act_bg)" -background "$GuiColor(btn_bg)"  \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)" \
    -padx {1} -pady {1} -highlightthickness {0} \
    -text "$GuiMisc(pop_up_text)" -width {2}

  # bindings
  bind $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame11.frame1.menubutton7 <Button-1> { MenuPopupPost $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame11.frame1.menubutton7.menu0 %X %Y}
  bind $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame11.frame1.menubutton7 <ButtonRelease-1> {MenuPopupRelease $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame11.frame1.menubutton7.menu0 %W }

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame11.frame1.menubutton7.menu0
  menu $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame11.frame1.menubutton7.menu0  \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" 

  $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame11.frame1.menubutton7.menu0 add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"  \
    -label { 3 } \
	 -command {set freq_filt(degrade_height) 3}
  $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame11.frame1.menubutton7.menu0 add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"  \
    -label { 4 } \
	 -command {set freq_filt(degrade_height) 4}
  $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame11.frame1.menubutton7.menu0 add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"  \
    -label { 5 } \
	 -command {set freq_filt(degrade_height) 5}
  $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame11.frame1.menubutton7.menu0 add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"  \
    -label { 6 } \
	 -command {set freq_filt(degrade_height) 6}
  $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame11.frame1.menubutton7.menu0 add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"  \
    -label { 7 } \
	 -command {set freq_filt(degrade_height) 7}

  bind $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame11.frame1.menubutton7.menu0 <B1-Motion> { MenuPopupMotion $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame11.frame1.menubutton7.menu0 %W %X %Y}
  bind $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame11.frame1.menubutton7.menu0 <ButtonRelease-1>  { MenuPopupRelease $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame11.frame1.menubutton7.menu0 %W }

  # pack master $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame11.frame1
  pack configure $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame11.frame1.menubutton7 \
    -fill both \
    -side right
  pack configure $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame11.frame1.entry \
    -fill both \
    -side left

  # pack widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame11
  pack append $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame11 \
    $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame11.label1 {left frame center expand fill} \
    $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame11.frame1 {left frame center expand fill}

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame12
  frame $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame12 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2} \
    -height {1} \
    -width {75}

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame12.label1
  label $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame12.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -text {Width:} \
    -width {5}

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame12.frame1
  frame $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame12.frame1 \
    -borderwidth {0} \
    -highlightthickness {0} \
	 -background "$GuiColor(frame_bg)" \
    -height {30} \
    -width {30}

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame12.frame1.entry
  entry $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame12.frame1.entry \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
    -textvariable {freq_filt(degrade_width)} -width {3}
  endEntry $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame12.frame1.entry 0 \
  	 $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame30.value \
  	 $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame11.frame1.entry
	bind $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame12.frame1.entry \
	<KeyRelease>  { set freq_filt(filter_val) "" }

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame12.frame1.menubutton7
  menubutton $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame12.frame1.menubutton7 \
	 -activebackground "$GuiColor(btn_act_bg)" -background "$GuiColor(btn_bg)"  \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)" \
    -padx {1} -pady {1} -highlightthickness {0} \
    -text "$GuiMisc(pop_up_text)" -width {2}

  # bindings
  bind $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame12.frame1.menubutton7 <Button-1> { MenuPopupPost $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame12.frame1.menubutton7.menu0 %X %Y}
  bind $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame12.frame1.menubutton7 <ButtonRelease-1> {MenuPopupRelease $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame12.frame1.menubutton7.menu0 %W }

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame12.frame1.menubutton7.menu0
  menu $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame12.frame1.menubutton7.menu0  \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" 

  $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame12.frame1.menubutton7.menu0 add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"  \
    -label { 3 } \
	 -command {set freq_filt(degrade_width) 3}
  $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame12.frame1.menubutton7.menu0 add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"  \
    -label { 4 } \
	 -command {set freq_filt(degrade_width) 4}
  $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame12.frame1.menubutton7.menu0 add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"  \
    -label { 5 } \
	 -command {set freq_filt(degrade_width) 5}
  $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame12.frame1.menubutton7.menu0 add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"  \
    -label { 6 } \
	 -command {set freq_filt(degrade_width) 6}
  $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame12.frame1.menubutton7.menu0 add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"  \
    -label { 7 } \
	 -command {set freq_filt(degrade_width) 7}

  bind $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame12.frame1.menubutton7.menu0 <B1-Motion> { MenuPopupMotion $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame12.frame1.menubutton7.menu0 %W %X %Y}
  bind $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame12.frame1.menubutton7.menu0 <ButtonRelease-1>  { MenuPopupRelease $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame12.frame1.menubutton7.menu0 %W }

  # pack master $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame12.frame1
  pack configure $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame12.frame1.menubutton7 \
    -fill both \
    -side right
  pack configure $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame12.frame1.entry \
    -fill both \
    -side left

  # pack widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame12
  pack append $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame12 \
    $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame12.label1 {left frame center expand fill} \
    $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame12.frame1 {left frame center expand fill}

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame1
  frame $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame1 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2} \
    -height {1} \
    -width {75}

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame1.label1
  label $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame1.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -text {Thickness:} 

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame1.value
  entry $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame1.value \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
    -textvariable {freq_filt(degrade_ang1)} \
    -width {3}

  # pack widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame1
  pack append $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame1 \
    $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame1.label1 {left frame center expand fill} \
    $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame1.value {left frame center expand fill}

  # pack widget $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12
  pack append $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12 \
    $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame0 {left frame w} \
    $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame11 {left frame center} \
    $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame12 {left frame center} 
#    $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame1 {left frame center}

	pack append $topwin(restore_name).info.frame13.frame2.frame28.frame10 \
	$topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12 {top frame w}\
	$topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0 {top frame w}

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.labelnull
  label $topwin(restore_name).info.frame13.frame2.frame28.labelnull \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -height {7} \
	 -text {}

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.frame1
  frame $topwin(restore_name).info.frame13.frame2.frame28.frame1 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2} \
	 -relief {groove}

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame0
  frame $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame0 \
    -background "$GuiColor(frame_bg)"
  # bindings
  bind $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame0 <Button-1> {MenuPopupPost $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame0.menu %X %Y}
  bind $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame0 <ButtonRelease-1> {MenuPopupRelease $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame0.menu %W}

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame0.label1
  label $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame0.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -text { Mask height: }

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame0.entry2
  entry $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame0.entry2 \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
    -textvariable {freq_filt(msk_height)} \
    -width {6}
	bind $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame0.entry2 \
	<KeyRelease>  { set freq_filt(filter_val) "" }
  endEntry $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame0.entry2 0\
  	 $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame4.entry2 \
  	 $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame4.frame3.entry6 

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame0.menubutton3
	menubutton $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame0.menubutton3 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)"\
    -text "$GuiMisc(pop_up_text)" \
	 -width {2}
  # bindings
  bind $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame0.menubutton3 <Button-1> {MenuPopupPost $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame0.menubutton3.menu %X %Y}
  bind $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame0.menubutton3 <ButtonRelease-1> {MenuPopupRelease $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame0.menubutton3.menu %W}

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame0.menubutton3.menu
  menu $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame0.menubutton3.menu \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"
  $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame0.menubutton3.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(msk_height) 3; set freq_filt(filter_val) "" } \
    -label {  3 }
  $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame0.menubutton3.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(msk_height) 5; set freq_filt(filter_val) "" }\
    -label {  5 }
  $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame0.menubutton3.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(msk_height) 7; set freq_filt(filter_val) "" }\
    -label {  7 }
  $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame0.menubutton3.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(msk_height) 9; set freq_filt(filter_val) "" }\
    -label {  9 }
  $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame0.menubutton3.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(msk_height) 11; set freq_filt(filter_val) "" }\
    -label { 11 }
  $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame0.menubutton3.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(msk_height) 13; set freq_filt(filter_val) "" }\
    -label { 13 }
  bindtags $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame0.menubutton3.menu  {Menu all}
  # bindings
  bind $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame0.menubutton3.menu <B1-Motion> {MenuPopupMotion $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame0.menubutton3.menu0 %W %X %Y}
  bind $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame0.menubutton3.menu <ButtonRelease-1> {MenuPopupRelease $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame0.menubutton3.menu0 %W}

  # pack widget $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame0
  pack append $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame0 \
    $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame0.label1 {left frame center} \
    $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame0.entry2 {left frame center} \
    $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame0.menubutton3 {left frame center}

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame4
  frame $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame4 \
    -background "$GuiColor(frame_bg)"
  # bindings
  bind $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame4 <Button-1> {MenuPopupPost $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame4.menu %X %Y}
  bind $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame4 <ButtonRelease-1> {MenuPopupRelease $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame4.menu %W}

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame4.label1
  label $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame4.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -text { Mask width: }

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame4.entry2
  entry $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame4.entry2 \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
    -textvariable {freq_filt(msk_width)} \
    -width {6}
  endEntry $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame4.entry2 0\
  	 $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame12.frame11.frame1.entry \
  	 $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame0.entry2

	bind $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame4.entry2 \
	<KeyRelease>  { set freq_filt(filter_val) "" }

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame4.menubutton3
	menubutton $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame4.menubutton3 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)"\
    -text "$GuiMisc(pop_up_text)" \
	 -width {2}
  # bindings
  bind $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame4.menubutton3 <Button-1> {MenuPopupPost $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame4.menubutton3.menu %X %Y}
  bind $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame4.menubutton3 <ButtonRelease-1> {MenuPopupRelease $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame4.menubutton3.menu %W}

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame4.menubutton3.menu
  menu $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame4.menubutton3.menu \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"
  $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame4.menubutton3.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(msk_width) 3; set freq_filt(filter_val) "" }\
    -label {  3 }
  $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame4.menubutton3.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(msk_width) 5; set freq_filt(filter_val) "" }\
    -label {  5 }
  $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame4.menubutton3.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(msk_width) 7; set freq_filt(filter_val) "" }\
    -label {  7 }
  $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame4.menubutton3.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(msk_width) 9; set freq_filt(filter_val) "" }\
    -label {  9 }
  $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame4.menubutton3.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(msk_width) 11; set freq_filt(filter_val) "" }\
    -label { 11 }
  $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame4.menubutton3.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(msk_width) 13; set freq_filt(filter_val) "" }\
    -label { 13 }
  bindtags $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame4.menubutton3.menu {Menu all}
  # bindings
  bind $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame4.menubutton3.menu <B1-Motion> {MenuPopupMotion $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame4.menubutton3.menu0 %W %X %Y}
  bind $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame4.menubutton3.menu <ButtonRelease-1> {MenuPopupRelease $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame4.menubutton3.menu0 %W}

  # pack widget $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame4
  pack append $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame4 \
    $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame4.label1 {left frame center} \
    $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame4.entry2 {left frame center} \
    $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame4.menubutton3 {left frame center}

  # build widget $topwin(restore_name).info.frame13.frame2.frame28.frame1.button5
	button $topwin(restore_name).info.frame13.frame2.frame28.frame1.button5 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(alert_act_bg)" \
    -background "$GuiColor(alert_bg)" \
    -foreground "$GuiColor(alert_fg)" \
    -text {Edit mask} \
	 -command {
      $topwin(restore_name) configure -cursor watch; update
		if [catch {expr $freq_filt(msk_height) + $freq_filt(msk_width) } ] {
         CvipAlert "Numerical values required for the degradation image sizes:\n\
			Height = $freq_filt(msk_height), Width = $freq_filt(msk_width)."
         $topwin(restore_name) configure -cursor top_left_arrow; update
         return
		} elseif { $freq_filt(msk_height) < "3" || $freq_filt(msk_width) < "3" || \
					  $freq_filt(msk_height) > "30" || $freq_filt(msk_width) > "30" } {
         CvipAlert "The degradation image size ($freq_filt(msk_height)x$freq_filt(msk_width)) should \
			be in the range \[3-30\]."
         $topwin(restore_name) configure -cursor top_left_arrow; update
         return
		}
		if [catch {expr $freq_filt(param4)} ] {
         CvipAlert "Positive Numerical value required for weight parameter:\n\
			Weight = $freq_filt(param4)."
         $topwin(restore_name) configure -cursor top_left_arrow; update
         return
		} elseif {$freq_filt(param4) < 0  } {
         CvipAlert "Positive Numerical value required for weight parameter ($freq_filt(param4))"
         $topwin(restore_name) configure -cursor top_left_arrow; update
         return
		}

		set temp [expr { $freq_filt(msk_height) > $freq_filt(msk_width) }\
		? $freq_filt(msk_width) : $freq_filt(msk_height) ]
#		set temp [expr $temp / 2]

		switch -exact -- $freq_filt(degrade_shape) {
			"circle" {
				if [catch {expr $freq_filt(degrade_rad) }] {
         		CvipAlert "Numerical values required for the circle diameter:\n\
					Diameter = $freq_filt(degrade_rad)."
					$topwin(restore_name) configure -cursor top_left_arrow; update
					return
				} elseif { $freq_filt(degrade_rad) < 1 || $freq_filt(degrade_rad) > $temp } {
         		CvipAlert "The circle diameter ($freq_filt(degrade_rad)) should be\
					in the range \[1-$temp\]."
					$topwin(restore_name) configure -cursor top_left_arrow; update
					return
				}
				set temp [eval "cvip circle $freq_filt(msk_width) $freq_filt(msk_height) \
									[expr $freq_filt(msk_width)/2] [expr $freq_filt(msk_height) /2]\
									[expr $freq_filt(degrade_rad) / 2]"]
			}
			"rectangle" {
				if [catch {expr $freq_filt(degrade_width) + $freq_filt(degrade_height)}] {
         		CvipAlert "Numerical values required for the mask sizes:\n\
					Height = $freq_filt(degrade_height), Width = $freq_filt(degrade_width)."
					$topwin(restore_name) configure -cursor top_left_arrow; update
					return
				} elseif {$freq_filt(degrade_width) < 1 || $freq_filt(degrade_width) > $freq_filt(msk_width) ||
					$freq_filt(degrade_height) < 1 || $freq_filt(degrade_height) > $freq_filt(msk_height) } {
         		CvipAlert "The mask size ($freq_filt(degrade_height)x$freq_filt(degrade_width)) must be \
					positive and no larger than the mask image size\
					($freq_filt(msk_height)x$freq_filt(msk_width))."
					$topwin(restore_name) configure -cursor top_left_arrow; update
					return
				}
				set temp [eval "cvip rectangle $freq_filt(msk_width) $freq_filt(msk_height) \
					[expr ($freq_filt(msk_width) - $freq_filt(degrade_width)) /2] \
					[expr ($freq_filt(msk_height) - $freq_filt(degrade_height)) /2] \
					$freq_filt(degrade_width) $freq_filt(degrade_height)"]


			}
			"Horizontal line" {
				set temp [eval "cvip line $freq_filt(msk_width) $freq_filt(msk_height) 0 \
				[expr $freq_filt(msk_height) /2 ] [expr $freq_filt(msk_width) - 1] \
				[expr $freq_filt(msk_height) /2 ]"]
			}
			"Vertical line" {
				set temp [eval "cvip line $freq_filt(msk_width) $freq_filt(msk_height) \
				[expr $freq_filt(msk_width) / 2] 0 \
				[expr $freq_filt(msk_width) / 2] [expr $freq_filt(msk_height) - 1]"]
			}
			"Slash" {
				set temp [eval "cvip line $freq_filt(msk_width) $freq_filt(msk_height) \
				0 [expr $freq_filt(msk_height) - 1] [expr $freq_filt(msk_width) - 1] 0"]
			}
			"Backslash" {
				set temp [eval "cvip line $freq_filt(msk_width) $freq_filt(msk_height) \
				0 0 [expr $freq_filt(msk_width) - 1] [expr $freq_filt(msk_height) - 1]"]
			}
		}

		set temp1 [cvip gray_multiply $temp 0.003922 0]; cvip dq $temp
		set temp $temp1

		switch -exact -- $freq_filt(blur_weight) {
			Constant {set temp1 [cvip h_image 1 $freq_filt(msk_height) $freq_filt(msk_width)]}
			"Center-weighted constant" {set temp1 [cvip h_image 2 $freq_filt(msk_height) $freq_filt(msk_width)]}
			Gaussian {set temp1 [cvip h_image 3 $freq_filt(msk_height) $freq_filt(msk_width)]}
		}


		set result [cvip multiply $temp $temp1]; cvip dq $temp; cvip dq $temp1
		set temp $result 
		set freq_filt(h_image) [cvip gray_multiply $temp $freq_filt(param4) 0]; cvip dq $temp
		mask_specify "1" "[cvip data_get $freq_filt(h_image)]"
	 }

  # pack widget $topwin(restore_name).info.frame13.frame2.frame28.frame1
  pack append $topwin(restore_name).info.frame13.frame2.frame28.frame1 \
    $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame0 {left frame center expand fillx} \
    $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame4 {left frame center expand fillx} \
    $topwin(restore_name).info.frame13.frame2.frame28.frame1.button5 {left frame center expand fillx}
  # pack widget $topwin(restore_name).info.frame13.frame2.frame28

  pack append $topwin(restore_name).info.frame13.frame2.frame28 \
    $topwin(restore_name).info.frame13.frame2.frame28.labelnull {left frame center} 
#    $topwin(restore_name).info.frame13.frame2.frame28.frame10 {top frame w} 
#    $topwin(restore_name).info.frame13.frame2.frame28.frame1 {top frame w} 

  # build widget $topwin(restore_name).info.frame13.frame2.frame0
  frame $topwin(restore_name).info.frame13.frame2.frame0 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(restore_name).info.frame13.frame2.frame0.frame0
  frame $topwin(restore_name).info.frame13.frame2.frame0.frame0 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame2
  frame $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame2 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame2.frame1
  frame $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame2.frame1 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame2.frame1.frame3
  frame $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame2.frame1.frame3 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame2.frame1.frame3.label0
  label $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame2.frame1.frame3.label0 \
    -background "$GuiColor(label_bg)" \
    -text {If denominator = 0 :}

  # pack widget $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame2.frame1.frame3
  pack append $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame2.frame1.frame3 \
    $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame2.frame1.frame3.label0 {top frame w}

  # build widget $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame2.frame1.frame4
  frame $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame2.frame1.frame4 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame2.frame1.frame4.radiobutton1
	radiobutton $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame2.frame1.frame4.radiobutton1 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -relief {flat} \
    -text {Let R(u,v) = 1} \
    -value {1} \
    -variable {freq_filt(handle_zero)}

  # build widget $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame2.frame1.frame4.radiobutton2
	radiobutton $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame2.frame1.frame4.radiobutton2 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -relief {flat} \
    -text {Let R(u,v) = 0} \
    -value {2} \
    -variable {freq_filt(handle_zero)}

  # pack widget $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame2.frame1.frame4
  pack append $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame2.frame1.frame4 \
    $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame2.frame1.frame4.radiobutton1 {left frame center padx 30} \
    $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame2.frame1.frame4.radiobutton2 {left frame w}

  # pack widget $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame2.frame1
  pack append $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame2.frame1 \
    $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame2.frame1.frame3 {left frame w} \
    $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame2.frame1.frame4 {left frame w}

  # pack widget $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame2
  pack append $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame2 \
    $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame2.frame1 {top frame center}

  # build widget $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame4
  frame $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame4 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame4.frame3
  frame $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame4.frame3 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame4.frame3.label5
  label $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame4.frame3.label5 \
    -background "$GuiColor(label_bg)" \
    -text {Cutoff frequency:}

  # build widget $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame4.frame3.entry6
  entry $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame4.frame3.entry6 \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
    -textvariable {freq_filt(cutoff)} \
    -width {5}
  endEntry $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame4.frame3.entry6 0 \
  	 $topwin(restore_name).info.frame13.frame2.frame28.frame1.frame0.entry2 \
  	 $topwin(restore_name).info.frame13.frame2.frame28.frame10.frame0.frame30.value

  # build widget $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame4.frame3.button0
	menubutton $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame4.frame3.button0 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)"\
    -text "$GuiMisc(pop_up_text)" \
	 -width {2}
  # bindings
  bind $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame4.frame3.button0 <Button-1> {MenuPopupPost $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame4.frame3.button0.menu %X %Y}
  bind $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame4.frame3.button0 <ButtonRelease-1> {MenuPopupRelease $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame4.frame3.button0.menu %W}

  # build widget $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame4.frame3.button0.menu
  menu $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame4.frame3.button0.menu \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"
  $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame4.frame3.button0.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(cutoff) 4} \
    -label {4}
  $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame4.frame3.button0.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(cutoff) 8} \
    -label {8}
  $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame4.frame3.button0.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(cutoff) 16} \
    -label {16}
  $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame4.frame3.button0.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(cutoff) 32} \
    -label {32}
  $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame4.frame3.button0.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(cutoff) 64} \
    -label {64}
  $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame4.frame3.button0.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(cutoff) 128} \
    -label {128}
  # bindings
  bind $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame4.frame3.button0.menu <B1-Motion> {MenuPopupMotion $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame4.frame3.button0.menu0 %W %X %Y}
  bind $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame4.frame3.button0.menu <ButtonRelease-1> {MenuPopupRelease $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame4.frame3.button0.menu0 %W}

  # pack widget $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame4.frame3
  pack append $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame4.frame3 \
    $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame4.frame3.label5 {left frame w} \
    $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame4.frame3.entry6 {left frame center} \
    $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame4.frame3.button0 {left frame center}

  # pack widget $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame4
  pack append $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame4 \
    $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame4.frame3 {top frame center}

  # pack widget $topwin(restore_name).info.frame13.frame2.frame0.frame0
  pack append $topwin(restore_name).info.frame13.frame2.frame0.frame0 \
    $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame4 {top frame w} \
    $topwin(restore_name).info.frame13.frame2.frame0.frame0.frame2 {top frame w}

  # pack widget $topwin(restore_name).info.frame13.frame2.frame0
  pack append $topwin(restore_name).info.frame13.frame2.frame0 \
    $topwin(restore_name).info.frame13.frame2.frame0.frame0 {top frame w expand fill}

  # pack widget $topwin(restore_name).info.frame13.frame2
  pack append $topwin(restore_name).info.frame13.frame2 \
    $topwin(restore_name).info.frame13.frame2.frame8 {top frame w expand fill} \
    $topwin(restore_name).info.frame13.frame2.frame9 {top frame w expand fill} \
    $topwin(restore_name).info.frame13.frame2.frame28 {top frame w expand fill} \
    $topwin(restore_name).info.frame13.frame2.frame0 {top frame w expand fill}

  # build widget $topwin(restore_name).info.frame13.frame4
  frame $topwin(restore_name).info.frame13.frame4 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2} \
    -relief {ridge}

  # build widget $topwin(restore_name).info.frame13.frame4.frame1
  frame $topwin(restore_name).info.frame13.frame4.frame1 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(restore_name).info.frame13.frame4.frame1.frame0
  frame $topwin(restore_name).info.frame13.frame4.frame1.frame0 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(restore_name).info.frame13.frame4.frame1.frame0.label23
	button $topwin(restore_name).info.frame13.frame4.frame1.frame0.label23 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -relief {flat} \
	 -command {set freq_filt(noise_fn) $current_image} \
    -text {Noise image: }

  # build widget $topwin(restore_name).info.frame13.frame4.frame1.frame0.frame
  frame $topwin(restore_name).info.frame13.frame4.frame1.frame0.frame \
    -background "$GuiColor(frame_bg)" 

  # build widget $topwin(restore_name).info.frame13.frame4.frame1.frame0.frame.scrollbar1
  scrollbar $topwin(restore_name).info.frame13.frame4.frame1.frame0.frame.scrollbar1 \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -command " $topwin(restore_name).info.frame13.frame4.frame1.frame0.frame.entry2 xview" \
    -orient {horizontal} \
    -relief {raised} \
    -width {7}

  # build widget $topwin(restore_name).info.frame13.frame4.frame1.frame0.frame.entry2
  entry $topwin(restore_name).info.frame13.frame4.frame1.frame0.frame.entry2 \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
 -xscrollcommand "$topwin(restore_name).info.frame13.frame4.frame1.frame0.frame.scrollbar1 set" \
    -textvariable {freq_filt(noise_fn)} \
    -width {20}
  endEntry $topwin(restore_name).info.frame13.frame4.frame1.frame0.frame.entry2 0\
  	 $topwin(restore_name).info.frame13.frame4.frame1.frame1.frame.entry2 \
  	 $topwin(restore_name).info.frame13.frame4.frame1.frame1.frame.entry2 

  # pack widget $topwin(restore_name).info.frame13.frame4.frame1.frame0.frame
  pack append $topwin(restore_name).info.frame13.frame4.frame1.frame0.frame \
    $topwin(restore_name).info.frame13.frame4.frame1.frame0.frame.entry2 {top frame center fill} \
    $topwin(restore_name).info.frame13.frame4.frame1.frame0.frame.scrollbar1 {top frame center fillx}

  # pack widget $topwin(restore_name).info.frame13.frame4.frame1.frame0
  pack append $topwin(restore_name).info.frame13.frame4.frame1.frame0 \
    $topwin(restore_name).info.frame13.frame4.frame1.frame0.label23 {left frame w filly} \
    $topwin(restore_name).info.frame13.frame4.frame1.frame0.frame {left frame w filly} 

  # build widget $topwin(restore_name).info.frame13.frame4.frame1.frame1
  frame $topwin(restore_name).info.frame13.frame4.frame1.frame1 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(restore_name).info.frame13.frame4.frame1.frame1.label23
	button $topwin(restore_name).info.frame13.frame4.frame1.frame1.label23 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -relief {flat} \
	 -command {set freq_filt(original_fn) $current_image} \
    -text {Original image: }

  # build widget $topwin(restore_name).info.frame13.frame4.frame1.frame1.frame
  frame $topwin(restore_name).info.frame13.frame4.frame1.frame1.frame \
    -background "$GuiColor(frame_bg)" 

  # build widget $topwin(restore_name).info.frame13.frame4.frame1.frame1.frame.scrollbar1
  scrollbar $topwin(restore_name).info.frame13.frame4.frame1.frame1.frame.scrollbar1 \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -command " $topwin(restore_name).info.frame13.frame4.frame1.frame1.frame.entry2 xview" \
    -orient {horizontal} \
    -relief {raised} \
    -width {7}

  # build widget $topwin(restore_name).info.frame13.frame4.frame1.frame1.frame.entry2
  entry $topwin(restore_name).info.frame13.frame4.frame1.frame1.frame.entry2 \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
 -xscrollcommand "$topwin(restore_name).info.frame13.frame4.frame1.frame1.frame.scrollbar1 set" \
    -textvariable {freq_filt(original_fn)} \
    -width {20}
  endEntry $topwin(restore_name).info.frame13.frame4.frame1.frame1.frame.entry2 0 \
  	 $topwin(restore_name).info.frame13.frame4.frame1.frame0.frame.entry2 \
  	 $topwin(restore_name).info.frame13.frame4.frame1.frame0.frame.entry2 

  # pack widget $topwin(restore_name).info.frame13.frame4.frame1.frame1.frame
  pack append $topwin(restore_name).info.frame13.frame4.frame1.frame1.frame \
    $topwin(restore_name).info.frame13.frame4.frame1.frame1.frame.entry2 {top frame center fill} \
    $topwin(restore_name).info.frame13.frame4.frame1.frame1.frame.scrollbar1 {top frame center fillx}

  # pack widget $topwin(restore_name).info.frame13.frame4.frame1.frame1
  pack append $topwin(restore_name).info.frame13.frame4.frame1.frame1 \
    $topwin(restore_name).info.frame13.frame4.frame1.frame1.label23 {left frame w filly} \
    $topwin(restore_name).info.frame13.frame4.frame1.frame1.frame {left frame w filly} 

  # pack widget $topwin(restore_name).info.frame13.frame4.frame1
  pack append $topwin(restore_name).info.frame13.frame4.frame1 \
    $topwin(restore_name).info.frame13.frame4.frame1.frame0 {left frame center expand } \
    $topwin(restore_name).info.frame13.frame4.frame1.frame1 {left frame center expand }

  # build widget $topwin(restore_name).info.frame13.frame4.labelnull
  label $topwin(restore_name).info.frame13.frame4.labelnull \
    -background "$GuiColor(label_bg)" \
    -height {3}  \
    -text { } 

  # pack widget $topwin(restore_name).info.frame13.frame4
  pack append $topwin(restore_name).info.frame13.frame4 \
    $topwin(restore_name).info.frame13.frame4.labelnull {left frame center expand fill}

  # build widget $topwin(restore_name).info.frame13.frame5
  frame $topwin(restore_name).info.frame13.frame5 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2} \
    -relief {ridge}

  # build widget $topwin(restore_name).info.frame13.frame5.frame1
  frame $topwin(restore_name).info.frame13.frame5.frame1 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(restore_name).info.frame13.frame5.frame1.frame3
  frame $topwin(restore_name).info.frame13.frame5.frame1.frame3 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(restore_name).info.frame13.frame5.frame1.frame3.label5
  label $topwin(restore_name).info.frame13.frame5.frame1.frame3.label5 \
    -background "$GuiColor(label_bg)" \
    -text {Smoothness constraint function:}

  # build widget $topwin(restore_name).info.frame13.frame5.frame1.frame3.entry6
  entry $topwin(restore_name).info.frame13.frame5.frame1.frame3.entry6 \
    -background "$GuiColor(rentry_bg)" \
    -foreground "$GuiColor(rentry_fg)" \
    -state {disabled} \
    -relief {sunken} \
    -textvariable {freq_filt(SNR)} \
    -width {9}

  # build widget $topwin(restore_name).info.frame13.frame5.frame1.frame3.button0
	menubutton $topwin(restore_name).info.frame13.frame5.frame1.frame3.button0 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)"\
    -text "$GuiMisc(pop_up_text)" \
	 -width {2}
  # bindings
  bind $topwin(restore_name).info.frame13.frame5.frame1.frame3.button0 <Button-1> {MenuPopupPost $topwin(restore_name).info.frame13.frame5.frame1.frame3.button0.menu %X %Y}
  bind $topwin(restore_name).info.frame13.frame5.frame1.frame3.button0 <ButtonRelease-1> {MenuPopupRelease $topwin(restore_name).info.frame13.frame5.frame1.frame3.button0.menu %W}

  # build widget $topwin(restore_name).info.frame13.frame5.frame1.frame3.button0.menu
  menu $topwin(restore_name).info.frame13.frame5.frame1.frame3.button0.menu \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"
  $topwin(restore_name).info.frame13.frame5.frame1.frame3.button0.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(SNR) laplacian1} \
    -label {laplacian1}
  $topwin(restore_name).info.frame13.frame5.frame1.frame3.button0.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(SNR) laplacian2} \
    -label {laplacian2}
  $topwin(restore_name).info.frame13.frame5.frame1.frame3.button0.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(SNR) laplacian3} \
    -label {laplacian3}
  $topwin(restore_name).info.frame13.frame5.frame1.frame3.button0.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(SNR) laplacian4} \
    -label {laplacian4}
  # bindings
  bind $topwin(restore_name).info.frame13.frame5.frame1.frame3.button0.menu <B1-Motion> {MenuPopupMotion $topwin(restore_name).info.frame13.frame5.frame1.frame3.button0.menu0 %W %X %Y}
  bind $topwin(restore_name).info.frame13.frame5.frame1.frame3.button0.menu <ButtonRelease-1> {MenuPopupRelease $topwin(restore_name).info.frame13.frame5.frame1.frame3.button0.menu0 %W}

  # pack widget $topwin(restore_name).info.frame13.frame5.frame1.frame3
  pack append $topwin(restore_name).info.frame13.frame5.frame1.frame3 \
    $topwin(restore_name).info.frame13.frame5.frame1.frame3.label5 {left frame w} \
    $topwin(restore_name).info.frame13.frame5.frame1.frame3.entry6 {left frame center} \
    $topwin(restore_name).info.frame13.frame5.frame1.frame3.button0 {left frame center}

  # pack widget $topwin(restore_name).info.frame13.frame5.frame1
  pack append $topwin(restore_name).info.frame13.frame5.frame1 \
    $topwin(restore_name).info.frame13.frame5.frame1.frame3 {top frame center}

  # build widget $topwin(restore_name).info.frame13.frame5.frame2
  frame $topwin(restore_name).info.frame13.frame5.frame2 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(restore_name).info.frame13.frame5.frame2.frame3
  frame $topwin(restore_name).info.frame13.frame5.frame2.frame3 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(restore_name).info.frame13.frame5.frame2.frame3.label5
  label $topwin(restore_name).info.frame13.frame5.frame2.frame3.label5 \
    -background "$GuiColor(label_bg)" \
    -text {Alpha:}

  # build widget $topwin(restore_name).info.frame13.frame5.frame2.frame3.entry6
  entry $topwin(restore_name).info.frame13.frame5.frame2.frame3.entry6 \
    -background "$GuiColor(rentry_bg)" \
    -foreground "$GuiColor(rentry_fg)" \
    -relief {sunken} \
    -state {disabled} \
    -textvariable {freq_filt(alpha)} \
    -width {5}

  # build widget $topwin(restore_name).info.frame13.frame5.frame2.frame3.button0
	menubutton $topwin(restore_name).info.frame13.frame5.frame2.frame3.button0 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)"\
    -text "$GuiMisc(pop_up_text)" \
	 -width {2}
  # bindings
  bind $topwin(restore_name).info.frame13.frame5.frame2.frame3.button0 <Button-1> {MenuPopupPost $topwin(restore_name).info.frame13.frame5.frame2.frame3.button0.menu %X %Y}
  bind $topwin(restore_name).info.frame13.frame5.frame2.frame3.button0 <ButtonRelease-1> {MenuPopupRelease $topwin(restore_name).info.frame13.frame5.frame2.frame3.button0.menu %W}

  # build widget $topwin(restore_name).info.frame13.frame5.frame2.frame3.button0.menu
  menu $topwin(restore_name).info.frame13.frame5.frame2.frame3.button0.menu \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"
  $topwin(restore_name).info.frame13.frame5.frame2.frame3.button0.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(alpha) 0.0} \
    -label {0}
  $topwin(restore_name).info.frame13.frame5.frame2.frame3.button0.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(alpha) 0.111111} \
    -label {1/9}
  $topwin(restore_name).info.frame13.frame5.frame2.frame3.button0.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(alpha) 0.125} \
    -label {1/8}
  $topwin(restore_name).info.frame13.frame5.frame2.frame3.button0.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(alpha) 0.166667} \
    -label {1/6}
  $topwin(restore_name).info.frame13.frame5.frame2.frame3.button0.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(alpha) 0.25} \
    -label {1/4}
  $topwin(restore_name).info.frame13.frame5.frame2.frame3.button0.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(alpha) 0.333333} \
    -label {1/3}
  $topwin(restore_name).info.frame13.frame5.frame2.frame3.button0.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(alpha) 0.5} \
    -label {1/2}
  $topwin(restore_name).info.frame13.frame5.frame2.frame3.button0.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(alpha) 0.666667} \
    -label {2/3}
  $topwin(restore_name).info.frame13.frame5.frame2.frame3.button0.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(alpha) 0.833333} \
    -label {5/6}
  $topwin(restore_name).info.frame13.frame5.frame2.frame3.button0.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(alpha) 1.0} \
    -label {1}
  # bindings
  bind $topwin(restore_name).info.frame13.frame5.frame2.frame3.button0.menu <B1-Motion> {MenuPopupMotion $topwin(restore_name).info.frame13.frame5.frame2.frame3.button0.menu0 %W %X %Y}
  bind $topwin(restore_name).info.frame13.frame5.frame2.frame3.button0.menu <ButtonRelease-1> {MenuPopupRelease $topwin(restore_name).info.frame13.frame5.frame2.frame3.button0.menu0 %W}

  # pack widget $topwin(restore_name).info.frame13.frame5.frame2.frame3
  pack append $topwin(restore_name).info.frame13.frame5.frame2.frame3 \
    $topwin(restore_name).info.frame13.frame5.frame2.frame3.label5 {left frame w} \
    $topwin(restore_name).info.frame13.frame5.frame2.frame3.entry6 {left frame center} \
    $topwin(restore_name).info.frame13.frame5.frame2.frame3.button0 {left frame center}

  # pack widget $topwin(restore_name).info.frame13.frame5.frame2
  pack append $topwin(restore_name).info.frame13.frame5.frame2 \
    $topwin(restore_name).info.frame13.frame5.frame2.frame3 {top frame center}

  # build widget $topwin(restore_name).info.frame13.frame5.frame6
  frame $topwin(restore_name).info.frame13.frame5.frame6 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(restore_name).info.frame13.frame5.frame6.frame3
  frame $topwin(restore_name).info.frame13.frame5.frame6.frame3 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(restore_name).info.frame13.frame5.frame6.frame3.label5
  label $topwin(restore_name).info.frame13.frame5.frame6.frame3.label5 \
    -background "$GuiColor(label_bg)" \
    -text {Gamma:}

  # build widget $topwin(restore_name).info.frame13.frame5.frame6.frame3.entry6
  entry $topwin(restore_name).info.frame13.frame5.frame6.frame3.entry6 \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
    -textvariable {freq_filt(gamma)} \
    -width {5}

  # build widget $topwin(restore_name).info.frame13.frame5.frame6.frame3.button0
	menubutton $topwin(restore_name).info.frame13.frame5.frame6.frame3.button0 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)"\
    -text "$GuiMisc(pop_up_text)" \
	 -width {2}
  # bindings
  bind $topwin(restore_name).info.frame13.frame5.frame6.frame3.button0 <Button-1> {MenuPopupPost $topwin(restore_name).info.frame13.frame5.frame6.frame3.button0.menu %X %Y}
  bind $topwin(restore_name).info.frame13.frame5.frame6.frame3.button0 <ButtonRelease-1> {MenuPopupRelease $topwin(restore_name).info.frame13.frame5.frame6.frame3.button0.menu %W}

  # build widget $topwin(restore_name).info.frame13.frame5.frame6.frame3.button0.menu
  menu $topwin(restore_name).info.frame13.frame5.frame6.frame3.button0.menu \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"
  $topwin(restore_name).info.frame13.frame5.frame6.frame3.button0.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(gamma) 0} \
    -label {0}
  $topwin(restore_name).info.frame13.frame5.frame6.frame3.button0.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(gamma) 0.5} \
    -label {0.5}
  $topwin(restore_name).info.frame13.frame5.frame6.frame3.button0.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(gamma) 0.8} \
    -label {0.8}
  # bindings
  bind $topwin(restore_name).info.frame13.frame5.frame6.frame3.button0.menu <B1-Motion> {MenuPopupMotion $topwin(restore_name).info.frame13.frame5.frame6.frame3.button0.menu0 %W %X %Y}
  bind $topwin(restore_name).info.frame13.frame5.frame6.frame3.button0.menu <ButtonRelease-1> {MenuPopupRelease $topwin(restore_name).info.frame13.frame5.frame6.frame3.button0.menu0 %W}

  # pack widget $topwin(restore_name).info.frame13.frame5.frame6.frame3
  pack append $topwin(restore_name).info.frame13.frame5.frame6.frame3 \
    $topwin(restore_name).info.frame13.frame5.frame6.frame3.label5 {left frame w} \
    $topwin(restore_name).info.frame13.frame5.frame6.frame3.entry6 {left frame center} \
    $topwin(restore_name).info.frame13.frame5.frame6.frame3.button0 {left frame center}

  # pack widget $topwin(restore_name).info.frame13.frame5.frame6
  pack append $topwin(restore_name).info.frame13.frame5.frame6 \
    $topwin(restore_name).info.frame13.frame5.frame6.frame3 {top frame center}

  # build widget $topwin(restore_name).info.frame13.frame5.labelnull
  label $topwin(restore_name).info.frame13.frame5.labelnull \
    -background "$GuiColor(label_bg)" \
    -borderwidth {2} \
    -height {2} \
    -text {}

  # build widget $topwin(restore_name).info.frame13.frame5.frame0
  frame $topwin(restore_name).info.frame13.frame5.frame0 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(restore_name).info.frame13.frame5.frame0.frame3
  frame $topwin(restore_name).info.frame13.frame5.frame0.frame3 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  # build widget $topwin(restore_name).info.frame13.frame5.frame0.frame3.label5
  label $topwin(restore_name).info.frame13.frame5.frame0.frame3.label5 \
    -background "$GuiColor(label_bg)" \
    -text {K: }

  # build widget $topwin(restore_name).info.frame13.frame5.frame0.frame3.entry6
  entry $topwin(restore_name).info.frame13.frame5.frame0.frame3.entry6 \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
    -textvariable {freq_filt(K)} \
    -width {5}

  # build widget $topwin(restore_name).info.frame13.frame5.frame0.frame3.button0
	menubutton $topwin(restore_name).info.frame13.frame5.frame0.frame3.button0 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -font "$GuiMisc(pop_up_fn)" \
    -relief "$GuiMisc(pop_up_relief)"\
    -text "$GuiMisc(pop_up_text)" \
	 -width {2}
  # bindings
  bind $topwin(restore_name).info.frame13.frame5.frame0.frame3.button0 <Button-1> {MenuPopupPost $topwin(restore_name).info.frame13.frame5.frame0.frame3.button0.menu %X %Y}
  bind $topwin(restore_name).info.frame13.frame5.frame0.frame3.button0 <ButtonRelease-1> {MenuPopupRelease $topwin(restore_name).info.frame13.frame5.frame0.frame3.button0.menu %W}

  # build widget $topwin(restore_name).info.frame13.frame5.frame0.frame3.button0.menu
  menu $topwin(restore_name).info.frame13.frame5.frame0.frame3.button0.menu \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)"
  $topwin(restore_name).info.frame13.frame5.frame0.frame3.button0.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(K) 0} \
    -label {  0}
  $topwin(restore_name).info.frame13.frame5.frame0.frame3.button0.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(K) 0.5} \
    -label { 0.5}
  $topwin(restore_name).info.frame13.frame5.frame0.frame3.button0.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(K) 0.8} \
    -label { 0.8}
  $topwin(restore_name).info.frame13.frame5.frame0.frame3.button0.menu add command \
    -activebackground "$GuiColor(menu_act_bg)" \
    -background "$GuiColor(menu_bg)" \
    -command {set freq_filt(K) 1.0} \
    -label { 1.0}
  # bindings
  bind $topwin(restore_name).info.frame13.frame5.frame0.frame3.button0.menu <B1-Motion> {MenuPopupMotion $topwin(restore_name).info.frame13.frame5.frame0.frame3.button0.menu0 %W %X %Y}
  bind $topwin(restore_name).info.frame13.frame5.frame0.frame3.button0.menu <ButtonRelease-1> {MenuPopupRelease $topwin(restore_name).info.frame13.frame5.frame0.frame3.button0.menu0 %W}

  # pack widget $topwin(restore_name).info.frame13.frame5.frame0.frame3
  pack append $topwin(restore_name).info.frame13.frame5.frame0.frame3 \
    $topwin(restore_name).info.frame13.frame5.frame0.frame3.label5 {left frame w} \
    $topwin(restore_name).info.frame13.frame5.frame0.frame3.entry6 {left frame center} \
    $topwin(restore_name).info.frame13.frame5.frame0.frame3.button0 {left frame center}

  # pack widget $topwin(restore_name).info.frame13.frame5.frame0
  pack append $topwin(restore_name).info.frame13.frame5.frame0 \
    $topwin(restore_name).info.frame13.frame5.frame0.frame3 {top frame center}

  # pack widget $topwin(restore_name).info.frame13.frame5
  pack append $topwin(restore_name).info.frame13.frame5 \
    $topwin(restore_name).info.frame13.frame5.labelnull {right frame center}

  # pack widget $topwin(restore_name).info.frame13
  pack append $topwin(restore_name).info.frame13 \
    $topwin(restore_name).info.frame13.frame1 {left frame w filly} \
    $topwin(restore_name).info.frame13.frame2 {top frame center fill} \
    $topwin(restore_name).info.frame13.frame4 {top frame center fill} \
    $topwin(restore_name).info.frame13.frame5 {top frame center fill}

  # build widget $topwin(restore_name).info.frame0
  frame $topwin(restore_name).info.frame0 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {3} \
    -relief {ridge}

  # build widget $topwin(restore_name).info.frame0.label1
  label $topwin(restore_name).info.frame0.label1 \
    -background "$GuiColor(label_bg)" \
    -text {The generalized restoration equation :}

  # build widget $topwin(restore_name).info.frame0.label2
  label $topwin(restore_name).info.frame0.label2 \
    -background "$GuiColor(label_bg)" \
    -bitmap "@$env(CVIPHOME)/CVIPTCL/GUI_SCRIPTS/bmp/eq5.xbm" \
    -foreground {black} \
    -text {label2}

  # pack widget $topwin(restore_name).info.frame0
  pack append $topwin(restore_name).info.frame0 \
    $topwin(restore_name).info.frame0.label1 {left frame center} \
    $topwin(restore_name).info.frame0.label2 {left frame center padx 50}

  # pack widget $topwin(restore_name).info
  pack append $topwin(restore_name).info \
    $topwin(restore_name).info.frame13 {top frame center fillx} \
    $topwin(restore_name).info.frame0 {top frame center fillx} \
    $topwin(restore_name).info.frame11 {top frame center fill}

  # build widget $topwin(restore_name).current_image
  frame $topwin(restore_name).current_image \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2} \
    -relief {raised}

  # build widget $topwin(restore_name).current_image.button9
	button $topwin(restore_name).current_image.button9 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(dis_act_bg)" \
    -background "$GuiColor(dis_bg)" \
    -foreground "$GuiColor(dis_fg)" \
	 -command {destroy $topwin(restore_name); set topwin(restore_name) NONE; \
			if { $freq_filt(notch_spect) != "" } {cvip dq $freq_filt(notch_spect; set freq_filt(notch_spect) "" )}} \
    -text {DISMISS}

  # build widget $topwin(restore_name).current_image.button10
	button $topwin(restore_name).current_image.button10 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(perf_act_bg)" \
    -background "$GuiColor(perf_bg)" \
    -foreground "$GuiColor(perf_fg)" \
    -command {
      $topwin(restore_name) configure -cursor watch; update
      if {$current_image == "" } {
         CvipAlert "Please select current image first."
         $topwin(restore_name) configure -cursor top_left_arrow; update
         return
      }  
		if { $freq_filt(type) != "notch" && $second_image == "" && $freq_filt(deg_fn) == "image"} {
         CvipAlert "Please select the degradation image."
         $topwin(restore_name) configure -cursor top_left_arrow; update
         return
		}
		if { $freq_filt(deg_fn) == "image" } { set freq_filt(h_image) $second_image } \
		elseif { $freq_filt(type) != "notch" && $freq_filt(h_image) == ""  && \
				$freq_filt(filter_val) == "" } {
		if [catch {expr $freq_filt(msk_height) + $freq_filt(msk_width) } ] {
         CvipAlert "Numerical values required for the degradation image sizes:\n\
			Height = $freq_filt(msk_height), Width = $freq_filt(msk_width)."
         $topwin(restore_name) configure -cursor top_left_arrow; update
         return
		} elseif { $freq_filt(msk_height) < "3" || $freq_filt(msk_width) < "3" || \
					  $freq_filt(msk_height) > "30" || $freq_filt(msk_width) > "30" } {
         CvipAlert "The degradation image size ($freq_filt(msk_height)x$freq_filt(msk_width)) should \
			be in the range \[3-30\]."
         $topwin(restore_name) configure -cursor top_left_arrow; update
         return
		}
		if [catch {expr $freq_filt(param4)} ] {
         CvipAlert "Positive Numerical value required for weight parameter:\n\
			Weight = $freq_filt(param4)."
         $topwin(restore_name) configure -cursor top_left_arrow; update
         return
		} elseif {$freq_filt(param4) < 0  } {
         CvipAlert "Positive Numerical value required for weight parameter ($freq_filt(param4))"
         $topwin(restore_name) configure -cursor top_left_arrow; update
         return
		}

		set temp [expr { $freq_filt(msk_height) > $freq_filt(msk_width) }\
		? $freq_filt(msk_width) : $freq_filt(msk_height) ]
		set temp [expr $temp / 2]

		switch -exact -- $freq_filt(degrade_shape) {
			"circle" {
				if [catch {expr $freq_filt(degrade_rad) }] {
         		CvipAlert "Numerical values required for the circle diameter:\n\
					Diameter = $freq_filt(degrade_rad)."
					$topwin(restore_name) configure -cursor top_left_arrow; update
					return
				} elseif { $freq_filt(degrade_rad) < 1 || $freq_filt(degrade_rad) > $temp } {
         		CvipAlert "The circle diameter ($freq_filt(degrade_rad)) should be\
					in the range \[1-$temp\]."
					$topwin(restore_name) configure -cursor top_left_arrow; update
					return
				}
				set temp [eval "cvip circle $freq_filt(msk_width) $freq_filt(msk_height) \
									[expr $freq_filt(msk_width)/2] [expr $freq_filt(msk_height) /2]\
									[expr $freq_filt(degrade_rad) / 2]"]
			}
			"rectangle" {
				if [catch {expr $freq_filt(degrade_width) + $freq_filt(degrade_height)}] {
         		CvipAlert "Numerical values required for the mask sizes:\n\
					Height = $freq_filt(degrade_height), Width = $freq_filt(degrade_width)."
					$topwin(restore_name) configure -cursor top_left_arrow; update
					return
				} elseif {$freq_filt(degrade_width) < 1 || $freq_filt(degrade_width) > $freq_filt(msk_width) ||
					$freq_filt(degrade_height) < 1 || $freq_filt(degrade_height) > $freq_filt(msk_height) } {
         		CvipAlert "The mask size ($freq_filt(degrade_height)x$freq_filt(degrade_width)) must be \
					positive and no larger than the mask image size\
					($freq_filt(msk_height)x$freq_filt(msk_width))."
					$topwin(restore_name) configure -cursor top_left_arrow; update
					return
				}
				set temp [eval "cvip rectangle $freq_filt(msk_width) $freq_filt(msk_height) \
					[expr ($freq_filt(msk_width) - $freq_filt(degrade_width)) /2] \
					[expr ($freq_filt(msk_height) - $freq_filt(degrade_height)) /2] \
					$freq_filt(degrade_width) $freq_filt(degrade_height)"]

			}
			"Horizontal line" {
				set temp [eval "cvip line $freq_filt(msk_width) $freq_filt(msk_height) 0 \
				[expr $freq_filt(msk_height) /2 ] [expr $freq_filt(msk_width) - 1] \
				[expr $freq_filt(msk_height) /2 ]"]
			}
			"Vertical line" {
				set temp [eval "cvip line $freq_filt(msk_width) $freq_filt(msk_height) \
				[expr $freq_filt(msk_width) / 2] 0 \
				[expr $freq_filt(msk_width) / 2] [expr $freq_filt(msk_height) - 1]"]
			}
			"Slash" {
				set temp [eval "cvip line $freq_filt(msk_width) $freq_filt(msk_height) \
				0 [expr $freq_filt(msk_height) - 1] [expr $freq_filt(msk_width) - 1] 0"]
			}
			"Backslash" {
				set temp [eval "cvip line $freq_filt(msk_width) $freq_filt(msk_height) \
				0 0 [expr $freq_filt(msk_width) - 1] [expr $freq_filt(msk_height) - 1]"]
			}
		}

		set temp1 [cvip gray_multiply $temp 0.003922 0]; cvip dq $temp
		set temp $temp1

		switch -exact -- $freq_filt(blur_weight) {
			Constant {set temp1 [cvip h_image 1 $freq_filt(msk_height) $freq_filt(msk_width)]}
			"Center-weighted constant" {set temp1 [cvip h_image 2 $freq_filt(msk_height) $freq_filt(msk_width)]}
			Gaussian {set temp1 [cvip h_image 3 $freq_filt(msk_height) $freq_filt(msk_width)]}
		}

		set result [cvip multiply $temp $temp1]; cvip dq $temp; cvip dq $temp1
		set temp $result
		set freq_filt(h_image) [cvip gray_multiply $temp $freq_filt(param4) 0]; cvip dq $temp
		} elseif { $freq_filt(type) != "notch" && $freq_filt(filter_val) != "" }  {
			if { $freq_filt(h_image) != "" } {cvip dq $freq_filt(h_image) }
			if { $GUI_DEBUG == 1 } { puts stderr [split $freq_filt(filter_val)] }
			set freq_filt(h_image)  [eval "cvip data_h_image [split $freq_filt(filter_val)]"]
		}
		if { $freq_filt(type) != "notch" } {
# commented out, useful for test purpose, so not deleted
#		mask_specify "0" "[cvip data_get $freq_filt(h_image)]"
#		update
	   scan [cvip info $freq_filt(h_image)] "%s %s %s %s %s %s %s" temp temp temp1 temp temp temp temp
		if { $bands != "1" && $temp1 == "1" } {
			set temp [cvip assemble_bands  $freq_filt(h_image) $freq_filt(h_image) $freq_filt(h_image)]
			cvip dq $freq_filt(h_image)
			set freq_filt(h_image) $temp
		}
#		cvip view $freq_filt(h_image)
		}

		switch -exact -- $freq_filt(type) {
			inverse_xformfilter {
				set temp2 $current_image
            if [catch {expr $freq_filt(cutoff)}] {
               CvipAlert "Numerical value required for the cutoff frequency: \n
								cutoff frequency = $freq_filt(cutoff)"
         		$topwin(restore_name) configure -cursor top_left_arrow; update
               return
            } elseif { $freq_filt(cutoff) < "0" } {
               CvipAlert "Positive integer required for the cutoff frequency($freq_filt(cutoff))"
         		$topwin(restore_name) configure -cursor top_left_arrow; update
               return
				}
            if [catch {set temp [cvip $freq_filt(type) $current_image $freq_filt(h_image)  $freq_filt(handle_zero) $freq_filt(cutoff)];
								set result [cvip fft $temp 0 -1]; cvip dq $temp;
						} cresult] {
               CvipAlert "$cresult"
         		$topwin(restore_name) configure -cursor top_left_arrow; update
               return
            }
				set new_name [cvip rename $result ${current_image}_InvFlt]
				img_q insert end $new_name
				image_post $new_name
				cvip view $new_name
				CvipMessage "inverse filter performed on image \"$temp2\""
				if { $GUI_DEBUG == 1 } { 
					puts stderr "cvip $freq_filt(type) $current_image $freq_filt(h_image)  $freq_filt(handle_zero) $freq_filt(cutoff)"
					puts stderr "cvip fft $temp 0 -1"
				}
			}

			simple_wiener {
				set temp2 $current_image
            if [catch {expr $freq_filt(K)}] {
               CvipAlert "Numerical value required for the noise-to-signal power density ratio: \n\
						NSR = $freq_filt(K)"
         		$topwin(restore_name) configure -cursor top_left_arrow; update
               return
            } elseif { $freq_filt(K) < "0" } {
               CvipAlert "Positive value required for the noise-to-signal power density ratio \
						($freq_filt(K))"
         		$topwin(restore_name) configure -cursor top_left_arrow; update
               return
				}
            if [catch {set temp [cvip $freq_filt(type) $current_image $freq_filt(h_image) $freq_filt(K)];
								set result [cvip fft $temp 0 -1]; cvip dq $temp;
						} cresult] {
               CvipAlert "$cresult"
         		$topwin(restore_name) configure -cursor top_left_arrow; update
               return
            }
				set new_name [cvip rename $result ${current_image}_PracWiener]
				img_q insert end $new_name
				image_post $new_name
				cvip view $new_name
				CvipMessage "practical viener filter performed on image \"$temp2\""
				if { $GUI_DEBUG == 1 } { 
					puts stderr "cvip $freq_filt(type) $current_image $freq_filt(h_image) $freq_filt(K)"
					puts stderr "cvip fft $temp 0 -1"
					puts stderr "simple_wiener filter"
				}
			}
			power_spect_eq -
			wiener {
				set temp2 $current_image
            if [catch {expr $freq_filt(cutoff)}] {
               CvipAlert "Numerical value required for the cutoff frequency: \n
								cutoff frequency = $freq_filt(cutoff)"
         		$topwin(restore_name) configure -cursor top_left_arrow; update
               return
            } elseif { $freq_filt(cutoff) < "0" } {
               CvipAlert "Positive integer required for the cutoff frequency($freq_filt(cutoff))"
         		$topwin(restore_name) configure -cursor top_left_arrow; update
               return
				}
            if [catch {set temp [cvip $freq_filt(type) $current_image $freq_filt(h_image)  \
					$freq_filt(noise_fn) $freq_filt(original_fn) $freq_filt(handle_zero) $freq_filt(cutoff)]} cresult] {
               CvipAlert "$cresult"
         		$topwin(restore_name) configure -cursor top_left_arrow; update
               return
            }
				if [catch {set result [cvip fft $temp 0 -1]; cvip dq $temp } cresult ] {
               CvipAlert "$cresult"
         		$topwin(restore_name) configure -cursor top_left_arrow; update
               return
				}
				if { $freq_filt(type) == "wiener" } {
					set new_name [cvip rename $result ${current_image}_Wiener]
				} else {
					set new_name [cvip rename $result ${current_image}_PSE_filt]
				}
				img_q insert end $new_name
				image_post $new_name
				cvip view $new_name
				if { $freq_filt(type) == "power_spect_eq" } {
					CvipMessage "power spectrum equalization filter performed on image \"$temp2\""
				} else {
					CvipMessage "viener filter performed on image \"$temp2\""
				}
				if { $GUI_DEBUG == 1 } { 
					puts stderr "cvip $freq_filt(type) $current_image $freq_filt(h_image) $freq_filt(noise_fn)\
						$freq_filt(original_fn) $freq_filt(handle_zero) $freq_filt(cutoff)"
					puts stderr "cvip fft $temp 0 -1"
				}
			}
			parametric_wiener {
				set temp2 $current_image
            if [catch {expr $freq_filt(cutoff)}] {
               CvipAlert "Numerical value required for the cutoff frequency: \n
								cutoff frequency = $freq_filt(cutoff)"
         		$topwin(restore_name) configure -cursor top_left_arrow; update
               return
            } elseif { $freq_filt(cutoff) < "0" } {
               CvipAlert "Positive integer required for the cutoff frequency($freq_filt(cutoff))"
         		$topwin(restore_name) configure -cursor top_left_arrow; update
               return
				}
				if [catch {expr $freq_filt(gamma) } ] {
					CvipAlert "Numerical value required for the value of Gamms:\n\
					Gamma = $freq_filt(gamma)."
         		$topwin(restore_name) configure -cursor top_left_arrow; update
					return
				} elseif {$freq_filt(gamma) < "0" } {
					CvipAlert "Gamma ($freq_filt(gamma)) should be non-negative number"
         		$topwin(restore_name) configure -cursor top_left_arrow; update
					return
				}
            if [catch {set temp [cvip $freq_filt(type) $current_image $freq_filt(h_image) \
								$freq_filt(noise_fn) $freq_filt(original_fn) $freq_filt(gamma) \
								$freq_filt(handle_zero) $freq_filt(cutoff)];
								set result [cvip fft $temp 0 -1]; cvip dq $temp;
						} cresult] {
               CvipAlert "$cresult"
         		$topwin(restore_name) configure -cursor top_left_arrow; update
               return
            }
				set new_name [cvip rename $result ${current_image}_ParaWiener]
				img_q insert end $new_name
				image_post $new_name
				cvip view $new_name
				CvipMessage "parametric wiener filter performed on image \"$temp2\""
				if { $GUI_DEBUG == 1 } { 
					puts stderr "cvip $freq_filt(type) $current_image $freq_filt(h_image) \
						$freq_filt(noise_fn) $freq_filt(original_fn) $freq_filt(gamma) \
						$freq_filt(handle_zero) $freq_filt(cutoff)"
					puts stderr "cvip fft $temp 0 -1"
					puts stderr "parametric wiener filter"
				}
			}

			geometric_mean {
				set temp2 $current_image
            if [catch {expr $freq_filt(cutoff)}] {
               CvipAlert "Numerical value required for the cutoff frequency: \n
								cutoff frequency = $freq_filt(cutoff)"
         		$topwin(restore_name) configure -cursor top_left_arrow; update
               return
            } elseif { $freq_filt(cutoff) < "0" } {
               CvipAlert "Positive integer required for the cutoff frequency($freq_filt(cutoff))"
         		$topwin(restore_name) configure -cursor top_left_arrow; update
               return
				}
				if [catch {expr $freq_filt(gamma) } ] {
					CvipAlert "Numerical value required for the value of Gamms:\n\
					Gamma = $freq_filt(gamma)."
         		$topwin(restore_name) configure -cursor top_left_arrow; update
					return
				} elseif {$freq_filt(gamma) < "0" } {
					CvipAlert "Gamma ($freq_filt(gamma)) should be non-negative number"
         		$topwin(restore_name) configure -cursor top_left_arrow; update
					return
				}
            if [catch {set temp [cvip $freq_filt(type) $current_image $freq_filt(h_image) \
								$freq_filt(noise_fn) $freq_filt(original_fn) $freq_filt(gamma) \
								$freq_filt(alpha) $freq_filt(handle_zero) $freq_filt(cutoff)];
								set result [cvip fft $temp 0 -1]; cvip dq $temp;
						} cresult] {
               CvipAlert "$cresult"
         		$topwin(restore_name) configure -cursor top_left_arrow; update
               return
            }
				set new_name [cvip rename $result ${current_image}_GeoMean]
				img_q insert end $new_name
				image_post $new_name
				cvip view $new_name
				CvipMessage "geometric mean frequency filter performed on image \"$temp2\""
				if { $GUI_DEBUG == 1 } { 
					puts stderr "cvip $freq_filt(type) $current_image $freq_filt(h_image) \
						$freq_filt(noise_fn) $freq_filt(original_fn) $freq_filt(gamma) \
						$freq_filt(alpha) $freq_filt(handle_zero) $freq_filt(cutoff)"
					puts stderr "cvip fft $temp 0 -1"
					puts stderr "Geometric Mean filter"
				}
			}

			least_squares {
				set temp2 $current_image
				switch -exact -- $freq_filt(SNR) {
					laplacian1 	{
						set temp [cvip data_h_image 3 3 0 -1 0 -1 4 -1 0 -1 0]
					}
					laplacian2 	{
						set temp [cvip data_h_image 3 3 1 -2 1 -2 4 -2 1 -2 1]
					}
					laplacian3 	{
						set temp [cvip data_h_image 3 3 -1 -1 -1 -1 8 -1 -1 -1 -1]
					}
					laplacian4 	{
						set temp [cvip data_h_image 3 3 -2 1 -2 1 4 1 -2 1 -2]
					}
				}
				 
				if { $bands != 1} {
					set temp1 "cvip assemble_bands "
					for {set temp2 0} {$temp2 < $bands} {incr temp2} { append temp1 " $temp"}
					set temp2 [eval $temp1]
					cvip dq $temp
					set temp $temp2
				}

				if [catch {expr $freq_filt(gamma) } ] {
					CvipAlert "Numerical value required for the value of Gamms:\n\
					Gamma = $freq_filt(gamma)."
         		$topwin(restore_name) configure -cursor top_left_arrow; update
					return
				} elseif {$freq_filt(gamma) < "0" } {
					CvipAlert "Gamma ($freq_filt(gamma)) should be non-negative number"
         		$topwin(restore_name) configure -cursor top_left_arrow; update
					return
				}
				if [catch { expr $freq_filt(cutoff) } ] {
					CvipAlert "Numerical value required for the cutoff frequency:\n\
					Cutoff frequency = $freq_filt(cutoff)."
         		$topwin(restore_name) configure -cursor top_left_arrow; update
					return
				} elseif {$freq_filt(cutoff) <= "0" } {
					CvipAlert "The cutoff frequency ($freq_filt(cutoff)) should be positive integer."
         		$topwin(restore_name) configure -cursor top_left_arrow; update
					return
				}
				if { $GUI_DEBUG == 1 } { 
					puts stderr "cvip $freq_filt(type) $current_image $freq_filt(h_image) \
						$temp $freq_filt(gamma) $freq_filt(handle_zero) $freq_filt(cutoff)"
				}
            if [catch { set temp1 [cvip $freq_filt(type) $current_image $freq_filt(h_image) \
						$temp $freq_filt(gamma) $freq_filt(handle_zero) $freq_filt(cutoff)]; \
						cvip dq $temp;\
						set result [cvip fft $temp1 0 -1];\
						cvip dq $temp1} cresult] {
               CvipAlert "$cresult"
         		$topwin(restore_name) configure -cursor top_left_arrow; update
               return
            }
				CvipMessage "geometric mean frequency filter performed on image \"$temp2\""
				set new_name [cvip rename $result ${current_image}_leastSQ]
				img_q insert end $new_name
				image_post $new_name
				cvip view $new_name
			}

			notch {
				if {$freq_filt(notch_spect) == ""} {
#					CvipAlert "No spectrum image specified."
#         		$topwin(restore_name) configure -cursor top_left_arrow; update
#					return
					set temp [expr { $rows > $cols } ? $cols : $rows ]
					set temp1 [expr int(pow(2, floor(log($temp) / log(2.0))))]
					if { $temp1 <= 0 } { set $temp1 256 }
					if [catch {set temp [cvip $transform $current_image 1 $temp1]} cresult] {
						CvipMessage "$cresult."
						$topwin(restore_name) configure -cursor top_left_arrow; update
						set freq_filt(notch_mask) ""
						return
					} else {
						set new_name [cvip rename $temp ${current_image}_$transform]
						set freq_filt(notch_spect) $new_name
					}
				}
				if {$freq_filt(notch_mask) == ""} {
					CvipAlert "Please specify the notch points and size first:\n\
					Notch points = $freq_filt(notch_mask)."
         		$topwin(restore_name) configure -cursor top_left_arrow; update
					return
				}
				set temp "cvip notch $freq_filt(notch_spect) $freq_filt(notch_mask)"
				if [ catch { set temp1 [eval $temp] } cresult ] {
               CvipAlert "$cresult"
         		$topwin(restore_name) configure -cursor top_left_arrow; update
					set freq_filt(notch_mask) ""
               return
				}
#				cvip dq $freq_filt(notch_spect)
#				set freq_filt(notch_spect) ""

            if [catch { set result [cvip $freq_filt(notch_transform) $temp1 0 -1]; \
						cvip dq $temp1} cresult] {
               CvipAlert "$cresult"
         		$topwin(restore_name) configure -cursor top_left_arrow; update
					set freq_filt(notch_mask) ""
               return
            }
				set new_name [cvip rename $result ${current_image}_notch]
				img_q insert end $new_name
				image_post $new_name
				cvip view $new_name
				CvipMessage "notch filter performed on image \"$current_image\""
				set freq_filt(notch_mask) ""
				if { $GUI_DEBUG == 1 } { 
					puts stderr "cvip $freq_filt(notch_transform) $temp1 0 -1"
				}
			}

		}
	 if { $freq_filt(h_image) != "" } {
		if { $freq_filt(h_image) !=  $second_image } { cvip dq $freq_filt(h_image) }
		set freq_filt(h_image) ""
	 }
    $topwin(restore_name) configure -cursor top_left_arrow; update
	 } \
    -text {PERFORM}

  # build widget $topwin(restore_name).current_image.frame11
  frame $topwin(restore_name).current_image.frame11 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2} \
    -height {1} \
    -width {75}

  # build widget $topwin(restore_name).current_image.frame11.label1
  label $topwin(restore_name).current_image.frame11.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -text {Current_image:}

  # build widget $topwin(restore_name).current_image.frame11.value
  entry $topwin(restore_name).current_image.frame11.value \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
    -textvariable {current_image} \
    -width {21}

  # pack widget $topwin(restore_name).current_image.frame11
  pack append $topwin(restore_name).current_image.frame11 \
    $topwin(restore_name).current_image.frame11.label1 {left frame center expand fill} \
    $topwin(restore_name).current_image.frame11.value {left frame center expand fill}

  # build widget $topwin(restore_name).current_image.frame1
  frame $topwin(restore_name).current_image.frame1 \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2} \
    -height {1} \
    -width {75}

  # build widget $topwin(restore_name).current_image.frame1.label1
  label $topwin(restore_name).current_image.frame1.label1 \
    -background "$GuiColor(label_bg)" \
    -borderwidth {0} \
    -text {second_image:}

  # build widget $topwin(restore_name).current_image.frame1.value
  entry $topwin(restore_name).current_image.frame1.value \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -relief {sunken} \
    -textvariable {second_image} \
    -width {21}

  # pack widget $topwin(restore_name).current_image.frame1
  pack append $topwin(restore_name).current_image.frame1 \
    $topwin(restore_name).current_image.frame1.label1 {left frame center expand fill} \
    $topwin(restore_name).current_image.frame1.value {left frame center expand fill}

  # pack widget $topwin(restore_name).current_image
  pack append $topwin(restore_name).current_image \
    $topwin(restore_name).current_image.button9 {left frame center} \
    $topwin(restore_name).current_image.button10 {right frame center} 
#    $topwin(restore_name).current_image.frame11 {left frame center} \
#    $topwin(restore_name).current_image.frame1 {left frame center}

  # build widget $topwin(restore_name).perform
  frame $topwin(restore_name).perform \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2} \
    -relief {ridge}

  # build widget $topwin(restore_name).perform.label15
  label $topwin(restore_name).perform.label15 \
    -background "$GuiColor(label_bg)" \
    -text {Current image: }

  # build widget $topwin(restore_name).perform.label16
  label $topwin(restore_name).perform.label16 \
    -background "$GuiColor(label_bg)" \
    -foreground {blue} \
    -textvariable {current_image}

  # pack widget $topwin(restore_name).perform
  pack append $topwin(restore_name).perform \
    $topwin(restore_name).perform.label15 {left frame center} \
    $topwin(restore_name).perform.label16 {left frame center}

  # pack widget $topwin(restore_name)
  pack append $topwin(restore_name) \
    $topwin(restore_name).info {top frame center expand fill} \
    $topwin(restore_name).current_image {bottom frame center fill} \
    $topwin(restore_name).perform {bottom frame center expand fill}

}


proc mask_specify {edit arg } {# xf ignore me 7

  global {GuiColor}
  global {GuiMisc}
  global {GUI_DEBUG}

  global {freq_filt}
  global {temp}
  if [ winfo exists .mask] {
  destroy .mask
  }
  toplevel .mask  -background "$GuiColor(frame_bg)"


  if { $edit == "1" } {grab set .mask}
  # Window manager configurations
  global tk_version
  global topwin
#  wm positionfrom .mask user
  wm sizefrom .mask user
  wm maxsize .mask 500 360
  wm minsize .mask 0 0
  if {$edit != "1" } {wm title .mask "Display degradation mask"} \
  else {wm title .mask "Specify degradation image"}

  # bindings

  if { $edit == "1" } {
  bind .mask <Destroy> { if {$temp == "1" } {set freq_filt(filter_val) ""} }
  }

	set llist [eval "list $arg"]
	set freq_filt(filter_height) [lindex $llist 0]
	set freq_filt(filter_width) [lindex $llist 1]
	
	for {set i 0 } { $i < $freq_filt(filter_height) } { incr i } {
		for {set j 0 } { $j < $freq_filt(filter_width) } { incr j } {	
			set freq_filt(${i}_${j}) [lindex $llist [expr $i * $freq_filt(filter_width) + $j + 2]]
#			set freq_filt(${i}_${j}) [expr $freq_filt(${i}_${j}) * $weight]
		}
	}

  # build widget .mask.info
  frame .mask.info \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  frame .mask.info.frame \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  for {set i 0} {$i <= $freq_filt(filter_width)} {incr i} {
  label .mask.info.frame.label${i} \
    -background "$GuiColor(label_bg)" \
	 -width {5} \
    -text $i
  pack append .mask.info.frame \
    .mask.info.frame.label${i} {left frame center expand} 
  }

  .mask.info.frame.label0 configure -text Mask
  pack append .mask.info \
    .mask.info.frame {top frame center expand fill} 

  for {set i 0} {$i < $freq_filt(filter_height)} {incr i} {
  # build widget .mask.info.frame$i
  frame .mask.info.frame$i \
    -background "$GuiColor(frame_bg)" \
    -borderwidth {2}

  label .mask.info.frame${i}.label${i} \
    -background "$GuiColor(label_bg)" \
	 -width {5} \
    -text [expr $i + 1]
  pack append .mask.info.frame$i \
	 .mask.info.frame${i}.label${i} {left frame center fill expand}

  for {set j 0} {$j < $freq_filt(filter_width)} {incr j} {

  # build widget .mask.info.frame${i}.entry${j}
  entry .mask.info.frame${i}.entry${j} \
    -background "$GuiColor(entry_bg)" \
    -foreground "$GuiColor(entry_fg)" \
    -borderwidth {1} \
    -relief {sunken} \
    -textvariable "freq_filt(${i}_${j})" \
    -width {4}
  if { $edit != "1" } {
	.mask.info.frame${i}.entry${j} configure -state disabled
  }
  if { $j == $freq_filt(filter_width) - 1 } {
	if { $i == $freq_filt(filter_height) -1 } {
		endEntry .mask.info.frame${i}.entry${j} 0
	} else {
		set temp [expr $i + 1]
		endEntry .mask.info.frame${i}.entry${j} 0 .mask.info.frame${temp}.entry0
	}
  } else {
		set temp [expr $j + 1]
		endEntry .mask.info.frame${i}.entry${j} 0 .mask.info.frame${i}.entry$temp
  }

  # pack widget .mask.info.frame0
  pack append .mask.info.frame${i} \
    .mask.info.frame${i}.entry${j} {left frame center expand fill}
  }
#end of j
  # pack widget .mask.info
  pack append .mask.info \
    .mask.info.frame$i {top frame center expand fill}
  }
  endEntry .mask.info.frame[expr ${i}-1].entry[expr ${j}-1] 0 .mask.info.frame0.entry0
#end of i

  # pack widget .mask
  pack append .mask \
    .mask.info {top frame center expand fill} 
  if { $edit == "1" } {
  # build widget .mask.perform
  frame .mask.perform \
	 -borderwidth {2} \
	 -relief {raised} \
    -background "$GuiColor(frame_bg)"

  # build widget .mask.perform.perform1
	button .mask.perform.perform1 -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(btn_act_bg)" \
    -background "$GuiColor(btn_bg)" \
    -foreground {white} \
    -command {
	 set temp 0
	 set freq_filt(filter_val) " "
	 append  freq_filt(filter_val) "$freq_filt(filter_height) "
	 append  freq_filt(filter_val) "$freq_filt(filter_width) "
    for {set i 0} {$i < $freq_filt(filter_height)} {incr i} {
    	for {set j 0} {$j < $freq_filt(filter_width)} {incr j} {
	      if [catch "expr $freq_filt(${i}_${j})"] {
				CvipAlert "Numerical values required for the mask element:\n\
				M([expr 1+${i}], [expr 1+${j}]) = $freq_filt(${i}_${j})"
				set freq_filt(filter_val) ""; set temp 1
  				grab set .mask
			} else {
			append freq_filt(filter_val) "$freq_filt(${i}_${j}) "
			unset freq_filt(${i}_${j})
			}
		}
	 }
	 if {$freq_filt(filter_val) != ""} {
		 append freq_filt(filter_val) " "
	 }
	 if { $temp == "1" } { set freq_filt(filter_val) ""}
	$topwin(restore_name) configure -cursor top_left_arrow; update
	set freq_filt(filter_val) [string trim $freq_filt(filter_val) ]
	if { $temp != "1" } {destroy .mask}
	} \
    -foreground {black} \
    -text {ACCEPT}

  # build widget .mask.perform.dismiss
	button .mask.perform.dismiss -padx {1} -pady {1} -highlightthickness {0}  \
    -activebackground "$GuiColor(dis_act_bg)" \
    -background "$GuiColor(dis_bg)" \
    -foreground "$GuiColor(dis_fg)" \
    -command {
		$topwin(restore_name) configure -cursor top_left_arrow; update
		set freq_filt(filter_val) ""
		destroy .mask} \
    -text {DISMISS} \
    -width {10}

  # pack widget .mask.perform
  pack append .mask.perform \
    .mask.perform.dismiss {left frame sw fill} \
    .mask.perform.perform1 {right frame se fill}
  # pack widget .mask
  pack append .mask \
    .mask.perform {bottom frame center pady 5 fill}
	}

}
